import{j as e,r as i,B as r,F as p,S as l}from"./index-DWaKf7-t.js";import{A as c}from"./index-mRBD1Iz5.js";import{B as g}from"./index-CsDOI5ae.js";import{T as d}from"./index-FZ8AkgCd.js";import"./index-B9lQzejH.js";import"./Table-DdQ_sLHU.js";import"./addEventListener-Bv6KfeSZ.js";import"./index-bFL3q_Rm.js";import"./index-DI9_eMz3.js";import"./index-w3UDGNJA.js";import"./index-DeigmNJB.js";import"./index-Bdd9AuZi.js";import"./index-DXS1j8KM.js";import"./index-DpT0T8kB.js";import"./index-Tb4IBdIf.js";import"./index-gW1uKEbV.js";import"./index-C8BxaXP_.js";const M=()=>e.jsxs(e.Fragment,{children:[e.jsx(d,{items:[{key:"gene_count",label:"gene_count",children:e.jsx(e.Fragment,{children:e.jsx(c,{analysisPipline:"gene_count",inputAnalysisMethod:[{name:"rna_seq",label:"RNA测序文件",inputKey:["V1_single_genome_NGS_RNA"],mode:"multiple"},{name:"genome_assembly",label:"基因组参考文件",inputKey:["ngs-individual-assembly","tgs_individual_assembly"],mode:"single"},{name:"prokka",label:"基因组注释文件",inputKey:["prokka"],mode:"single"}],analysisMethod:[{name:"bowtie2_RNA_mapping",label:"bowtie2_RNA_mapping",inputKey:["bowtie2_RNA_mapping"],mode:"multiple"},{name:"feature_counts_RNA_mapping",label:"feature_counts_RNA_mapping",inputKey:["feature_counts_RNA_mapping"],mode:"multiple"}],analysisType:"sample",children:e.jsx(x,{})})})}]}),e.jsx("p",{})]}),x=({resultTableList:t,plot:n,activeTabKey:m})=>{const[o,_]=i.useState([]);i.useEffect(()=>{if(console.log(t.feature_counts_RNA_mapping),t.feature_counts_RNA_mapping){const s=t.feature_counts_RNA_mapping.map(a=>({label:`${a.sample_key} `,value:`${a.id}`}));_(s)}},[t]);const u=s=>t[s].map(a=>a.content.log);return e.jsxs(e.Fragment,{children:[m=="bowtie2_RNA_mapping"&&e.jsx(e.Fragment,{children:e.jsx(g,{resultTableList:t,plot:n,analysisKey:"bowtie2_RNA_mapping"})}),m=="feature_counts_RNA_mapping"&&e.jsxs(e.Fragment,{children:[e.jsx(r,{type:"primary",onClick:()=>{n({saveAnalysisMethod:"feature_counts_RNA_mapping_stat",moduleName:"feature_counts_stat",params:{log_path_list:u("feature_counts_RNA_mapping")},tableDesc:`
                                `})},children:"基因计数统计"}),e.jsx(r,{type:"primary",onClick:()=>{n({name:"差异表达基因分析",saveAnalysisMethod:"feature_counts_RNA_deg",moduleName:"feature_counts_RNA_deg",params:{},formDom:e.jsxs(e.Fragment,{children:[e.jsx(p.Item,{label:"选择处理组",name:"treatment",children:e.jsx(l,{mode:"multiple",style:{maxWidth:"20rem"},allowClear:!0,options:o})}),e.jsx(p.Item,{label:"选择对照组",name:"control",children:e.jsx(l,{mode:"multiple",style:{maxWidth:"20rem"},allowClear:!0,options:o})})]}),tableDesc:`

                                `})},children:"差异表达基因分析"})]})]})};export{M as default};
