import{w as r,y as o,j as t}from"./index-DWaKf7-t.js";import e from"./index-BftDD2BP.js";import"./index-FZ8AkgCd.js";import"./index-bFL3q_Rm.js";import"./utils-D2B0dIuM.js";import"./index-DpT0T8kB.js";import"./index-Tb4IBdIf.js";import"./index-B8cIBqEp.js";import"./Table-DdQ_sLHU.js";import"./addEventListener-Bv6KfeSZ.js";import"./index-DXS1j8KM.js";const f=()=>{const{project:i}=r();return o(),t.jsxs(t.Fragment,{children:[t.jsx(e,{}),!1]})};export{f as default};
