import{x as t,w as i,j as e,B as r,a as l}from"./index-DWaKf7-t.js";import{A as m}from"./index-mRBD1Iz5.js";import{f as p}from"./utils-D2B0dIuM.js";import{T as c}from"./index-FZ8AkgCd.js";import"./index-B9lQzejH.js";import"./Table-DdQ_sLHU.js";import"./addEventListener-Bv6KfeSZ.js";import"./index-bFL3q_Rm.js";import"./index-DI9_eMz3.js";import"./index-w3UDGNJA.js";import"./index-DeigmNJB.js";import"./index-Bdd9AuZi.js";import"./index-DXS1j8KM.js";import"./index-DpT0T8kB.js";import"./index-Tb4IBdIf.js";import"./index-gW1uKEbV.js";import"./index-C8BxaXP_.js";const q=()=>{const[a,s]=t.useMessage(),{project:n}=i(),o=async()=>{await l.post(`/fast-api/add-sample-analysis?project=${n}`),t.success("导入成功!")};return e.jsxs(e.Fragment,{children:[s,e.jsx(c,{items:[{key:"ngs-sample-qc",label:"NGS样本质控",children:e.jsx(e.Fragment,{children:e.jsx(m,{cardExtra:e.jsx(r,{onClick:o,children:"从样本导入"}),analysisMethod:[{name:"fastp_clean_reads",label:"fastp_clean_reads",inputKey:["fastp_clean_reads"],mode:"multiple"}],analysisPipline:"ngs-sample-qc",inputAnalysisMethod:[{name:"reads",label:"reads",inputKey:["V1_meta_genome_NGS_DNA","V1_single_genome_NGS_DNA"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group",rules:[{required:!0,message:"该字段不能为空!"}]}],analysisType:"sample",children:e.jsx(d,{})})})}]}),e.jsx("p",{})]})},d=({plot:a,record:s})=>e.jsx(e.Fragment,{children:s&&e.jsx(e.Fragment,{children:e.jsxs(r,{onClick:()=>{a({url:p(s.content.html),tableDesc:""})},children:["查看报告(",s.sample_name,")"]})})});export{q as default};
