import{j as e}from"./index-DWaKf7-t.js";import{A as a}from"./index-mRBD1Iz5.js";import{B as i}from"./index-CsDOI5ae.js";import{T as t}from"./index-FZ8AkgCd.js";import"./index-B9lQzejH.js";import"./Table-DdQ_sLHU.js";import"./addEventListener-Bv6KfeSZ.js";import"./index-bFL3q_Rm.js";import"./index-DI9_eMz3.js";import"./index-w3UDGNJA.js";import"./index-DeigmNJB.js";import"./index-Bdd9AuZi.js";import"./index-DXS1j8KM.js";import"./index-DpT0T8kB.js";import"./index-Tb4IBdIf.js";import"./index-gW1uKEbV.js";import"./index-C8BxaXP_.js";const j=()=>e.jsxs(e.Fragment,{children:[e.jsx(t,{items:[{key:"function_analysis",label:"功能分析",children:e.jsx(e.Fragment,{children:e.jsx(a,{inputAnalysisMethod:[{name:"reads",label:"去宿主后的reads",inputKey:["samtools_remove_hosts"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group",rules:[{required:!0,message:"该字段不能为空!"}]},{name:"abundance",label:"metaphlan丰度文件",inputKey:["metaphlan_sam_abundance"],mode:"multiple",type:"GroupSelectSampleButton",groupField:"sample_group",rules:[{required:!0,message:"该字段不能为空!"}]}],analysisPipline:"pipeline_humann",analysisMethod:[{name:"humann_profile",label:"humann_profile",inputKey:["humann_profile"],mode:"multiple"}],analysisType:"sample",children:e.jsx(i,{analysisKey:"bowtie2_align_host"})})})}]}),e.jsx("p",{})]});export{j as default};
