import{w as o,j as t}from"./index-DWaKf7-t.js";import{R as i}from"./index-w3UDGNJA.js";import"./index-DeigmNJB.js";import"./index-Bdd9AuZi.js";import"./index-FZ8AkgCd.js";import"./index-bFL3q_Rm.js";import"./index-DXS1j8KM.js";import"./Table-DdQ_sLHU.js";import"./addEventListener-Bv6KfeSZ.js";import"./index-DpT0T8kB.js";import"./index-Tb4IBdIf.js";import"./index-B9lQzejH.js";import"./index-gW1uKEbV.js";import"./index-C8BxaXP_.js";const P=()=>{const{project:r}=o();return t.jsx("div",{style:{maxWidth:"1500px",margin:"1rem auto"},children:t.jsx(i,{project:r,setRecord:m=>{}})})};export{P as default};
