const __vite__mapDeps=(i,m=__vite__mapDeps,d=(m.f||(m.f=["./index-BLN7-LTg.js","./Table-DdQ_sLHU.js","./addEventListener-Bv6KfeSZ.js","./index-bFL3q_Rm.js","./index-CXBP5K8x.js","./index-C8BxaXP_.js","./index-BCx1q_2Y.css","./index-BVBwOKbU.js","./index-mRBD1Iz5.js","./index-B9lQzejH.js","./index-FZ8AkgCd.js","./index-DI9_eMz3.js","./index-w3UDGNJA.js","./index-DeigmNJB.js","./index-Bdd9AuZi.js","./index-DXS1j8KM.js","./index-DpT0T8kB.js","./index-Tb4IBdIf.js","./index-D4zjdeIP.css","./index-gW1uKEbV.js","./utils-D2B0dIuM.js","./index-M_wrQPIz.js","./rgb-BPXy-Sdz.js","./index-C7OIdMoc.js","./index-D7Hz1C2T.css","./index-BI8UhFk2.js","./index-ClXvvpnN.js","./index-C3Zjrj6g.js","./index-B0u7af-A.js","./index-CsDOI5ae.js","./index-CItUtfYW.js","./index-Bw7eTjsh.js","./index-BnDZ6nzB.js","./index-C4moZeKQ.js","./index-DBzhi4Gf.js","./index-BVb3PRmR.js","./index-Czd431l7.js","./index-BftDD2BP.js","./index-B8cIBqEp.js","./index-OSGuOcRd.js","./index-CicSOEzM.js","./index-JPYxfMCx.js","./index-5L7HdvIp.js","./index-n9x2b4FL.css","./index-MdICUmuo.js"])))=>i.map(i=>d[i]);
function iw(e,t){for(var n=0;n<t.length;n++){const r=t[n];if(typeof r!="string"&&!Array.isArray(r)){for(const o in r)if(o!=="default"&&!(o in e)){const l=Object.getOwnPropertyDescriptor(r,o);l&&Object.defineProperty(e,o,l.get?l:{enumerable:!0,get:()=>r[o]})}}}return Object.freeze(Object.defineProperty(e,Symbol.toStringTag,{value:"Module"}))}(function(){const t=document.createElement("link").relList;if(t&&t.supports&&t.supports("modulepreload"))return;for(const o of document.querySelectorAll('link[rel="modulepreload"]'))r(o);new MutationObserver(o=>{for(const l of o)if(l.type==="childList")for(const c of l.addedNodes)c.tagName==="LINK"&&c.rel==="modulepreload"&&r(c)}).observe(document,{childList:!0,subtree:!0});function n(o){const l={};return o.integrity&&(l.integrity=o.integrity),o.referrerPolicy&&(l.referrerPolicy=o.referrerPolicy),o.crossOrigin==="use-credentials"?l.credentials="include":o.crossOrigin==="anonymous"?l.credentials="omit":l.credentials="same-origin",l}function r(o){if(o.ep)return;o.ep=!0;const l=n(o);fetch(o.href,l)}})();var aW=typeof globalThis<"u"?globalThis:typeof window<"u"?window:typeof global<"u"?global:typeof self<"u"?self:{};function Sd(e){return e&&e.__esModule&&Object.prototype.hasOwnProperty.call(e,"default")?e.default:e}var dg={exports:{}},ec={};/**
 * @license React
 * react-jsx-runtime.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var RC;function b_(){if(RC)return ec;RC=1;var e=Symbol.for("react.transitional.element"),t=Symbol.for("react.fragment");function n(r,o,l){var c=null;if(l!==void 0&&(c=""+l),o.key!==void 0&&(c=""+o.key),"key"in o){l={};for(var d in o)d!=="key"&&(l[d]=o[d])}else l=o;return o=l.ref,{$$typeof:e,type:r,key:c,ref:o!==void 0?o:null,props:l}}return ec.Fragment=t,ec.jsx=n,ec.jsxs=n,ec}var TC;function S_(){return TC||(TC=1,dg.exports=b_()),dg.exports}var Le=S_(),mg={exports:{}},tc={},vg={exports:{}},gg={};/**
 * @license React
 * scheduler.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var MC;function C_(){return MC||(MC=1,function(e){function t(_,j){var B=_.length;_.push(j);e:for(;0<B;){var k=B-1>>>1,I=_[k];if(0<o(I,j))_[k]=j,_[B]=I,B=k;else break e}}function n(_){return _.length===0?null:_[0]}function r(_){if(_.length===0)return null;var j=_[0],B=_.pop();if(B!==j){_[0]=B;e:for(var k=0,I=_.length,H=I>>>1;k<H;){var G=2*(k+1)-1,X=_[G],U=G+1,J=_[U];if(0>o(X,B))U<I&&0>o(J,X)?(_[k]=J,_[U]=B,k=U):(_[k]=X,_[G]=B,k=G);else if(U<I&&0>o(J,B))_[k]=J,_[U]=B,k=U;else break e}}return j}function o(_,j){var B=_.sortIndex-j.sortIndex;return B!==0?B:_.id-j.id}if(e.unstable_now=void 0,typeof performance=="object"&&typeof performance.now=="function"){var l=performance;e.unstable_now=function(){return l.now()}}else{var c=Date,d=c.now();e.unstable_now=function(){return c.now()-d}}var m=[],v=[],g=1,p=null,y=3,E=!1,C=!1,x=!1,b=!1,w=typeof setTimeout=="function"?setTimeout:null,$=typeof clearTimeout=="function"?clearTimeout:null,O=typeof setImmediate<"u"?setImmediate:null;function T(_){for(var j=n(v);j!==null;){if(j.callback===null)r(v);else if(j.startTime<=_)r(v),j.sortIndex=j.expirationTime,t(m,j);else break;j=n(v)}}function R(_){if(x=!1,T(_),!C)if(n(m)!==null)C=!0,M||(M=!0,W());else{var j=n(v);j!==null&&K(R,j.startTime-_)}}var M=!1,N=-1,z=5,P=-1;function D(){return b?!0:!(e.unstable_now()-P<z)}function F(){if(b=!1,M){var _=e.unstable_now();P=_;var j=!0;try{e:{C=!1,x&&(x=!1,$(N),N=-1),E=!0;var B=y;try{t:{for(T(_),p=n(m);p!==null&&!(p.expirationTime>_&&D());){var k=p.callback;if(typeof k=="function"){p.callback=null,y=p.priorityLevel;var I=k(p.expirationTime<=_);if(_=e.unstable_now(),typeof I=="function"){p.callback=I,T(_),j=!0;break t}p===n(m)&&r(m),T(_)}else r(m);p=n(m)}if(p!==null)j=!0;else{var H=n(v);H!==null&&K(R,H.startTime-_),j=!1}}break e}finally{p=null,y=B,E=!1}j=void 0}}finally{j?W():M=!1}}}var W;if(typeof O=="function")W=function(){O(F)};else if(typeof MessageChannel<"u"){var V=new MessageChannel,q=V.port2;V.port1.onmessage=F,W=function(){q.postMessage(null)}}else W=function(){w(F,0)};function K(_,j){N=w(function(){_(e.unstable_now())},j)}e.unstable_IdlePriority=5,e.unstable_ImmediatePriority=1,e.unstable_LowPriority=4,e.unstable_NormalPriority=3,e.unstable_Profiling=null,e.unstable_UserBlockingPriority=2,e.unstable_cancelCallback=function(_){_.callback=null},e.unstable_forceFrameRate=function(_){0>_||125<_?console.error("forceFrameRate takes a positive int between 0 and 125, forcing frame rates higher than 125 fps is not supported"):z=0<_?Math.floor(1e3/_):5},e.unstable_getCurrentPriorityLevel=function(){return y},e.unstable_next=function(_){switch(y){case 1:case 2:case 3:var j=3;break;default:j=y}var B=y;y=j;try{return _()}finally{y=B}},e.unstable_requestPaint=function(){b=!0},e.unstable_runWithPriority=function(_,j){switch(_){case 1:case 2:case 3:case 4:case 5:break;default:_=3}var B=y;y=_;try{return j()}finally{y=B}},e.unstable_scheduleCallback=function(_,j,B){var k=e.unstable_now();switch(typeof B=="object"&&B!==null?(B=B.delay,B=typeof B=="number"&&0<B?k+B:k):B=k,_){case 1:var I=-1;break;case 2:I=250;break;case 5:I=1073741823;break;case 4:I=1e4;break;default:I=5e3}return I=B+I,_={id:g++,callback:j,priorityLevel:_,startTime:B,expirationTime:I,sortIndex:-1},B>k?(_.sortIndex=B,t(v,_),n(m)===null&&_===n(v)&&(x?($(N),N=-1):x=!0,K(R,B-k))):(_.sortIndex=I,t(m,_),C||E||(C=!0,M||(M=!0,W()))),_},e.unstable_shouldYield=D,e.unstable_wrapCallback=function(_){var j=y;return function(){var B=y;y=j;try{return _.apply(this,arguments)}finally{y=B}}}}(gg)),gg}var _C;function x_(){return _C||(_C=1,vg.exports=C_()),vg.exports}var hg={exports:{}},Ot={};/**
 * @license React
 * react.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var NC;function E_(){if(NC)return Ot;NC=1;var e=Symbol.for("react.transitional.element"),t=Symbol.for("react.portal"),n=Symbol.for("react.fragment"),r=Symbol.for("react.strict_mode"),o=Symbol.for("react.profiler"),l=Symbol.for("react.consumer"),c=Symbol.for("react.context"),d=Symbol.for("react.forward_ref"),m=Symbol.for("react.suspense"),v=Symbol.for("react.memo"),g=Symbol.for("react.lazy"),p=Symbol.iterator;function y(I){return I===null||typeof I!="object"?null:(I=p&&I[p]||I["@@iterator"],typeof I=="function"?I:null)}var E={isMounted:function(){return!1},enqueueForceUpdate:function(){},enqueueReplaceState:function(){},enqueueSetState:function(){}},C=Object.assign,x={};function b(I,H,G){this.props=I,this.context=H,this.refs=x,this.updater=G||E}b.prototype.isReactComponent={},b.prototype.setState=function(I,H){if(typeof I!="object"&&typeof I!="function"&&I!=null)throw Error("takes an object of state variables to update or a function which returns an object of state variables.");this.updater.enqueueSetState(this,I,H,"setState")},b.prototype.forceUpdate=function(I){this.updater.enqueueForceUpdate(this,I,"forceUpdate")};function w(){}w.prototype=b.prototype;function $(I,H,G){this.props=I,this.context=H,this.refs=x,this.updater=G||E}var O=$.prototype=new w;O.constructor=$,C(O,b.prototype),O.isPureReactComponent=!0;var T=Array.isArray,R={H:null,A:null,T:null,S:null,V:null},M=Object.prototype.hasOwnProperty;function N(I,H,G,X,U,J){return G=J.ref,{$$typeof:e,type:I,key:H,ref:G!==void 0?G:null,props:J}}function z(I,H){return N(I.type,H,void 0,void 0,void 0,I.props)}function P(I){return typeof I=="object"&&I!==null&&I.$$typeof===e}function D(I){var H={"=":"=0",":":"=2"};return"$"+I.replace(/[=:]/g,function(G){return H[G]})}var F=/\/+/g;function W(I,H){return typeof I=="object"&&I!==null&&I.key!=null?D(""+I.key):H.toString(36)}function V(){}function q(I){switch(I.status){case"fulfilled":return I.value;case"rejected":throw I.reason;default:switch(typeof I.status=="string"?I.then(V,V):(I.status="pending",I.then(function(H){I.status==="pending"&&(I.status="fulfilled",I.value=H)},function(H){I.status==="pending"&&(I.status="rejected",I.reason=H)})),I.status){case"fulfilled":return I.value;case"rejected":throw I.reason}}throw I}function K(I,H,G,X,U){var J=typeof I;(J==="undefined"||J==="boolean")&&(I=null);var te=!1;if(I===null)te=!0;else switch(J){case"bigint":case"string":case"number":te=!0;break;case"object":switch(I.$$typeof){case e:case t:te=!0;break;case g:return te=I._init,K(te(I._payload),H,G,X,U)}}if(te)return U=U(I),te=X===""?"."+W(I,0):X,T(U)?(G="",te!=null&&(G=te.replace(F,"$&/")+"/"),K(U,H,G,"",function(le){return le})):U!=null&&(P(U)&&(U=z(U,G+(U.key==null||I&&I.key===U.key?"":(""+U.key).replace(F,"$&/")+"/")+te)),H.push(U)),1;te=0;var ve=X===""?".":X+":";if(T(I))for(var ne=0;ne<I.length;ne++)X=I[ne],J=ve+W(X,ne),te+=K(X,H,G,J,U);else if(ne=y(I),typeof ne=="function")for(I=ne.call(I),ne=0;!(X=I.next()).done;)X=X.value,J=ve+W(X,ne++),te+=K(X,H,G,J,U);else if(J==="object"){if(typeof I.then=="function")return K(q(I),H,G,X,U);throw H=String(I),Error("Objects are not valid as a React child (found: "+(H==="[object Object]"?"object with keys {"+Object.keys(I).join(", ")+"}":H)+"). If you meant to render a collection of children, use an array instead.")}return te}function _(I,H,G){if(I==null)return I;var X=[],U=0;return K(I,X,"","",function(J){return H.call(G,J,U++)}),X}function j(I){if(I._status===-1){var H=I._result;H=H(),H.then(function(G){(I._status===0||I._status===-1)&&(I._status=1,I._result=G)},function(G){(I._status===0||I._status===-1)&&(I._status=2,I._result=G)}),I._status===-1&&(I._status=0,I._result=H)}if(I._status===1)return I._result.default;throw I._result}var B=typeof reportError=="function"?reportError:function(I){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var H=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof I=="object"&&I!==null&&typeof I.message=="string"?String(I.message):String(I),error:I});if(!window.dispatchEvent(H))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",I);return}console.error(I)};function k(){}return Ot.Children={map:_,forEach:function(I,H,G){_(I,function(){H.apply(this,arguments)},G)},count:function(I){var H=0;return _(I,function(){H++}),H},toArray:function(I){return _(I,function(H){return H})||[]},only:function(I){if(!P(I))throw Error("React.Children.only expected to receive a single React element child.");return I}},Ot.Component=b,Ot.Fragment=n,Ot.Profiler=o,Ot.PureComponent=$,Ot.StrictMode=r,Ot.Suspense=m,Ot.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=R,Ot.__COMPILER_RUNTIME={__proto__:null,c:function(I){return R.H.useMemoCache(I)}},Ot.cache=function(I){return function(){return I.apply(null,arguments)}},Ot.cloneElement=function(I,H,G){if(I==null)throw Error("The argument must be a React element, but you passed "+I+".");var X=C({},I.props),U=I.key,J=void 0;if(H!=null)for(te in H.ref!==void 0&&(J=void 0),H.key!==void 0&&(U=""+H.key),H)!M.call(H,te)||te==="key"||te==="__self"||te==="__source"||te==="ref"&&H.ref===void 0||(X[te]=H[te]);var te=arguments.length-2;if(te===1)X.children=G;else if(1<te){for(var ve=Array(te),ne=0;ne<te;ne++)ve[ne]=arguments[ne+2];X.children=ve}return N(I.type,U,void 0,void 0,J,X)},Ot.createContext=function(I){return I={$$typeof:c,_currentValue:I,_currentValue2:I,_threadCount:0,Provider:null,Consumer:null},I.Provider=I,I.Consumer={$$typeof:l,_context:I},I},Ot.createElement=function(I,H,G){var X,U={},J=null;if(H!=null)for(X in H.key!==void 0&&(J=""+H.key),H)M.call(H,X)&&X!=="key"&&X!=="__self"&&X!=="__source"&&(U[X]=H[X]);var te=arguments.length-2;if(te===1)U.children=G;else if(1<te){for(var ve=Array(te),ne=0;ne<te;ne++)ve[ne]=arguments[ne+2];U.children=ve}if(I&&I.defaultProps)for(X in te=I.defaultProps,te)U[X]===void 0&&(U[X]=te[X]);return N(I,J,void 0,void 0,null,U)},Ot.createRef=function(){return{current:null}},Ot.forwardRef=function(I){return{$$typeof:d,render:I}},Ot.isValidElement=P,Ot.lazy=function(I){return{$$typeof:g,_payload:{_status:-1,_result:I},_init:j}},Ot.memo=function(I,H){return{$$typeof:v,type:I,compare:H===void 0?null:H}},Ot.startTransition=function(I){var H=R.T,G={};R.T=G;try{var X=I(),U=R.S;U!==null&&U(G,X),typeof X=="object"&&X!==null&&typeof X.then=="function"&&X.then(k,B)}catch(J){B(J)}finally{R.T=H}},Ot.unstable_useCacheRefresh=function(){return R.H.useCacheRefresh()},Ot.use=function(I){return R.H.use(I)},Ot.useActionState=function(I,H,G){return R.H.useActionState(I,H,G)},Ot.useCallback=function(I,H){return R.H.useCallback(I,H)},Ot.useContext=function(I){return R.H.useContext(I)},Ot.useDebugValue=function(){},Ot.useDeferredValue=function(I,H){return R.H.useDeferredValue(I,H)},Ot.useEffect=function(I,H,G){var X=R.H;if(typeof G=="function")throw Error("useEffect CRUD overload is not enabled in this build of React.");return X.useEffect(I,H)},Ot.useId=function(){return R.H.useId()},Ot.useImperativeHandle=function(I,H,G){return R.H.useImperativeHandle(I,H,G)},Ot.useInsertionEffect=function(I,H){return R.H.useInsertionEffect(I,H)},Ot.useLayoutEffect=function(I,H){return R.H.useLayoutEffect(I,H)},Ot.useMemo=function(I,H){return R.H.useMemo(I,H)},Ot.useOptimistic=function(I,H){return R.H.useOptimistic(I,H)},Ot.useReducer=function(I,H,G){return R.H.useReducer(I,H,G)},Ot.useRef=function(I){return R.H.useRef(I)},Ot.useState=function(I){return R.H.useState(I)},Ot.useSyncExternalStore=function(I,H,G){return R.H.useSyncExternalStore(I,H,G)},Ot.useTransition=function(){return R.H.useTransition()},Ot.version="19.1.0",Ot}var AC;function Cd(){return AC||(AC=1,hg.exports=E_()),hg.exports}var pg={exports:{}},yr={};/**
 * @license React
 * react-dom.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var IC;function w_(){if(IC)return yr;IC=1;var e=Cd();function t(m){var v="https://react.dev/errors/"+m;if(1<arguments.length){v+="?args[]="+encodeURIComponent(arguments[1]);for(var g=2;g<arguments.length;g++)v+="&args[]="+encodeURIComponent(arguments[g])}return"Minified React error #"+m+"; visit "+v+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function n(){}var r={d:{f:n,r:function(){throw Error(t(522))},D:n,C:n,L:n,m:n,X:n,S:n,M:n},p:0,findDOMNode:null},o=Symbol.for("react.portal");function l(m,v,g){var p=3<arguments.length&&arguments[3]!==void 0?arguments[3]:null;return{$$typeof:o,key:p==null?null:""+p,children:m,containerInfo:v,implementation:g}}var c=e.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE;function d(m,v){if(m==="font")return"";if(typeof v=="string")return v==="use-credentials"?v:""}return yr.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE=r,yr.createPortal=function(m,v){var g=2<arguments.length&&arguments[2]!==void 0?arguments[2]:null;if(!v||v.nodeType!==1&&v.nodeType!==9&&v.nodeType!==11)throw Error(t(299));return l(m,v,null,g)},yr.flushSync=function(m){var v=c.T,g=r.p;try{if(c.T=null,r.p=2,m)return m()}finally{c.T=v,r.p=g,r.d.f()}},yr.preconnect=function(m,v){typeof m=="string"&&(v?(v=v.crossOrigin,v=typeof v=="string"?v==="use-credentials"?v:"":void 0):v=null,r.d.C(m,v))},yr.prefetchDNS=function(m){typeof m=="string"&&r.d.D(m)},yr.preinit=function(m,v){if(typeof m=="string"&&v&&typeof v.as=="string"){var g=v.as,p=d(g,v.crossOrigin),y=typeof v.integrity=="string"?v.integrity:void 0,E=typeof v.fetchPriority=="string"?v.fetchPriority:void 0;g==="style"?r.d.S(m,typeof v.precedence=="string"?v.precedence:void 0,{crossOrigin:p,integrity:y,fetchPriority:E}):g==="script"&&r.d.X(m,{crossOrigin:p,integrity:y,fetchPriority:E,nonce:typeof v.nonce=="string"?v.nonce:void 0})}},yr.preinitModule=function(m,v){if(typeof m=="string")if(typeof v=="object"&&v!==null){if(v.as==null||v.as==="script"){var g=d(v.as,v.crossOrigin);r.d.M(m,{crossOrigin:g,integrity:typeof v.integrity=="string"?v.integrity:void 0,nonce:typeof v.nonce=="string"?v.nonce:void 0})}}else v==null&&r.d.M(m)},yr.preload=function(m,v){if(typeof m=="string"&&typeof v=="object"&&v!==null&&typeof v.as=="string"){var g=v.as,p=d(g,v.crossOrigin);r.d.L(m,g,{crossOrigin:p,integrity:typeof v.integrity=="string"?v.integrity:void 0,nonce:typeof v.nonce=="string"?v.nonce:void 0,type:typeof v.type=="string"?v.type:void 0,fetchPriority:typeof v.fetchPriority=="string"?v.fetchPriority:void 0,referrerPolicy:typeof v.referrerPolicy=="string"?v.referrerPolicy:void 0,imageSrcSet:typeof v.imageSrcSet=="string"?v.imageSrcSet:void 0,imageSizes:typeof v.imageSizes=="string"?v.imageSizes:void 0,media:typeof v.media=="string"?v.media:void 0})}},yr.preloadModule=function(m,v){if(typeof m=="string")if(v){var g=d(v.as,v.crossOrigin);r.d.m(m,{as:typeof v.as=="string"&&v.as!=="script"?v.as:void 0,crossOrigin:g,integrity:typeof v.integrity=="string"?v.integrity:void 0})}else r.d.m(m)},yr.requestFormReset=function(m){r.d.r(m)},yr.unstable_batchedUpdates=function(m,v){return m(v)},yr.useFormState=function(m,v,g){return c.H.useFormState(m,v,g)},yr.useFormStatus=function(){return c.H.useHostTransitionStatus()},yr.version="19.1.0",yr}var jC;function lw(){if(jC)return pg.exports;jC=1;function e(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(t){console.error(t)}}return e(),pg.exports=w_(),pg.exports}/**
 * @license React
 * react-dom-client.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var PC;function $_(){if(PC)return tc;PC=1;var e=x_(),t=Cd(),n=lw();function r(a){var i="https://react.dev/errors/"+a;if(1<arguments.length){i+="?args[]="+encodeURIComponent(arguments[1]);for(var s=2;s<arguments.length;s++)i+="&args[]="+encodeURIComponent(arguments[s])}return"Minified React error #"+a+"; visit "+i+" for the full message or use the non-minified dev environment for full errors and additional helpful warnings."}function o(a){return!(!a||a.nodeType!==1&&a.nodeType!==9&&a.nodeType!==11)}function l(a){var i=a,s=a;if(a.alternate)for(;i.return;)i=i.return;else{a=i;do i=a,(i.flags&4098)!==0&&(s=i.return),a=i.return;while(a)}return i.tag===3?s:null}function c(a){if(a.tag===13){var i=a.memoizedState;if(i===null&&(a=a.alternate,a!==null&&(i=a.memoizedState)),i!==null)return i.dehydrated}return null}function d(a){if(l(a)!==a)throw Error(r(188))}function m(a){var i=a.alternate;if(!i){if(i=l(a),i===null)throw Error(r(188));return i!==a?null:a}for(var s=a,f=i;;){var h=s.return;if(h===null)break;var S=h.alternate;if(S===null){if(f=h.return,f!==null){s=f;continue}break}if(h.child===S.child){for(S=h.child;S;){if(S===s)return d(h),a;if(S===f)return d(h),i;S=S.sibling}throw Error(r(188))}if(s.return!==f.return)s=h,f=S;else{for(var A=!1,L=h.child;L;){if(L===s){A=!0,s=h,f=S;break}if(L===f){A=!0,f=h,s=S;break}L=L.sibling}if(!A){for(L=S.child;L;){if(L===s){A=!0,s=S,f=h;break}if(L===f){A=!0,f=S,s=h;break}L=L.sibling}if(!A)throw Error(r(189))}}if(s.alternate!==f)throw Error(r(190))}if(s.tag!==3)throw Error(r(188));return s.stateNode.current===s?a:i}function v(a){var i=a.tag;if(i===5||i===26||i===27||i===6)return a;for(a=a.child;a!==null;){if(i=v(a),i!==null)return i;a=a.sibling}return null}var g=Object.assign,p=Symbol.for("react.element"),y=Symbol.for("react.transitional.element"),E=Symbol.for("react.portal"),C=Symbol.for("react.fragment"),x=Symbol.for("react.strict_mode"),b=Symbol.for("react.profiler"),w=Symbol.for("react.provider"),$=Symbol.for("react.consumer"),O=Symbol.for("react.context"),T=Symbol.for("react.forward_ref"),R=Symbol.for("react.suspense"),M=Symbol.for("react.suspense_list"),N=Symbol.for("react.memo"),z=Symbol.for("react.lazy"),P=Symbol.for("react.activity"),D=Symbol.for("react.memo_cache_sentinel"),F=Symbol.iterator;function W(a){return a===null||typeof a!="object"?null:(a=F&&a[F]||a["@@iterator"],typeof a=="function"?a:null)}var V=Symbol.for("react.client.reference");function q(a){if(a==null)return null;if(typeof a=="function")return a.$$typeof===V?null:a.displayName||a.name||null;if(typeof a=="string")return a;switch(a){case C:return"Fragment";case b:return"Profiler";case x:return"StrictMode";case R:return"Suspense";case M:return"SuspenseList";case P:return"Activity"}if(typeof a=="object")switch(a.$$typeof){case E:return"Portal";case O:return(a.displayName||"Context")+".Provider";case $:return(a._context.displayName||"Context")+".Consumer";case T:var i=a.render;return a=a.displayName,a||(a=i.displayName||i.name||"",a=a!==""?"ForwardRef("+a+")":"ForwardRef"),a;case N:return i=a.displayName||null,i!==null?i:q(a.type)||"Memo";case z:i=a._payload,a=a._init;try{return q(a(i))}catch{}}return null}var K=Array.isArray,_=t.__CLIENT_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,j=n.__DOM_INTERNALS_DO_NOT_USE_OR_WARN_USERS_THEY_CANNOT_UPGRADE,B={pending:!1,data:null,method:null,action:null},k=[],I=-1;function H(a){return{current:a}}function G(a){0>I||(a.current=k[I],k[I]=null,I--)}function X(a,i){I++,k[I]=a.current,a.current=i}var U=H(null),J=H(null),te=H(null),ve=H(null);function ne(a,i){switch(X(te,i),X(J,a),X(U,null),i.nodeType){case 9:case 11:a=(a=i.documentElement)&&(a=a.namespaceURI)?tC(a):0;break;default:if(a=i.tagName,i=i.namespaceURI)i=tC(i),a=nC(i,a);else switch(a){case"svg":a=1;break;case"math":a=2;break;default:a=0}}G(U),X(U,a)}function le(){G(U),G(J),G(te)}function be(a){a.memoizedState!==null&&X(ve,a);var i=U.current,s=nC(i,a.type);i!==s&&(X(J,a),X(U,s))}function fe(a){J.current===a&&(G(U),G(J)),ve.current===a&&(G(ve),Xs._currentValue=B)}var ie=Object.prototype.hasOwnProperty,pe=e.unstable_scheduleCallback,Re=e.unstable_cancelCallback,se=e.unstable_shouldYield,ye=e.unstable_requestPaint,$e=e.unstable_now,we=e.unstable_getCurrentPriorityLevel,Ne=e.unstable_ImmediatePriority,Ze=e.unstable_UserBlockingPriority,He=e.unstable_NormalPriority,Xe=e.unstable_LowPriority,xe=e.unstable_IdlePriority,Me=e.log,Ve=e.unstable_setDisableYieldValue,Ke=null,We=null;function at(a){if(typeof Me=="function"&&Ve(a),We&&typeof We.setStrictMode=="function")try{We.setStrictMode(Ke,a)}catch{}}var Ue=Math.clz32?Math.clz32:ht,Ye=Math.log,st=Math.LN2;function ht(a){return a>>>=0,a===0?32:31-(Ye(a)/st|0)|0}var Qe=256,Pe=4194304;function Fe(a){var i=a&42;if(i!==0)return i;switch(a&-a){case 1:return 1;case 2:return 2;case 4:return 4;case 8:return 8;case 16:return 16;case 32:return 32;case 64:return 64;case 128:return 128;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return a&4194048;case 4194304:case 8388608:case 16777216:case 33554432:return a&62914560;case 67108864:return 67108864;case 134217728:return 134217728;case 268435456:return 268435456;case 536870912:return 536870912;case 1073741824:return 0;default:return a}}function _e(a,i,s){var f=a.pendingLanes;if(f===0)return 0;var h=0,S=a.suspendedLanes,A=a.pingedLanes;a=a.warmLanes;var L=f&134217727;return L!==0?(f=L&~S,f!==0?h=Fe(f):(A&=L,A!==0?h=Fe(A):s||(s=L&~a,s!==0&&(h=Fe(s))))):(L=f&~S,L!==0?h=Fe(L):A!==0?h=Fe(A):s||(s=f&~a,s!==0&&(h=Fe(s)))),h===0?0:i!==0&&i!==h&&(i&S)===0&&(S=h&-h,s=i&-i,S>=s||S===32&&(s&4194048)!==0)?i:h}function qe(a,i){return(a.pendingLanes&~(a.suspendedLanes&~a.pingedLanes)&i)===0}function Ct(a,i){switch(a){case 1:case 2:case 4:case 8:case 64:return i+250;case 16:case 32:case 128:case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:return i+5e3;case 4194304:case 8388608:case 16777216:case 33554432:return-1;case 67108864:case 134217728:case 268435456:case 536870912:case 1073741824:return-1;default:return-1}}function Nt(){var a=Qe;return Qe<<=1,(Qe&4194048)===0&&(Qe=256),a}function _n(){var a=Pe;return Pe<<=1,(Pe&62914560)===0&&(Pe=4194304),a}function bn(a){for(var i=[],s=0;31>s;s++)i.push(a);return i}function $n(a,i){a.pendingLanes|=i,i!==268435456&&(a.suspendedLanes=0,a.pingedLanes=0,a.warmLanes=0)}function sn(a,i,s,f,h,S){var A=a.pendingLanes;a.pendingLanes=s,a.suspendedLanes=0,a.pingedLanes=0,a.warmLanes=0,a.expiredLanes&=s,a.entangledLanes&=s,a.errorRecoveryDisabledLanes&=s,a.shellSuspendCounter=0;var L=a.entanglements,Q=a.expirationTimes,ue=a.hiddenUpdates;for(s=A&~s;0<s;){var Ce=31-Ue(s),Oe=1<<Ce;L[Ce]=0,Q[Ce]=-1;var me=ue[Ce];if(me!==null)for(ue[Ce]=null,Ce=0;Ce<me.length;Ce++){var ge=me[Ce];ge!==null&&(ge.lane&=-536870913)}s&=~Oe}f!==0&&Ft(a,f,0),S!==0&&h===0&&a.tag!==0&&(a.suspendedLanes|=S&~(A&~i))}function Ft(a,i,s){a.pendingLanes|=i,a.suspendedLanes&=~i;var f=31-Ue(i);a.entangledLanes|=i,a.entanglements[f]=a.entanglements[f]|1073741824|s&4194090}function nn(a,i){var s=a.entangledLanes|=i;for(a=a.entanglements;s;){var f=31-Ue(s),h=1<<f;h&i|a[f]&i&&(a[f]|=i),s&=~h}}function Et(a){switch(a){case 2:a=1;break;case 8:a=4;break;case 32:a=16;break;case 256:case 512:case 1024:case 2048:case 4096:case 8192:case 16384:case 32768:case 65536:case 131072:case 262144:case 524288:case 1048576:case 2097152:case 4194304:case 8388608:case 16777216:case 33554432:a=128;break;case 268435456:a=134217728;break;default:a=0}return a}function nt(a){return a&=-a,2<a?8<a?(a&134217727)!==0?32:268435456:8:2}function ke(){var a=j.p;return a!==0?a:(a=window.event,a===void 0?32:CC(a.type))}function De(a,i){var s=j.p;try{return j.p=a,i()}finally{j.p=s}}var pt=Math.random().toString(36).slice(2),St="__reactFiber$"+pt,Be="__reactProps$"+pt,je="__reactContainer$"+pt,rt="__reactEvents$"+pt,ot="__reactListeners$"+pt,tt="__reactHandles$"+pt,ut="__reactResources$"+pt,Vt="__reactMarker$"+pt;function Tt(a){delete a[St],delete a[Be],delete a[rt],delete a[ot],delete a[tt]}function At(a){var i=a[St];if(i)return i;for(var s=a.parentNode;s;){if(i=s[je]||s[St]){if(s=i.alternate,i.child!==null||s!==null&&s.child!==null)for(a=iC(a);a!==null;){if(s=a[St])return s;a=iC(a)}return i}a=s,s=a.parentNode}return null}function It(a){if(a=a[St]||a[je]){var i=a.tag;if(i===5||i===6||i===13||i===26||i===27||i===3)return a}return null}function Qt(a){var i=a.tag;if(i===5||i===26||i===27||i===6)return a.stateNode;throw Error(r(33))}function rn(a){var i=a[ut];return i||(i=a[ut]={hoistableStyles:new Map,hoistableScripts:new Map}),i}function Rt(a){a[Vt]=!0}var er=new Set,pn={};function an(a,i){Ae(a,i),Ae(a+"Capture",i)}function Ae(a,i){for(pn[a]=i,a=0;a<i.length;a++)er.add(i[a])}var et=RegExp("^[:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD][:A-Z_a-z\\u00C0-\\u00D6\\u00D8-\\u00F6\\u00F8-\\u02FF\\u0370-\\u037D\\u037F-\\u1FFF\\u200C-\\u200D\\u2070-\\u218F\\u2C00-\\u2FEF\\u3001-\\uD7FF\\uF900-\\uFDCF\\uFDF0-\\uFFFD\\-.0-9\\u00B7\\u0300-\\u036F\\u203F-\\u2040]*$"),yt={},xt={};function Gt(a){return ie.call(xt,a)?!0:ie.call(yt,a)?!1:et.test(a)?xt[a]=!0:(yt[a]=!0,!1)}function kn(a,i,s){if(Gt(i))if(s===null)a.removeAttribute(i);else{switch(typeof s){case"undefined":case"function":case"symbol":a.removeAttribute(i);return;case"boolean":var f=i.toLowerCase().slice(0,5);if(f!=="data-"&&f!=="aria-"){a.removeAttribute(i);return}}a.setAttribute(i,""+s)}}function Pn(a,i,s){if(s===null)a.removeAttribute(i);else{switch(typeof s){case"undefined":case"function":case"symbol":case"boolean":a.removeAttribute(i);return}a.setAttribute(i,""+s)}}function Ln(a,i,s,f){if(f===null)a.removeAttribute(s);else{switch(typeof f){case"undefined":case"function":case"symbol":case"boolean":a.removeAttribute(s);return}a.setAttributeNS(i,s,""+f)}}var cr,hr;function On(a){if(cr===void 0)try{throw Error()}catch(s){var i=s.stack.trim().match(/\n( *(at )?)/);cr=i&&i[1]||"",hr=-1<s.stack.indexOf(`
    at`)?" (<anonymous>)":-1<s.stack.indexOf("@")?"@unknown:0:0":""}return`
`+cr+a+hr}var tr=!1;function Dr(a,i){if(!a||tr)return"";tr=!0;var s=Error.prepareStackTrace;Error.prepareStackTrace=void 0;try{var f={DetermineComponentFrameRoot:function(){try{if(i){var Oe=function(){throw Error()};if(Object.defineProperty(Oe.prototype,"props",{set:function(){throw Error()}}),typeof Reflect=="object"&&Reflect.construct){try{Reflect.construct(Oe,[])}catch(ge){var me=ge}Reflect.construct(a,[],Oe)}else{try{Oe.call()}catch(ge){me=ge}a.call(Oe.prototype)}}else{try{throw Error()}catch(ge){me=ge}(Oe=a())&&typeof Oe.catch=="function"&&Oe.catch(function(){})}}catch(ge){if(ge&&me&&typeof ge.stack=="string")return[ge.stack,me.stack]}return[null,null]}};f.DetermineComponentFrameRoot.displayName="DetermineComponentFrameRoot";var h=Object.getOwnPropertyDescriptor(f.DetermineComponentFrameRoot,"name");h&&h.configurable&&Object.defineProperty(f.DetermineComponentFrameRoot,"name",{value:"DetermineComponentFrameRoot"});var S=f.DetermineComponentFrameRoot(),A=S[0],L=S[1];if(A&&L){var Q=A.split(`
`),ue=L.split(`
`);for(h=f=0;f<Q.length&&!Q[f].includes("DetermineComponentFrameRoot");)f++;for(;h<ue.length&&!ue[h].includes("DetermineComponentFrameRoot");)h++;if(f===Q.length||h===ue.length)for(f=Q.length-1,h=ue.length-1;1<=f&&0<=h&&Q[f]!==ue[h];)h--;for(;1<=f&&0<=h;f--,h--)if(Q[f]!==ue[h]){if(f!==1||h!==1)do if(f--,h--,0>h||Q[f]!==ue[h]){var Ce=`
`+Q[f].replace(" at new "," at ");return a.displayName&&Ce.includes("<anonymous>")&&(Ce=Ce.replace("<anonymous>",a.displayName)),Ce}while(1<=f&&0<=h);break}}}finally{tr=!1,Error.prepareStackTrace=s}return(s=a?a.displayName||a.name:"")?On(s):""}function Da(a){switch(a.tag){case 26:case 27:case 5:return On(a.type);case 16:return On("Lazy");case 13:return On("Suspense");case 19:return On("SuspenseList");case 0:case 15:return Dr(a.type,!1);case 11:return Dr(a.type.render,!1);case 1:return Dr(a.type,!0);case 31:return On("Activity");default:return""}}function Gn(a){try{var i="";do i+=Da(a),a=a.return;while(a);return i}catch(s){return`
Error generating stack: `+s.message+`
`+s.stack}}function cn(a){switch(typeof a){case"bigint":case"boolean":case"number":case"string":case"undefined":return a;case"object":return a;default:return""}}function Br(a){var i=a.type;return(a=a.nodeName)&&a.toLowerCase()==="input"&&(i==="checkbox"||i==="radio")}function ra(a){var i=Br(a)?"checked":"value",s=Object.getOwnPropertyDescriptor(a.constructor.prototype,i),f=""+a[i];if(!a.hasOwnProperty(i)&&typeof s<"u"&&typeof s.get=="function"&&typeof s.set=="function"){var h=s.get,S=s.set;return Object.defineProperty(a,i,{configurable:!0,get:function(){return h.call(this)},set:function(A){f=""+A,S.call(this,A)}}),Object.defineProperty(a,i,{enumerable:s.enumerable}),{getValue:function(){return f},setValue:function(A){f=""+A},stopTracking:function(){a._valueTracker=null,delete a[i]}}}}function Kn(a){a._valueTracker||(a._valueTracker=ra(a))}function Ca(a){if(!a)return!1;var i=a._valueTracker;if(!i)return!0;var s=i.getValue(),f="";return a&&(f=Br(a)?a.checked?"true":"false":a.value),a=f,a!==s?(i.setValue(a),!0):!1}function Bt(a){if(a=a||(typeof document<"u"?document:void 0),typeof a>"u")return null;try{return a.activeElement||a.body}catch{return a.body}}var Je=/[\n"\\]/g;function ze(a){return a.replace(Je,function(i){return"\\"+i.charCodeAt(0).toString(16)+" "})}function Lt(a,i,s,f,h,S,A,L){a.name="",A!=null&&typeof A!="function"&&typeof A!="symbol"&&typeof A!="boolean"?a.type=A:a.removeAttribute("type"),i!=null?A==="number"?(i===0&&a.value===""||a.value!=i)&&(a.value=""+cn(i)):a.value!==""+cn(i)&&(a.value=""+cn(i)):A!=="submit"&&A!=="reset"||a.removeAttribute("value"),i!=null?Kt(a,A,cn(i)):s!=null?Kt(a,A,cn(s)):f!=null&&a.removeAttribute("value"),h==null&&S!=null&&(a.defaultChecked=!!S),h!=null&&(a.checked=h&&typeof h!="function"&&typeof h!="symbol"),L!=null&&typeof L!="function"&&typeof L!="symbol"&&typeof L!="boolean"?a.name=""+cn(L):a.removeAttribute("name")}function Ut(a,i,s,f,h,S,A,L){if(S!=null&&typeof S!="function"&&typeof S!="symbol"&&typeof S!="boolean"&&(a.type=S),i!=null||s!=null){if(!(S!=="submit"&&S!=="reset"||i!=null))return;s=s!=null?""+cn(s):"",i=i!=null?""+cn(i):s,L||i===a.value||(a.value=i),a.defaultValue=i}f=f??h,f=typeof f!="function"&&typeof f!="symbol"&&!!f,a.checked=L?a.checked:!!f,a.defaultChecked=!!f,A!=null&&typeof A!="function"&&typeof A!="symbol"&&typeof A!="boolean"&&(a.name=A)}function Kt(a,i,s){i==="number"&&Bt(a.ownerDocument)===a||a.defaultValue===""+s||(a.defaultValue=""+s)}function Xt(a,i,s,f){if(a=a.options,i){i={};for(var h=0;h<s.length;h++)i["$"+s[h]]=!0;for(s=0;s<a.length;s++)h=i.hasOwnProperty("$"+a[s].value),a[s].selected!==h&&(a[s].selected=h),h&&f&&(a[s].defaultSelected=!0)}else{for(s=""+cn(s),i=null,h=0;h<a.length;h++){if(a[h].value===s){a[h].selected=!0,f&&(a[h].defaultSelected=!0);return}i!==null||a[h].disabled||(i=a[h])}i!==null&&(i.selected=!0)}}function nr(a,i,s){if(i!=null&&(i=""+cn(i),i!==a.value&&(a.value=i),s==null)){a.defaultValue!==i&&(a.defaultValue=i);return}a.defaultValue=s!=null?""+cn(s):""}function Hn(a,i,s,f){if(i==null){if(f!=null){if(s!=null)throw Error(r(92));if(K(f)){if(1<f.length)throw Error(r(93));f=f[0]}s=f}s==null&&(s=""),i=s}s=cn(i),a.defaultValue=s,f=a.textContent,f===s&&f!==""&&f!==null&&(a.value=f)}function Nn(a,i){if(i){var s=a.firstChild;if(s&&s===a.lastChild&&s.nodeType===3){s.nodeValue=i;return}}a.textContent=i}var rr=new Set("animationIterationCount aspectRatio borderImageOutset borderImageSlice borderImageWidth boxFlex boxFlexGroup boxOrdinalGroup columnCount columns flex flexGrow flexPositive flexShrink flexNegative flexOrder gridArea gridRow gridRowEnd gridRowSpan gridRowStart gridColumn gridColumnEnd gridColumnSpan gridColumnStart fontWeight lineClamp lineHeight opacity order orphans scale tabSize widows zIndex zoom fillOpacity floodOpacity stopOpacity strokeDasharray strokeDashoffset strokeMiterlimit strokeOpacity strokeWidth MozAnimationIterationCount MozBoxFlex MozBoxFlexGroup MozLineClamp msAnimationIterationCount msFlex msZoom msFlexGrow msFlexNegative msFlexOrder msFlexPositive msFlexShrink msGridColumn msGridColumnSpan msGridRow msGridRowSpan WebkitAnimationIterationCount WebkitBoxFlex WebKitBoxFlexGroup WebkitBoxOrdinalGroup WebkitColumnCount WebkitColumns WebkitFlex WebkitFlexGrow WebkitFlexPositive WebkitFlexShrink WebkitLineClamp".split(" "));function ar(a,i,s){var f=i.indexOf("--")===0;s==null||typeof s=="boolean"||s===""?f?a.setProperty(i,""):i==="float"?a.cssFloat="":a[i]="":f?a.setProperty(i,s):typeof s!="number"||s===0||rr.has(i)?i==="float"?a.cssFloat=s:a[i]=(""+s).trim():a[i]=s+"px"}function xr(a,i,s){if(i!=null&&typeof i!="object")throw Error(r(62));if(a=a.style,s!=null){for(var f in s)!s.hasOwnProperty(f)||i!=null&&i.hasOwnProperty(f)||(f.indexOf("--")===0?a.setProperty(f,""):f==="float"?a.cssFloat="":a[f]="");for(var h in i)f=i[h],i.hasOwnProperty(h)&&s[h]!==f&&ar(a,h,f)}else for(var S in i)i.hasOwnProperty(S)&&ar(a,S,i[S])}function Xn(a){if(a.indexOf("-")===-1)return!1;switch(a){case"annotation-xml":case"color-profile":case"font-face":case"font-face-src":case"font-face-uri":case"font-face-format":case"font-face-name":case"missing-glyph":return!1;default:return!0}}var cs=new Map([["acceptCharset","accept-charset"],["htmlFor","for"],["httpEquiv","http-equiv"],["crossOrigin","crossorigin"],["accentHeight","accent-height"],["alignmentBaseline","alignment-baseline"],["arabicForm","arabic-form"],["baselineShift","baseline-shift"],["capHeight","cap-height"],["clipPath","clip-path"],["clipRule","clip-rule"],["colorInterpolation","color-interpolation"],["colorInterpolationFilters","color-interpolation-filters"],["colorProfile","color-profile"],["colorRendering","color-rendering"],["dominantBaseline","dominant-baseline"],["enableBackground","enable-background"],["fillOpacity","fill-opacity"],["fillRule","fill-rule"],["floodColor","flood-color"],["floodOpacity","flood-opacity"],["fontFamily","font-family"],["fontSize","font-size"],["fontSizeAdjust","font-size-adjust"],["fontStretch","font-stretch"],["fontStyle","font-style"],["fontVariant","font-variant"],["fontWeight","font-weight"],["glyphName","glyph-name"],["glyphOrientationHorizontal","glyph-orientation-horizontal"],["glyphOrientationVertical","glyph-orientation-vertical"],["horizAdvX","horiz-adv-x"],["horizOriginX","horiz-origin-x"],["imageRendering","image-rendering"],["letterSpacing","letter-spacing"],["lightingColor","lighting-color"],["markerEnd","marker-end"],["markerMid","marker-mid"],["markerStart","marker-start"],["overlinePosition","overline-position"],["overlineThickness","overline-thickness"],["paintOrder","paint-order"],["panose-1","panose-1"],["pointerEvents","pointer-events"],["renderingIntent","rendering-intent"],["shapeRendering","shape-rendering"],["stopColor","stop-color"],["stopOpacity","stop-opacity"],["strikethroughPosition","strikethrough-position"],["strikethroughThickness","strikethrough-thickness"],["strokeDasharray","stroke-dasharray"],["strokeDashoffset","stroke-dashoffset"],["strokeLinecap","stroke-linecap"],["strokeLinejoin","stroke-linejoin"],["strokeMiterlimit","stroke-miterlimit"],["strokeOpacity","stroke-opacity"],["strokeWidth","stroke-width"],["textAnchor","text-anchor"],["textDecoration","text-decoration"],["textRendering","text-rendering"],["transformOrigin","transform-origin"],["underlinePosition","underline-position"],["underlineThickness","underline-thickness"],["unicodeBidi","unicode-bidi"],["unicodeRange","unicode-range"],["unitsPerEm","units-per-em"],["vAlphabetic","v-alphabetic"],["vHanging","v-hanging"],["vIdeographic","v-ideographic"],["vMathematical","v-mathematical"],["vectorEffect","vector-effect"],["vertAdvY","vert-adv-y"],["vertOriginX","vert-origin-x"],["vertOriginY","vert-origin-y"],["wordSpacing","word-spacing"],["writingMode","writing-mode"],["xmlnsXlink","xmlns:xlink"],["xHeight","x-height"]]),Mr=/^[\u0000-\u001F ]*j[\r\n\t]*a[\r\n\t]*v[\r\n\t]*a[\r\n\t]*s[\r\n\t]*c[\r\n\t]*r[\r\n\t]*i[\r\n\t]*p[\r\n\t]*t[\r\n\t]*:/i;function Za(a){return Mr.test(""+a)?"javascript:throw new Error('React has blocked a javascript: URL as a security precaution.')":a}var Ba=null;function aa(a){return a=a.target||a.srcElement||window,a.correspondingUseElement&&(a=a.correspondingUseElement),a.nodeType===3?a.parentNode:a}var Er=null,oa=null;function ui(a){var i=It(a);if(i&&(a=i.stateNode)){var s=a[Be]||null;e:switch(a=i.stateNode,i.type){case"input":if(Lt(a,s.value,s.defaultValue,s.defaultValue,s.checked,s.defaultChecked,s.type,s.name),i=s.name,s.type==="radio"&&i!=null){for(s=a;s.parentNode;)s=s.parentNode;for(s=s.querySelectorAll('input[name="'+ze(""+i)+'"][type="radio"]'),i=0;i<s.length;i++){var f=s[i];if(f!==a&&f.form===a.form){var h=f[Be]||null;if(!h)throw Error(r(90));Lt(f,h.value,h.defaultValue,h.defaultValue,h.checked,h.defaultChecked,h.type,h.name)}}for(i=0;i<s.length;i++)f=s[i],f.form===a.form&&Ca(f)}break e;case"textarea":nr(a,s.value,s.defaultValue);break e;case"select":i=s.value,i!=null&&Xt(a,!!s.multiple,i,!1)}}}var fi=!1;function Yi(a,i,s){if(fi)return a(i,s);fi=!0;try{var f=a(i);return f}finally{if(fi=!1,(Er!==null||oa!==null)&&(Vu(),Er&&(i=Er,a=oa,oa=Er=null,ui(i),a)))for(i=0;i<a.length;i++)ui(a[i])}}function Ja(a,i){var s=a.stateNode;if(s===null)return null;var f=s[Be]||null;if(f===null)return null;s=f[i];e:switch(i){case"onClick":case"onClickCapture":case"onDoubleClick":case"onDoubleClickCapture":case"onMouseDown":case"onMouseDownCapture":case"onMouseMove":case"onMouseMoveCapture":case"onMouseUp":case"onMouseUpCapture":case"onMouseEnter":(f=!f.disabled)||(a=a.type,f=!(a==="button"||a==="input"||a==="select"||a==="textarea")),a=!f;break e;default:a=!1}if(a)return null;if(s&&typeof s!="function")throw Error(r(231,i,typeof s));return s}var Ge=!(typeof window>"u"||typeof window.document>"u"||typeof window.document.createElement>"u"),dt=!1;if(Ge)try{var en={};Object.defineProperty(en,"passive",{get:function(){dt=!0}}),window.addEventListener("test",en,en),window.removeEventListener("test",en,en)}catch{dt=!1}var un=null,Sn=null,eo=null;function Qi(){if(eo)return eo;var a,i=Sn,s=i.length,f,h="value"in un?un.value:un.textContent,S=h.length;for(a=0;a<s&&i[a]===h[a];a++);var A=s-a;for(f=1;f<=A&&i[s-f]===h[S-f];f++);return eo=h.slice(a,1<f?1-f:void 0)}function di(a){var i=a.keyCode;return"charCode"in a?(a=a.charCode,a===0&&i===13&&(a=13)):a=i,a===10&&(a=13),32<=a||a===13?a:0}function to(){return!0}function iy(){return!1}function _r(a){function i(s,f,h,S,A){this._reactName=s,this._targetInst=h,this.type=f,this.nativeEvent=S,this.target=A,this.currentTarget=null;for(var L in a)a.hasOwnProperty(L)&&(s=a[L],this[L]=s?s(S):S[L]);return this.isDefaultPrevented=(S.defaultPrevented!=null?S.defaultPrevented:S.returnValue===!1)?to:iy,this.isPropagationStopped=iy,this}return g(i.prototype,{preventDefault:function(){this.defaultPrevented=!0;var s=this.nativeEvent;s&&(s.preventDefault?s.preventDefault():typeof s.returnValue!="unknown"&&(s.returnValue=!1),this.isDefaultPrevented=to)},stopPropagation:function(){var s=this.nativeEvent;s&&(s.stopPropagation?s.stopPropagation():typeof s.cancelBubble!="unknown"&&(s.cancelBubble=!0),this.isPropagationStopped=to)},persist:function(){},isPersistent:to}),i}var mi={eventPhase:0,bubbles:0,cancelable:0,timeStamp:function(a){return a.timeStamp||Date.now()},defaultPrevented:0,isTrusted:0},lu=_r(mi),us=g({},mi,{view:0,detail:0}),bT=_r(us),pm,ym,fs,su=g({},us,{screenX:0,screenY:0,clientX:0,clientY:0,pageX:0,pageY:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,getModifierState:Sm,button:0,buttons:0,relatedTarget:function(a){return a.relatedTarget===void 0?a.fromElement===a.srcElement?a.toElement:a.fromElement:a.relatedTarget},movementX:function(a){return"movementX"in a?a.movementX:(a!==fs&&(fs&&a.type==="mousemove"?(pm=a.screenX-fs.screenX,ym=a.screenY-fs.screenY):ym=pm=0,fs=a),pm)},movementY:function(a){return"movementY"in a?a.movementY:ym}}),ly=_r(su),ST=g({},su,{dataTransfer:0}),CT=_r(ST),xT=g({},us,{relatedTarget:0}),bm=_r(xT),ET=g({},mi,{animationName:0,elapsedTime:0,pseudoElement:0}),wT=_r(ET),$T=g({},mi,{clipboardData:function(a){return"clipboardData"in a?a.clipboardData:window.clipboardData}}),OT=_r($T),RT=g({},mi,{data:0}),sy=_r(RT),TT={Esc:"Escape",Spacebar:" ",Left:"ArrowLeft",Up:"ArrowUp",Right:"ArrowRight",Down:"ArrowDown",Del:"Delete",Win:"OS",Menu:"ContextMenu",Apps:"ContextMenu",Scroll:"ScrollLock",MozPrintableKey:"Unidentified"},MT={8:"Backspace",9:"Tab",12:"Clear",13:"Enter",16:"Shift",17:"Control",18:"Alt",19:"Pause",20:"CapsLock",27:"Escape",32:" ",33:"PageUp",34:"PageDown",35:"End",36:"Home",37:"ArrowLeft",38:"ArrowUp",39:"ArrowRight",40:"ArrowDown",45:"Insert",46:"Delete",112:"F1",113:"F2",114:"F3",115:"F4",116:"F5",117:"F6",118:"F7",119:"F8",120:"F9",121:"F10",122:"F11",123:"F12",144:"NumLock",145:"ScrollLock",224:"Meta"},_T={Alt:"altKey",Control:"ctrlKey",Meta:"metaKey",Shift:"shiftKey"};function NT(a){var i=this.nativeEvent;return i.getModifierState?i.getModifierState(a):(a=_T[a])?!!i[a]:!1}function Sm(){return NT}var AT=g({},us,{key:function(a){if(a.key){var i=TT[a.key]||a.key;if(i!=="Unidentified")return i}return a.type==="keypress"?(a=di(a),a===13?"Enter":String.fromCharCode(a)):a.type==="keydown"||a.type==="keyup"?MT[a.keyCode]||"Unidentified":""},code:0,location:0,ctrlKey:0,shiftKey:0,altKey:0,metaKey:0,repeat:0,locale:0,getModifierState:Sm,charCode:function(a){return a.type==="keypress"?di(a):0},keyCode:function(a){return a.type==="keydown"||a.type==="keyup"?a.keyCode:0},which:function(a){return a.type==="keypress"?di(a):a.type==="keydown"||a.type==="keyup"?a.keyCode:0}}),IT=_r(AT),jT=g({},su,{pointerId:0,width:0,height:0,pressure:0,tangentialPressure:0,tiltX:0,tiltY:0,twist:0,pointerType:0,isPrimary:0}),cy=_r(jT),PT=g({},us,{touches:0,targetTouches:0,changedTouches:0,altKey:0,metaKey:0,ctrlKey:0,shiftKey:0,getModifierState:Sm}),zT=_r(PT),DT=g({},mi,{propertyName:0,elapsedTime:0,pseudoElement:0}),BT=_r(DT),LT=g({},su,{deltaX:function(a){return"deltaX"in a?a.deltaX:"wheelDeltaX"in a?-a.wheelDeltaX:0},deltaY:function(a){return"deltaY"in a?a.deltaY:"wheelDeltaY"in a?-a.wheelDeltaY:"wheelDelta"in a?-a.wheelDelta:0},deltaZ:0,deltaMode:0}),HT=_r(LT),FT=g({},mi,{newState:0,oldState:0}),VT=_r(FT),UT=[9,13,27,32],Cm=Ge&&"CompositionEvent"in window,ds=null;Ge&&"documentMode"in document&&(ds=document.documentMode);var qT=Ge&&"TextEvent"in window&&!ds,uy=Ge&&(!Cm||ds&&8<ds&&11>=ds),fy=" ",dy=!1;function my(a,i){switch(a){case"keyup":return UT.indexOf(i.keyCode)!==-1;case"keydown":return i.keyCode!==229;case"keypress":case"mousedown":case"focusout":return!0;default:return!1}}function vy(a){return a=a.detail,typeof a=="object"&&"data"in a?a.data:null}var Zi=!1;function WT(a,i){switch(a){case"compositionend":return vy(i);case"keypress":return i.which!==32?null:(dy=!0,fy);case"textInput":return a=i.data,a===fy&&dy?null:a;default:return null}}function kT(a,i){if(Zi)return a==="compositionend"||!Cm&&my(a,i)?(a=Qi(),eo=Sn=un=null,Zi=!1,a):null;switch(a){case"paste":return null;case"keypress":if(!(i.ctrlKey||i.altKey||i.metaKey)||i.ctrlKey&&i.altKey){if(i.char&&1<i.char.length)return i.char;if(i.which)return String.fromCharCode(i.which)}return null;case"compositionend":return uy&&i.locale!=="ko"?null:i.data;default:return null}}var GT={color:!0,date:!0,datetime:!0,"datetime-local":!0,email:!0,month:!0,number:!0,password:!0,range:!0,search:!0,tel:!0,text:!0,time:!0,url:!0,week:!0};function gy(a){var i=a&&a.nodeName&&a.nodeName.toLowerCase();return i==="input"?!!GT[a.type]:i==="textarea"}function hy(a,i,s,f){Er?oa?oa.push(f):oa=[f]:Er=f,i=Ku(i,"onChange"),0<i.length&&(s=new lu("onChange","change",null,s,f),a.push({event:s,listeners:i}))}var ms=null,vs=null;function KT(a){YS(a,0)}function cu(a){var i=Qt(a);if(Ca(i))return a}function py(a,i){if(a==="change")return i}var yy=!1;if(Ge){var xm;if(Ge){var Em="oninput"in document;if(!Em){var by=document.createElement("div");by.setAttribute("oninput","return;"),Em=typeof by.oninput=="function"}xm=Em}else xm=!1;yy=xm&&(!document.documentMode||9<document.documentMode)}function Sy(){ms&&(ms.detachEvent("onpropertychange",Cy),vs=ms=null)}function Cy(a){if(a.propertyName==="value"&&cu(vs)){var i=[];hy(i,vs,a,aa(a)),Yi(KT,i)}}function XT(a,i,s){a==="focusin"?(Sy(),ms=i,vs=s,ms.attachEvent("onpropertychange",Cy)):a==="focusout"&&Sy()}function YT(a){if(a==="selectionchange"||a==="keyup"||a==="keydown")return cu(vs)}function QT(a,i){if(a==="click")return cu(i)}function ZT(a,i){if(a==="input"||a==="change")return cu(i)}function JT(a,i){return a===i&&(a!==0||1/a===1/i)||a!==a&&i!==i}var Lr=typeof Object.is=="function"?Object.is:JT;function gs(a,i){if(Lr(a,i))return!0;if(typeof a!="object"||a===null||typeof i!="object"||i===null)return!1;var s=Object.keys(a),f=Object.keys(i);if(s.length!==f.length)return!1;for(f=0;f<s.length;f++){var h=s[f];if(!ie.call(i,h)||!Lr(a[h],i[h]))return!1}return!0}function xy(a){for(;a&&a.firstChild;)a=a.firstChild;return a}function Ey(a,i){var s=xy(a);a=0;for(var f;s;){if(s.nodeType===3){if(f=a+s.textContent.length,a<=i&&f>=i)return{node:s,offset:i-a};a=f}e:{for(;s;){if(s.nextSibling){s=s.nextSibling;break e}s=s.parentNode}s=void 0}s=xy(s)}}function wy(a,i){return a&&i?a===i?!0:a&&a.nodeType===3?!1:i&&i.nodeType===3?wy(a,i.parentNode):"contains"in a?a.contains(i):a.compareDocumentPosition?!!(a.compareDocumentPosition(i)&16):!1:!1}function $y(a){a=a!=null&&a.ownerDocument!=null&&a.ownerDocument.defaultView!=null?a.ownerDocument.defaultView:window;for(var i=Bt(a.document);i instanceof a.HTMLIFrameElement;){try{var s=typeof i.contentWindow.location.href=="string"}catch{s=!1}if(s)a=i.contentWindow;else break;i=Bt(a.document)}return i}function wm(a){var i=a&&a.nodeName&&a.nodeName.toLowerCase();return i&&(i==="input"&&(a.type==="text"||a.type==="search"||a.type==="tel"||a.type==="url"||a.type==="password")||i==="textarea"||a.contentEditable==="true")}var eM=Ge&&"documentMode"in document&&11>=document.documentMode,Ji=null,$m=null,hs=null,Om=!1;function Oy(a,i,s){var f=s.window===s?s.document:s.nodeType===9?s:s.ownerDocument;Om||Ji==null||Ji!==Bt(f)||(f=Ji,"selectionStart"in f&&wm(f)?f={start:f.selectionStart,end:f.selectionEnd}:(f=(f.ownerDocument&&f.ownerDocument.defaultView||window).getSelection(),f={anchorNode:f.anchorNode,anchorOffset:f.anchorOffset,focusNode:f.focusNode,focusOffset:f.focusOffset}),hs&&gs(hs,f)||(hs=f,f=Ku($m,"onSelect"),0<f.length&&(i=new lu("onSelect","select",null,i,s),a.push({event:i,listeners:f}),i.target=Ji)))}function vi(a,i){var s={};return s[a.toLowerCase()]=i.toLowerCase(),s["Webkit"+a]="webkit"+i,s["Moz"+a]="moz"+i,s}var el={animationend:vi("Animation","AnimationEnd"),animationiteration:vi("Animation","AnimationIteration"),animationstart:vi("Animation","AnimationStart"),transitionrun:vi("Transition","TransitionRun"),transitionstart:vi("Transition","TransitionStart"),transitioncancel:vi("Transition","TransitionCancel"),transitionend:vi("Transition","TransitionEnd")},Rm={},Ry={};Ge&&(Ry=document.createElement("div").style,"AnimationEvent"in window||(delete el.animationend.animation,delete el.animationiteration.animation,delete el.animationstart.animation),"TransitionEvent"in window||delete el.transitionend.transition);function gi(a){if(Rm[a])return Rm[a];if(!el[a])return a;var i=el[a],s;for(s in i)if(i.hasOwnProperty(s)&&s in Ry)return Rm[a]=i[s];return a}var Ty=gi("animationend"),My=gi("animationiteration"),_y=gi("animationstart"),tM=gi("transitionrun"),nM=gi("transitionstart"),rM=gi("transitioncancel"),Ny=gi("transitionend"),Ay=new Map,Tm="abort auxClick beforeToggle cancel canPlay canPlayThrough click close contextMenu copy cut drag dragEnd dragEnter dragExit dragLeave dragOver dragStart drop durationChange emptied encrypted ended error gotPointerCapture input invalid keyDown keyPress keyUp load loadedData loadedMetadata loadStart lostPointerCapture mouseDown mouseMove mouseOut mouseOver mouseUp paste pause play playing pointerCancel pointerDown pointerMove pointerOut pointerOver pointerUp progress rateChange reset resize seeked seeking stalled submit suspend timeUpdate touchCancel touchEnd touchStart volumeChange scroll toggle touchMove waiting wheel".split(" ");Tm.push("scrollEnd");function xa(a,i){Ay.set(a,i),an(i,[a])}var Iy=new WeakMap;function ia(a,i){if(typeof a=="object"&&a!==null){var s=Iy.get(a);return s!==void 0?s:(i={value:a,source:i,stack:Gn(i)},Iy.set(a,i),i)}return{value:a,source:i,stack:Gn(i)}}var la=[],tl=0,Mm=0;function uu(){for(var a=tl,i=Mm=tl=0;i<a;){var s=la[i];la[i++]=null;var f=la[i];la[i++]=null;var h=la[i];la[i++]=null;var S=la[i];if(la[i++]=null,f!==null&&h!==null){var A=f.pending;A===null?h.next=h:(h.next=A.next,A.next=h),f.pending=h}S!==0&&jy(s,h,S)}}function fu(a,i,s,f){la[tl++]=a,la[tl++]=i,la[tl++]=s,la[tl++]=f,Mm|=f,a.lanes|=f,a=a.alternate,a!==null&&(a.lanes|=f)}function _m(a,i,s,f){return fu(a,i,s,f),du(a)}function nl(a,i){return fu(a,null,null,i),du(a)}function jy(a,i,s){a.lanes|=s;var f=a.alternate;f!==null&&(f.lanes|=s);for(var h=!1,S=a.return;S!==null;)S.childLanes|=s,f=S.alternate,f!==null&&(f.childLanes|=s),S.tag===22&&(a=S.stateNode,a===null||a._visibility&1||(h=!0)),a=S,S=S.return;return a.tag===3?(S=a.stateNode,h&&i!==null&&(h=31-Ue(s),a=S.hiddenUpdates,f=a[h],f===null?a[h]=[i]:f.push(i),i.lane=s|536870912),S):null}function du(a){if(50<Fs)throw Fs=0,zv=null,Error(r(185));for(var i=a.return;i!==null;)a=i,i=a.return;return a.tag===3?a.stateNode:null}var rl={};function aM(a,i,s,f){this.tag=a,this.key=s,this.sibling=this.child=this.return=this.stateNode=this.type=this.elementType=null,this.index=0,this.refCleanup=this.ref=null,this.pendingProps=i,this.dependencies=this.memoizedState=this.updateQueue=this.memoizedProps=null,this.mode=f,this.subtreeFlags=this.flags=0,this.deletions=null,this.childLanes=this.lanes=0,this.alternate=null}function Hr(a,i,s,f){return new aM(a,i,s,f)}function Nm(a){return a=a.prototype,!(!a||!a.isReactComponent)}function no(a,i){var s=a.alternate;return s===null?(s=Hr(a.tag,i,a.key,a.mode),s.elementType=a.elementType,s.type=a.type,s.stateNode=a.stateNode,s.alternate=a,a.alternate=s):(s.pendingProps=i,s.type=a.type,s.flags=0,s.subtreeFlags=0,s.deletions=null),s.flags=a.flags&65011712,s.childLanes=a.childLanes,s.lanes=a.lanes,s.child=a.child,s.memoizedProps=a.memoizedProps,s.memoizedState=a.memoizedState,s.updateQueue=a.updateQueue,i=a.dependencies,s.dependencies=i===null?null:{lanes:i.lanes,firstContext:i.firstContext},s.sibling=a.sibling,s.index=a.index,s.ref=a.ref,s.refCleanup=a.refCleanup,s}function Py(a,i){a.flags&=65011714;var s=a.alternate;return s===null?(a.childLanes=0,a.lanes=i,a.child=null,a.subtreeFlags=0,a.memoizedProps=null,a.memoizedState=null,a.updateQueue=null,a.dependencies=null,a.stateNode=null):(a.childLanes=s.childLanes,a.lanes=s.lanes,a.child=s.child,a.subtreeFlags=0,a.deletions=null,a.memoizedProps=s.memoizedProps,a.memoizedState=s.memoizedState,a.updateQueue=s.updateQueue,a.type=s.type,i=s.dependencies,a.dependencies=i===null?null:{lanes:i.lanes,firstContext:i.firstContext}),a}function mu(a,i,s,f,h,S){var A=0;if(f=a,typeof a=="function")Nm(a)&&(A=1);else if(typeof a=="string")A=i_(a,s,U.current)?26:a==="html"||a==="head"||a==="body"?27:5;else e:switch(a){case P:return a=Hr(31,s,i,h),a.elementType=P,a.lanes=S,a;case C:return hi(s.children,h,S,i);case x:A=8,h|=24;break;case b:return a=Hr(12,s,i,h|2),a.elementType=b,a.lanes=S,a;case R:return a=Hr(13,s,i,h),a.elementType=R,a.lanes=S,a;case M:return a=Hr(19,s,i,h),a.elementType=M,a.lanes=S,a;default:if(typeof a=="object"&&a!==null)switch(a.$$typeof){case w:case O:A=10;break e;case $:A=9;break e;case T:A=11;break e;case N:A=14;break e;case z:A=16,f=null;break e}A=29,s=Error(r(130,a===null?"null":typeof a,"")),f=null}return i=Hr(A,s,i,h),i.elementType=a,i.type=f,i.lanes=S,i}function hi(a,i,s,f){return a=Hr(7,a,f,i),a.lanes=s,a}function Am(a,i,s){return a=Hr(6,a,null,i),a.lanes=s,a}function Im(a,i,s){return i=Hr(4,a.children!==null?a.children:[],a.key,i),i.lanes=s,i.stateNode={containerInfo:a.containerInfo,pendingChildren:null,implementation:a.implementation},i}var al=[],ol=0,vu=null,gu=0,sa=[],ca=0,pi=null,ro=1,ao="";function yi(a,i){al[ol++]=gu,al[ol++]=vu,vu=a,gu=i}function zy(a,i,s){sa[ca++]=ro,sa[ca++]=ao,sa[ca++]=pi,pi=a;var f=ro;a=ao;var h=32-Ue(f)-1;f&=~(1<<h),s+=1;var S=32-Ue(i)+h;if(30<S){var A=h-h%5;S=(f&(1<<A)-1).toString(32),f>>=A,h-=A,ro=1<<32-Ue(i)+h|s<<h|f,ao=S+a}else ro=1<<S|s<<h|f,ao=a}function jm(a){a.return!==null&&(yi(a,1),zy(a,1,0))}function Pm(a){for(;a===vu;)vu=al[--ol],al[ol]=null,gu=al[--ol],al[ol]=null;for(;a===pi;)pi=sa[--ca],sa[ca]=null,ao=sa[--ca],sa[ca]=null,ro=sa[--ca],sa[ca]=null}var wr=null,An=null,Zt=!1,bi=null,La=!1,zm=Error(r(519));function Si(a){var i=Error(r(418,""));throw bs(ia(i,a)),zm}function Dy(a){var i=a.stateNode,s=a.type,f=a.memoizedProps;switch(i[St]=a,i[Be]=f,s){case"dialog":Dt("cancel",i),Dt("close",i);break;case"iframe":case"object":case"embed":Dt("load",i);break;case"video":case"audio":for(s=0;s<Us.length;s++)Dt(Us[s],i);break;case"source":Dt("error",i);break;case"img":case"image":case"link":Dt("error",i),Dt("load",i);break;case"details":Dt("toggle",i);break;case"input":Dt("invalid",i),Ut(i,f.value,f.defaultValue,f.checked,f.defaultChecked,f.type,f.name,!0),Kn(i);break;case"select":Dt("invalid",i);break;case"textarea":Dt("invalid",i),Hn(i,f.value,f.defaultValue,f.children),Kn(i)}s=f.children,typeof s!="string"&&typeof s!="number"&&typeof s!="bigint"||i.textContent===""+s||f.suppressHydrationWarning===!0||eC(i.textContent,s)?(f.popover!=null&&(Dt("beforetoggle",i),Dt("toggle",i)),f.onScroll!=null&&Dt("scroll",i),f.onScrollEnd!=null&&Dt("scrollend",i),f.onClick!=null&&(i.onclick=Xu),i=!0):i=!1,i||Si(a)}function By(a){for(wr=a.return;wr;)switch(wr.tag){case 5:case 13:La=!1;return;case 27:case 3:La=!0;return;default:wr=wr.return}}function ps(a){if(a!==wr)return!1;if(!Zt)return By(a),Zt=!0,!1;var i=a.tag,s;if((s=i!==3&&i!==27)&&((s=i===5)&&(s=a.type,s=!(s!=="form"&&s!=="button")||Zv(a.type,a.memoizedProps)),s=!s),s&&An&&Si(a),By(a),i===13){if(a=a.memoizedState,a=a!==null?a.dehydrated:null,!a)throw Error(r(317));e:{for(a=a.nextSibling,i=0;a;){if(a.nodeType===8)if(s=a.data,s==="/$"){if(i===0){An=wa(a.nextSibling);break e}i--}else s!=="$"&&s!=="$!"&&s!=="$?"||i++;a=a.nextSibling}An=null}}else i===27?(i=An,Wo(a.type)?(a=ng,ng=null,An=a):An=i):An=wr?wa(a.stateNode.nextSibling):null;return!0}function ys(){An=wr=null,Zt=!1}function Ly(){var a=bi;return a!==null&&(Ir===null?Ir=a:Ir.push.apply(Ir,a),bi=null),a}function bs(a){bi===null?bi=[a]:bi.push(a)}var Dm=H(null),Ci=null,oo=null;function _o(a,i,s){X(Dm,i._currentValue),i._currentValue=s}function io(a){a._currentValue=Dm.current,G(Dm)}function Bm(a,i,s){for(;a!==null;){var f=a.alternate;if((a.childLanes&i)!==i?(a.childLanes|=i,f!==null&&(f.childLanes|=i)):f!==null&&(f.childLanes&i)!==i&&(f.childLanes|=i),a===s)break;a=a.return}}function Lm(a,i,s,f){var h=a.child;for(h!==null&&(h.return=a);h!==null;){var S=h.dependencies;if(S!==null){var A=h.child;S=S.firstContext;e:for(;S!==null;){var L=S;S=h;for(var Q=0;Q<i.length;Q++)if(L.context===i[Q]){S.lanes|=s,L=S.alternate,L!==null&&(L.lanes|=s),Bm(S.return,s,a),f||(A=null);break e}S=L.next}}else if(h.tag===18){if(A=h.return,A===null)throw Error(r(341));A.lanes|=s,S=A.alternate,S!==null&&(S.lanes|=s),Bm(A,s,a),A=null}else A=h.child;if(A!==null)A.return=h;else for(A=h;A!==null;){if(A===a){A=null;break}if(h=A.sibling,h!==null){h.return=A.return,A=h;break}A=A.return}h=A}}function Ss(a,i,s,f){a=null;for(var h=i,S=!1;h!==null;){if(!S){if((h.flags&524288)!==0)S=!0;else if((h.flags&262144)!==0)break}if(h.tag===10){var A=h.alternate;if(A===null)throw Error(r(387));if(A=A.memoizedProps,A!==null){var L=h.type;Lr(h.pendingProps.value,A.value)||(a!==null?a.push(L):a=[L])}}else if(h===ve.current){if(A=h.alternate,A===null)throw Error(r(387));A.memoizedState.memoizedState!==h.memoizedState.memoizedState&&(a!==null?a.push(Xs):a=[Xs])}h=h.return}a!==null&&Lm(i,a,s,f),i.flags|=262144}function hu(a){for(a=a.firstContext;a!==null;){if(!Lr(a.context._currentValue,a.memoizedValue))return!0;a=a.next}return!1}function xi(a){Ci=a,oo=null,a=a.dependencies,a!==null&&(a.firstContext=null)}function pr(a){return Hy(Ci,a)}function pu(a,i){return Ci===null&&xi(a),Hy(a,i)}function Hy(a,i){var s=i._currentValue;if(i={context:i,memoizedValue:s,next:null},oo===null){if(a===null)throw Error(r(308));oo=i,a.dependencies={lanes:0,firstContext:i},a.flags|=524288}else oo=oo.next=i;return s}var oM=typeof AbortController<"u"?AbortController:function(){var a=[],i=this.signal={aborted:!1,addEventListener:function(s,f){a.push(f)}};this.abort=function(){i.aborted=!0,a.forEach(function(s){return s()})}},iM=e.unstable_scheduleCallback,lM=e.unstable_NormalPriority,Yn={$$typeof:O,Consumer:null,Provider:null,_currentValue:null,_currentValue2:null,_threadCount:0};function Hm(){return{controller:new oM,data:new Map,refCount:0}}function Cs(a){a.refCount--,a.refCount===0&&iM(lM,function(){a.controller.abort()})}var xs=null,Fm=0,il=0,ll=null;function sM(a,i){if(xs===null){var s=xs=[];Fm=0,il=Uv(),ll={status:"pending",value:void 0,then:function(f){s.push(f)}}}return Fm++,i.then(Fy,Fy),i}function Fy(){if(--Fm===0&&xs!==null){ll!==null&&(ll.status="fulfilled");var a=xs;xs=null,il=0,ll=null;for(var i=0;i<a.length;i++)(0,a[i])()}}function cM(a,i){var s=[],f={status:"pending",value:null,reason:null,then:function(h){s.push(h)}};return a.then(function(){f.status="fulfilled",f.value=i;for(var h=0;h<s.length;h++)(0,s[h])(i)},function(h){for(f.status="rejected",f.reason=h,h=0;h<s.length;h++)(0,s[h])(void 0)}),f}var Vy=_.S;_.S=function(a,i){typeof i=="object"&&i!==null&&typeof i.then=="function"&&sM(a,i),Vy!==null&&Vy(a,i)};var Ei=H(null);function Vm(){var a=Ei.current;return a!==null?a:Cn.pooledCache}function yu(a,i){i===null?X(Ei,Ei.current):X(Ei,i.pool)}function Uy(){var a=Vm();return a===null?null:{parent:Yn._currentValue,pool:a}}var Es=Error(r(460)),qy=Error(r(474)),bu=Error(r(542)),Um={then:function(){}};function Wy(a){return a=a.status,a==="fulfilled"||a==="rejected"}function Su(){}function ky(a,i,s){switch(s=a[s],s===void 0?a.push(i):s!==i&&(i.then(Su,Su),i=s),i.status){case"fulfilled":return i.value;case"rejected":throw a=i.reason,Ky(a),a;default:if(typeof i.status=="string")i.then(Su,Su);else{if(a=Cn,a!==null&&100<a.shellSuspendCounter)throw Error(r(482));a=i,a.status="pending",a.then(function(f){if(i.status==="pending"){var h=i;h.status="fulfilled",h.value=f}},function(f){if(i.status==="pending"){var h=i;h.status="rejected",h.reason=f}})}switch(i.status){case"fulfilled":return i.value;case"rejected":throw a=i.reason,Ky(a),a}throw ws=i,Es}}var ws=null;function Gy(){if(ws===null)throw Error(r(459));var a=ws;return ws=null,a}function Ky(a){if(a===Es||a===bu)throw Error(r(483))}var No=!1;function qm(a){a.updateQueue={baseState:a.memoizedState,firstBaseUpdate:null,lastBaseUpdate:null,shared:{pending:null,lanes:0,hiddenCallbacks:null},callbacks:null}}function Wm(a,i){a=a.updateQueue,i.updateQueue===a&&(i.updateQueue={baseState:a.baseState,firstBaseUpdate:a.firstBaseUpdate,lastBaseUpdate:a.lastBaseUpdate,shared:a.shared,callbacks:null})}function Ao(a){return{lane:a,tag:0,payload:null,callback:null,next:null}}function Io(a,i,s){var f=a.updateQueue;if(f===null)return null;if(f=f.shared,(on&2)!==0){var h=f.pending;return h===null?i.next=i:(i.next=h.next,h.next=i),f.pending=i,i=du(a),jy(a,null,s),i}return fu(a,f,i,s),du(a)}function $s(a,i,s){if(i=i.updateQueue,i!==null&&(i=i.shared,(s&4194048)!==0)){var f=i.lanes;f&=a.pendingLanes,s|=f,i.lanes=s,nn(a,s)}}function km(a,i){var s=a.updateQueue,f=a.alternate;if(f!==null&&(f=f.updateQueue,s===f)){var h=null,S=null;if(s=s.firstBaseUpdate,s!==null){do{var A={lane:s.lane,tag:s.tag,payload:s.payload,callback:null,next:null};S===null?h=S=A:S=S.next=A,s=s.next}while(s!==null);S===null?h=S=i:S=S.next=i}else h=S=i;s={baseState:f.baseState,firstBaseUpdate:h,lastBaseUpdate:S,shared:f.shared,callbacks:f.callbacks},a.updateQueue=s;return}a=s.lastBaseUpdate,a===null?s.firstBaseUpdate=i:a.next=i,s.lastBaseUpdate=i}var Gm=!1;function Os(){if(Gm){var a=ll;if(a!==null)throw a}}function Rs(a,i,s,f){Gm=!1;var h=a.updateQueue;No=!1;var S=h.firstBaseUpdate,A=h.lastBaseUpdate,L=h.shared.pending;if(L!==null){h.shared.pending=null;var Q=L,ue=Q.next;Q.next=null,A===null?S=ue:A.next=ue,A=Q;var Ce=a.alternate;Ce!==null&&(Ce=Ce.updateQueue,L=Ce.lastBaseUpdate,L!==A&&(L===null?Ce.firstBaseUpdate=ue:L.next=ue,Ce.lastBaseUpdate=Q))}if(S!==null){var Oe=h.baseState;A=0,Ce=ue=Q=null,L=S;do{var me=L.lane&-536870913,ge=me!==L.lane;if(ge?(qt&me)===me:(f&me)===me){me!==0&&me===il&&(Gm=!0),Ce!==null&&(Ce=Ce.next={lane:0,tag:L.tag,payload:L.payload,callback:null,next:null});e:{var bt=a,mt=L;me=i;var mn=s;switch(mt.tag){case 1:if(bt=mt.payload,typeof bt=="function"){Oe=bt.call(mn,Oe,me);break e}Oe=bt;break e;case 3:bt.flags=bt.flags&-65537|128;case 0:if(bt=mt.payload,me=typeof bt=="function"?bt.call(mn,Oe,me):bt,me==null)break e;Oe=g({},Oe,me);break e;case 2:No=!0}}me=L.callback,me!==null&&(a.flags|=64,ge&&(a.flags|=8192),ge=h.callbacks,ge===null?h.callbacks=[me]:ge.push(me))}else ge={lane:me,tag:L.tag,payload:L.payload,callback:L.callback,next:null},Ce===null?(ue=Ce=ge,Q=Oe):Ce=Ce.next=ge,A|=me;if(L=L.next,L===null){if(L=h.shared.pending,L===null)break;ge=L,L=ge.next,ge.next=null,h.lastBaseUpdate=ge,h.shared.pending=null}}while(!0);Ce===null&&(Q=Oe),h.baseState=Q,h.firstBaseUpdate=ue,h.lastBaseUpdate=Ce,S===null&&(h.shared.lanes=0),Fo|=A,a.lanes=A,a.memoizedState=Oe}}function Xy(a,i){if(typeof a!="function")throw Error(r(191,a));a.call(i)}function Yy(a,i){var s=a.callbacks;if(s!==null)for(a.callbacks=null,a=0;a<s.length;a++)Xy(s[a],i)}var sl=H(null),Cu=H(0);function Qy(a,i){a=vo,X(Cu,a),X(sl,i),vo=a|i.baseLanes}function Km(){X(Cu,vo),X(sl,sl.current)}function Xm(){vo=Cu.current,G(sl),G(Cu)}var jo=0,Mt=null,fn=null,Fn=null,xu=!1,cl=!1,wi=!1,Eu=0,Ts=0,ul=null,uM=0;function zn(){throw Error(r(321))}function Ym(a,i){if(i===null)return!1;for(var s=0;s<i.length&&s<a.length;s++)if(!Lr(a[s],i[s]))return!1;return!0}function Qm(a,i,s,f,h,S){return jo=S,Mt=i,i.memoizedState=null,i.updateQueue=null,i.lanes=0,_.H=a===null||a.memoizedState===null?jb:Pb,wi=!1,S=s(f,h),wi=!1,cl&&(S=Jy(i,s,f,h)),Zy(a),S}function Zy(a){_.H=Mu;var i=fn!==null&&fn.next!==null;if(jo=0,Fn=fn=Mt=null,xu=!1,Ts=0,ul=null,i)throw Error(r(300));a===null||or||(a=a.dependencies,a!==null&&hu(a)&&(or=!0))}function Jy(a,i,s,f){Mt=a;var h=0;do{if(cl&&(ul=null),Ts=0,cl=!1,25<=h)throw Error(r(301));if(h+=1,Fn=fn=null,a.updateQueue!=null){var S=a.updateQueue;S.lastEffect=null,S.events=null,S.stores=null,S.memoCache!=null&&(S.memoCache.index=0)}_.H=pM,S=i(s,f)}while(cl);return S}function fM(){var a=_.H,i=a.useState()[0];return i=typeof i.then=="function"?Ms(i):i,a=a.useState()[0],(fn!==null?fn.memoizedState:null)!==a&&(Mt.flags|=1024),i}function Zm(){var a=Eu!==0;return Eu=0,a}function Jm(a,i,s){i.updateQueue=a.updateQueue,i.flags&=-2053,a.lanes&=~s}function ev(a){if(xu){for(a=a.memoizedState;a!==null;){var i=a.queue;i!==null&&(i.pending=null),a=a.next}xu=!1}jo=0,Fn=fn=Mt=null,cl=!1,Ts=Eu=0,ul=null}function Nr(){var a={memoizedState:null,baseState:null,baseQueue:null,queue:null,next:null};return Fn===null?Mt.memoizedState=Fn=a:Fn=Fn.next=a,Fn}function Vn(){if(fn===null){var a=Mt.alternate;a=a!==null?a.memoizedState:null}else a=fn.next;var i=Fn===null?Mt.memoizedState:Fn.next;if(i!==null)Fn=i,fn=a;else{if(a===null)throw Mt.alternate===null?Error(r(467)):Error(r(310));fn=a,a={memoizedState:fn.memoizedState,baseState:fn.baseState,baseQueue:fn.baseQueue,queue:fn.queue,next:null},Fn===null?Mt.memoizedState=Fn=a:Fn=Fn.next=a}return Fn}function tv(){return{lastEffect:null,events:null,stores:null,memoCache:null}}function Ms(a){var i=Ts;return Ts+=1,ul===null&&(ul=[]),a=ky(ul,a,i),i=Mt,(Fn===null?i.memoizedState:Fn.next)===null&&(i=i.alternate,_.H=i===null||i.memoizedState===null?jb:Pb),a}function wu(a){if(a!==null&&typeof a=="object"){if(typeof a.then=="function")return Ms(a);if(a.$$typeof===O)return pr(a)}throw Error(r(438,String(a)))}function nv(a){var i=null,s=Mt.updateQueue;if(s!==null&&(i=s.memoCache),i==null){var f=Mt.alternate;f!==null&&(f=f.updateQueue,f!==null&&(f=f.memoCache,f!=null&&(i={data:f.data.map(function(h){return h.slice()}),index:0})))}if(i==null&&(i={data:[],index:0}),s===null&&(s=tv(),Mt.updateQueue=s),s.memoCache=i,s=i.data[i.index],s===void 0)for(s=i.data[i.index]=Array(a),f=0;f<a;f++)s[f]=D;return i.index++,s}function lo(a,i){return typeof i=="function"?i(a):i}function $u(a){var i=Vn();return rv(i,fn,a)}function rv(a,i,s){var f=a.queue;if(f===null)throw Error(r(311));f.lastRenderedReducer=s;var h=a.baseQueue,S=f.pending;if(S!==null){if(h!==null){var A=h.next;h.next=S.next,S.next=A}i.baseQueue=h=S,f.pending=null}if(S=a.baseState,h===null)a.memoizedState=S;else{i=h.next;var L=A=null,Q=null,ue=i,Ce=!1;do{var Oe=ue.lane&-536870913;if(Oe!==ue.lane?(qt&Oe)===Oe:(jo&Oe)===Oe){var me=ue.revertLane;if(me===0)Q!==null&&(Q=Q.next={lane:0,revertLane:0,action:ue.action,hasEagerState:ue.hasEagerState,eagerState:ue.eagerState,next:null}),Oe===il&&(Ce=!0);else if((jo&me)===me){ue=ue.next,me===il&&(Ce=!0);continue}else Oe={lane:0,revertLane:ue.revertLane,action:ue.action,hasEagerState:ue.hasEagerState,eagerState:ue.eagerState,next:null},Q===null?(L=Q=Oe,A=S):Q=Q.next=Oe,Mt.lanes|=me,Fo|=me;Oe=ue.action,wi&&s(S,Oe),S=ue.hasEagerState?ue.eagerState:s(S,Oe)}else me={lane:Oe,revertLane:ue.revertLane,action:ue.action,hasEagerState:ue.hasEagerState,eagerState:ue.eagerState,next:null},Q===null?(L=Q=me,A=S):Q=Q.next=me,Mt.lanes|=Oe,Fo|=Oe;ue=ue.next}while(ue!==null&&ue!==i);if(Q===null?A=S:Q.next=L,!Lr(S,a.memoizedState)&&(or=!0,Ce&&(s=ll,s!==null)))throw s;a.memoizedState=S,a.baseState=A,a.baseQueue=Q,f.lastRenderedState=S}return h===null&&(f.lanes=0),[a.memoizedState,f.dispatch]}function av(a){var i=Vn(),s=i.queue;if(s===null)throw Error(r(311));s.lastRenderedReducer=a;var f=s.dispatch,h=s.pending,S=i.memoizedState;if(h!==null){s.pending=null;var A=h=h.next;do S=a(S,A.action),A=A.next;while(A!==h);Lr(S,i.memoizedState)||(or=!0),i.memoizedState=S,i.baseQueue===null&&(i.baseState=S),s.lastRenderedState=S}return[S,f]}function eb(a,i,s){var f=Mt,h=Vn(),S=Zt;if(S){if(s===void 0)throw Error(r(407));s=s()}else s=i();var A=!Lr((fn||h).memoizedState,s);A&&(h.memoizedState=s,or=!0),h=h.queue;var L=rb.bind(null,f,h,a);if(_s(2048,8,L,[a]),h.getSnapshot!==i||A||Fn!==null&&Fn.memoizedState.tag&1){if(f.flags|=2048,fl(9,Ou(),nb.bind(null,f,h,s,i),null),Cn===null)throw Error(r(349));S||(jo&124)!==0||tb(f,i,s)}return s}function tb(a,i,s){a.flags|=16384,a={getSnapshot:i,value:s},i=Mt.updateQueue,i===null?(i=tv(),Mt.updateQueue=i,i.stores=[a]):(s=i.stores,s===null?i.stores=[a]:s.push(a))}function nb(a,i,s,f){i.value=s,i.getSnapshot=f,ab(i)&&ob(a)}function rb(a,i,s){return s(function(){ab(i)&&ob(a)})}function ab(a){var i=a.getSnapshot;a=a.value;try{var s=i();return!Lr(a,s)}catch{return!0}}function ob(a){var i=nl(a,2);i!==null&&Wr(i,a,2)}function ov(a){var i=Nr();if(typeof a=="function"){var s=a;if(a=s(),wi){at(!0);try{s()}finally{at(!1)}}}return i.memoizedState=i.baseState=a,i.queue={pending:null,lanes:0,dispatch:null,lastRenderedReducer:lo,lastRenderedState:a},i}function ib(a,i,s,f){return a.baseState=s,rv(a,fn,typeof f=="function"?f:lo)}function dM(a,i,s,f,h){if(Tu(a))throw Error(r(485));if(a=i.action,a!==null){var S={payload:h,action:a,next:null,isTransition:!0,status:"pending",value:null,reason:null,listeners:[],then:function(A){S.listeners.push(A)}};_.T!==null?s(!0):S.isTransition=!1,f(S),s=i.pending,s===null?(S.next=i.pending=S,lb(i,S)):(S.next=s.next,i.pending=s.next=S)}}function lb(a,i){var s=i.action,f=i.payload,h=a.state;if(i.isTransition){var S=_.T,A={};_.T=A;try{var L=s(h,f),Q=_.S;Q!==null&&Q(A,L),sb(a,i,L)}catch(ue){iv(a,i,ue)}finally{_.T=S}}else try{S=s(h,f),sb(a,i,S)}catch(ue){iv(a,i,ue)}}function sb(a,i,s){s!==null&&typeof s=="object"&&typeof s.then=="function"?s.then(function(f){cb(a,i,f)},function(f){return iv(a,i,f)}):cb(a,i,s)}function cb(a,i,s){i.status="fulfilled",i.value=s,ub(i),a.state=s,i=a.pending,i!==null&&(s=i.next,s===i?a.pending=null:(s=s.next,i.next=s,lb(a,s)))}function iv(a,i,s){var f=a.pending;if(a.pending=null,f!==null){f=f.next;do i.status="rejected",i.reason=s,ub(i),i=i.next;while(i!==f)}a.action=null}function ub(a){a=a.listeners;for(var i=0;i<a.length;i++)(0,a[i])()}function fb(a,i){return i}function db(a,i){if(Zt){var s=Cn.formState;if(s!==null){e:{var f=Mt;if(Zt){if(An){t:{for(var h=An,S=La;h.nodeType!==8;){if(!S){h=null;break t}if(h=wa(h.nextSibling),h===null){h=null;break t}}S=h.data,h=S==="F!"||S==="F"?h:null}if(h){An=wa(h.nextSibling),f=h.data==="F!";break e}}Si(f)}f=!1}f&&(i=s[0])}}return s=Nr(),s.memoizedState=s.baseState=i,f={pending:null,lanes:0,dispatch:null,lastRenderedReducer:fb,lastRenderedState:i},s.queue=f,s=Nb.bind(null,Mt,f),f.dispatch=s,f=ov(!1),S=fv.bind(null,Mt,!1,f.queue),f=Nr(),h={state:i,dispatch:null,action:a,pending:null},f.queue=h,s=dM.bind(null,Mt,h,S,s),h.dispatch=s,f.memoizedState=a,[i,s,!1]}function mb(a){var i=Vn();return vb(i,fn,a)}function vb(a,i,s){if(i=rv(a,i,fb)[0],a=$u(lo)[0],typeof i=="object"&&i!==null&&typeof i.then=="function")try{var f=Ms(i)}catch(A){throw A===Es?bu:A}else f=i;i=Vn();var h=i.queue,S=h.dispatch;return s!==i.memoizedState&&(Mt.flags|=2048,fl(9,Ou(),mM.bind(null,h,s),null)),[f,S,a]}function mM(a,i){a.action=i}function gb(a){var i=Vn(),s=fn;if(s!==null)return vb(i,s,a);Vn(),i=i.memoizedState,s=Vn();var f=s.queue.dispatch;return s.memoizedState=a,[i,f,!1]}function fl(a,i,s,f){return a={tag:a,create:s,deps:f,inst:i,next:null},i=Mt.updateQueue,i===null&&(i=tv(),Mt.updateQueue=i),s=i.lastEffect,s===null?i.lastEffect=a.next=a:(f=s.next,s.next=a,a.next=f,i.lastEffect=a),a}function Ou(){return{destroy:void 0,resource:void 0}}function hb(){return Vn().memoizedState}function Ru(a,i,s,f){var h=Nr();f=f===void 0?null:f,Mt.flags|=a,h.memoizedState=fl(1|i,Ou(),s,f)}function _s(a,i,s,f){var h=Vn();f=f===void 0?null:f;var S=h.memoizedState.inst;fn!==null&&f!==null&&Ym(f,fn.memoizedState.deps)?h.memoizedState=fl(i,S,s,f):(Mt.flags|=a,h.memoizedState=fl(1|i,S,s,f))}function pb(a,i){Ru(8390656,8,a,i)}function yb(a,i){_s(2048,8,a,i)}function bb(a,i){return _s(4,2,a,i)}function Sb(a,i){return _s(4,4,a,i)}function Cb(a,i){if(typeof i=="function"){a=a();var s=i(a);return function(){typeof s=="function"?s():i(null)}}if(i!=null)return a=a(),i.current=a,function(){i.current=null}}function xb(a,i,s){s=s!=null?s.concat([a]):null,_s(4,4,Cb.bind(null,i,a),s)}function lv(){}function Eb(a,i){var s=Vn();i=i===void 0?null:i;var f=s.memoizedState;return i!==null&&Ym(i,f[1])?f[0]:(s.memoizedState=[a,i],a)}function wb(a,i){var s=Vn();i=i===void 0?null:i;var f=s.memoizedState;if(i!==null&&Ym(i,f[1]))return f[0];if(f=a(),wi){at(!0);try{a()}finally{at(!1)}}return s.memoizedState=[f,i],f}function sv(a,i,s){return s===void 0||(jo&1073741824)!==0?a.memoizedState=i:(a.memoizedState=s,a=RS(),Mt.lanes|=a,Fo|=a,s)}function $b(a,i,s,f){return Lr(s,i)?s:sl.current!==null?(a=sv(a,s,f),Lr(a,i)||(or=!0),a):(jo&42)===0?(or=!0,a.memoizedState=s):(a=RS(),Mt.lanes|=a,Fo|=a,i)}function Ob(a,i,s,f,h){var S=j.p;j.p=S!==0&&8>S?S:8;var A=_.T,L={};_.T=L,fv(a,!1,i,s);try{var Q=h(),ue=_.S;if(ue!==null&&ue(L,Q),Q!==null&&typeof Q=="object"&&typeof Q.then=="function"){var Ce=cM(Q,f);Ns(a,i,Ce,qr(a))}else Ns(a,i,f,qr(a))}catch(Oe){Ns(a,i,{then:function(){},status:"rejected",reason:Oe},qr())}finally{j.p=S,_.T=A}}function vM(){}function cv(a,i,s,f){if(a.tag!==5)throw Error(r(476));var h=Rb(a).queue;Ob(a,h,i,B,s===null?vM:function(){return Tb(a),s(f)})}function Rb(a){var i=a.memoizedState;if(i!==null)return i;i={memoizedState:B,baseState:B,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:lo,lastRenderedState:B},next:null};var s={};return i.next={memoizedState:s,baseState:s,baseQueue:null,queue:{pending:null,lanes:0,dispatch:null,lastRenderedReducer:lo,lastRenderedState:s},next:null},a.memoizedState=i,a=a.alternate,a!==null&&(a.memoizedState=i),i}function Tb(a){var i=Rb(a).next.queue;Ns(a,i,{},qr())}function uv(){return pr(Xs)}function Mb(){return Vn().memoizedState}function _b(){return Vn().memoizedState}function gM(a){for(var i=a.return;i!==null;){switch(i.tag){case 24:case 3:var s=qr();a=Ao(s);var f=Io(i,a,s);f!==null&&(Wr(f,i,s),$s(f,i,s)),i={cache:Hm()},a.payload=i;return}i=i.return}}function hM(a,i,s){var f=qr();s={lane:f,revertLane:0,action:s,hasEagerState:!1,eagerState:null,next:null},Tu(a)?Ab(i,s):(s=_m(a,i,s,f),s!==null&&(Wr(s,a,f),Ib(s,i,f)))}function Nb(a,i,s){var f=qr();Ns(a,i,s,f)}function Ns(a,i,s,f){var h={lane:f,revertLane:0,action:s,hasEagerState:!1,eagerState:null,next:null};if(Tu(a))Ab(i,h);else{var S=a.alternate;if(a.lanes===0&&(S===null||S.lanes===0)&&(S=i.lastRenderedReducer,S!==null))try{var A=i.lastRenderedState,L=S(A,s);if(h.hasEagerState=!0,h.eagerState=L,Lr(L,A))return fu(a,i,h,0),Cn===null&&uu(),!1}catch{}finally{}if(s=_m(a,i,h,f),s!==null)return Wr(s,a,f),Ib(s,i,f),!0}return!1}function fv(a,i,s,f){if(f={lane:2,revertLane:Uv(),action:f,hasEagerState:!1,eagerState:null,next:null},Tu(a)){if(i)throw Error(r(479))}else i=_m(a,s,f,2),i!==null&&Wr(i,a,2)}function Tu(a){var i=a.alternate;return a===Mt||i!==null&&i===Mt}function Ab(a,i){cl=xu=!0;var s=a.pending;s===null?i.next=i:(i.next=s.next,s.next=i),a.pending=i}function Ib(a,i,s){if((s&4194048)!==0){var f=i.lanes;f&=a.pendingLanes,s|=f,i.lanes=s,nn(a,s)}}var Mu={readContext:pr,use:wu,useCallback:zn,useContext:zn,useEffect:zn,useImperativeHandle:zn,useLayoutEffect:zn,useInsertionEffect:zn,useMemo:zn,useReducer:zn,useRef:zn,useState:zn,useDebugValue:zn,useDeferredValue:zn,useTransition:zn,useSyncExternalStore:zn,useId:zn,useHostTransitionStatus:zn,useFormState:zn,useActionState:zn,useOptimistic:zn,useMemoCache:zn,useCacheRefresh:zn},jb={readContext:pr,use:wu,useCallback:function(a,i){return Nr().memoizedState=[a,i===void 0?null:i],a},useContext:pr,useEffect:pb,useImperativeHandle:function(a,i,s){s=s!=null?s.concat([a]):null,Ru(4194308,4,Cb.bind(null,i,a),s)},useLayoutEffect:function(a,i){return Ru(4194308,4,a,i)},useInsertionEffect:function(a,i){Ru(4,2,a,i)},useMemo:function(a,i){var s=Nr();i=i===void 0?null:i;var f=a();if(wi){at(!0);try{a()}finally{at(!1)}}return s.memoizedState=[f,i],f},useReducer:function(a,i,s){var f=Nr();if(s!==void 0){var h=s(i);if(wi){at(!0);try{s(i)}finally{at(!1)}}}else h=i;return f.memoizedState=f.baseState=h,a={pending:null,lanes:0,dispatch:null,lastRenderedReducer:a,lastRenderedState:h},f.queue=a,a=a.dispatch=hM.bind(null,Mt,a),[f.memoizedState,a]},useRef:function(a){var i=Nr();return a={current:a},i.memoizedState=a},useState:function(a){a=ov(a);var i=a.queue,s=Nb.bind(null,Mt,i);return i.dispatch=s,[a.memoizedState,s]},useDebugValue:lv,useDeferredValue:function(a,i){var s=Nr();return sv(s,a,i)},useTransition:function(){var a=ov(!1);return a=Ob.bind(null,Mt,a.queue,!0,!1),Nr().memoizedState=a,[!1,a]},useSyncExternalStore:function(a,i,s){var f=Mt,h=Nr();if(Zt){if(s===void 0)throw Error(r(407));s=s()}else{if(s=i(),Cn===null)throw Error(r(349));(qt&124)!==0||tb(f,i,s)}h.memoizedState=s;var S={value:s,getSnapshot:i};return h.queue=S,pb(rb.bind(null,f,S,a),[a]),f.flags|=2048,fl(9,Ou(),nb.bind(null,f,S,s,i),null),s},useId:function(){var a=Nr(),i=Cn.identifierPrefix;if(Zt){var s=ao,f=ro;s=(f&~(1<<32-Ue(f)-1)).toString(32)+s,i="«"+i+"R"+s,s=Eu++,0<s&&(i+="H"+s.toString(32)),i+="»"}else s=uM++,i="«"+i+"r"+s.toString(32)+"»";return a.memoizedState=i},useHostTransitionStatus:uv,useFormState:db,useActionState:db,useOptimistic:function(a){var i=Nr();i.memoizedState=i.baseState=a;var s={pending:null,lanes:0,dispatch:null,lastRenderedReducer:null,lastRenderedState:null};return i.queue=s,i=fv.bind(null,Mt,!0,s),s.dispatch=i,[a,i]},useMemoCache:nv,useCacheRefresh:function(){return Nr().memoizedState=gM.bind(null,Mt)}},Pb={readContext:pr,use:wu,useCallback:Eb,useContext:pr,useEffect:yb,useImperativeHandle:xb,useInsertionEffect:bb,useLayoutEffect:Sb,useMemo:wb,useReducer:$u,useRef:hb,useState:function(){return $u(lo)},useDebugValue:lv,useDeferredValue:function(a,i){var s=Vn();return $b(s,fn.memoizedState,a,i)},useTransition:function(){var a=$u(lo)[0],i=Vn().memoizedState;return[typeof a=="boolean"?a:Ms(a),i]},useSyncExternalStore:eb,useId:Mb,useHostTransitionStatus:uv,useFormState:mb,useActionState:mb,useOptimistic:function(a,i){var s=Vn();return ib(s,fn,a,i)},useMemoCache:nv,useCacheRefresh:_b},pM={readContext:pr,use:wu,useCallback:Eb,useContext:pr,useEffect:yb,useImperativeHandle:xb,useInsertionEffect:bb,useLayoutEffect:Sb,useMemo:wb,useReducer:av,useRef:hb,useState:function(){return av(lo)},useDebugValue:lv,useDeferredValue:function(a,i){var s=Vn();return fn===null?sv(s,a,i):$b(s,fn.memoizedState,a,i)},useTransition:function(){var a=av(lo)[0],i=Vn().memoizedState;return[typeof a=="boolean"?a:Ms(a),i]},useSyncExternalStore:eb,useId:Mb,useHostTransitionStatus:uv,useFormState:gb,useActionState:gb,useOptimistic:function(a,i){var s=Vn();return fn!==null?ib(s,fn,a,i):(s.baseState=a,[a,s.queue.dispatch])},useMemoCache:nv,useCacheRefresh:_b},dl=null,As=0;function _u(a){var i=As;return As+=1,dl===null&&(dl=[]),ky(dl,a,i)}function Is(a,i){i=i.props.ref,a.ref=i!==void 0?i:null}function Nu(a,i){throw i.$$typeof===p?Error(r(525)):(a=Object.prototype.toString.call(i),Error(r(31,a==="[object Object]"?"object with keys {"+Object.keys(i).join(", ")+"}":a)))}function zb(a){var i=a._init;return i(a._payload)}function Db(a){function i(ae,ee){if(a){var ce=ae.deletions;ce===null?(ae.deletions=[ee],ae.flags|=16):ce.push(ee)}}function s(ae,ee){if(!a)return null;for(;ee!==null;)i(ae,ee),ee=ee.sibling;return null}function f(ae){for(var ee=new Map;ae!==null;)ae.key!==null?ee.set(ae.key,ae):ee.set(ae.index,ae),ae=ae.sibling;return ee}function h(ae,ee){return ae=no(ae,ee),ae.index=0,ae.sibling=null,ae}function S(ae,ee,ce){return ae.index=ce,a?(ce=ae.alternate,ce!==null?(ce=ce.index,ce<ee?(ae.flags|=67108866,ee):ce):(ae.flags|=67108866,ee)):(ae.flags|=1048576,ee)}function A(ae){return a&&ae.alternate===null&&(ae.flags|=67108866),ae}function L(ae,ee,ce,Ee){return ee===null||ee.tag!==6?(ee=Am(ce,ae.mode,Ee),ee.return=ae,ee):(ee=h(ee,ce),ee.return=ae,ee)}function Q(ae,ee,ce,Ee){var it=ce.type;return it===C?Ce(ae,ee,ce.props.children,Ee,ce.key):ee!==null&&(ee.elementType===it||typeof it=="object"&&it!==null&&it.$$typeof===z&&zb(it)===ee.type)?(ee=h(ee,ce.props),Is(ee,ce),ee.return=ae,ee):(ee=mu(ce.type,ce.key,ce.props,null,ae.mode,Ee),Is(ee,ce),ee.return=ae,ee)}function ue(ae,ee,ce,Ee){return ee===null||ee.tag!==4||ee.stateNode.containerInfo!==ce.containerInfo||ee.stateNode.implementation!==ce.implementation?(ee=Im(ce,ae.mode,Ee),ee.return=ae,ee):(ee=h(ee,ce.children||[]),ee.return=ae,ee)}function Ce(ae,ee,ce,Ee,it){return ee===null||ee.tag!==7?(ee=hi(ce,ae.mode,Ee,it),ee.return=ae,ee):(ee=h(ee,ce),ee.return=ae,ee)}function Oe(ae,ee,ce){if(typeof ee=="string"&&ee!==""||typeof ee=="number"||typeof ee=="bigint")return ee=Am(""+ee,ae.mode,ce),ee.return=ae,ee;if(typeof ee=="object"&&ee!==null){switch(ee.$$typeof){case y:return ce=mu(ee.type,ee.key,ee.props,null,ae.mode,ce),Is(ce,ee),ce.return=ae,ce;case E:return ee=Im(ee,ae.mode,ce),ee.return=ae,ee;case z:var Ee=ee._init;return ee=Ee(ee._payload),Oe(ae,ee,ce)}if(K(ee)||W(ee))return ee=hi(ee,ae.mode,ce,null),ee.return=ae,ee;if(typeof ee.then=="function")return Oe(ae,_u(ee),ce);if(ee.$$typeof===O)return Oe(ae,pu(ae,ee),ce);Nu(ae,ee)}return null}function me(ae,ee,ce,Ee){var it=ee!==null?ee.key:null;if(typeof ce=="string"&&ce!==""||typeof ce=="number"||typeof ce=="bigint")return it!==null?null:L(ae,ee,""+ce,Ee);if(typeof ce=="object"&&ce!==null){switch(ce.$$typeof){case y:return ce.key===it?Q(ae,ee,ce,Ee):null;case E:return ce.key===it?ue(ae,ee,ce,Ee):null;case z:return it=ce._init,ce=it(ce._payload),me(ae,ee,ce,Ee)}if(K(ce)||W(ce))return it!==null?null:Ce(ae,ee,ce,Ee,null);if(typeof ce.then=="function")return me(ae,ee,_u(ce),Ee);if(ce.$$typeof===O)return me(ae,ee,pu(ae,ce),Ee);Nu(ae,ce)}return null}function ge(ae,ee,ce,Ee,it){if(typeof Ee=="string"&&Ee!==""||typeof Ee=="number"||typeof Ee=="bigint")return ae=ae.get(ce)||null,L(ee,ae,""+Ee,it);if(typeof Ee=="object"&&Ee!==null){switch(Ee.$$typeof){case y:return ae=ae.get(Ee.key===null?ce:Ee.key)||null,Q(ee,ae,Ee,it);case E:return ae=ae.get(Ee.key===null?ce:Ee.key)||null,ue(ee,ae,Ee,it);case z:var jt=Ee._init;return Ee=jt(Ee._payload),ge(ae,ee,ce,Ee,it)}if(K(Ee)||W(Ee))return ae=ae.get(ce)||null,Ce(ee,ae,Ee,it,null);if(typeof Ee.then=="function")return ge(ae,ee,ce,_u(Ee),it);if(Ee.$$typeof===O)return ge(ae,ee,ce,pu(ee,Ee),it);Nu(ee,Ee)}return null}function bt(ae,ee,ce,Ee){for(var it=null,jt=null,ft=ee,gt=ee=0,lr=null;ft!==null&&gt<ce.length;gt++){ft.index>gt?(lr=ft,ft=null):lr=ft.sibling;var Yt=me(ae,ft,ce[gt],Ee);if(Yt===null){ft===null&&(ft=lr);break}a&&ft&&Yt.alternate===null&&i(ae,ft),ee=S(Yt,ee,gt),jt===null?it=Yt:jt.sibling=Yt,jt=Yt,ft=lr}if(gt===ce.length)return s(ae,ft),Zt&&yi(ae,gt),it;if(ft===null){for(;gt<ce.length;gt++)ft=Oe(ae,ce[gt],Ee),ft!==null&&(ee=S(ft,ee,gt),jt===null?it=ft:jt.sibling=ft,jt=ft);return Zt&&yi(ae,gt),it}for(ft=f(ft);gt<ce.length;gt++)lr=ge(ft,ae,gt,ce[gt],Ee),lr!==null&&(a&&lr.alternate!==null&&ft.delete(lr.key===null?gt:lr.key),ee=S(lr,ee,gt),jt===null?it=lr:jt.sibling=lr,jt=lr);return a&&ft.forEach(function(Yo){return i(ae,Yo)}),Zt&&yi(ae,gt),it}function mt(ae,ee,ce,Ee){if(ce==null)throw Error(r(151));for(var it=null,jt=null,ft=ee,gt=ee=0,lr=null,Yt=ce.next();ft!==null&&!Yt.done;gt++,Yt=ce.next()){ft.index>gt?(lr=ft,ft=null):lr=ft.sibling;var Yo=me(ae,ft,Yt.value,Ee);if(Yo===null){ft===null&&(ft=lr);break}a&&ft&&Yo.alternate===null&&i(ae,ft),ee=S(Yo,ee,gt),jt===null?it=Yo:jt.sibling=Yo,jt=Yo,ft=lr}if(Yt.done)return s(ae,ft),Zt&&yi(ae,gt),it;if(ft===null){for(;!Yt.done;gt++,Yt=ce.next())Yt=Oe(ae,Yt.value,Ee),Yt!==null&&(ee=S(Yt,ee,gt),jt===null?it=Yt:jt.sibling=Yt,jt=Yt);return Zt&&yi(ae,gt),it}for(ft=f(ft);!Yt.done;gt++,Yt=ce.next())Yt=ge(ft,ae,gt,Yt.value,Ee),Yt!==null&&(a&&Yt.alternate!==null&&ft.delete(Yt.key===null?gt:Yt.key),ee=S(Yt,ee,gt),jt===null?it=Yt:jt.sibling=Yt,jt=Yt);return a&&ft.forEach(function(y_){return i(ae,y_)}),Zt&&yi(ae,gt),it}function mn(ae,ee,ce,Ee){if(typeof ce=="object"&&ce!==null&&ce.type===C&&ce.key===null&&(ce=ce.props.children),typeof ce=="object"&&ce!==null){switch(ce.$$typeof){case y:e:{for(var it=ce.key;ee!==null;){if(ee.key===it){if(it=ce.type,it===C){if(ee.tag===7){s(ae,ee.sibling),Ee=h(ee,ce.props.children),Ee.return=ae,ae=Ee;break e}}else if(ee.elementType===it||typeof it=="object"&&it!==null&&it.$$typeof===z&&zb(it)===ee.type){s(ae,ee.sibling),Ee=h(ee,ce.props),Is(Ee,ce),Ee.return=ae,ae=Ee;break e}s(ae,ee);break}else i(ae,ee);ee=ee.sibling}ce.type===C?(Ee=hi(ce.props.children,ae.mode,Ee,ce.key),Ee.return=ae,ae=Ee):(Ee=mu(ce.type,ce.key,ce.props,null,ae.mode,Ee),Is(Ee,ce),Ee.return=ae,ae=Ee)}return A(ae);case E:e:{for(it=ce.key;ee!==null;){if(ee.key===it)if(ee.tag===4&&ee.stateNode.containerInfo===ce.containerInfo&&ee.stateNode.implementation===ce.implementation){s(ae,ee.sibling),Ee=h(ee,ce.children||[]),Ee.return=ae,ae=Ee;break e}else{s(ae,ee);break}else i(ae,ee);ee=ee.sibling}Ee=Im(ce,ae.mode,Ee),Ee.return=ae,ae=Ee}return A(ae);case z:return it=ce._init,ce=it(ce._payload),mn(ae,ee,ce,Ee)}if(K(ce))return bt(ae,ee,ce,Ee);if(W(ce)){if(it=W(ce),typeof it!="function")throw Error(r(150));return ce=it.call(ce),mt(ae,ee,ce,Ee)}if(typeof ce.then=="function")return mn(ae,ee,_u(ce),Ee);if(ce.$$typeof===O)return mn(ae,ee,pu(ae,ce),Ee);Nu(ae,ce)}return typeof ce=="string"&&ce!==""||typeof ce=="number"||typeof ce=="bigint"?(ce=""+ce,ee!==null&&ee.tag===6?(s(ae,ee.sibling),Ee=h(ee,ce),Ee.return=ae,ae=Ee):(s(ae,ee),Ee=Am(ce,ae.mode,Ee),Ee.return=ae,ae=Ee),A(ae)):s(ae,ee)}return function(ae,ee,ce,Ee){try{As=0;var it=mn(ae,ee,ce,Ee);return dl=null,it}catch(ft){if(ft===Es||ft===bu)throw ft;var jt=Hr(29,ft,null,ae.mode);return jt.lanes=Ee,jt.return=ae,jt}finally{}}}var ml=Db(!0),Bb=Db(!1),ua=H(null),Ha=null;function Po(a){var i=a.alternate;X(Qn,Qn.current&1),X(ua,a),Ha===null&&(i===null||sl.current!==null||i.memoizedState!==null)&&(Ha=a)}function Lb(a){if(a.tag===22){if(X(Qn,Qn.current),X(ua,a),Ha===null){var i=a.alternate;i!==null&&i.memoizedState!==null&&(Ha=a)}}else zo()}function zo(){X(Qn,Qn.current),X(ua,ua.current)}function so(a){G(ua),Ha===a&&(Ha=null),G(Qn)}var Qn=H(0);function Au(a){for(var i=a;i!==null;){if(i.tag===13){var s=i.memoizedState;if(s!==null&&(s=s.dehydrated,s===null||s.data==="$?"||tg(s)))return i}else if(i.tag===19&&i.memoizedProps.revealOrder!==void 0){if((i.flags&128)!==0)return i}else if(i.child!==null){i.child.return=i,i=i.child;continue}if(i===a)break;for(;i.sibling===null;){if(i.return===null||i.return===a)return null;i=i.return}i.sibling.return=i.return,i=i.sibling}return null}function dv(a,i,s,f){i=a.memoizedState,s=s(f,i),s=s==null?i:g({},i,s),a.memoizedState=s,a.lanes===0&&(a.updateQueue.baseState=s)}var mv={enqueueSetState:function(a,i,s){a=a._reactInternals;var f=qr(),h=Ao(f);h.payload=i,s!=null&&(h.callback=s),i=Io(a,h,f),i!==null&&(Wr(i,a,f),$s(i,a,f))},enqueueReplaceState:function(a,i,s){a=a._reactInternals;var f=qr(),h=Ao(f);h.tag=1,h.payload=i,s!=null&&(h.callback=s),i=Io(a,h,f),i!==null&&(Wr(i,a,f),$s(i,a,f))},enqueueForceUpdate:function(a,i){a=a._reactInternals;var s=qr(),f=Ao(s);f.tag=2,i!=null&&(f.callback=i),i=Io(a,f,s),i!==null&&(Wr(i,a,s),$s(i,a,s))}};function Hb(a,i,s,f,h,S,A){return a=a.stateNode,typeof a.shouldComponentUpdate=="function"?a.shouldComponentUpdate(f,S,A):i.prototype&&i.prototype.isPureReactComponent?!gs(s,f)||!gs(h,S):!0}function Fb(a,i,s,f){a=i.state,typeof i.componentWillReceiveProps=="function"&&i.componentWillReceiveProps(s,f),typeof i.UNSAFE_componentWillReceiveProps=="function"&&i.UNSAFE_componentWillReceiveProps(s,f),i.state!==a&&mv.enqueueReplaceState(i,i.state,null)}function $i(a,i){var s=i;if("ref"in i){s={};for(var f in i)f!=="ref"&&(s[f]=i[f])}if(a=a.defaultProps){s===i&&(s=g({},s));for(var h in a)s[h]===void 0&&(s[h]=a[h])}return s}var Iu=typeof reportError=="function"?reportError:function(a){if(typeof window=="object"&&typeof window.ErrorEvent=="function"){var i=new window.ErrorEvent("error",{bubbles:!0,cancelable:!0,message:typeof a=="object"&&a!==null&&typeof a.message=="string"?String(a.message):String(a),error:a});if(!window.dispatchEvent(i))return}else if(typeof process=="object"&&typeof process.emit=="function"){process.emit("uncaughtException",a);return}console.error(a)};function Vb(a){Iu(a)}function Ub(a){console.error(a)}function qb(a){Iu(a)}function ju(a,i){try{var s=a.onUncaughtError;s(i.value,{componentStack:i.stack})}catch(f){setTimeout(function(){throw f})}}function Wb(a,i,s){try{var f=a.onCaughtError;f(s.value,{componentStack:s.stack,errorBoundary:i.tag===1?i.stateNode:null})}catch(h){setTimeout(function(){throw h})}}function vv(a,i,s){return s=Ao(s),s.tag=3,s.payload={element:null},s.callback=function(){ju(a,i)},s}function kb(a){return a=Ao(a),a.tag=3,a}function Gb(a,i,s,f){var h=s.type.getDerivedStateFromError;if(typeof h=="function"){var S=f.value;a.payload=function(){return h(S)},a.callback=function(){Wb(i,s,f)}}var A=s.stateNode;A!==null&&typeof A.componentDidCatch=="function"&&(a.callback=function(){Wb(i,s,f),typeof h!="function"&&(Vo===null?Vo=new Set([this]):Vo.add(this));var L=f.stack;this.componentDidCatch(f.value,{componentStack:L!==null?L:""})})}function yM(a,i,s,f,h){if(s.flags|=32768,f!==null&&typeof f=="object"&&typeof f.then=="function"){if(i=s.alternate,i!==null&&Ss(i,s,h,!0),s=ua.current,s!==null){switch(s.tag){case 13:return Ha===null?Bv():s.alternate===null&&In===0&&(In=3),s.flags&=-257,s.flags|=65536,s.lanes=h,f===Um?s.flags|=16384:(i=s.updateQueue,i===null?s.updateQueue=new Set([f]):i.add(f),Hv(a,f,h)),!1;case 22:return s.flags|=65536,f===Um?s.flags|=16384:(i=s.updateQueue,i===null?(i={transitions:null,markerInstances:null,retryQueue:new Set([f])},s.updateQueue=i):(s=i.retryQueue,s===null?i.retryQueue=new Set([f]):s.add(f)),Hv(a,f,h)),!1}throw Error(r(435,s.tag))}return Hv(a,f,h),Bv(),!1}if(Zt)return i=ua.current,i!==null?((i.flags&65536)===0&&(i.flags|=256),i.flags|=65536,i.lanes=h,f!==zm&&(a=Error(r(422),{cause:f}),bs(ia(a,s)))):(f!==zm&&(i=Error(r(423),{cause:f}),bs(ia(i,s))),a=a.current.alternate,a.flags|=65536,h&=-h,a.lanes|=h,f=ia(f,s),h=vv(a.stateNode,f,h),km(a,h),In!==4&&(In=2)),!1;var S=Error(r(520),{cause:f});if(S=ia(S,s),Hs===null?Hs=[S]:Hs.push(S),In!==4&&(In=2),i===null)return!0;f=ia(f,s),s=i;do{switch(s.tag){case 3:return s.flags|=65536,a=h&-h,s.lanes|=a,a=vv(s.stateNode,f,a),km(s,a),!1;case 1:if(i=s.type,S=s.stateNode,(s.flags&128)===0&&(typeof i.getDerivedStateFromError=="function"||S!==null&&typeof S.componentDidCatch=="function"&&(Vo===null||!Vo.has(S))))return s.flags|=65536,h&=-h,s.lanes|=h,h=kb(h),Gb(h,a,s,f),km(s,h),!1}s=s.return}while(s!==null);return!1}var Kb=Error(r(461)),or=!1;function ur(a,i,s,f){i.child=a===null?Bb(i,null,s,f):ml(i,a.child,s,f)}function Xb(a,i,s,f,h){s=s.render;var S=i.ref;if("ref"in f){var A={};for(var L in f)L!=="ref"&&(A[L]=f[L])}else A=f;return xi(i),f=Qm(a,i,s,A,S,h),L=Zm(),a!==null&&!or?(Jm(a,i,h),co(a,i,h)):(Zt&&L&&jm(i),i.flags|=1,ur(a,i,f,h),i.child)}function Yb(a,i,s,f,h){if(a===null){var S=s.type;return typeof S=="function"&&!Nm(S)&&S.defaultProps===void 0&&s.compare===null?(i.tag=15,i.type=S,Qb(a,i,S,f,h)):(a=mu(s.type,null,f,i,i.mode,h),a.ref=i.ref,a.return=i,i.child=a)}if(S=a.child,!xv(a,h)){var A=S.memoizedProps;if(s=s.compare,s=s!==null?s:gs,s(A,f)&&a.ref===i.ref)return co(a,i,h)}return i.flags|=1,a=no(S,f),a.ref=i.ref,a.return=i,i.child=a}function Qb(a,i,s,f,h){if(a!==null){var S=a.memoizedProps;if(gs(S,f)&&a.ref===i.ref)if(or=!1,i.pendingProps=f=S,xv(a,h))(a.flags&131072)!==0&&(or=!0);else return i.lanes=a.lanes,co(a,i,h)}return gv(a,i,s,f,h)}function Zb(a,i,s){var f=i.pendingProps,h=f.children,S=a!==null?a.memoizedState:null;if(f.mode==="hidden"){if((i.flags&128)!==0){if(f=S!==null?S.baseLanes|s:s,a!==null){for(h=i.child=a.child,S=0;h!==null;)S=S|h.lanes|h.childLanes,h=h.sibling;i.childLanes=S&~f}else i.childLanes=0,i.child=null;return Jb(a,i,f,s)}if((s&536870912)!==0)i.memoizedState={baseLanes:0,cachePool:null},a!==null&&yu(i,S!==null?S.cachePool:null),S!==null?Qy(i,S):Km(),Lb(i);else return i.lanes=i.childLanes=536870912,Jb(a,i,S!==null?S.baseLanes|s:s,s)}else S!==null?(yu(i,S.cachePool),Qy(i,S),zo(),i.memoizedState=null):(a!==null&&yu(i,null),Km(),zo());return ur(a,i,h,s),i.child}function Jb(a,i,s,f){var h=Vm();return h=h===null?null:{parent:Yn._currentValue,pool:h},i.memoizedState={baseLanes:s,cachePool:h},a!==null&&yu(i,null),Km(),Lb(i),a!==null&&Ss(a,i,f,!0),null}function Pu(a,i){var s=i.ref;if(s===null)a!==null&&a.ref!==null&&(i.flags|=4194816);else{if(typeof s!="function"&&typeof s!="object")throw Error(r(284));(a===null||a.ref!==s)&&(i.flags|=4194816)}}function gv(a,i,s,f,h){return xi(i),s=Qm(a,i,s,f,void 0,h),f=Zm(),a!==null&&!or?(Jm(a,i,h),co(a,i,h)):(Zt&&f&&jm(i),i.flags|=1,ur(a,i,s,h),i.child)}function eS(a,i,s,f,h,S){return xi(i),i.updateQueue=null,s=Jy(i,f,s,h),Zy(a),f=Zm(),a!==null&&!or?(Jm(a,i,S),co(a,i,S)):(Zt&&f&&jm(i),i.flags|=1,ur(a,i,s,S),i.child)}function tS(a,i,s,f,h){if(xi(i),i.stateNode===null){var S=rl,A=s.contextType;typeof A=="object"&&A!==null&&(S=pr(A)),S=new s(f,S),i.memoizedState=S.state!==null&&S.state!==void 0?S.state:null,S.updater=mv,i.stateNode=S,S._reactInternals=i,S=i.stateNode,S.props=f,S.state=i.memoizedState,S.refs={},qm(i),A=s.contextType,S.context=typeof A=="object"&&A!==null?pr(A):rl,S.state=i.memoizedState,A=s.getDerivedStateFromProps,typeof A=="function"&&(dv(i,s,A,f),S.state=i.memoizedState),typeof s.getDerivedStateFromProps=="function"||typeof S.getSnapshotBeforeUpdate=="function"||typeof S.UNSAFE_componentWillMount!="function"&&typeof S.componentWillMount!="function"||(A=S.state,typeof S.componentWillMount=="function"&&S.componentWillMount(),typeof S.UNSAFE_componentWillMount=="function"&&S.UNSAFE_componentWillMount(),A!==S.state&&mv.enqueueReplaceState(S,S.state,null),Rs(i,f,S,h),Os(),S.state=i.memoizedState),typeof S.componentDidMount=="function"&&(i.flags|=4194308),f=!0}else if(a===null){S=i.stateNode;var L=i.memoizedProps,Q=$i(s,L);S.props=Q;var ue=S.context,Ce=s.contextType;A=rl,typeof Ce=="object"&&Ce!==null&&(A=pr(Ce));var Oe=s.getDerivedStateFromProps;Ce=typeof Oe=="function"||typeof S.getSnapshotBeforeUpdate=="function",L=i.pendingProps!==L,Ce||typeof S.UNSAFE_componentWillReceiveProps!="function"&&typeof S.componentWillReceiveProps!="function"||(L||ue!==A)&&Fb(i,S,f,A),No=!1;var me=i.memoizedState;S.state=me,Rs(i,f,S,h),Os(),ue=i.memoizedState,L||me!==ue||No?(typeof Oe=="function"&&(dv(i,s,Oe,f),ue=i.memoizedState),(Q=No||Hb(i,s,Q,f,me,ue,A))?(Ce||typeof S.UNSAFE_componentWillMount!="function"&&typeof S.componentWillMount!="function"||(typeof S.componentWillMount=="function"&&S.componentWillMount(),typeof S.UNSAFE_componentWillMount=="function"&&S.UNSAFE_componentWillMount()),typeof S.componentDidMount=="function"&&(i.flags|=4194308)):(typeof S.componentDidMount=="function"&&(i.flags|=4194308),i.memoizedProps=f,i.memoizedState=ue),S.props=f,S.state=ue,S.context=A,f=Q):(typeof S.componentDidMount=="function"&&(i.flags|=4194308),f=!1)}else{S=i.stateNode,Wm(a,i),A=i.memoizedProps,Ce=$i(s,A),S.props=Ce,Oe=i.pendingProps,me=S.context,ue=s.contextType,Q=rl,typeof ue=="object"&&ue!==null&&(Q=pr(ue)),L=s.getDerivedStateFromProps,(ue=typeof L=="function"||typeof S.getSnapshotBeforeUpdate=="function")||typeof S.UNSAFE_componentWillReceiveProps!="function"&&typeof S.componentWillReceiveProps!="function"||(A!==Oe||me!==Q)&&Fb(i,S,f,Q),No=!1,me=i.memoizedState,S.state=me,Rs(i,f,S,h),Os();var ge=i.memoizedState;A!==Oe||me!==ge||No||a!==null&&a.dependencies!==null&&hu(a.dependencies)?(typeof L=="function"&&(dv(i,s,L,f),ge=i.memoizedState),(Ce=No||Hb(i,s,Ce,f,me,ge,Q)||a!==null&&a.dependencies!==null&&hu(a.dependencies))?(ue||typeof S.UNSAFE_componentWillUpdate!="function"&&typeof S.componentWillUpdate!="function"||(typeof S.componentWillUpdate=="function"&&S.componentWillUpdate(f,ge,Q),typeof S.UNSAFE_componentWillUpdate=="function"&&S.UNSAFE_componentWillUpdate(f,ge,Q)),typeof S.componentDidUpdate=="function"&&(i.flags|=4),typeof S.getSnapshotBeforeUpdate=="function"&&(i.flags|=1024)):(typeof S.componentDidUpdate!="function"||A===a.memoizedProps&&me===a.memoizedState||(i.flags|=4),typeof S.getSnapshotBeforeUpdate!="function"||A===a.memoizedProps&&me===a.memoizedState||(i.flags|=1024),i.memoizedProps=f,i.memoizedState=ge),S.props=f,S.state=ge,S.context=Q,f=Ce):(typeof S.componentDidUpdate!="function"||A===a.memoizedProps&&me===a.memoizedState||(i.flags|=4),typeof S.getSnapshotBeforeUpdate!="function"||A===a.memoizedProps&&me===a.memoizedState||(i.flags|=1024),f=!1)}return S=f,Pu(a,i),f=(i.flags&128)!==0,S||f?(S=i.stateNode,s=f&&typeof s.getDerivedStateFromError!="function"?null:S.render(),i.flags|=1,a!==null&&f?(i.child=ml(i,a.child,null,h),i.child=ml(i,null,s,h)):ur(a,i,s,h),i.memoizedState=S.state,a=i.child):a=co(a,i,h),a}function nS(a,i,s,f){return ys(),i.flags|=256,ur(a,i,s,f),i.child}var hv={dehydrated:null,treeContext:null,retryLane:0,hydrationErrors:null};function pv(a){return{baseLanes:a,cachePool:Uy()}}function yv(a,i,s){return a=a!==null?a.childLanes&~s:0,i&&(a|=fa),a}function rS(a,i,s){var f=i.pendingProps,h=!1,S=(i.flags&128)!==0,A;if((A=S)||(A=a!==null&&a.memoizedState===null?!1:(Qn.current&2)!==0),A&&(h=!0,i.flags&=-129),A=(i.flags&32)!==0,i.flags&=-33,a===null){if(Zt){if(h?Po(i):zo(),Zt){var L=An,Q;if(Q=L){e:{for(Q=L,L=La;Q.nodeType!==8;){if(!L){L=null;break e}if(Q=wa(Q.nextSibling),Q===null){L=null;break e}}L=Q}L!==null?(i.memoizedState={dehydrated:L,treeContext:pi!==null?{id:ro,overflow:ao}:null,retryLane:536870912,hydrationErrors:null},Q=Hr(18,null,null,0),Q.stateNode=L,Q.return=i,i.child=Q,wr=i,An=null,Q=!0):Q=!1}Q||Si(i)}if(L=i.memoizedState,L!==null&&(L=L.dehydrated,L!==null))return tg(L)?i.lanes=32:i.lanes=536870912,null;so(i)}return L=f.children,f=f.fallback,h?(zo(),h=i.mode,L=zu({mode:"hidden",children:L},h),f=hi(f,h,s,null),L.return=i,f.return=i,L.sibling=f,i.child=L,h=i.child,h.memoizedState=pv(s),h.childLanes=yv(a,A,s),i.memoizedState=hv,f):(Po(i),bv(i,L))}if(Q=a.memoizedState,Q!==null&&(L=Q.dehydrated,L!==null)){if(S)i.flags&256?(Po(i),i.flags&=-257,i=Sv(a,i,s)):i.memoizedState!==null?(zo(),i.child=a.child,i.flags|=128,i=null):(zo(),h=f.fallback,L=i.mode,f=zu({mode:"visible",children:f.children},L),h=hi(h,L,s,null),h.flags|=2,f.return=i,h.return=i,f.sibling=h,i.child=f,ml(i,a.child,null,s),f=i.child,f.memoizedState=pv(s),f.childLanes=yv(a,A,s),i.memoizedState=hv,i=h);else if(Po(i),tg(L)){if(A=L.nextSibling&&L.nextSibling.dataset,A)var ue=A.dgst;A=ue,f=Error(r(419)),f.stack="",f.digest=A,bs({value:f,source:null,stack:null}),i=Sv(a,i,s)}else if(or||Ss(a,i,s,!1),A=(s&a.childLanes)!==0,or||A){if(A=Cn,A!==null&&(f=s&-s,f=(f&42)!==0?1:Et(f),f=(f&(A.suspendedLanes|s))!==0?0:f,f!==0&&f!==Q.retryLane))throw Q.retryLane=f,nl(a,f),Wr(A,a,f),Kb;L.data==="$?"||Bv(),i=Sv(a,i,s)}else L.data==="$?"?(i.flags|=192,i.child=a.child,i=null):(a=Q.treeContext,An=wa(L.nextSibling),wr=i,Zt=!0,bi=null,La=!1,a!==null&&(sa[ca++]=ro,sa[ca++]=ao,sa[ca++]=pi,ro=a.id,ao=a.overflow,pi=i),i=bv(i,f.children),i.flags|=4096);return i}return h?(zo(),h=f.fallback,L=i.mode,Q=a.child,ue=Q.sibling,f=no(Q,{mode:"hidden",children:f.children}),f.subtreeFlags=Q.subtreeFlags&65011712,ue!==null?h=no(ue,h):(h=hi(h,L,s,null),h.flags|=2),h.return=i,f.return=i,f.sibling=h,i.child=f,f=h,h=i.child,L=a.child.memoizedState,L===null?L=pv(s):(Q=L.cachePool,Q!==null?(ue=Yn._currentValue,Q=Q.parent!==ue?{parent:ue,pool:ue}:Q):Q=Uy(),L={baseLanes:L.baseLanes|s,cachePool:Q}),h.memoizedState=L,h.childLanes=yv(a,A,s),i.memoizedState=hv,f):(Po(i),s=a.child,a=s.sibling,s=no(s,{mode:"visible",children:f.children}),s.return=i,s.sibling=null,a!==null&&(A=i.deletions,A===null?(i.deletions=[a],i.flags|=16):A.push(a)),i.child=s,i.memoizedState=null,s)}function bv(a,i){return i=zu({mode:"visible",children:i},a.mode),i.return=a,a.child=i}function zu(a,i){return a=Hr(22,a,null,i),a.lanes=0,a.stateNode={_visibility:1,_pendingMarkers:null,_retryCache:null,_transitions:null},a}function Sv(a,i,s){return ml(i,a.child,null,s),a=bv(i,i.pendingProps.children),a.flags|=2,i.memoizedState=null,a}function aS(a,i,s){a.lanes|=i;var f=a.alternate;f!==null&&(f.lanes|=i),Bm(a.return,i,s)}function Cv(a,i,s,f,h){var S=a.memoizedState;S===null?a.memoizedState={isBackwards:i,rendering:null,renderingStartTime:0,last:f,tail:s,tailMode:h}:(S.isBackwards=i,S.rendering=null,S.renderingStartTime=0,S.last=f,S.tail=s,S.tailMode=h)}function oS(a,i,s){var f=i.pendingProps,h=f.revealOrder,S=f.tail;if(ur(a,i,f.children,s),f=Qn.current,(f&2)!==0)f=f&1|2,i.flags|=128;else{if(a!==null&&(a.flags&128)!==0)e:for(a=i.child;a!==null;){if(a.tag===13)a.memoizedState!==null&&aS(a,s,i);else if(a.tag===19)aS(a,s,i);else if(a.child!==null){a.child.return=a,a=a.child;continue}if(a===i)break e;for(;a.sibling===null;){if(a.return===null||a.return===i)break e;a=a.return}a.sibling.return=a.return,a=a.sibling}f&=1}switch(X(Qn,f),h){case"forwards":for(s=i.child,h=null;s!==null;)a=s.alternate,a!==null&&Au(a)===null&&(h=s),s=s.sibling;s=h,s===null?(h=i.child,i.child=null):(h=s.sibling,s.sibling=null),Cv(i,!1,h,s,S);break;case"backwards":for(s=null,h=i.child,i.child=null;h!==null;){if(a=h.alternate,a!==null&&Au(a)===null){i.child=h;break}a=h.sibling,h.sibling=s,s=h,h=a}Cv(i,!0,s,null,S);break;case"together":Cv(i,!1,null,null,void 0);break;default:i.memoizedState=null}return i.child}function co(a,i,s){if(a!==null&&(i.dependencies=a.dependencies),Fo|=i.lanes,(s&i.childLanes)===0)if(a!==null){if(Ss(a,i,s,!1),(s&i.childLanes)===0)return null}else return null;if(a!==null&&i.child!==a.child)throw Error(r(153));if(i.child!==null){for(a=i.child,s=no(a,a.pendingProps),i.child=s,s.return=i;a.sibling!==null;)a=a.sibling,s=s.sibling=no(a,a.pendingProps),s.return=i;s.sibling=null}return i.child}function xv(a,i){return(a.lanes&i)!==0?!0:(a=a.dependencies,!!(a!==null&&hu(a)))}function bM(a,i,s){switch(i.tag){case 3:ne(i,i.stateNode.containerInfo),_o(i,Yn,a.memoizedState.cache),ys();break;case 27:case 5:be(i);break;case 4:ne(i,i.stateNode.containerInfo);break;case 10:_o(i,i.type,i.memoizedProps.value);break;case 13:var f=i.memoizedState;if(f!==null)return f.dehydrated!==null?(Po(i),i.flags|=128,null):(s&i.child.childLanes)!==0?rS(a,i,s):(Po(i),a=co(a,i,s),a!==null?a.sibling:null);Po(i);break;case 19:var h=(a.flags&128)!==0;if(f=(s&i.childLanes)!==0,f||(Ss(a,i,s,!1),f=(s&i.childLanes)!==0),h){if(f)return oS(a,i,s);i.flags|=128}if(h=i.memoizedState,h!==null&&(h.rendering=null,h.tail=null,h.lastEffect=null),X(Qn,Qn.current),f)break;return null;case 22:case 23:return i.lanes=0,Zb(a,i,s);case 24:_o(i,Yn,a.memoizedState.cache)}return co(a,i,s)}function iS(a,i,s){if(a!==null)if(a.memoizedProps!==i.pendingProps)or=!0;else{if(!xv(a,s)&&(i.flags&128)===0)return or=!1,bM(a,i,s);or=(a.flags&131072)!==0}else or=!1,Zt&&(i.flags&1048576)!==0&&zy(i,gu,i.index);switch(i.lanes=0,i.tag){case 16:e:{a=i.pendingProps;var f=i.elementType,h=f._init;if(f=h(f._payload),i.type=f,typeof f=="function")Nm(f)?(a=$i(f,a),i.tag=1,i=tS(null,i,f,a,s)):(i.tag=0,i=gv(null,i,f,a,s));else{if(f!=null){if(h=f.$$typeof,h===T){i.tag=11,i=Xb(null,i,f,a,s);break e}else if(h===N){i.tag=14,i=Yb(null,i,f,a,s);break e}}throw i=q(f)||f,Error(r(306,i,""))}}return i;case 0:return gv(a,i,i.type,i.pendingProps,s);case 1:return f=i.type,h=$i(f,i.pendingProps),tS(a,i,f,h,s);case 3:e:{if(ne(i,i.stateNode.containerInfo),a===null)throw Error(r(387));f=i.pendingProps;var S=i.memoizedState;h=S.element,Wm(a,i),Rs(i,f,null,s);var A=i.memoizedState;if(f=A.cache,_o(i,Yn,f),f!==S.cache&&Lm(i,[Yn],s,!0),Os(),f=A.element,S.isDehydrated)if(S={element:f,isDehydrated:!1,cache:A.cache},i.updateQueue.baseState=S,i.memoizedState=S,i.flags&256){i=nS(a,i,f,s);break e}else if(f!==h){h=ia(Error(r(424)),i),bs(h),i=nS(a,i,f,s);break e}else{switch(a=i.stateNode.containerInfo,a.nodeType){case 9:a=a.body;break;default:a=a.nodeName==="HTML"?a.ownerDocument.body:a}for(An=wa(a.firstChild),wr=i,Zt=!0,bi=null,La=!0,s=Bb(i,null,f,s),i.child=s;s;)s.flags=s.flags&-3|4096,s=s.sibling}else{if(ys(),f===h){i=co(a,i,s);break e}ur(a,i,f,s)}i=i.child}return i;case 26:return Pu(a,i),a===null?(s=uC(i.type,null,i.pendingProps,null))?i.memoizedState=s:Zt||(s=i.type,a=i.pendingProps,f=Yu(te.current).createElement(s),f[St]=i,f[Be]=a,dr(f,s,a),Rt(f),i.stateNode=f):i.memoizedState=uC(i.type,a.memoizedProps,i.pendingProps,a.memoizedState),null;case 27:return be(i),a===null&&Zt&&(f=i.stateNode=lC(i.type,i.pendingProps,te.current),wr=i,La=!0,h=An,Wo(i.type)?(ng=h,An=wa(f.firstChild)):An=h),ur(a,i,i.pendingProps.children,s),Pu(a,i),a===null&&(i.flags|=4194304),i.child;case 5:return a===null&&Zt&&((h=f=An)&&(f=GM(f,i.type,i.pendingProps,La),f!==null?(i.stateNode=f,wr=i,An=wa(f.firstChild),La=!1,h=!0):h=!1),h||Si(i)),be(i),h=i.type,S=i.pendingProps,A=a!==null?a.memoizedProps:null,f=S.children,Zv(h,S)?f=null:A!==null&&Zv(h,A)&&(i.flags|=32),i.memoizedState!==null&&(h=Qm(a,i,fM,null,null,s),Xs._currentValue=h),Pu(a,i),ur(a,i,f,s),i.child;case 6:return a===null&&Zt&&((a=s=An)&&(s=KM(s,i.pendingProps,La),s!==null?(i.stateNode=s,wr=i,An=null,a=!0):a=!1),a||Si(i)),null;case 13:return rS(a,i,s);case 4:return ne(i,i.stateNode.containerInfo),f=i.pendingProps,a===null?i.child=ml(i,null,f,s):ur(a,i,f,s),i.child;case 11:return Xb(a,i,i.type,i.pendingProps,s);case 7:return ur(a,i,i.pendingProps,s),i.child;case 8:return ur(a,i,i.pendingProps.children,s),i.child;case 12:return ur(a,i,i.pendingProps.children,s),i.child;case 10:return f=i.pendingProps,_o(i,i.type,f.value),ur(a,i,f.children,s),i.child;case 9:return h=i.type._context,f=i.pendingProps.children,xi(i),h=pr(h),f=f(h),i.flags|=1,ur(a,i,f,s),i.child;case 14:return Yb(a,i,i.type,i.pendingProps,s);case 15:return Qb(a,i,i.type,i.pendingProps,s);case 19:return oS(a,i,s);case 31:return f=i.pendingProps,s=i.mode,f={mode:f.mode,children:f.children},a===null?(s=zu(f,s),s.ref=i.ref,i.child=s,s.return=i,i=s):(s=no(a.child,f),s.ref=i.ref,i.child=s,s.return=i,i=s),i;case 22:return Zb(a,i,s);case 24:return xi(i),f=pr(Yn),a===null?(h=Vm(),h===null&&(h=Cn,S=Hm(),h.pooledCache=S,S.refCount++,S!==null&&(h.pooledCacheLanes|=s),h=S),i.memoizedState={parent:f,cache:h},qm(i),_o(i,Yn,h)):((a.lanes&s)!==0&&(Wm(a,i),Rs(i,null,null,s),Os()),h=a.memoizedState,S=i.memoizedState,h.parent!==f?(h={parent:f,cache:f},i.memoizedState=h,i.lanes===0&&(i.memoizedState=i.updateQueue.baseState=h),_o(i,Yn,f)):(f=S.cache,_o(i,Yn,f),f!==h.cache&&Lm(i,[Yn],s,!0))),ur(a,i,i.pendingProps.children,s),i.child;case 29:throw i.pendingProps}throw Error(r(156,i.tag))}function uo(a){a.flags|=4}function lS(a,i){if(i.type!=="stylesheet"||(i.state.loading&4)!==0)a.flags&=-16777217;else if(a.flags|=16777216,!gC(i)){if(i=ua.current,i!==null&&((qt&4194048)===qt?Ha!==null:(qt&62914560)!==qt&&(qt&536870912)===0||i!==Ha))throw ws=Um,qy;a.flags|=8192}}function Du(a,i){i!==null&&(a.flags|=4),a.flags&16384&&(i=a.tag!==22?_n():536870912,a.lanes|=i,pl|=i)}function js(a,i){if(!Zt)switch(a.tailMode){case"hidden":i=a.tail;for(var s=null;i!==null;)i.alternate!==null&&(s=i),i=i.sibling;s===null?a.tail=null:s.sibling=null;break;case"collapsed":s=a.tail;for(var f=null;s!==null;)s.alternate!==null&&(f=s),s=s.sibling;f===null?i||a.tail===null?a.tail=null:a.tail.sibling=null:f.sibling=null}}function Mn(a){var i=a.alternate!==null&&a.alternate.child===a.child,s=0,f=0;if(i)for(var h=a.child;h!==null;)s|=h.lanes|h.childLanes,f|=h.subtreeFlags&65011712,f|=h.flags&65011712,h.return=a,h=h.sibling;else for(h=a.child;h!==null;)s|=h.lanes|h.childLanes,f|=h.subtreeFlags,f|=h.flags,h.return=a,h=h.sibling;return a.subtreeFlags|=f,a.childLanes=s,i}function SM(a,i,s){var f=i.pendingProps;switch(Pm(i),i.tag){case 31:case 16:case 15:case 0:case 11:case 7:case 8:case 12:case 9:case 14:return Mn(i),null;case 1:return Mn(i),null;case 3:return s=i.stateNode,f=null,a!==null&&(f=a.memoizedState.cache),i.memoizedState.cache!==f&&(i.flags|=2048),io(Yn),le(),s.pendingContext&&(s.context=s.pendingContext,s.pendingContext=null),(a===null||a.child===null)&&(ps(i)?uo(i):a===null||a.memoizedState.isDehydrated&&(i.flags&256)===0||(i.flags|=1024,Ly())),Mn(i),null;case 26:return s=i.memoizedState,a===null?(uo(i),s!==null?(Mn(i),lS(i,s)):(Mn(i),i.flags&=-16777217)):s?s!==a.memoizedState?(uo(i),Mn(i),lS(i,s)):(Mn(i),i.flags&=-16777217):(a.memoizedProps!==f&&uo(i),Mn(i),i.flags&=-16777217),null;case 27:fe(i),s=te.current;var h=i.type;if(a!==null&&i.stateNode!=null)a.memoizedProps!==f&&uo(i);else{if(!f){if(i.stateNode===null)throw Error(r(166));return Mn(i),null}a=U.current,ps(i)?Dy(i):(a=lC(h,f,s),i.stateNode=a,uo(i))}return Mn(i),null;case 5:if(fe(i),s=i.type,a!==null&&i.stateNode!=null)a.memoizedProps!==f&&uo(i);else{if(!f){if(i.stateNode===null)throw Error(r(166));return Mn(i),null}if(a=U.current,ps(i))Dy(i);else{switch(h=Yu(te.current),a){case 1:a=h.createElementNS("http://www.w3.org/2000/svg",s);break;case 2:a=h.createElementNS("http://www.w3.org/1998/Math/MathML",s);break;default:switch(s){case"svg":a=h.createElementNS("http://www.w3.org/2000/svg",s);break;case"math":a=h.createElementNS("http://www.w3.org/1998/Math/MathML",s);break;case"script":a=h.createElement("div"),a.innerHTML="<script><\/script>",a=a.removeChild(a.firstChild);break;case"select":a=typeof f.is=="string"?h.createElement("select",{is:f.is}):h.createElement("select"),f.multiple?a.multiple=!0:f.size&&(a.size=f.size);break;default:a=typeof f.is=="string"?h.createElement(s,{is:f.is}):h.createElement(s)}}a[St]=i,a[Be]=f;e:for(h=i.child;h!==null;){if(h.tag===5||h.tag===6)a.appendChild(h.stateNode);else if(h.tag!==4&&h.tag!==27&&h.child!==null){h.child.return=h,h=h.child;continue}if(h===i)break e;for(;h.sibling===null;){if(h.return===null||h.return===i)break e;h=h.return}h.sibling.return=h.return,h=h.sibling}i.stateNode=a;e:switch(dr(a,s,f),s){case"button":case"input":case"select":case"textarea":a=!!f.autoFocus;break e;case"img":a=!0;break e;default:a=!1}a&&uo(i)}}return Mn(i),i.flags&=-16777217,null;case 6:if(a&&i.stateNode!=null)a.memoizedProps!==f&&uo(i);else{if(typeof f!="string"&&i.stateNode===null)throw Error(r(166));if(a=te.current,ps(i)){if(a=i.stateNode,s=i.memoizedProps,f=null,h=wr,h!==null)switch(h.tag){case 27:case 5:f=h.memoizedProps}a[St]=i,a=!!(a.nodeValue===s||f!==null&&f.suppressHydrationWarning===!0||eC(a.nodeValue,s)),a||Si(i)}else a=Yu(a).createTextNode(f),a[St]=i,i.stateNode=a}return Mn(i),null;case 13:if(f=i.memoizedState,a===null||a.memoizedState!==null&&a.memoizedState.dehydrated!==null){if(h=ps(i),f!==null&&f.dehydrated!==null){if(a===null){if(!h)throw Error(r(318));if(h=i.memoizedState,h=h!==null?h.dehydrated:null,!h)throw Error(r(317));h[St]=i}else ys(),(i.flags&128)===0&&(i.memoizedState=null),i.flags|=4;Mn(i),h=!1}else h=Ly(),a!==null&&a.memoizedState!==null&&(a.memoizedState.hydrationErrors=h),h=!0;if(!h)return i.flags&256?(so(i),i):(so(i),null)}if(so(i),(i.flags&128)!==0)return i.lanes=s,i;if(s=f!==null,a=a!==null&&a.memoizedState!==null,s){f=i.child,h=null,f.alternate!==null&&f.alternate.memoizedState!==null&&f.alternate.memoizedState.cachePool!==null&&(h=f.alternate.memoizedState.cachePool.pool);var S=null;f.memoizedState!==null&&f.memoizedState.cachePool!==null&&(S=f.memoizedState.cachePool.pool),S!==h&&(f.flags|=2048)}return s!==a&&s&&(i.child.flags|=8192),Du(i,i.updateQueue),Mn(i),null;case 4:return le(),a===null&&Gv(i.stateNode.containerInfo),Mn(i),null;case 10:return io(i.type),Mn(i),null;case 19:if(G(Qn),h=i.memoizedState,h===null)return Mn(i),null;if(f=(i.flags&128)!==0,S=h.rendering,S===null)if(f)js(h,!1);else{if(In!==0||a!==null&&(a.flags&128)!==0)for(a=i.child;a!==null;){if(S=Au(a),S!==null){for(i.flags|=128,js(h,!1),a=S.updateQueue,i.updateQueue=a,Du(i,a),i.subtreeFlags=0,a=s,s=i.child;s!==null;)Py(s,a),s=s.sibling;return X(Qn,Qn.current&1|2),i.child}a=a.sibling}h.tail!==null&&$e()>Hu&&(i.flags|=128,f=!0,js(h,!1),i.lanes=4194304)}else{if(!f)if(a=Au(S),a!==null){if(i.flags|=128,f=!0,a=a.updateQueue,i.updateQueue=a,Du(i,a),js(h,!0),h.tail===null&&h.tailMode==="hidden"&&!S.alternate&&!Zt)return Mn(i),null}else 2*$e()-h.renderingStartTime>Hu&&s!==536870912&&(i.flags|=128,f=!0,js(h,!1),i.lanes=4194304);h.isBackwards?(S.sibling=i.child,i.child=S):(a=h.last,a!==null?a.sibling=S:i.child=S,h.last=S)}return h.tail!==null?(i=h.tail,h.rendering=i,h.tail=i.sibling,h.renderingStartTime=$e(),i.sibling=null,a=Qn.current,X(Qn,f?a&1|2:a&1),i):(Mn(i),null);case 22:case 23:return so(i),Xm(),f=i.memoizedState!==null,a!==null?a.memoizedState!==null!==f&&(i.flags|=8192):f&&(i.flags|=8192),f?(s&536870912)!==0&&(i.flags&128)===0&&(Mn(i),i.subtreeFlags&6&&(i.flags|=8192)):Mn(i),s=i.updateQueue,s!==null&&Du(i,s.retryQueue),s=null,a!==null&&a.memoizedState!==null&&a.memoizedState.cachePool!==null&&(s=a.memoizedState.cachePool.pool),f=null,i.memoizedState!==null&&i.memoizedState.cachePool!==null&&(f=i.memoizedState.cachePool.pool),f!==s&&(i.flags|=2048),a!==null&&G(Ei),null;case 24:return s=null,a!==null&&(s=a.memoizedState.cache),i.memoizedState.cache!==s&&(i.flags|=2048),io(Yn),Mn(i),null;case 25:return null;case 30:return null}throw Error(r(156,i.tag))}function CM(a,i){switch(Pm(i),i.tag){case 1:return a=i.flags,a&65536?(i.flags=a&-65537|128,i):null;case 3:return io(Yn),le(),a=i.flags,(a&65536)!==0&&(a&128)===0?(i.flags=a&-65537|128,i):null;case 26:case 27:case 5:return fe(i),null;case 13:if(so(i),a=i.memoizedState,a!==null&&a.dehydrated!==null){if(i.alternate===null)throw Error(r(340));ys()}return a=i.flags,a&65536?(i.flags=a&-65537|128,i):null;case 19:return G(Qn),null;case 4:return le(),null;case 10:return io(i.type),null;case 22:case 23:return so(i),Xm(),a!==null&&G(Ei),a=i.flags,a&65536?(i.flags=a&-65537|128,i):null;case 24:return io(Yn),null;case 25:return null;default:return null}}function sS(a,i){switch(Pm(i),i.tag){case 3:io(Yn),le();break;case 26:case 27:case 5:fe(i);break;case 4:le();break;case 13:so(i);break;case 19:G(Qn);break;case 10:io(i.type);break;case 22:case 23:so(i),Xm(),a!==null&&G(Ei);break;case 24:io(Yn)}}function Ps(a,i){try{var s=i.updateQueue,f=s!==null?s.lastEffect:null;if(f!==null){var h=f.next;s=h;do{if((s.tag&a)===a){f=void 0;var S=s.create,A=s.inst;f=S(),A.destroy=f}s=s.next}while(s!==h)}}catch(L){yn(i,i.return,L)}}function Do(a,i,s){try{var f=i.updateQueue,h=f!==null?f.lastEffect:null;if(h!==null){var S=h.next;f=S;do{if((f.tag&a)===a){var A=f.inst,L=A.destroy;if(L!==void 0){A.destroy=void 0,h=i;var Q=s,ue=L;try{ue()}catch(Ce){yn(h,Q,Ce)}}}f=f.next}while(f!==S)}}catch(Ce){yn(i,i.return,Ce)}}function cS(a){var i=a.updateQueue;if(i!==null){var s=a.stateNode;try{Yy(i,s)}catch(f){yn(a,a.return,f)}}}function uS(a,i,s){s.props=$i(a.type,a.memoizedProps),s.state=a.memoizedState;try{s.componentWillUnmount()}catch(f){yn(a,i,f)}}function zs(a,i){try{var s=a.ref;if(s!==null){switch(a.tag){case 26:case 27:case 5:var f=a.stateNode;break;case 30:f=a.stateNode;break;default:f=a.stateNode}typeof s=="function"?a.refCleanup=s(f):s.current=f}}catch(h){yn(a,i,h)}}function Fa(a,i){var s=a.ref,f=a.refCleanup;if(s!==null)if(typeof f=="function")try{f()}catch(h){yn(a,i,h)}finally{a.refCleanup=null,a=a.alternate,a!=null&&(a.refCleanup=null)}else if(typeof s=="function")try{s(null)}catch(h){yn(a,i,h)}else s.current=null}function fS(a){var i=a.type,s=a.memoizedProps,f=a.stateNode;try{e:switch(i){case"button":case"input":case"select":case"textarea":s.autoFocus&&f.focus();break e;case"img":s.src?f.src=s.src:s.srcSet&&(f.srcset=s.srcSet)}}catch(h){yn(a,a.return,h)}}function Ev(a,i,s){try{var f=a.stateNode;VM(f,a.type,s,i),f[Be]=i}catch(h){yn(a,a.return,h)}}function dS(a){return a.tag===5||a.tag===3||a.tag===26||a.tag===27&&Wo(a.type)||a.tag===4}function wv(a){e:for(;;){for(;a.sibling===null;){if(a.return===null||dS(a.return))return null;a=a.return}for(a.sibling.return=a.return,a=a.sibling;a.tag!==5&&a.tag!==6&&a.tag!==18;){if(a.tag===27&&Wo(a.type)||a.flags&2||a.child===null||a.tag===4)continue e;a.child.return=a,a=a.child}if(!(a.flags&2))return a.stateNode}}function $v(a,i,s){var f=a.tag;if(f===5||f===6)a=a.stateNode,i?(s.nodeType===9?s.body:s.nodeName==="HTML"?s.ownerDocument.body:s).insertBefore(a,i):(i=s.nodeType===9?s.body:s.nodeName==="HTML"?s.ownerDocument.body:s,i.appendChild(a),s=s._reactRootContainer,s!=null||i.onclick!==null||(i.onclick=Xu));else if(f!==4&&(f===27&&Wo(a.type)&&(s=a.stateNode,i=null),a=a.child,a!==null))for($v(a,i,s),a=a.sibling;a!==null;)$v(a,i,s),a=a.sibling}function Bu(a,i,s){var f=a.tag;if(f===5||f===6)a=a.stateNode,i?s.insertBefore(a,i):s.appendChild(a);else if(f!==4&&(f===27&&Wo(a.type)&&(s=a.stateNode),a=a.child,a!==null))for(Bu(a,i,s),a=a.sibling;a!==null;)Bu(a,i,s),a=a.sibling}function mS(a){var i=a.stateNode,s=a.memoizedProps;try{for(var f=a.type,h=i.attributes;h.length;)i.removeAttributeNode(h[0]);dr(i,f,s),i[St]=a,i[Be]=s}catch(S){yn(a,a.return,S)}}var fo=!1,Dn=!1,Ov=!1,vS=typeof WeakSet=="function"?WeakSet:Set,ir=null;function xM(a,i){if(a=a.containerInfo,Yv=nf,a=$y(a),wm(a)){if("selectionStart"in a)var s={start:a.selectionStart,end:a.selectionEnd};else e:{s=(s=a.ownerDocument)&&s.defaultView||window;var f=s.getSelection&&s.getSelection();if(f&&f.rangeCount!==0){s=f.anchorNode;var h=f.anchorOffset,S=f.focusNode;f=f.focusOffset;try{s.nodeType,S.nodeType}catch{s=null;break e}var A=0,L=-1,Q=-1,ue=0,Ce=0,Oe=a,me=null;t:for(;;){for(var ge;Oe!==s||h!==0&&Oe.nodeType!==3||(L=A+h),Oe!==S||f!==0&&Oe.nodeType!==3||(Q=A+f),Oe.nodeType===3&&(A+=Oe.nodeValue.length),(ge=Oe.firstChild)!==null;)me=Oe,Oe=ge;for(;;){if(Oe===a)break t;if(me===s&&++ue===h&&(L=A),me===S&&++Ce===f&&(Q=A),(ge=Oe.nextSibling)!==null)break;Oe=me,me=Oe.parentNode}Oe=ge}s=L===-1||Q===-1?null:{start:L,end:Q}}else s=null}s=s||{start:0,end:0}}else s=null;for(Qv={focusedElem:a,selectionRange:s},nf=!1,ir=i;ir!==null;)if(i=ir,a=i.child,(i.subtreeFlags&1024)!==0&&a!==null)a.return=i,ir=a;else for(;ir!==null;){switch(i=ir,S=i.alternate,a=i.flags,i.tag){case 0:break;case 11:case 15:break;case 1:if((a&1024)!==0&&S!==null){a=void 0,s=i,h=S.memoizedProps,S=S.memoizedState,f=s.stateNode;try{var bt=$i(s.type,h,s.elementType===s.type);a=f.getSnapshotBeforeUpdate(bt,S),f.__reactInternalSnapshotBeforeUpdate=a}catch(mt){yn(s,s.return,mt)}}break;case 3:if((a&1024)!==0){if(a=i.stateNode.containerInfo,s=a.nodeType,s===9)eg(a);else if(s===1)switch(a.nodeName){case"HEAD":case"HTML":case"BODY":eg(a);break;default:a.textContent=""}}break;case 5:case 26:case 27:case 6:case 4:case 17:break;default:if((a&1024)!==0)throw Error(r(163))}if(a=i.sibling,a!==null){a.return=i.return,ir=a;break}ir=i.return}}function gS(a,i,s){var f=s.flags;switch(s.tag){case 0:case 11:case 15:Bo(a,s),f&4&&Ps(5,s);break;case 1:if(Bo(a,s),f&4)if(a=s.stateNode,i===null)try{a.componentDidMount()}catch(A){yn(s,s.return,A)}else{var h=$i(s.type,i.memoizedProps);i=i.memoizedState;try{a.componentDidUpdate(h,i,a.__reactInternalSnapshotBeforeUpdate)}catch(A){yn(s,s.return,A)}}f&64&&cS(s),f&512&&zs(s,s.return);break;case 3:if(Bo(a,s),f&64&&(a=s.updateQueue,a!==null)){if(i=null,s.child!==null)switch(s.child.tag){case 27:case 5:i=s.child.stateNode;break;case 1:i=s.child.stateNode}try{Yy(a,i)}catch(A){yn(s,s.return,A)}}break;case 27:i===null&&f&4&&mS(s);case 26:case 5:Bo(a,s),i===null&&f&4&&fS(s),f&512&&zs(s,s.return);break;case 12:Bo(a,s);break;case 13:Bo(a,s),f&4&&yS(a,s),f&64&&(a=s.memoizedState,a!==null&&(a=a.dehydrated,a!==null&&(s=NM.bind(null,s),XM(a,s))));break;case 22:if(f=s.memoizedState!==null||fo,!f){i=i!==null&&i.memoizedState!==null||Dn,h=fo;var S=Dn;fo=f,(Dn=i)&&!S?Lo(a,s,(s.subtreeFlags&8772)!==0):Bo(a,s),fo=h,Dn=S}break;case 30:break;default:Bo(a,s)}}function hS(a){var i=a.alternate;i!==null&&(a.alternate=null,hS(i)),a.child=null,a.deletions=null,a.sibling=null,a.tag===5&&(i=a.stateNode,i!==null&&Tt(i)),a.stateNode=null,a.return=null,a.dependencies=null,a.memoizedProps=null,a.memoizedState=null,a.pendingProps=null,a.stateNode=null,a.updateQueue=null}var Rn=null,Ar=!1;function mo(a,i,s){for(s=s.child;s!==null;)pS(a,i,s),s=s.sibling}function pS(a,i,s){if(We&&typeof We.onCommitFiberUnmount=="function")try{We.onCommitFiberUnmount(Ke,s)}catch{}switch(s.tag){case 26:Dn||Fa(s,i),mo(a,i,s),s.memoizedState?s.memoizedState.count--:s.stateNode&&(s=s.stateNode,s.parentNode.removeChild(s));break;case 27:Dn||Fa(s,i);var f=Rn,h=Ar;Wo(s.type)&&(Rn=s.stateNode,Ar=!1),mo(a,i,s),Ws(s.stateNode),Rn=f,Ar=h;break;case 5:Dn||Fa(s,i);case 6:if(f=Rn,h=Ar,Rn=null,mo(a,i,s),Rn=f,Ar=h,Rn!==null)if(Ar)try{(Rn.nodeType===9?Rn.body:Rn.nodeName==="HTML"?Rn.ownerDocument.body:Rn).removeChild(s.stateNode)}catch(S){yn(s,i,S)}else try{Rn.removeChild(s.stateNode)}catch(S){yn(s,i,S)}break;case 18:Rn!==null&&(Ar?(a=Rn,oC(a.nodeType===9?a.body:a.nodeName==="HTML"?a.ownerDocument.body:a,s.stateNode),Js(a)):oC(Rn,s.stateNode));break;case 4:f=Rn,h=Ar,Rn=s.stateNode.containerInfo,Ar=!0,mo(a,i,s),Rn=f,Ar=h;break;case 0:case 11:case 14:case 15:Dn||Do(2,s,i),Dn||Do(4,s,i),mo(a,i,s);break;case 1:Dn||(Fa(s,i),f=s.stateNode,typeof f.componentWillUnmount=="function"&&uS(s,i,f)),mo(a,i,s);break;case 21:mo(a,i,s);break;case 22:Dn=(f=Dn)||s.memoizedState!==null,mo(a,i,s),Dn=f;break;default:mo(a,i,s)}}function yS(a,i){if(i.memoizedState===null&&(a=i.alternate,a!==null&&(a=a.memoizedState,a!==null&&(a=a.dehydrated,a!==null))))try{Js(a)}catch(s){yn(i,i.return,s)}}function EM(a){switch(a.tag){case 13:case 19:var i=a.stateNode;return i===null&&(i=a.stateNode=new vS),i;case 22:return a=a.stateNode,i=a._retryCache,i===null&&(i=a._retryCache=new vS),i;default:throw Error(r(435,a.tag))}}function Rv(a,i){var s=EM(a);i.forEach(function(f){var h=AM.bind(null,a,f);s.has(f)||(s.add(f),f.then(h,h))})}function Fr(a,i){var s=i.deletions;if(s!==null)for(var f=0;f<s.length;f++){var h=s[f],S=a,A=i,L=A;e:for(;L!==null;){switch(L.tag){case 27:if(Wo(L.type)){Rn=L.stateNode,Ar=!1;break e}break;case 5:Rn=L.stateNode,Ar=!1;break e;case 3:case 4:Rn=L.stateNode.containerInfo,Ar=!0;break e}L=L.return}if(Rn===null)throw Error(r(160));pS(S,A,h),Rn=null,Ar=!1,S=h.alternate,S!==null&&(S.return=null),h.return=null}if(i.subtreeFlags&13878)for(i=i.child;i!==null;)bS(i,a),i=i.sibling}var Ea=null;function bS(a,i){var s=a.alternate,f=a.flags;switch(a.tag){case 0:case 11:case 14:case 15:Fr(i,a),Vr(a),f&4&&(Do(3,a,a.return),Ps(3,a),Do(5,a,a.return));break;case 1:Fr(i,a),Vr(a),f&512&&(Dn||s===null||Fa(s,s.return)),f&64&&fo&&(a=a.updateQueue,a!==null&&(f=a.callbacks,f!==null&&(s=a.shared.hiddenCallbacks,a.shared.hiddenCallbacks=s===null?f:s.concat(f))));break;case 26:var h=Ea;if(Fr(i,a),Vr(a),f&512&&(Dn||s===null||Fa(s,s.return)),f&4){var S=s!==null?s.memoizedState:null;if(f=a.memoizedState,s===null)if(f===null)if(a.stateNode===null){e:{f=a.type,s=a.memoizedProps,h=h.ownerDocument||h;t:switch(f){case"title":S=h.getElementsByTagName("title")[0],(!S||S[Vt]||S[St]||S.namespaceURI==="http://www.w3.org/2000/svg"||S.hasAttribute("itemprop"))&&(S=h.createElement(f),h.head.insertBefore(S,h.querySelector("head > title"))),dr(S,f,s),S[St]=a,Rt(S),f=S;break e;case"link":var A=mC("link","href",h).get(f+(s.href||""));if(A){for(var L=0;L<A.length;L++)if(S=A[L],S.getAttribute("href")===(s.href==null||s.href===""?null:s.href)&&S.getAttribute("rel")===(s.rel==null?null:s.rel)&&S.getAttribute("title")===(s.title==null?null:s.title)&&S.getAttribute("crossorigin")===(s.crossOrigin==null?null:s.crossOrigin)){A.splice(L,1);break t}}S=h.createElement(f),dr(S,f,s),h.head.appendChild(S);break;case"meta":if(A=mC("meta","content",h).get(f+(s.content||""))){for(L=0;L<A.length;L++)if(S=A[L],S.getAttribute("content")===(s.content==null?null:""+s.content)&&S.getAttribute("name")===(s.name==null?null:s.name)&&S.getAttribute("property")===(s.property==null?null:s.property)&&S.getAttribute("http-equiv")===(s.httpEquiv==null?null:s.httpEquiv)&&S.getAttribute("charset")===(s.charSet==null?null:s.charSet)){A.splice(L,1);break t}}S=h.createElement(f),dr(S,f,s),h.head.appendChild(S);break;default:throw Error(r(468,f))}S[St]=a,Rt(S),f=S}a.stateNode=f}else vC(h,a.type,a.stateNode);else a.stateNode=dC(h,f,a.memoizedProps);else S!==f?(S===null?s.stateNode!==null&&(s=s.stateNode,s.parentNode.removeChild(s)):S.count--,f===null?vC(h,a.type,a.stateNode):dC(h,f,a.memoizedProps)):f===null&&a.stateNode!==null&&Ev(a,a.memoizedProps,s.memoizedProps)}break;case 27:Fr(i,a),Vr(a),f&512&&(Dn||s===null||Fa(s,s.return)),s!==null&&f&4&&Ev(a,a.memoizedProps,s.memoizedProps);break;case 5:if(Fr(i,a),Vr(a),f&512&&(Dn||s===null||Fa(s,s.return)),a.flags&32){h=a.stateNode;try{Nn(h,"")}catch(ge){yn(a,a.return,ge)}}f&4&&a.stateNode!=null&&(h=a.memoizedProps,Ev(a,h,s!==null?s.memoizedProps:h)),f&1024&&(Ov=!0);break;case 6:if(Fr(i,a),Vr(a),f&4){if(a.stateNode===null)throw Error(r(162));f=a.memoizedProps,s=a.stateNode;try{s.nodeValue=f}catch(ge){yn(a,a.return,ge)}}break;case 3:if(Ju=null,h=Ea,Ea=Qu(i.containerInfo),Fr(i,a),Ea=h,Vr(a),f&4&&s!==null&&s.memoizedState.isDehydrated)try{Js(i.containerInfo)}catch(ge){yn(a,a.return,ge)}Ov&&(Ov=!1,SS(a));break;case 4:f=Ea,Ea=Qu(a.stateNode.containerInfo),Fr(i,a),Vr(a),Ea=f;break;case 12:Fr(i,a),Vr(a);break;case 13:Fr(i,a),Vr(a),a.child.flags&8192&&a.memoizedState!==null!=(s!==null&&s.memoizedState!==null)&&(Iv=$e()),f&4&&(f=a.updateQueue,f!==null&&(a.updateQueue=null,Rv(a,f)));break;case 22:h=a.memoizedState!==null;var Q=s!==null&&s.memoizedState!==null,ue=fo,Ce=Dn;if(fo=ue||h,Dn=Ce||Q,Fr(i,a),Dn=Ce,fo=ue,Vr(a),f&8192)e:for(i=a.stateNode,i._visibility=h?i._visibility&-2:i._visibility|1,h&&(s===null||Q||fo||Dn||Oi(a)),s=null,i=a;;){if(i.tag===5||i.tag===26){if(s===null){Q=s=i;try{if(S=Q.stateNode,h)A=S.style,typeof A.setProperty=="function"?A.setProperty("display","none","important"):A.display="none";else{L=Q.stateNode;var Oe=Q.memoizedProps.style,me=Oe!=null&&Oe.hasOwnProperty("display")?Oe.display:null;L.style.display=me==null||typeof me=="boolean"?"":(""+me).trim()}}catch(ge){yn(Q,Q.return,ge)}}}else if(i.tag===6){if(s===null){Q=i;try{Q.stateNode.nodeValue=h?"":Q.memoizedProps}catch(ge){yn(Q,Q.return,ge)}}}else if((i.tag!==22&&i.tag!==23||i.memoizedState===null||i===a)&&i.child!==null){i.child.return=i,i=i.child;continue}if(i===a)break e;for(;i.sibling===null;){if(i.return===null||i.return===a)break e;s===i&&(s=null),i=i.return}s===i&&(s=null),i.sibling.return=i.return,i=i.sibling}f&4&&(f=a.updateQueue,f!==null&&(s=f.retryQueue,s!==null&&(f.retryQueue=null,Rv(a,s))));break;case 19:Fr(i,a),Vr(a),f&4&&(f=a.updateQueue,f!==null&&(a.updateQueue=null,Rv(a,f)));break;case 30:break;case 21:break;default:Fr(i,a),Vr(a)}}function Vr(a){var i=a.flags;if(i&2){try{for(var s,f=a.return;f!==null;){if(dS(f)){s=f;break}f=f.return}if(s==null)throw Error(r(160));switch(s.tag){case 27:var h=s.stateNode,S=wv(a);Bu(a,S,h);break;case 5:var A=s.stateNode;s.flags&32&&(Nn(A,""),s.flags&=-33);var L=wv(a);Bu(a,L,A);break;case 3:case 4:var Q=s.stateNode.containerInfo,ue=wv(a);$v(a,ue,Q);break;default:throw Error(r(161))}}catch(Ce){yn(a,a.return,Ce)}a.flags&=-3}i&4096&&(a.flags&=-4097)}function SS(a){if(a.subtreeFlags&1024)for(a=a.child;a!==null;){var i=a;SS(i),i.tag===5&&i.flags&1024&&i.stateNode.reset(),a=a.sibling}}function Bo(a,i){if(i.subtreeFlags&8772)for(i=i.child;i!==null;)gS(a,i.alternate,i),i=i.sibling}function Oi(a){for(a=a.child;a!==null;){var i=a;switch(i.tag){case 0:case 11:case 14:case 15:Do(4,i,i.return),Oi(i);break;case 1:Fa(i,i.return);var s=i.stateNode;typeof s.componentWillUnmount=="function"&&uS(i,i.return,s),Oi(i);break;case 27:Ws(i.stateNode);case 26:case 5:Fa(i,i.return),Oi(i);break;case 22:i.memoizedState===null&&Oi(i);break;case 30:Oi(i);break;default:Oi(i)}a=a.sibling}}function Lo(a,i,s){for(s=s&&(i.subtreeFlags&8772)!==0,i=i.child;i!==null;){var f=i.alternate,h=a,S=i,A=S.flags;switch(S.tag){case 0:case 11:case 15:Lo(h,S,s),Ps(4,S);break;case 1:if(Lo(h,S,s),f=S,h=f.stateNode,typeof h.componentDidMount=="function")try{h.componentDidMount()}catch(ue){yn(f,f.return,ue)}if(f=S,h=f.updateQueue,h!==null){var L=f.stateNode;try{var Q=h.shared.hiddenCallbacks;if(Q!==null)for(h.shared.hiddenCallbacks=null,h=0;h<Q.length;h++)Xy(Q[h],L)}catch(ue){yn(f,f.return,ue)}}s&&A&64&&cS(S),zs(S,S.return);break;case 27:mS(S);case 26:case 5:Lo(h,S,s),s&&f===null&&A&4&&fS(S),zs(S,S.return);break;case 12:Lo(h,S,s);break;case 13:Lo(h,S,s),s&&A&4&&yS(h,S);break;case 22:S.memoizedState===null&&Lo(h,S,s),zs(S,S.return);break;case 30:break;default:Lo(h,S,s)}i=i.sibling}}function Tv(a,i){var s=null;a!==null&&a.memoizedState!==null&&a.memoizedState.cachePool!==null&&(s=a.memoizedState.cachePool.pool),a=null,i.memoizedState!==null&&i.memoizedState.cachePool!==null&&(a=i.memoizedState.cachePool.pool),a!==s&&(a!=null&&a.refCount++,s!=null&&Cs(s))}function Mv(a,i){a=null,i.alternate!==null&&(a=i.alternate.memoizedState.cache),i=i.memoizedState.cache,i!==a&&(i.refCount++,a!=null&&Cs(a))}function Va(a,i,s,f){if(i.subtreeFlags&10256)for(i=i.child;i!==null;)CS(a,i,s,f),i=i.sibling}function CS(a,i,s,f){var h=i.flags;switch(i.tag){case 0:case 11:case 15:Va(a,i,s,f),h&2048&&Ps(9,i);break;case 1:Va(a,i,s,f);break;case 3:Va(a,i,s,f),h&2048&&(a=null,i.alternate!==null&&(a=i.alternate.memoizedState.cache),i=i.memoizedState.cache,i!==a&&(i.refCount++,a!=null&&Cs(a)));break;case 12:if(h&2048){Va(a,i,s,f),a=i.stateNode;try{var S=i.memoizedProps,A=S.id,L=S.onPostCommit;typeof L=="function"&&L(A,i.alternate===null?"mount":"update",a.passiveEffectDuration,-0)}catch(Q){yn(i,i.return,Q)}}else Va(a,i,s,f);break;case 13:Va(a,i,s,f);break;case 23:break;case 22:S=i.stateNode,A=i.alternate,i.memoizedState!==null?S._visibility&2?Va(a,i,s,f):Ds(a,i):S._visibility&2?Va(a,i,s,f):(S._visibility|=2,vl(a,i,s,f,(i.subtreeFlags&10256)!==0)),h&2048&&Tv(A,i);break;case 24:Va(a,i,s,f),h&2048&&Mv(i.alternate,i);break;default:Va(a,i,s,f)}}function vl(a,i,s,f,h){for(h=h&&(i.subtreeFlags&10256)!==0,i=i.child;i!==null;){var S=a,A=i,L=s,Q=f,ue=A.flags;switch(A.tag){case 0:case 11:case 15:vl(S,A,L,Q,h),Ps(8,A);break;case 23:break;case 22:var Ce=A.stateNode;A.memoizedState!==null?Ce._visibility&2?vl(S,A,L,Q,h):Ds(S,A):(Ce._visibility|=2,vl(S,A,L,Q,h)),h&&ue&2048&&Tv(A.alternate,A);break;case 24:vl(S,A,L,Q,h),h&&ue&2048&&Mv(A.alternate,A);break;default:vl(S,A,L,Q,h)}i=i.sibling}}function Ds(a,i){if(i.subtreeFlags&10256)for(i=i.child;i!==null;){var s=a,f=i,h=f.flags;switch(f.tag){case 22:Ds(s,f),h&2048&&Tv(f.alternate,f);break;case 24:Ds(s,f),h&2048&&Mv(f.alternate,f);break;default:Ds(s,f)}i=i.sibling}}var Bs=8192;function gl(a){if(a.subtreeFlags&Bs)for(a=a.child;a!==null;)xS(a),a=a.sibling}function xS(a){switch(a.tag){case 26:gl(a),a.flags&Bs&&a.memoizedState!==null&&s_(Ea,a.memoizedState,a.memoizedProps);break;case 5:gl(a);break;case 3:case 4:var i=Ea;Ea=Qu(a.stateNode.containerInfo),gl(a),Ea=i;break;case 22:a.memoizedState===null&&(i=a.alternate,i!==null&&i.memoizedState!==null?(i=Bs,Bs=16777216,gl(a),Bs=i):gl(a));break;default:gl(a)}}function ES(a){var i=a.alternate;if(i!==null&&(a=i.child,a!==null)){i.child=null;do i=a.sibling,a.sibling=null,a=i;while(a!==null)}}function Ls(a){var i=a.deletions;if((a.flags&16)!==0){if(i!==null)for(var s=0;s<i.length;s++){var f=i[s];ir=f,$S(f,a)}ES(a)}if(a.subtreeFlags&10256)for(a=a.child;a!==null;)wS(a),a=a.sibling}function wS(a){switch(a.tag){case 0:case 11:case 15:Ls(a),a.flags&2048&&Do(9,a,a.return);break;case 3:Ls(a);break;case 12:Ls(a);break;case 22:var i=a.stateNode;a.memoizedState!==null&&i._visibility&2&&(a.return===null||a.return.tag!==13)?(i._visibility&=-3,Lu(a)):Ls(a);break;default:Ls(a)}}function Lu(a){var i=a.deletions;if((a.flags&16)!==0){if(i!==null)for(var s=0;s<i.length;s++){var f=i[s];ir=f,$S(f,a)}ES(a)}for(a=a.child;a!==null;){switch(i=a,i.tag){case 0:case 11:case 15:Do(8,i,i.return),Lu(i);break;case 22:s=i.stateNode,s._visibility&2&&(s._visibility&=-3,Lu(i));break;default:Lu(i)}a=a.sibling}}function $S(a,i){for(;ir!==null;){var s=ir;switch(s.tag){case 0:case 11:case 15:Do(8,s,i);break;case 23:case 22:if(s.memoizedState!==null&&s.memoizedState.cachePool!==null){var f=s.memoizedState.cachePool.pool;f!=null&&f.refCount++}break;case 24:Cs(s.memoizedState.cache)}if(f=s.child,f!==null)f.return=s,ir=f;else e:for(s=a;ir!==null;){f=ir;var h=f.sibling,S=f.return;if(hS(f),f===s){ir=null;break e}if(h!==null){h.return=S,ir=h;break e}ir=S}}}var wM={getCacheForType:function(a){var i=pr(Yn),s=i.data.get(a);return s===void 0&&(s=a(),i.data.set(a,s)),s}},$M=typeof WeakMap=="function"?WeakMap:Map,on=0,Cn=null,zt=null,qt=0,ln=0,Ur=null,Ho=!1,hl=!1,_v=!1,vo=0,In=0,Fo=0,Ri=0,Nv=0,fa=0,pl=0,Hs=null,Ir=null,Av=!1,Iv=0,Hu=1/0,Fu=null,Vo=null,fr=0,Uo=null,yl=null,bl=0,jv=0,Pv=null,OS=null,Fs=0,zv=null;function qr(){if((on&2)!==0&&qt!==0)return qt&-qt;if(_.T!==null){var a=il;return a!==0?a:Uv()}return ke()}function RS(){fa===0&&(fa=(qt&536870912)===0||Zt?Nt():536870912);var a=ua.current;return a!==null&&(a.flags|=32),fa}function Wr(a,i,s){(a===Cn&&(ln===2||ln===9)||a.cancelPendingCommit!==null)&&(Sl(a,0),qo(a,qt,fa,!1)),$n(a,s),((on&2)===0||a!==Cn)&&(a===Cn&&((on&2)===0&&(Ri|=s),In===4&&qo(a,qt,fa,!1)),Ua(a))}function TS(a,i,s){if((on&6)!==0)throw Error(r(327));var f=!s&&(i&124)===0&&(i&a.expiredLanes)===0||qe(a,i),h=f?TM(a,i):Lv(a,i,!0),S=f;do{if(h===0){hl&&!f&&qo(a,i,0,!1);break}else{if(s=a.current.alternate,S&&!OM(s)){h=Lv(a,i,!1),S=!1;continue}if(h===2){if(S=i,a.errorRecoveryDisabledLanes&S)var A=0;else A=a.pendingLanes&-536870913,A=A!==0?A:A&536870912?536870912:0;if(A!==0){i=A;e:{var L=a;h=Hs;var Q=L.current.memoizedState.isDehydrated;if(Q&&(Sl(L,A).flags|=256),A=Lv(L,A,!1),A!==2){if(_v&&!Q){L.errorRecoveryDisabledLanes|=S,Ri|=S,h=4;break e}S=Ir,Ir=h,S!==null&&(Ir===null?Ir=S:Ir.push.apply(Ir,S))}h=A}if(S=!1,h!==2)continue}}if(h===1){Sl(a,0),qo(a,i,0,!0);break}e:{switch(f=a,S=h,S){case 0:case 1:throw Error(r(345));case 4:if((i&4194048)!==i)break;case 6:qo(f,i,fa,!Ho);break e;case 2:Ir=null;break;case 3:case 5:break;default:throw Error(r(329))}if((i&62914560)===i&&(h=Iv+300-$e(),10<h)){if(qo(f,i,fa,!Ho),_e(f,0,!0)!==0)break e;f.timeoutHandle=rC(MS.bind(null,f,s,Ir,Fu,Av,i,fa,Ri,pl,Ho,S,2,-0,0),h);break e}MS(f,s,Ir,Fu,Av,i,fa,Ri,pl,Ho,S,0,-0,0)}}break}while(!0);Ua(a)}function MS(a,i,s,f,h,S,A,L,Q,ue,Ce,Oe,me,ge){if(a.timeoutHandle=-1,Oe=i.subtreeFlags,(Oe&8192||(Oe&16785408)===16785408)&&(Ks={stylesheets:null,count:0,unsuspend:l_},xS(i),Oe=c_(),Oe!==null)){a.cancelPendingCommit=Oe(zS.bind(null,a,i,S,s,f,h,A,L,Q,Ce,1,me,ge)),qo(a,S,A,!ue);return}zS(a,i,S,s,f,h,A,L,Q)}function OM(a){for(var i=a;;){var s=i.tag;if((s===0||s===11||s===15)&&i.flags&16384&&(s=i.updateQueue,s!==null&&(s=s.stores,s!==null)))for(var f=0;f<s.length;f++){var h=s[f],S=h.getSnapshot;h=h.value;try{if(!Lr(S(),h))return!1}catch{return!1}}if(s=i.child,i.subtreeFlags&16384&&s!==null)s.return=i,i=s;else{if(i===a)break;for(;i.sibling===null;){if(i.return===null||i.return===a)return!0;i=i.return}i.sibling.return=i.return,i=i.sibling}}return!0}function qo(a,i,s,f){i&=~Nv,i&=~Ri,a.suspendedLanes|=i,a.pingedLanes&=~i,f&&(a.warmLanes|=i),f=a.expirationTimes;for(var h=i;0<h;){var S=31-Ue(h),A=1<<S;f[S]=-1,h&=~A}s!==0&&Ft(a,s,i)}function Vu(){return(on&6)===0?(Vs(0),!1):!0}function Dv(){if(zt!==null){if(ln===0)var a=zt.return;else a=zt,oo=Ci=null,ev(a),dl=null,As=0,a=zt;for(;a!==null;)sS(a.alternate,a),a=a.return;zt=null}}function Sl(a,i){var s=a.timeoutHandle;s!==-1&&(a.timeoutHandle=-1,qM(s)),s=a.cancelPendingCommit,s!==null&&(a.cancelPendingCommit=null,s()),Dv(),Cn=a,zt=s=no(a.current,null),qt=i,ln=0,Ur=null,Ho=!1,hl=qe(a,i),_v=!1,pl=fa=Nv=Ri=Fo=In=0,Ir=Hs=null,Av=!1,(i&8)!==0&&(i|=i&32);var f=a.entangledLanes;if(f!==0)for(a=a.entanglements,f&=i;0<f;){var h=31-Ue(f),S=1<<h;i|=a[h],f&=~S}return vo=i,uu(),s}function _S(a,i){Mt=null,_.H=Mu,i===Es||i===bu?(i=Gy(),ln=3):i===qy?(i=Gy(),ln=4):ln=i===Kb?8:i!==null&&typeof i=="object"&&typeof i.then=="function"?6:1,Ur=i,zt===null&&(In=1,ju(a,ia(i,a.current)))}function NS(){var a=_.H;return _.H=Mu,a===null?Mu:a}function AS(){var a=_.A;return _.A=wM,a}function Bv(){In=4,Ho||(qt&4194048)!==qt&&ua.current!==null||(hl=!0),(Fo&134217727)===0&&(Ri&134217727)===0||Cn===null||qo(Cn,qt,fa,!1)}function Lv(a,i,s){var f=on;on|=2;var h=NS(),S=AS();(Cn!==a||qt!==i)&&(Fu=null,Sl(a,i)),i=!1;var A=In;e:do try{if(ln!==0&&zt!==null){var L=zt,Q=Ur;switch(ln){case 8:Dv(),A=6;break e;case 3:case 2:case 9:case 6:ua.current===null&&(i=!0);var ue=ln;if(ln=0,Ur=null,Cl(a,L,Q,ue),s&&hl){A=0;break e}break;default:ue=ln,ln=0,Ur=null,Cl(a,L,Q,ue)}}RM(),A=In;break}catch(Ce){_S(a,Ce)}while(!0);return i&&a.shellSuspendCounter++,oo=Ci=null,on=f,_.H=h,_.A=S,zt===null&&(Cn=null,qt=0,uu()),A}function RM(){for(;zt!==null;)IS(zt)}function TM(a,i){var s=on;on|=2;var f=NS(),h=AS();Cn!==a||qt!==i?(Fu=null,Hu=$e()+500,Sl(a,i)):hl=qe(a,i);e:do try{if(ln!==0&&zt!==null){i=zt;var S=Ur;t:switch(ln){case 1:ln=0,Ur=null,Cl(a,i,S,1);break;case 2:case 9:if(Wy(S)){ln=0,Ur=null,jS(i);break}i=function(){ln!==2&&ln!==9||Cn!==a||(ln=7),Ua(a)},S.then(i,i);break e;case 3:ln=7;break e;case 4:ln=5;break e;case 7:Wy(S)?(ln=0,Ur=null,jS(i)):(ln=0,Ur=null,Cl(a,i,S,7));break;case 5:var A=null;switch(zt.tag){case 26:A=zt.memoizedState;case 5:case 27:var L=zt;if(!A||gC(A)){ln=0,Ur=null;var Q=L.sibling;if(Q!==null)zt=Q;else{var ue=L.return;ue!==null?(zt=ue,Uu(ue)):zt=null}break t}}ln=0,Ur=null,Cl(a,i,S,5);break;case 6:ln=0,Ur=null,Cl(a,i,S,6);break;case 8:Dv(),In=6;break e;default:throw Error(r(462))}}MM();break}catch(Ce){_S(a,Ce)}while(!0);return oo=Ci=null,_.H=f,_.A=h,on=s,zt!==null?0:(Cn=null,qt=0,uu(),In)}function MM(){for(;zt!==null&&!se();)IS(zt)}function IS(a){var i=iS(a.alternate,a,vo);a.memoizedProps=a.pendingProps,i===null?Uu(a):zt=i}function jS(a){var i=a,s=i.alternate;switch(i.tag){case 15:case 0:i=eS(s,i,i.pendingProps,i.type,void 0,qt);break;case 11:i=eS(s,i,i.pendingProps,i.type.render,i.ref,qt);break;case 5:ev(i);default:sS(s,i),i=zt=Py(i,vo),i=iS(s,i,vo)}a.memoizedProps=a.pendingProps,i===null?Uu(a):zt=i}function Cl(a,i,s,f){oo=Ci=null,ev(i),dl=null,As=0;var h=i.return;try{if(yM(a,h,i,s,qt)){In=1,ju(a,ia(s,a.current)),zt=null;return}}catch(S){if(h!==null)throw zt=h,S;In=1,ju(a,ia(s,a.current)),zt=null;return}i.flags&32768?(Zt||f===1?a=!0:hl||(qt&536870912)!==0?a=!1:(Ho=a=!0,(f===2||f===9||f===3||f===6)&&(f=ua.current,f!==null&&f.tag===13&&(f.flags|=16384))),PS(i,a)):Uu(i)}function Uu(a){var i=a;do{if((i.flags&32768)!==0){PS(i,Ho);return}a=i.return;var s=SM(i.alternate,i,vo);if(s!==null){zt=s;return}if(i=i.sibling,i!==null){zt=i;return}zt=i=a}while(i!==null);In===0&&(In=5)}function PS(a,i){do{var s=CM(a.alternate,a);if(s!==null){s.flags&=32767,zt=s;return}if(s=a.return,s!==null&&(s.flags|=32768,s.subtreeFlags=0,s.deletions=null),!i&&(a=a.sibling,a!==null)){zt=a;return}zt=a=s}while(a!==null);In=6,zt=null}function zS(a,i,s,f,h,S,A,L,Q){a.cancelPendingCommit=null;do qu();while(fr!==0);if((on&6)!==0)throw Error(r(327));if(i!==null){if(i===a.current)throw Error(r(177));if(S=i.lanes|i.childLanes,S|=Mm,sn(a,s,S,A,L,Q),a===Cn&&(zt=Cn=null,qt=0),yl=i,Uo=a,bl=s,jv=S,Pv=h,OS=f,(i.subtreeFlags&10256)!==0||(i.flags&10256)!==0?(a.callbackNode=null,a.callbackPriority=0,IM(He,function(){return FS(),null})):(a.callbackNode=null,a.callbackPriority=0),f=(i.flags&13878)!==0,(i.subtreeFlags&13878)!==0||f){f=_.T,_.T=null,h=j.p,j.p=2,A=on,on|=4;try{xM(a,i,s)}finally{on=A,j.p=h,_.T=f}}fr=1,DS(),BS(),LS()}}function DS(){if(fr===1){fr=0;var a=Uo,i=yl,s=(i.flags&13878)!==0;if((i.subtreeFlags&13878)!==0||s){s=_.T,_.T=null;var f=j.p;j.p=2;var h=on;on|=4;try{bS(i,a);var S=Qv,A=$y(a.containerInfo),L=S.focusedElem,Q=S.selectionRange;if(A!==L&&L&&L.ownerDocument&&wy(L.ownerDocument.documentElement,L)){if(Q!==null&&wm(L)){var ue=Q.start,Ce=Q.end;if(Ce===void 0&&(Ce=ue),"selectionStart"in L)L.selectionStart=ue,L.selectionEnd=Math.min(Ce,L.value.length);else{var Oe=L.ownerDocument||document,me=Oe&&Oe.defaultView||window;if(me.getSelection){var ge=me.getSelection(),bt=L.textContent.length,mt=Math.min(Q.start,bt),mn=Q.end===void 0?mt:Math.min(Q.end,bt);!ge.extend&&mt>mn&&(A=mn,mn=mt,mt=A);var ae=Ey(L,mt),ee=Ey(L,mn);if(ae&&ee&&(ge.rangeCount!==1||ge.anchorNode!==ae.node||ge.anchorOffset!==ae.offset||ge.focusNode!==ee.node||ge.focusOffset!==ee.offset)){var ce=Oe.createRange();ce.setStart(ae.node,ae.offset),ge.removeAllRanges(),mt>mn?(ge.addRange(ce),ge.extend(ee.node,ee.offset)):(ce.setEnd(ee.node,ee.offset),ge.addRange(ce))}}}}for(Oe=[],ge=L;ge=ge.parentNode;)ge.nodeType===1&&Oe.push({element:ge,left:ge.scrollLeft,top:ge.scrollTop});for(typeof L.focus=="function"&&L.focus(),L=0;L<Oe.length;L++){var Ee=Oe[L];Ee.element.scrollLeft=Ee.left,Ee.element.scrollTop=Ee.top}}nf=!!Yv,Qv=Yv=null}finally{on=h,j.p=f,_.T=s}}a.current=i,fr=2}}function BS(){if(fr===2){fr=0;var a=Uo,i=yl,s=(i.flags&8772)!==0;if((i.subtreeFlags&8772)!==0||s){s=_.T,_.T=null;var f=j.p;j.p=2;var h=on;on|=4;try{gS(a,i.alternate,i)}finally{on=h,j.p=f,_.T=s}}fr=3}}function LS(){if(fr===4||fr===3){fr=0,ye();var a=Uo,i=yl,s=bl,f=OS;(i.subtreeFlags&10256)!==0||(i.flags&10256)!==0?fr=5:(fr=0,yl=Uo=null,HS(a,a.pendingLanes));var h=a.pendingLanes;if(h===0&&(Vo=null),nt(s),i=i.stateNode,We&&typeof We.onCommitFiberRoot=="function")try{We.onCommitFiberRoot(Ke,i,void 0,(i.current.flags&128)===128)}catch{}if(f!==null){i=_.T,h=j.p,j.p=2,_.T=null;try{for(var S=a.onRecoverableError,A=0;A<f.length;A++){var L=f[A];S(L.value,{componentStack:L.stack})}}finally{_.T=i,j.p=h}}(bl&3)!==0&&qu(),Ua(a),h=a.pendingLanes,(s&4194090)!==0&&(h&42)!==0?a===zv?Fs++:(Fs=0,zv=a):Fs=0,Vs(0)}}function HS(a,i){(a.pooledCacheLanes&=i)===0&&(i=a.pooledCache,i!=null&&(a.pooledCache=null,Cs(i)))}function qu(a){return DS(),BS(),LS(),FS()}function FS(){if(fr!==5)return!1;var a=Uo,i=jv;jv=0;var s=nt(bl),f=_.T,h=j.p;try{j.p=32>s?32:s,_.T=null,s=Pv,Pv=null;var S=Uo,A=bl;if(fr=0,yl=Uo=null,bl=0,(on&6)!==0)throw Error(r(331));var L=on;if(on|=4,wS(S.current),CS(S,S.current,A,s),on=L,Vs(0,!1),We&&typeof We.onPostCommitFiberRoot=="function")try{We.onPostCommitFiberRoot(Ke,S)}catch{}return!0}finally{j.p=h,_.T=f,HS(a,i)}}function VS(a,i,s){i=ia(s,i),i=vv(a.stateNode,i,2),a=Io(a,i,2),a!==null&&($n(a,2),Ua(a))}function yn(a,i,s){if(a.tag===3)VS(a,a,s);else for(;i!==null;){if(i.tag===3){VS(i,a,s);break}else if(i.tag===1){var f=i.stateNode;if(typeof i.type.getDerivedStateFromError=="function"||typeof f.componentDidCatch=="function"&&(Vo===null||!Vo.has(f))){a=ia(s,a),s=kb(2),f=Io(i,s,2),f!==null&&(Gb(s,f,i,a),$n(f,2),Ua(f));break}}i=i.return}}function Hv(a,i,s){var f=a.pingCache;if(f===null){f=a.pingCache=new $M;var h=new Set;f.set(i,h)}else h=f.get(i),h===void 0&&(h=new Set,f.set(i,h));h.has(s)||(_v=!0,h.add(s),a=_M.bind(null,a,i,s),i.then(a,a))}function _M(a,i,s){var f=a.pingCache;f!==null&&f.delete(i),a.pingedLanes|=a.suspendedLanes&s,a.warmLanes&=~s,Cn===a&&(qt&s)===s&&(In===4||In===3&&(qt&62914560)===qt&&300>$e()-Iv?(on&2)===0&&Sl(a,0):Nv|=s,pl===qt&&(pl=0)),Ua(a)}function US(a,i){i===0&&(i=_n()),a=nl(a,i),a!==null&&($n(a,i),Ua(a))}function NM(a){var i=a.memoizedState,s=0;i!==null&&(s=i.retryLane),US(a,s)}function AM(a,i){var s=0;switch(a.tag){case 13:var f=a.stateNode,h=a.memoizedState;h!==null&&(s=h.retryLane);break;case 19:f=a.stateNode;break;case 22:f=a.stateNode._retryCache;break;default:throw Error(r(314))}f!==null&&f.delete(i),US(a,s)}function IM(a,i){return pe(a,i)}var Wu=null,xl=null,Fv=!1,ku=!1,Vv=!1,Ti=0;function Ua(a){a!==xl&&a.next===null&&(xl===null?Wu=xl=a:xl=xl.next=a),ku=!0,Fv||(Fv=!0,PM())}function Vs(a,i){if(!Vv&&ku){Vv=!0;do for(var s=!1,f=Wu;f!==null;){if(a!==0){var h=f.pendingLanes;if(h===0)var S=0;else{var A=f.suspendedLanes,L=f.pingedLanes;S=(1<<31-Ue(42|a)+1)-1,S&=h&~(A&~L),S=S&201326741?S&201326741|1:S?S|2:0}S!==0&&(s=!0,GS(f,S))}else S=qt,S=_e(f,f===Cn?S:0,f.cancelPendingCommit!==null||f.timeoutHandle!==-1),(S&3)===0||qe(f,S)||(s=!0,GS(f,S));f=f.next}while(s);Vv=!1}}function jM(){qS()}function qS(){ku=Fv=!1;var a=0;Ti!==0&&(UM()&&(a=Ti),Ti=0);for(var i=$e(),s=null,f=Wu;f!==null;){var h=f.next,S=WS(f,i);S===0?(f.next=null,s===null?Wu=h:s.next=h,h===null&&(xl=s)):(s=f,(a!==0||(S&3)!==0)&&(ku=!0)),f=h}Vs(a)}function WS(a,i){for(var s=a.suspendedLanes,f=a.pingedLanes,h=a.expirationTimes,S=a.pendingLanes&-62914561;0<S;){var A=31-Ue(S),L=1<<A,Q=h[A];Q===-1?((L&s)===0||(L&f)!==0)&&(h[A]=Ct(L,i)):Q<=i&&(a.expiredLanes|=L),S&=~L}if(i=Cn,s=qt,s=_e(a,a===i?s:0,a.cancelPendingCommit!==null||a.timeoutHandle!==-1),f=a.callbackNode,s===0||a===i&&(ln===2||ln===9)||a.cancelPendingCommit!==null)return f!==null&&f!==null&&Re(f),a.callbackNode=null,a.callbackPriority=0;if((s&3)===0||qe(a,s)){if(i=s&-s,i===a.callbackPriority)return i;switch(f!==null&&Re(f),nt(s)){case 2:case 8:s=Ze;break;case 32:s=He;break;case 268435456:s=xe;break;default:s=He}return f=kS.bind(null,a),s=pe(s,f),a.callbackPriority=i,a.callbackNode=s,i}return f!==null&&f!==null&&Re(f),a.callbackPriority=2,a.callbackNode=null,2}function kS(a,i){if(fr!==0&&fr!==5)return a.callbackNode=null,a.callbackPriority=0,null;var s=a.callbackNode;if(qu()&&a.callbackNode!==s)return null;var f=qt;return f=_e(a,a===Cn?f:0,a.cancelPendingCommit!==null||a.timeoutHandle!==-1),f===0?null:(TS(a,f,i),WS(a,$e()),a.callbackNode!=null&&a.callbackNode===s?kS.bind(null,a):null)}function GS(a,i){if(qu())return null;TS(a,i,!0)}function PM(){WM(function(){(on&6)!==0?pe(Ne,jM):qS()})}function Uv(){return Ti===0&&(Ti=Nt()),Ti}function KS(a){return a==null||typeof a=="symbol"||typeof a=="boolean"?null:typeof a=="function"?a:Za(""+a)}function XS(a,i){var s=i.ownerDocument.createElement("input");return s.name=i.name,s.value=i.value,a.id&&s.setAttribute("form",a.id),i.parentNode.insertBefore(s,i),a=new FormData(a),s.parentNode.removeChild(s),a}function zM(a,i,s,f,h){if(i==="submit"&&s&&s.stateNode===h){var S=KS((h[Be]||null).action),A=f.submitter;A&&(i=(i=A[Be]||null)?KS(i.formAction):A.getAttribute("formAction"),i!==null&&(S=i,A=null));var L=new lu("action","action",null,f,h);a.push({event:L,listeners:[{instance:null,listener:function(){if(f.defaultPrevented){if(Ti!==0){var Q=A?XS(h,A):new FormData(h);cv(s,{pending:!0,data:Q,method:h.method,action:S},null,Q)}}else typeof S=="function"&&(L.preventDefault(),Q=A?XS(h,A):new FormData(h),cv(s,{pending:!0,data:Q,method:h.method,action:S},S,Q))},currentTarget:h}]})}}for(var qv=0;qv<Tm.length;qv++){var Wv=Tm[qv],DM=Wv.toLowerCase(),BM=Wv[0].toUpperCase()+Wv.slice(1);xa(DM,"on"+BM)}xa(Ty,"onAnimationEnd"),xa(My,"onAnimationIteration"),xa(_y,"onAnimationStart"),xa("dblclick","onDoubleClick"),xa("focusin","onFocus"),xa("focusout","onBlur"),xa(tM,"onTransitionRun"),xa(nM,"onTransitionStart"),xa(rM,"onTransitionCancel"),xa(Ny,"onTransitionEnd"),Ae("onMouseEnter",["mouseout","mouseover"]),Ae("onMouseLeave",["mouseout","mouseover"]),Ae("onPointerEnter",["pointerout","pointerover"]),Ae("onPointerLeave",["pointerout","pointerover"]),an("onChange","change click focusin focusout input keydown keyup selectionchange".split(" ")),an("onSelect","focusout contextmenu dragend focusin keydown keyup mousedown mouseup selectionchange".split(" ")),an("onBeforeInput",["compositionend","keypress","textInput","paste"]),an("onCompositionEnd","compositionend focusout keydown keypress keyup mousedown".split(" ")),an("onCompositionStart","compositionstart focusout keydown keypress keyup mousedown".split(" ")),an("onCompositionUpdate","compositionupdate focusout keydown keypress keyup mousedown".split(" "));var Us="abort canplay canplaythrough durationchange emptied encrypted ended error loadeddata loadedmetadata loadstart pause play playing progress ratechange resize seeked seeking stalled suspend timeupdate volumechange waiting".split(" "),LM=new Set("beforetoggle cancel close invalid load scroll scrollend toggle".split(" ").concat(Us));function YS(a,i){i=(i&4)!==0;for(var s=0;s<a.length;s++){var f=a[s],h=f.event;f=f.listeners;e:{var S=void 0;if(i)for(var A=f.length-1;0<=A;A--){var L=f[A],Q=L.instance,ue=L.currentTarget;if(L=L.listener,Q!==S&&h.isPropagationStopped())break e;S=L,h.currentTarget=ue;try{S(h)}catch(Ce){Iu(Ce)}h.currentTarget=null,S=Q}else for(A=0;A<f.length;A++){if(L=f[A],Q=L.instance,ue=L.currentTarget,L=L.listener,Q!==S&&h.isPropagationStopped())break e;S=L,h.currentTarget=ue;try{S(h)}catch(Ce){Iu(Ce)}h.currentTarget=null,S=Q}}}}function Dt(a,i){var s=i[rt];s===void 0&&(s=i[rt]=new Set);var f=a+"__bubble";s.has(f)||(QS(i,a,2,!1),s.add(f))}function kv(a,i,s){var f=0;i&&(f|=4),QS(s,a,f,i)}var Gu="_reactListening"+Math.random().toString(36).slice(2);function Gv(a){if(!a[Gu]){a[Gu]=!0,er.forEach(function(s){s!=="selectionchange"&&(LM.has(s)||kv(s,!1,a),kv(s,!0,a))});var i=a.nodeType===9?a:a.ownerDocument;i===null||i[Gu]||(i[Gu]=!0,kv("selectionchange",!1,i))}}function QS(a,i,s,f){switch(CC(i)){case 2:var h=d_;break;case 8:h=m_;break;default:h=lg}s=h.bind(null,i,s,a),h=void 0,!dt||i!=="touchstart"&&i!=="touchmove"&&i!=="wheel"||(h=!0),f?h!==void 0?a.addEventListener(i,s,{capture:!0,passive:h}):a.addEventListener(i,s,!0):h!==void 0?a.addEventListener(i,s,{passive:h}):a.addEventListener(i,s,!1)}function Kv(a,i,s,f,h){var S=f;if((i&1)===0&&(i&2)===0&&f!==null)e:for(;;){if(f===null)return;var A=f.tag;if(A===3||A===4){var L=f.stateNode.containerInfo;if(L===h)break;if(A===4)for(A=f.return;A!==null;){var Q=A.tag;if((Q===3||Q===4)&&A.stateNode.containerInfo===h)return;A=A.return}for(;L!==null;){if(A=At(L),A===null)return;if(Q=A.tag,Q===5||Q===6||Q===26||Q===27){f=S=A;continue e}L=L.parentNode}}f=f.return}Yi(function(){var ue=S,Ce=aa(s),Oe=[];e:{var me=Ay.get(a);if(me!==void 0){var ge=lu,bt=a;switch(a){case"keypress":if(di(s)===0)break e;case"keydown":case"keyup":ge=IT;break;case"focusin":bt="focus",ge=bm;break;case"focusout":bt="blur",ge=bm;break;case"beforeblur":case"afterblur":ge=bm;break;case"click":if(s.button===2)break e;case"auxclick":case"dblclick":case"mousedown":case"mousemove":case"mouseup":case"mouseout":case"mouseover":case"contextmenu":ge=ly;break;case"drag":case"dragend":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"dragstart":case"drop":ge=CT;break;case"touchcancel":case"touchend":case"touchmove":case"touchstart":ge=zT;break;case Ty:case My:case _y:ge=wT;break;case Ny:ge=BT;break;case"scroll":case"scrollend":ge=bT;break;case"wheel":ge=HT;break;case"copy":case"cut":case"paste":ge=OT;break;case"gotpointercapture":case"lostpointercapture":case"pointercancel":case"pointerdown":case"pointermove":case"pointerout":case"pointerover":case"pointerup":ge=cy;break;case"toggle":case"beforetoggle":ge=VT}var mt=(i&4)!==0,mn=!mt&&(a==="scroll"||a==="scrollend"),ae=mt?me!==null?me+"Capture":null:me;mt=[];for(var ee=ue,ce;ee!==null;){var Ee=ee;if(ce=Ee.stateNode,Ee=Ee.tag,Ee!==5&&Ee!==26&&Ee!==27||ce===null||ae===null||(Ee=Ja(ee,ae),Ee!=null&&mt.push(qs(ee,Ee,ce))),mn)break;ee=ee.return}0<mt.length&&(me=new ge(me,bt,null,s,Ce),Oe.push({event:me,listeners:mt}))}}if((i&7)===0){e:{if(me=a==="mouseover"||a==="pointerover",ge=a==="mouseout"||a==="pointerout",me&&s!==Ba&&(bt=s.relatedTarget||s.fromElement)&&(At(bt)||bt[je]))break e;if((ge||me)&&(me=Ce.window===Ce?Ce:(me=Ce.ownerDocument)?me.defaultView||me.parentWindow:window,ge?(bt=s.relatedTarget||s.toElement,ge=ue,bt=bt?At(bt):null,bt!==null&&(mn=l(bt),mt=bt.tag,bt!==mn||mt!==5&&mt!==27&&mt!==6)&&(bt=null)):(ge=null,bt=ue),ge!==bt)){if(mt=ly,Ee="onMouseLeave",ae="onMouseEnter",ee="mouse",(a==="pointerout"||a==="pointerover")&&(mt=cy,Ee="onPointerLeave",ae="onPointerEnter",ee="pointer"),mn=ge==null?me:Qt(ge),ce=bt==null?me:Qt(bt),me=new mt(Ee,ee+"leave",ge,s,Ce),me.target=mn,me.relatedTarget=ce,Ee=null,At(Ce)===ue&&(mt=new mt(ae,ee+"enter",bt,s,Ce),mt.target=ce,mt.relatedTarget=mn,Ee=mt),mn=Ee,ge&&bt)t:{for(mt=ge,ae=bt,ee=0,ce=mt;ce;ce=El(ce))ee++;for(ce=0,Ee=ae;Ee;Ee=El(Ee))ce++;for(;0<ee-ce;)mt=El(mt),ee--;for(;0<ce-ee;)ae=El(ae),ce--;for(;ee--;){if(mt===ae||ae!==null&&mt===ae.alternate)break t;mt=El(mt),ae=El(ae)}mt=null}else mt=null;ge!==null&&ZS(Oe,me,ge,mt,!1),bt!==null&&mn!==null&&ZS(Oe,mn,bt,mt,!0)}}e:{if(me=ue?Qt(ue):window,ge=me.nodeName&&me.nodeName.toLowerCase(),ge==="select"||ge==="input"&&me.type==="file")var it=py;else if(gy(me))if(yy)it=ZT;else{it=YT;var jt=XT}else ge=me.nodeName,!ge||ge.toLowerCase()!=="input"||me.type!=="checkbox"&&me.type!=="radio"?ue&&Xn(ue.elementType)&&(it=py):it=QT;if(it&&(it=it(a,ue))){hy(Oe,it,s,Ce);break e}jt&&jt(a,me,ue),a==="focusout"&&ue&&me.type==="number"&&ue.memoizedProps.value!=null&&Kt(me,"number",me.value)}switch(jt=ue?Qt(ue):window,a){case"focusin":(gy(jt)||jt.contentEditable==="true")&&(Ji=jt,$m=ue,hs=null);break;case"focusout":hs=$m=Ji=null;break;case"mousedown":Om=!0;break;case"contextmenu":case"mouseup":case"dragend":Om=!1,Oy(Oe,s,Ce);break;case"selectionchange":if(eM)break;case"keydown":case"keyup":Oy(Oe,s,Ce)}var ft;if(Cm)e:{switch(a){case"compositionstart":var gt="onCompositionStart";break e;case"compositionend":gt="onCompositionEnd";break e;case"compositionupdate":gt="onCompositionUpdate";break e}gt=void 0}else Zi?my(a,s)&&(gt="onCompositionEnd"):a==="keydown"&&s.keyCode===229&&(gt="onCompositionStart");gt&&(uy&&s.locale!=="ko"&&(Zi||gt!=="onCompositionStart"?gt==="onCompositionEnd"&&Zi&&(ft=Qi()):(un=Ce,Sn="value"in un?un.value:un.textContent,Zi=!0)),jt=Ku(ue,gt),0<jt.length&&(gt=new sy(gt,a,null,s,Ce),Oe.push({event:gt,listeners:jt}),ft?gt.data=ft:(ft=vy(s),ft!==null&&(gt.data=ft)))),(ft=qT?WT(a,s):kT(a,s))&&(gt=Ku(ue,"onBeforeInput"),0<gt.length&&(jt=new sy("onBeforeInput","beforeinput",null,s,Ce),Oe.push({event:jt,listeners:gt}),jt.data=ft)),zM(Oe,a,ue,s,Ce)}YS(Oe,i)})}function qs(a,i,s){return{instance:a,listener:i,currentTarget:s}}function Ku(a,i){for(var s=i+"Capture",f=[];a!==null;){var h=a,S=h.stateNode;if(h=h.tag,h!==5&&h!==26&&h!==27||S===null||(h=Ja(a,s),h!=null&&f.unshift(qs(a,h,S)),h=Ja(a,i),h!=null&&f.push(qs(a,h,S))),a.tag===3)return f;a=a.return}return[]}function El(a){if(a===null)return null;do a=a.return;while(a&&a.tag!==5&&a.tag!==27);return a||null}function ZS(a,i,s,f,h){for(var S=i._reactName,A=[];s!==null&&s!==f;){var L=s,Q=L.alternate,ue=L.stateNode;if(L=L.tag,Q!==null&&Q===f)break;L!==5&&L!==26&&L!==27||ue===null||(Q=ue,h?(ue=Ja(s,S),ue!=null&&A.unshift(qs(s,ue,Q))):h||(ue=Ja(s,S),ue!=null&&A.push(qs(s,ue,Q)))),s=s.return}A.length!==0&&a.push({event:i,listeners:A})}var HM=/\r\n?/g,FM=/\u0000|\uFFFD/g;function JS(a){return(typeof a=="string"?a:""+a).replace(HM,`
`).replace(FM,"")}function eC(a,i){return i=JS(i),JS(a)===i}function Xu(){}function dn(a,i,s,f,h,S){switch(s){case"children":typeof f=="string"?i==="body"||i==="textarea"&&f===""||Nn(a,f):(typeof f=="number"||typeof f=="bigint")&&i!=="body"&&Nn(a,""+f);break;case"className":Pn(a,"class",f);break;case"tabIndex":Pn(a,"tabindex",f);break;case"dir":case"role":case"viewBox":case"width":case"height":Pn(a,s,f);break;case"style":xr(a,f,S);break;case"data":if(i!=="object"){Pn(a,"data",f);break}case"src":case"href":if(f===""&&(i!=="a"||s!=="href")){a.removeAttribute(s);break}if(f==null||typeof f=="function"||typeof f=="symbol"||typeof f=="boolean"){a.removeAttribute(s);break}f=Za(""+f),a.setAttribute(s,f);break;case"action":case"formAction":if(typeof f=="function"){a.setAttribute(s,"javascript:throw new Error('A React form was unexpectedly submitted. If you called form.submit() manually, consider using form.requestSubmit() instead. If you\\'re trying to use event.stopPropagation() in a submit event handler, consider also calling event.preventDefault().')");break}else typeof S=="function"&&(s==="formAction"?(i!=="input"&&dn(a,i,"name",h.name,h,null),dn(a,i,"formEncType",h.formEncType,h,null),dn(a,i,"formMethod",h.formMethod,h,null),dn(a,i,"formTarget",h.formTarget,h,null)):(dn(a,i,"encType",h.encType,h,null),dn(a,i,"method",h.method,h,null),dn(a,i,"target",h.target,h,null)));if(f==null||typeof f=="symbol"||typeof f=="boolean"){a.removeAttribute(s);break}f=Za(""+f),a.setAttribute(s,f);break;case"onClick":f!=null&&(a.onclick=Xu);break;case"onScroll":f!=null&&Dt("scroll",a);break;case"onScrollEnd":f!=null&&Dt("scrollend",a);break;case"dangerouslySetInnerHTML":if(f!=null){if(typeof f!="object"||!("__html"in f))throw Error(r(61));if(s=f.__html,s!=null){if(h.children!=null)throw Error(r(60));a.innerHTML=s}}break;case"multiple":a.multiple=f&&typeof f!="function"&&typeof f!="symbol";break;case"muted":a.muted=f&&typeof f!="function"&&typeof f!="symbol";break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"defaultValue":case"defaultChecked":case"innerHTML":case"ref":break;case"autoFocus":break;case"xlinkHref":if(f==null||typeof f=="function"||typeof f=="boolean"||typeof f=="symbol"){a.removeAttribute("xlink:href");break}s=Za(""+f),a.setAttributeNS("http://www.w3.org/1999/xlink","xlink:href",s);break;case"contentEditable":case"spellCheck":case"draggable":case"value":case"autoReverse":case"externalResourcesRequired":case"focusable":case"preserveAlpha":f!=null&&typeof f!="function"&&typeof f!="symbol"?a.setAttribute(s,""+f):a.removeAttribute(s);break;case"inert":case"allowFullScreen":case"async":case"autoPlay":case"controls":case"default":case"defer":case"disabled":case"disablePictureInPicture":case"disableRemotePlayback":case"formNoValidate":case"hidden":case"loop":case"noModule":case"noValidate":case"open":case"playsInline":case"readOnly":case"required":case"reversed":case"scoped":case"seamless":case"itemScope":f&&typeof f!="function"&&typeof f!="symbol"?a.setAttribute(s,""):a.removeAttribute(s);break;case"capture":case"download":f===!0?a.setAttribute(s,""):f!==!1&&f!=null&&typeof f!="function"&&typeof f!="symbol"?a.setAttribute(s,f):a.removeAttribute(s);break;case"cols":case"rows":case"size":case"span":f!=null&&typeof f!="function"&&typeof f!="symbol"&&!isNaN(f)&&1<=f?a.setAttribute(s,f):a.removeAttribute(s);break;case"rowSpan":case"start":f==null||typeof f=="function"||typeof f=="symbol"||isNaN(f)?a.removeAttribute(s):a.setAttribute(s,f);break;case"popover":Dt("beforetoggle",a),Dt("toggle",a),kn(a,"popover",f);break;case"xlinkActuate":Ln(a,"http://www.w3.org/1999/xlink","xlink:actuate",f);break;case"xlinkArcrole":Ln(a,"http://www.w3.org/1999/xlink","xlink:arcrole",f);break;case"xlinkRole":Ln(a,"http://www.w3.org/1999/xlink","xlink:role",f);break;case"xlinkShow":Ln(a,"http://www.w3.org/1999/xlink","xlink:show",f);break;case"xlinkTitle":Ln(a,"http://www.w3.org/1999/xlink","xlink:title",f);break;case"xlinkType":Ln(a,"http://www.w3.org/1999/xlink","xlink:type",f);break;case"xmlBase":Ln(a,"http://www.w3.org/XML/1998/namespace","xml:base",f);break;case"xmlLang":Ln(a,"http://www.w3.org/XML/1998/namespace","xml:lang",f);break;case"xmlSpace":Ln(a,"http://www.w3.org/XML/1998/namespace","xml:space",f);break;case"is":kn(a,"is",f);break;case"innerText":case"textContent":break;default:(!(2<s.length)||s[0]!=="o"&&s[0]!=="O"||s[1]!=="n"&&s[1]!=="N")&&(s=cs.get(s)||s,kn(a,s,f))}}function Xv(a,i,s,f,h,S){switch(s){case"style":xr(a,f,S);break;case"dangerouslySetInnerHTML":if(f!=null){if(typeof f!="object"||!("__html"in f))throw Error(r(61));if(s=f.__html,s!=null){if(h.children!=null)throw Error(r(60));a.innerHTML=s}}break;case"children":typeof f=="string"?Nn(a,f):(typeof f=="number"||typeof f=="bigint")&&Nn(a,""+f);break;case"onScroll":f!=null&&Dt("scroll",a);break;case"onScrollEnd":f!=null&&Dt("scrollend",a);break;case"onClick":f!=null&&(a.onclick=Xu);break;case"suppressContentEditableWarning":case"suppressHydrationWarning":case"innerHTML":case"ref":break;case"innerText":case"textContent":break;default:if(!pn.hasOwnProperty(s))e:{if(s[0]==="o"&&s[1]==="n"&&(h=s.endsWith("Capture"),i=s.slice(2,h?s.length-7:void 0),S=a[Be]||null,S=S!=null?S[s]:null,typeof S=="function"&&a.removeEventListener(i,S,h),typeof f=="function")){typeof S!="function"&&S!==null&&(s in a?a[s]=null:a.hasAttribute(s)&&a.removeAttribute(s)),a.addEventListener(i,f,h);break e}s in a?a[s]=f:f===!0?a.setAttribute(s,""):kn(a,s,f)}}}function dr(a,i,s){switch(i){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"img":Dt("error",a),Dt("load",a);var f=!1,h=!1,S;for(S in s)if(s.hasOwnProperty(S)){var A=s[S];if(A!=null)switch(S){case"src":f=!0;break;case"srcSet":h=!0;break;case"children":case"dangerouslySetInnerHTML":throw Error(r(137,i));default:dn(a,i,S,A,s,null)}}h&&dn(a,i,"srcSet",s.srcSet,s,null),f&&dn(a,i,"src",s.src,s,null);return;case"input":Dt("invalid",a);var L=S=A=h=null,Q=null,ue=null;for(f in s)if(s.hasOwnProperty(f)){var Ce=s[f];if(Ce!=null)switch(f){case"name":h=Ce;break;case"type":A=Ce;break;case"checked":Q=Ce;break;case"defaultChecked":ue=Ce;break;case"value":S=Ce;break;case"defaultValue":L=Ce;break;case"children":case"dangerouslySetInnerHTML":if(Ce!=null)throw Error(r(137,i));break;default:dn(a,i,f,Ce,s,null)}}Ut(a,S,L,Q,ue,A,h,!1),Kn(a);return;case"select":Dt("invalid",a),f=A=S=null;for(h in s)if(s.hasOwnProperty(h)&&(L=s[h],L!=null))switch(h){case"value":S=L;break;case"defaultValue":A=L;break;case"multiple":f=L;default:dn(a,i,h,L,s,null)}i=S,s=A,a.multiple=!!f,i!=null?Xt(a,!!f,i,!1):s!=null&&Xt(a,!!f,s,!0);return;case"textarea":Dt("invalid",a),S=h=f=null;for(A in s)if(s.hasOwnProperty(A)&&(L=s[A],L!=null))switch(A){case"value":f=L;break;case"defaultValue":h=L;break;case"children":S=L;break;case"dangerouslySetInnerHTML":if(L!=null)throw Error(r(91));break;default:dn(a,i,A,L,s,null)}Hn(a,f,h,S),Kn(a);return;case"option":for(Q in s)if(s.hasOwnProperty(Q)&&(f=s[Q],f!=null))switch(Q){case"selected":a.selected=f&&typeof f!="function"&&typeof f!="symbol";break;default:dn(a,i,Q,f,s,null)}return;case"dialog":Dt("beforetoggle",a),Dt("toggle",a),Dt("cancel",a),Dt("close",a);break;case"iframe":case"object":Dt("load",a);break;case"video":case"audio":for(f=0;f<Us.length;f++)Dt(Us[f],a);break;case"image":Dt("error",a),Dt("load",a);break;case"details":Dt("toggle",a);break;case"embed":case"source":case"link":Dt("error",a),Dt("load",a);case"area":case"base":case"br":case"col":case"hr":case"keygen":case"meta":case"param":case"track":case"wbr":case"menuitem":for(ue in s)if(s.hasOwnProperty(ue)&&(f=s[ue],f!=null))switch(ue){case"children":case"dangerouslySetInnerHTML":throw Error(r(137,i));default:dn(a,i,ue,f,s,null)}return;default:if(Xn(i)){for(Ce in s)s.hasOwnProperty(Ce)&&(f=s[Ce],f!==void 0&&Xv(a,i,Ce,f,s,void 0));return}}for(L in s)s.hasOwnProperty(L)&&(f=s[L],f!=null&&dn(a,i,L,f,s,null))}function VM(a,i,s,f){switch(i){case"div":case"span":case"svg":case"path":case"a":case"g":case"p":case"li":break;case"input":var h=null,S=null,A=null,L=null,Q=null,ue=null,Ce=null;for(ge in s){var Oe=s[ge];if(s.hasOwnProperty(ge)&&Oe!=null)switch(ge){case"checked":break;case"value":break;case"defaultValue":Q=Oe;default:f.hasOwnProperty(ge)||dn(a,i,ge,null,f,Oe)}}for(var me in f){var ge=f[me];if(Oe=s[me],f.hasOwnProperty(me)&&(ge!=null||Oe!=null))switch(me){case"type":S=ge;break;case"name":h=ge;break;case"checked":ue=ge;break;case"defaultChecked":Ce=ge;break;case"value":A=ge;break;case"defaultValue":L=ge;break;case"children":case"dangerouslySetInnerHTML":if(ge!=null)throw Error(r(137,i));break;default:ge!==Oe&&dn(a,i,me,ge,f,Oe)}}Lt(a,A,L,Q,ue,Ce,S,h);return;case"select":ge=A=L=me=null;for(S in s)if(Q=s[S],s.hasOwnProperty(S)&&Q!=null)switch(S){case"value":break;case"multiple":ge=Q;default:f.hasOwnProperty(S)||dn(a,i,S,null,f,Q)}for(h in f)if(S=f[h],Q=s[h],f.hasOwnProperty(h)&&(S!=null||Q!=null))switch(h){case"value":me=S;break;case"defaultValue":L=S;break;case"multiple":A=S;default:S!==Q&&dn(a,i,h,S,f,Q)}i=L,s=A,f=ge,me!=null?Xt(a,!!s,me,!1):!!f!=!!s&&(i!=null?Xt(a,!!s,i,!0):Xt(a,!!s,s?[]:"",!1));return;case"textarea":ge=me=null;for(L in s)if(h=s[L],s.hasOwnProperty(L)&&h!=null&&!f.hasOwnProperty(L))switch(L){case"value":break;case"children":break;default:dn(a,i,L,null,f,h)}for(A in f)if(h=f[A],S=s[A],f.hasOwnProperty(A)&&(h!=null||S!=null))switch(A){case"value":me=h;break;case"defaultValue":ge=h;break;case"children":break;case"dangerouslySetInnerHTML":if(h!=null)throw Error(r(91));break;default:h!==S&&dn(a,i,A,h,f,S)}nr(a,me,ge);return;case"option":for(var bt in s)if(me=s[bt],s.hasOwnProperty(bt)&&me!=null&&!f.hasOwnProperty(bt))switch(bt){case"selected":a.selected=!1;break;default:dn(a,i,bt,null,f,me)}for(Q in f)if(me=f[Q],ge=s[Q],f.hasOwnProperty(Q)&&me!==ge&&(me!=null||ge!=null))switch(Q){case"selected":a.selected=me&&typeof me!="function"&&typeof me!="symbol";break;default:dn(a,i,Q,me,f,ge)}return;case"img":case"link":case"area":case"base":case"br":case"col":case"embed":case"hr":case"keygen":case"meta":case"param":case"source":case"track":case"wbr":case"menuitem":for(var mt in s)me=s[mt],s.hasOwnProperty(mt)&&me!=null&&!f.hasOwnProperty(mt)&&dn(a,i,mt,null,f,me);for(ue in f)if(me=f[ue],ge=s[ue],f.hasOwnProperty(ue)&&me!==ge&&(me!=null||ge!=null))switch(ue){case"children":case"dangerouslySetInnerHTML":if(me!=null)throw Error(r(137,i));break;default:dn(a,i,ue,me,f,ge)}return;default:if(Xn(i)){for(var mn in s)me=s[mn],s.hasOwnProperty(mn)&&me!==void 0&&!f.hasOwnProperty(mn)&&Xv(a,i,mn,void 0,f,me);for(Ce in f)me=f[Ce],ge=s[Ce],!f.hasOwnProperty(Ce)||me===ge||me===void 0&&ge===void 0||Xv(a,i,Ce,me,f,ge);return}}for(var ae in s)me=s[ae],s.hasOwnProperty(ae)&&me!=null&&!f.hasOwnProperty(ae)&&dn(a,i,ae,null,f,me);for(Oe in f)me=f[Oe],ge=s[Oe],!f.hasOwnProperty(Oe)||me===ge||me==null&&ge==null||dn(a,i,Oe,me,f,ge)}var Yv=null,Qv=null;function Yu(a){return a.nodeType===9?a:a.ownerDocument}function tC(a){switch(a){case"http://www.w3.org/2000/svg":return 1;case"http://www.w3.org/1998/Math/MathML":return 2;default:return 0}}function nC(a,i){if(a===0)switch(i){case"svg":return 1;case"math":return 2;default:return 0}return a===1&&i==="foreignObject"?0:a}function Zv(a,i){return a==="textarea"||a==="noscript"||typeof i.children=="string"||typeof i.children=="number"||typeof i.children=="bigint"||typeof i.dangerouslySetInnerHTML=="object"&&i.dangerouslySetInnerHTML!==null&&i.dangerouslySetInnerHTML.__html!=null}var Jv=null;function UM(){var a=window.event;return a&&a.type==="popstate"?a===Jv?!1:(Jv=a,!0):(Jv=null,!1)}var rC=typeof setTimeout=="function"?setTimeout:void 0,qM=typeof clearTimeout=="function"?clearTimeout:void 0,aC=typeof Promise=="function"?Promise:void 0,WM=typeof queueMicrotask=="function"?queueMicrotask:typeof aC<"u"?function(a){return aC.resolve(null).then(a).catch(kM)}:rC;function kM(a){setTimeout(function(){throw a})}function Wo(a){return a==="head"}function oC(a,i){var s=i,f=0,h=0;do{var S=s.nextSibling;if(a.removeChild(s),S&&S.nodeType===8)if(s=S.data,s==="/$"){if(0<f&&8>f){s=f;var A=a.ownerDocument;if(s&1&&Ws(A.documentElement),s&2&&Ws(A.body),s&4)for(s=A.head,Ws(s),A=s.firstChild;A;){var L=A.nextSibling,Q=A.nodeName;A[Vt]||Q==="SCRIPT"||Q==="STYLE"||Q==="LINK"&&A.rel.toLowerCase()==="stylesheet"||s.removeChild(A),A=L}}if(h===0){a.removeChild(S),Js(i);return}h--}else s==="$"||s==="$?"||s==="$!"?h++:f=s.charCodeAt(0)-48;else f=0;s=S}while(s);Js(i)}function eg(a){var i=a.firstChild;for(i&&i.nodeType===10&&(i=i.nextSibling);i;){var s=i;switch(i=i.nextSibling,s.nodeName){case"HTML":case"HEAD":case"BODY":eg(s),Tt(s);continue;case"SCRIPT":case"STYLE":continue;case"LINK":if(s.rel.toLowerCase()==="stylesheet")continue}a.removeChild(s)}}function GM(a,i,s,f){for(;a.nodeType===1;){var h=s;if(a.nodeName.toLowerCase()!==i.toLowerCase()){if(!f&&(a.nodeName!=="INPUT"||a.type!=="hidden"))break}else if(f){if(!a[Vt])switch(i){case"meta":if(!a.hasAttribute("itemprop"))break;return a;case"link":if(S=a.getAttribute("rel"),S==="stylesheet"&&a.hasAttribute("data-precedence"))break;if(S!==h.rel||a.getAttribute("href")!==(h.href==null||h.href===""?null:h.href)||a.getAttribute("crossorigin")!==(h.crossOrigin==null?null:h.crossOrigin)||a.getAttribute("title")!==(h.title==null?null:h.title))break;return a;case"style":if(a.hasAttribute("data-precedence"))break;return a;case"script":if(S=a.getAttribute("src"),(S!==(h.src==null?null:h.src)||a.getAttribute("type")!==(h.type==null?null:h.type)||a.getAttribute("crossorigin")!==(h.crossOrigin==null?null:h.crossOrigin))&&S&&a.hasAttribute("async")&&!a.hasAttribute("itemprop"))break;return a;default:return a}}else if(i==="input"&&a.type==="hidden"){var S=h.name==null?null:""+h.name;if(h.type==="hidden"&&a.getAttribute("name")===S)return a}else return a;if(a=wa(a.nextSibling),a===null)break}return null}function KM(a,i,s){if(i==="")return null;for(;a.nodeType!==3;)if((a.nodeType!==1||a.nodeName!=="INPUT"||a.type!=="hidden")&&!s||(a=wa(a.nextSibling),a===null))return null;return a}function tg(a){return a.data==="$!"||a.data==="$?"&&a.ownerDocument.readyState==="complete"}function XM(a,i){var s=a.ownerDocument;if(a.data!=="$?"||s.readyState==="complete")i();else{var f=function(){i(),s.removeEventListener("DOMContentLoaded",f)};s.addEventListener("DOMContentLoaded",f),a._reactRetry=f}}function wa(a){for(;a!=null;a=a.nextSibling){var i=a.nodeType;if(i===1||i===3)break;if(i===8){if(i=a.data,i==="$"||i==="$!"||i==="$?"||i==="F!"||i==="F")break;if(i==="/$")return null}}return a}var ng=null;function iC(a){a=a.previousSibling;for(var i=0;a;){if(a.nodeType===8){var s=a.data;if(s==="$"||s==="$!"||s==="$?"){if(i===0)return a;i--}else s==="/$"&&i++}a=a.previousSibling}return null}function lC(a,i,s){switch(i=Yu(s),a){case"html":if(a=i.documentElement,!a)throw Error(r(452));return a;case"head":if(a=i.head,!a)throw Error(r(453));return a;case"body":if(a=i.body,!a)throw Error(r(454));return a;default:throw Error(r(451))}}function Ws(a){for(var i=a.attributes;i.length;)a.removeAttributeNode(i[0]);Tt(a)}var da=new Map,sC=new Set;function Qu(a){return typeof a.getRootNode=="function"?a.getRootNode():a.nodeType===9?a:a.ownerDocument}var go=j.d;j.d={f:YM,r:QM,D:ZM,C:JM,L:e_,m:t_,X:r_,S:n_,M:a_};function YM(){var a=go.f(),i=Vu();return a||i}function QM(a){var i=It(a);i!==null&&i.tag===5&&i.type==="form"?Tb(i):go.r(a)}var wl=typeof document>"u"?null:document;function cC(a,i,s){var f=wl;if(f&&typeof i=="string"&&i){var h=ze(i);h='link[rel="'+a+'"][href="'+h+'"]',typeof s=="string"&&(h+='[crossorigin="'+s+'"]'),sC.has(h)||(sC.add(h),a={rel:a,crossOrigin:s,href:i},f.querySelector(h)===null&&(i=f.createElement("link"),dr(i,"link",a),Rt(i),f.head.appendChild(i)))}}function ZM(a){go.D(a),cC("dns-prefetch",a,null)}function JM(a,i){go.C(a,i),cC("preconnect",a,i)}function e_(a,i,s){go.L(a,i,s);var f=wl;if(f&&a&&i){var h='link[rel="preload"][as="'+ze(i)+'"]';i==="image"&&s&&s.imageSrcSet?(h+='[imagesrcset="'+ze(s.imageSrcSet)+'"]',typeof s.imageSizes=="string"&&(h+='[imagesizes="'+ze(s.imageSizes)+'"]')):h+='[href="'+ze(a)+'"]';var S=h;switch(i){case"style":S=$l(a);break;case"script":S=Ol(a)}da.has(S)||(a=g({rel:"preload",href:i==="image"&&s&&s.imageSrcSet?void 0:a,as:i},s),da.set(S,a),f.querySelector(h)!==null||i==="style"&&f.querySelector(ks(S))||i==="script"&&f.querySelector(Gs(S))||(i=f.createElement("link"),dr(i,"link",a),Rt(i),f.head.appendChild(i)))}}function t_(a,i){go.m(a,i);var s=wl;if(s&&a){var f=i&&typeof i.as=="string"?i.as:"script",h='link[rel="modulepreload"][as="'+ze(f)+'"][href="'+ze(a)+'"]',S=h;switch(f){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":S=Ol(a)}if(!da.has(S)&&(a=g({rel:"modulepreload",href:a},i),da.set(S,a),s.querySelector(h)===null)){switch(f){case"audioworklet":case"paintworklet":case"serviceworker":case"sharedworker":case"worker":case"script":if(s.querySelector(Gs(S)))return}f=s.createElement("link"),dr(f,"link",a),Rt(f),s.head.appendChild(f)}}}function n_(a,i,s){go.S(a,i,s);var f=wl;if(f&&a){var h=rn(f).hoistableStyles,S=$l(a);i=i||"default";var A=h.get(S);if(!A){var L={loading:0,preload:null};if(A=f.querySelector(ks(S)))L.loading=5;else{a=g({rel:"stylesheet",href:a,"data-precedence":i},s),(s=da.get(S))&&rg(a,s);var Q=A=f.createElement("link");Rt(Q),dr(Q,"link",a),Q._p=new Promise(function(ue,Ce){Q.onload=ue,Q.onerror=Ce}),Q.addEventListener("load",function(){L.loading|=1}),Q.addEventListener("error",function(){L.loading|=2}),L.loading|=4,Zu(A,i,f)}A={type:"stylesheet",instance:A,count:1,state:L},h.set(S,A)}}}function r_(a,i){go.X(a,i);var s=wl;if(s&&a){var f=rn(s).hoistableScripts,h=Ol(a),S=f.get(h);S||(S=s.querySelector(Gs(h)),S||(a=g({src:a,async:!0},i),(i=da.get(h))&&ag(a,i),S=s.createElement("script"),Rt(S),dr(S,"link",a),s.head.appendChild(S)),S={type:"script",instance:S,count:1,state:null},f.set(h,S))}}function a_(a,i){go.M(a,i);var s=wl;if(s&&a){var f=rn(s).hoistableScripts,h=Ol(a),S=f.get(h);S||(S=s.querySelector(Gs(h)),S||(a=g({src:a,async:!0,type:"module"},i),(i=da.get(h))&&ag(a,i),S=s.createElement("script"),Rt(S),dr(S,"link",a),s.head.appendChild(S)),S={type:"script",instance:S,count:1,state:null},f.set(h,S))}}function uC(a,i,s,f){var h=(h=te.current)?Qu(h):null;if(!h)throw Error(r(446));switch(a){case"meta":case"title":return null;case"style":return typeof s.precedence=="string"&&typeof s.href=="string"?(i=$l(s.href),s=rn(h).hoistableStyles,f=s.get(i),f||(f={type:"style",instance:null,count:0,state:null},s.set(i,f)),f):{type:"void",instance:null,count:0,state:null};case"link":if(s.rel==="stylesheet"&&typeof s.href=="string"&&typeof s.precedence=="string"){a=$l(s.href);var S=rn(h).hoistableStyles,A=S.get(a);if(A||(h=h.ownerDocument||h,A={type:"stylesheet",instance:null,count:0,state:{loading:0,preload:null}},S.set(a,A),(S=h.querySelector(ks(a)))&&!S._p&&(A.instance=S,A.state.loading=5),da.has(a)||(s={rel:"preload",as:"style",href:s.href,crossOrigin:s.crossOrigin,integrity:s.integrity,media:s.media,hrefLang:s.hrefLang,referrerPolicy:s.referrerPolicy},da.set(a,s),S||o_(h,a,s,A.state))),i&&f===null)throw Error(r(528,""));return A}if(i&&f!==null)throw Error(r(529,""));return null;case"script":return i=s.async,s=s.src,typeof s=="string"&&i&&typeof i!="function"&&typeof i!="symbol"?(i=Ol(s),s=rn(h).hoistableScripts,f=s.get(i),f||(f={type:"script",instance:null,count:0,state:null},s.set(i,f)),f):{type:"void",instance:null,count:0,state:null};default:throw Error(r(444,a))}}function $l(a){return'href="'+ze(a)+'"'}function ks(a){return'link[rel="stylesheet"]['+a+"]"}function fC(a){return g({},a,{"data-precedence":a.precedence,precedence:null})}function o_(a,i,s,f){a.querySelector('link[rel="preload"][as="style"]['+i+"]")?f.loading=1:(i=a.createElement("link"),f.preload=i,i.addEventListener("load",function(){return f.loading|=1}),i.addEventListener("error",function(){return f.loading|=2}),dr(i,"link",s),Rt(i),a.head.appendChild(i))}function Ol(a){return'[src="'+ze(a)+'"]'}function Gs(a){return"script[async]"+a}function dC(a,i,s){if(i.count++,i.instance===null)switch(i.type){case"style":var f=a.querySelector('style[data-href~="'+ze(s.href)+'"]');if(f)return i.instance=f,Rt(f),f;var h=g({},s,{"data-href":s.href,"data-precedence":s.precedence,href:null,precedence:null});return f=(a.ownerDocument||a).createElement("style"),Rt(f),dr(f,"style",h),Zu(f,s.precedence,a),i.instance=f;case"stylesheet":h=$l(s.href);var S=a.querySelector(ks(h));if(S)return i.state.loading|=4,i.instance=S,Rt(S),S;f=fC(s),(h=da.get(h))&&rg(f,h),S=(a.ownerDocument||a).createElement("link"),Rt(S);var A=S;return A._p=new Promise(function(L,Q){A.onload=L,A.onerror=Q}),dr(S,"link",f),i.state.loading|=4,Zu(S,s.precedence,a),i.instance=S;case"script":return S=Ol(s.src),(h=a.querySelector(Gs(S)))?(i.instance=h,Rt(h),h):(f=s,(h=da.get(S))&&(f=g({},s),ag(f,h)),a=a.ownerDocument||a,h=a.createElement("script"),Rt(h),dr(h,"link",f),a.head.appendChild(h),i.instance=h);case"void":return null;default:throw Error(r(443,i.type))}else i.type==="stylesheet"&&(i.state.loading&4)===0&&(f=i.instance,i.state.loading|=4,Zu(f,s.precedence,a));return i.instance}function Zu(a,i,s){for(var f=s.querySelectorAll('link[rel="stylesheet"][data-precedence],style[data-precedence]'),h=f.length?f[f.length-1]:null,S=h,A=0;A<f.length;A++){var L=f[A];if(L.dataset.precedence===i)S=L;else if(S!==h)break}S?S.parentNode.insertBefore(a,S.nextSibling):(i=s.nodeType===9?s.head:s,i.insertBefore(a,i.firstChild))}function rg(a,i){a.crossOrigin==null&&(a.crossOrigin=i.crossOrigin),a.referrerPolicy==null&&(a.referrerPolicy=i.referrerPolicy),a.title==null&&(a.title=i.title)}function ag(a,i){a.crossOrigin==null&&(a.crossOrigin=i.crossOrigin),a.referrerPolicy==null&&(a.referrerPolicy=i.referrerPolicy),a.integrity==null&&(a.integrity=i.integrity)}var Ju=null;function mC(a,i,s){if(Ju===null){var f=new Map,h=Ju=new Map;h.set(s,f)}else h=Ju,f=h.get(s),f||(f=new Map,h.set(s,f));if(f.has(a))return f;for(f.set(a,null),s=s.getElementsByTagName(a),h=0;h<s.length;h++){var S=s[h];if(!(S[Vt]||S[St]||a==="link"&&S.getAttribute("rel")==="stylesheet")&&S.namespaceURI!=="http://www.w3.org/2000/svg"){var A=S.getAttribute(i)||"";A=a+A;var L=f.get(A);L?L.push(S):f.set(A,[S])}}return f}function vC(a,i,s){a=a.ownerDocument||a,a.head.insertBefore(s,i==="title"?a.querySelector("head > title"):null)}function i_(a,i,s){if(s===1||i.itemProp!=null)return!1;switch(a){case"meta":case"title":return!0;case"style":if(typeof i.precedence!="string"||typeof i.href!="string"||i.href==="")break;return!0;case"link":if(typeof i.rel!="string"||typeof i.href!="string"||i.href===""||i.onLoad||i.onError)break;switch(i.rel){case"stylesheet":return a=i.disabled,typeof i.precedence=="string"&&a==null;default:return!0}case"script":if(i.async&&typeof i.async!="function"&&typeof i.async!="symbol"&&!i.onLoad&&!i.onError&&i.src&&typeof i.src=="string")return!0}return!1}function gC(a){return!(a.type==="stylesheet"&&(a.state.loading&3)===0)}var Ks=null;function l_(){}function s_(a,i,s){if(Ks===null)throw Error(r(475));var f=Ks;if(i.type==="stylesheet"&&(typeof s.media!="string"||matchMedia(s.media).matches!==!1)&&(i.state.loading&4)===0){if(i.instance===null){var h=$l(s.href),S=a.querySelector(ks(h));if(S){a=S._p,a!==null&&typeof a=="object"&&typeof a.then=="function"&&(f.count++,f=ef.bind(f),a.then(f,f)),i.state.loading|=4,i.instance=S,Rt(S);return}S=a.ownerDocument||a,s=fC(s),(h=da.get(h))&&rg(s,h),S=S.createElement("link"),Rt(S);var A=S;A._p=new Promise(function(L,Q){A.onload=L,A.onerror=Q}),dr(S,"link",s),i.instance=S}f.stylesheets===null&&(f.stylesheets=new Map),f.stylesheets.set(i,a),(a=i.state.preload)&&(i.state.loading&3)===0&&(f.count++,i=ef.bind(f),a.addEventListener("load",i),a.addEventListener("error",i))}}function c_(){if(Ks===null)throw Error(r(475));var a=Ks;return a.stylesheets&&a.count===0&&og(a,a.stylesheets),0<a.count?function(i){var s=setTimeout(function(){if(a.stylesheets&&og(a,a.stylesheets),a.unsuspend){var f=a.unsuspend;a.unsuspend=null,f()}},6e4);return a.unsuspend=i,function(){a.unsuspend=null,clearTimeout(s)}}:null}function ef(){if(this.count--,this.count===0){if(this.stylesheets)og(this,this.stylesheets);else if(this.unsuspend){var a=this.unsuspend;this.unsuspend=null,a()}}}var tf=null;function og(a,i){a.stylesheets=null,a.unsuspend!==null&&(a.count++,tf=new Map,i.forEach(u_,a),tf=null,ef.call(a))}function u_(a,i){if(!(i.state.loading&4)){var s=tf.get(a);if(s)var f=s.get(null);else{s=new Map,tf.set(a,s);for(var h=a.querySelectorAll("link[data-precedence],style[data-precedence]"),S=0;S<h.length;S++){var A=h[S];(A.nodeName==="LINK"||A.getAttribute("media")!=="not all")&&(s.set(A.dataset.precedence,A),f=A)}f&&s.set(null,f)}h=i.instance,A=h.getAttribute("data-precedence"),S=s.get(A)||f,S===f&&s.set(null,h),s.set(A,h),this.count++,f=ef.bind(this),h.addEventListener("load",f),h.addEventListener("error",f),S?S.parentNode.insertBefore(h,S.nextSibling):(a=a.nodeType===9?a.head:a,a.insertBefore(h,a.firstChild)),i.state.loading|=4}}var Xs={$$typeof:O,Provider:null,Consumer:null,_currentValue:B,_currentValue2:B,_threadCount:0};function f_(a,i,s,f,h,S,A,L){this.tag=1,this.containerInfo=a,this.pingCache=this.current=this.pendingChildren=null,this.timeoutHandle=-1,this.callbackNode=this.next=this.pendingContext=this.context=this.cancelPendingCommit=null,this.callbackPriority=0,this.expirationTimes=bn(-1),this.entangledLanes=this.shellSuspendCounter=this.errorRecoveryDisabledLanes=this.expiredLanes=this.warmLanes=this.pingedLanes=this.suspendedLanes=this.pendingLanes=0,this.entanglements=bn(0),this.hiddenUpdates=bn(null),this.identifierPrefix=f,this.onUncaughtError=h,this.onCaughtError=S,this.onRecoverableError=A,this.pooledCache=null,this.pooledCacheLanes=0,this.formState=L,this.incompleteTransitions=new Map}function hC(a,i,s,f,h,S,A,L,Q,ue,Ce,Oe){return a=new f_(a,i,s,A,L,Q,ue,Oe),i=1,S===!0&&(i|=24),S=Hr(3,null,null,i),a.current=S,S.stateNode=a,i=Hm(),i.refCount++,a.pooledCache=i,i.refCount++,S.memoizedState={element:f,isDehydrated:s,cache:i},qm(S),a}function pC(a){return a?(a=rl,a):rl}function yC(a,i,s,f,h,S){h=pC(h),f.context===null?f.context=h:f.pendingContext=h,f=Ao(i),f.payload={element:s},S=S===void 0?null:S,S!==null&&(f.callback=S),s=Io(a,f,i),s!==null&&(Wr(s,a,i),$s(s,a,i))}function bC(a,i){if(a=a.memoizedState,a!==null&&a.dehydrated!==null){var s=a.retryLane;a.retryLane=s!==0&&s<i?s:i}}function ig(a,i){bC(a,i),(a=a.alternate)&&bC(a,i)}function SC(a){if(a.tag===13){var i=nl(a,67108864);i!==null&&Wr(i,a,67108864),ig(a,67108864)}}var nf=!0;function d_(a,i,s,f){var h=_.T;_.T=null;var S=j.p;try{j.p=2,lg(a,i,s,f)}finally{j.p=S,_.T=h}}function m_(a,i,s,f){var h=_.T;_.T=null;var S=j.p;try{j.p=8,lg(a,i,s,f)}finally{j.p=S,_.T=h}}function lg(a,i,s,f){if(nf){var h=sg(f);if(h===null)Kv(a,i,f,rf,s),xC(a,f);else if(g_(h,a,i,s,f))f.stopPropagation();else if(xC(a,f),i&4&&-1<v_.indexOf(a)){for(;h!==null;){var S=It(h);if(S!==null)switch(S.tag){case 3:if(S=S.stateNode,S.current.memoizedState.isDehydrated){var A=Fe(S.pendingLanes);if(A!==0){var L=S;for(L.pendingLanes|=2,L.entangledLanes|=2;A;){var Q=1<<31-Ue(A);L.entanglements[1]|=Q,A&=~Q}Ua(S),(on&6)===0&&(Hu=$e()+500,Vs(0))}}break;case 13:L=nl(S,2),L!==null&&Wr(L,S,2),Vu(),ig(S,2)}if(S=sg(f),S===null&&Kv(a,i,f,rf,s),S===h)break;h=S}h!==null&&f.stopPropagation()}else Kv(a,i,f,null,s)}}function sg(a){return a=aa(a),cg(a)}var rf=null;function cg(a){if(rf=null,a=At(a),a!==null){var i=l(a);if(i===null)a=null;else{var s=i.tag;if(s===13){if(a=c(i),a!==null)return a;a=null}else if(s===3){if(i.stateNode.current.memoizedState.isDehydrated)return i.tag===3?i.stateNode.containerInfo:null;a=null}else i!==a&&(a=null)}}return rf=a,null}function CC(a){switch(a){case"beforetoggle":case"cancel":case"click":case"close":case"contextmenu":case"copy":case"cut":case"auxclick":case"dblclick":case"dragend":case"dragstart":case"drop":case"focusin":case"focusout":case"input":case"invalid":case"keydown":case"keypress":case"keyup":case"mousedown":case"mouseup":case"paste":case"pause":case"play":case"pointercancel":case"pointerdown":case"pointerup":case"ratechange":case"reset":case"resize":case"seeked":case"submit":case"toggle":case"touchcancel":case"touchend":case"touchstart":case"volumechange":case"change":case"selectionchange":case"textInput":case"compositionstart":case"compositionend":case"compositionupdate":case"beforeblur":case"afterblur":case"beforeinput":case"blur":case"fullscreenchange":case"focus":case"hashchange":case"popstate":case"select":case"selectstart":return 2;case"drag":case"dragenter":case"dragexit":case"dragleave":case"dragover":case"mousemove":case"mouseout":case"mouseover":case"pointermove":case"pointerout":case"pointerover":case"scroll":case"touchmove":case"wheel":case"mouseenter":case"mouseleave":case"pointerenter":case"pointerleave":return 8;case"message":switch(we()){case Ne:return 2;case Ze:return 8;case He:case Xe:return 32;case xe:return 268435456;default:return 32}default:return 32}}var ug=!1,ko=null,Go=null,Ko=null,Ys=new Map,Qs=new Map,Xo=[],v_="mousedown mouseup touchcancel touchend touchstart auxclick dblclick pointercancel pointerdown pointerup dragend dragstart drop compositionend compositionstart keydown keypress keyup input textInput copy cut paste click change contextmenu reset".split(" ");function xC(a,i){switch(a){case"focusin":case"focusout":ko=null;break;case"dragenter":case"dragleave":Go=null;break;case"mouseover":case"mouseout":Ko=null;break;case"pointerover":case"pointerout":Ys.delete(i.pointerId);break;case"gotpointercapture":case"lostpointercapture":Qs.delete(i.pointerId)}}function Zs(a,i,s,f,h,S){return a===null||a.nativeEvent!==S?(a={blockedOn:i,domEventName:s,eventSystemFlags:f,nativeEvent:S,targetContainers:[h]},i!==null&&(i=It(i),i!==null&&SC(i)),a):(a.eventSystemFlags|=f,i=a.targetContainers,h!==null&&i.indexOf(h)===-1&&i.push(h),a)}function g_(a,i,s,f,h){switch(i){case"focusin":return ko=Zs(ko,a,i,s,f,h),!0;case"dragenter":return Go=Zs(Go,a,i,s,f,h),!0;case"mouseover":return Ko=Zs(Ko,a,i,s,f,h),!0;case"pointerover":var S=h.pointerId;return Ys.set(S,Zs(Ys.get(S)||null,a,i,s,f,h)),!0;case"gotpointercapture":return S=h.pointerId,Qs.set(S,Zs(Qs.get(S)||null,a,i,s,f,h)),!0}return!1}function EC(a){var i=At(a.target);if(i!==null){var s=l(i);if(s!==null){if(i=s.tag,i===13){if(i=c(s),i!==null){a.blockedOn=i,De(a.priority,function(){if(s.tag===13){var f=qr();f=Et(f);var h=nl(s,f);h!==null&&Wr(h,s,f),ig(s,f)}});return}}else if(i===3&&s.stateNode.current.memoizedState.isDehydrated){a.blockedOn=s.tag===3?s.stateNode.containerInfo:null;return}}}a.blockedOn=null}function af(a){if(a.blockedOn!==null)return!1;for(var i=a.targetContainers;0<i.length;){var s=sg(a.nativeEvent);if(s===null){s=a.nativeEvent;var f=new s.constructor(s.type,s);Ba=f,s.target.dispatchEvent(f),Ba=null}else return i=It(s),i!==null&&SC(i),a.blockedOn=s,!1;i.shift()}return!0}function wC(a,i,s){af(a)&&s.delete(i)}function h_(){ug=!1,ko!==null&&af(ko)&&(ko=null),Go!==null&&af(Go)&&(Go=null),Ko!==null&&af(Ko)&&(Ko=null),Ys.forEach(wC),Qs.forEach(wC)}function of(a,i){a.blockedOn===i&&(a.blockedOn=null,ug||(ug=!0,e.unstable_scheduleCallback(e.unstable_NormalPriority,h_)))}var lf=null;function $C(a){lf!==a&&(lf=a,e.unstable_scheduleCallback(e.unstable_NormalPriority,function(){lf===a&&(lf=null);for(var i=0;i<a.length;i+=3){var s=a[i],f=a[i+1],h=a[i+2];if(typeof f!="function"){if(cg(f||s)===null)continue;break}var S=It(s);S!==null&&(a.splice(i,3),i-=3,cv(S,{pending:!0,data:h,method:s.method,action:f},f,h))}}))}function Js(a){function i(Q){return of(Q,a)}ko!==null&&of(ko,a),Go!==null&&of(Go,a),Ko!==null&&of(Ko,a),Ys.forEach(i),Qs.forEach(i);for(var s=0;s<Xo.length;s++){var f=Xo[s];f.blockedOn===a&&(f.blockedOn=null)}for(;0<Xo.length&&(s=Xo[0],s.blockedOn===null);)EC(s),s.blockedOn===null&&Xo.shift();if(s=(a.ownerDocument||a).$$reactFormReplay,s!=null)for(f=0;f<s.length;f+=3){var h=s[f],S=s[f+1],A=h[Be]||null;if(typeof S=="function")A||$C(s);else if(A){var L=null;if(S&&S.hasAttribute("formAction")){if(h=S,A=S[Be]||null)L=A.formAction;else if(cg(h)!==null)continue}else L=A.action;typeof L=="function"?s[f+1]=L:(s.splice(f,3),f-=3),$C(s)}}}function fg(a){this._internalRoot=a}sf.prototype.render=fg.prototype.render=function(a){var i=this._internalRoot;if(i===null)throw Error(r(409));var s=i.current,f=qr();yC(s,f,a,i,null,null)},sf.prototype.unmount=fg.prototype.unmount=function(){var a=this._internalRoot;if(a!==null){this._internalRoot=null;var i=a.containerInfo;yC(a.current,2,null,a,null,null),Vu(),i[je]=null}};function sf(a){this._internalRoot=a}sf.prototype.unstable_scheduleHydration=function(a){if(a){var i=ke();a={blockedOn:null,target:a,priority:i};for(var s=0;s<Xo.length&&i!==0&&i<Xo[s].priority;s++);Xo.splice(s,0,a),s===0&&EC(a)}};var OC=t.version;if(OC!=="19.1.0")throw Error(r(527,OC,"19.1.0"));j.findDOMNode=function(a){var i=a._reactInternals;if(i===void 0)throw typeof a.render=="function"?Error(r(188)):(a=Object.keys(a).join(","),Error(r(268,a)));return a=m(i),a=a!==null?v(a):null,a=a===null?null:a.stateNode,a};var p_={bundleType:0,version:"19.1.0",rendererPackageName:"react-dom",currentDispatcherRef:_,reconcilerVersion:"19.1.0"};if(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__<"u"){var cf=__REACT_DEVTOOLS_GLOBAL_HOOK__;if(!cf.isDisabled&&cf.supportsFiber)try{Ke=cf.inject(p_),We=cf}catch{}}return tc.createRoot=function(a,i){if(!o(a))throw Error(r(299));var s=!1,f="",h=Vb,S=Ub,A=qb,L=null;return i!=null&&(i.unstable_strictMode===!0&&(s=!0),i.identifierPrefix!==void 0&&(f=i.identifierPrefix),i.onUncaughtError!==void 0&&(h=i.onUncaughtError),i.onCaughtError!==void 0&&(S=i.onCaughtError),i.onRecoverableError!==void 0&&(A=i.onRecoverableError),i.unstable_transitionCallbacks!==void 0&&(L=i.unstable_transitionCallbacks)),i=hC(a,1,!1,null,null,s,f,h,S,A,L,null),a[je]=i.current,Gv(a),new fg(i)},tc.hydrateRoot=function(a,i,s){if(!o(a))throw Error(r(299));var f=!1,h="",S=Vb,A=Ub,L=qb,Q=null,ue=null;return s!=null&&(s.unstable_strictMode===!0&&(f=!0),s.identifierPrefix!==void 0&&(h=s.identifierPrefix),s.onUncaughtError!==void 0&&(S=s.onUncaughtError),s.onCaughtError!==void 0&&(A=s.onCaughtError),s.onRecoverableError!==void 0&&(L=s.onRecoverableError),s.unstable_transitionCallbacks!==void 0&&(Q=s.unstable_transitionCallbacks),s.formState!==void 0&&(ue=s.formState)),i=hC(a,1,!0,i,s??null,f,h,S,A,L,Q,ue),i.context=pC(null),s=i.current,f=qr(),f=Et(f),h=Ao(f),h.callback=null,Io(s,h,f),s=f,i.current.lanes=s,$n(i,s),Ua(i),a[je]=i.current,Gv(a),new sf(i)},tc.version="19.1.0",tc}var zC;function O_(){if(zC)return mg.exports;zC=1;function e(){if(!(typeof __REACT_DEVTOOLS_GLOBAL_HOOK__>"u"||typeof __REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE!="function"))try{__REACT_DEVTOOLS_GLOBAL_HOOK__.checkDCE(e)}catch(t){console.error(t)}}return e(),mg.exports=$_(),mg.exports}var R_=O_();const T_="modulepreload",M_=function(e,t){return new URL(e,t).href},DC={},xn=function(t,n,r){let o=Promise.resolve();if(n&&n.length>0){let c=function(g){return Promise.all(g.map(p=>Promise.resolve(p).then(y=>({status:"fulfilled",value:y}),y=>({status:"rejected",reason:y}))))};const d=document.getElementsByTagName("link"),m=document.querySelector("meta[property=csp-nonce]"),v=(m==null?void 0:m.nonce)||(m==null?void 0:m.getAttribute("nonce"));o=c(n.map(g=>{if(g=M_(g,r),g in DC)return;DC[g]=!0;const p=g.endsWith(".css"),y=p?'[rel="stylesheet"]':"";if(!!r)for(let x=d.length-1;x>=0;x--){const b=d[x];if(b.href===g&&(!p||b.rel==="stylesheet"))return}else if(document.querySelector(`link[href="${g}"]${y}`))return;const C=document.createElement("link");if(C.rel=p?"stylesheet":T_,p||(C.as="script"),C.crossOrigin="",C.href=g,v&&C.setAttribute("nonce",v),document.head.appendChild(C),p)return new Promise((x,b)=>{C.addEventListener("load",x),C.addEventListener("error",()=>b(new Error(`Unable to preload CSS for ${g}`)))})}))}function l(c){const d=new Event("vite:preloadError",{cancelable:!0});if(d.payload=c,window.dispatchEvent(d),!d.defaultPrevented)throw c}return o.then(c=>{for(const d of c||[])d.status==="rejected"&&l(d.reason);return t().catch(l)})};var u=Cd();const oe=Sd(u),xd=iw({__proto__:null,default:oe},[u]);var Vp=u.createContext({});function Ie(){return Ie=Object.assign?Object.assign.bind():function(e){for(var t=1;t<arguments.length;t++){var n=arguments[t];for(var r in n)({}).hasOwnProperty.call(n,r)&&(e[r]=n[r])}return e},Ie.apply(null,arguments)}function sw(e){if(Array.isArray(e))return e}function __(e,t){var n=e==null?null:typeof Symbol<"u"&&e[Symbol.iterator]||e["@@iterator"];if(n!=null){var r,o,l,c,d=[],m=!0,v=!1;try{if(l=(n=n.call(e)).next,t===0){if(Object(n)!==n)return;m=!1}else for(;!(m=(r=l.call(n)).done)&&(d.push(r.value),d.length!==t);m=!0);}catch(g){v=!0,o=g}finally{try{if(!m&&n.return!=null&&(c=n.return(),Object(c)!==c))return}finally{if(v)throw o}}return d}}function Oh(e,t){(t==null||t>e.length)&&(t=e.length);for(var n=0,r=Array(t);n<t;n++)r[n]=e[n];return r}function Up(e,t){if(e){if(typeof e=="string")return Oh(e,t);var n={}.toString.call(e).slice(8,-1);return n==="Object"&&e.constructor&&(n=e.constructor.name),n==="Map"||n==="Set"?Array.from(e):n==="Arguments"||/^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)?Oh(e,t):void 0}}function cw(){throw new TypeError(`Invalid attempt to destructure non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function de(e,t){return sw(e)||__(e,t)||Up(e,t)||cw()}function lt(e){"@babel/helpers - typeof";return lt=typeof Symbol=="function"&&typeof Symbol.iterator=="symbol"?function(t){return typeof t}:function(t){return t&&typeof Symbol=="function"&&t.constructor===Symbol&&t!==Symbol.prototype?"symbol":typeof t},lt(e)}function N_(e,t){if(lt(e)!="object"||!e)return e;var n=e[Symbol.toPrimitive];if(n!==void 0){var r=n.call(e,t);if(lt(r)!="object")return r;throw new TypeError("@@toPrimitive must return a primitive value.")}return(t==="string"?String:Number)(e)}function uw(e){var t=N_(e,"string");return lt(t)=="symbol"?t:t+""}function Y(e,t,n){return(t=uw(t))in e?Object.defineProperty(e,t,{value:n,enumerable:!0,configurable:!0,writable:!0}):e[t]=n,e}function A_(e,t){if(e==null)return{};var n={};for(var r in e)if({}.hasOwnProperty.call(e,r)){if(t.indexOf(r)!==-1)continue;n[r]=e[r]}return n}function wt(e,t){if(e==null)return{};var n,r,o=A_(e,t);if(Object.getOwnPropertySymbols){var l=Object.getOwnPropertySymbols(e);for(r=0;r<l.length;r++)n=l[r],t.indexOf(n)===-1&&{}.propertyIsEnumerable.call(e,n)&&(o[n]=e[n])}return o}var yg={exports:{}};/*!
	Copyright (c) 2018 Jed Watson.
	Licensed under the MIT License (MIT), see
	http://jedwatson.github.io/classnames
*/var BC;function I_(){return BC||(BC=1,function(e){(function(){var t={}.hasOwnProperty;function n(){for(var l="",c=0;c<arguments.length;c++){var d=arguments[c];d&&(l=o(l,r(d)))}return l}function r(l){if(typeof l=="string"||typeof l=="number")return l;if(typeof l!="object")return"";if(Array.isArray(l))return n.apply(null,l);if(l.toString!==Object.prototype.toString&&!l.toString.toString().includes("[native code]"))return l.toString();var c="";for(var d in l)t.call(l,d)&&l[d]&&(c=o(c,d));return c}function o(l,c){return c?l?l+" "+c:l+c:l}e.exports?(n.default=n,e.exports=n):window.classNames=n})()}(yg)),yg.exports}var j_=I_();const re=Sd(j_),sr=Math.round;function bg(e,t){const n=e.replace(/^[^(]*\((.*)/,"$1").replace(/\).*/,"").match(/\d*\.?\d+%?/g)||[],r=n.map(o=>parseFloat(o));for(let o=0;o<3;o+=1)r[o]=t(r[o]||0,n[o]||"",o);return n[3]?r[3]=n[3].includes("%")?r[3]/100:r[3]:r[3]=1,r}const LC=(e,t,n)=>n===0?e:e/100;function nc(e,t){const n=t||255;return e>n?n:e<0?0:e}class Jt{constructor(t){Y(this,"isValid",!0),Y(this,"r",0),Y(this,"g",0),Y(this,"b",0),Y(this,"a",1),Y(this,"_h",void 0),Y(this,"_s",void 0),Y(this,"_l",void 0),Y(this,"_v",void 0),Y(this,"_max",void 0),Y(this,"_min",void 0),Y(this,"_brightness",void 0);function n(r){return r[0]in t&&r[1]in t&&r[2]in t}if(t)if(typeof t=="string"){let o=function(l){return r.startsWith(l)};const r=t.trim();/^#?[A-F\d]{3,8}$/i.test(r)?this.fromHexString(r):o("rgb")?this.fromRgbString(r):o("hsl")?this.fromHslString(r):(o("hsv")||o("hsb"))&&this.fromHsvString(r)}else if(t instanceof Jt)this.r=t.r,this.g=t.g,this.b=t.b,this.a=t.a,this._h=t._h,this._s=t._s,this._l=t._l,this._v=t._v;else if(n("rgb"))this.r=nc(t.r),this.g=nc(t.g),this.b=nc(t.b),this.a=typeof t.a=="number"?nc(t.a,1):1;else if(n("hsl"))this.fromHsl(t);else if(n("hsv"))this.fromHsv(t);else throw new Error("@ant-design/fast-color: unsupported input "+JSON.stringify(t))}setR(t){return this._sc("r",t)}setG(t){return this._sc("g",t)}setB(t){return this._sc("b",t)}setA(t){return this._sc("a",t,1)}setHue(t){const n=this.toHsv();return n.h=t,this._c(n)}getLuminance(){function t(l){const c=l/255;return c<=.03928?c/12.92:Math.pow((c+.055)/1.055,2.4)}const n=t(this.r),r=t(this.g),o=t(this.b);return .2126*n+.7152*r+.0722*o}getHue(){if(typeof this._h>"u"){const t=this.getMax()-this.getMin();t===0?this._h=0:this._h=sr(60*(this.r===this.getMax()?(this.g-this.b)/t+(this.g<this.b?6:0):this.g===this.getMax()?(this.b-this.r)/t+2:(this.r-this.g)/t+4))}return this._h}getSaturation(){if(typeof this._s>"u"){const t=this.getMax()-this.getMin();t===0?this._s=0:this._s=t/this.getMax()}return this._s}getLightness(){return typeof this._l>"u"&&(this._l=(this.getMax()+this.getMin())/510),this._l}getValue(){return typeof this._v>"u"&&(this._v=this.getMax()/255),this._v}getBrightness(){return typeof this._brightness>"u"&&(this._brightness=(this.r*299+this.g*587+this.b*114)/1e3),this._brightness}darken(t=10){const n=this.getHue(),r=this.getSaturation();let o=this.getLightness()-t/100;return o<0&&(o=0),this._c({h:n,s:r,l:o,a:this.a})}lighten(t=10){const n=this.getHue(),r=this.getSaturation();let o=this.getLightness()+t/100;return o>1&&(o=1),this._c({h:n,s:r,l:o,a:this.a})}mix(t,n=50){const r=this._c(t),o=n/100,l=d=>(r[d]-this[d])*o+this[d],c={r:sr(l("r")),g:sr(l("g")),b:sr(l("b")),a:sr(l("a")*100)/100};return this._c(c)}tint(t=10){return this.mix({r:255,g:255,b:255,a:1},t)}shade(t=10){return this.mix({r:0,g:0,b:0,a:1},t)}onBackground(t){const n=this._c(t),r=this.a+n.a*(1-this.a),o=l=>sr((this[l]*this.a+n[l]*n.a*(1-this.a))/r);return this._c({r:o("r"),g:o("g"),b:o("b"),a:r})}isDark(){return this.getBrightness()<128}isLight(){return this.getBrightness()>=128}equals(t){return this.r===t.r&&this.g===t.g&&this.b===t.b&&this.a===t.a}clone(){return this._c(this)}toHexString(){let t="#";const n=(this.r||0).toString(16);t+=n.length===2?n:"0"+n;const r=(this.g||0).toString(16);t+=r.length===2?r:"0"+r;const o=(this.b||0).toString(16);if(t+=o.length===2?o:"0"+o,typeof this.a=="number"&&this.a>=0&&this.a<1){const l=sr(this.a*255).toString(16);t+=l.length===2?l:"0"+l}return t}toHsl(){return{h:this.getHue(),s:this.getSaturation(),l:this.getLightness(),a:this.a}}toHslString(){const t=this.getHue(),n=sr(this.getSaturation()*100),r=sr(this.getLightness()*100);return this.a!==1?`hsla(${t},${n}%,${r}%,${this.a})`:`hsl(${t},${n}%,${r}%)`}toHsv(){return{h:this.getHue(),s:this.getSaturation(),v:this.getValue(),a:this.a}}toRgb(){return{r:this.r,g:this.g,b:this.b,a:this.a}}toRgbString(){return this.a!==1?`rgba(${this.r},${this.g},${this.b},${this.a})`:`rgb(${this.r},${this.g},${this.b})`}toString(){return this.toRgbString()}_sc(t,n,r){const o=this.clone();return o[t]=nc(n,r),o}_c(t){return new this.constructor(t)}getMax(){return typeof this._max>"u"&&(this._max=Math.max(this.r,this.g,this.b)),this._max}getMin(){return typeof this._min>"u"&&(this._min=Math.min(this.r,this.g,this.b)),this._min}fromHexString(t){const n=t.replace("#","");function r(o,l){return parseInt(n[o]+n[l||o],16)}n.length<6?(this.r=r(0),this.g=r(1),this.b=r(2),this.a=n[3]?r(3)/255:1):(this.r=r(0,1),this.g=r(2,3),this.b=r(4,5),this.a=n[6]?r(6,7)/255:1)}fromHsl({h:t,s:n,l:r,a:o}){if(this._h=t%360,this._s=n,this._l=r,this.a=typeof o=="number"?o:1,n<=0){const y=sr(r*255);this.r=y,this.g=y,this.b=y}let l=0,c=0,d=0;const m=t/60,v=(1-Math.abs(2*r-1))*n,g=v*(1-Math.abs(m%2-1));m>=0&&m<1?(l=v,c=g):m>=1&&m<2?(l=g,c=v):m>=2&&m<3?(c=v,d=g):m>=3&&m<4?(c=g,d=v):m>=4&&m<5?(l=g,d=v):m>=5&&m<6&&(l=v,d=g);const p=r-v/2;this.r=sr((l+p)*255),this.g=sr((c+p)*255),this.b=sr((d+p)*255)}fromHsv({h:t,s:n,v:r,a:o}){this._h=t%360,this._s=n,this._v=r,this.a=typeof o=="number"?o:1;const l=sr(r*255);if(this.r=l,this.g=l,this.b=l,n<=0)return;const c=t/60,d=Math.floor(c),m=c-d,v=sr(r*(1-n)*255),g=sr(r*(1-n*m)*255),p=sr(r*(1-n*(1-m))*255);switch(d){case 0:this.g=p,this.b=v;break;case 1:this.r=g,this.b=v;break;case 2:this.r=v,this.b=p;break;case 3:this.r=v,this.g=g;break;case 4:this.r=p,this.g=v;break;case 5:default:this.g=v,this.b=g;break}}fromHsvString(t){const n=bg(t,LC);this.fromHsv({h:n[0],s:n[1],v:n[2],a:n[3]})}fromHslString(t){const n=bg(t,LC);this.fromHsl({h:n[0],s:n[1],l:n[2],a:n[3]})}fromRgbString(t){const n=bg(t,(r,o)=>o.includes("%")?sr(r/100*255):r);this.r=n[0],this.g=n[1],this.b=n[2],this.a=n[3]}}var uf=2,HC=.16,P_=.05,z_=.05,D_=.15,fw=5,dw=4,B_=[{index:7,amount:15},{index:6,amount:25},{index:5,amount:30},{index:5,amount:45},{index:5,amount:65},{index:5,amount:85},{index:4,amount:90},{index:3,amount:95},{index:2,amount:97},{index:1,amount:98}];function FC(e,t,n){var r;return Math.round(e.h)>=60&&Math.round(e.h)<=240?r=n?Math.round(e.h)-uf*t:Math.round(e.h)+uf*t:r=n?Math.round(e.h)+uf*t:Math.round(e.h)-uf*t,r<0?r+=360:r>=360&&(r-=360),r}function VC(e,t,n){if(e.h===0&&e.s===0)return e.s;var r;return n?r=e.s-HC*t:t===dw?r=e.s+HC:r=e.s+P_*t,r>1&&(r=1),n&&t===fw&&r>.1&&(r=.1),r<.06&&(r=.06),Math.round(r*100)/100}function UC(e,t,n){var r;return n?r=e.v+z_*t:r=e.v-D_*t,r=Math.max(0,Math.min(1,r)),Math.round(r*100)/100}function Bi(e){for(var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=[],r=new Jt(e),o=r.toHsv(),l=fw;l>0;l-=1){var c=new Jt({h:FC(o,l,!0),s:VC(o,l,!0),v:UC(o,l,!0)});n.push(c)}n.push(r);for(var d=1;d<=dw;d+=1){var m=new Jt({h:FC(o,d),s:VC(o,d),v:UC(o,d)});n.push(m)}return t.theme==="dark"?B_.map(function(v){var g=v.index,p=v.amount;return new Jt(t.backgroundColor||"#141414").mix(n[g],p).toHexString()}):n.map(function(v){return v.toHexString()})}var Sg={red:"#F5222D",volcano:"#FA541C",orange:"#FA8C16",gold:"#FAAD14",yellow:"#FADB14",lime:"#A0D911",green:"#52C41A",cyan:"#13C2C2",blue:"#1677FF",geekblue:"#2F54EB",purple:"#722ED1",magenta:"#EB2F96",grey:"#666666"},Rh=["#fff1f0","#ffccc7","#ffa39e","#ff7875","#ff4d4f","#f5222d","#cf1322","#a8071a","#820014","#5c0011"];Rh.primary=Rh[5];var Th=["#fff2e8","#ffd8bf","#ffbb96","#ff9c6e","#ff7a45","#fa541c","#d4380d","#ad2102","#871400","#610b00"];Th.primary=Th[5];var Mh=["#fff7e6","#ffe7ba","#ffd591","#ffc069","#ffa940","#fa8c16","#d46b08","#ad4e00","#873800","#612500"];Mh.primary=Mh[5];var Wf=["#fffbe6","#fff1b8","#ffe58f","#ffd666","#ffc53d","#faad14","#d48806","#ad6800","#874d00","#613400"];Wf.primary=Wf[5];var _h=["#feffe6","#ffffb8","#fffb8f","#fff566","#ffec3d","#fadb14","#d4b106","#ad8b00","#876800","#614700"];_h.primary=_h[5];var Nh=["#fcffe6","#f4ffb8","#eaff8f","#d3f261","#bae637","#a0d911","#7cb305","#5b8c00","#3f6600","#254000"];Nh.primary=Nh[5];var Ah=["#f6ffed","#d9f7be","#b7eb8f","#95de64","#73d13d","#52c41a","#389e0d","#237804","#135200","#092b00"];Ah.primary=Ah[5];var Ih=["#e6fffb","#b5f5ec","#87e8de","#5cdbd3","#36cfc9","#13c2c2","#08979c","#006d75","#00474f","#002329"];Ih.primary=Ih[5];var kf=["#e6f4ff","#bae0ff","#91caff","#69b1ff","#4096ff","#1677ff","#0958d9","#003eb3","#002c8c","#001d66"];kf.primary=kf[5];var jh=["#f0f5ff","#d6e4ff","#adc6ff","#85a5ff","#597ef7","#2f54eb","#1d39c4","#10239e","#061178","#030852"];jh.primary=jh[5];var Ph=["#f9f0ff","#efdbff","#d3adf7","#b37feb","#9254de","#722ed1","#531dab","#391085","#22075e","#120338"];Ph.primary=Ph[5];var zh=["#fff0f6","#ffd6e7","#ffadd2","#ff85c0","#f759ab","#eb2f96","#c41d7f","#9e1068","#780650","#520339"];zh.primary=zh[5];var Dh=["#a6a6a6","#999999","#8c8c8c","#808080","#737373","#666666","#404040","#1a1a1a","#000000","#000000"];Dh.primary=Dh[5];var Cg={red:Rh,volcano:Th,orange:Mh,gold:Wf,yellow:_h,lime:Nh,green:Ah,cyan:Ih,blue:kf,geekblue:jh,purple:Ph,magenta:zh,grey:Dh};function qC(e,t){var n=Object.keys(e);if(Object.getOwnPropertySymbols){var r=Object.getOwnPropertySymbols(e);t&&(r=r.filter(function(o){return Object.getOwnPropertyDescriptor(e,o).enumerable})),n.push.apply(n,r)}return n}function Z(e){for(var t=1;t<arguments.length;t++){var n=arguments[t]!=null?arguments[t]:{};t%2?qC(Object(n),!0).forEach(function(r){Y(e,r,n[r])}):Object.getOwnPropertyDescriptors?Object.defineProperties(e,Object.getOwnPropertyDescriptors(n)):qC(Object(n)).forEach(function(r){Object.defineProperty(e,r,Object.getOwnPropertyDescriptor(n,r))})}return e}function Cr(){return!!(typeof window<"u"&&window.document&&window.document.createElement)}function Bh(e,t){if(!e)return!1;if(e.contains)return e.contains(t);for(var n=t;n;){if(n===e)return!0;n=n.parentNode}return!1}var WC="data-rc-order",kC="data-rc-priority",L_="rc-util-key",Lh=new Map;function mw(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},t=e.mark;return t?t.startsWith("data-")?t:"data-".concat(t):L_}function Ed(e){if(e.attachTo)return e.attachTo;var t=document.querySelector("head");return t||document.body}function H_(e){return e==="queue"?"prependQueue":e?"prepend":"append"}function qp(e){return Array.from((Lh.get(e)||e).children).filter(function(t){return t.tagName==="STYLE"})}function vw(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};if(!Cr())return null;var n=t.csp,r=t.prepend,o=t.priority,l=o===void 0?0:o,c=H_(r),d=c==="prependQueue",m=document.createElement("style");m.setAttribute(WC,c),d&&l&&m.setAttribute(kC,"".concat(l)),n!=null&&n.nonce&&(m.nonce=n==null?void 0:n.nonce),m.innerHTML=e;var v=Ed(t),g=v.firstChild;if(r){if(d){var p=(t.styles||qp(v)).filter(function(y){if(!["prepend","prependQueue"].includes(y.getAttribute(WC)))return!1;var E=Number(y.getAttribute(kC)||0);return l>=E});if(p.length)return v.insertBefore(m,p[p.length-1].nextSibling),m}v.insertBefore(m,g)}else v.appendChild(m);return m}function gw(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=Ed(t);return(t.styles||qp(n)).find(function(r){return r.getAttribute(mw(t))===e})}function Cc(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=gw(e,t);if(n){var r=Ed(t);r.removeChild(n)}}function F_(e,t){var n=Lh.get(e);if(!n||!Bh(document,n)){var r=vw("",t),o=r.parentNode;Lh.set(e,o),e.removeChild(r)}}function po(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{},r=Ed(n),o=qp(r),l=Z(Z({},n),{},{styles:o});F_(r,l);var c=gw(t,l);if(c){var d,m;if((d=l.csp)!==null&&d!==void 0&&d.nonce&&c.nonce!==((m=l.csp)===null||m===void 0?void 0:m.nonce)){var v;c.nonce=(v=l.csp)===null||v===void 0?void 0:v.nonce}return c.innerHTML!==e&&(c.innerHTML=e),c}var g=vw(e,l);return g.setAttribute(mw(l),t),g}function hw(e){var t;return e==null||(t=e.getRootNode)===null||t===void 0?void 0:t.call(e)}function V_(e){return hw(e)instanceof ShadowRoot}function Gf(e){return V_(e)?hw(e):null}var Hh={},U_=function(t){};function q_(e,t){}function W_(e,t){}function k_(){Hh={}}function pw(e,t,n){!t&&!Hh[n]&&(e(!1,n),Hh[n]=!0)}function vr(e,t){pw(q_,e,t)}function G_(e,t){pw(W_,e,t)}vr.preMessage=U_;vr.resetWarned=k_;vr.noteOnce=G_;function K_(e){return e.replace(/-(.)/g,function(t,n){return n.toUpperCase()})}function X_(e,t){vr(e,"[@ant-design/icons] ".concat(t))}function GC(e){return lt(e)==="object"&&typeof e.name=="string"&&typeof e.theme=="string"&&(lt(e.icon)==="object"||typeof e.icon=="function")}function KC(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};return Object.keys(e).reduce(function(t,n){var r=e[n];switch(n){case"class":t.className=r,delete t.class;break;default:delete t[n],t[K_(n)]=r}return t},{})}function Fh(e,t,n){return n?oe.createElement(e.tag,Z(Z({key:t},KC(e.attrs)),n),(e.children||[]).map(function(r,o){return Fh(r,"".concat(t,"-").concat(e.tag,"-").concat(o))})):oe.createElement(e.tag,Z({key:t},KC(e.attrs)),(e.children||[]).map(function(r,o){return Fh(r,"".concat(t,"-").concat(e.tag,"-").concat(o))}))}function yw(e){return Bi(e)[0]}function bw(e){return e?Array.isArray(e)?e:[e]:[]}var Y_=`
.anticon {
  display: inline-flex;
  align-items: center;
  color: inherit;
  font-style: normal;
  line-height: 0;
  text-align: center;
  text-transform: none;
  vertical-align: -0.125em;
  text-rendering: optimizeLegibility;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.anticon > * {
  line-height: 1;
}

.anticon svg {
  display: inline-block;
}

.anticon::before {
  display: none;
}

.anticon .anticon-icon {
  display: block;
}

.anticon[tabindex] {
  cursor: pointer;
}

.anticon-spin::before,
.anticon-spin {
  display: inline-block;
  -webkit-animation: loadingCircle 1s infinite linear;
  animation: loadingCircle 1s infinite linear;
}

@-webkit-keyframes loadingCircle {
  100% {
    -webkit-transform: rotate(360deg);
    transform: rotate(360deg);
  }
}

@keyframes loadingCircle {
  100% {
    -webkit-transform: rotate(360deg);
    transform: rotate(360deg);
  }
}
`,Q_=function(t){var n=u.useContext(Vp),r=n.csp,o=n.prefixCls,l=n.layer,c=Y_;o&&(c=c.replace(/anticon/g,o)),l&&(c="@layer ".concat(l,` {
`).concat(c,`
}`)),u.useEffect(function(){var d=t.current,m=Gf(d);po(c,"@ant-design-icons",{prepend:!l,csp:r,attachTo:m})},[])},Z_=["icon","className","onClick","style","primaryColor","secondaryColor"],hc={primaryColor:"#333",secondaryColor:"#E6E6E6",calculated:!1};function J_(e){var t=e.primaryColor,n=e.secondaryColor;hc.primaryColor=t,hc.secondaryColor=n||yw(t),hc.calculated=!!n}function eN(){return Z({},hc)}var Kl=function(t){var n=t.icon,r=t.className,o=t.onClick,l=t.style,c=t.primaryColor,d=t.secondaryColor,m=wt(t,Z_),v=u.useRef(),g=hc;if(c&&(g={primaryColor:c,secondaryColor:d||yw(c)}),Q_(v),X_(GC(n),"icon should be icon definiton, but got ".concat(n)),!GC(n))return null;var p=n;return p&&typeof p.icon=="function"&&(p=Z(Z({},p),{},{icon:p.icon(g.primaryColor,g.secondaryColor)})),Fh(p.icon,"svg-".concat(p.name),Z(Z({className:r,onClick:o,style:l,"data-icon":p.name,width:"1em",height:"1em",fill:"currentColor","aria-hidden":"true"},m),{},{ref:v}))};Kl.displayName="IconReact";Kl.getTwoToneColors=eN;Kl.setTwoToneColors=J_;function Sw(e){var t=bw(e),n=de(t,2),r=n[0],o=n[1];return Kl.setTwoToneColors({primaryColor:r,secondaryColor:o})}function tN(){var e=Kl.getTwoToneColors();return e.calculated?[e.primaryColor,e.secondaryColor]:e.primaryColor}var nN=["className","icon","spin","rotate","tabIndex","onClick","twoToneColor"];Sw(kf.primary);var En=u.forwardRef(function(e,t){var n=e.className,r=e.icon,o=e.spin,l=e.rotate,c=e.tabIndex,d=e.onClick,m=e.twoToneColor,v=wt(e,nN),g=u.useContext(Vp),p=g.prefixCls,y=p===void 0?"anticon":p,E=g.rootClassName,C=re(E,y,Y(Y({},"".concat(y,"-").concat(r.name),!!r.name),"".concat(y,"-spin"),!!o||r.name==="loading"),n),x=c;x===void 0&&d&&(x=-1);var b=l?{msTransform:"rotate(".concat(l,"deg)"),transform:"rotate(".concat(l,"deg)")}:void 0,w=bw(m),$=de(w,2),O=$[0],T=$[1];return u.createElement("span",Ie({role:"img","aria-label":r.name},v,{ref:t,tabIndex:x,onClick:d,className:C}),u.createElement(Kl,{icon:r,primaryColor:O,secondaryColor:T,style:b}))});En.displayName="AntdIcon";En.getTwoToneColor=tN;En.setTwoToneColor=Sw;var rN={icon:{tag:"svg",attrs:{viewBox:"0 0 1024 1024",focusable:"false"},children:[{tag:"path",attrs:{d:"M912 192H328c-4.4 0-8 3.6-8 8v56c0 4.4 3.6 8 8 8h584c4.4 0 8-3.6 8-8v-56c0-4.4-3.6-8-8-8zm0 284H328c-4.4 0-8 3.6-8 8v56c0 4.4 3.6 8 8 8h584c4.4 0 8-3.6 8-8v-56c0-4.4-3.6-8-8-8zm0 284H328c-4.4 0-8 3.6-8 8v56c0 4.4 3.6 8 8 8h584c4.4 0 8-3.6 8-8v-56c0-4.4-3.6-8-8-8zM104 228a56 56 0 10112 0 56 56 0 10-112 0zm0 284a56 56 0 10112 0 56 56 0 10-112 0zm0 284a56 56 0 10112 0 56 56 0 10-112 0z"}}]},name:"bars",theme:"outlined"},aN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:rN}))},oN=u.forwardRef(aN),iN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M512 64C264.6 64 64 264.6 64 512s200.6 448 448 448 448-200.6 448-448S759.4 64 512 64zm193.5 301.7l-210.6 292a31.8 31.8 0 01-51.7 0L318.5 484.9c-3.8-5.3 0-12.7 6.5-12.7h46.9c10.2 0 19.9 4.9 25.9 13.3l71.2 98.8 157.2-218c6-8.3 15.6-13.3 25.9-13.3H699c6.5 0 10.3 7.4 6.5 12.7z"}}]},name:"check-circle",theme:"filled"},lN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:iN}))},wd=u.forwardRef(lN),sN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M912 190h-69.9c-9.8 0-19.1 4.5-25.1 12.2L404.7 724.5 207 474a32 32 0 00-25.1-12.2H112c-6.7 0-10.4 7.7-6.3 12.9l273.9 347c12.8 16.2 37.4 16.2 50.3 0l488.4-618.9c4.1-5.1.4-12.8-6.3-12.8z"}}]},name:"check",theme:"outlined"},cN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:sN}))},Cw=u.forwardRef(cN),uN={icon:{tag:"svg",attrs:{"fill-rule":"evenodd",viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M512 64c247.4 0 448 200.6 448 448S759.4 960 512 960 64 759.4 64 512 264.6 64 512 64zm127.98 274.82h-.04l-.08.06L512 466.75 384.14 338.88c-.04-.05-.06-.06-.08-.06a.12.12 0 00-.07 0c-.03 0-.05.01-.09.05l-45.02 45.02a.2.2 0 00-.05.09.12.12 0 000 .07v.02a.27.27 0 00.06.06L466.75 512 338.88 639.86c-.05.04-.06.06-.06.08a.12.12 0 000 .07c0 .03.01.05.05.09l45.02 45.02a.2.2 0 00.09.05.12.12 0 00.07 0c.02 0 .04-.01.08-.05L512 557.25l127.86 127.87c.04.04.06.05.08.05a.12.12 0 00.07 0c.03 0 .05-.01.09-.05l45.02-45.02a.2.2 0 00.05-.09.12.12 0 000-.07v-.02a.27.27 0 00-.05-.06L557.25 512l127.87-127.86c.04-.04.05-.06.05-.08a.12.12 0 000-.07c0-.03-.01-.05-.05-.09l-45.02-45.02a.2.2 0 00-.09-.05.12.12 0 00-.07 0z"}}]},name:"close-circle",theme:"filled"},fN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:uN}))},Xl=u.forwardRef(fN),dN={icon:{tag:"svg",attrs:{"fill-rule":"evenodd",viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M799.86 166.31c.02 0 .04.02.08.06l57.69 57.7c.04.03.05.05.06.08a.12.12 0 010 .06c0 .03-.02.05-.06.09L569.93 512l287.7 287.7c.04.04.05.06.06.09a.12.12 0 010 .07c0 .02-.02.04-.06.08l-57.7 57.69c-.03.04-.05.05-.07.06a.12.12 0 01-.07 0c-.03 0-.05-.02-.09-.06L512 569.93l-287.7 287.7c-.04.04-.06.05-.09.06a.12.12 0 01-.07 0c-.02 0-.04-.02-.08-.06l-57.69-57.7c-.04-.03-.05-.05-.06-.07a.12.12 0 010-.07c0-.03.02-.05.06-.09L454.07 512l-287.7-287.7c-.04-.04-.05-.06-.06-.09a.12.12 0 010-.07c0-.02.02-.04.06-.08l57.7-57.69c.03-.04.05-.05.07-.06a.12.12 0 01.07 0c.03 0 .05.02.09.06L512 454.07l287.7-287.7c.04-.04.06-.05.09-.06a.12.12 0 01.07 0z"}}]},name:"close",theme:"outlined"},mN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:dN}))},Yl=u.forwardRef(mN),vN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M832 64H296c-4.4 0-8 3.6-8 8v56c0 4.4 3.6 8 8 8h496v688c0 4.4 3.6 8 8 8h56c4.4 0 8-3.6 8-8V96c0-17.7-14.3-32-32-32zM704 192H192c-17.7 0-32 14.3-32 32v530.7c0 8.5 3.4 16.6 9.4 22.6l173.3 173.3c2.2 2.2 4.7 4 7.4 5.5v1.9h4.2c3.5 1.3 7.2 2 11 2H704c17.7 0 32-14.3 32-32V224c0-17.7-14.3-32-32-32zM350 856.2L263.9 770H350v86.2zM664 888H414V746c0-22.1-17.9-40-40-40H232V264h432v624z"}}]},name:"copy",theme:"outlined"},gN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:vN}))},hN=u.forwardRef(gN),pN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M884 256h-75c-5.1 0-9.9 2.5-12.9 6.6L512 654.2 227.9 262.6c-3-4.1-7.8-6.6-12.9-6.6h-75c-6.5 0-10.3 7.4-6.5 12.7l352.6 486.1c12.8 17.6 39 17.6 51.7 0l352.6-486.1c3.9-5.3.1-12.7-6.4-12.7z"}}]},name:"down",theme:"outlined"},yN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:pN}))},bN=u.forwardRef(yN),SN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M257.7 752c2 0 4-.2 6-.5L431.9 722c2-.4 3.9-1.3 5.3-2.8l423.9-423.9a9.96 9.96 0 000-14.1L694.9 114.9c-1.9-1.9-4.4-2.9-7.1-2.9s-5.2 1-7.1 2.9L256.8 538.8c-1.5 1.5-2.4 3.3-2.8 5.3l-29.5 168.2a33.5 33.5 0 009.4 29.8c6.6 6.4 14.9 9.9 23.8 9.9zm67.4-174.4L687.8 215l73.3 73.3-362.7 362.6-88.9 15.7 15.6-89zM880 836H144c-17.7 0-32 14.3-32 32v36c0 4.4 3.6 8 8 8h784c4.4 0 8-3.6 8-8v-36c0-17.7-14.3-32-32-32z"}}]},name:"edit",theme:"outlined"},CN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:SN}))},xN=u.forwardRef(CN),EN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M176 511a56 56 0 10112 0 56 56 0 10-112 0zm280 0a56 56 0 10112 0 56 56 0 10-112 0zm280 0a56 56 0 10112 0 56 56 0 10-112 0z"}}]},name:"ellipsis",theme:"outlined"},wN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:EN}))},$N=u.forwardRef(wN),ON={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M864 170h-60c-4.4 0-8 3.6-8 8v518H310v-73c0-6.7-7.8-10.5-13-6.3l-141.9 112a8 8 0 000 12.6l141.9 112c5.3 4.2 13 .4 13-6.3v-75h498c35.3 0 64-28.7 64-64V178c0-4.4-3.6-8-8-8z"}}]},name:"enter",theme:"outlined"},RN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:ON}))},TN=u.forwardRef(RN),MN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M512 64C264.6 64 64 264.6 64 512s200.6 448 448 448 448-200.6 448-448S759.4 64 512 64zm-32 232c0-4.4 3.6-8 8-8h48c4.4 0 8 3.6 8 8v272c0 4.4-3.6 8-8 8h-48c-4.4 0-8-3.6-8-8V296zm32 440a48.01 48.01 0 010-96 48.01 48.01 0 010 96z"}}]},name:"exclamation-circle",theme:"filled"},_N=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:MN}))},Ql=u.forwardRef(_N),NN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M942.2 486.2Q889.47 375.11 816.7 305l-50.88 50.88C807.31 395.53 843.45 447.4 874.7 512 791.5 684.2 673.4 766 512 766q-72.67 0-133.87-22.38L323 798.75Q408 838 512 838q288.3 0 430.2-300.3a60.29 60.29 0 000-51.5zm-63.57-320.64L836 122.88a8 8 0 00-11.32 0L715.31 232.2Q624.86 186 512 186q-288.3 0-430.2 300.3a60.3 60.3 0 000 51.5q56.69 119.4 136.5 191.41L112.48 835a8 8 0 000 11.31L155.17 889a8 8 0 0011.31 0l712.15-712.12a8 8 0 000-11.32zM149.3 512C232.6 339.8 350.7 258 512 258c54.54 0 104.13 9.36 149.12 28.39l-70.3 70.3a176 176 0 00-238.13 238.13l-83.42 83.42C223.1 637.49 183.3 582.28 149.3 512zm246.7 0a112.11 112.11 0 01146.2-106.69L401.31 546.2A112 112 0 01396 512z"}},{tag:"path",attrs:{d:"M508 624c-3.46 0-6.87-.16-10.25-.47l-52.82 52.82a176.09 176.09 0 00227.42-227.42l-52.82 52.82c.31 3.38.47 6.79.47 10.25a111.94 111.94 0 01-112 112z"}}]},name:"eye-invisible",theme:"outlined"},AN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:NN}))},IN=u.forwardRef(AN),jN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M942.2 486.2C847.4 286.5 704.1 186 512 186c-192.2 0-335.4 100.5-430.2 300.3a60.3 60.3 0 000 51.5C176.6 737.5 319.9 838 512 838c192.2 0 335.4-100.5 430.2-300.3 7.7-16.2 7.7-35 0-51.5zM512 766c-161.3 0-279.4-81.8-362.7-254C232.6 339.8 350.7 258 512 258c161.3 0 279.4 81.8 362.7 254C791.5 684.2 673.4 766 512 766zm-4-430c-97.2 0-176 78.8-176 176s78.8 176 176 176 176-78.8 176-176-78.8-176-176-176zm0 288c-61.9 0-112-50.1-112-112s50.1-112 112-112 112 50.1 112 112-50.1 112-112 112z"}}]},name:"eye",theme:"outlined"},PN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:jN}))},zN=u.forwardRef(PN),DN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M512 64C264.6 64 64 264.6 64 512s200.6 448 448 448 448-200.6 448-448S759.4 64 512 64zm32 664c0 4.4-3.6 8-8 8h-48c-4.4 0-8-3.6-8-8V456c0-4.4 3.6-8 8-8h48c4.4 0 8 3.6 8 8v272zm-32-344a48.01 48.01 0 010-96 48.01 48.01 0 010 96z"}}]},name:"info-circle",theme:"filled"},BN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:DN}))},Wp=u.forwardRef(BN),LN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M956.9 845.1L896.4 632V168c0-17.7-14.3-32-32-32h-704c-17.7 0-32 14.3-32 32v464L67.9 845.1C60.4 866 75.8 888 98 888h828.8c22.2 0 37.6-22 30.1-42.9zM200.4 208h624v395h-624V208zm228.3 608l8.1-37h150.3l8.1 37H428.7zm224 0l-19.1-86.7c-.8-3.7-4.1-6.3-7.8-6.3H398.2c-3.8 0-7 2.6-7.8 6.3L371.3 816H151l42.3-149h638.2l42.3 149H652.7z"}}]},name:"laptop",theme:"outlined"},HN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:LN}))},FN=u.forwardRef(HN),VN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M724 218.3V141c0-6.7-7.7-10.4-12.9-6.3L260.3 486.8a31.86 31.86 0 000 50.3l450.8 352.1c5.3 4.1 12.9.4 12.9-6.3v-77.3c0-4.9-2.3-9.6-6.1-12.6l-360-281 360-281.1c3.8-3 6.1-7.7 6.1-12.6z"}}]},name:"left",theme:"outlined"},UN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:VN}))},XC=u.forwardRef(UN),qN={icon:{tag:"svg",attrs:{viewBox:"0 0 1024 1024",focusable:"false"},children:[{tag:"path",attrs:{d:"M988 548c-19.9 0-36-16.1-36-36 0-59.4-11.6-117-34.6-171.3a440.45 440.45 0 00-94.3-139.9 437.71 437.71 0 00-139.9-94.3C629 83.6 571.4 72 512 72c-19.9 0-36-16.1-36-36s16.1-36 36-36c69.1 0 136.2 13.5 199.3 40.3C772.3 66 827 103 874 150c47 47 83.9 101.8 109.7 162.7 26.7 63.1 40.2 130.2 40.2 199.3.1 19.9-16 36-35.9 36z"}}]},name:"loading",theme:"outlined"},WN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:qN}))},Uc=u.forwardRef(WN),kN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M880 112c-3.8 0-7.7.7-11.6 2.3L292 345.9H128c-8.8 0-16 7.4-16 16.6v299c0 9.2 7.2 16.6 16 16.6h101.7c-3.7 11.6-5.7 23.9-5.7 36.4 0 65.9 53.8 119.5 120 119.5 55.4 0 102.1-37.6 115.9-88.4l408.6 164.2c3.9 1.5 7.8 2.3 11.6 2.3 16.9 0 32-14.2 32-33.2V145.2C912 126.2 897 112 880 112zM344 762.3c-26.5 0-48-21.4-48-47.8 0-11.2 3.9-21.9 11-30.4l84.9 34.1c-2 24.6-22.7 44.1-47.9 44.1zm496 58.4L318.8 611.3l-12.9-5.2H184V417.9h121.9l12.9-5.2L840 203.3v617.4z"}}]},name:"notification",theme:"outlined"},GN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:kN}))},KN=u.forwardRef(GN),XN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M512 64C264.6 64 64 264.6 64 512s200.6 448 448 448 448-200.6 448-448S759.4 64 512 64zm0 820c-205.4 0-372-166.6-372-372s166.6-372 372-372 372 166.6 372 372-166.6 372-372 372z"}},{tag:"path",attrs:{d:"M623.6 316.7C593.6 290.4 554 276 512 276s-81.6 14.5-111.6 40.7C369.2 344 352 380.7 352 420v7.6c0 4.4 3.6 8 8 8h48c4.4 0 8-3.6 8-8V420c0-44.1 43.1-80 96-80s96 35.9 96 80c0 31.1-22 59.6-56.1 72.7-21.2 8.1-39.2 22.3-52.1 40.9-13.1 19-19.9 41.8-19.9 64.9V620c0 4.4 3.6 8 8 8h48c4.4 0 8-3.6 8-8v-22.7a48.3 48.3 0 0130.9-44.8c59-22.7 97.1-74.7 97.1-132.5.1-39.3-17.1-76-48.3-103.3zM472 732a40 40 0 1080 0 40 40 0 10-80 0z"}}]},name:"question-circle",theme:"outlined"},YN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:XN}))},QN=u.forwardRef(YN),ZN={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M765.7 486.8L314.9 134.7A7.97 7.97 0 00302 141v77.3c0 4.9 2.3 9.6 6.1 12.6l360 281.1-360 281.1c-3.9 3-6.1 7.7-6.1 12.6V883c0 6.7 7.7 10.4 12.9 6.3l450.8-352.1a31.96 31.96 0 000-50.4z"}}]},name:"right",theme:"outlined"},JN=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:ZN}))},Vh=u.forwardRef(JN),eA={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M909.6 854.5L649.9 594.8C690.2 542.7 712 479 712 412c0-80.2-31.3-155.4-87.9-212.1-56.6-56.7-132-87.9-212.1-87.9s-155.5 31.3-212.1 87.9C143.2 256.5 112 331.8 112 412c0 80.1 31.3 155.5 87.9 212.1C256.5 680.8 331.8 712 412 712c67 0 130.6-21.8 182.7-62l259.7 259.6a8.2 8.2 0 0011.6 0l43.6-43.5a8.2 8.2 0 000-11.6zM570.4 570.4C528 612.7 471.8 636 412 636s-116-23.3-158.4-65.6C211.3 528 188 471.8 188 412s23.3-116.1 65.6-158.4C296 211.3 352.2 188 412 188s116.1 23.2 158.4 65.6S636 352.2 636 412s-23.3 116.1-65.6 158.4z"}}]},name:"search",theme:"outlined"},tA=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:eA}))},xw=u.forwardRef(tA),nA={icon:{tag:"svg",attrs:{viewBox:"64 64 896 896",focusable:"false"},children:[{tag:"path",attrs:{d:"M858.5 763.6a374 374 0 00-80.6-119.5 375.63 375.63 0 00-119.5-80.6c-.4-.2-.8-.3-1.2-.5C719.5 518 760 444.7 760 362c0-137-111-248-248-248S264 225 264 362c0 82.7 40.5 156 102.8 201.1-.4.2-.8.3-1.2.5-44.8 18.9-85 46-119.5 80.6a375.63 375.63 0 00-80.6 119.5A371.7 371.7 0 00136 901.8a8 8 0 008 8.2h60c4.4 0 7.9-3.5 8-7.8 2-77.2 33-149.5 87.8-204.3 56.7-56.7 132-87.9 212.2-87.9s155.5 31.2 212.2 87.9C779 752.7 810 825 812 902.2c.1 4.4 3.6 7.8 8 7.8h60a8 8 0 008-8.2c-1-47.8-10.9-94.3-29.5-138.2zM512 534c-45.9 0-89.1-17.9-121.6-50.4S340 407.9 340 362c0-45.9 17.9-89.1 50.4-121.6S466.1 190 512 190s89.1 17.9 121.6 50.4S684 316.1 684 362c0 45.9-17.9 89.1-50.4 121.6S557.9 534 512 534z"}}]},name:"user",theme:"outlined"},rA=function(t,n){return u.createElement(En,Ie({},t,{ref:n,icon:nA}))},aA=u.forwardRef(rA),xg={exports:{}},tn={};/**
 * @license React
 * react-is.production.min.js
 *
 * Copyright (c) Facebook, Inc. and its affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var YC;function oA(){if(YC)return tn;YC=1;var e=Symbol.for("react.element"),t=Symbol.for("react.portal"),n=Symbol.for("react.fragment"),r=Symbol.for("react.strict_mode"),o=Symbol.for("react.profiler"),l=Symbol.for("react.provider"),c=Symbol.for("react.context"),d=Symbol.for("react.server_context"),m=Symbol.for("react.forward_ref"),v=Symbol.for("react.suspense"),g=Symbol.for("react.suspense_list"),p=Symbol.for("react.memo"),y=Symbol.for("react.lazy"),E=Symbol.for("react.offscreen"),C;C=Symbol.for("react.module.reference");function x(b){if(typeof b=="object"&&b!==null){var w=b.$$typeof;switch(w){case e:switch(b=b.type,b){case n:case o:case r:case v:case g:return b;default:switch(b=b&&b.$$typeof,b){case d:case c:case m:case y:case p:case l:return b;default:return w}}case t:return w}}}return tn.ContextConsumer=c,tn.ContextProvider=l,tn.Element=e,tn.ForwardRef=m,tn.Fragment=n,tn.Lazy=y,tn.Memo=p,tn.Portal=t,tn.Profiler=o,tn.StrictMode=r,tn.Suspense=v,tn.SuspenseList=g,tn.isAsyncMode=function(){return!1},tn.isConcurrentMode=function(){return!1},tn.isContextConsumer=function(b){return x(b)===c},tn.isContextProvider=function(b){return x(b)===l},tn.isElement=function(b){return typeof b=="object"&&b!==null&&b.$$typeof===e},tn.isForwardRef=function(b){return x(b)===m},tn.isFragment=function(b){return x(b)===n},tn.isLazy=function(b){return x(b)===y},tn.isMemo=function(b){return x(b)===p},tn.isPortal=function(b){return x(b)===t},tn.isProfiler=function(b){return x(b)===o},tn.isStrictMode=function(b){return x(b)===r},tn.isSuspense=function(b){return x(b)===v},tn.isSuspenseList=function(b){return x(b)===g},tn.isValidElementType=function(b){return typeof b=="string"||typeof b=="function"||b===n||b===o||b===r||b===v||b===g||b===E||typeof b=="object"&&b!==null&&(b.$$typeof===y||b.$$typeof===p||b.$$typeof===l||b.$$typeof===c||b.$$typeof===m||b.$$typeof===C||b.getModuleId!==void 0)},tn.typeOf=x,tn}var QC;function iA(){return QC||(QC=1,xg.exports=oA()),xg.exports}var Eg=iA();function qc(e,t,n){var r=u.useRef({});return(!("value"in r.current)||n(r.current.condition,t))&&(r.current.value=e(),r.current.condition=t),r.current.value}var lA=Symbol.for("react.element"),sA=Symbol.for("react.transitional.element"),cA=Symbol.for("react.fragment");function Ew(e){return e&&lt(e)==="object"&&(e.$$typeof===lA||e.$$typeof===sA)&&e.type===cA}var uA=Number(u.version.split(".")[0]),kp=function(t,n){typeof t=="function"?t(n):lt(t)==="object"&&t&&"current"in t&&(t.current=n)},Ia=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];var o=n.filter(Boolean);return o.length<=1?o[0]:function(l){n.forEach(function(c){kp(c,l)})}},li=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return qc(function(){return Ia.apply(void 0,n)},n,function(o,l){return o.length!==l.length||o.every(function(c,d){return c!==l[d]})})},ki=function(t){var n,r;if(!t)return!1;if(Gp(t)&&uA>=19)return!0;var o=Eg.isMemo(t)?t.type.type:t.type;return!(typeof o=="function"&&!((n=o.prototype)!==null&&n!==void 0&&n.render)&&o.$$typeof!==Eg.ForwardRef||typeof t=="function"&&!((r=t.prototype)!==null&&r!==void 0&&r.render)&&t.$$typeof!==Eg.ForwardRef)};function Gp(e){return u.isValidElement(e)&&!Ew(e)}var fA=function(t){return Gp(t)&&ki(t)},Zl=function(t){if(t&&Gp(t)){var n=t;return n.props.propertyIsEnumerable("ref")?n.props.ref:n.ref}return null};function Pr(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=[];return oe.Children.forEach(e,function(r){r==null&&!t.keepEmpty||(Array.isArray(r)?n=n.concat(Pr(r)):Ew(r)&&r.props?n=n.concat(Pr(r.props.children,t)):n.push(r))}),n}var ri=lw();const Uh=Sd(ri),dA=iw({__proto__:null,default:Uh},[ri]);function xc(e){return e instanceof HTMLElement||e instanceof SVGElement}function ww(e){return e&&lt(e)==="object"&&xc(e.nativeElement)?e.nativeElement:xc(e)?e:null}function Rf(e){var t=ww(e);if(t)return t;if(e instanceof oe.Component){var n;return(n=Uh.findDOMNode)===null||n===void 0?void 0:n.call(Uh,e)}return null}var qh=u.createContext(null);function mA(e){var t=e.children,n=e.onBatchResize,r=u.useRef(0),o=u.useRef([]),l=u.useContext(qh),c=u.useCallback(function(d,m,v){r.current+=1;var g=r.current;o.current.push({size:d,element:m,data:v}),Promise.resolve().then(function(){g===r.current&&(n==null||n(o.current),o.current=[])}),l==null||l(d,m,v)},[n,l]);return u.createElement(qh.Provider,{value:c},t)}var $w=function(){if(typeof Map<"u")return Map;function e(t,n){var r=-1;return t.some(function(o,l){return o[0]===n?(r=l,!0):!1}),r}return function(){function t(){this.__entries__=[]}return Object.defineProperty(t.prototype,"size",{get:function(){return this.__entries__.length},enumerable:!0,configurable:!0}),t.prototype.get=function(n){var r=e(this.__entries__,n),o=this.__entries__[r];return o&&o[1]},t.prototype.set=function(n,r){var o=e(this.__entries__,n);~o?this.__entries__[o][1]=r:this.__entries__.push([n,r])},t.prototype.delete=function(n){var r=this.__entries__,o=e(r,n);~o&&r.splice(o,1)},t.prototype.has=function(n){return!!~e(this.__entries__,n)},t.prototype.clear=function(){this.__entries__.splice(0)},t.prototype.forEach=function(n,r){r===void 0&&(r=null);for(var o=0,l=this.__entries__;o<l.length;o++){var c=l[o];n.call(r,c[1],c[0])}},t}()}(),Wh=typeof window<"u"&&typeof document<"u"&&window.document===document,Kf=function(){return typeof global<"u"&&global.Math===Math?global:typeof self<"u"&&self.Math===Math?self:typeof window<"u"&&window.Math===Math?window:Function("return this")()}(),vA=function(){return typeof requestAnimationFrame=="function"?requestAnimationFrame.bind(Kf):function(e){return setTimeout(function(){return e(Date.now())},1e3/60)}}(),gA=2;function hA(e,t){var n=!1,r=!1,o=0;function l(){n&&(n=!1,e()),r&&d()}function c(){vA(l)}function d(){var m=Date.now();if(n){if(m-o<gA)return;r=!0}else n=!0,r=!1,setTimeout(c,t);o=m}return d}var pA=20,yA=["top","right","bottom","left","width","height","size","weight"],bA=typeof MutationObserver<"u",SA=function(){function e(){this.connected_=!1,this.mutationEventsAdded_=!1,this.mutationsObserver_=null,this.observers_=[],this.onTransitionEnd_=this.onTransitionEnd_.bind(this),this.refresh=hA(this.refresh.bind(this),pA)}return e.prototype.addObserver=function(t){~this.observers_.indexOf(t)||this.observers_.push(t),this.connected_||this.connect_()},e.prototype.removeObserver=function(t){var n=this.observers_,r=n.indexOf(t);~r&&n.splice(r,1),!n.length&&this.connected_&&this.disconnect_()},e.prototype.refresh=function(){var t=this.updateObservers_();t&&this.refresh()},e.prototype.updateObservers_=function(){var t=this.observers_.filter(function(n){return n.gatherActive(),n.hasActive()});return t.forEach(function(n){return n.broadcastActive()}),t.length>0},e.prototype.connect_=function(){!Wh||this.connected_||(document.addEventListener("transitionend",this.onTransitionEnd_),window.addEventListener("resize",this.refresh),bA?(this.mutationsObserver_=new MutationObserver(this.refresh),this.mutationsObserver_.observe(document,{attributes:!0,childList:!0,characterData:!0,subtree:!0})):(document.addEventListener("DOMSubtreeModified",this.refresh),this.mutationEventsAdded_=!0),this.connected_=!0)},e.prototype.disconnect_=function(){!Wh||!this.connected_||(document.removeEventListener("transitionend",this.onTransitionEnd_),window.removeEventListener("resize",this.refresh),this.mutationsObserver_&&this.mutationsObserver_.disconnect(),this.mutationEventsAdded_&&document.removeEventListener("DOMSubtreeModified",this.refresh),this.mutationsObserver_=null,this.mutationEventsAdded_=!1,this.connected_=!1)},e.prototype.onTransitionEnd_=function(t){var n=t.propertyName,r=n===void 0?"":n,o=yA.some(function(l){return!!~r.indexOf(l)});o&&this.refresh()},e.getInstance=function(){return this.instance_||(this.instance_=new e),this.instance_},e.instance_=null,e}(),Ow=function(e,t){for(var n=0,r=Object.keys(t);n<r.length;n++){var o=r[n];Object.defineProperty(e,o,{value:t[o],enumerable:!1,writable:!1,configurable:!0})}return e},Fl=function(e){var t=e&&e.ownerDocument&&e.ownerDocument.defaultView;return t||Kf},Rw=$d(0,0,0,0);function Xf(e){return parseFloat(e)||0}function ZC(e){for(var t=[],n=1;n<arguments.length;n++)t[n-1]=arguments[n];return t.reduce(function(r,o){var l=e["border-"+o+"-width"];return r+Xf(l)},0)}function CA(e){for(var t=["top","right","bottom","left"],n={},r=0,o=t;r<o.length;r++){var l=o[r],c=e["padding-"+l];n[l]=Xf(c)}return n}function xA(e){var t=e.getBBox();return $d(0,0,t.width,t.height)}function EA(e){var t=e.clientWidth,n=e.clientHeight;if(!t&&!n)return Rw;var r=Fl(e).getComputedStyle(e),o=CA(r),l=o.left+o.right,c=o.top+o.bottom,d=Xf(r.width),m=Xf(r.height);if(r.boxSizing==="border-box"&&(Math.round(d+l)!==t&&(d-=ZC(r,"left","right")+l),Math.round(m+c)!==n&&(m-=ZC(r,"top","bottom")+c)),!$A(e)){var v=Math.round(d+l)-t,g=Math.round(m+c)-n;Math.abs(v)!==1&&(d-=v),Math.abs(g)!==1&&(m-=g)}return $d(o.left,o.top,d,m)}var wA=function(){return typeof SVGGraphicsElement<"u"?function(e){return e instanceof Fl(e).SVGGraphicsElement}:function(e){return e instanceof Fl(e).SVGElement&&typeof e.getBBox=="function"}}();function $A(e){return e===Fl(e).document.documentElement}function OA(e){return Wh?wA(e)?xA(e):EA(e):Rw}function RA(e){var t=e.x,n=e.y,r=e.width,o=e.height,l=typeof DOMRectReadOnly<"u"?DOMRectReadOnly:Object,c=Object.create(l.prototype);return Ow(c,{x:t,y:n,width:r,height:o,top:n,right:t+r,bottom:o+n,left:t}),c}function $d(e,t,n,r){return{x:e,y:t,width:n,height:r}}var TA=function(){function e(t){this.broadcastWidth=0,this.broadcastHeight=0,this.contentRect_=$d(0,0,0,0),this.target=t}return e.prototype.isActive=function(){var t=OA(this.target);return this.contentRect_=t,t.width!==this.broadcastWidth||t.height!==this.broadcastHeight},e.prototype.broadcastRect=function(){var t=this.contentRect_;return this.broadcastWidth=t.width,this.broadcastHeight=t.height,t},e}(),MA=function(){function e(t,n){var r=RA(n);Ow(this,{target:t,contentRect:r})}return e}(),_A=function(){function e(t,n,r){if(this.activeObservations_=[],this.observations_=new $w,typeof t!="function")throw new TypeError("The callback provided as parameter 1 is not a function.");this.callback_=t,this.controller_=n,this.callbackCtx_=r}return e.prototype.observe=function(t){if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");if(!(typeof Element>"u"||!(Element instanceof Object))){if(!(t instanceof Fl(t).Element))throw new TypeError('parameter 1 is not of type "Element".');var n=this.observations_;n.has(t)||(n.set(t,new TA(t)),this.controller_.addObserver(this),this.controller_.refresh())}},e.prototype.unobserve=function(t){if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");if(!(typeof Element>"u"||!(Element instanceof Object))){if(!(t instanceof Fl(t).Element))throw new TypeError('parameter 1 is not of type "Element".');var n=this.observations_;n.has(t)&&(n.delete(t),n.size||this.controller_.removeObserver(this))}},e.prototype.disconnect=function(){this.clearActive(),this.observations_.clear(),this.controller_.removeObserver(this)},e.prototype.gatherActive=function(){var t=this;this.clearActive(),this.observations_.forEach(function(n){n.isActive()&&t.activeObservations_.push(n)})},e.prototype.broadcastActive=function(){if(this.hasActive()){var t=this.callbackCtx_,n=this.activeObservations_.map(function(r){return new MA(r.target,r.broadcastRect())});this.callback_.call(t,n,t),this.clearActive()}},e.prototype.clearActive=function(){this.activeObservations_.splice(0)},e.prototype.hasActive=function(){return this.activeObservations_.length>0},e}(),Tw=typeof WeakMap<"u"?new WeakMap:new $w,Mw=function(){function e(t){if(!(this instanceof e))throw new TypeError("Cannot call a class as a function.");if(!arguments.length)throw new TypeError("1 argument required, but only 0 present.");var n=SA.getInstance(),r=new _A(t,n,this);Tw.set(this,r)}return e}();["observe","unobserve","disconnect"].forEach(function(e){Mw.prototype[e]=function(){var t;return(t=Tw.get(this))[e].apply(t,arguments)}});var NA=function(){return typeof Kf.ResizeObserver<"u"?Kf.ResizeObserver:Mw}(),ei=new Map;function AA(e){e.forEach(function(t){var n,r=t.target;(n=ei.get(r))===null||n===void 0||n.forEach(function(o){return o(r)})})}var _w=new NA(AA);function IA(e,t){ei.has(e)||(ei.set(e,new Set),_w.observe(e)),ei.get(e).add(t)}function jA(e,t){ei.has(e)&&(ei.get(e).delete(t),ei.get(e).size||(_w.unobserve(e),ei.delete(e)))}function qn(e,t){if(!(e instanceof t))throw new TypeError("Cannot call a class as a function")}function JC(e,t){for(var n=0;n<t.length;n++){var r=t[n];r.enumerable=r.enumerable||!1,r.configurable=!0,"value"in r&&(r.writable=!0),Object.defineProperty(e,uw(r.key),r)}}function Wn(e,t,n){return t&&JC(e.prototype,t),n&&JC(e,n),Object.defineProperty(e,"prototype",{writable:!1}),e}function Ec(e,t){return Ec=Object.setPrototypeOf?Object.setPrototypeOf.bind():function(n,r){return n.__proto__=r,n},Ec(e,t)}function $o(e,t){if(typeof t!="function"&&t!==null)throw new TypeError("Super expression must either be null or a function");e.prototype=Object.create(t&&t.prototype,{constructor:{value:e,writable:!0,configurable:!0}}),Object.defineProperty(e,"prototype",{writable:!1}),t&&Ec(e,t)}function wc(e){return wc=Object.setPrototypeOf?Object.getPrototypeOf.bind():function(t){return t.__proto__||Object.getPrototypeOf(t)},wc(e)}function Kp(){try{var e=!Boolean.prototype.valueOf.call(Reflect.construct(Boolean,[],function(){}))}catch{}return(Kp=function(){return!!e})()}function _t(e){if(e===void 0)throw new ReferenceError("this hasn't been initialised - super() hasn't been called");return e}function PA(e,t){if(t&&(lt(t)=="object"||typeof t=="function"))return t;if(t!==void 0)throw new TypeError("Derived constructors may only return object or undefined");return _t(e)}function Oo(e){var t=Kp();return function(){var n,r=wc(e);if(t){var o=wc(this).constructor;n=Reflect.construct(r,arguments,o)}else n=r.apply(this,arguments);return PA(this,n)}}var zA=function(e){$o(n,e);var t=Oo(n);function n(){return qn(this,n),t.apply(this,arguments)}return Wn(n,[{key:"render",value:function(){return this.props.children}}]),n}(u.Component);function DA(e,t){var n=e.children,r=e.disabled,o=u.useRef(null),l=u.useRef(null),c=u.useContext(qh),d=typeof n=="function",m=d?n(o):n,v=u.useRef({width:-1,height:-1,offsetWidth:-1,offsetHeight:-1}),g=!d&&u.isValidElement(m)&&ki(m),p=g?Zl(m):null,y=li(p,o),E=function(){var w;return Rf(o.current)||(o.current&&lt(o.current)==="object"?Rf((w=o.current)===null||w===void 0?void 0:w.nativeElement):null)||Rf(l.current)};u.useImperativeHandle(t,function(){return E()});var C=u.useRef(e);C.current=e;var x=u.useCallback(function(b){var w=C.current,$=w.onResize,O=w.data,T=b.getBoundingClientRect(),R=T.width,M=T.height,N=b.offsetWidth,z=b.offsetHeight,P=Math.floor(R),D=Math.floor(M);if(v.current.width!==P||v.current.height!==D||v.current.offsetWidth!==N||v.current.offsetHeight!==z){var F={width:P,height:D,offsetWidth:N,offsetHeight:z};v.current=F;var W=N===Math.round(R)?R:N,V=z===Math.round(M)?M:z,q=Z(Z({},F),{},{offsetWidth:W,offsetHeight:V});c==null||c(q,b,O),$&&Promise.resolve().then(function(){$(q,b)})}},[]);return u.useEffect(function(){var b=E();return b&&!r&&IA(b,x),function(){return jA(b,x)}},[o.current,r]),u.createElement(zA,{ref:l},g?u.cloneElement(m,{ref:y}):m)}var BA=u.forwardRef(DA),LA="rc-observer-key";function HA(e,t){var n=e.children,r=typeof n=="function"?[n]:Pr(n);return r.map(function(o,l){var c=(o==null?void 0:o.key)||"".concat(LA,"-").concat(l);return u.createElement(BA,Ie({},e,{key:c,ref:l===0?t:void 0}),o)})}var Ro=u.forwardRef(HA);Ro.Collection=mA;function FA(e){if(Array.isArray(e))return Oh(e)}function Nw(e){if(typeof Symbol<"u"&&e[Symbol.iterator]!=null||e["@@iterator"]!=null)return Array.from(e)}function VA(){throw new TypeError(`Invalid attempt to spread non-iterable instance.
In order to be iterable, non-array objects must have a [Symbol.iterator]() method.`)}function Te(e){return FA(e)||Nw(e)||Up(e)||VA()}var Aw=function(t){return+setTimeout(t,16)},Iw=function(t){return clearTimeout(t)};typeof window<"u"&&"requestAnimationFrame"in window&&(Aw=function(t){return window.requestAnimationFrame(t)},Iw=function(t){return window.cancelAnimationFrame(t)});var e1=0,Xp=new Map;function jw(e){Xp.delete(e)}var gn=function(t){var n=arguments.length>1&&arguments[1]!==void 0?arguments[1]:1;e1+=1;var r=e1;function o(l){if(l===0)jw(r),t();else{var c=Aw(function(){o(l-1)});Xp.set(r,c)}}return o(n),r};gn.cancel=function(e){var t=Xp.get(e);return jw(e),Iw(t)};function $c(e){for(var t=0,n,r=0,o=e.length;o>=4;++r,o-=4)n=e.charCodeAt(r)&255|(e.charCodeAt(++r)&255)<<8|(e.charCodeAt(++r)&255)<<16|(e.charCodeAt(++r)&255)<<24,n=(n&65535)*1540483477+((n>>>16)*59797<<16),n^=n>>>24,t=(n&65535)*1540483477+((n>>>16)*59797<<16)^(t&65535)*1540483477+((t>>>16)*59797<<16);switch(o){case 3:t^=(e.charCodeAt(r+2)&255)<<16;case 2:t^=(e.charCodeAt(r+1)&255)<<8;case 1:t^=e.charCodeAt(r)&255,t=(t&65535)*1540483477+((t>>>16)*59797<<16)}return t^=t>>>13,t=(t&65535)*1540483477+((t>>>16)*59797<<16),((t^t>>>15)>>>0).toString(36)}function Oc(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!1,r=new Set;function o(l,c){var d=arguments.length>2&&arguments[2]!==void 0?arguments[2]:1,m=r.has(l);if(vr(!m,"Warning: There may be circular references"),m)return!1;if(l===c)return!0;if(n&&d>1)return!1;r.add(l);var v=d+1;if(Array.isArray(l)){if(!Array.isArray(c)||l.length!==c.length)return!1;for(var g=0;g<l.length;g++)if(!o(l[g],c[g],v))return!1;return!0}if(l&&c&&lt(l)==="object"&&lt(c)==="object"){var p=Object.keys(l);return p.length!==Object.keys(c).length?!1:p.every(function(y){return o(l[y],c[y],v)})}return!1}return o(e,t)}var UA="%";function kh(e){return e.join(UA)}var qA=function(){function e(t){qn(this,e),Y(this,"instanceId",void 0),Y(this,"cache",new Map),this.instanceId=t}return Wn(e,[{key:"get",value:function(n){return this.opGet(kh(n))}},{key:"opGet",value:function(n){return this.cache.get(n)||null}},{key:"update",value:function(n,r){return this.opUpdate(kh(n),r)}},{key:"opUpdate",value:function(n,r){var o=this.cache.get(n),l=r(o);l===null?this.cache.delete(n):this.cache.set(n,l)}}]),e}(),Vl="data-token-hash",Ma="data-css-hash",ti="__cssinjs_instance__";function WA(){var e=Math.random().toString(12).slice(2);if(typeof document<"u"&&document.head&&document.body){var t=document.body.querySelectorAll("style[".concat(Ma,"]"))||[],n=document.head.firstChild;Array.from(t).forEach(function(o){o[ti]=o[ti]||e,o[ti]===e&&document.head.insertBefore(o,n)});var r={};Array.from(document.querySelectorAll("style[".concat(Ma,"]"))).forEach(function(o){var l=o.getAttribute(Ma);if(r[l]){if(o[ti]===e){var c;(c=o.parentNode)===null||c===void 0||c.removeChild(o)}}else r[l]=!0})}return new qA(e)}var Wc=u.createContext({hashPriority:"low",cache:WA(),defaultCache:!0});function kA(e,t){if(e.length!==t.length)return!1;for(var n=0;n<e.length;n++)if(e[n]!==t[n])return!1;return!0}var Yp=function(){function e(){qn(this,e),Y(this,"cache",void 0),Y(this,"keys",void 0),Y(this,"cacheCallTimes",void 0),this.cache=new Map,this.keys=[],this.cacheCallTimes=0}return Wn(e,[{key:"size",value:function(){return this.keys.length}},{key:"internalGet",value:function(n){var r,o,l=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,c={map:this.cache};return n.forEach(function(d){if(!c)c=void 0;else{var m;c=(m=c)===null||m===void 0||(m=m.map)===null||m===void 0?void 0:m.get(d)}}),(r=c)!==null&&r!==void 0&&r.value&&l&&(c.value[1]=this.cacheCallTimes++),(o=c)===null||o===void 0?void 0:o.value}},{key:"get",value:function(n){var r;return(r=this.internalGet(n,!0))===null||r===void 0?void 0:r[0]}},{key:"has",value:function(n){return!!this.internalGet(n)}},{key:"set",value:function(n,r){var o=this;if(!this.has(n)){if(this.size()+1>e.MAX_CACHE_SIZE+e.MAX_CACHE_OFFSET){var l=this.keys.reduce(function(v,g){var p=de(v,2),y=p[1];return o.internalGet(g)[1]<y?[g,o.internalGet(g)[1]]:v},[this.keys[0],this.cacheCallTimes]),c=de(l,1),d=c[0];this.delete(d)}this.keys.push(n)}var m=this.cache;n.forEach(function(v,g){if(g===n.length-1)m.set(v,{value:[r,o.cacheCallTimes++]});else{var p=m.get(v);p?p.map||(p.map=new Map):m.set(v,{map:new Map}),m=m.get(v).map}})}},{key:"deleteByPath",value:function(n,r){var o=n.get(r[0]);if(r.length===1){var l;return o.map?n.set(r[0],{map:o.map}):n.delete(r[0]),(l=o.value)===null||l===void 0?void 0:l[0]}var c=this.deleteByPath(o.map,r.slice(1));return(!o.map||o.map.size===0)&&!o.value&&n.delete(r[0]),c}},{key:"delete",value:function(n){if(this.has(n))return this.keys=this.keys.filter(function(r){return!kA(r,n)}),this.deleteByPath(this.cache,n)}}]),e}();Y(Yp,"MAX_CACHE_SIZE",20);Y(Yp,"MAX_CACHE_OFFSET",5);var t1=0,Pw=function(){function e(t){qn(this,e),Y(this,"derivatives",void 0),Y(this,"id",void 0),this.derivatives=Array.isArray(t)?t:[t],this.id=t1,t.length===0&&(t.length>0,void 0),t1+=1}return Wn(e,[{key:"getDerivativeToken",value:function(n){return this.derivatives.reduce(function(r,o){return o(n,r)},void 0)}}]),e}(),wg=new Yp;function Yf(e){var t=Array.isArray(e)?e:[e];return wg.has(t)||wg.set(t,new Pw(t)),wg.get(t)}var GA=new WeakMap,$g={};function KA(e,t){for(var n=GA,r=0;r<t.length;r+=1){var o=t[r];n.has(o)||n.set(o,new WeakMap),n=n.get(o)}return n.has($g)||n.set($g,e()),n.get($g)}var n1=new WeakMap;function pc(e){var t=n1.get(e)||"";return t||(Object.keys(e).forEach(function(n){var r=e[n];t+=n,r instanceof Pw?t+=r.id:r&&lt(r)==="object"?t+=pc(r):t+=r}),t=$c(t),n1.set(e,t)),t}function r1(e,t){return $c("".concat(t,"_").concat(pc(e)))}var Gh=Cr();function he(e){return typeof e=="number"?"".concat(e,"px"):e}function Qf(e,t,n){var r,o=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{},l=arguments.length>4&&arguments[4]!==void 0?arguments[4]:!1;if(l)return e;var c=Z(Z({},o),{},(r={},Y(r,Vl,t),Y(r,Ma,n),r)),d=Object.keys(c).map(function(m){var v=c[m];return v?"".concat(m,'="').concat(v,'"'):null}).filter(function(m){return m}).join(" ");return"<style ".concat(d,">").concat(e,"</style>")}var Tf=function(t){var n=arguments.length>1&&arguments[1]!==void 0?arguments[1]:"";return"--".concat(n?"".concat(n,"-"):"").concat(t).replace(/([a-z0-9])([A-Z])/g,"$1-$2").replace(/([A-Z]+)([A-Z][a-z0-9]+)/g,"$1-$2").replace(/([a-z])([A-Z0-9])/g,"$1-$2").toLowerCase()},XA=function(t,n,r){return Object.keys(t).length?".".concat(n).concat(r!=null&&r.scope?".".concat(r.scope):"","{").concat(Object.entries(t).map(function(o){var l=de(o,2),c=l[0],d=l[1];return"".concat(c,":").concat(d,";")}).join(""),"}"):""},zw=function(t,n,r){var o={},l={};return Object.entries(t).forEach(function(c){var d,m,v=de(c,2),g=v[0],p=v[1];if(r!=null&&(d=r.preserve)!==null&&d!==void 0&&d[g])l[g]=p;else if((typeof p=="string"||typeof p=="number")&&!(r!=null&&(m=r.ignore)!==null&&m!==void 0&&m[g])){var y,E=Tf(g,r==null?void 0:r.prefix);o[E]=typeof p=="number"&&!(r!=null&&(y=r.unitless)!==null&&y!==void 0&&y[g])?"".concat(p,"px"):String(p),l[g]="var(".concat(E,")")}}),[l,XA(o,n,{scope:r==null?void 0:r.scope})]},a1=Cr()?u.useLayoutEffect:u.useEffect,Wt=function(t,n){var r=u.useRef(!0);a1(function(){return t(r.current)},n),a1(function(){return r.current=!1,function(){r.current=!0}},[])},o1=function(t,n){Wt(function(r){if(!r)return t()},n)},YA=Z({},xd),i1=YA.useInsertionEffect,QA=function(t,n,r){u.useMemo(t,r),Wt(function(){return n(!0)},r)},ZA=i1?function(e,t,n){return i1(function(){return e(),t()},n)}:QA,JA=Z({},xd),eI=JA.useInsertionEffect,tI=function(t){var n=[],r=!1;function o(l){r||n.push(l)}return u.useEffect(function(){return r=!1,function(){r=!0,n.length&&n.forEach(function(l){return l()})}},t),o},nI=function(){return function(t){t()}},rI=typeof eI<"u"?tI:nI;function Qp(e,t,n,r,o){var l=u.useContext(Wc),c=l.cache,d=[e].concat(Te(t)),m=kh(d),v=rI([m]),g=function(C){c.opUpdate(m,function(x){var b=x||[void 0,void 0],w=de(b,2),$=w[0],O=$===void 0?0:$,T=w[1],R=T,M=R||n(),N=[O,M];return C?C(N):N})};u.useMemo(function(){g()},[m]);var p=c.opGet(m),y=p[1];return ZA(function(){o==null||o(y)},function(E){return g(function(C){var x=de(C,2),b=x[0],w=x[1];return E&&b===0&&(o==null||o(y)),[b+1,w]}),function(){c.opUpdate(m,function(C){var x=C||[],b=de(x,2),w=b[0],$=w===void 0?0:w,O=b[1],T=$-1;return T===0?(v(function(){(E||!c.opGet(m))&&(r==null||r(O,!1))}),null):[$-1,O]})}},[m]),y}var aI={},oI="css",Ni=new Map;function iI(e){Ni.set(e,(Ni.get(e)||0)+1)}function lI(e,t){if(typeof document<"u"){var n=document.querySelectorAll("style[".concat(Vl,'="').concat(e,'"]'));n.forEach(function(r){if(r[ti]===t){var o;(o=r.parentNode)===null||o===void 0||o.removeChild(r)}})}}var sI=0;function cI(e,t){Ni.set(e,(Ni.get(e)||0)-1);var n=Array.from(Ni.keys()),r=n.filter(function(o){var l=Ni.get(o)||0;return l<=0});n.length-r.length>sI&&r.forEach(function(o){lI(o,t),Ni.delete(o)})}var Dw=function(t,n,r,o){var l=r.getDerivativeToken(t),c=Z(Z({},l),n);return o&&(c=o(c)),c},Bw="token";function uI(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{},r=u.useContext(Wc),o=r.cache.instanceId,l=r.container,c=n.salt,d=c===void 0?"":c,m=n.override,v=m===void 0?aI:m,g=n.formatToken,p=n.getComputedToken,y=n.cssVar,E=KA(function(){return Object.assign.apply(Object,[{}].concat(Te(t)))},t),C=pc(E),x=pc(v),b=y?pc(y):"",w=Qp(Bw,[d,e.id,C,x,b],function(){var $,O=p?p(E,v,e):Dw(E,v,e,g),T=Z({},O),R="";if(y){var M=zw(O,y.key,{prefix:y.prefix,ignore:y.ignore,unitless:y.unitless,preserve:y.preserve}),N=de(M,2);O=N[0],R=N[1]}var z=r1(O,d);O._tokenKey=z,T._tokenKey=r1(T,d);var P=($=y==null?void 0:y.key)!==null&&$!==void 0?$:z;O._themeKey=P,iI(P);var D="".concat(oI,"-").concat($c(z));return O._hashId=D,[O,D,T,R,(y==null?void 0:y.key)||""]},function($){cI($[0]._themeKey,o)},function($){var O=de($,4),T=O[0],R=O[3];if(y&&R){var M=po(R,$c("css-variables-".concat(T._themeKey)),{mark:Ma,prepend:"queue",attachTo:l,priority:-999});M[ti]=o,M.setAttribute(Vl,T._themeKey)}});return w}var fI=function(t,n,r){var o=de(t,5),l=o[2],c=o[3],d=o[4],m=r||{},v=m.plain;if(!c)return null;var g=l._tokenKey,p=-999,y={"data-rc-order":"prependQueue","data-rc-priority":"".concat(p)},E=Qf(c,d,g,y,v);return[p,g,E]},dI={animationIterationCount:1,borderImageOutset:1,borderImageSlice:1,borderImageWidth:1,boxFlex:1,boxFlexGroup:1,boxOrdinalGroup:1,columnCount:1,columns:1,flex:1,flexGrow:1,flexPositive:1,flexShrink:1,flexNegative:1,flexOrder:1,gridRow:1,gridRowEnd:1,gridRowSpan:1,gridRowStart:1,gridColumn:1,gridColumnEnd:1,gridColumnSpan:1,gridColumnStart:1,msGridRow:1,msGridRowSpan:1,msGridColumn:1,msGridColumnSpan:1,fontWeight:1,lineHeight:1,opacity:1,order:1,orphans:1,tabSize:1,widows:1,zIndex:1,zoom:1,WebkitLineClamp:1,fillOpacity:1,floodOpacity:1,stopOpacity:1,strokeDasharray:1,strokeDashoffset:1,strokeMiterlimit:1,strokeOpacity:1,strokeWidth:1},Lw="comm",Hw="rule",Fw="decl",mI="@import",vI="@namespace",gI="@keyframes",hI="@layer",Vw=Math.abs,Zp=String.fromCharCode;function Uw(e){return e.trim()}function Mf(e,t,n){return e.replace(t,n)}function pI(e,t,n){return e.indexOf(t,n)}function Bl(e,t){return e.charCodeAt(t)|0}function Ul(e,t,n){return e.slice(t,n)}function Wa(e){return e.length}function yI(e){return e.length}function ff(e,t){return t.push(e),e}var Od=1,ql=1,qw=0,Sa=0,Un=0,Jl="";function Jp(e,t,n,r,o,l,c,d){return{value:e,root:t,parent:n,type:r,props:o,children:l,line:Od,column:ql,length:c,return:"",siblings:d}}function bI(){return Un}function SI(){return Un=Sa>0?Bl(Jl,--Sa):0,ql--,Un===10&&(ql=1,Od--),Un}function _a(){return Un=Sa<qw?Bl(Jl,Sa++):0,ql++,Un===10&&(ql=1,Od++),Un}function ni(){return Bl(Jl,Sa)}function _f(){return Sa}function Rd(e,t){return Ul(Jl,e,t)}function Rc(e){switch(e){case 0:case 9:case 10:case 13:case 32:return 5;case 33:case 43:case 44:case 47:case 62:case 64:case 126:case 59:case 123:case 125:return 4;case 58:return 3;case 34:case 39:case 40:case 91:return 2;case 41:case 93:return 1}return 0}function CI(e){return Od=ql=1,qw=Wa(Jl=e),Sa=0,[]}function xI(e){return Jl="",e}function Og(e){return Uw(Rd(Sa-1,Kh(e===91?e+2:e===40?e+1:e)))}function EI(e){for(;(Un=ni())&&Un<33;)_a();return Rc(e)>2||Rc(Un)>3?"":" "}function wI(e,t){for(;--t&&_a()&&!(Un<48||Un>102||Un>57&&Un<65||Un>70&&Un<97););return Rd(e,_f()+(t<6&&ni()==32&&_a()==32))}function Kh(e){for(;_a();)switch(Un){case e:return Sa;case 34:case 39:e!==34&&e!==39&&Kh(Un);break;case 40:e===41&&Kh(e);break;case 92:_a();break}return Sa}function $I(e,t){for(;_a()&&e+Un!==57;)if(e+Un===84&&ni()===47)break;return"/*"+Rd(t,Sa-1)+"*"+Zp(e===47?e:_a())}function OI(e){for(;!Rc(ni());)_a();return Rd(e,Sa)}function RI(e){return xI(Nf("",null,null,null,[""],e=CI(e),0,[0],e))}function Nf(e,t,n,r,o,l,c,d,m){for(var v=0,g=0,p=c,y=0,E=0,C=0,x=1,b=1,w=1,$=0,O="",T=o,R=l,M=r,N=O;b;)switch(C=$,$=_a()){case 40:if(C!=108&&Bl(N,p-1)==58){pI(N+=Mf(Og($),"&","&\f"),"&\f",Vw(v?d[v-1]:0))!=-1&&(w=-1);break}case 34:case 39:case 91:N+=Og($);break;case 9:case 10:case 13:case 32:N+=EI(C);break;case 92:N+=wI(_f()-1,7);continue;case 47:switch(ni()){case 42:case 47:ff(TI($I(_a(),_f()),t,n,m),m),(Rc(C||1)==5||Rc(ni()||1)==5)&&Wa(N)&&Ul(N,-1,void 0)!==" "&&(N+=" ");break;default:N+="/"}break;case 123*x:d[v++]=Wa(N)*w;case 125*x:case 59:case 0:switch($){case 0:case 125:b=0;case 59+g:w==-1&&(N=Mf(N,/\f/g,"")),E>0&&(Wa(N)-p||x===0&&C===47)&&ff(E>32?s1(N+";",r,n,p-1,m):s1(Mf(N," ","")+";",r,n,p-2,m),m);break;case 59:N+=";";default:if(ff(M=l1(N,t,n,v,g,o,d,O,T=[],R=[],p,l),l),$===123)if(g===0)Nf(N,t,M,M,T,l,p,d,R);else{switch(y){case 99:if(Bl(N,3)===110)break;case 108:if(Bl(N,2)===97)break;default:g=0;case 100:case 109:case 115:}g?Nf(e,M,M,r&&ff(l1(e,M,M,0,0,o,d,O,o,T=[],p,R),R),o,R,p,d,r?T:R):Nf(N,M,M,M,[""],R,0,d,R)}}v=g=E=0,x=w=1,O=N="",p=c;break;case 58:p=1+Wa(N),E=C;default:if(x<1){if($==123)--x;else if($==125&&x++==0&&SI()==125)continue}switch(N+=Zp($),$*x){case 38:w=g>0?1:(N+="\f",-1);break;case 44:d[v++]=(Wa(N)-1)*w,w=1;break;case 64:ni()===45&&(N+=Og(_a())),y=ni(),g=p=Wa(O=N+=OI(_f())),$++;break;case 45:C===45&&Wa(N)==2&&(x=0)}}return l}function l1(e,t,n,r,o,l,c,d,m,v,g,p){for(var y=o-1,E=o===0?l:[""],C=yI(E),x=0,b=0,w=0;x<r;++x)for(var $=0,O=Ul(e,y+1,y=Vw(b=c[x])),T=e;$<C;++$)(T=Uw(b>0?E[$]+" "+O:Mf(O,/&\f/g,E[$])))&&(m[w++]=T);return Jp(e,t,n,o===0?Hw:d,m,v,g,p)}function TI(e,t,n,r){return Jp(e,t,n,Lw,Zp(bI()),Ul(e,2,-2),0,r)}function s1(e,t,n,r,o){return Jp(e,t,n,Fw,Ul(e,0,r),Ul(e,r+1,-1),r,o)}function Xh(e,t){for(var n="",r=0;r<e.length;r++)n+=t(e[r],r,e,t)||"";return n}function MI(e,t,n,r){switch(e.type){case hI:if(e.children.length)break;case mI:case vI:case Fw:return e.return=e.return||e.value;case Lw:return"";case gI:return e.return=e.value+"{"+Xh(e.children,r)+"}";case Hw:if(!Wa(e.value=e.props.join(",")))return""}return Wa(n=Xh(e.children,r))?e.return=e.value+"{"+n+"}":""}var c1="data-ant-cssinjs-cache-path",Ww="_FILE_STYLE__",zi,kw=!0;function _I(){if(!zi&&(zi={},Cr())){var e=document.createElement("div");e.className=c1,e.style.position="fixed",e.style.visibility="hidden",e.style.top="-9999px",document.body.appendChild(e);var t=getComputedStyle(e).content||"";t=t.replace(/^"/,"").replace(/"$/,""),t.split(";").forEach(function(o){var l=o.split(":"),c=de(l,2),d=c[0],m=c[1];zi[d]=m});var n=document.querySelector("style[".concat(c1,"]"));if(n){var r;kw=!1,(r=n.parentNode)===null||r===void 0||r.removeChild(n)}document.body.removeChild(e)}}function NI(e){return _I(),!!zi[e]}function AI(e){var t=zi[e],n=null;if(t&&Cr())if(kw)n=Ww;else{var r=document.querySelector("style[".concat(Ma,'="').concat(zi[e],'"]'));r?n=r.innerHTML:delete zi[e]}return[n,t]}var II="_skip_check_",Gw="_multi_value_";function Af(e){var t=Xh(RI(e),MI);return t.replace(/\{%%%\:[^;];}/g,";")}function jI(e){return lt(e)==="object"&&e&&(II in e||Gw in e)}function u1(e,t,n){if(!t)return e;var r=".".concat(t),o=n==="low"?":where(".concat(r,")"):r,l=e.split(",").map(function(c){var d,m=c.trim().split(/\s+/),v=m[0]||"",g=((d=v.match(/^\w+/))===null||d===void 0?void 0:d[0])||"";return v="".concat(g).concat(o).concat(v.slice(g.length)),[v].concat(Te(m.slice(1))).join(" ")});return l.join(",")}var PI=function e(t){var n=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},r=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{root:!0,parentSelectors:[]},o=r.root,l=r.injectHash,c=r.parentSelectors,d=n.hashId,m=n.layer;n.path;var v=n.hashPriority,g=n.transformers,p=g===void 0?[]:g;n.linters;var y="",E={};function C(w){var $=w.getName(d);if(!E[$]){var O=e(w.style,n,{root:!1,parentSelectors:c}),T=de(O,1),R=T[0];E[$]="@keyframes ".concat(w.getName(d)).concat(R)}}function x(w){var $=arguments.length>1&&arguments[1]!==void 0?arguments[1]:[];return w.forEach(function(O){Array.isArray(O)?x(O,$):O&&$.push(O)}),$}var b=x(Array.isArray(t)?t:[t]);return b.forEach(function(w){var $=typeof w=="string"&&!o?{}:w;if(typeof $=="string")y+="".concat($,`
`);else if($._keyframe)C($);else{var O=p.reduce(function(T,R){var M;return(R==null||(M=R.visit)===null||M===void 0?void 0:M.call(R,T))||T},$);Object.keys(O).forEach(function(T){var R=O[T];if(lt(R)==="object"&&R&&(T!=="animationName"||!R._keyframe)&&!jI(R)){var M=!1,N=T.trim(),z=!1;(o||l)&&d?N.startsWith("@")?M=!0:N==="&"?N=u1("",d,v):N=u1(T,d,v):o&&!d&&(N==="&"||N==="")&&(N="",z=!0);var P=e(R,n,{root:z,injectHash:M,parentSelectors:[].concat(Te(c),[N])}),D=de(P,2),F=D[0],W=D[1];E=Z(Z({},E),W),y+="".concat(N).concat(F)}else{let K=function(_,j){var B=_.replace(/[A-Z]/g,function(I){return"-".concat(I.toLowerCase())}),k=j;!dI[_]&&typeof k=="number"&&k!==0&&(k="".concat(k,"px")),_==="animationName"&&j!==null&&j!==void 0&&j._keyframe&&(C(j),k=j.getName(d)),y+="".concat(B,":").concat(k,";")};var V,q=(V=R==null?void 0:R.value)!==null&&V!==void 0?V:R;lt(R)==="object"&&R!==null&&R!==void 0&&R[Gw]&&Array.isArray(q)?q.forEach(function(_){K(T,_)}):K(T,q)}})}}),o?m&&(y&&(y="@layer ".concat(m.name," {").concat(y,"}")),m.dependencies&&(E["@layer ".concat(m.name)]=m.dependencies.map(function(w){return"@layer ".concat(w,", ").concat(m.name,";")}).join(`
`))):y="{".concat(y,"}"),[y,E]};function Kw(e,t){return $c("".concat(e.join("%")).concat(t))}function zI(){return null}var Xw="style";function Yh(e,t){var n=e.token,r=e.path,o=e.hashId,l=e.layer,c=e.nonce,d=e.clientOnly,m=e.order,v=m===void 0?0:m,g=u.useContext(Wc),p=g.autoClear;g.mock;var y=g.defaultCache,E=g.hashPriority,C=g.container,x=g.ssrInline,b=g.transformers,w=g.linters,$=g.cache,O=g.layer,T=n._tokenKey,R=[T];O&&R.push("layer"),R.push.apply(R,Te(r));var M=Gh,N=Qp(Xw,R,function(){var W=R.join("|");if(NI(W)){var V=AI(W),q=de(V,2),K=q[0],_=q[1];if(K)return[K,T,_,{},d,v]}var j=t(),B=PI(j,{hashId:o,hashPriority:E,layer:O?l:void 0,path:r.join("-"),transformers:b,linters:w}),k=de(B,2),I=k[0],H=k[1],G=Af(I),X=Kw(R,G);return[G,T,X,H,d,v]},function(W,V){var q=de(W,3),K=q[2];(V||p)&&Gh&&Cc(K,{mark:Ma})},function(W){var V=de(W,4),q=V[0];V[1];var K=V[2],_=V[3];if(M&&q!==Ww){var j={mark:Ma,prepend:O?!1:"queue",attachTo:C,priority:v},B=typeof c=="function"?c():c;B&&(j.csp={nonce:B});var k=[],I=[];Object.keys(_).forEach(function(G){G.startsWith("@layer")?k.push(G):I.push(G)}),k.forEach(function(G){po(Af(_[G]),"_layer-".concat(G),Z(Z({},j),{},{prepend:!0}))});var H=po(q,K,j);H[ti]=$.instanceId,H.setAttribute(Vl,T),I.forEach(function(G){po(Af(_[G]),"_effect-".concat(G),j)})}}),z=de(N,3),P=z[0],D=z[1],F=z[2];return function(W){var V;if(!x||M||!y)V=u.createElement(zI,null);else{var q;V=u.createElement("style",Ie({},(q={},Y(q,Vl,D),Y(q,Ma,F),q),{dangerouslySetInnerHTML:{__html:P}}))}return u.createElement(u.Fragment,null,V,W)}}var DI=function(t,n,r){var o=de(t,6),l=o[0],c=o[1],d=o[2],m=o[3],v=o[4],g=o[5],p=r||{},y=p.plain;if(v)return null;var E=l,C={"data-rc-order":"prependQueue","data-rc-priority":"".concat(g)};return E=Qf(l,c,d,C,y),m&&Object.keys(m).forEach(function(x){if(!n[x]){n[x]=!0;var b=Af(m[x]),w=Qf(b,c,"_effect-".concat(x),C,y);x.startsWith("@layer")?E=w+E:E+=w}}),[g,d,E]},Yw="cssVar",BI=function(t,n){var r=t.key,o=t.prefix,l=t.unitless,c=t.ignore,d=t.token,m=t.scope,v=m===void 0?"":m,g=u.useContext(Wc),p=g.cache.instanceId,y=g.container,E=d._tokenKey,C=[].concat(Te(t.path),[r,v,E]),x=Qp(Yw,C,function(){var b=n(),w=zw(b,r,{prefix:o,unitless:l,ignore:c,scope:v}),$=de(w,2),O=$[0],T=$[1],R=Kw(C,T);return[O,T,R,r]},function(b){var w=de(b,3),$=w[2];Gh&&Cc($,{mark:Ma})},function(b){var w=de(b,3),$=w[1],O=w[2];if($){var T=po($,O,{mark:Ma,prepend:"queue",attachTo:y,priority:-999});T[ti]=p,T.setAttribute(Vl,r)}});return x},LI=function(t,n,r){var o=de(t,4),l=o[1],c=o[2],d=o[3],m=r||{},v=m.plain;if(!l)return null;var g=-999,p={"data-rc-order":"prependQueue","data-rc-priority":"".concat(g)},y=Qf(l,d,c,p,v);return[g,c,y]},rc;rc={},Y(rc,Xw,DI),Y(rc,Bw,fI),Y(rc,Yw,LI);var Pt=function(){function e(t,n){qn(this,e),Y(this,"name",void 0),Y(this,"style",void 0),Y(this,"_keyframe",!0),this.name=t,this.style=n}return Wn(e,[{key:"getName",value:function(){var n=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"";return n?"".concat(n,"-").concat(this.name):this.name}}]),e}();function Rl(e){return e.notSplit=!0,e}Rl(["borderTop","borderBottom"]),Rl(["borderTop"]),Rl(["borderBottom"]),Rl(["borderLeft","borderRight"]),Rl(["borderLeft"]),Rl(["borderRight"]);function Qw(e){return sw(e)||Nw(e)||Up(e)||cw()}function ba(e,t){for(var n=e,r=0;r<t.length;r+=1){if(n==null)return;n=n[t[r]]}return n}function Zw(e,t,n,r){if(!t.length)return n;var o=Qw(t),l=o[0],c=o.slice(1),d;return!e&&typeof l=="number"?d=[]:Array.isArray(e)?d=Te(e):d=Z({},e),r&&n===void 0&&c.length===1?delete d[l][c[0]]:d[l]=Zw(d[l],c,n,r),d}function ya(e,t,n){var r=arguments.length>3&&arguments[3]!==void 0?arguments[3]:!1;return t.length&&r&&n===void 0&&!ba(e,t.slice(0,-1))?e:Zw(e,t,n,r)}function HI(e){return lt(e)==="object"&&e!==null&&Object.getPrototypeOf(e)===Object.prototype}function f1(e){return Array.isArray(e)?[]:{}}var FI=typeof Reflect>"u"?Object.keys:Reflect.ownKeys;function Pl(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];var r=f1(t[0]);return t.forEach(function(o){function l(c,d){var m=new Set(d),v=ba(o,c),g=Array.isArray(v);if(g||HI(v)){if(!m.has(v)){m.add(v);var p=ba(r,c);g?r=ya(r,c,[]):(!p||lt(p)!=="object")&&(r=ya(r,c,f1(v))),FI(v).forEach(function(y){l([].concat(Te(c),[y]),m)})}}else r=ya(r,c,v)}l([])}),r}function VI(){}const UI=u.createContext({}),kc=()=>{const e=()=>{};return e.deprecated=VI,e},Jw=u.createContext(void 0);var qI={items_per_page:"/ page",jump_to:"Go to",jump_to_confirm:"confirm",page:"Page",prev_page:"Previous Page",next_page:"Next Page",prev_5:"Previous 5 Pages",next_5:"Next 5 Pages",prev_3:"Previous 3 Pages",next_3:"Next 3 Pages",page_size:"Page Size"},WI={yearFormat:"YYYY",dayFormat:"D",cellMeridiemFormat:"A",monthBeforeYear:!0},kI=Z(Z({},WI),{},{locale:"en_US",today:"Today",now:"Now",backToToday:"Back to today",ok:"OK",clear:"Clear",week:"Week",month:"Month",year:"Year",timeSelect:"select time",dateSelect:"select date",weekSelect:"Choose a week",monthSelect:"Choose a month",yearSelect:"Choose a year",decadeSelect:"Choose a decade",dateFormat:"M/D/YYYY",dateTimeFormat:"M/D/YYYY HH:mm:ss",previousMonth:"Previous month (PageUp)",nextMonth:"Next month (PageDown)",previousYear:"Last year (Control + left)",nextYear:"Next year (Control + right)",previousDecade:"Last decade",nextDecade:"Next decade",previousCentury:"Last century",nextCentury:"Next century"});const e$={placeholder:"Select time",rangePlaceholder:["Start time","End time"]},d1={lang:Object.assign({placeholder:"Select date",yearPlaceholder:"Select year",quarterPlaceholder:"Select quarter",monthPlaceholder:"Select month",weekPlaceholder:"Select week",rangePlaceholder:["Start date","End date"],rangeYearPlaceholder:["Start year","End year"],rangeQuarterPlaceholder:["Start quarter","End quarter"],rangeMonthPlaceholder:["Start month","End month"],rangeWeekPlaceholder:["Start week","End week"]},kI),timePickerLocale:Object.assign({},e$)},kr="${label} is not a valid ${type}",So={locale:"en",Pagination:qI,DatePicker:d1,TimePicker:e$,Calendar:d1,global:{placeholder:"Please select"},Table:{filterTitle:"Filter menu",filterConfirm:"OK",filterReset:"Reset",filterEmptyText:"No filters",filterCheckAll:"Select all items",filterSearchPlaceholder:"Search in filters",emptyText:"No data",selectAll:"Select current page",selectInvert:"Invert current page",selectNone:"Clear all data",selectionAll:"Select all data",sortTitle:"Sort",expand:"Expand row",collapse:"Collapse row",triggerDesc:"Click to sort descending",triggerAsc:"Click to sort ascending",cancelSort:"Click to cancel sorting"},Tour:{Next:"Next",Previous:"Previous",Finish:"Finish"},Modal:{okText:"OK",cancelText:"Cancel",justOkText:"OK"},Popconfirm:{okText:"OK",cancelText:"Cancel"},Transfer:{titles:["",""],searchPlaceholder:"Search here",itemUnit:"item",itemsUnit:"items",remove:"Remove",selectCurrent:"Select current page",removeCurrent:"Remove current page",selectAll:"Select all data",deselectAll:"Deselect all data",removeAll:"Remove all data",selectInvert:"Invert current page"},Upload:{uploading:"Uploading...",removeFile:"Remove file",uploadError:"Upload error",previewFile:"Preview file",downloadFile:"Download file"},Empty:{description:"No data"},Icon:{icon:"icon"},Text:{edit:"Edit",copy:"Copy",copied:"Copied",expand:"Expand",collapse:"Collapse"},Form:{optional:"(optional)",defaultValidateMessages:{default:"Field validation error for ${label}",required:"Please enter ${label}",enum:"${label} must be one of [${enum}]",whitespace:"${label} cannot be a blank character",date:{format:"${label} date format is invalid",parse:"${label} cannot be converted to a date",invalid:"${label} is an invalid date"},types:{string:kr,method:kr,array:kr,object:kr,number:kr,date:kr,boolean:kr,integer:kr,float:kr,regexp:kr,email:kr,url:kr,hex:kr},string:{len:"${label} must be ${len} characters",min:"${label} must be at least ${min} characters",max:"${label} must be up to ${max} characters",range:"${label} must be between ${min}-${max} characters"},number:{len:"${label} must be equal to ${len}",min:"${label} must be minimum ${min}",max:"${label} must be maximum ${max}",range:"${label} must be between ${min}-${max}"},array:{len:"Must be ${len} ${label}",min:"At least ${min} ${label}",max:"At most ${max} ${label}",range:"The amount of ${label} must be between ${min}-${max}"},pattern:{mismatch:"${label} does not match the pattern ${pattern}"}}},Image:{preview:"Preview"},QRCode:{expired:"QR code expired",refresh:"Refresh",scanned:"Scanned"},ColorPicker:{presetEmpty:"Empty",transparent:"Transparent",singleColor:"Single",gradientColor:"Gradient"}};let If=Object.assign({},So.Modal),jf=[];const m1=()=>jf.reduce((e,t)=>Object.assign(Object.assign({},e),t),So.Modal);function GI(e){if(e){const t=Object.assign({},e);return jf.push(t),If=m1(),()=>{jf=jf.filter(n=>n!==t),If=m1()}}If=Object.assign({},So.Modal)}function t$(){return If}const e0=u.createContext(void 0),To=(e,t)=>{const n=u.useContext(e0),r=u.useMemo(()=>{var l;const c=t||So[e],d=(l=n==null?void 0:n[e])!==null&&l!==void 0?l:{};return Object.assign(Object.assign({},typeof c=="function"?c():c),d||{})},[e,t,n]),o=u.useMemo(()=>{const l=n==null?void 0:n.locale;return n!=null&&n.exist&&!l?So.locale:l},[n]);return[r,o]},KI="internalMark",XI=e=>{const{locale:t={},children:n,_ANT_MARK__:r}=e;u.useEffect(()=>GI(t==null?void 0:t.Modal),[t]);const o=u.useMemo(()=>Object.assign(Object.assign({},t),{exist:!0}),[t]);return u.createElement(e0.Provider,{value:o},n)},t0={blue:"#1677FF",purple:"#722ED1",cyan:"#13C2C2",green:"#52C41A",magenta:"#EB2F96",pink:"#EB2F96",red:"#F5222D",orange:"#FA8C16",yellow:"#FADB14",volcano:"#FA541C",geekblue:"#2F54EB",gold:"#FAAD14",lime:"#A0D911"},Wl=Object.assign(Object.assign({},t0),{colorPrimary:"#1677ff",colorSuccess:"#52c41a",colorWarning:"#faad14",colorError:"#ff4d4f",colorInfo:"#1677ff",colorLink:"",colorTextBase:"",colorBgBase:"",fontFamily:`-apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, 'Helvetica Neue', Arial,
'Noto Sans', sans-serif, 'Apple Color Emoji', 'Segoe UI Emoji', 'Segoe UI Symbol',
'Noto Color Emoji'`,fontFamilyCode:"'SFMono-Regular', Consolas, 'Liberation Mono', Menlo, Courier, monospace",fontSize:14,lineWidth:1,lineType:"solid",motionUnit:.1,motionBase:0,motionEaseOutCirc:"cubic-bezier(0.08, 0.82, 0.17, 1)",motionEaseInOutCirc:"cubic-bezier(0.78, 0.14, 0.15, 0.86)",motionEaseOut:"cubic-bezier(0.215, 0.61, 0.355, 1)",motionEaseInOut:"cubic-bezier(0.645, 0.045, 0.355, 1)",motionEaseOutBack:"cubic-bezier(0.12, 0.4, 0.29, 1.46)",motionEaseInBack:"cubic-bezier(0.71, -0.46, 0.88, 0.6)",motionEaseInQuint:"cubic-bezier(0.755, 0.05, 0.855, 0.06)",motionEaseOutQuint:"cubic-bezier(0.23, 1, 0.32, 1)",borderRadius:6,sizeUnit:4,sizeStep:4,sizePopupArrow:16,controlHeight:32,zIndexBase:0,zIndexPopupBase:1e3,opacityImage:1,wireframe:!1,motion:!0});function n$(e,t){let{generateColorPalettes:n,generateNeutralColorPalettes:r}=t;const{colorSuccess:o,colorWarning:l,colorError:c,colorInfo:d,colorPrimary:m,colorBgBase:v,colorTextBase:g}=e,p=n(m),y=n(o),E=n(l),C=n(c),x=n(d),b=r(v,g),w=e.colorLink||e.colorInfo,$=n(w),O=new Jt(C[1]).mix(new Jt(C[3]),50).toHexString();return Object.assign(Object.assign({},b),{colorPrimaryBg:p[1],colorPrimaryBgHover:p[2],colorPrimaryBorder:p[3],colorPrimaryBorderHover:p[4],colorPrimaryHover:p[5],colorPrimary:p[6],colorPrimaryActive:p[7],colorPrimaryTextHover:p[8],colorPrimaryText:p[9],colorPrimaryTextActive:p[10],colorSuccessBg:y[1],colorSuccessBgHover:y[2],colorSuccessBorder:y[3],colorSuccessBorderHover:y[4],colorSuccessHover:y[4],colorSuccess:y[6],colorSuccessActive:y[7],colorSuccessTextHover:y[8],colorSuccessText:y[9],colorSuccessTextActive:y[10],colorErrorBg:C[1],colorErrorBgHover:C[2],colorErrorBgFilledHover:O,colorErrorBgActive:C[3],colorErrorBorder:C[3],colorErrorBorderHover:C[4],colorErrorHover:C[5],colorError:C[6],colorErrorActive:C[7],colorErrorTextHover:C[8],colorErrorText:C[9],colorErrorTextActive:C[10],colorWarningBg:E[1],colorWarningBgHover:E[2],colorWarningBorder:E[3],colorWarningBorderHover:E[4],colorWarningHover:E[4],colorWarning:E[6],colorWarningActive:E[7],colorWarningTextHover:E[8],colorWarningText:E[9],colorWarningTextActive:E[10],colorInfoBg:x[1],colorInfoBgHover:x[2],colorInfoBorder:x[3],colorInfoBorderHover:x[4],colorInfoHover:x[4],colorInfo:x[6],colorInfoActive:x[7],colorInfoTextHover:x[8],colorInfoText:x[9],colorInfoTextActive:x[10],colorLinkHover:$[4],colorLink:$[6],colorLinkActive:$[7],colorBgMask:new Jt("#000").setA(.45).toRgbString(),colorWhite:"#fff"})}const YI=e=>{let t=e,n=e,r=e,o=e;return e<6&&e>=5?t=e+1:e<16&&e>=6?t=e+2:e>=16&&(t=16),e<7&&e>=5?n=4:e<8&&e>=7?n=5:e<14&&e>=8?n=6:e<16&&e>=14?n=7:e>=16&&(n=8),e<6&&e>=2?r=1:e>=6&&(r=2),e>4&&e<8?o=4:e>=8&&(o=6),{borderRadius:e,borderRadiusXS:r,borderRadiusSM:n,borderRadiusLG:t,borderRadiusOuter:o}};function QI(e){const{motionUnit:t,motionBase:n,borderRadius:r,lineWidth:o}=e;return Object.assign({motionDurationFast:`${(n+t).toFixed(1)}s`,motionDurationMid:`${(n+t*2).toFixed(1)}s`,motionDurationSlow:`${(n+t*3).toFixed(1)}s`,lineWidthBold:o+1},YI(r))}const r$=e=>{const{controlHeight:t}=e;return{controlHeightSM:t*.75,controlHeightXS:t*.5,controlHeightLG:t*1.25}};function Pf(e){return(e+8)/e}function ZI(e){const t=Array.from({length:10}).map((n,r)=>{const o=r-1,l=e*Math.pow(Math.E,o/5),c=r>1?Math.floor(l):Math.ceil(l);return Math.floor(c/2)*2});return t[1]=e,t.map(n=>({size:n,lineHeight:Pf(n)}))}const a$=e=>{const t=ZI(e),n=t.map(g=>g.size),r=t.map(g=>g.lineHeight),o=n[1],l=n[0],c=n[2],d=r[1],m=r[0],v=r[2];return{fontSizeSM:l,fontSize:o,fontSizeLG:c,fontSizeXL:n[3],fontSizeHeading1:n[6],fontSizeHeading2:n[5],fontSizeHeading3:n[4],fontSizeHeading4:n[3],fontSizeHeading5:n[2],lineHeight:d,lineHeightLG:v,lineHeightSM:m,fontHeight:Math.round(d*o),fontHeightLG:Math.round(v*c),fontHeightSM:Math.round(m*l),lineHeightHeading1:r[6],lineHeightHeading2:r[5],lineHeightHeading3:r[4],lineHeightHeading4:r[3],lineHeightHeading5:r[2]}};function JI(e){const{sizeUnit:t,sizeStep:n}=e;return{sizeXXL:t*(n+8),sizeXL:t*(n+4),sizeLG:t*(n+2),sizeMD:t*(n+1),sizeMS:t*n,size:t*n,sizeSM:t*(n-1),sizeXS:t*(n-2),sizeXXS:t*(n-3)}}const ma=(e,t)=>new Jt(e).setA(t).toRgbString(),ac=(e,t)=>new Jt(e).darken(t).toHexString(),ej=e=>{const t=Bi(e);return{1:t[0],2:t[1],3:t[2],4:t[3],5:t[4],6:t[5],7:t[6],8:t[4],9:t[5],10:t[6]}},tj=(e,t)=>{const n=e||"#fff",r=t||"#000";return{colorBgBase:n,colorTextBase:r,colorText:ma(r,.88),colorTextSecondary:ma(r,.65),colorTextTertiary:ma(r,.45),colorTextQuaternary:ma(r,.25),colorFill:ma(r,.15),colorFillSecondary:ma(r,.06),colorFillTertiary:ma(r,.04),colorFillQuaternary:ma(r,.02),colorBgSolid:ma(r,1),colorBgSolidHover:ma(r,.75),colorBgSolidActive:ma(r,.95),colorBgLayout:ac(n,4),colorBgContainer:ac(n,0),colorBgElevated:ac(n,0),colorBgSpotlight:ma(r,.85),colorBgBlur:"transparent",colorBorder:ac(n,15),colorBorderSecondary:ac(n,6)}};function Td(e){Sg.pink=Sg.magenta,Cg.pink=Cg.magenta;const t=Object.keys(t0).map(n=>{const r=e[n]===Sg[n]?Cg[n]:Bi(e[n]);return Array.from({length:10},()=>1).reduce((o,l,c)=>(o[`${n}-${c+1}`]=r[c],o[`${n}${c+1}`]=r[c],o),{})}).reduce((n,r)=>(n=Object.assign(Object.assign({},n),r),n),{});return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},e),t),n$(e,{generateColorPalettes:ej,generateNeutralColorPalettes:tj})),a$(e.fontSize)),JI(e)),r$(e)),QI(e))}const n0=Yf(Td),Tc={token:Wl,override:{override:Wl},hashed:!0},r0=oe.createContext(Tc),Mc="ant",Md="anticon",nj=["outlined","borderless","filled","underlined"],rj=(e,t)=>t||(e?`${Mc}-${e}`:Mc),vt=u.createContext({getPrefixCls:rj,iconPrefixCls:Md}),{Consumer:oW}=vt,v1={};function zr(e){const t=u.useContext(vt),{getPrefixCls:n,direction:r,getPopupContainer:o}=t,l=t[e];return Object.assign(Object.assign({classNames:v1,styles:v1},l),{getPrefixCls:n,direction:r,getPopupContainer:o})}const aj=`-ant-${Date.now()}-${Math.random()}`;function oj(e,t){const n={},r=(c,d)=>{let m=c.clone();return m=(d==null?void 0:d(m))||m,m.toRgbString()},o=(c,d)=>{const m=new Jt(c),v=Bi(m.toRgbString());n[`${d}-color`]=r(m),n[`${d}-color-disabled`]=v[1],n[`${d}-color-hover`]=v[4],n[`${d}-color-active`]=v[6],n[`${d}-color-outline`]=m.clone().setA(.2).toRgbString(),n[`${d}-color-deprecated-bg`]=v[0],n[`${d}-color-deprecated-border`]=v[2]};if(t.primaryColor){o(t.primaryColor,"primary");const c=new Jt(t.primaryColor),d=Bi(c.toRgbString());d.forEach((v,g)=>{n[`primary-${g+1}`]=v}),n["primary-color-deprecated-l-35"]=r(c,v=>v.lighten(35)),n["primary-color-deprecated-l-20"]=r(c,v=>v.lighten(20)),n["primary-color-deprecated-t-20"]=r(c,v=>v.tint(20)),n["primary-color-deprecated-t-50"]=r(c,v=>v.tint(50)),n["primary-color-deprecated-f-12"]=r(c,v=>v.setA(v.a*.12));const m=new Jt(d[0]);n["primary-color-active-deprecated-f-30"]=r(m,v=>v.setA(v.a*.3)),n["primary-color-active-deprecated-d-02"]=r(m,v=>v.darken(2))}return t.successColor&&o(t.successColor,"success"),t.warningColor&&o(t.warningColor,"warning"),t.errorColor&&o(t.errorColor,"error"),t.infoColor&&o(t.infoColor,"info"),`
  :root {
    ${Object.keys(n).map(c=>`--${e}-${c}: ${n[c]};`).join(`
`)}
  }
  `.trim()}function ij(e,t){const n=oj(e,t);Cr()&&po(n,`${aj}-dynamic-theme`)}const Co=u.createContext(!1),a0=e=>{let{children:t,disabled:n}=e;const r=u.useContext(Co);return u.createElement(Co.Provider,{value:n??r},t)},Li=u.createContext(void 0),lj=e=>{let{children:t,size:n}=e;const r=u.useContext(Li);return u.createElement(Li.Provider,{value:n||r},t)};function sj(){const e=u.useContext(Co),t=u.useContext(Li);return{componentDisabled:e,componentSize:t}}var o$=Wn(function e(){qn(this,e)}),i$="CALC_UNIT",cj=new RegExp(i$,"g");function Rg(e){return typeof e=="number"?"".concat(e).concat(i$):e}var uj=function(e){$o(n,e);var t=Oo(n);function n(r,o){var l;qn(this,n),l=t.call(this),Y(_t(l),"result",""),Y(_t(l),"unitlessCssVar",void 0),Y(_t(l),"lowPriority",void 0);var c=lt(r);return l.unitlessCssVar=o,r instanceof n?l.result="(".concat(r.result,")"):c==="number"?l.result=Rg(r):c==="string"&&(l.result=r),l}return Wn(n,[{key:"add",value:function(o){return o instanceof n?this.result="".concat(this.result," + ").concat(o.getResult()):(typeof o=="number"||typeof o=="string")&&(this.result="".concat(this.result," + ").concat(Rg(o))),this.lowPriority=!0,this}},{key:"sub",value:function(o){return o instanceof n?this.result="".concat(this.result," - ").concat(o.getResult()):(typeof o=="number"||typeof o=="string")&&(this.result="".concat(this.result," - ").concat(Rg(o))),this.lowPriority=!0,this}},{key:"mul",value:function(o){return this.lowPriority&&(this.result="(".concat(this.result,")")),o instanceof n?this.result="".concat(this.result," * ").concat(o.getResult(!0)):(typeof o=="number"||typeof o=="string")&&(this.result="".concat(this.result," * ").concat(o)),this.lowPriority=!1,this}},{key:"div",value:function(o){return this.lowPriority&&(this.result="(".concat(this.result,")")),o instanceof n?this.result="".concat(this.result," / ").concat(o.getResult(!0)):(typeof o=="number"||typeof o=="string")&&(this.result="".concat(this.result," / ").concat(o)),this.lowPriority=!1,this}},{key:"getResult",value:function(o){return this.lowPriority||o?"(".concat(this.result,")"):this.result}},{key:"equal",value:function(o){var l=this,c=o||{},d=c.unit,m=!0;return typeof d=="boolean"?m=d:Array.from(this.unitlessCssVar).some(function(v){return l.result.includes(v)})&&(m=!1),this.result=this.result.replace(cj,m?"px":""),typeof this.lowPriority<"u"?"calc(".concat(this.result,")"):this.result}}]),n}(o$),fj=function(e){$o(n,e);var t=Oo(n);function n(r){var o;return qn(this,n),o=t.call(this),Y(_t(o),"result",0),r instanceof n?o.result=r.result:typeof r=="number"&&(o.result=r),o}return Wn(n,[{key:"add",value:function(o){return o instanceof n?this.result+=o.result:typeof o=="number"&&(this.result+=o),this}},{key:"sub",value:function(o){return o instanceof n?this.result-=o.result:typeof o=="number"&&(this.result-=o),this}},{key:"mul",value:function(o){return o instanceof n?this.result*=o.result:typeof o=="number"&&(this.result*=o),this}},{key:"div",value:function(o){return o instanceof n?this.result/=o.result:typeof o=="number"&&(this.result/=o),this}},{key:"equal",value:function(){return this.result}}]),n}(o$),dj=function(t,n){var r=t==="css"?uj:fj;return function(o){return new r(o,n)}},g1=function(t,n){return"".concat([n,t.replace(/([A-Z]+)([A-Z][a-z]+)/g,"$1-$2").replace(/([a-z])([A-Z])/g,"$1-$2")].filter(Boolean).join("-"))};function jn(e){var t=u.useRef();t.current=e;var n=u.useCallback(function(){for(var r,o=arguments.length,l=new Array(o),c=0;c<o;c++)l[c]=arguments[c];return(r=t.current)===null||r===void 0?void 0:r.call.apply(r,[t].concat(l))},[]);return n}function Hi(e){var t=u.useRef(!1),n=u.useState(e),r=de(n,2),o=r[0],l=r[1];u.useEffect(function(){return t.current=!1,function(){t.current=!0}},[]);function c(d,m){m&&t.current||l(d)}return[o,c]}function Tg(e){return e!==void 0}function gr(e,t){var n=t||{},r=n.defaultValue,o=n.value,l=n.onChange,c=n.postState,d=Hi(function(){return Tg(o)?o:Tg(r)?typeof r=="function"?r():r:typeof e=="function"?e():e}),m=de(d,2),v=m[0],g=m[1],p=o!==void 0?o:v,y=c?c(p):p,E=jn(l),C=Hi([p]),x=de(C,2),b=x[0],w=x[1];o1(function(){var O=b[0];v!==O&&E(v,O)},[b]),o1(function(){Tg(o)||g(o)},[o]);var $=jn(function(O,T){g(O,T),w([p],T)});return[y,$]}function h1(e,t,n,r){var o=Z({},t[e]);if(r!=null&&r.deprecatedTokens){var l=r.deprecatedTokens;l.forEach(function(d){var m=de(d,2),v=m[0],g=m[1];if(o!=null&&o[v]||o!=null&&o[g]){var p;(p=o[g])!==null&&p!==void 0||(o[g]=o==null?void 0:o[v])}})}var c=Z(Z({},n),o);return Object.keys(c).forEach(function(d){c[d]===t[d]&&delete c[d]}),c}var l$=typeof CSSINJS_STATISTIC<"u",Qh=!0;function kt(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];if(!l$)return Object.assign.apply(Object,[{}].concat(t));Qh=!1;var r={};return t.forEach(function(o){if(lt(o)==="object"){var l=Object.keys(o);l.forEach(function(c){Object.defineProperty(r,c,{configurable:!0,enumerable:!0,get:function(){return o[c]}})})}}),Qh=!0,r}var p1={};function mj(){}var vj=function(t){var n,r=t,o=mj;return l$&&typeof Proxy<"u"&&(n=new Set,r=new Proxy(t,{get:function(c,d){if(Qh){var m;(m=n)===null||m===void 0||m.add(d)}return c[d]}}),o=function(c,d){var m;p1[c]={global:Array.from(n),component:Z(Z({},(m=p1[c])===null||m===void 0?void 0:m.component),d)}}),{token:r,keys:n,flush:o}};function y1(e,t,n){if(typeof n=="function"){var r;return n(kt(t,(r=t[e])!==null&&r!==void 0?r:{}))}return n??{}}function gj(e){return e==="js"?{max:Math.max,min:Math.min}:{max:function(){for(var n=arguments.length,r=new Array(n),o=0;o<n;o++)r[o]=arguments[o];return"max(".concat(r.map(function(l){return he(l)}).join(","),")")},min:function(){for(var n=arguments.length,r=new Array(n),o=0;o<n;o++)r[o]=arguments[o];return"min(".concat(r.map(function(l){return he(l)}).join(","),")")}}}var hj=1e3*60*10,pj=function(){function e(){qn(this,e),Y(this,"map",new Map),Y(this,"objectIDMap",new WeakMap),Y(this,"nextID",0),Y(this,"lastAccessBeat",new Map),Y(this,"accessBeat",0)}return Wn(e,[{key:"set",value:function(n,r){this.clear();var o=this.getCompositeKey(n);this.map.set(o,r),this.lastAccessBeat.set(o,Date.now())}},{key:"get",value:function(n){var r=this.getCompositeKey(n),o=this.map.get(r);return this.lastAccessBeat.set(r,Date.now()),this.accessBeat+=1,o}},{key:"getCompositeKey",value:function(n){var r=this,o=n.map(function(l){return l&&lt(l)==="object"?"obj_".concat(r.getObjectID(l)):"".concat(lt(l),"_").concat(l)});return o.join("|")}},{key:"getObjectID",value:function(n){if(this.objectIDMap.has(n))return this.objectIDMap.get(n);var r=this.nextID;return this.objectIDMap.set(n,r),this.nextID+=1,r}},{key:"clear",value:function(){var n=this;if(this.accessBeat>1e4){var r=Date.now();this.lastAccessBeat.forEach(function(o,l){r-o>hj&&(n.map.delete(l),n.lastAccessBeat.delete(l))}),this.accessBeat=0}}}]),e}(),b1=new pj;function yj(e,t){return oe.useMemo(function(){var n=b1.get(t);if(n)return n;var r=e();return b1.set(t,r),r},t)}var bj=function(){return{}};function Sj(e){var t=e.useCSP,n=t===void 0?bj:t,r=e.useToken,o=e.usePrefix,l=e.getResetStyles,c=e.getCommonStyle,d=e.getCompUnitless;function m(y,E,C,x){var b=Array.isArray(y)?y[0]:y;function w(z){return"".concat(String(b)).concat(z.slice(0,1).toUpperCase()).concat(z.slice(1))}var $=(x==null?void 0:x.unitless)||{},O=typeof d=="function"?d(y):{},T=Z(Z({},O),{},Y({},w("zIndexPopup"),!0));Object.keys($).forEach(function(z){T[w(z)]=$[z]});var R=Z(Z({},x),{},{unitless:T,prefixToken:w}),M=g(y,E,C,R),N=v(b,C,R);return function(z){var P=arguments.length>1&&arguments[1]!==void 0?arguments[1]:z,D=M(z,P),F=de(D,2),W=F[1],V=N(P),q=de(V,2),K=q[0],_=q[1];return[K,W,_]}}function v(y,E,C){var x=C.unitless,b=C.injectStyle,w=b===void 0?!0:b,$=C.prefixToken,O=C.ignore,T=function(N){var z=N.rootCls,P=N.cssVar,D=P===void 0?{}:P,F=r(),W=F.realToken;return BI({path:[y],prefix:D.prefix,key:D.key,unitless:x,ignore:O,token:W,scope:z},function(){var V=y1(y,W,E),q=h1(y,W,V,{deprecatedTokens:C==null?void 0:C.deprecatedTokens});return Object.keys(V).forEach(function(K){q[$(K)]=q[K],delete q[K]}),q}),null},R=function(N){var z=r(),P=z.cssVar;return[function(D){return w&&P?oe.createElement(oe.Fragment,null,oe.createElement(T,{rootCls:N,cssVar:P,component:y}),D):D},P==null?void 0:P.key]};return R}function g(y,E,C){var x=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{},b=Array.isArray(y)?y:[y,y],w=de(b,1),$=w[0],O=b.join("-"),T=e.layer||{name:"antd"};return function(R){var M=arguments.length>1&&arguments[1]!==void 0?arguments[1]:R,N=r(),z=N.theme,P=N.realToken,D=N.hashId,F=N.token,W=N.cssVar,V=o(),q=V.rootPrefixCls,K=V.iconPrefixCls,_=n(),j=W?"css":"js",B=yj(function(){var U=new Set;return W&&Object.keys(x.unitless||{}).forEach(function(J){U.add(Tf(J,W.prefix)),U.add(Tf(J,g1($,W.prefix)))}),dj(j,U)},[j,$,W==null?void 0:W.prefix]),k=gj(j),I=k.max,H=k.min,G={theme:z,token:F,hashId:D,nonce:function(){return _.nonce},clientOnly:x.clientOnly,layer:T,order:x.order||-999};typeof l=="function"&&Yh(Z(Z({},G),{},{clientOnly:!1,path:["Shared",q]}),function(){return l(F,{prefix:{rootPrefixCls:q,iconPrefixCls:K},csp:_})});var X=Yh(Z(Z({},G),{},{path:[O,R,K]}),function(){if(x.injectStyle===!1)return[];var U=vj(F),J=U.token,te=U.flush,ve=y1($,P,C),ne=".".concat(R),le=h1($,P,ve,{deprecatedTokens:x.deprecatedTokens});W&&ve&&lt(ve)==="object"&&Object.keys(ve).forEach(function(pe){ve[pe]="var(".concat(Tf(pe,g1($,W.prefix)),")")});var be=kt(J,{componentCls:ne,prefixCls:R,iconCls:".".concat(K),antCls:".".concat(q),calc:B,max:I,min:H},W?ve:le),fe=E(be,{hashId:D,prefixCls:R,rootPrefixCls:q,iconPrefixCls:K});te($,le);var ie=typeof c=="function"?c(be,R,M,x.resetFont):null;return[x.resetStyle===!1?null:ie,fe]});return[X,D]}}function p(y,E,C){var x=arguments.length>3&&arguments[3]!==void 0?arguments[3]:{},b=g(y,E,C,Z({resetStyle:!1,order:-998},x)),w=function(O){var T=O.prefixCls,R=O.rootCls,M=R===void 0?T:R;return b(T,M),null};return w}return{genStyleHooks:m,genSubStyleComponent:p,genComponentStyleHook:g}}const ai=["blue","purple","cyan","green","magenta","pink","red","orange","yellow","volcano","geekblue","lime","gold"],Cj="5.24.8";function Mg(e){return e>=0&&e<=255}function dc(e,t){const{r:n,g:r,b:o,a:l}=new Jt(e).toRgb();if(l<1)return e;const{r:c,g:d,b:m}=new Jt(t).toRgb();for(let v=.01;v<=1;v+=.01){const g=Math.round((n-c*(1-v))/v),p=Math.round((r-d*(1-v))/v),y=Math.round((o-m*(1-v))/v);if(Mg(g)&&Mg(p)&&Mg(y))return new Jt({r:g,g:p,b:y,a:Math.round(v*100)/100}).toRgbString()}return new Jt({r:n,g:r,b:o,a:1}).toRgbString()}var xj=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function o0(e){const{override:t}=e,n=xj(e,["override"]),r=Object.assign({},t);Object.keys(Wl).forEach(y=>{delete r[y]});const o=Object.assign(Object.assign({},n),r),l=480,c=576,d=768,m=992,v=1200,g=1600;if(o.motion===!1){const y="0s";o.motionDurationFast=y,o.motionDurationMid=y,o.motionDurationSlow=y}return Object.assign(Object.assign(Object.assign({},o),{colorFillContent:o.colorFillSecondary,colorFillContentHover:o.colorFill,colorFillAlter:o.colorFillQuaternary,colorBgContainerDisabled:o.colorFillTertiary,colorBorderBg:o.colorBgContainer,colorSplit:dc(o.colorBorderSecondary,o.colorBgContainer),colorTextPlaceholder:o.colorTextQuaternary,colorTextDisabled:o.colorTextQuaternary,colorTextHeading:o.colorText,colorTextLabel:o.colorTextSecondary,colorTextDescription:o.colorTextTertiary,colorTextLightSolid:o.colorWhite,colorHighlight:o.colorError,colorBgTextHover:o.colorFillSecondary,colorBgTextActive:o.colorFill,colorIcon:o.colorTextTertiary,colorIconHover:o.colorText,colorErrorOutline:dc(o.colorErrorBg,o.colorBgContainer),colorWarningOutline:dc(o.colorWarningBg,o.colorBgContainer),fontSizeIcon:o.fontSizeSM,lineWidthFocus:o.lineWidth*3,lineWidth:o.lineWidth,controlOutlineWidth:o.lineWidth*2,controlInteractiveSize:o.controlHeight/2,controlItemBgHover:o.colorFillTertiary,controlItemBgActive:o.colorPrimaryBg,controlItemBgActiveHover:o.colorPrimaryBgHover,controlItemBgActiveDisabled:o.colorFill,controlTmpOutline:o.colorFillQuaternary,controlOutline:dc(o.colorPrimaryBg,o.colorBgContainer),lineType:o.lineType,borderRadius:o.borderRadius,borderRadiusXS:o.borderRadiusXS,borderRadiusSM:o.borderRadiusSM,borderRadiusLG:o.borderRadiusLG,fontWeightStrong:600,opacityLoading:.65,linkDecoration:"none",linkHoverDecoration:"none",linkFocusDecoration:"none",controlPaddingHorizontal:12,controlPaddingHorizontalSM:8,paddingXXS:o.sizeXXS,paddingXS:o.sizeXS,paddingSM:o.sizeSM,padding:o.size,paddingMD:o.sizeMD,paddingLG:o.sizeLG,paddingXL:o.sizeXL,paddingContentHorizontalLG:o.sizeLG,paddingContentVerticalLG:o.sizeMS,paddingContentHorizontal:o.sizeMS,paddingContentVertical:o.sizeSM,paddingContentHorizontalSM:o.size,paddingContentVerticalSM:o.sizeXS,marginXXS:o.sizeXXS,marginXS:o.sizeXS,marginSM:o.sizeSM,margin:o.size,marginMD:o.sizeMD,marginLG:o.sizeLG,marginXL:o.sizeXL,marginXXL:o.sizeXXL,boxShadow:`
      0 6px 16px 0 rgba(0, 0, 0, 0.08),
      0 3px 6px -4px rgba(0, 0, 0, 0.12),
      0 9px 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowSecondary:`
      0 6px 16px 0 rgba(0, 0, 0, 0.08),
      0 3px 6px -4px rgba(0, 0, 0, 0.12),
      0 9px 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowTertiary:`
      0 1px 2px 0 rgba(0, 0, 0, 0.03),
      0 1px 6px -1px rgba(0, 0, 0, 0.02),
      0 2px 4px 0 rgba(0, 0, 0, 0.02)
    `,screenXS:l,screenXSMin:l,screenXSMax:c-1,screenSM:c,screenSMMin:c,screenSMMax:d-1,screenMD:d,screenMDMin:d,screenMDMax:m-1,screenLG:m,screenLGMin:m,screenLGMax:v-1,screenXL:v,screenXLMin:v,screenXLMax:g-1,screenXXL:g,screenXXLMin:g,boxShadowPopoverArrow:"2px 2px 5px rgba(0, 0, 0, 0.05)",boxShadowCard:`
      0 1px 2px -2px ${new Jt("rgba(0, 0, 0, 0.16)").toRgbString()},
      0 3px 6px 0 ${new Jt("rgba(0, 0, 0, 0.12)").toRgbString()},
      0 5px 12px 4px ${new Jt("rgba(0, 0, 0, 0.09)").toRgbString()}
    `,boxShadowDrawerRight:`
      -6px 0 16px 0 rgba(0, 0, 0, 0.08),
      -3px 0 6px -4px rgba(0, 0, 0, 0.12),
      -9px 0 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowDrawerLeft:`
      6px 0 16px 0 rgba(0, 0, 0, 0.08),
      3px 0 6px -4px rgba(0, 0, 0, 0.12),
      9px 0 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowDrawerUp:`
      0 6px 16px 0 rgba(0, 0, 0, 0.08),
      0 3px 6px -4px rgba(0, 0, 0, 0.12),
      0 9px 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowDrawerDown:`
      0 -6px 16px 0 rgba(0, 0, 0, 0.08),
      0 -3px 6px -4px rgba(0, 0, 0, 0.12),
      0 -9px 28px 8px rgba(0, 0, 0, 0.05)
    `,boxShadowTabsOverflowLeft:"inset 10px 0 8px -8px rgba(0, 0, 0, 0.08)",boxShadowTabsOverflowRight:"inset -10px 0 8px -8px rgba(0, 0, 0, 0.08)",boxShadowTabsOverflowTop:"inset 0 10px 8px -8px rgba(0, 0, 0, 0.08)",boxShadowTabsOverflowBottom:"inset 0 -10px 8px -8px rgba(0, 0, 0, 0.08)"}),r)}var S1=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const s$={lineHeight:!0,lineHeightSM:!0,lineHeightLG:!0,lineHeightHeading1:!0,lineHeightHeading2:!0,lineHeightHeading3:!0,lineHeightHeading4:!0,lineHeightHeading5:!0,opacityLoading:!0,fontWeightStrong:!0,zIndexPopupBase:!0,zIndexBase:!0,opacityImage:!0},Ej={size:!0,sizeSM:!0,sizeLG:!0,sizeMD:!0,sizeXS:!0,sizeXXS:!0,sizeMS:!0,sizeXL:!0,sizeXXL:!0,sizeUnit:!0,sizeStep:!0,motionBase:!0,motionUnit:!0},wj={screenXS:!0,screenXSMin:!0,screenXSMax:!0,screenSM:!0,screenSMMin:!0,screenSMMax:!0,screenMD:!0,screenMDMin:!0,screenMDMax:!0,screenLG:!0,screenLGMin:!0,screenLGMax:!0,screenXL:!0,screenXLMin:!0,screenXLMax:!0,screenXXL:!0,screenXXLMin:!0},c$=(e,t,n)=>{const r=n.getDerivativeToken(e),{override:o}=t,l=S1(t,["override"]);let c=Object.assign(Object.assign({},r),{override:o});return c=o0(c),l&&Object.entries(l).forEach(d=>{let[m,v]=d;const{theme:g}=v,p=S1(v,["theme"]);let y=p;g&&(y=c$(Object.assign(Object.assign({},c),p),{override:p},g)),c[m]=y}),c};function Rr(){const{token:e,hashed:t,theme:n,override:r,cssVar:o}=oe.useContext(r0),l=`${Cj}-${t||""}`,c=n||n0,[d,m,v]=uI(c,[Wl,e],{salt:l,override:r,getComputedToken:c$,formatToken:o0,cssVar:o&&{prefix:o.prefix,key:o.key,unitless:s$,ignore:Ej,preserve:wj}});return[c,v,t?m:"",d,o]}const Zf={overflow:"hidden",whiteSpace:"nowrap",textOverflow:"ellipsis"},Tr=function(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;return{boxSizing:"border-box",margin:0,padding:0,color:e.colorText,fontSize:e.fontSize,lineHeight:e.lineHeight,listStyle:"none",fontFamily:t?"inherit":e.fontFamily}},es=()=>({display:"inline-flex",alignItems:"center",color:"inherit",fontStyle:"normal",lineHeight:0,textAlign:"center",textTransform:"none",verticalAlign:"-0.125em",textRendering:"optimizeLegibility","-webkit-font-smoothing":"antialiased","-moz-osx-font-smoothing":"grayscale","> *":{lineHeight:1},svg:{display:"inline-block"}}),Jf=()=>({"&::before":{display:"table",content:'""'},"&::after":{display:"table",clear:"both",content:'""'}}),$j=e=>({a:{color:e.colorLink,textDecoration:e.linkDecoration,backgroundColor:"transparent",outline:"none",cursor:"pointer",transition:`color ${e.motionDurationSlow}`,"-webkit-text-decoration-skip":"objects","&:hover":{color:e.colorLinkHover},"&:active":{color:e.colorLinkActive},"&:active, &:hover":{textDecoration:e.linkHoverDecoration,outline:0},"&:focus":{textDecoration:e.linkFocusDecoration,outline:0},"&[disabled]":{color:e.colorTextDisabled,cursor:"not-allowed"}}}),Oj=(e,t,n,r)=>{const o=`[class^="${t}"], [class*=" ${t}"]`,l=n?`.${n}`:o,c={boxSizing:"border-box","&::before, &::after":{boxSizing:"border-box"}};let d={};return r!==!1&&(d={fontFamily:e.fontFamily,fontSize:e.fontSize}),{[l]:Object.assign(Object.assign(Object.assign({},d),c),{[o]:c})}},u$=(e,t)=>({outline:`${he(e.lineWidthFocus)} solid ${e.colorPrimaryBorder}`,outlineOffset:t??1,transition:"outline-offset 0s, outline 0s"}),Gc=(e,t)=>({"&:focus-visible":Object.assign({},u$(e,t))}),f$=e=>({[`.${e}`]:Object.assign(Object.assign({},es()),{[`.${e} .${e}-icon`]:{display:"block"}})}),d$=e=>Object.assign(Object.assign({color:e.colorLink,textDecoration:e.linkDecoration,outline:"none",cursor:"pointer",transition:`all ${e.motionDurationSlow}`,border:0,padding:0,background:"none",userSelect:"none"},Gc(e)),{"&:focus, &:hover":{color:e.colorLinkHover},"&:active":{color:e.colorLinkActive}}),{genStyleHooks:wn,genComponentStyleHook:Rj,genSubStyleComponent:_d}=Sj({usePrefix:()=>{const{getPrefixCls:e,iconPrefixCls:t}=u.useContext(vt);return{rootPrefixCls:e(),iconPrefixCls:t}},useToken:()=>{const[e,t,n,r,o]=Rr();return{theme:e,realToken:t,hashId:n,token:r,cssVar:o}},useCSP:()=>{const{csp:e}=u.useContext(vt);return e??{}},getResetStyles:(e,t)=>{var n;const r=$j(e);return[r,{"&":r},f$((n=t==null?void 0:t.prefix.iconPrefixCls)!==null&&n!==void 0?n:Md)]},getCommonStyle:Oj,getCompUnitless:()=>s$});function Tj(e,t){return ai.reduce((n,r)=>{const o=e[`${r}1`],l=e[`${r}3`],c=e[`${r}6`],d=e[`${r}7`];return Object.assign(Object.assign({},n),t(r,{lightColor:o,lightBorderColor:l,darkColor:c,textColor:d}))},{})}const Mj=(e,t)=>{const[n,r]=Rr();return Yh({token:r,hashId:"",path:["ant-design-icons",e],nonce:()=>t==null?void 0:t.nonce,layer:{name:"antd"}},()=>[f$(e)])},_j=Object.assign({},xd),{useId:C1}=_j,Nj=()=>"",Aj=typeof C1>"u"?Nj:C1;function Ij(e,t,n){var r;kc();const o=e||{},l=o.inherit===!1||!t?Object.assign(Object.assign({},Tc),{hashed:(r=t==null?void 0:t.hashed)!==null&&r!==void 0?r:Tc.hashed,cssVar:t==null?void 0:t.cssVar}):t,c=Aj();return qc(()=>{var d,m;if(!e)return t;const v=Object.assign({},l.components);Object.keys(e.components||{}).forEach(y=>{v[y]=Object.assign(Object.assign({},v[y]),e.components[y])});const g=`css-var-${c.replace(/:/g,"")}`,p=((d=o.cssVar)!==null&&d!==void 0?d:l.cssVar)&&Object.assign(Object.assign(Object.assign({prefix:n==null?void 0:n.prefixCls},typeof l.cssVar=="object"?l.cssVar:{}),typeof o.cssVar=="object"?o.cssVar:{}),{key:typeof o.cssVar=="object"&&((m=o.cssVar)===null||m===void 0?void 0:m.key)||g});return Object.assign(Object.assign(Object.assign({},l),o),{token:Object.assign(Object.assign({},l.token),o.token),components:v,cssVar:p})},[o,l],(d,m)=>d.some((v,g)=>{const p=m[g];return!Oc(v,p,!0)}))}var jj=["children"],m$=u.createContext({});function Pj(e){var t=e.children,n=wt(e,jj);return u.createElement(m$.Provider,{value:n},t)}var zj=function(e){$o(n,e);var t=Oo(n);function n(){return qn(this,n),t.apply(this,arguments)}return Wn(n,[{key:"render",value:function(){return this.props.children}}]),n}(u.Component);function Dj(e){var t=u.useReducer(function(d){return d+1},0),n=de(t,2),r=n[1],o=u.useRef(e),l=jn(function(){return o.current}),c=jn(function(d){o.current=typeof d=="function"?d(o.current):d,r()});return[l,c]}var Zo="none",df="appear",mf="enter",vf="leave",x1="none",$a="prepare",zl="start",Dl="active",i0="end",v$="prepared";function E1(e,t){var n={};return n[e.toLowerCase()]=t.toLowerCase(),n["Webkit".concat(e)]="webkit".concat(t),n["Moz".concat(e)]="moz".concat(t),n["ms".concat(e)]="MS".concat(t),n["O".concat(e)]="o".concat(t.toLowerCase()),n}function Bj(e,t){var n={animationend:E1("Animation","AnimationEnd"),transitionend:E1("Transition","TransitionEnd")};return e&&("AnimationEvent"in t||delete n.animationend.animation,"TransitionEvent"in t||delete n.transitionend.transition),n}var Lj=Bj(Cr(),typeof window<"u"?window:{}),g$={};if(Cr()){var Hj=document.createElement("div");g$=Hj.style}var gf={};function h$(e){if(gf[e])return gf[e];var t=Lj[e];if(t)for(var n=Object.keys(t),r=n.length,o=0;o<r;o+=1){var l=n[o];if(Object.prototype.hasOwnProperty.call(t,l)&&l in g$)return gf[e]=t[l],gf[e]}return""}var p$=h$("animationend"),y$=h$("transitionend"),b$=!!(p$&&y$),w1=p$||"animationend",$1=y$||"transitionend";function O1(e,t){if(!e)return null;if(lt(e)==="object"){var n=t.replace(/-\w/g,function(r){return r[1].toUpperCase()});return e[n]}return"".concat(e,"-").concat(t)}const Fj=function(e){var t=u.useRef();function n(o){o&&(o.removeEventListener($1,e),o.removeEventListener(w1,e))}function r(o){t.current&&t.current!==o&&n(t.current),o&&o!==t.current&&(o.addEventListener($1,e),o.addEventListener(w1,e),t.current=o)}return u.useEffect(function(){return function(){n(t.current)}},[]),[r,n]};var S$=Cr()?u.useLayoutEffect:u.useEffect;const Vj=function(){var e=u.useRef(null);function t(){gn.cancel(e.current)}function n(r){var o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:2;t();var l=gn(function(){o<=1?r({isCanceled:function(){return l!==e.current}}):n(r,o-1)});e.current=l}return u.useEffect(function(){return function(){t()}},[]),[n,t]};var Uj=[$a,zl,Dl,i0],qj=[$a,v$],C$=!1,Wj=!0;function x$(e){return e===Dl||e===i0}const kj=function(e,t,n){var r=Hi(x1),o=de(r,2),l=o[0],c=o[1],d=Vj(),m=de(d,2),v=m[0],g=m[1];function p(){c($a,!0)}var y=t?qj:Uj;return S$(function(){if(l!==x1&&l!==i0){var E=y.indexOf(l),C=y[E+1],x=n(l);x===C$?c(C,!0):C&&v(function(b){function w(){b.isCanceled()||c(C,!0)}x===!0?w():Promise.resolve(x).then(w)})}},[e,l]),u.useEffect(function(){return function(){g()}},[]),[p,l]};function Gj(e,t,n,r){var o=r.motionEnter,l=o===void 0?!0:o,c=r.motionAppear,d=c===void 0?!0:c,m=r.motionLeave,v=m===void 0?!0:m,g=r.motionDeadline,p=r.motionLeaveImmediately,y=r.onAppearPrepare,E=r.onEnterPrepare,C=r.onLeavePrepare,x=r.onAppearStart,b=r.onEnterStart,w=r.onLeaveStart,$=r.onAppearActive,O=r.onEnterActive,T=r.onLeaveActive,R=r.onAppearEnd,M=r.onEnterEnd,N=r.onLeaveEnd,z=r.onVisibleChanged,P=Hi(),D=de(P,2),F=D[0],W=D[1],V=Dj(Zo),q=de(V,2),K=q[0],_=q[1],j=Hi(null),B=de(j,2),k=B[0],I=B[1],H=K(),G=u.useRef(!1),X=u.useRef(null);function U(){return n()}var J=u.useRef(!1);function te(){_(Zo),I(null,!0)}var ve=jn(function(He){var Xe=K();if(Xe!==Zo){var xe=U();if(!(He&&!He.deadline&&He.target!==xe)){var Me=J.current,Ve;Xe===df&&Me?Ve=R==null?void 0:R(xe,He):Xe===mf&&Me?Ve=M==null?void 0:M(xe,He):Xe===vf&&Me&&(Ve=N==null?void 0:N(xe,He)),Me&&Ve!==!1&&te()}}}),ne=Fj(ve),le=de(ne,1),be=le[0],fe=function(Xe){switch(Xe){case df:return Y(Y(Y({},$a,y),zl,x),Dl,$);case mf:return Y(Y(Y({},$a,E),zl,b),Dl,O);case vf:return Y(Y(Y({},$a,C),zl,w),Dl,T);default:return{}}},ie=u.useMemo(function(){return fe(H)},[H]),pe=kj(H,!e,function(He){if(He===$a){var Xe=ie[$a];return Xe?Xe(U()):C$}if(ye in ie){var xe;I(((xe=ie[ye])===null||xe===void 0?void 0:xe.call(ie,U(),null))||null)}return ye===Dl&&H!==Zo&&(be(U()),g>0&&(clearTimeout(X.current),X.current=setTimeout(function(){ve({deadline:!0})},g))),ye===v$&&te(),Wj}),Re=de(pe,2),se=Re[0],ye=Re[1],$e=x$(ye);J.current=$e;var we=u.useRef(null);S$(function(){if(!(G.current&&we.current===t)){W(t);var He=G.current;G.current=!0;var Xe;!He&&t&&d&&(Xe=df),He&&t&&l&&(Xe=mf),(He&&!t&&v||!He&&p&&!t&&v)&&(Xe=vf);var xe=fe(Xe);Xe&&(e||xe[$a])?(_(Xe),se()):_(Zo),we.current=t}},[t]),u.useEffect(function(){(H===df&&!d||H===mf&&!l||H===vf&&!v)&&_(Zo)},[d,l,v]),u.useEffect(function(){return function(){G.current=!1,clearTimeout(X.current)}},[]);var Ne=u.useRef(!1);u.useEffect(function(){F&&(Ne.current=!0),F!==void 0&&H===Zo&&((Ne.current||F)&&(z==null||z(F)),Ne.current=!0)},[F,H]);var Ze=k;return ie[$a]&&ye===zl&&(Ze=Z({transition:"none"},Ze)),[H,ye,Ze,F??t]}function Kj(e){var t=e;lt(e)==="object"&&(t=e.transitionSupport);function n(o,l){return!!(o.motionName&&t&&l!==!1)}var r=u.forwardRef(function(o,l){var c=o.visible,d=c===void 0?!0:c,m=o.removeOnLeave,v=m===void 0?!0:m,g=o.forceRender,p=o.children,y=o.motionName,E=o.leavedClassName,C=o.eventProps,x=u.useContext(m$),b=x.motion,w=n(o,b),$=u.useRef(),O=u.useRef();function T(){try{return $.current instanceof HTMLElement?$.current:Rf(O.current)}catch{return null}}var R=Gj(w,d,T,o),M=de(R,4),N=M[0],z=M[1],P=M[2],D=M[3],F=u.useRef(D);D&&(F.current=!0);var W=u.useCallback(function(B){$.current=B,kp(l,B)},[l]),V,q=Z(Z({},C),{},{visible:d});if(!p)V=null;else if(N===Zo)D?V=p(Z({},q),W):!v&&F.current&&E?V=p(Z(Z({},q),{},{className:E}),W):g||!v&&!E?V=p(Z(Z({},q),{},{style:{display:"none"}}),W):V=null;else{var K;z===$a?K="prepare":x$(z)?K="active":z===zl&&(K="start");var _=O1(y,"".concat(N,"-").concat(K));V=p(Z(Z({},q),{},{className:re(O1(y,N),Y(Y({},_,_&&K),y,typeof y=="string")),style:P}),W)}if(u.isValidElement(V)&&ki(V)){var j=Zl(V);j||(V=u.cloneElement(V,{ref:W}))}return u.createElement(zj,{ref:O},V)});return r.displayName="CSSMotion",r}const Xa=Kj(b$);var Zh="add",Jh="keep",ep="remove",_g="removed";function Xj(e){var t;return e&&lt(e)==="object"&&"key"in e?t=e:t={key:e},Z(Z({},t),{},{key:String(t.key)})}function tp(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[];return e.map(Xj)}function Yj(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[],t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:[],n=[],r=0,o=t.length,l=tp(e),c=tp(t);l.forEach(function(v){for(var g=!1,p=r;p<o;p+=1){var y=c[p];if(y.key===v.key){r<p&&(n=n.concat(c.slice(r,p).map(function(E){return Z(Z({},E),{},{status:Zh})})),r=p),n.push(Z(Z({},y),{},{status:Jh})),r+=1,g=!0;break}}g||n.push(Z(Z({},v),{},{status:ep}))}),r<o&&(n=n.concat(c.slice(r).map(function(v){return Z(Z({},v),{},{status:Zh})})));var d={};n.forEach(function(v){var g=v.key;d[g]=(d[g]||0)+1});var m=Object.keys(d).filter(function(v){return d[v]>1});return m.forEach(function(v){n=n.filter(function(g){var p=g.key,y=g.status;return p!==v||y!==ep}),n.forEach(function(g){g.key===v&&(g.status=Jh)})}),n}var Qj=["component","children","onVisibleChanged","onAllRemoved"],Zj=["status"],Jj=["eventProps","visible","children","motionName","motionAppear","motionEnter","motionLeave","motionLeaveImmediately","motionDeadline","removeOnLeave","leavedClassName","onAppearPrepare","onAppearStart","onAppearActive","onAppearEnd","onEnterStart","onEnterActive","onEnterEnd","onLeaveStart","onLeaveActive","onLeaveEnd"];function e4(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:Xa,n=function(r){$o(l,r);var o=Oo(l);function l(){var c;qn(this,l);for(var d=arguments.length,m=new Array(d),v=0;v<d;v++)m[v]=arguments[v];return c=o.call.apply(o,[this].concat(m)),Y(_t(c),"state",{keyEntities:[]}),Y(_t(c),"removeKey",function(g){c.setState(function(p){var y=p.keyEntities.map(function(E){return E.key!==g?E:Z(Z({},E),{},{status:_g})});return{keyEntities:y}},function(){var p=c.state.keyEntities,y=p.filter(function(E){var C=E.status;return C!==_g}).length;y===0&&c.props.onAllRemoved&&c.props.onAllRemoved()})}),c}return Wn(l,[{key:"render",value:function(){var d=this,m=this.state.keyEntities,v=this.props,g=v.component,p=v.children,y=v.onVisibleChanged;v.onAllRemoved;var E=wt(v,Qj),C=g||u.Fragment,x={};return Jj.forEach(function(b){x[b]=E[b],delete E[b]}),delete E.keys,u.createElement(C,E,m.map(function(b,w){var $=b.status,O=wt(b,Zj),T=$===Zh||$===Jh;return u.createElement(t,Ie({},x,{key:O.key,visible:T,eventProps:O,onVisibleChanged:function(M){y==null||y(M,{key:O.key}),M||d.removeKey(O.key)}}),function(R,M){return p(Z(Z({},R),{},{index:w}),M)})}))}}],[{key:"getDerivedStateFromProps",value:function(d,m){var v=d.keys,g=m.keyEntities,p=tp(v),y=Yj(g,p);return{keyEntities:y.filter(function(E){var C=g.find(function(x){var b=x.key;return E.key===b});return!(C&&C.status===_g&&E.status===ep)})}}}]),l}(u.Component);return Y(n,"defaultProps",{component:"div"}),n}const E$=e4(b$);function t4(e){const{children:t}=e,[,n]=Rr(),{motion:r}=n,o=u.useRef(!1);return o.current=o.current||r===!1,o.current?u.createElement(Pj,{motion:r},t):t}const n4=()=>null;var r4=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const a4=["getTargetContainer","getPopupContainer","renderEmpty","input","pagination","form","select","button"];let ed,w$,$$,O$;function zf(){return ed||Mc}function o4(){return w$||Md}function i4(e){return Object.keys(e).some(t=>t.endsWith("Color"))}const l4=e=>{const{prefixCls:t,iconPrefixCls:n,theme:r,holderRender:o}=e;t!==void 0&&(ed=t),n!==void 0&&(w$=n),"holderRender"in e&&(O$=o),r&&(i4(r)?ij(zf(),r):$$=r)},l0=()=>({getPrefixCls:(e,t)=>t||(e?`${zf()}-${e}`:zf()),getIconPrefixCls:o4,getRootPrefixCls:()=>ed||zf(),getTheme:()=>$$,holderRender:O$}),s4=e=>{const{children:t,csp:n,autoInsertSpaceInButton:r,alert:o,anchor:l,form:c,locale:d,componentSize:m,direction:v,space:g,splitter:p,virtual:y,dropdownMatchSelectWidth:E,popupMatchSelectWidth:C,popupOverflow:x,legacyLocale:b,parentContext:w,iconPrefixCls:$,theme:O,componentDisabled:T,segmented:R,statistic:M,spin:N,calendar:z,carousel:P,cascader:D,collapse:F,typography:W,checkbox:V,descriptions:q,divider:K,drawer:_,skeleton:j,steps:B,image:k,layout:I,list:H,mentions:G,modal:X,progress:U,result:J,slider:te,breadcrumb:ve,menu:ne,pagination:le,input:be,textArea:fe,empty:ie,badge:pe,radio:Re,rate:se,switch:ye,transfer:$e,avatar:we,message:Ne,tag:Ze,table:He,card:Xe,tabs:xe,timeline:Me,timePicker:Ve,upload:Ke,notification:We,tree:at,colorPicker:Ue,datePicker:Ye,rangePicker:st,flex:ht,wave:Qe,dropdown:Pe,warning:Fe,tour:_e,tooltip:qe,popover:Ct,popconfirm:Nt,floatButtonGroup:_n,variant:bn,inputNumber:$n,treeSelect:sn}=e,Ft=u.useCallback((tt,ut)=>{const{prefixCls:Vt}=e;if(ut)return ut;const Tt=Vt||w.getPrefixCls("");return tt?`${Tt}-${tt}`:Tt},[w.getPrefixCls,e.prefixCls]),nn=$||w.iconPrefixCls||Md,Et=n||w.csp;Mj(nn,Et);const nt=Ij(O,w.theme,{prefixCls:Ft("")}),ke={csp:Et,autoInsertSpaceInButton:r,alert:o,anchor:l,locale:d||b,direction:v,space:g,splitter:p,virtual:y,popupMatchSelectWidth:C??E,popupOverflow:x,getPrefixCls:Ft,iconPrefixCls:nn,theme:nt,segmented:R,statistic:M,spin:N,calendar:z,carousel:P,cascader:D,collapse:F,typography:W,checkbox:V,descriptions:q,divider:K,drawer:_,skeleton:j,steps:B,image:k,input:be,textArea:fe,layout:I,list:H,mentions:G,modal:X,progress:U,result:J,slider:te,breadcrumb:ve,menu:ne,pagination:le,empty:ie,badge:pe,radio:Re,rate:se,switch:ye,transfer:$e,avatar:we,message:Ne,tag:Ze,table:He,card:Xe,tabs:xe,timeline:Me,timePicker:Ve,upload:Ke,notification:We,tree:at,colorPicker:Ue,datePicker:Ye,rangePicker:st,flex:ht,wave:Qe,dropdown:Pe,warning:Fe,tour:_e,tooltip:qe,popover:Ct,popconfirm:Nt,floatButtonGroup:_n,variant:bn,inputNumber:$n,treeSelect:sn},De=Object.assign({},w);Object.keys(ke).forEach(tt=>{ke[tt]!==void 0&&(De[tt]=ke[tt])}),a4.forEach(tt=>{const ut=e[tt];ut&&(De[tt]=ut)}),typeof r<"u"&&(De.button=Object.assign({autoInsertSpace:r},De.button));const pt=qc(()=>De,De,(tt,ut)=>{const Vt=Object.keys(tt),Tt=Object.keys(ut);return Vt.length!==Tt.length||Vt.some(At=>tt[At]!==ut[At])}),{layer:St}=u.useContext(Wc),Be=u.useMemo(()=>({prefixCls:nn,csp:Et,layer:St?"antd":void 0}),[nn,Et,St]);let je=u.createElement(u.Fragment,null,u.createElement(n4,{dropdownMatchSelectWidth:E}),t);const rt=u.useMemo(()=>{var tt,ut,Vt,Tt;return Pl(((tt=So.Form)===null||tt===void 0?void 0:tt.defaultValidateMessages)||{},((Vt=(ut=pt.locale)===null||ut===void 0?void 0:ut.Form)===null||Vt===void 0?void 0:Vt.defaultValidateMessages)||{},((Tt=pt.form)===null||Tt===void 0?void 0:Tt.validateMessages)||{},(c==null?void 0:c.validateMessages)||{})},[pt,c==null?void 0:c.validateMessages]);Object.keys(rt).length>0&&(je=u.createElement(Jw.Provider,{value:rt},je)),d&&(je=u.createElement(XI,{locale:d,_ANT_MARK__:KI},je)),je=u.createElement(Vp.Provider,{value:Be},je),m&&(je=u.createElement(lj,{size:m},je)),je=u.createElement(t4,null,je);const ot=u.useMemo(()=>{const tt=nt||{},{algorithm:ut,token:Vt,components:Tt,cssVar:At}=tt,It=r4(tt,["algorithm","token","components","cssVar"]),Qt=ut&&(!Array.isArray(ut)||ut.length>0)?Yf(ut):n0,rn={};Object.entries(Tt||{}).forEach(er=>{let[pn,an]=er;const Ae=Object.assign({},an);"algorithm"in Ae&&(Ae.algorithm===!0?Ae.theme=Qt:(Array.isArray(Ae.algorithm)||typeof Ae.algorithm=="function")&&(Ae.theme=Yf(Ae.algorithm)),delete Ae.algorithm),rn[pn]=Ae});const Rt=Object.assign(Object.assign({},Wl),Vt);return Object.assign(Object.assign({},It),{theme:Qt,token:Rt,components:rn,override:Object.assign({override:Rt},rn),cssVar:At})},[nt]);return O&&(je=u.createElement(r0.Provider,{value:ot},je)),pt.warning&&(je=u.createElement(UI.Provider,{value:pt.warning},je)),T!==void 0&&(je=u.createElement(a0,{disabled:T},je)),u.createElement(vt.Provider,{value:pt},je)},Ya=e=>{const t=u.useContext(vt),n=u.useContext(e0);return u.createElement(s4,Object.assign({parentContext:t,legacyLocale:n},e))};Ya.ConfigContext=vt;Ya.SizeContext=Li;Ya.config=l4;Ya.useConfig=sj;Object.defineProperty(Ya,"SizeContext",{get:()=>Li});var c4=`accept acceptCharset accessKey action allowFullScreen allowTransparency
    alt async autoComplete autoFocus autoPlay capture cellPadding cellSpacing challenge
    charSet checked classID className colSpan cols content contentEditable contextMenu
    controls coords crossOrigin data dateTime default defer dir disabled download draggable
    encType form formAction formEncType formMethod formNoValidate formTarget frameBorder
    headers height hidden high href hrefLang htmlFor httpEquiv icon id inputMode integrity
    is keyParams keyType kind label lang list loop low manifest marginHeight marginWidth max maxLength media
    mediaGroup method min minLength multiple muted name noValidate nonce open
    optimum pattern placeholder poster preload radioGroup readOnly rel required
    reversed role rowSpan rows sandbox scope scoped scrolling seamless selected
    shape size sizes span spellCheck src srcDoc srcLang srcSet start step style
    summary tabIndex target title type useMap value width wmode wrap`,u4=`onCopy onCut onPaste onCompositionEnd onCompositionStart onCompositionUpdate onKeyDown
    onKeyPress onKeyUp onFocus onBlur onChange onInput onSubmit onClick onContextMenu onDoubleClick
    onDrag onDragEnd onDragEnter onDragExit onDragLeave onDragOver onDragStart onDrop onMouseDown
    onMouseEnter onMouseLeave onMouseMove onMouseOut onMouseOver onMouseUp onSelect onTouchCancel
    onTouchEnd onTouchMove onTouchStart onScroll onWheel onAbort onCanPlay onCanPlayThrough
    onDurationChange onEmptied onEncrypted onEnded onError onLoadedData onLoadedMetadata
    onLoadStart onPause onPlay onPlaying onProgress onRateChange onSeeked onSeeking onStalled onSuspend onTimeUpdate onVolumeChange onWaiting onLoad onError`,f4="".concat(c4," ").concat(u4).split(/[\s\n]+/),d4="aria-",m4="data-";function R1(e,t){return e.indexOf(t)===0}function Ga(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,n;t===!1?n={aria:!0,data:!0,attr:!0}:t===!0?n={aria:!0}:n=Z({},t);var r={};return Object.keys(e).forEach(function(o){(n.aria&&(o==="role"||R1(o,d4))||n.data&&R1(o,m4)||n.attr&&f4.includes(o))&&(r[o]=e[o])}),r}function R$(e){return e&&oe.isValidElement(e)&&e.type===oe.Fragment}const v4=(e,t,n)=>oe.isValidElement(e)?oe.cloneElement(e,typeof n=="function"?n(e.props||{}):n):t;function Zr(e,t){return v4(e,e,t)}const T1=e=>typeof e=="object"&&e!=null&&e.nodeType===1,M1=(e,t)=>(!t||e!=="hidden")&&e!=="visible"&&e!=="clip",hf=(e,t)=>{if(e.clientHeight<e.scrollHeight||e.clientWidth<e.scrollWidth){const n=getComputedStyle(e,null);return M1(n.overflowY,t)||M1(n.overflowX,t)||(r=>{const o=(l=>{if(!l.ownerDocument||!l.ownerDocument.defaultView)return null;try{return l.ownerDocument.defaultView.frameElement}catch{return null}})(r);return!!o&&(o.clientHeight<r.scrollHeight||o.clientWidth<r.scrollWidth)})(e)}return!1},pf=(e,t,n,r,o,l,c,d)=>l<e&&c>t||l>e&&c<t?0:l<=e&&d<=n||c>=t&&d>=n?l-e-r:c>t&&d<n||l<e&&d>n?c-t+o:0,g4=e=>{const t=e.parentElement;return t??(e.getRootNode().host||null)},_1=(e,t)=>{var n,r,o,l;if(typeof document>"u")return[];const{scrollMode:c,block:d,inline:m,boundary:v,skipOverflowHiddenElements:g}=t,p=typeof v=="function"?v:_=>_!==v;if(!T1(e))throw new TypeError("Invalid target");const y=document.scrollingElement||document.documentElement,E=[];let C=e;for(;T1(C)&&p(C);){if(C=g4(C),C===y){E.push(C);break}C!=null&&C===document.body&&hf(C)&&!hf(document.documentElement)||C!=null&&hf(C,g)&&E.push(C)}const x=(r=(n=window.visualViewport)==null?void 0:n.width)!=null?r:innerWidth,b=(l=(o=window.visualViewport)==null?void 0:o.height)!=null?l:innerHeight,{scrollX:w,scrollY:$}=window,{height:O,width:T,top:R,right:M,bottom:N,left:z}=e.getBoundingClientRect(),{top:P,right:D,bottom:F,left:W}=(_=>{const j=window.getComputedStyle(_);return{top:parseFloat(j.scrollMarginTop)||0,right:parseFloat(j.scrollMarginRight)||0,bottom:parseFloat(j.scrollMarginBottom)||0,left:parseFloat(j.scrollMarginLeft)||0}})(e);let V=d==="start"||d==="nearest"?R-P:d==="end"?N+F:R+O/2-P+F,q=m==="center"?z+T/2-W+D:m==="end"?M+D:z-W;const K=[];for(let _=0;_<E.length;_++){const j=E[_],{height:B,width:k,top:I,right:H,bottom:G,left:X}=j.getBoundingClientRect();if(c==="if-needed"&&R>=0&&z>=0&&N<=b&&M<=x&&(j===y&&!hf(j)||R>=I&&N<=G&&z>=X&&M<=H))return K;const U=getComputedStyle(j),J=parseInt(U.borderLeftWidth,10),te=parseInt(U.borderTopWidth,10),ve=parseInt(U.borderRightWidth,10),ne=parseInt(U.borderBottomWidth,10);let le=0,be=0;const fe="offsetWidth"in j?j.offsetWidth-j.clientWidth-J-ve:0,ie="offsetHeight"in j?j.offsetHeight-j.clientHeight-te-ne:0,pe="offsetWidth"in j?j.offsetWidth===0?0:k/j.offsetWidth:0,Re="offsetHeight"in j?j.offsetHeight===0?0:B/j.offsetHeight:0;if(y===j)le=d==="start"?V:d==="end"?V-b:d==="nearest"?pf($,$+b,b,te,ne,$+V,$+V+O,O):V-b/2,be=m==="start"?q:m==="center"?q-x/2:m==="end"?q-x:pf(w,w+x,x,J,ve,w+q,w+q+T,T),le=Math.max(0,le+$),be=Math.max(0,be+w);else{le=d==="start"?V-I-te:d==="end"?V-G+ne+ie:d==="nearest"?pf(I,G,B,te,ne+ie,V,V+O,O):V-(I+B/2)+ie/2,be=m==="start"?q-X-J:m==="center"?q-(X+k/2)+fe/2:m==="end"?q-H+ve+fe:pf(X,H,k,J,ve+fe,q,q+T,T);const{scrollLeft:se,scrollTop:ye}=j;le=Re===0?0:Math.max(0,Math.min(ye+le/Re,j.scrollHeight-B/Re+ie)),be=pe===0?0:Math.max(0,Math.min(se+be/pe,j.scrollWidth-k/pe+fe)),V+=ye-le,q+=se-be}K.push({el:j,top:le,left:be})}return K},h4=e=>e===!1?{block:"end",inline:"nearest"}:(t=>t===Object(t)&&Object.keys(t).length!==0)(e)?e:{block:"start",inline:"nearest"};function p4(e,t){if(!e.isConnected||!(o=>{let l=o;for(;l&&l.parentNode;){if(l.parentNode===document)return!0;l=l.parentNode instanceof ShadowRoot?l.parentNode.host:l.parentNode}return!1})(e))return;const n=(o=>{const l=window.getComputedStyle(o);return{top:parseFloat(l.scrollMarginTop)||0,right:parseFloat(l.scrollMarginRight)||0,bottom:parseFloat(l.scrollMarginBottom)||0,left:parseFloat(l.scrollMarginLeft)||0}})(e);if((o=>typeof o=="object"&&typeof o.behavior=="function")(t))return t.behavior(_1(e,t));const r=typeof t=="boolean"||t==null?void 0:t.behavior;for(const{el:o,top:l,left:c}of _1(e,h4(t))){const d=l-n.top+n.bottom,m=c-n.left+n.right;o.scroll({top:d,left:m,behavior:r})}}const na=e=>{const[,,,,t]=Rr();return t?`${e}-css-var`:""};var ct={BACKSPACE:8,TAB:9,ENTER:13,SHIFT:16,CTRL:17,ALT:18,CAPS_LOCK:20,ESC:27,SPACE:32,END:35,HOME:36,LEFT:37,UP:38,RIGHT:39,DOWN:40,N:78,P:80,META:91,WIN_KEY_RIGHT:92,CONTEXT_MENU:93,F1:112,F2:113,F3:114,F4:115,F5:116,F6:117,F7:118,F8:119,F9:120,F10:121,F11:122,F12:123,SEMICOLON:186,EQUALS:187,WIN_KEY:224},s0=u.forwardRef(function(e,t){var n=e.prefixCls,r=e.style,o=e.className,l=e.duration,c=l===void 0?4.5:l,d=e.showProgress,m=e.pauseOnHover,v=m===void 0?!0:m,g=e.eventKey,p=e.content,y=e.closable,E=e.closeIcon,C=E===void 0?"x":E,x=e.props,b=e.onClick,w=e.onNoticeClose,$=e.times,O=e.hovering,T=u.useState(!1),R=de(T,2),M=R[0],N=R[1],z=u.useState(0),P=de(z,2),D=P[0],F=P[1],W=u.useState(0),V=de(W,2),q=V[0],K=V[1],_=O||M,j=c>0&&d,B=function(){w(g)},k=function(J){(J.key==="Enter"||J.code==="Enter"||J.keyCode===ct.ENTER)&&B()};u.useEffect(function(){if(!_&&c>0){var U=Date.now()-q,J=setTimeout(function(){B()},c*1e3-q);return function(){v&&clearTimeout(J),K(Date.now()-U)}}},[c,_,$]),u.useEffect(function(){if(!_&&j&&(v||q===0)){var U=performance.now(),J,te=function ve(){cancelAnimationFrame(J),J=requestAnimationFrame(function(ne){var le=ne+q-U,be=Math.min(le/(c*1e3),1);F(be*100),be<1&&ve()})};return te(),function(){v&&cancelAnimationFrame(J)}}},[c,q,_,j,$]);var I=u.useMemo(function(){return lt(y)==="object"&&y!==null?y:y?{closeIcon:C}:{}},[y,C]),H=Ga(I,!0),G=100-(!D||D<0?0:D>100?100:D),X="".concat(n,"-notice");return u.createElement("div",Ie({},x,{ref:t,className:re(X,o,Y({},"".concat(X,"-closable"),y)),style:r,onMouseEnter:function(J){var te;N(!0),x==null||(te=x.onMouseEnter)===null||te===void 0||te.call(x,J)},onMouseLeave:function(J){var te;N(!1),x==null||(te=x.onMouseLeave)===null||te===void 0||te.call(x,J)},onClick:b}),u.createElement("div",{className:"".concat(X,"-content")},p),y&&u.createElement("a",Ie({tabIndex:0,className:"".concat(X,"-close"),onKeyDown:k,"aria-label":"Close"},H,{onClick:function(J){J.preventDefault(),J.stopPropagation(),B()}}),I.closeIcon),j&&u.createElement("progress",{className:"".concat(X,"-progress"),max:"100",value:G},G+"%"))}),T$=oe.createContext({}),M$=function(t){var n=t.children,r=t.classNames;return oe.createElement(T$.Provider,{value:{classNames:r}},n)},N1=8,A1=3,I1=16,y4=function(t){var n={offset:N1,threshold:A1,gap:I1};if(t&&lt(t)==="object"){var r,o,l;n.offset=(r=t.offset)!==null&&r!==void 0?r:N1,n.threshold=(o=t.threshold)!==null&&o!==void 0?o:A1,n.gap=(l=t.gap)!==null&&l!==void 0?l:I1}return[!!t,n]},b4=["className","style","classNames","styles"],S4=function(t){var n=t.configList,r=t.placement,o=t.prefixCls,l=t.className,c=t.style,d=t.motion,m=t.onAllNoticeRemoved,v=t.onNoticeClose,g=t.stack,p=u.useContext(T$),y=p.classNames,E=u.useRef({}),C=u.useState(null),x=de(C,2),b=x[0],w=x[1],$=u.useState([]),O=de($,2),T=O[0],R=O[1],M=n.map(function(_){return{config:_,key:String(_.key)}}),N=y4(g),z=de(N,2),P=z[0],D=z[1],F=D.offset,W=D.threshold,V=D.gap,q=P&&(T.length>0||M.length<=W),K=typeof d=="function"?d(r):d;return u.useEffect(function(){P&&T.length>1&&R(function(_){return _.filter(function(j){return M.some(function(B){var k=B.key;return j===k})})})},[T,M,P]),u.useEffect(function(){var _;if(P&&E.current[(_=M[M.length-1])===null||_===void 0?void 0:_.key]){var j;w(E.current[(j=M[M.length-1])===null||j===void 0?void 0:j.key])}},[M,P]),oe.createElement(E$,Ie({key:r,className:re(o,"".concat(o,"-").concat(r),y==null?void 0:y.list,l,Y(Y({},"".concat(o,"-stack"),!!P),"".concat(o,"-stack-expanded"),q)),style:c,keys:M,motionAppear:!0},K,{onAllRemoved:function(){m(r)}}),function(_,j){var B=_.config,k=_.className,I=_.style,H=_.index,G=B,X=G.key,U=G.times,J=String(X),te=B,ve=te.className,ne=te.style,le=te.classNames,be=te.styles,fe=wt(te,b4),ie=M.findIndex(function(Me){return Me.key===J}),pe={};if(P){var Re=M.length-1-(ie>-1?ie:H-1),se=r==="top"||r==="bottom"?"-50%":"0";if(Re>0){var ye,$e,we;pe.height=q?(ye=E.current[J])===null||ye===void 0?void 0:ye.offsetHeight:b==null?void 0:b.offsetHeight;for(var Ne=0,Ze=0;Ze<Re;Ze++){var He;Ne+=((He=E.current[M[M.length-1-Ze].key])===null||He===void 0?void 0:He.offsetHeight)+V}var Xe=(q?Ne:Re*F)*(r.startsWith("top")?1:-1),xe=!q&&b!==null&&b!==void 0&&b.offsetWidth&&($e=E.current[J])!==null&&$e!==void 0&&$e.offsetWidth?((b==null?void 0:b.offsetWidth)-F*2*(Re<3?Re:3))/((we=E.current[J])===null||we===void 0?void 0:we.offsetWidth):1;pe.transform="translate3d(".concat(se,", ").concat(Xe,"px, 0) scaleX(").concat(xe,")")}else pe.transform="translate3d(".concat(se,", 0, 0)")}return oe.createElement("div",{ref:j,className:re("".concat(o,"-notice-wrapper"),k,le==null?void 0:le.wrapper),style:Z(Z(Z({},I),pe),be==null?void 0:be.wrapper),onMouseEnter:function(){return R(function(Ve){return Ve.includes(J)?Ve:[].concat(Te(Ve),[J])})},onMouseLeave:function(){return R(function(Ve){return Ve.filter(function(Ke){return Ke!==J})})}},oe.createElement(s0,Ie({},fe,{ref:function(Ve){ie>-1?E.current[J]=Ve:delete E.current[J]},prefixCls:o,classNames:le,styles:be,className:re(ve,y==null?void 0:y.notice),style:ne,times:U,key:X,eventKey:X,onNoticeClose:v,hovering:P&&T.length>0})))})},C4=u.forwardRef(function(e,t){var n=e.prefixCls,r=n===void 0?"rc-notification":n,o=e.container,l=e.motion,c=e.maxCount,d=e.className,m=e.style,v=e.onAllRemoved,g=e.stack,p=e.renderNotifications,y=u.useState([]),E=de(y,2),C=E[0],x=E[1],b=function(P){var D,F=C.find(function(W){return W.key===P});F==null||(D=F.onClose)===null||D===void 0||D.call(F),x(function(W){return W.filter(function(V){return V.key!==P})})};u.useImperativeHandle(t,function(){return{open:function(P){x(function(D){var F=Te(D),W=F.findIndex(function(K){return K.key===P.key}),V=Z({},P);if(W>=0){var q;V.times=(((q=D[W])===null||q===void 0?void 0:q.times)||0)+1,F[W]=V}else V.times=0,F.push(V);return c>0&&F.length>c&&(F=F.slice(-c)),F})},close:function(P){b(P)},destroy:function(){x([])}}});var w=u.useState({}),$=de(w,2),O=$[0],T=$[1];u.useEffect(function(){var z={};C.forEach(function(P){var D=P.placement,F=D===void 0?"topRight":D;F&&(z[F]=z[F]||[],z[F].push(P))}),Object.keys(O).forEach(function(P){z[P]=z[P]||[]}),T(z)},[C]);var R=function(P){T(function(D){var F=Z({},D),W=F[P]||[];return W.length||delete F[P],F})},M=u.useRef(!1);if(u.useEffect(function(){Object.keys(O).length>0?M.current=!0:M.current&&(v==null||v(),M.current=!1)},[O]),!o)return null;var N=Object.keys(O);return ri.createPortal(u.createElement(u.Fragment,null,N.map(function(z){var P=O[z],D=u.createElement(S4,{key:z,configList:P,placement:z,prefixCls:r,className:d==null?void 0:d(z),style:m==null?void 0:m(z),motion:l,onNoticeClose:b,onAllNoticeRemoved:R,stack:g});return p?p(D,{prefixCls:r,key:z}):D})),o)}),x4=["getContainer","motion","prefixCls","maxCount","className","style","onAllRemoved","stack","renderNotifications"],E4=function(){return document.body},j1=0;function w4(){for(var e={},t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return n.forEach(function(o){o&&Object.keys(o).forEach(function(l){var c=o[l];c!==void 0&&(e[l]=c)})}),e}function _$(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},t=e.getContainer,n=t===void 0?E4:t,r=e.motion,o=e.prefixCls,l=e.maxCount,c=e.className,d=e.style,m=e.onAllRemoved,v=e.stack,g=e.renderNotifications,p=wt(e,x4),y=u.useState(),E=de(y,2),C=E[0],x=E[1],b=u.useRef(),w=u.createElement(C4,{container:C,ref:b,prefixCls:o,motion:r,maxCount:l,className:c,style:d,onAllRemoved:m,stack:v,renderNotifications:g}),$=u.useState([]),O=de($,2),T=O[0],R=O[1],M=jn(function(z){var P=w4(p,z);(P.key===null||P.key===void 0)&&(P.key="rc-notification-".concat(j1),j1+=1),R(function(D){return[].concat(Te(D),[{type:"open",config:P}])})}),N=u.useMemo(function(){return{open:M,close:function(P){R(function(D){return[].concat(Te(D),[{type:"close",key:P}])})},destroy:function(){R(function(P){return[].concat(Te(P),[{type:"destroy"}])})}}},[]);return u.useEffect(function(){x(n())}),u.useEffect(function(){if(b.current&&T.length){T.forEach(function(D){switch(D.type){case"open":b.current.open(D.config);break;case"close":b.current.close(D.key);break;case"destroy":b.current.destroy();break}});var z,P;R(function(D){return(z!==D||!P)&&(z=D,P=D.filter(function(F){return!T.includes(F)})),P})}},[T]),[N,w]}const c0=oe.createContext(void 0),Jo=100,$4=10,u0=Jo*$4,N$={Modal:Jo,Drawer:Jo,Popover:Jo,Popconfirm:Jo,Tooltip:Jo,Tour:Jo,FloatButton:Jo},O4={SelectLike:50,Dropdown:50,DatePicker:50,Menu:50,ImagePreview:1};function R4(e){return e in N$}const Nd=(e,t)=>{const[,n]=Rr(),r=oe.useContext(c0),o=R4(e);let l;if(t!==void 0)l=[t,t];else{let c=r??0;o?c+=(r?0:n.zIndexPopupBase)+N$[e]:c+=O4[e],l=[r===void 0?t:c,c]}return l},T4=e=>{const{componentCls:t,iconCls:n,boxShadow:r,colorText:o,colorSuccess:l,colorError:c,colorWarning:d,colorInfo:m,fontSizeLG:v,motionEaseInOutCirc:g,motionDurationSlow:p,marginXS:y,paddingXS:E,borderRadiusLG:C,zIndexPopup:x,contentPadding:b,contentBg:w}=e,$=`${t}-notice`,O=new Pt("MessageMoveIn",{"0%":{padding:0,transform:"translateY(-100%)",opacity:0},"100%":{padding:E,transform:"translateY(0)",opacity:1}}),T=new Pt("MessageMoveOut",{"0%":{maxHeight:e.height,padding:E,opacity:1},"100%":{maxHeight:0,padding:0,opacity:0}}),R={padding:E,textAlign:"center",[`${t}-custom-content`]:{display:"flex",alignItems:"center"},[`${t}-custom-content > ${n}`]:{marginInlineEnd:y,fontSize:v},[`${$}-content`]:{display:"inline-block",padding:b,background:w,borderRadius:C,boxShadow:r,pointerEvents:"all"},[`${t}-success > ${n}`]:{color:l},[`${t}-error > ${n}`]:{color:c},[`${t}-warning > ${n}`]:{color:d},[`${t}-info > ${n},
      ${t}-loading > ${n}`]:{color:m}};return[{[t]:Object.assign(Object.assign({},Tr(e)),{color:o,position:"fixed",top:y,width:"100%",pointerEvents:"none",zIndex:x,[`${t}-move-up`]:{animationFillMode:"forwards"},[`
        ${t}-move-up-appear,
        ${t}-move-up-enter
      `]:{animationName:O,animationDuration:p,animationPlayState:"paused",animationTimingFunction:g},[`
        ${t}-move-up-appear${t}-move-up-appear-active,
        ${t}-move-up-enter${t}-move-up-enter-active
      `]:{animationPlayState:"running"},[`${t}-move-up-leave`]:{animationName:T,animationDuration:p,animationPlayState:"paused",animationTimingFunction:g},[`${t}-move-up-leave${t}-move-up-leave-active`]:{animationPlayState:"running"},"&-rtl":{direction:"rtl",span:{direction:"rtl"}}})},{[t]:{[`${$}-wrapper`]:Object.assign({},R)}},{[`${t}-notice-pure-panel`]:Object.assign(Object.assign({},R),{padding:0,textAlign:"start"})}]},M4=e=>({zIndexPopup:e.zIndexPopupBase+u0+10,contentBg:e.colorBgElevated,contentPadding:`${(e.controlHeightLG-e.fontSize*e.lineHeight)/2}px ${e.paddingSM}px`}),A$=wn("Message",e=>{const t=kt(e,{height:150});return[T4(t)]},M4);var _4=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const N4={info:u.createElement(Wp,null),success:u.createElement(wd,null),error:u.createElement(Xl,null),warning:u.createElement(Ql,null),loading:u.createElement(Uc,null)},I$=e=>{let{prefixCls:t,type:n,icon:r,children:o}=e;return u.createElement("div",{className:re(`${t}-custom-content`,`${t}-${n}`)},r||N4[n],u.createElement("span",null,o))},A4=e=>{const{prefixCls:t,className:n,type:r,icon:o,content:l}=e,c=_4(e,["prefixCls","className","type","icon","content"]),{getPrefixCls:d}=u.useContext(vt),m=t||d("message"),v=na(m),[g,p,y]=A$(m,v);return g(u.createElement(s0,Object.assign({},c,{prefixCls:m,className:re(n,p,`${m}-notice-pure-panel`,y,v),eventKey:"pure",duration:null,content:u.createElement(I$,{prefixCls:m,type:r,icon:o},l)})))};function I4(e,t){return{motionName:t??`${e}-move-up`}}function f0(e){let t;const n=new Promise(o=>{t=e(()=>{o(!0)})}),r=()=>{t==null||t()};return r.then=(o,l)=>n.then(o,l),r.promise=n,r}var j4=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const P4=8,z4=3,D4=e=>{let{children:t,prefixCls:n}=e;const r=na(n),[o,l,c]=A$(n,r);return o(u.createElement(M$,{classNames:{list:re(l,c,r)}},t))},B4=(e,t)=>{let{prefixCls:n,key:r}=t;return u.createElement(D4,{prefixCls:n,key:r},e)},L4=u.forwardRef((e,t)=>{const{top:n,prefixCls:r,getContainer:o,maxCount:l,duration:c=z4,rtl:d,transitionName:m,onAllRemoved:v}=e,{getPrefixCls:g,getPopupContainer:p,message:y,direction:E}=u.useContext(vt),C=r||g("message"),x=()=>({left:"50%",transform:"translateX(-50%)",top:n??P4}),b=()=>re({[`${C}-rtl`]:d??E==="rtl"}),w=()=>I4(C,m),$=u.createElement("span",{className:`${C}-close-x`},u.createElement(Yl,{className:`${C}-close-icon`})),[O,T]=_$({prefixCls:C,style:x,className:b,motion:w,closable:!1,closeIcon:$,duration:c,getContainer:()=>(o==null?void 0:o())||(p==null?void 0:p())||document.body,maxCount:l,onAllRemoved:v,renderNotifications:B4});return u.useImperativeHandle(t,()=>Object.assign(Object.assign({},O),{prefixCls:C,message:y})),T});let P1=0;function j$(e){const t=u.useRef(null);return kc(),[u.useMemo(()=>{const r=m=>{var v;(v=t.current)===null||v===void 0||v.close(m)},o=m=>{if(!t.current){const M=()=>{};return M.then=()=>{},M}const{open:v,prefixCls:g,message:p}=t.current,y=`${g}-notice`,{content:E,icon:C,type:x,key:b,className:w,style:$,onClose:O}=m,T=j4(m,["content","icon","type","key","className","style","onClose"]);let R=b;return R==null&&(P1+=1,R=`antd-message-${P1}`),f0(M=>(v(Object.assign(Object.assign({},T),{key:R,content:u.createElement(I$,{prefixCls:g,type:x,icon:C},E),placement:"top",className:re(x&&`${y}-${x}`,w,p==null?void 0:p.className),style:Object.assign(Object.assign({},p==null?void 0:p.style),$),onClose:()=>{O==null||O(),M()}})),()=>{r(R)}))},c={open:o,destroy:m=>{var v;m!==void 0?r(m):(v=t.current)===null||v===void 0||v.destroy()}};return["info","success","warning","error","loading"].forEach(m=>{const v=(g,p,y)=>{let E;g&&typeof g=="object"&&"content"in g?E=g:E={content:g};let C,x;typeof p=="function"?x=p:(C=p,x=y);const b=Object.assign(Object.assign({onClose:x,duration:C},E),{type:m});return o(b)};c[m]=v}),c},[]),u.createElement(L4,Object.assign({key:"message-holder"},e,{ref:t}))]}function H4(e){return j$(e)}function F4(){const[e,t]=u.useState([]),n=u.useCallback(r=>(t(o=>[].concat(Te(o),[r])),()=>{t(o=>o.filter(l=>l!==r))}),[]);return[e,n]}function Or(){Or=function(){return t};var e,t={},n=Object.prototype,r=n.hasOwnProperty,o=Object.defineProperty||function(_,j,B){_[j]=B.value},l=typeof Symbol=="function"?Symbol:{},c=l.iterator||"@@iterator",d=l.asyncIterator||"@@asyncIterator",m=l.toStringTag||"@@toStringTag";function v(_,j,B){return Object.defineProperty(_,j,{value:B,enumerable:!0,configurable:!0,writable:!0}),_[j]}try{v({},"")}catch{v=function(B,k,I){return B[k]=I}}function g(_,j,B,k){var I=j&&j.prototype instanceof w?j:w,H=Object.create(I.prototype),G=new q(k||[]);return o(H,"_invoke",{value:D(_,B,G)}),H}function p(_,j,B){try{return{type:"normal",arg:_.call(j,B)}}catch(k){return{type:"throw",arg:k}}}t.wrap=g;var y="suspendedStart",E="suspendedYield",C="executing",x="completed",b={};function w(){}function $(){}function O(){}var T={};v(T,c,function(){return this});var R=Object.getPrototypeOf,M=R&&R(R(K([])));M&&M!==n&&r.call(M,c)&&(T=M);var N=O.prototype=w.prototype=Object.create(T);function z(_){["next","throw","return"].forEach(function(j){v(_,j,function(B){return this._invoke(j,B)})})}function P(_,j){function B(I,H,G,X){var U=p(_[I],_,H);if(U.type!=="throw"){var J=U.arg,te=J.value;return te&&lt(te)=="object"&&r.call(te,"__await")?j.resolve(te.__await).then(function(ve){B("next",ve,G,X)},function(ve){B("throw",ve,G,X)}):j.resolve(te).then(function(ve){J.value=ve,G(J)},function(ve){return B("throw",ve,G,X)})}X(U.arg)}var k;o(this,"_invoke",{value:function(H,G){function X(){return new j(function(U,J){B(H,G,U,J)})}return k=k?k.then(X,X):X()}})}function D(_,j,B){var k=y;return function(I,H){if(k===C)throw Error("Generator is already running");if(k===x){if(I==="throw")throw H;return{value:e,done:!0}}for(B.method=I,B.arg=H;;){var G=B.delegate;if(G){var X=F(G,B);if(X){if(X===b)continue;return X}}if(B.method==="next")B.sent=B._sent=B.arg;else if(B.method==="throw"){if(k===y)throw k=x,B.arg;B.dispatchException(B.arg)}else B.method==="return"&&B.abrupt("return",B.arg);k=C;var U=p(_,j,B);if(U.type==="normal"){if(k=B.done?x:E,U.arg===b)continue;return{value:U.arg,done:B.done}}U.type==="throw"&&(k=x,B.method="throw",B.arg=U.arg)}}}function F(_,j){var B=j.method,k=_.iterator[B];if(k===e)return j.delegate=null,B==="throw"&&_.iterator.return&&(j.method="return",j.arg=e,F(_,j),j.method==="throw")||B!=="return"&&(j.method="throw",j.arg=new TypeError("The iterator does not provide a '"+B+"' method")),b;var I=p(k,_.iterator,j.arg);if(I.type==="throw")return j.method="throw",j.arg=I.arg,j.delegate=null,b;var H=I.arg;return H?H.done?(j[_.resultName]=H.value,j.next=_.nextLoc,j.method!=="return"&&(j.method="next",j.arg=e),j.delegate=null,b):H:(j.method="throw",j.arg=new TypeError("iterator result is not an object"),j.delegate=null,b)}function W(_){var j={tryLoc:_[0]};1 in _&&(j.catchLoc=_[1]),2 in _&&(j.finallyLoc=_[2],j.afterLoc=_[3]),this.tryEntries.push(j)}function V(_){var j=_.completion||{};j.type="normal",delete j.arg,_.completion=j}function q(_){this.tryEntries=[{tryLoc:"root"}],_.forEach(W,this),this.reset(!0)}function K(_){if(_||_===""){var j=_[c];if(j)return j.call(_);if(typeof _.next=="function")return _;if(!isNaN(_.length)){var B=-1,k=function I(){for(;++B<_.length;)if(r.call(_,B))return I.value=_[B],I.done=!1,I;return I.value=e,I.done=!0,I};return k.next=k}}throw new TypeError(lt(_)+" is not iterable")}return $.prototype=O,o(N,"constructor",{value:O,configurable:!0}),o(O,"constructor",{value:$,configurable:!0}),$.displayName=v(O,m,"GeneratorFunction"),t.isGeneratorFunction=function(_){var j=typeof _=="function"&&_.constructor;return!!j&&(j===$||(j.displayName||j.name)==="GeneratorFunction")},t.mark=function(_){return Object.setPrototypeOf?Object.setPrototypeOf(_,O):(_.__proto__=O,v(_,m,"GeneratorFunction")),_.prototype=Object.create(N),_},t.awrap=function(_){return{__await:_}},z(P.prototype),v(P.prototype,d,function(){return this}),t.AsyncIterator=P,t.async=function(_,j,B,k,I){I===void 0&&(I=Promise);var H=new P(g(_,j,B,k),I);return t.isGeneratorFunction(j)?H:H.next().then(function(G){return G.done?G.value:H.next()})},z(N),v(N,m,"Generator"),v(N,c,function(){return this}),v(N,"toString",function(){return"[object Generator]"}),t.keys=function(_){var j=Object(_),B=[];for(var k in j)B.push(k);return B.reverse(),function I(){for(;B.length;){var H=B.pop();if(H in j)return I.value=H,I.done=!1,I}return I.done=!0,I}},t.values=K,q.prototype={constructor:q,reset:function(j){if(this.prev=0,this.next=0,this.sent=this._sent=e,this.done=!1,this.delegate=null,this.method="next",this.arg=e,this.tryEntries.forEach(V),!j)for(var B in this)B.charAt(0)==="t"&&r.call(this,B)&&!isNaN(+B.slice(1))&&(this[B]=e)},stop:function(){this.done=!0;var j=this.tryEntries[0].completion;if(j.type==="throw")throw j.arg;return this.rval},dispatchException:function(j){if(this.done)throw j;var B=this;function k(J,te){return G.type="throw",G.arg=j,B.next=J,te&&(B.method="next",B.arg=e),!!te}for(var I=this.tryEntries.length-1;I>=0;--I){var H=this.tryEntries[I],G=H.completion;if(H.tryLoc==="root")return k("end");if(H.tryLoc<=this.prev){var X=r.call(H,"catchLoc"),U=r.call(H,"finallyLoc");if(X&&U){if(this.prev<H.catchLoc)return k(H.catchLoc,!0);if(this.prev<H.finallyLoc)return k(H.finallyLoc)}else if(X){if(this.prev<H.catchLoc)return k(H.catchLoc,!0)}else{if(!U)throw Error("try statement without catch or finally");if(this.prev<H.finallyLoc)return k(H.finallyLoc)}}}},abrupt:function(j,B){for(var k=this.tryEntries.length-1;k>=0;--k){var I=this.tryEntries[k];if(I.tryLoc<=this.prev&&r.call(I,"finallyLoc")&&this.prev<I.finallyLoc){var H=I;break}}H&&(j==="break"||j==="continue")&&H.tryLoc<=B&&B<=H.finallyLoc&&(H=null);var G=H?H.completion:{};return G.type=j,G.arg=B,H?(this.method="next",this.next=H.finallyLoc,b):this.complete(G)},complete:function(j,B){if(j.type==="throw")throw j.arg;return j.type==="break"||j.type==="continue"?this.next=j.arg:j.type==="return"?(this.rval=this.arg=j.arg,this.method="return",this.next="end"):j.type==="normal"&&B&&(this.next=B),b},finish:function(j){for(var B=this.tryEntries.length-1;B>=0;--B){var k=this.tryEntries[B];if(k.finallyLoc===j)return this.complete(k.completion,k.afterLoc),V(k),b}},catch:function(j){for(var B=this.tryEntries.length-1;B>=0;--B){var k=this.tryEntries[B];if(k.tryLoc===j){var I=k.completion;if(I.type==="throw"){var H=I.arg;V(k)}return H}}throw Error("illegal catch attempt")},delegateYield:function(j,B,k){return this.delegate={iterator:K(j),resultName:B,nextLoc:k},this.method==="next"&&(this.arg=e),b}},t}function z1(e,t,n,r,o,l,c){try{var d=e[l](c),m=d.value}catch(v){return void n(v)}d.done?t(m):Promise.resolve(m).then(r,o)}function Gi(e){return function(){var t=this,n=arguments;return new Promise(function(r,o){var l=e.apply(t,n);function c(m){z1(l,r,o,c,d,"next",m)}function d(m){z1(l,r,o,c,d,"throw",m)}c(void 0)})}}var Kc=Z({},dA),V4=Kc.version,Ng=Kc.render,U4=Kc.unmountComponentAtNode,Ad;try{var q4=Number((V4||"").split(".")[0]);q4>=18&&(Ad=Kc.createRoot)}catch{}function D1(e){var t=Kc.__SECRET_INTERNALS_DO_NOT_USE_OR_YOU_WILL_BE_FIRED;t&&lt(t)==="object"&&(t.usingClientEntryPoint=e)}var td="__rc_react_root__";function W4(e,t){D1(!0);var n=t[td]||Ad(t);D1(!1),n.render(e),t[td]=n}function k4(e,t){Ng==null||Ng(e,t)}function G4(e,t){if(Ad){W4(e,t);return}k4(e,t)}function K4(e){return np.apply(this,arguments)}function np(){return np=Gi(Or().mark(function e(t){return Or().wrap(function(r){for(;;)switch(r.prev=r.next){case 0:return r.abrupt("return",Promise.resolve().then(function(){var o;(o=t[td])===null||o===void 0||o.unmount(),delete t[td]}));case 1:case"end":return r.stop()}},e)})),np.apply(this,arguments)}function X4(e){U4(e)}function Y4(e){return rp.apply(this,arguments)}function rp(){return rp=Gi(Or().mark(function e(t){return Or().wrap(function(r){for(;;)switch(r.prev=r.next){case 0:if(Ad===void 0){r.next=2;break}return r.abrupt("return",K4(t));case 2:X4(t);case 3:case"end":return r.stop()}},e)})),rp.apply(this,arguments)}const Q4=(e,t)=>(G4(e,t),()=>Y4(t));let Z4=Q4;function Id(){return Z4}const Ag=()=>({height:0,opacity:0}),B1=e=>{const{scrollHeight:t}=e;return{height:t,opacity:1}},J4=e=>({height:e?e.offsetHeight:0}),Ig=(e,t)=>(t==null?void 0:t.deadline)===!0||t.propertyName==="height",nd=function(){return{motionName:`${arguments.length>0&&arguments[0]!==void 0?arguments[0]:Mc}-motion-collapse`,onAppearStart:Ag,onEnterStart:Ag,onAppearActive:B1,onEnterActive:B1,onLeaveStart:J4,onLeaveActive:Ag,onAppearEnd:Ig,onEnterEnd:Ig,onLeaveEnd:Ig,motionDeadline:500}},Fi=(e,t,n)=>n!==void 0?n:`${e}-${t}`;function Tn(e,t){var n=Object.assign({},e);return Array.isArray(t)&&t.forEach(function(r){delete n[r]}),n}const jd=function(e){if(!e)return!1;if(e instanceof Element){if(e.offsetParent)return!0;if(e.getBBox){var t=e.getBBox(),n=t.width,r=t.height;if(n||r)return!0}if(e.getBoundingClientRect){var o=e.getBoundingClientRect(),l=o.width,c=o.height;if(l||c)return!0}}return!1},eP=e=>{const{componentCls:t,colorPrimary:n}=e;return{[t]:{position:"absolute",background:"transparent",pointerEvents:"none",boxSizing:"border-box",color:`var(--wave-color, ${n})`,boxShadow:"0 0 0 0 currentcolor",opacity:.2,"&.wave-motion-appear":{transition:[`box-shadow 0.4s ${e.motionEaseOutCirc}`,`opacity 2s ${e.motionEaseOutCirc}`].join(","),"&-active":{boxShadow:"0 0 0 6px currentcolor",opacity:0},"&.wave-quick":{transition:[`box-shadow ${e.motionDurationSlow} ${e.motionEaseInOut}`,`opacity ${e.motionDurationSlow} ${e.motionEaseInOut}`].join(",")}}}}},tP=Rj("Wave",e=>[eP(e)]),P$=`${Mc}-wave-target`;function jg(e){return e&&e!=="#fff"&&e!=="#ffffff"&&e!=="rgb(255, 255, 255)"&&e!=="rgba(255, 255, 255, 1)"&&!/rgba\((?:\d*, ){3}0\)/.test(e)&&e!=="transparent"}function nP(e){const{borderTopColor:t,borderColor:n,backgroundColor:r}=getComputedStyle(e);return jg(t)?t:jg(n)?n:jg(r)?r:null}function Pg(e){return Number.isNaN(e)?0:e}const rP=e=>{const{className:t,target:n,component:r,registerUnmount:o}=e,l=u.useRef(null),c=u.useRef(null);u.useEffect(()=>{c.current=o()},[]);const[d,m]=u.useState(null),[v,g]=u.useState([]),[p,y]=u.useState(0),[E,C]=u.useState(0),[x,b]=u.useState(0),[w,$]=u.useState(0),[O,T]=u.useState(!1),R={left:p,top:E,width:x,height:w,borderRadius:v.map(z=>`${z}px`).join(" ")};d&&(R["--wave-color"]=d);function M(){const z=getComputedStyle(n);m(nP(n));const P=z.position==="static",{borderLeftWidth:D,borderTopWidth:F}=z;y(P?n.offsetLeft:Pg(-parseFloat(D))),C(P?n.offsetTop:Pg(-parseFloat(F))),b(n.offsetWidth),$(n.offsetHeight);const{borderTopLeftRadius:W,borderTopRightRadius:V,borderBottomLeftRadius:q,borderBottomRightRadius:K}=z;g([W,V,K,q].map(_=>Pg(parseFloat(_))))}if(u.useEffect(()=>{if(n){const z=gn(()=>{M(),T(!0)});let P;return typeof ResizeObserver<"u"&&(P=new ResizeObserver(M),P.observe(n)),()=>{gn.cancel(z),P==null||P.disconnect()}}},[]),!O)return null;const N=(r==="Checkbox"||r==="Radio")&&(n==null?void 0:n.classList.contains(P$));return u.createElement(Xa,{visible:!0,motionAppear:!0,motionName:"wave-motion",motionDeadline:5e3,onAppearEnd:(z,P)=>{var D,F;if(P.deadline||P.propertyName==="opacity"){const W=(D=l.current)===null||D===void 0?void 0:D.parentElement;(F=c.current)===null||F===void 0||F.call(c).then(()=>{W==null||W.remove()})}return!1}},(z,P)=>{let{className:D}=z;return u.createElement("div",{ref:Ia(l,P),className:re(t,D,{"wave-quick":N}),style:R})})},aP=(e,t)=>{var n;const{component:r}=t;if(r==="Checkbox"&&!(!((n=e.querySelector("input"))===null||n===void 0)&&n.checked))return;const o=document.createElement("div");o.style.position="absolute",o.style.left="0px",o.style.top="0px",e==null||e.insertBefore(o,e==null?void 0:e.firstChild);const l=Id();let c=null;function d(){return c}c=l(u.createElement(rP,Object.assign({},t,{target:e,registerUnmount:d})),o)},oP=(e,t,n)=>{const{wave:r}=u.useContext(vt),[,o,l]=Rr(),c=jn(v=>{const g=e.current;if(r!=null&&r.disabled||!g)return;const p=g.querySelector(`.${P$}`)||g,{showEffect:y}=r||{};(y||aP)(p,{className:t,token:o,component:n,event:v,hashId:l})}),d=u.useRef(null);return v=>{gn.cancel(d.current),d.current=gn(()=>{c(v)})}},iP=e=>{const{children:t,disabled:n,component:r}=e,{getPrefixCls:o}=u.useContext(vt),l=u.useRef(null),c=o("wave"),[,d]=tP(c),m=oP(l,re(c,d),r);if(oe.useEffect(()=>{const g=l.current;if(!g||g.nodeType!==1||n)return;const p=y=>{!jd(y.target)||!g.getAttribute||g.getAttribute("disabled")||g.disabled||g.className.includes("disabled")||g.className.includes("-leave")||m(y)};return g.addEventListener("click",p,!0),()=>{g.removeEventListener("click",p,!0)}},[n]),!oe.isValidElement(t))return t??null;const v=ki(t)?Ia(Zl(t),l):l;return Zr(t,{ref:v})},Mo=e=>{const t=oe.useContext(Li);return oe.useMemo(()=>e?typeof e=="string"?e??t:typeof e=="function"?e(t):t:t,[e,t])},lP=e=>{const{componentCls:t}=e;return{[t]:{"&-block":{display:"flex",width:"100%"},"&-vertical":{flexDirection:"column"}}}},sP=e=>{const{componentCls:t,antCls:n}=e;return{[t]:{display:"inline-flex","&-rtl":{direction:"rtl"},"&-vertical":{flexDirection:"column"},"&-align":{flexDirection:"column","&-center":{alignItems:"center"},"&-start":{alignItems:"flex-start"},"&-end":{alignItems:"flex-end"},"&-baseline":{alignItems:"baseline"}},[`${t}-item:empty`]:{display:"none"},[`${t}-item > ${n}-badge-not-a-wrapper:only-child`]:{display:"block"}}}},cP=e=>{const{componentCls:t}=e;return{[t]:{"&-gap-row-small":{rowGap:e.spaceGapSmallSize},"&-gap-row-middle":{rowGap:e.spaceGapMiddleSize},"&-gap-row-large":{rowGap:e.spaceGapLargeSize},"&-gap-col-small":{columnGap:e.spaceGapSmallSize},"&-gap-col-middle":{columnGap:e.spaceGapMiddleSize},"&-gap-col-large":{columnGap:e.spaceGapLargeSize}}}},uP=wn("Space",e=>{const t=kt(e,{spaceGapSmallSize:e.paddingXS,spaceGapMiddleSize:e.padding,spaceGapLargeSize:e.paddingLG});return[sP(t),cP(t),lP(t)]},()=>({}),{resetStyle:!1});var z$=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const Pd=u.createContext(null),Xc=(e,t)=>{const n=u.useContext(Pd),r=u.useMemo(()=>{if(!n)return"";const{compactDirection:o,isFirstItem:l,isLastItem:c}=n,d=o==="vertical"?"-vertical-":"-";return re(`${e}-compact${d}item`,{[`${e}-compact${d}first-item`]:l,[`${e}-compact${d}last-item`]:c,[`${e}-compact${d}item-rtl`]:t==="rtl"})},[e,t,n]);return{compactSize:n==null?void 0:n.compactSize,compactDirection:n==null?void 0:n.compactDirection,compactItemClassnames:r}},fP=e=>{const{children:t}=e;return u.createElement(Pd.Provider,{value:null},t)},dP=e=>{const{children:t}=e,n=z$(e,["children"]);return u.createElement(Pd.Provider,{value:u.useMemo(()=>n,[n])},t)},iW=e=>{const{getPrefixCls:t,direction:n}=u.useContext(vt),{size:r,direction:o,block:l,prefixCls:c,className:d,rootClassName:m,children:v}=e,g=z$(e,["size","direction","block","prefixCls","className","rootClassName","children"]),p=Mo(O=>r??O),y=t("space-compact",c),[E,C]=uP(y),x=re(y,C,{[`${y}-rtl`]:n==="rtl",[`${y}-block`]:l,[`${y}-vertical`]:o==="vertical"},d,m),b=u.useContext(Pd),w=Pr(v),$=u.useMemo(()=>w.map((O,T)=>{const R=(O==null?void 0:O.key)||`${y}-item-${T}`;return u.createElement(dP,{key:R,compactSize:p,compactDirection:o,isFirstItem:T===0&&(!b||(b==null?void 0:b.isFirstItem)),isLastItem:T===w.length-1&&(!b||(b==null?void 0:b.isLastItem))},O)}),[r,w,b]);return w.length===0?null:E(u.createElement("div",Object.assign({className:x},g),$))};var mP=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const D$=u.createContext(void 0),vP=e=>{const{getPrefixCls:t,direction:n}=u.useContext(vt),{prefixCls:r,size:o,className:l}=e,c=mP(e,["prefixCls","size","className"]),d=t("btn-group",r),[,,m]=Rr(),v=u.useMemo(()=>{switch(o){case"large":return"lg";case"small":return"sm";default:return""}},[o]),g=re(d,{[`${d}-${v}`]:v,[`${d}-rtl`]:n==="rtl"},l,m);return u.createElement(D$.Provider,{value:o},u.createElement("div",Object.assign({},c,{className:g})))},L1=/^[\u4E00-\u9FA5]{2}$/,ap=L1.test.bind(L1);function d0(e){return e==="danger"?{danger:!0}:{type:e}}function H1(e){return typeof e=="string"}function zg(e){return e==="text"||e==="link"}function gP(e,t){if(e==null)return;const n=t?" ":"";return typeof e!="string"&&typeof e!="number"&&H1(e.type)&&ap(e.props.children)?Zr(e,{children:e.props.children.split("").join(n)}):H1(e)?ap(e)?oe.createElement("span",null,e.split("").join(n)):oe.createElement("span",null,e):R$(e)?oe.createElement("span",null,e):e}function hP(e,t){let n=!1;const r=[];return oe.Children.forEach(e,o=>{const l=typeof o,c=l==="string"||l==="number";if(n&&c){const d=r.length-1,m=r[d];r[d]=`${m}${o}`}else r.push(o);n=c}),oe.Children.map(r,o=>gP(o,t))}["default","primary","danger"].concat(Te(ai));const op=u.forwardRef((e,t)=>{const{className:n,style:r,children:o,prefixCls:l}=e,c=re(`${l}-icon`,n);return oe.createElement("span",{ref:t,className:c,style:r},o)}),F1=u.forwardRef((e,t)=>{const{prefixCls:n,className:r,style:o,iconClassName:l}=e,c=re(`${n}-loading-icon`,r);return oe.createElement(op,{prefixCls:n,className:c,style:o,ref:t},oe.createElement(Uc,{className:l}))}),Dg=()=>({width:0,opacity:0,transform:"scale(0)"}),Bg=e=>({width:e.scrollWidth,opacity:1,transform:"scale(1)"}),pP=e=>{const{prefixCls:t,loading:n,existIcon:r,className:o,style:l,mount:c}=e,d=!!n;return r?oe.createElement(F1,{prefixCls:t,className:o,style:l}):oe.createElement(Xa,{visible:d,motionName:`${t}-loading-icon-motion`,motionAppear:!c,motionEnter:!c,motionLeave:!c,removeOnLeave:!0,onAppearStart:Dg,onAppearActive:Bg,onEnterStart:Dg,onEnterActive:Bg,onLeaveStart:Bg,onLeaveActive:Dg},(m,v)=>{let{className:g,style:p}=m;const y=Object.assign(Object.assign({},l),p);return oe.createElement(F1,{prefixCls:t,className:re(o,g),style:y,ref:v})})},V1=(e,t)=>({[`> span, > ${e}`]:{"&:not(:last-child)":{[`&, & > ${e}`]:{"&:not(:disabled)":{borderInlineEndColor:t}}},"&:not(:first-child)":{[`&, & > ${e}`]:{"&:not(:disabled)":{borderInlineStartColor:t}}}}}),yP=e=>{const{componentCls:t,fontSize:n,lineWidth:r,groupBorderColor:o,colorErrorHover:l}=e;return{[`${t}-group`]:[{position:"relative",display:"inline-flex",[`> span, > ${t}`]:{"&:not(:last-child)":{[`&, & > ${t}`]:{borderStartEndRadius:0,borderEndEndRadius:0}},"&:not(:first-child)":{marginInlineStart:e.calc(r).mul(-1).equal(),[`&, & > ${t}`]:{borderStartStartRadius:0,borderEndStartRadius:0}}},[t]:{position:"relative",zIndex:1,"&:hover, &:focus, &:active":{zIndex:2},"&[disabled]":{zIndex:0}},[`${t}-icon-only`]:{fontSize:n}},V1(`${t}-primary`,o),V1(`${t}-danger`,l)]}};var bP=["b"],SP=["v"],Lg=function(t){return Math.round(Number(t||0))},CP=function(t){if(t instanceof Jt)return t;if(t&&lt(t)==="object"&&"h"in t&&"b"in t){var n=t,r=n.b,o=wt(n,bP);return Z(Z({},o),{},{v:r})}return typeof t=="string"&&/hsb/.test(t)?t.replace(/hsb/,"hsv"):t},_c=function(e){$o(n,e);var t=Oo(n);function n(r){return qn(this,n),t.call(this,CP(r))}return Wn(n,[{key:"toHsbString",value:function(){var o=this.toHsb(),l=Lg(o.s*100),c=Lg(o.b*100),d=Lg(o.h),m=o.a,v="hsb(".concat(d,", ").concat(l,"%, ").concat(c,"%)"),g="hsba(".concat(d,", ").concat(l,"%, ").concat(c,"%, ").concat(m.toFixed(m===0?0:2),")");return m===1?v:g}},{key:"toHsb",value:function(){var o=this.toHsv(),l=o.v,c=wt(o,SP);return Z(Z({},c),{},{b:l,a:this.a})}}]),n}(Jt),xP=function(t){return t instanceof _c?t:new _c(t)};xP("#1677ff");const EP=(e,t)=>(e==null?void 0:e.replace(/[^\w/]/g,"").slice(0,t?8:6))||"",wP=(e,t)=>e?EP(e,t):"";let $P=function(){function e(t){qn(this,e);var n;if(this.cleared=!1,t instanceof e){this.metaColor=t.metaColor.clone(),this.colors=(n=t.colors)===null||n===void 0?void 0:n.map(o=>({color:new e(o.color),percent:o.percent})),this.cleared=t.cleared;return}const r=Array.isArray(t);r&&t.length?(this.colors=t.map(o=>{let{color:l,percent:c}=o;return{color:new e(l),percent:c}}),this.metaColor=new _c(this.colors[0].color.metaColor)):this.metaColor=new _c(r?"":t),(!t||r&&!this.colors)&&(this.metaColor=this.metaColor.setA(0),this.cleared=!0)}return Wn(e,[{key:"toHsb",value:function(){return this.metaColor.toHsb()}},{key:"toHsbString",value:function(){return this.metaColor.toHsbString()}},{key:"toHex",value:function(){return wP(this.toHexString(),this.metaColor.a<1)}},{key:"toHexString",value:function(){return this.metaColor.toHexString()}},{key:"toRgb",value:function(){return this.metaColor.toRgb()}},{key:"toRgbString",value:function(){return this.metaColor.toRgbString()}},{key:"isGradient",value:function(){return!!this.colors&&!this.cleared}},{key:"getColors",value:function(){return this.colors||[{color:this,percent:0}]}},{key:"toCssString",value:function(){const{colors:n}=this;return n?`linear-gradient(90deg, ${n.map(o=>`${o.color.toRgbString()} ${o.percent}%`).join(", ")})`:this.metaColor.toRgbString()}},{key:"equals",value:function(n){return!n||this.isGradient()!==n.isGradient()?!1:this.isGradient()?this.colors.length===n.colors.length&&this.colors.every((r,o)=>{const l=n.colors[o];return r.percent===l.percent&&r.color.equals(l.color)}):this.toHexString()===n.toHexString()}}])}();var B$=oe.forwardRef(function(e,t){var n=e.prefixCls,r=e.forceRender,o=e.className,l=e.style,c=e.children,d=e.isActive,m=e.role,v=e.classNames,g=e.styles,p=oe.useState(d||r),y=de(p,2),E=y[0],C=y[1];return oe.useEffect(function(){(r||d)&&C(!0)},[r,d]),E?oe.createElement("div",{ref:t,className:re("".concat(n,"-content"),Y(Y({},"".concat(n,"-content-active"),d),"".concat(n,"-content-inactive"),!d),o),style:l,role:m},oe.createElement("div",{className:re("".concat(n,"-content-box"),v==null?void 0:v.body),style:g==null?void 0:g.body},c)):null});B$.displayName="PanelContent";var OP=["showArrow","headerClass","isActive","onItemClick","forceRender","className","classNames","styles","prefixCls","collapsible","accordion","panelKey","extra","header","expandIcon","openMotion","destroyInactivePanel","children"],L$=oe.forwardRef(function(e,t){var n=e.showArrow,r=n===void 0?!0:n,o=e.headerClass,l=e.isActive,c=e.onItemClick,d=e.forceRender,m=e.className,v=e.classNames,g=v===void 0?{}:v,p=e.styles,y=p===void 0?{}:p,E=e.prefixCls,C=e.collapsible,x=e.accordion,b=e.panelKey,w=e.extra,$=e.header,O=e.expandIcon,T=e.openMotion,R=e.destroyInactivePanel,M=e.children,N=wt(e,OP),z=C==="disabled",P=w!=null&&typeof w!="boolean",D=Y(Y(Y({onClick:function(){c==null||c(b)},onKeyDown:function(j){(j.key==="Enter"||j.keyCode===ct.ENTER||j.which===ct.ENTER)&&(c==null||c(b))},role:x?"tab":"button"},"aria-expanded",l),"aria-disabled",z),"tabIndex",z?-1:0),F=typeof O=="function"?O(e):oe.createElement("i",{className:"arrow"}),W=F&&oe.createElement("div",Ie({className:"".concat(E,"-expand-icon")},["header","icon"].includes(C)?D:{}),F),V=re("".concat(E,"-item"),Y(Y({},"".concat(E,"-item-active"),l),"".concat(E,"-item-disabled"),z),m),q=re(o,"".concat(E,"-header"),Y({},"".concat(E,"-collapsible-").concat(C),!!C),g.header),K=Z({className:q,style:y.header},["header","icon"].includes(C)?{}:D);return oe.createElement("div",Ie({},N,{ref:t,className:V}),oe.createElement("div",K,r&&W,oe.createElement("span",Ie({className:"".concat(E,"-header-text")},C==="header"?D:{}),$),P&&oe.createElement("div",{className:"".concat(E,"-extra")},w)),oe.createElement(Xa,Ie({visible:l,leavedClassName:"".concat(E,"-content-hidden")},T,{forceRender:d,removeOnLeave:R}),function(_,j){var B=_.className,k=_.style;return oe.createElement(B$,{ref:j,prefixCls:E,className:B,classNames:g,style:k,styles:y,isActive:l,forceRender:d,role:x?"tabpanel":void 0},M)}))}),RP=["children","label","key","collapsible","onItemClick","destroyInactivePanel"],TP=function(t,n){var r=n.prefixCls,o=n.accordion,l=n.collapsible,c=n.destroyInactivePanel,d=n.onItemClick,m=n.activeKey,v=n.openMotion,g=n.expandIcon;return t.map(function(p,y){var E=p.children,C=p.label,x=p.key,b=p.collapsible,w=p.onItemClick,$=p.destroyInactivePanel,O=wt(p,RP),T=String(x??y),R=b??l,M=$??c,N=function(D){R!=="disabled"&&(d(D),w==null||w(D))},z=!1;return o?z=m[0]===T:z=m.indexOf(T)>-1,oe.createElement(L$,Ie({},O,{prefixCls:r,key:T,panelKey:T,isActive:z,accordion:o,openMotion:v,expandIcon:g,header:C,collapsible:R,onItemClick:N,destroyInactivePanel:M}),E)})},MP=function(t,n,r){if(!t)return null;var o=r.prefixCls,l=r.accordion,c=r.collapsible,d=r.destroyInactivePanel,m=r.onItemClick,v=r.activeKey,g=r.openMotion,p=r.expandIcon,y=t.key||String(n),E=t.props,C=E.header,x=E.headerClass,b=E.destroyInactivePanel,w=E.collapsible,$=E.onItemClick,O=!1;l?O=v[0]===y:O=v.indexOf(y)>-1;var T=w??c,R=function(z){T!=="disabled"&&(m(z),$==null||$(z))},M={key:y,panelKey:y,header:C,headerClass:x,isActive:O,prefixCls:o,destroyInactivePanel:b??d,openMotion:g,accordion:l,children:t.props.children,onItemClick:R,expandIcon:p,collapsible:T};return typeof t.type=="string"?t:(Object.keys(M).forEach(function(N){typeof M[N]>"u"&&delete M[N]}),oe.cloneElement(t,M))};function _P(e,t,n){return Array.isArray(e)?TP(e,n):Pr(t).map(function(r,o){return MP(r,o,n)})}function NP(e){var t=e;if(!Array.isArray(t)){var n=lt(t);t=n==="number"||n==="string"?[t]:[]}return t.map(function(r){return String(r)})}var AP=oe.forwardRef(function(e,t){var n=e.prefixCls,r=n===void 0?"rc-collapse":n,o=e.destroyInactivePanel,l=o===void 0?!1:o,c=e.style,d=e.accordion,m=e.className,v=e.children,g=e.collapsible,p=e.openMotion,y=e.expandIcon,E=e.activeKey,C=e.defaultActiveKey,x=e.onChange,b=e.items,w=re(r,m),$=gr([],{value:E,onChange:function(P){return x==null?void 0:x(P)},defaultValue:C,postState:NP}),O=de($,2),T=O[0],R=O[1],M=function(P){return R(function(){if(d)return T[0]===P?[]:[P];var D=T.indexOf(P),F=D>-1;return F?T.filter(function(W){return W!==P}):[].concat(Te(T),[P])})};vr(!v,"[rc-collapse] `children` will be removed in next major version. Please use `items` instead.");var N=_P(b,v,{prefixCls:r,accordion:d,openMotion:p,expandIcon:y,collapsible:g,destroyInactivePanel:l,onItemClick:M,activeKey:T});return oe.createElement("div",Ie({ref:t,className:w,style:c,role:d?"tablist":void 0},Ga(e,{aria:!0,data:!0})),N)});const m0=Object.assign(AP,{Panel:L$});m0.Panel;const IP=u.forwardRef((e,t)=>{const{getPrefixCls:n}=u.useContext(vt),{prefixCls:r,className:o,showArrow:l=!0}=e,c=n("collapse",r),d=re({[`${c}-no-arrow`]:!l},o);return u.createElement(m0.Panel,Object.assign({ref:t},e,{prefixCls:c,className:d}))}),v0=e=>({[e.componentCls]:{[`${e.antCls}-motion-collapse-legacy`]:{overflow:"hidden","&-active":{transition:`height ${e.motionDurationMid} ${e.motionEaseInOut},
        opacity ${e.motionDurationMid} ${e.motionEaseInOut} !important`}},[`${e.antCls}-motion-collapse`]:{overflow:"hidden",transition:`height ${e.motionDurationMid} ${e.motionEaseInOut},
        opacity ${e.motionDurationMid} ${e.motionEaseInOut} !important`}}}),jP=e=>({animationDuration:e,animationFillMode:"both"}),PP=e=>({animationDuration:e,animationFillMode:"both"}),zd=function(e,t,n,r){const l=(arguments.length>4&&arguments[4]!==void 0?arguments[4]:!1)?"&":"";return{[`
      ${l}${e}-enter,
      ${l}${e}-appear
    `]:Object.assign(Object.assign({},jP(r)),{animationPlayState:"paused"}),[`${l}${e}-leave`]:Object.assign(Object.assign({},PP(r)),{animationPlayState:"paused"}),[`
      ${l}${e}-enter${e}-enter-active,
      ${l}${e}-appear${e}-appear-active
    `]:{animationName:t,animationPlayState:"running"},[`${l}${e}-leave${e}-leave-active`]:{animationName:n,animationPlayState:"running",pointerEvents:"none"}}},zP=new Pt("antFadeIn",{"0%":{opacity:0},"100%":{opacity:1}}),DP=new Pt("antFadeOut",{"0%":{opacity:1},"100%":{opacity:0}}),BP=function(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;const{antCls:n}=e,r=`${n}-fade`,o=t?"&":"";return[zd(r,zP,DP,e.motionDurationMid,t),{[`
        ${o}${r}-enter,
        ${o}${r}-appear
      `]:{opacity:0,animationTimingFunction:"linear"},[`${o}${r}-leave`]:{animationTimingFunction:"linear"}}]},LP=new Pt("antMoveDownIn",{"0%":{transform:"translate3d(0, 100%, 0)",transformOrigin:"0 0",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1}}),HP=new Pt("antMoveDownOut",{"0%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1},"100%":{transform:"translate3d(0, 100%, 0)",transformOrigin:"0 0",opacity:0}}),FP=new Pt("antMoveLeftIn",{"0%":{transform:"translate3d(-100%, 0, 0)",transformOrigin:"0 0",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1}}),VP=new Pt("antMoveLeftOut",{"0%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1},"100%":{transform:"translate3d(-100%, 0, 0)",transformOrigin:"0 0",opacity:0}}),UP=new Pt("antMoveRightIn",{"0%":{transform:"translate3d(100%, 0, 0)",transformOrigin:"0 0",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1}}),qP=new Pt("antMoveRightOut",{"0%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1},"100%":{transform:"translate3d(100%, 0, 0)",transformOrigin:"0 0",opacity:0}}),WP=new Pt("antMoveUpIn",{"0%":{transform:"translate3d(0, -100%, 0)",transformOrigin:"0 0",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1}}),kP=new Pt("antMoveUpOut",{"0%":{transform:"translate3d(0, 0, 0)",transformOrigin:"0 0",opacity:1},"100%":{transform:"translate3d(0, -100%, 0)",transformOrigin:"0 0",opacity:0}}),GP={"move-up":{inKeyframes:WP,outKeyframes:kP},"move-down":{inKeyframes:LP,outKeyframes:HP},"move-left":{inKeyframes:FP,outKeyframes:VP},"move-right":{inKeyframes:UP,outKeyframes:qP}},U1=(e,t)=>{const{antCls:n}=e,r=`${n}-${t}`,{inKeyframes:o,outKeyframes:l}=GP[t];return[zd(r,o,l,e.motionDurationMid),{[`
        ${r}-enter,
        ${r}-appear
      `]:{opacity:0,animationTimingFunction:e.motionEaseOutCirc},[`${r}-leave`]:{animationTimingFunction:e.motionEaseInOutCirc}}]},H$=new Pt("antSlideUpIn",{"0%":{transform:"scaleY(0.8)",transformOrigin:"0% 0%",opacity:0},"100%":{transform:"scaleY(1)",transformOrigin:"0% 0%",opacity:1}}),F$=new Pt("antSlideUpOut",{"0%":{transform:"scaleY(1)",transformOrigin:"0% 0%",opacity:1},"100%":{transform:"scaleY(0.8)",transformOrigin:"0% 0%",opacity:0}}),V$=new Pt("antSlideDownIn",{"0%":{transform:"scaleY(0.8)",transformOrigin:"100% 100%",opacity:0},"100%":{transform:"scaleY(1)",transformOrigin:"100% 100%",opacity:1}}),U$=new Pt("antSlideDownOut",{"0%":{transform:"scaleY(1)",transformOrigin:"100% 100%",opacity:1},"100%":{transform:"scaleY(0.8)",transformOrigin:"100% 100%",opacity:0}}),KP=new Pt("antSlideLeftIn",{"0%":{transform:"scaleX(0.8)",transformOrigin:"0% 0%",opacity:0},"100%":{transform:"scaleX(1)",transformOrigin:"0% 0%",opacity:1}}),XP=new Pt("antSlideLeftOut",{"0%":{transform:"scaleX(1)",transformOrigin:"0% 0%",opacity:1},"100%":{transform:"scaleX(0.8)",transformOrigin:"0% 0%",opacity:0}}),YP=new Pt("antSlideRightIn",{"0%":{transform:"scaleX(0.8)",transformOrigin:"100% 0%",opacity:0},"100%":{transform:"scaleX(1)",transformOrigin:"100% 0%",opacity:1}}),QP=new Pt("antSlideRightOut",{"0%":{transform:"scaleX(1)",transformOrigin:"100% 0%",opacity:1},"100%":{transform:"scaleX(0.8)",transformOrigin:"100% 0%",opacity:0}}),ZP={"slide-up":{inKeyframes:H$,outKeyframes:F$},"slide-down":{inKeyframes:V$,outKeyframes:U$},"slide-left":{inKeyframes:KP,outKeyframes:XP},"slide-right":{inKeyframes:YP,outKeyframes:QP}},rd=(e,t)=>{const{antCls:n}=e,r=`${n}-${t}`,{inKeyframes:o,outKeyframes:l}=ZP[t];return[zd(r,o,l,e.motionDurationMid),{[`
      ${r}-enter,
      ${r}-appear
    `]:{transform:"scale(0)",transformOrigin:"0% 0%",opacity:0,animationTimingFunction:e.motionEaseOutQuint,"&-prepare":{transform:"scale(1)"}},[`${r}-leave`]:{animationTimingFunction:e.motionEaseInQuint}}]},g0=new Pt("antZoomIn",{"0%":{transform:"scale(0.2)",opacity:0},"100%":{transform:"scale(1)",opacity:1}}),JP=new Pt("antZoomOut",{"0%":{transform:"scale(1)"},"100%":{transform:"scale(0.2)",opacity:0}}),q1=new Pt("antZoomBigIn",{"0%":{transform:"scale(0.8)",opacity:0},"100%":{transform:"scale(1)",opacity:1}}),W1=new Pt("antZoomBigOut",{"0%":{transform:"scale(1)"},"100%":{transform:"scale(0.8)",opacity:0}}),ez=new Pt("antZoomUpIn",{"0%":{transform:"scale(0.8)",transformOrigin:"50% 0%",opacity:0},"100%":{transform:"scale(1)",transformOrigin:"50% 0%"}}),tz=new Pt("antZoomUpOut",{"0%":{transform:"scale(1)",transformOrigin:"50% 0%"},"100%":{transform:"scale(0.8)",transformOrigin:"50% 0%",opacity:0}}),nz=new Pt("antZoomLeftIn",{"0%":{transform:"scale(0.8)",transformOrigin:"0% 50%",opacity:0},"100%":{transform:"scale(1)",transformOrigin:"0% 50%"}}),rz=new Pt("antZoomLeftOut",{"0%":{transform:"scale(1)",transformOrigin:"0% 50%"},"100%":{transform:"scale(0.8)",transformOrigin:"0% 50%",opacity:0}}),az=new Pt("antZoomRightIn",{"0%":{transform:"scale(0.8)",transformOrigin:"100% 50%",opacity:0},"100%":{transform:"scale(1)",transformOrigin:"100% 50%"}}),oz=new Pt("antZoomRightOut",{"0%":{transform:"scale(1)",transformOrigin:"100% 50%"},"100%":{transform:"scale(0.8)",transformOrigin:"100% 50%",opacity:0}}),iz=new Pt("antZoomDownIn",{"0%":{transform:"scale(0.8)",transformOrigin:"50% 100%",opacity:0},"100%":{transform:"scale(1)",transformOrigin:"50% 100%"}}),lz=new Pt("antZoomDownOut",{"0%":{transform:"scale(1)",transformOrigin:"50% 100%"},"100%":{transform:"scale(0.8)",transformOrigin:"50% 100%",opacity:0}}),sz={zoom:{inKeyframes:g0,outKeyframes:JP},"zoom-big":{inKeyframes:q1,outKeyframes:W1},"zoom-big-fast":{inKeyframes:q1,outKeyframes:W1},"zoom-left":{inKeyframes:nz,outKeyframes:rz},"zoom-right":{inKeyframes:az,outKeyframes:oz},"zoom-up":{inKeyframes:ez,outKeyframes:tz},"zoom-down":{inKeyframes:iz,outKeyframes:lz}},Dd=(e,t)=>{const{antCls:n}=e,r=`${n}-${t}`,{inKeyframes:o,outKeyframes:l}=sz[t];return[zd(r,o,l,t==="zoom-big-fast"?e.motionDurationFast:e.motionDurationMid),{[`
        ${r}-enter,
        ${r}-appear
      `]:{transform:"scale(0)",opacity:0,animationTimingFunction:e.motionEaseOutCirc,"&-prepare":{transform:"none"}},[`${r}-leave`]:{animationTimingFunction:e.motionEaseInOutCirc}}]},cz=e=>{const{componentCls:t,contentBg:n,padding:r,headerBg:o,headerPadding:l,collapseHeaderPaddingSM:c,collapseHeaderPaddingLG:d,collapsePanelBorderRadius:m,lineWidth:v,lineType:g,colorBorder:p,colorText:y,colorTextHeading:E,colorTextDisabled:C,fontSizeLG:x,lineHeight:b,lineHeightLG:w,marginSM:$,paddingSM:O,paddingLG:T,paddingXS:R,motionDurationSlow:M,fontSizeIcon:N,contentPadding:z,fontHeight:P,fontHeightLG:D}=e,F=`${he(v)} ${g} ${p}`;return{[t]:Object.assign(Object.assign({},Tr(e)),{backgroundColor:o,border:F,borderRadius:m,"&-rtl":{direction:"rtl"},[`& > ${t}-item`]:{borderBottom:F,"&:first-child":{[`
            &,
            & > ${t}-header`]:{borderRadius:`${he(m)} ${he(m)} 0 0`}},"&:last-child":{[`
            &,
            & > ${t}-header`]:{borderRadius:`0 0 ${he(m)} ${he(m)}`}},[`> ${t}-header`]:Object.assign(Object.assign({position:"relative",display:"flex",flexWrap:"nowrap",alignItems:"flex-start",padding:l,color:E,lineHeight:b,cursor:"pointer",transition:`all ${M}, visibility 0s`},Gc(e)),{[`> ${t}-header-text`]:{flex:"auto"},[`${t}-expand-icon`]:{height:P,display:"flex",alignItems:"center",paddingInlineEnd:$},[`${t}-arrow`]:Object.assign(Object.assign({},es()),{fontSize:N,transition:`transform ${M}`,svg:{transition:`transform ${M}`}}),[`${t}-header-text`]:{marginInlineEnd:"auto"}}),[`${t}-collapsible-header`]:{cursor:"default",[`${t}-header-text`]:{flex:"none",cursor:"pointer"}},[`${t}-collapsible-icon`]:{cursor:"unset",[`${t}-expand-icon`]:{cursor:"pointer"}}},[`${t}-content`]:{color:y,backgroundColor:n,borderTop:F,[`& > ${t}-content-box`]:{padding:z},"&-hidden":{display:"none"}},"&-small":{[`> ${t}-item`]:{[`> ${t}-header`]:{padding:c,paddingInlineStart:R,[`> ${t}-expand-icon`]:{marginInlineStart:e.calc(O).sub(R).equal()}},[`> ${t}-content > ${t}-content-box`]:{padding:O}}},"&-large":{[`> ${t}-item`]:{fontSize:x,lineHeight:w,[`> ${t}-header`]:{padding:d,paddingInlineStart:r,[`> ${t}-expand-icon`]:{height:D,marginInlineStart:e.calc(T).sub(r).equal()}},[`> ${t}-content > ${t}-content-box`]:{padding:T}}},[`${t}-item:last-child`]:{borderBottom:0,[`> ${t}-content`]:{borderRadius:`0 0 ${he(m)} ${he(m)}`}},[`& ${t}-item-disabled > ${t}-header`]:{"\n          &,\n          & > .arrow\n        ":{color:C,cursor:"not-allowed"}},[`&${t}-icon-position-end`]:{[`& > ${t}-item`]:{[`> ${t}-header`]:{[`${t}-expand-icon`]:{order:1,paddingInlineEnd:0,paddingInlineStart:$}}}}})}},uz=e=>{const{componentCls:t}=e,n=`> ${t}-item > ${t}-header ${t}-arrow`;return{[`${t}-rtl`]:{[n]:{transform:"rotate(180deg)"}}}},fz=e=>{const{componentCls:t,headerBg:n,paddingXXS:r,colorBorder:o}=e;return{[`${t}-borderless`]:{backgroundColor:n,border:0,[`> ${t}-item`]:{borderBottom:`1px solid ${o}`},[`
        > ${t}-item:last-child,
        > ${t}-item:last-child ${t}-header
      `]:{borderRadius:0},[`> ${t}-item:last-child`]:{borderBottom:0},[`> ${t}-item > ${t}-content`]:{backgroundColor:"transparent",borderTop:0},[`> ${t}-item > ${t}-content > ${t}-content-box`]:{paddingTop:r}}}},dz=e=>{const{componentCls:t,paddingSM:n}=e;return{[`${t}-ghost`]:{backgroundColor:"transparent",border:0,[`> ${t}-item`]:{borderBottom:0,[`> ${t}-content`]:{backgroundColor:"transparent",border:0,[`> ${t}-content-box`]:{paddingBlock:n}}}}}},mz=e=>({headerPadding:`${e.paddingSM}px ${e.padding}px`,headerBg:e.colorFillAlter,contentPadding:`${e.padding}px 16px`,contentBg:e.colorBgContainer}),vz=wn("Collapse",e=>{const t=kt(e,{collapseHeaderPaddingSM:`${he(e.paddingXS)} ${he(e.paddingSM)}`,collapseHeaderPaddingLG:`${he(e.padding)} ${he(e.paddingLG)}`,collapsePanelBorderRadius:e.borderRadiusLG});return[cz(t),fz(t),dz(t),uz(t),v0(t)]},mz),gz=u.forwardRef((e,t)=>{const{getPrefixCls:n,direction:r,expandIcon:o,className:l,style:c}=zr("collapse"),{prefixCls:d,className:m,rootClassName:v,style:g,bordered:p=!0,ghost:y,size:E,expandIconPosition:C="start",children:x,expandIcon:b}=e,w=Mo(V=>{var q;return(q=E??V)!==null&&q!==void 0?q:"middle"}),$=n("collapse",d),O=n(),[T,R,M]=vz($),N=u.useMemo(()=>C==="left"?"start":C==="right"?"end":C,[C]),z=b??o,P=u.useCallback(function(){let V=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{};const q=typeof z=="function"?z(V):u.createElement(Vh,{rotate:V.isActive?r==="rtl"?-90:90:void 0,"aria-label":V.isActive?"expanded":"collapsed"});return Zr(q,()=>{var K;return{className:re((K=q==null?void 0:q.props)===null||K===void 0?void 0:K.className,`${$}-arrow`)}})},[z,$]),D=re(`${$}-icon-position-${N}`,{[`${$}-borderless`]:!p,[`${$}-rtl`]:r==="rtl",[`${$}-ghost`]:!!y,[`${$}-${w}`]:w!=="middle"},l,m,v,R,M),F=Object.assign(Object.assign({},nd(O)),{motionAppear:!1,leavedClassName:`${$}-content-hidden`}),W=u.useMemo(()=>x?Pr(x).map((V,q)=>{var K,_;const j=V.props;if(j!=null&&j.disabled){const B=(K=V.key)!==null&&K!==void 0?K:String(q),k=Object.assign(Object.assign({},Tn(V.props,["disabled"])),{key:B,collapsible:(_=j.collapsible)!==null&&_!==void 0?_:"disabled"});return Zr(V,k)}return V}):null,[x]);return T(u.createElement(m0,Object.assign({ref:t,openMotion:F},Tn(e,["rootClassName"]),{expandIcon:P,prefixCls:$,className:D,style:Object.assign(Object.assign({},c),g)}),W))}),hz=Object.assign(gz,{Panel:IP}),pz=(e,t)=>{const{r:n,g:r,b:o,a:l}=e.toRgb(),c=new _c(e.toRgbString()).onBackground(t).toHsv();return l<=.5?c.v>.5:n*.299+r*.587+o*.114>192},q$=e=>{const{paddingInline:t,onlyIconSize:n}=e;return kt(e,{buttonPaddingHorizontal:t,buttonPaddingVertical:0,buttonIconOnlyFontSize:n})},W$=e=>{var t,n,r,o,l,c;const d=(t=e.contentFontSize)!==null&&t!==void 0?t:e.fontSize,m=(n=e.contentFontSizeSM)!==null&&n!==void 0?n:e.fontSize,v=(r=e.contentFontSizeLG)!==null&&r!==void 0?r:e.fontSizeLG,g=(o=e.contentLineHeight)!==null&&o!==void 0?o:Pf(d),p=(l=e.contentLineHeightSM)!==null&&l!==void 0?l:Pf(m),y=(c=e.contentLineHeightLG)!==null&&c!==void 0?c:Pf(v),E=pz(new $P(e.colorBgSolid),"#fff")?"#000":"#fff",C=ai.reduce((x,b)=>Object.assign(Object.assign({},x),{[`${b}ShadowColor`]:`0 ${he(e.controlOutlineWidth)} 0 ${dc(e[`${b}1`],e.colorBgContainer)}`}),{});return Object.assign(Object.assign({},C),{fontWeight:400,defaultShadow:`0 ${e.controlOutlineWidth}px 0 ${e.controlTmpOutline}`,primaryShadow:`0 ${e.controlOutlineWidth}px 0 ${e.controlOutline}`,dangerShadow:`0 ${e.controlOutlineWidth}px 0 ${e.colorErrorOutline}`,primaryColor:e.colorTextLightSolid,dangerColor:e.colorTextLightSolid,borderColorDisabled:e.colorBorder,defaultGhostColor:e.colorBgContainer,ghostBg:"transparent",defaultGhostBorderColor:e.colorBgContainer,paddingInline:e.paddingContentHorizontal-e.lineWidth,paddingInlineLG:e.paddingContentHorizontal-e.lineWidth,paddingInlineSM:8-e.lineWidth,onlyIconSize:"inherit",onlyIconSizeSM:"inherit",onlyIconSizeLG:"inherit",groupBorderColor:e.colorPrimaryHover,linkHoverBg:"transparent",textTextColor:e.colorText,textTextHoverColor:e.colorText,textTextActiveColor:e.colorText,textHoverBg:e.colorFillTertiary,defaultColor:e.colorText,defaultBg:e.colorBgContainer,defaultBorderColor:e.colorBorder,defaultBorderColorDisabled:e.colorBorder,defaultHoverBg:e.colorBgContainer,defaultHoverColor:e.colorPrimaryHover,defaultHoverBorderColor:e.colorPrimaryHover,defaultActiveBg:e.colorBgContainer,defaultActiveColor:e.colorPrimaryActive,defaultActiveBorderColor:e.colorPrimaryActive,solidTextColor:E,contentFontSize:d,contentFontSizeSM:m,contentFontSizeLG:v,contentLineHeight:g,contentLineHeightSM:p,contentLineHeightLG:y,paddingBlock:Math.max((e.controlHeight-d*g)/2-e.lineWidth,0),paddingBlockSM:Math.max((e.controlHeightSM-m*p)/2-e.lineWidth,0),paddingBlockLG:Math.max((e.controlHeightLG-v*y)/2-e.lineWidth,0)})},yz=e=>{const{componentCls:t,iconCls:n,fontWeight:r,opacityLoading:o,motionDurationSlow:l,motionEaseInOut:c,marginXS:d,calc:m}=e;return{[t]:{outline:"none",position:"relative",display:"inline-flex",gap:e.marginXS,alignItems:"center",justifyContent:"center",fontWeight:r,whiteSpace:"nowrap",textAlign:"center",backgroundImage:"none",background:"transparent",border:`${he(e.lineWidth)} ${e.lineType} transparent`,cursor:"pointer",transition:`all ${e.motionDurationMid} ${e.motionEaseInOut}`,userSelect:"none",touchAction:"manipulation",color:e.colorText,"&:disabled > *":{pointerEvents:"none"},[`${t}-icon > svg`]:es(),"> a":{color:"currentColor"},"&:not(:disabled)":Gc(e),[`&${t}-two-chinese-chars::first-letter`]:{letterSpacing:"0.34em"},[`&${t}-two-chinese-chars > *:not(${n})`]:{marginInlineEnd:"-0.34em",letterSpacing:"0.34em"},[`&${t}-icon-only`]:{paddingInline:0,[`&${t}-compact-item`]:{flex:"none"},[`&${t}-round`]:{width:"auto"}},[`&${t}-loading`]:{opacity:o,cursor:"default"},[`${t}-loading-icon`]:{transition:["width","opacity","margin"].map(v=>`${v} ${l} ${c}`).join(",")},[`&:not(${t}-icon-end)`]:{[`${t}-loading-icon-motion`]:{"&-appear-start, &-enter-start":{marginInlineEnd:m(d).mul(-1).equal()},"&-appear-active, &-enter-active":{marginInlineEnd:0},"&-leave-start":{marginInlineEnd:0},"&-leave-active":{marginInlineEnd:m(d).mul(-1).equal()}}},"&-icon-end":{flexDirection:"row-reverse",[`${t}-loading-icon-motion`]:{"&-appear-start, &-enter-start":{marginInlineStart:m(d).mul(-1).equal()},"&-appear-active, &-enter-active":{marginInlineStart:0},"&-leave-start":{marginInlineStart:0},"&-leave-active":{marginInlineStart:m(d).mul(-1).equal()}}}}}},k$=(e,t,n)=>({[`&:not(:disabled):not(${e}-disabled)`]:{"&:hover":t,"&:active":n}}),bz=e=>({minWidth:e.controlHeight,paddingInlineStart:0,paddingInlineEnd:0,borderRadius:"50%"}),Sz=e=>({borderRadius:e.controlHeight,paddingInlineStart:e.calc(e.controlHeight).div(2).equal(),paddingInlineEnd:e.calc(e.controlHeight).div(2).equal()}),Cz=e=>({cursor:"not-allowed",borderColor:e.borderColorDisabled,color:e.colorTextDisabled,background:e.colorBgContainerDisabled,boxShadow:"none"}),Bd=(e,t,n,r,o,l,c,d)=>({[`&${e}-background-ghost`]:Object.assign(Object.assign({color:n||void 0,background:t,borderColor:r||void 0,boxShadow:"none"},k$(e,Object.assign({background:t},c),Object.assign({background:t},d))),{"&:disabled":{cursor:"not-allowed",color:o||void 0,borderColor:l||void 0}})}),xz=e=>({[`&:disabled, &${e.componentCls}-disabled`]:Object.assign({},Cz(e))}),Ez=e=>({[`&:disabled, &${e.componentCls}-disabled`]:{cursor:"not-allowed",color:e.colorTextDisabled}}),Ld=(e,t,n,r)=>{const l=r&&["link","text"].includes(r)?Ez:xz;return Object.assign(Object.assign({},l(e)),k$(e.componentCls,t,n))},Hd=(e,t,n,r,o)=>({[`&${e.componentCls}-variant-solid`]:Object.assign({color:t,background:n},Ld(e,r,o))}),Fd=(e,t,n,r,o)=>({[`&${e.componentCls}-variant-outlined, &${e.componentCls}-variant-dashed`]:Object.assign({borderColor:t,background:n},Ld(e,r,o))}),Vd=e=>({[`&${e.componentCls}-variant-dashed`]:{borderStyle:"dashed"}}),Ud=(e,t,n,r)=>({[`&${e.componentCls}-variant-filled`]:Object.assign({boxShadow:"none",background:t},Ld(e,n,r))}),Ka=(e,t,n,r,o)=>({[`&${e.componentCls}-variant-${n}`]:Object.assign({color:t,boxShadow:"none"},Ld(e,r,o,n))}),wz=e=>{const{componentCls:t}=e;return ai.reduce((n,r)=>{const o=e[`${r}6`],l=e[`${r}1`],c=e[`${r}5`],d=e[`${r}2`],m=e[`${r}3`],v=e[`${r}7`];return Object.assign(Object.assign({},n),{[`&${t}-color-${r}`]:Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({color:o,boxShadow:e[`${r}ShadowColor`]},Hd(e,e.colorTextLightSolid,o,{background:c},{background:v})),Fd(e,o,e.colorBgContainer,{color:c,borderColor:c,background:e.colorBgContainer},{color:v,borderColor:v,background:e.colorBgContainer})),Vd(e)),Ud(e,l,{background:d},{background:m})),Ka(e,o,"link",{color:c},{color:v})),Ka(e,o,"text",{color:c,background:l},{color:v,background:m}))})},{})},$z=e=>Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({color:e.defaultColor,boxShadow:e.defaultShadow},Hd(e,e.solidTextColor,e.colorBgSolid,{color:e.solidTextColor,background:e.colorBgSolidHover},{color:e.solidTextColor,background:e.colorBgSolidActive})),Vd(e)),Ud(e,e.colorFillTertiary,{background:e.colorFillSecondary},{background:e.colorFill})),Bd(e.componentCls,e.ghostBg,e.defaultGhostColor,e.defaultGhostBorderColor,e.colorTextDisabled,e.colorBorder)),Ka(e,e.textTextColor,"link",{color:e.colorLinkHover,background:e.linkHoverBg},{color:e.colorLinkActive})),Oz=e=>Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({color:e.colorPrimary,boxShadow:e.primaryShadow},Fd(e,e.colorPrimary,e.colorBgContainer,{color:e.colorPrimaryTextHover,borderColor:e.colorPrimaryHover,background:e.colorBgContainer},{color:e.colorPrimaryTextActive,borderColor:e.colorPrimaryActive,background:e.colorBgContainer})),Vd(e)),Ud(e,e.colorPrimaryBg,{background:e.colorPrimaryBgHover},{background:e.colorPrimaryBorder})),Ka(e,e.colorPrimaryText,"text",{color:e.colorPrimaryTextHover,background:e.colorPrimaryBg},{color:e.colorPrimaryTextActive,background:e.colorPrimaryBorder})),Ka(e,e.colorPrimaryText,"link",{color:e.colorPrimaryTextHover,background:e.linkHoverBg},{color:e.colorPrimaryTextActive})),Bd(e.componentCls,e.ghostBg,e.colorPrimary,e.colorPrimary,e.colorTextDisabled,e.colorBorder,{color:e.colorPrimaryHover,borderColor:e.colorPrimaryHover},{color:e.colorPrimaryActive,borderColor:e.colorPrimaryActive})),Rz=e=>Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({color:e.colorError,boxShadow:e.dangerShadow},Hd(e,e.dangerColor,e.colorError,{background:e.colorErrorHover},{background:e.colorErrorActive})),Fd(e,e.colorError,e.colorBgContainer,{color:e.colorErrorHover,borderColor:e.colorErrorBorderHover},{color:e.colorErrorActive,borderColor:e.colorErrorActive})),Vd(e)),Ud(e,e.colorErrorBg,{background:e.colorErrorBgFilledHover},{background:e.colorErrorBgActive})),Ka(e,e.colorError,"text",{color:e.colorErrorHover,background:e.colorErrorBg},{color:e.colorErrorHover,background:e.colorErrorBgActive})),Ka(e,e.colorError,"link",{color:e.colorErrorHover},{color:e.colorErrorActive})),Bd(e.componentCls,e.ghostBg,e.colorError,e.colorError,e.colorTextDisabled,e.colorBorder,{color:e.colorErrorHover,borderColor:e.colorErrorHover},{color:e.colorErrorActive,borderColor:e.colorErrorActive})),Tz=e=>Object.assign(Object.assign({},Ka(e,e.colorLink,"link",{color:e.colorLinkHover},{color:e.colorLinkActive})),Bd(e.componentCls,e.ghostBg,e.colorInfo,e.colorInfo,e.colorTextDisabled,e.colorBorder,{color:e.colorInfoHover,borderColor:e.colorInfoHover},{color:e.colorInfoActive,borderColor:e.colorInfoActive})),Mz=e=>{const{componentCls:t}=e;return Object.assign({[`${t}-color-default`]:$z(e),[`${t}-color-primary`]:Oz(e),[`${t}-color-dangerous`]:Rz(e),[`${t}-color-link`]:Tz(e)},wz(e))},_z=e=>Object.assign(Object.assign(Object.assign(Object.assign({},Fd(e,e.defaultBorderColor,e.defaultBg,{color:e.defaultHoverColor,borderColor:e.defaultHoverBorderColor,background:e.defaultHoverBg},{color:e.defaultActiveColor,borderColor:e.defaultActiveBorderColor,background:e.defaultActiveBg})),Ka(e,e.textTextColor,"text",{color:e.textTextHoverColor,background:e.textHoverBg},{color:e.textTextActiveColor,background:e.colorBgTextActive})),Hd(e,e.primaryColor,e.colorPrimary,{background:e.colorPrimaryHover,color:e.primaryColor},{background:e.colorPrimaryActive,color:e.primaryColor})),Ka(e,e.colorLink,"link",{color:e.colorLinkHover,background:e.linkHoverBg},{color:e.colorLinkActive})),h0=function(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:"";const{componentCls:n,controlHeight:r,fontSize:o,borderRadius:l,buttonPaddingHorizontal:c,iconCls:d,buttonPaddingVertical:m,buttonIconOnlyFontSize:v}=e;return[{[t]:{fontSize:o,height:r,padding:`${he(m)} ${he(c)}`,borderRadius:l,[`&${n}-icon-only`]:{width:r,[d]:{fontSize:v}}}},{[`${n}${n}-circle${t}`]:bz(e)},{[`${n}${n}-round${t}`]:Sz(e)}]},Nz=e=>{const t=kt(e,{fontSize:e.contentFontSize});return h0(t,e.componentCls)},Az=e=>{const t=kt(e,{controlHeight:e.controlHeightSM,fontSize:e.contentFontSizeSM,padding:e.paddingXS,buttonPaddingHorizontal:e.paddingInlineSM,buttonPaddingVertical:0,borderRadius:e.borderRadiusSM,buttonIconOnlyFontSize:e.onlyIconSizeSM});return h0(t,`${e.componentCls}-sm`)},Iz=e=>{const t=kt(e,{controlHeight:e.controlHeightLG,fontSize:e.contentFontSizeLG,buttonPaddingHorizontal:e.paddingInlineLG,buttonPaddingVertical:0,borderRadius:e.borderRadiusLG,buttonIconOnlyFontSize:e.onlyIconSizeLG});return h0(t,`${e.componentCls}-lg`)},jz=e=>{const{componentCls:t}=e;return{[t]:{[`&${t}-block`]:{width:"100%"}}}},Pz=wn("Button",e=>{const t=q$(e);return[yz(t),Nz(t),Az(t),Iz(t),jz(t),Mz(t),_z(t),yP(t)]},W$,{unitless:{fontWeight:!0,contentLineHeight:!0,contentLineHeightSM:!0,contentLineHeightLG:!0}});function zz(e,t,n){const{focusElCls:r,focus:o,borderElCls:l}=n,c=l?"> *":"",d=["hover",o?"focus":null,"active"].filter(Boolean).map(m=>`&:${m} ${c}`).join(",");return{[`&-item:not(${t}-last-item)`]:{marginInlineEnd:e.calc(e.lineWidth).mul(-1).equal()},"&-item":Object.assign(Object.assign({[d]:{zIndex:2}},r?{[`&${r}`]:{zIndex:2}}:{}),{[`&[disabled] ${c}`]:{zIndex:0}})}}function Dz(e,t,n){const{borderElCls:r}=n,o=r?`> ${r}`:"";return{[`&-item:not(${t}-first-item):not(${t}-last-item) ${o}`]:{borderRadius:0},[`&-item:not(${t}-last-item)${t}-first-item`]:{[`& ${o}, &${e}-sm ${o}, &${e}-lg ${o}`]:{borderStartEndRadius:0,borderEndEndRadius:0}},[`&-item:not(${t}-first-item)${t}-last-item`]:{[`& ${o}, &${e}-sm ${o}, &${e}-lg ${o}`]:{borderStartStartRadius:0,borderEndStartRadius:0}}}}function p0(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{focus:!0};const{componentCls:n}=e,r=`${n}-compact`;return{[r]:Object.assign(Object.assign({},zz(e,r,t)),Dz(n,r,t))}}function Bz(e,t){return{[`&-item:not(${t}-last-item)`]:{marginBottom:e.calc(e.lineWidth).mul(-1).equal()},"&-item":{"&:hover,&:focus,&:active":{zIndex:2},"&[disabled]":{zIndex:0}}}}function Lz(e,t){return{[`&-item:not(${t}-first-item):not(${t}-last-item)`]:{borderRadius:0},[`&-item${t}-first-item:not(${t}-last-item)`]:{[`&, &${e}-sm, &${e}-lg`]:{borderEndEndRadius:0,borderEndStartRadius:0}},[`&-item${t}-last-item:not(${t}-first-item)`]:{[`&, &${e}-sm, &${e}-lg`]:{borderStartStartRadius:0,borderStartEndRadius:0}}}}function Hz(e){const t=`${e.componentCls}-compact-vertical`;return{[t]:Object.assign(Object.assign({},Bz(e,t)),Lz(e.componentCls,t))}}const Fz=e=>{const{componentCls:t,colorPrimaryHover:n,lineWidth:r,calc:o}=e,l=o(r).mul(-1).equal(),c=d=>{const m=`${t}-compact${d?"-vertical":""}-item${t}-primary:not([disabled])`;return{[`${m} + ${m}::before`]:{position:"absolute",top:d?l:0,insetInlineStart:d?0:l,backgroundColor:n,content:'""',width:d?"100%":r,height:d?r:"100%"}}};return Object.assign(Object.assign({},c()),c(!0))},Vz=_d(["Button","compact"],e=>{const t=q$(e);return[p0(t),Hz(t),Fz(t)]},W$);var Uz=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function qz(e){if(typeof e=="object"&&e){let t=e==null?void 0:e.delay;return t=!Number.isNaN(t)&&typeof t=="number"?t:0,{loading:t<=0,delay:t}}return{loading:!!e,delay:0}}const Wz={default:["default","outlined"],primary:["primary","solid"],dashed:["default","dashed"],link:["link","link"],text:["default","text"]},kz=oe.forwardRef((e,t)=>{var n,r;const{loading:o=!1,prefixCls:l,color:c,variant:d,type:m,danger:v=!1,shape:g="default",size:p,styles:y,disabled:E,className:C,rootClassName:x,children:b,icon:w,iconPosition:$="start",ghost:O=!1,block:T=!1,htmlType:R="button",classNames:M,style:N={},autoInsertSpace:z,autoFocus:P}=e,D=Uz(e,["loading","prefixCls","color","variant","type","danger","shape","size","styles","disabled","className","rootClassName","children","icon","iconPosition","ghost","block","htmlType","classNames","style","autoInsertSpace","autoFocus"]),F=m||"default",[W,V]=u.useMemo(()=>{if(c&&d)return[c,d];const Fe=Wz[F]||[];return v?["danger",Fe[1]]:Fe},[m,c,d,v]),K=W==="danger"?"dangerous":W,{getPrefixCls:_,direction:j,autoInsertSpace:B,className:k,style:I,classNames:H,styles:G}=zr("button"),X=(n=z??B)!==null&&n!==void 0?n:!0,U=_("btn",l),[J,te,ve]=Pz(U),ne=u.useContext(Co),le=E??ne,be=u.useContext(D$),fe=u.useMemo(()=>qz(o),[o]),[ie,pe]=u.useState(fe.loading),[Re,se]=u.useState(!1),ye=u.useRef(null),$e=li(t,ye),we=u.Children.count(b)===1&&!w&&!zg(V),Ne=u.useRef(!0);oe.useEffect(()=>(Ne.current=!1,()=>{Ne.current=!0}),[]),u.useEffect(()=>{let Fe=null;fe.delay>0?Fe=setTimeout(()=>{Fe=null,pe(!0)},fe.delay):pe(fe.loading);function _e(){Fe&&(clearTimeout(Fe),Fe=null)}return _e},[fe]),u.useEffect(()=>{if(!ye.current||!X)return;const Fe=ye.current.textContent||"";we&&ap(Fe)?Re||se(!0):Re&&se(!1)}),u.useEffect(()=>{P&&ye.current&&ye.current.focus()},[]);const Ze=oe.useCallback(Fe=>{var _e;if(ie||le){Fe.preventDefault();return}(_e=e.onClick)===null||_e===void 0||_e.call(e,("href"in e,Fe))},[e.onClick,ie,le]),{compactSize:He,compactItemClassnames:Xe}=Xc(U,j),xe={large:"lg",small:"sm",middle:void 0},Me=Mo(Fe=>{var _e,qe;return(qe=(_e=p??He)!==null&&_e!==void 0?_e:be)!==null&&qe!==void 0?qe:Fe}),Ve=Me&&(r=xe[Me])!==null&&r!==void 0?r:"",Ke=ie?"loading":w,We=Tn(D,["navigate"]),at=re(U,te,ve,{[`${U}-${g}`]:g!=="default"&&g,[`${U}-${F}`]:F,[`${U}-dangerous`]:v,[`${U}-color-${K}`]:K,[`${U}-variant-${V}`]:V,[`${U}-${Ve}`]:Ve,[`${U}-icon-only`]:!b&&b!==0&&!!Ke,[`${U}-background-ghost`]:O&&!zg(V),[`${U}-loading`]:ie,[`${U}-two-chinese-chars`]:Re&&X&&!ie,[`${U}-block`]:T,[`${U}-rtl`]:j==="rtl",[`${U}-icon-end`]:$==="end"},Xe,C,x,k),Ue=Object.assign(Object.assign({},I),N),Ye=re(M==null?void 0:M.icon,H.icon),st=Object.assign(Object.assign({},(y==null?void 0:y.icon)||{}),G.icon||{}),ht=w&&!ie?oe.createElement(op,{prefixCls:U,className:Ye,style:st},w):o&&typeof o=="object"&&o.icon?oe.createElement(op,{prefixCls:U,className:Ye,style:st},o.icon):oe.createElement(pP,{existIcon:!!w,prefixCls:U,loading:ie,mount:Ne.current}),Qe=b||b===0?hP(b,we&&X):null;if(We.href!==void 0)return J(oe.createElement("a",Object.assign({},We,{className:re(at,{[`${U}-disabled`]:le}),href:le?void 0:We.href,style:Ue,onClick:Ze,ref:$e,tabIndex:le?-1:0}),ht,Qe));let Pe=oe.createElement("button",Object.assign({},D,{type:R,className:at,style:Ue,onClick:Ze,disabled:le,ref:$e}),ht,Qe,Xe&&oe.createElement(Vz,{prefixCls:U}));return zg(V)||(Pe=oe.createElement(iP,{component:"Button",disabled:ie},Pe)),J(Pe)}),$r=kz;$r.Group=vP;$r.__ANT_BUTTON=!0;function Hg(e){return!!(e!=null&&e.then)}const y0=e=>{const{type:t,children:n,prefixCls:r,buttonProps:o,close:l,autoFocus:c,emitEvent:d,isSilent:m,quitOnNullishReturnValue:v,actionFn:g}=e,p=u.useRef(!1),y=u.useRef(null),[E,C]=Hi(!1),x=function(){l==null||l.apply(void 0,arguments)};u.useEffect(()=>{let $=null;return c&&($=setTimeout(()=>{var O;(O=y.current)===null||O===void 0||O.focus({preventScroll:!0})})),()=>{$&&clearTimeout($)}},[]);const b=$=>{Hg($)&&(C(!0),$.then(function(){C(!1,!0),x.apply(void 0,arguments),p.current=!1},O=>{if(C(!1,!0),p.current=!1,!(m!=null&&m()))return Promise.reject(O)}))},w=$=>{if(p.current)return;if(p.current=!0,!g){x();return}let O;if(d){if(O=g($),v&&!Hg(O)){p.current=!1,x($);return}}else if(g.length)O=g(l),p.current=!1;else if(O=g(),!Hg(O)){x();return}b(O)};return u.createElement($r,Object.assign({},d0(t),{onClick:w,loading:E,prefixCls:r},o,{ref:y}),n)},Yc=oe.createContext({}),{Provider:G$}=Yc,k1=()=>{const{autoFocusButton:e,cancelButtonProps:t,cancelTextLocale:n,isSilent:r,mergedOkCancel:o,rootPrefixCls:l,close:c,onCancel:d,onConfirm:m}=u.useContext(Yc);return o?oe.createElement(y0,{isSilent:r,actionFn:d,close:function(){c==null||c.apply(void 0,arguments),m==null||m(!1)},autoFocus:e==="cancel",buttonProps:t,prefixCls:`${l}-btn`},n):null},G1=()=>{const{autoFocusButton:e,close:t,isSilent:n,okButtonProps:r,rootPrefixCls:o,okTextLocale:l,okType:c,onConfirm:d,onOk:m}=u.useContext(Yc);return oe.createElement(y0,{isSilent:n,type:c||"primary",actionFn:m,close:function(){t==null||t.apply(void 0,arguments),d==null||d(!0)},autoFocus:e==="ok",buttonProps:r,prefixCls:`${o}-btn`},l)};var K$=u.createContext(null),K1=[];function Gz(e,t){var n=u.useState(function(){if(!Cr())return null;var C=document.createElement("div");return C}),r=de(n,1),o=r[0],l=u.useRef(!1),c=u.useContext(K$),d=u.useState(K1),m=de(d,2),v=m[0],g=m[1],p=c||(l.current?void 0:function(C){g(function(x){var b=[C].concat(Te(x));return b})});function y(){o.parentElement||document.body.appendChild(o),l.current=!0}function E(){var C;(C=o.parentElement)===null||C===void 0||C.removeChild(o),l.current=!1}return Wt(function(){return e?c?c(y):y():E(),E},[e]),Wt(function(){v.length&&(v.forEach(function(C){return C()}),g(K1))},[v]),[o,p]}var Fg;function X$(e){var t="rc-scrollbar-measure-".concat(Math.random().toString(36).substring(7)),n=document.createElement("div");n.id=t;var r=n.style;r.position="absolute",r.left="0",r.top="0",r.width="100px",r.height="100px",r.overflow="scroll";var o,l;if(e){var c=getComputedStyle(e);r.scrollbarColor=c.scrollbarColor,r.scrollbarWidth=c.scrollbarWidth;var d=getComputedStyle(e,"::-webkit-scrollbar"),m=parseInt(d.width,10),v=parseInt(d.height,10);try{var g=m?"width: ".concat(d.width,";"):"",p=v?"height: ".concat(d.height,";"):"";po(`
#`.concat(t,`::-webkit-scrollbar {
`).concat(g,`
`).concat(p,`
}`),t)}catch(C){console.error(C),o=m,l=v}}document.body.appendChild(n);var y=e&&o&&!isNaN(o)?o:n.offsetWidth-n.clientWidth,E=e&&l&&!isNaN(l)?l:n.offsetHeight-n.clientHeight;return document.body.removeChild(n),Cc(t),{width:y,height:E}}function lW(e){return typeof document>"u"?0:(Fg===void 0&&(Fg=X$()),Fg.width)}function Kz(e){return typeof document>"u"||!e||!(e instanceof Element)?{width:0,height:0}:X$(e)}function Xz(){return document.body.scrollHeight>(window.innerHeight||document.documentElement.clientHeight)&&window.innerWidth>document.body.offsetWidth}var Yz="rc-util-locker-".concat(Date.now()),X1=0;function Qz(e){var t=!!e,n=u.useState(function(){return X1+=1,"".concat(Yz,"_").concat(X1)}),r=de(n,1),o=r[0];Wt(function(){if(t){var l=Kz(document.body).width,c=Xz();po(`
html body {
  overflow-y: hidden;
  `.concat(c?"width: calc(100% - ".concat(l,"px);"):"",`
}`),o)}else Cc(o);return function(){Cc(o)}},[t,o])}var Zz=!1;function Jz(e){return Zz}var Y1=function(t){return t===!1?!1:!Cr()||!t?null:typeof t=="string"?document.querySelector(t):typeof t=="function"?t():t},b0=u.forwardRef(function(e,t){var n=e.open,r=e.autoLock,o=e.getContainer;e.debug;var l=e.autoDestroy,c=l===void 0?!0:l,d=e.children,m=u.useState(n),v=de(m,2),g=v[0],p=v[1],y=g||n;u.useEffect(function(){(c||n)&&p(n)},[n,c]);var E=u.useState(function(){return Y1(o)}),C=de(E,2),x=C[0],b=C[1];u.useEffect(function(){var F=Y1(o);b(F??null)});var w=Gz(y&&!x),$=de(w,2),O=$[0],T=$[1],R=x??O;Qz(r&&n&&Cr()&&(R===O||R===document.body));var M=null;if(d&&ki(d)&&t){var N=d;M=N.ref}var z=li(M,t);if(!y||!Cr()||x===void 0)return null;var P=R===!1||Jz(),D=d;return t&&(D=u.cloneElement(d,{ref:z})),u.createElement(K$.Provider,{value:T},P?D:ri.createPortal(D,R))}),Y$=u.createContext({});function e3(){var e=Z({},xd);return e.useId}var Q1=0,Z1=e3();const S0=Z1?function(t){var n=Z1();return t||n}:function(t){var n=u.useState("ssr-id"),r=de(n,2),o=r[0],l=r[1];return u.useEffect(function(){var c=Q1;Q1+=1,l("rc_unique_".concat(c))},[]),t||o};function J1(e,t,n){var r=t;return!r&&n&&(r="".concat(e,"-").concat(n)),r}function ex(e,t){var n=e["page".concat(t?"Y":"X","Offset")],r="scroll".concat(t?"Top":"Left");if(typeof n!="number"){var o=e.document;n=o.documentElement[r],typeof n!="number"&&(n=o.body[r])}return n}function t3(e){var t=e.getBoundingClientRect(),n={left:t.left,top:t.top},r=e.ownerDocument,o=r.defaultView||r.parentWindow;return n.left+=ex(o),n.top+=ex(o,!0),n}const n3=u.memo(function(e){var t=e.children;return t},function(e,t){var n=t.shouldUpdate;return!n});var r3={width:0,height:0,overflow:"hidden",outline:"none"},a3={outline:"none"},Q$=oe.forwardRef(function(e,t){var n=e.prefixCls,r=e.className,o=e.style,l=e.title,c=e.ariaId,d=e.footer,m=e.closable,v=e.closeIcon,g=e.onClose,p=e.children,y=e.bodyStyle,E=e.bodyProps,C=e.modalRender,x=e.onMouseDown,b=e.onMouseUp,w=e.holderRef,$=e.visible,O=e.forceRender,T=e.width,R=e.height,M=e.classNames,N=e.styles,z=oe.useContext(Y$),P=z.panel,D=li(w,P),F=u.useRef(),W=u.useRef();oe.useImperativeHandle(t,function(){return{focus:function(){var G;(G=F.current)===null||G===void 0||G.focus({preventScroll:!0})},changeActive:function(G){var X=document,U=X.activeElement;G&&U===W.current?F.current.focus({preventScroll:!0}):!G&&U===F.current&&W.current.focus({preventScroll:!0})}}});var V={};T!==void 0&&(V.width=T),R!==void 0&&(V.height=R);var q=d?oe.createElement("div",{className:re("".concat(n,"-footer"),M==null?void 0:M.footer),style:Z({},N==null?void 0:N.footer)},d):null,K=l?oe.createElement("div",{className:re("".concat(n,"-header"),M==null?void 0:M.header),style:Z({},N==null?void 0:N.header)},oe.createElement("div",{className:"".concat(n,"-title"),id:c},l)):null,_=u.useMemo(function(){return lt(m)==="object"&&m!==null?m:m?{closeIcon:v??oe.createElement("span",{className:"".concat(n,"-close-x")})}:{}},[m,v,n]),j=Ga(_,!0),B=lt(m)==="object"&&m.disabled,k=m?oe.createElement("button",Ie({type:"button",onClick:g,"aria-label":"Close"},j,{className:"".concat(n,"-close"),disabled:B}),_.closeIcon):null,I=oe.createElement("div",{className:re("".concat(n,"-content"),M==null?void 0:M.content),style:N==null?void 0:N.content},k,K,oe.createElement("div",Ie({className:re("".concat(n,"-body"),M==null?void 0:M.body),style:Z(Z({},y),N==null?void 0:N.body)},E),p),q);return oe.createElement("div",{key:"dialog-element",role:"dialog","aria-labelledby":l?c:null,"aria-modal":"true",ref:D,style:Z(Z({},o),V),className:re(n,r),onMouseDown:x,onMouseUp:b},oe.createElement("div",{ref:F,tabIndex:0,style:a3},oe.createElement(n3,{shouldUpdate:$||O},C?C(I):I)),oe.createElement("div",{tabIndex:0,ref:W,style:r3}))}),Z$=u.forwardRef(function(e,t){var n=e.prefixCls,r=e.title,o=e.style,l=e.className,c=e.visible,d=e.forceRender,m=e.destroyOnClose,v=e.motionName,g=e.ariaId,p=e.onVisibleChanged,y=e.mousePosition,E=u.useRef(),C=u.useState(),x=de(C,2),b=x[0],w=x[1],$={};b&&($.transformOrigin=b);function O(){var T=t3(E.current);w(y&&(y.x||y.y)?"".concat(y.x-T.left,"px ").concat(y.y-T.top,"px"):"")}return u.createElement(Xa,{visible:c,onVisibleChanged:p,onAppearPrepare:O,onEnterPrepare:O,forceRender:d,motionName:v,removeOnLeave:m,ref:E},function(T,R){var M=T.className,N=T.style;return u.createElement(Q$,Ie({},e,{ref:t,title:r,ariaId:g,prefixCls:n,holderRef:R,style:Z(Z(Z({},N),o),$),className:re(l,M)}))})});Z$.displayName="Content";var o3=function(t){var n=t.prefixCls,r=t.style,o=t.visible,l=t.maskProps,c=t.motionName,d=t.className;return u.createElement(Xa,{key:"mask",visible:o,motionName:c,leavedClassName:"".concat(n,"-mask-hidden")},function(m,v){var g=m.className,p=m.style;return u.createElement("div",Ie({ref:v,style:Z(Z({},p),r),className:re("".concat(n,"-mask"),g,d)},l))})},i3=function(t){var n=t.prefixCls,r=n===void 0?"rc-dialog":n,o=t.zIndex,l=t.visible,c=l===void 0?!1:l,d=t.keyboard,m=d===void 0?!0:d,v=t.focusTriggerAfterClose,g=v===void 0?!0:v,p=t.wrapStyle,y=t.wrapClassName,E=t.wrapProps,C=t.onClose,x=t.afterOpenChange,b=t.afterClose,w=t.transitionName,$=t.animation,O=t.closable,T=O===void 0?!0:O,R=t.mask,M=R===void 0?!0:R,N=t.maskTransitionName,z=t.maskAnimation,P=t.maskClosable,D=P===void 0?!0:P,F=t.maskStyle,W=t.maskProps,V=t.rootClassName,q=t.classNames,K=t.styles,_=u.useRef(),j=u.useRef(),B=u.useRef(),k=u.useState(c),I=de(k,2),H=I[0],G=I[1],X=S0();function U(){Bh(j.current,document.activeElement)||(_.current=document.activeElement)}function J(){if(!Bh(j.current,document.activeElement)){var se;(se=B.current)===null||se===void 0||se.focus()}}function te(se){if(se)J();else{if(G(!1),M&&_.current&&g){try{_.current.focus({preventScroll:!0})}catch{}_.current=null}H&&(b==null||b())}x==null||x(se)}function ve(se){C==null||C(se)}var ne=u.useRef(!1),le=u.useRef(),be=function(){clearTimeout(le.current),ne.current=!0},fe=function(){le.current=setTimeout(function(){ne.current=!1})},ie=null;D&&(ie=function(ye){ne.current?ne.current=!1:j.current===ye.target&&ve(ye)});function pe(se){if(m&&se.keyCode===ct.ESC){se.stopPropagation(),ve(se);return}c&&se.keyCode===ct.TAB&&B.current.changeActive(!se.shiftKey)}u.useEffect(function(){c&&(G(!0),U())},[c]),u.useEffect(function(){return function(){clearTimeout(le.current)}},[]);var Re=Z(Z(Z({zIndex:o},p),K==null?void 0:K.wrapper),{},{display:H?null:"none"});return u.createElement("div",Ie({className:re("".concat(r,"-root"),V)},Ga(t,{data:!0})),u.createElement(o3,{prefixCls:r,visible:M&&c,motionName:J1(r,N,z),style:Z(Z({zIndex:o},F),K==null?void 0:K.mask),maskProps:W,className:q==null?void 0:q.mask}),u.createElement("div",Ie({tabIndex:-1,onKeyDown:pe,className:re("".concat(r,"-wrap"),y,q==null?void 0:q.wrapper),ref:j,onClick:ie,style:Re},E),u.createElement(Z$,Ie({},t,{onMouseDown:be,onMouseUp:fe,ref:B,closable:T,ariaId:X,prefixCls:r,visible:c&&H,onClose:ve,onVisibleChanged:te,motionName:J1(r,w,$)}))))},J$=function(t){var n=t.visible,r=t.getContainer,o=t.forceRender,l=t.destroyOnClose,c=l===void 0?!1:l,d=t.afterClose,m=t.panelRef,v=u.useState(n),g=de(v,2),p=g[0],y=g[1],E=u.useMemo(function(){return{panel:m}},[m]);return u.useEffect(function(){n&&y(!0)},[n]),!o&&c&&!p?null:u.createElement(Y$.Provider,{value:E},u.createElement(b0,{open:n||o||p,autoDestroy:!1,getContainer:r,autoLock:n||p},u.createElement(i3,Ie({},t,{destroyOnClose:c,afterClose:function(){d==null||d(),y(!1)}}))))};J$.displayName="Dialog";var Ai="RC_FORM_INTERNAL_HOOKS",vn=function(){vr(!1,"Can not find FormContext. Please make sure you wrap Field under Form.")},Vi=u.createContext({getFieldValue:vn,getFieldsValue:vn,getFieldError:vn,getFieldWarning:vn,getFieldsError:vn,isFieldsTouched:vn,isFieldTouched:vn,isFieldValidating:vn,isFieldsValidating:vn,resetFields:vn,setFields:vn,setFieldValue:vn,setFieldsValue:vn,validateFields:vn,submit:vn,getInternalHooks:function(){return vn(),{dispatch:vn,initEntityValue:vn,registerField:vn,useSubscribe:vn,setInitialValues:vn,destroyForm:vn,setCallbacks:vn,registerWatch:vn,getFields:vn,setValidateMessages:vn,setPreserve:vn,getInitialValue:vn}}}),Nc=u.createContext(null);function ip(e){return e==null?[]:Array.isArray(e)?e:[e]}function l3(e){return e&&!!e._init}function lp(){return{default:"Validation error on field %s",required:"%s is required",enum:"%s must be one of %s",whitespace:"%s cannot be empty",date:{format:"%s date %s is invalid for format %s",parse:"%s date could not be parsed, %s is invalid ",invalid:"%s date %s is invalid"},types:{string:"%s is not a %s",method:"%s is not a %s (function)",array:"%s is not an %s",object:"%s is not an %s",number:"%s is not a %s",date:"%s is not a %s",boolean:"%s is not a %s",integer:"%s is not an %s",float:"%s is not a %s",regexp:"%s is not a valid %s",email:"%s is not a valid %s",url:"%s is not a valid %s",hex:"%s is not a valid %s"},string:{len:"%s must be exactly %s characters",min:"%s must be at least %s characters",max:"%s cannot be longer than %s characters",range:"%s must be between %s and %s characters"},number:{len:"%s must equal %s",min:"%s cannot be less than %s",max:"%s cannot be greater than %s",range:"%s must be between %s and %s"},array:{len:"%s must be exactly %s in length",min:"%s cannot be less than %s in length",max:"%s cannot be greater than %s in length",range:"%s must be between %s and %s in length"},pattern:{mismatch:"%s value %s does not match pattern %s"},clone:function(){var t=JSON.parse(JSON.stringify(this));return t.clone=this.clone,t}}}var sp=lp();function s3(e){try{return Function.toString.call(e).indexOf("[native code]")!==-1}catch{return typeof e=="function"}}function c3(e,t,n){if(Kp())return Reflect.construct.apply(null,arguments);var r=[null];r.push.apply(r,t);var o=new(e.bind.apply(e,r));return n&&Ec(o,n.prototype),o}function cp(e){var t=typeof Map=="function"?new Map:void 0;return cp=function(r){if(r===null||!s3(r))return r;if(typeof r!="function")throw new TypeError("Super expression must either be null or a function");if(t!==void 0){if(t.has(r))return t.get(r);t.set(r,o)}function o(){return c3(r,arguments,wc(this).constructor)}return o.prototype=Object.create(r.prototype,{constructor:{value:o,enumerable:!1,writable:!0,configurable:!0}}),Ec(o,r)},cp(e)}var u3=/%[sdj%]/g,f3=function(){};function up(e){if(!e||!e.length)return null;var t={};return e.forEach(function(n){var r=n.field;t[r]=t[r]||[],t[r].push(n)}),t}function Yr(e){for(var t=arguments.length,n=new Array(t>1?t-1:0),r=1;r<t;r++)n[r-1]=arguments[r];var o=0,l=n.length;if(typeof e=="function")return e.apply(null,n);if(typeof e=="string"){var c=e.replace(u3,function(d){if(d==="%%")return"%";if(o>=l)return d;switch(d){case"%s":return String(n[o++]);case"%d":return Number(n[o++]);case"%j":try{return JSON.stringify(n[o++])}catch{return"[Circular]"}break;default:return d}});return c}return e}function d3(e){return e==="string"||e==="url"||e==="hex"||e==="email"||e==="date"||e==="pattern"}function Jn(e,t){return!!(e==null||t==="array"&&Array.isArray(e)&&!e.length||d3(t)&&typeof e=="string"&&!e)}function m3(e,t,n){var r=[],o=0,l=e.length;function c(d){r.push.apply(r,Te(d||[])),o++,o===l&&n(r)}e.forEach(function(d){t(d,c)})}function tx(e,t,n){var r=0,o=e.length;function l(c){if(c&&c.length){n(c);return}var d=r;r=r+1,d<o?t(e[d],l):n([])}l([])}function v3(e){var t=[];return Object.keys(e).forEach(function(n){t.push.apply(t,Te(e[n]||[]))}),t}var nx=function(e){$o(n,e);var t=Oo(n);function n(r,o){var l;return qn(this,n),l=t.call(this,"Async Validation Error"),Y(_t(l),"errors",void 0),Y(_t(l),"fields",void 0),l.errors=r,l.fields=o,l}return Wn(n)}(cp(Error));function g3(e,t,n,r,o){if(t.first){var l=new Promise(function(y,E){var C=function(w){return r(w),w.length?E(new nx(w,up(w))):y(o)},x=v3(e);tx(x,n,C)});return l.catch(function(y){return y}),l}var c=t.firstFields===!0?Object.keys(e):t.firstFields||[],d=Object.keys(e),m=d.length,v=0,g=[],p=new Promise(function(y,E){var C=function(b){if(g.push.apply(g,b),v++,v===m)return r(g),g.length?E(new nx(g,up(g))):y(o)};d.length||(r(g),y(o)),d.forEach(function(x){var b=e[x];c.indexOf(x)!==-1?tx(b,n,C):m3(b,n,C)})});return p.catch(function(y){return y}),p}function h3(e){return!!(e&&e.message!==void 0)}function p3(e,t){for(var n=e,r=0;r<t.length;r++){if(n==null)return n;n=n[t[r]]}return n}function rx(e,t){return function(n){var r;return e.fullFields?r=p3(t,e.fullFields):r=t[n.field||e.fullField],h3(n)?(n.field=n.field||e.fullField,n.fieldValue=r,n):{message:typeof n=="function"?n():n,fieldValue:r,field:n.field||e.fullField}}}function ax(e,t){if(t){for(var n in t)if(t.hasOwnProperty(n)){var r=t[n];lt(r)==="object"&&lt(e[n])==="object"?e[n]=Z(Z({},e[n]),r):e[n]=r}}return e}var Tl="enum",y3=function(t,n,r,o,l){t[Tl]=Array.isArray(t[Tl])?t[Tl]:[],t[Tl].indexOf(n)===-1&&o.push(Yr(l.messages[Tl],t.fullField,t[Tl].join(", ")))},b3=function(t,n,r,o,l){if(t.pattern){if(t.pattern instanceof RegExp)t.pattern.lastIndex=0,t.pattern.test(n)||o.push(Yr(l.messages.pattern.mismatch,t.fullField,n,t.pattern));else if(typeof t.pattern=="string"){var c=new RegExp(t.pattern);c.test(n)||o.push(Yr(l.messages.pattern.mismatch,t.fullField,n,t.pattern))}}},S3=function(t,n,r,o,l){var c=typeof t.len=="number",d=typeof t.min=="number",m=typeof t.max=="number",v=/[\uD800-\uDBFF][\uDC00-\uDFFF]/g,g=n,p=null,y=typeof n=="number",E=typeof n=="string",C=Array.isArray(n);if(y?p="number":E?p="string":C&&(p="array"),!p)return!1;C&&(g=n.length),E&&(g=n.replace(v,"_").length),c?g!==t.len&&o.push(Yr(l.messages[p].len,t.fullField,t.len)):d&&!m&&g<t.min?o.push(Yr(l.messages[p].min,t.fullField,t.min)):m&&!d&&g>t.max?o.push(Yr(l.messages[p].max,t.fullField,t.max)):d&&m&&(g<t.min||g>t.max)&&o.push(Yr(l.messages[p].range,t.fullField,t.min,t.max))},eO=function(t,n,r,o,l,c){t.required&&(!r.hasOwnProperty(t.field)||Jn(n,c||t.type))&&o.push(Yr(l.messages.required,t.fullField))},yf;const C3=function(){if(yf)return yf;var e="[a-fA-F\\d:]",t=function(M){return M&&M.includeBoundaries?"(?:(?<=\\s|^)(?=".concat(e,")|(?<=").concat(e,")(?=\\s|$))"):""},n="(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)(?:\\.(?:25[0-5]|2[0-4]\\d|1\\d\\d|[1-9]\\d|\\d)){3}",r="[a-fA-F\\d]{1,4}",o=["(?:".concat(r,":){7}(?:").concat(r,"|:)"),"(?:".concat(r,":){6}(?:").concat(n,"|:").concat(r,"|:)"),"(?:".concat(r,":){5}(?::").concat(n,"|(?::").concat(r,"){1,2}|:)"),"(?:".concat(r,":){4}(?:(?::").concat(r,"){0,1}:").concat(n,"|(?::").concat(r,"){1,3}|:)"),"(?:".concat(r,":){3}(?:(?::").concat(r,"){0,2}:").concat(n,"|(?::").concat(r,"){1,4}|:)"),"(?:".concat(r,":){2}(?:(?::").concat(r,"){0,3}:").concat(n,"|(?::").concat(r,"){1,5}|:)"),"(?:".concat(r,":){1}(?:(?::").concat(r,"){0,4}:").concat(n,"|(?::").concat(r,"){1,6}|:)"),"(?::(?:(?::".concat(r,"){0,5}:").concat(n,"|(?::").concat(r,"){1,7}|:))")],l="(?:%[0-9a-zA-Z]{1,})?",c="(?:".concat(o.join("|"),")").concat(l),d=new RegExp("(?:^".concat(n,"$)|(?:^").concat(c,"$)")),m=new RegExp("^".concat(n,"$")),v=new RegExp("^".concat(c,"$")),g=function(M){return M&&M.exact?d:new RegExp("(?:".concat(t(M)).concat(n).concat(t(M),")|(?:").concat(t(M)).concat(c).concat(t(M),")"),"g")};g.v4=function(R){return R&&R.exact?m:new RegExp("".concat(t(R)).concat(n).concat(t(R)),"g")},g.v6=function(R){return R&&R.exact?v:new RegExp("".concat(t(R)).concat(c).concat(t(R)),"g")};var p="(?:(?:[a-z]+:)?//)",y="(?:\\S+(?::\\S*)?@)?",E=g.v4().source,C=g.v6().source,x="(?:(?:[a-z\\u00a1-\\uffff0-9][-_]*)*[a-z\\u00a1-\\uffff0-9]+)",b="(?:\\.(?:[a-z\\u00a1-\\uffff0-9]-*)*[a-z\\u00a1-\\uffff0-9]+)*",w="(?:\\.(?:[a-z\\u00a1-\\uffff]{2,}))",$="(?::\\d{2,5})?",O='(?:[/?#][^\\s"]*)?',T="(?:".concat(p,"|www\\.)").concat(y,"(?:localhost|").concat(E,"|").concat(C,"|").concat(x).concat(b).concat(w,")").concat($).concat(O);return yf=new RegExp("(?:^".concat(T,"$)"),"i"),yf};var ox={email:/^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]+\.)+[a-zA-Z\u00A0-\uD7FF\uF900-\uFDCF\uFDF0-\uFFEF]{2,}))$/,hex:/^#?([a-f0-9]{6}|[a-f0-9]{3})$/i},mc={integer:function(t){return mc.number(t)&&parseInt(t,10)===t},float:function(t){return mc.number(t)&&!mc.integer(t)},array:function(t){return Array.isArray(t)},regexp:function(t){if(t instanceof RegExp)return!0;try{return!!new RegExp(t)}catch{return!1}},date:function(t){return typeof t.getTime=="function"&&typeof t.getMonth=="function"&&typeof t.getYear=="function"&&!isNaN(t.getTime())},number:function(t){return isNaN(t)?!1:typeof t=="number"},object:function(t){return lt(t)==="object"&&!mc.array(t)},method:function(t){return typeof t=="function"},email:function(t){return typeof t=="string"&&t.length<=320&&!!t.match(ox.email)},url:function(t){return typeof t=="string"&&t.length<=2048&&!!t.match(C3())},hex:function(t){return typeof t=="string"&&!!t.match(ox.hex)}},x3=function(t,n,r,o,l){if(t.required&&n===void 0){eO(t,n,r,o,l);return}var c=["integer","float","array","regexp","object","method","email","number","date","url","hex"],d=t.type;c.indexOf(d)>-1?mc[d](n)||o.push(Yr(l.messages.types[d],t.fullField,t.type)):d&&lt(n)!==t.type&&o.push(Yr(l.messages.types[d],t.fullField,t.type))},E3=function(t,n,r,o,l){(/^\s+$/.test(n)||n==="")&&o.push(Yr(l.messages.whitespace,t.fullField))};const Ht={required:eO,whitespace:E3,type:x3,range:S3,enum:y3,pattern:b3};var w3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n)&&!t.required)return r();Ht.required(t,n,o,c,l)}r(c)},$3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(n==null&&!t.required)return r();Ht.required(t,n,o,c,l,"array"),n!=null&&(Ht.type(t,n,o,c,l),Ht.range(t,n,o,c,l))}r(c)},O3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n)&&!t.required)return r();Ht.required(t,n,o,c,l),n!==void 0&&Ht.type(t,n,o,c,l)}r(c)},R3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n,"date")&&!t.required)return r();if(Ht.required(t,n,o,c,l),!Jn(n,"date")){var m;n instanceof Date?m=n:m=new Date(n),Ht.type(t,m,o,c,l),m&&Ht.range(t,m.getTime(),o,c,l)}}r(c)},T3="enum",M3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n)&&!t.required)return r();Ht.required(t,n,o,c,l),n!==void 0&&Ht[T3](t,n,o,c,l)}r(c)},_3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n)&&!t.required)return r();Ht.required(t,n,o,c,l),n!==void 0&&(Ht.type(t,n,o,c,l),Ht.range(t,n,o,c,l))}r(c)},N3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n)&&!t.required)return r();Ht.required(t,n,o,c,l),n!==void 0&&(Ht.type(t,n,o,c,l),Ht.range(t,n,o,c,l))}r(c)},A3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n)&&!t.required)return r();Ht.required(t,n,o,c,l),n!==void 0&&Ht.type(t,n,o,c,l)}r(c)},I3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(n===""&&(n=void 0),Jn(n)&&!t.required)return r();Ht.required(t,n,o,c,l),n!==void 0&&(Ht.type(t,n,o,c,l),Ht.range(t,n,o,c,l))}r(c)},j3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n)&&!t.required)return r();Ht.required(t,n,o,c,l),n!==void 0&&Ht.type(t,n,o,c,l)}r(c)},P3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n,"string")&&!t.required)return r();Ht.required(t,n,o,c,l),Jn(n,"string")||Ht.pattern(t,n,o,c,l)}r(c)},z3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n)&&!t.required)return r();Ht.required(t,n,o,c,l),Jn(n)||Ht.type(t,n,o,c,l)}r(c)},D3=function(t,n,r,o,l){var c=[],d=Array.isArray(n)?"array":lt(n);Ht.required(t,n,o,c,l,d),r(c)},B3=function(t,n,r,o,l){var c=[],d=t.required||!t.required&&o.hasOwnProperty(t.field);if(d){if(Jn(n,"string")&&!t.required)return r();Ht.required(t,n,o,c,l,"string"),Jn(n,"string")||(Ht.type(t,n,o,c,l),Ht.range(t,n,o,c,l),Ht.pattern(t,n,o,c,l),t.whitespace===!0&&Ht.whitespace(t,n,o,c,l))}r(c)},Vg=function(t,n,r,o,l){var c=t.type,d=[],m=t.required||!t.required&&o.hasOwnProperty(t.field);if(m){if(Jn(n,c)&&!t.required)return r();Ht.required(t,n,o,d,l,c),Jn(n,c)||Ht.type(t,n,o,d,l)}r(d)};const yc={string:B3,method:A3,number:I3,boolean:O3,regexp:z3,integer:N3,float:_3,array:$3,object:j3,enum:M3,pattern:P3,date:R3,url:Vg,hex:Vg,email:Vg,required:D3,any:w3};var Qc=function(){function e(t){qn(this,e),Y(this,"rules",null),Y(this,"_messages",sp),this.define(t)}return Wn(e,[{key:"define",value:function(n){var r=this;if(!n)throw new Error("Cannot configure a schema with no rules");if(lt(n)!=="object"||Array.isArray(n))throw new Error("Rules must be an object");this.rules={},Object.keys(n).forEach(function(o){var l=n[o];r.rules[o]=Array.isArray(l)?l:[l]})}},{key:"messages",value:function(n){return n&&(this._messages=ax(lp(),n)),this._messages}},{key:"validate",value:function(n){var r=this,o=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},l=arguments.length>2&&arguments[2]!==void 0?arguments[2]:function(){},c=n,d=o,m=l;if(typeof d=="function"&&(m=d,d={}),!this.rules||Object.keys(this.rules).length===0)return m&&m(null,c),Promise.resolve(c);function v(C){var x=[],b={};function w(O){if(Array.isArray(O)){var T;x=(T=x).concat.apply(T,Te(O))}else x.push(O)}for(var $=0;$<C.length;$++)w(C[$]);x.length?(b=up(x),m(x,b)):m(null,c)}if(d.messages){var g=this.messages();g===sp&&(g=lp()),ax(g,d.messages),d.messages=g}else d.messages=this.messages();var p={},y=d.keys||Object.keys(this.rules);y.forEach(function(C){var x=r.rules[C],b=c[C];x.forEach(function(w){var $=w;typeof $.transform=="function"&&(c===n&&(c=Z({},c)),b=c[C]=$.transform(b),b!=null&&($.type=$.type||(Array.isArray(b)?"array":lt(b)))),typeof $=="function"?$={validator:$}:$=Z({},$),$.validator=r.getValidationMethod($),$.validator&&($.field=C,$.fullField=$.fullField||C,$.type=r.getType($),p[C]=p[C]||[],p[C].push({rule:$,value:b,source:c,field:C}))})});var E={};return g3(p,d,function(C,x){var b=C.rule,w=(b.type==="object"||b.type==="array")&&(lt(b.fields)==="object"||lt(b.defaultField)==="object");w=w&&(b.required||!b.required&&C.value),b.field=C.field;function $(N,z){return Z(Z({},z),{},{fullField:"".concat(b.fullField,".").concat(N),fullFields:b.fullFields?[].concat(Te(b.fullFields),[N]):[N]})}function O(){var N=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[],z=Array.isArray(N)?N:[N];!d.suppressWarning&&z.length&&e.warning("async-validator:",z),z.length&&b.message!==void 0&&(z=[].concat(b.message));var P=z.map(rx(b,c));if(d.first&&P.length)return E[b.field]=1,x(P);if(!w)x(P);else{if(b.required&&!C.value)return b.message!==void 0?P=[].concat(b.message).map(rx(b,c)):d.error&&(P=[d.error(b,Yr(d.messages.required,b.field))]),x(P);var D={};b.defaultField&&Object.keys(C.value).map(function(V){D[V]=b.defaultField}),D=Z(Z({},D),C.rule.fields);var F={};Object.keys(D).forEach(function(V){var q=D[V],K=Array.isArray(q)?q:[q];F[V]=K.map($.bind(null,V))});var W=new e(F);W.messages(d.messages),C.rule.options&&(C.rule.options.messages=d.messages,C.rule.options.error=d.error),W.validate(C.value,C.rule.options||d,function(V){var q=[];P&&P.length&&q.push.apply(q,Te(P)),V&&V.length&&q.push.apply(q,Te(V)),x(q.length?q:null)})}}var T;if(b.asyncValidator)T=b.asyncValidator(b,C.value,O,C.source,d);else if(b.validator){try{T=b.validator(b,C.value,O,C.source,d)}catch(N){var R,M;(R=(M=console).error)===null||R===void 0||R.call(M,N),d.suppressValidatorError||setTimeout(function(){throw N},0),O(N.message)}T===!0?O():T===!1?O(typeof b.message=="function"?b.message(b.fullField||b.field):b.message||"".concat(b.fullField||b.field," fails")):T instanceof Array?O(T):T instanceof Error&&O(T.message)}T&&T.then&&T.then(function(){return O()},function(N){return O(N)})},function(C){v(C)},c)}},{key:"getType",value:function(n){if(n.type===void 0&&n.pattern instanceof RegExp&&(n.type="pattern"),typeof n.validator!="function"&&n.type&&!yc.hasOwnProperty(n.type))throw new Error(Yr("Unknown rule type %s",n.type));return n.type||"string"}},{key:"getValidationMethod",value:function(n){if(typeof n.validator=="function")return n.validator;var r=Object.keys(n),o=r.indexOf("message");return o!==-1&&r.splice(o,1),r.length===1&&r[0]==="required"?yc.required:yc[this.getType(n)]||void 0}}]),e}();Y(Qc,"register",function(t,n){if(typeof n!="function")throw new Error("Cannot register a validator by type, validator is not a function");yc[t]=n});Y(Qc,"warning",f3);Y(Qc,"messages",sp);Y(Qc,"validators",yc);var Gr="'${name}' is not a valid ${type}",tO={default:"Validation error on field '${name}'",required:"'${name}' is required",enum:"'${name}' must be one of [${enum}]",whitespace:"'${name}' cannot be empty",date:{format:"'${name}' is invalid for format date",parse:"'${name}' could not be parsed as date",invalid:"'${name}' is invalid date"},types:{string:Gr,method:Gr,array:Gr,object:Gr,number:Gr,date:Gr,boolean:Gr,integer:Gr,float:Gr,regexp:Gr,email:Gr,url:Gr,hex:Gr},string:{len:"'${name}' must be exactly ${len} characters",min:"'${name}' must be at least ${min} characters",max:"'${name}' cannot be longer than ${max} characters",range:"'${name}' must be between ${min} and ${max} characters"},number:{len:"'${name}' must equal ${len}",min:"'${name}' cannot be less than ${min}",max:"'${name}' cannot be greater than ${max}",range:"'${name}' must be between ${min} and ${max}"},array:{len:"'${name}' must be exactly ${len} in length",min:"'${name}' cannot be less than ${min} in length",max:"'${name}' cannot be greater than ${max} in length",range:"'${name}' must be between ${min} and ${max} in length"},pattern:{mismatch:"'${name}' does not match pattern ${pattern}"}},ix=Qc;function L3(e,t){return e.replace(/\\?\$\{\w+\}/g,function(n){if(n.startsWith("\\"))return n.slice(1);var r=n.slice(2,-1);return t[r]})}var lx="CODE_LOGIC_ERROR";function fp(e,t,n,r,o){return dp.apply(this,arguments)}function dp(){return dp=Gi(Or().mark(function e(t,n,r,o,l){var c,d,m,v,g,p,y,E,C;return Or().wrap(function(b){for(;;)switch(b.prev=b.next){case 0:return c=Z({},r),delete c.ruleIndex,ix.warning=function(){},c.validator&&(d=c.validator,c.validator=function(){try{return d.apply(void 0,arguments)}catch(w){return console.error(w),Promise.reject(lx)}}),m=null,c&&c.type==="array"&&c.defaultField&&(m=c.defaultField,delete c.defaultField),v=new ix(Y({},t,[c])),g=Pl(tO,o.validateMessages),v.messages(g),p=[],b.prev=10,b.next=13,Promise.resolve(v.validate(Y({},t,n),Z({},o)));case 13:b.next=18;break;case 15:b.prev=15,b.t0=b.catch(10),b.t0.errors&&(p=b.t0.errors.map(function(w,$){var O=w.message,T=O===lx?g.default:O;return u.isValidElement(T)?u.cloneElement(T,{key:"error_".concat($)}):T}));case 18:if(!(!p.length&&m)){b.next=23;break}return b.next=21,Promise.all(n.map(function(w,$){return fp("".concat(t,".").concat($),w,m,o,l)}));case 21:return y=b.sent,b.abrupt("return",y.reduce(function(w,$){return[].concat(Te(w),Te($))},[]));case 23:return E=Z(Z({},r),{},{name:t,enum:(r.enum||[]).join(", ")},l),C=p.map(function(w){return typeof w=="string"?L3(w,E):w}),b.abrupt("return",C);case 26:case"end":return b.stop()}},e,null,[[10,15]])})),dp.apply(this,arguments)}function H3(e,t,n,r,o,l){var c=e.join("."),d=n.map(function(g,p){var y=g.validator,E=Z(Z({},g),{},{ruleIndex:p});return y&&(E.validator=function(C,x,b){var w=!1,$=function(){for(var R=arguments.length,M=new Array(R),N=0;N<R;N++)M[N]=arguments[N];Promise.resolve().then(function(){vr(!w,"Your validator function has already return a promise. `callback` will be ignored."),w||b.apply(void 0,M)})},O=y(C,x,$);w=O&&typeof O.then=="function"&&typeof O.catch=="function",vr(w,"`callback` is deprecated. Please return a promise instead."),w&&O.then(function(){b()}).catch(function(T){b(T||" ")})}),E}).sort(function(g,p){var y=g.warningOnly,E=g.ruleIndex,C=p.warningOnly,x=p.ruleIndex;return!!y==!!C?E-x:y?1:-1}),m;if(o===!0)m=new Promise(function(){var g=Gi(Or().mark(function p(y,E){var C,x,b;return Or().wrap(function($){for(;;)switch($.prev=$.next){case 0:C=0;case 1:if(!(C<d.length)){$.next=12;break}return x=d[C],$.next=5,fp(c,t,x,r,l);case 5:if(b=$.sent,!b.length){$.next=9;break}return E([{errors:b,rule:x}]),$.abrupt("return");case 9:C+=1,$.next=1;break;case 12:y([]);case 13:case"end":return $.stop()}},p)}));return function(p,y){return g.apply(this,arguments)}}());else{var v=d.map(function(g){return fp(c,t,g,r,l).then(function(p){return{errors:p,rule:g}})});m=(o?V3(v):F3(v)).then(function(g){return Promise.reject(g)})}return m.catch(function(g){return g}),m}function F3(e){return mp.apply(this,arguments)}function mp(){return mp=Gi(Or().mark(function e(t){return Or().wrap(function(r){for(;;)switch(r.prev=r.next){case 0:return r.abrupt("return",Promise.all(t).then(function(o){var l,c=(l=[]).concat.apply(l,Te(o));return c}));case 1:case"end":return r.stop()}},e)})),mp.apply(this,arguments)}function V3(e){return vp.apply(this,arguments)}function vp(){return vp=Gi(Or().mark(function e(t){var n;return Or().wrap(function(o){for(;;)switch(o.prev=o.next){case 0:return n=0,o.abrupt("return",new Promise(function(l){t.forEach(function(c){c.then(function(d){d.errors.length&&l([d]),n+=1,n===t.length&&l([])})})}));case 2:case"end":return o.stop()}},e)})),vp.apply(this,arguments)}function Bn(e){return ip(e)}function sx(e,t){var n={};return t.forEach(function(r){var o=ba(e,r);n=ya(n,r,o)}),n}function Ll(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!1;return e&&e.some(function(r){return nO(t,r,n)})}function nO(e,t){var n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!1;return!e||!t||!n&&e.length!==t.length?!1:t.every(function(r,o){return e[o]===r})}function U3(e,t){if(e===t)return!0;if(!e&&t||e&&!t||!e||!t||lt(e)!=="object"||lt(t)!=="object")return!1;var n=Object.keys(e),r=Object.keys(t),o=new Set([].concat(n,r));return Te(o).every(function(l){var c=e[l],d=t[l];return typeof c=="function"&&typeof d=="function"?!0:c===d})}function q3(e){var t=arguments.length<=1?void 0:arguments[1];return t&&t.target&&lt(t.target)==="object"&&e in t.target?t.target[e]:t}function cx(e,t,n){var r=e.length;if(t<0||t>=r||n<0||n>=r)return e;var o=e[t],l=t-n;return l>0?[].concat(Te(e.slice(0,n)),[o],Te(e.slice(n,t)),Te(e.slice(t+1,r))):l<0?[].concat(Te(e.slice(0,t)),Te(e.slice(t+1,n+1)),[o],Te(e.slice(n+1,r))):e}var W3=["name"],va=[];function Ug(e,t,n,r,o,l){return typeof e=="function"?e(t,n,"source"in l?{source:l.source}:{}):r!==o}var C0=function(e){$o(n,e);var t=Oo(n);function n(r){var o;if(qn(this,n),o=t.call(this,r),Y(_t(o),"state",{resetCount:0}),Y(_t(o),"cancelRegisterFunc",null),Y(_t(o),"mounted",!1),Y(_t(o),"touched",!1),Y(_t(o),"dirty",!1),Y(_t(o),"validatePromise",void 0),Y(_t(o),"prevValidating",void 0),Y(_t(o),"errors",va),Y(_t(o),"warnings",va),Y(_t(o),"cancelRegister",function(){var m=o.props,v=m.preserve,g=m.isListField,p=m.name;o.cancelRegisterFunc&&o.cancelRegisterFunc(g,v,Bn(p)),o.cancelRegisterFunc=null}),Y(_t(o),"getNamePath",function(){var m=o.props,v=m.name,g=m.fieldContext,p=g.prefixName,y=p===void 0?[]:p;return v!==void 0?[].concat(Te(y),Te(v)):[]}),Y(_t(o),"getRules",function(){var m=o.props,v=m.rules,g=v===void 0?[]:v,p=m.fieldContext;return g.map(function(y){return typeof y=="function"?y(p):y})}),Y(_t(o),"refresh",function(){o.mounted&&o.setState(function(m){var v=m.resetCount;return{resetCount:v+1}})}),Y(_t(o),"metaCache",null),Y(_t(o),"triggerMetaEvent",function(m){var v=o.props.onMetaChange;if(v){var g=Z(Z({},o.getMeta()),{},{destroy:m});Oc(o.metaCache,g)||v(g),o.metaCache=g}else o.metaCache=null}),Y(_t(o),"onStoreChange",function(m,v,g){var p=o.props,y=p.shouldUpdate,E=p.dependencies,C=E===void 0?[]:E,x=p.onReset,b=g.store,w=o.getNamePath(),$=o.getValue(m),O=o.getValue(b),T=v&&Ll(v,w);switch(g.type==="valueUpdate"&&g.source==="external"&&!Oc($,O)&&(o.touched=!0,o.dirty=!0,o.validatePromise=null,o.errors=va,o.warnings=va,o.triggerMetaEvent()),g.type){case"reset":if(!v||T){o.touched=!1,o.dirty=!1,o.validatePromise=void 0,o.errors=va,o.warnings=va,o.triggerMetaEvent(),x==null||x(),o.refresh();return}break;case"remove":{if(y&&Ug(y,m,b,$,O,g)){o.reRender();return}break}case"setField":{var R=g.data;if(T){"touched"in R&&(o.touched=R.touched),"validating"in R&&!("originRCField"in R)&&(o.validatePromise=R.validating?Promise.resolve([]):null),"errors"in R&&(o.errors=R.errors||va),"warnings"in R&&(o.warnings=R.warnings||va),o.dirty=!0,o.triggerMetaEvent(),o.reRender();return}else if("value"in R&&Ll(v,w,!0)){o.reRender();return}if(y&&!w.length&&Ug(y,m,b,$,O,g)){o.reRender();return}break}case"dependenciesUpdate":{var M=C.map(Bn);if(M.some(function(N){return Ll(g.relatedFields,N)})){o.reRender();return}break}default:if(T||(!C.length||w.length||y)&&Ug(y,m,b,$,O,g)){o.reRender();return}break}y===!0&&o.reRender()}),Y(_t(o),"validateRules",function(m){var v=o.getNamePath(),g=o.getValue(),p=m||{},y=p.triggerName,E=p.validateOnly,C=E===void 0?!1:E,x=Promise.resolve().then(Gi(Or().mark(function b(){var w,$,O,T,R,M,N;return Or().wrap(function(P){for(;;)switch(P.prev=P.next){case 0:if(o.mounted){P.next=2;break}return P.abrupt("return",[]);case 2:if(w=o.props,$=w.validateFirst,O=$===void 0?!1:$,T=w.messageVariables,R=w.validateDebounce,M=o.getRules(),y&&(M=M.filter(function(D){return D}).filter(function(D){var F=D.validateTrigger;if(!F)return!0;var W=ip(F);return W.includes(y)})),!(R&&y)){P.next=10;break}return P.next=8,new Promise(function(D){setTimeout(D,R)});case 8:if(o.validatePromise===x){P.next=10;break}return P.abrupt("return",[]);case 10:return N=H3(v,g,M,m,O,T),N.catch(function(D){return D}).then(function(){var D=arguments.length>0&&arguments[0]!==void 0?arguments[0]:va;if(o.validatePromise===x){var F;o.validatePromise=null;var W=[],V=[];(F=D.forEach)===null||F===void 0||F.call(D,function(q){var K=q.rule.warningOnly,_=q.errors,j=_===void 0?va:_;K?V.push.apply(V,Te(j)):W.push.apply(W,Te(j))}),o.errors=W,o.warnings=V,o.triggerMetaEvent(),o.reRender()}}),P.abrupt("return",N);case 13:case"end":return P.stop()}},b)})));return C||(o.validatePromise=x,o.dirty=!0,o.errors=va,o.warnings=va,o.triggerMetaEvent(),o.reRender()),x}),Y(_t(o),"isFieldValidating",function(){return!!o.validatePromise}),Y(_t(o),"isFieldTouched",function(){return o.touched}),Y(_t(o),"isFieldDirty",function(){if(o.dirty||o.props.initialValue!==void 0)return!0;var m=o.props.fieldContext,v=m.getInternalHooks(Ai),g=v.getInitialValue;return g(o.getNamePath())!==void 0}),Y(_t(o),"getErrors",function(){return o.errors}),Y(_t(o),"getWarnings",function(){return o.warnings}),Y(_t(o),"isListField",function(){return o.props.isListField}),Y(_t(o),"isList",function(){return o.props.isList}),Y(_t(o),"isPreserve",function(){return o.props.preserve}),Y(_t(o),"getMeta",function(){o.prevValidating=o.isFieldValidating();var m={touched:o.isFieldTouched(),validating:o.prevValidating,errors:o.errors,warnings:o.warnings,name:o.getNamePath(),validated:o.validatePromise===null};return m}),Y(_t(o),"getOnlyChild",function(m){if(typeof m=="function"){var v=o.getMeta();return Z(Z({},o.getOnlyChild(m(o.getControlled(),v,o.props.fieldContext))),{},{isFunction:!0})}var g=Pr(m);return g.length!==1||!u.isValidElement(g[0])?{child:g,isFunction:!1}:{child:g[0],isFunction:!1}}),Y(_t(o),"getValue",function(m){var v=o.props.fieldContext.getFieldsValue,g=o.getNamePath();return ba(m||v(!0),g)}),Y(_t(o),"getControlled",function(){var m=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},v=o.props,g=v.name,p=v.trigger,y=v.validateTrigger,E=v.getValueFromEvent,C=v.normalize,x=v.valuePropName,b=v.getValueProps,w=v.fieldContext,$=y!==void 0?y:w.validateTrigger,O=o.getNamePath(),T=w.getInternalHooks,R=w.getFieldsValue,M=T(Ai),N=M.dispatch,z=o.getValue(),P=b||function(q){return Y({},x,q)},D=m[p],F=g!==void 0?P(z):{},W=Z(Z({},m),F);W[p]=function(){o.touched=!0,o.dirty=!0,o.triggerMetaEvent();for(var q,K=arguments.length,_=new Array(K),j=0;j<K;j++)_[j]=arguments[j];E?q=E.apply(void 0,_):q=q3.apply(void 0,[x].concat(_)),C&&(q=C(q,z,R(!0))),q!==z&&N({type:"updateValue",namePath:O,value:q}),D&&D.apply(void 0,_)};var V=ip($||[]);return V.forEach(function(q){var K=W[q];W[q]=function(){K&&K.apply(void 0,arguments);var _=o.props.rules;_&&_.length&&N({type:"validateField",namePath:O,triggerName:q})}}),W}),r.fieldContext){var l=r.fieldContext.getInternalHooks,c=l(Ai),d=c.initEntityValue;d(_t(o))}return o}return Wn(n,[{key:"componentDidMount",value:function(){var o=this.props,l=o.shouldUpdate,c=o.fieldContext;if(this.mounted=!0,c){var d=c.getInternalHooks,m=d(Ai),v=m.registerField;this.cancelRegisterFunc=v(this)}l===!0&&this.reRender()}},{key:"componentWillUnmount",value:function(){this.cancelRegister(),this.triggerMetaEvent(!0),this.mounted=!1}},{key:"reRender",value:function(){this.mounted&&this.forceUpdate()}},{key:"render",value:function(){var o=this.state.resetCount,l=this.props.children,c=this.getOnlyChild(l),d=c.child,m=c.isFunction,v;return m?v=d:u.isValidElement(d)?v=u.cloneElement(d,this.getControlled(d.props)):(vr(!d,"`children` of Field is not validate ReactElement."),v=d),u.createElement(u.Fragment,{key:o},v)}}]),n}(u.Component);Y(C0,"contextType",Vi);Y(C0,"defaultProps",{trigger:"onChange",valuePropName:"value"});function x0(e){var t,n=e.name,r=wt(e,W3),o=u.useContext(Vi),l=u.useContext(Nc),c=n!==void 0?Bn(n):void 0,d=(t=r.isListField)!==null&&t!==void 0?t:!!l,m="keep";return d||(m="_".concat((c||[]).join("_"))),u.createElement(C0,Ie({key:m,name:c,isListField:d},r,{fieldContext:o}))}function rO(e){var t=e.name,n=e.initialValue,r=e.children,o=e.rules,l=e.validateTrigger,c=e.isListField,d=u.useContext(Vi),m=u.useContext(Nc),v=u.useRef({keys:[],id:0}),g=v.current,p=u.useMemo(function(){var x=Bn(d.prefixName)||[];return[].concat(Te(x),Te(Bn(t)))},[d.prefixName,t]),y=u.useMemo(function(){return Z(Z({},d),{},{prefixName:p})},[d,p]),E=u.useMemo(function(){return{getKey:function(b){var w=p.length,$=b[w];return[g.keys[$],b.slice(w+1)]}}},[p]);if(typeof r!="function")return vr(!1,"Form.List only accepts function as children."),null;var C=function(b,w,$){var O=$.source;return O==="internal"?!1:b!==w};return u.createElement(Nc.Provider,{value:E},u.createElement(Vi.Provider,{value:y},u.createElement(x0,{name:[],shouldUpdate:C,rules:o,validateTrigger:l,initialValue:n,isList:!0,isListField:c??!!m},function(x,b){var w=x.value,$=w===void 0?[]:w,O=x.onChange,T=d.getFieldValue,R=function(){var P=T(p||[]);return P||[]},M={add:function(P,D){var F=R();D>=0&&D<=F.length?(g.keys=[].concat(Te(g.keys.slice(0,D)),[g.id],Te(g.keys.slice(D))),O([].concat(Te(F.slice(0,D)),[P],Te(F.slice(D))))):(g.keys=[].concat(Te(g.keys),[g.id]),O([].concat(Te(F),[P]))),g.id+=1},remove:function(P){var D=R(),F=new Set(Array.isArray(P)?P:[P]);F.size<=0||(g.keys=g.keys.filter(function(W,V){return!F.has(V)}),O(D.filter(function(W,V){return!F.has(V)})))},move:function(P,D){if(P!==D){var F=R();P<0||P>=F.length||D<0||D>=F.length||(g.keys=cx(g.keys,P,D),O(cx(F,P,D)))}}},N=$||[];return Array.isArray(N)||(N=[]),r(N.map(function(z,P){var D=g.keys[P];return D===void 0&&(g.keys[P]=g.id,D=g.keys[P],g.id+=1),{name:P,key:D,isListField:!0}}),M,b)})))}function k3(e){var t=!1,n=e.length,r=[];return e.length?new Promise(function(o,l){e.forEach(function(c,d){c.catch(function(m){return t=!0,m}).then(function(m){n-=1,r[d]=m,!(n>0)&&(t&&l(r),o(r))})})}):Promise.resolve([])}var aO="__@field_split__";function qg(e){return e.map(function(t){return"".concat(lt(t),":").concat(t)}).join(aO)}var Ml=function(){function e(){qn(this,e),Y(this,"kvs",new Map)}return Wn(e,[{key:"set",value:function(n,r){this.kvs.set(qg(n),r)}},{key:"get",value:function(n){return this.kvs.get(qg(n))}},{key:"update",value:function(n,r){var o=this.get(n),l=r(o);l?this.set(n,l):this.delete(n)}},{key:"delete",value:function(n){this.kvs.delete(qg(n))}},{key:"map",value:function(n){return Te(this.kvs.entries()).map(function(r){var o=de(r,2),l=o[0],c=o[1],d=l.split(aO);return n({key:d.map(function(m){var v=m.match(/^([^:]*):(.*)$/),g=de(v,3),p=g[1],y=g[2];return p==="number"?Number(y):y}),value:c})})}},{key:"toJSON",value:function(){var n={};return this.map(function(r){var o=r.key,l=r.value;return n[o.join(".")]=l,null}),n}}]),e}(),G3=["name"],K3=Wn(function e(t){var n=this;qn(this,e),Y(this,"formHooked",!1),Y(this,"forceRootUpdate",void 0),Y(this,"subscribable",!0),Y(this,"store",{}),Y(this,"fieldEntities",[]),Y(this,"initialValues",{}),Y(this,"callbacks",{}),Y(this,"validateMessages",null),Y(this,"preserve",null),Y(this,"lastValidatePromise",null),Y(this,"getForm",function(){return{getFieldValue:n.getFieldValue,getFieldsValue:n.getFieldsValue,getFieldError:n.getFieldError,getFieldWarning:n.getFieldWarning,getFieldsError:n.getFieldsError,isFieldsTouched:n.isFieldsTouched,isFieldTouched:n.isFieldTouched,isFieldValidating:n.isFieldValidating,isFieldsValidating:n.isFieldsValidating,resetFields:n.resetFields,setFields:n.setFields,setFieldValue:n.setFieldValue,setFieldsValue:n.setFieldsValue,validateFields:n.validateFields,submit:n.submit,_init:!0,getInternalHooks:n.getInternalHooks}}),Y(this,"getInternalHooks",function(r){return r===Ai?(n.formHooked=!0,{dispatch:n.dispatch,initEntityValue:n.initEntityValue,registerField:n.registerField,useSubscribe:n.useSubscribe,setInitialValues:n.setInitialValues,destroyForm:n.destroyForm,setCallbacks:n.setCallbacks,setValidateMessages:n.setValidateMessages,getFields:n.getFields,setPreserve:n.setPreserve,getInitialValue:n.getInitialValue,registerWatch:n.registerWatch}):(vr(!1,"`getInternalHooks` is internal usage. Should not call directly."),null)}),Y(this,"useSubscribe",function(r){n.subscribable=r}),Y(this,"prevWithoutPreserves",null),Y(this,"setInitialValues",function(r,o){if(n.initialValues=r||{},o){var l,c=Pl(r,n.store);(l=n.prevWithoutPreserves)===null||l===void 0||l.map(function(d){var m=d.key;c=ya(c,m,ba(r,m))}),n.prevWithoutPreserves=null,n.updateStore(c)}}),Y(this,"destroyForm",function(r){if(r)n.updateStore({});else{var o=new Ml;n.getFieldEntities(!0).forEach(function(l){n.isMergedPreserve(l.isPreserve())||o.set(l.getNamePath(),!0)}),n.prevWithoutPreserves=o}}),Y(this,"getInitialValue",function(r){var o=ba(n.initialValues,r);return r.length?Pl(o):o}),Y(this,"setCallbacks",function(r){n.callbacks=r}),Y(this,"setValidateMessages",function(r){n.validateMessages=r}),Y(this,"setPreserve",function(r){n.preserve=r}),Y(this,"watchList",[]),Y(this,"registerWatch",function(r){return n.watchList.push(r),function(){n.watchList=n.watchList.filter(function(o){return o!==r})}}),Y(this,"notifyWatch",function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[];if(n.watchList.length){var o=n.getFieldsValue(),l=n.getFieldsValue(!0);n.watchList.forEach(function(c){c(o,l,r)})}}),Y(this,"timeoutId",null),Y(this,"warningUnhooked",function(){}),Y(this,"updateStore",function(r){n.store=r}),Y(this,"getFieldEntities",function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1;return r?n.fieldEntities.filter(function(o){return o.getNamePath().length}):n.fieldEntities}),Y(this,"getFieldsMap",function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1,o=new Ml;return n.getFieldEntities(r).forEach(function(l){var c=l.getNamePath();o.set(c,l)}),o}),Y(this,"getFieldEntitiesForNamePathList",function(r){if(!r)return n.getFieldEntities(!0);var o=n.getFieldsMap(!0);return r.map(function(l){var c=Bn(l);return o.get(c)||{INVALIDATE_NAME_PATH:Bn(l)}})}),Y(this,"getFieldsValue",function(r,o){n.warningUnhooked();var l,c,d;if(r===!0||Array.isArray(r)?(l=r,c=o):r&&lt(r)==="object"&&(d=r.strict,c=r.filter),l===!0&&!c)return n.store;var m=n.getFieldEntitiesForNamePathList(Array.isArray(l)?l:null),v=[];return m.forEach(function(g){var p,y,E="INVALIDATE_NAME_PATH"in g?g.INVALIDATE_NAME_PATH:g.getNamePath();if(d){var C,x;if((C=(x=g).isList)!==null&&C!==void 0&&C.call(x))return}else if(!l&&(p=(y=g).isListField)!==null&&p!==void 0&&p.call(y))return;if(!c)v.push(E);else{var b="getMeta"in g?g.getMeta():null;c(b)&&v.push(E)}}),sx(n.store,v.map(Bn))}),Y(this,"getFieldValue",function(r){n.warningUnhooked();var o=Bn(r);return ba(n.store,o)}),Y(this,"getFieldsError",function(r){n.warningUnhooked();var o=n.getFieldEntitiesForNamePathList(r);return o.map(function(l,c){return l&&!("INVALIDATE_NAME_PATH"in l)?{name:l.getNamePath(),errors:l.getErrors(),warnings:l.getWarnings()}:{name:Bn(r[c]),errors:[],warnings:[]}})}),Y(this,"getFieldError",function(r){n.warningUnhooked();var o=Bn(r),l=n.getFieldsError([o])[0];return l.errors}),Y(this,"getFieldWarning",function(r){n.warningUnhooked();var o=Bn(r),l=n.getFieldsError([o])[0];return l.warnings}),Y(this,"isFieldsTouched",function(){n.warningUnhooked();for(var r=arguments.length,o=new Array(r),l=0;l<r;l++)o[l]=arguments[l];var c=o[0],d=o[1],m,v=!1;o.length===0?m=null:o.length===1?Array.isArray(c)?(m=c.map(Bn),v=!1):(m=null,v=c):(m=c.map(Bn),v=d);var g=n.getFieldEntities(!0),p=function(b){return b.isFieldTouched()};if(!m)return v?g.every(function(x){return p(x)||x.isList()}):g.some(p);var y=new Ml;m.forEach(function(x){y.set(x,[])}),g.forEach(function(x){var b=x.getNamePath();m.forEach(function(w){w.every(function($,O){return b[O]===$})&&y.update(w,function($){return[].concat(Te($),[x])})})});var E=function(b){return b.some(p)},C=y.map(function(x){var b=x.value;return b});return v?C.every(E):C.some(E)}),Y(this,"isFieldTouched",function(r){return n.warningUnhooked(),n.isFieldsTouched([r])}),Y(this,"isFieldsValidating",function(r){n.warningUnhooked();var o=n.getFieldEntities();if(!r)return o.some(function(c){return c.isFieldValidating()});var l=r.map(Bn);return o.some(function(c){var d=c.getNamePath();return Ll(l,d)&&c.isFieldValidating()})}),Y(this,"isFieldValidating",function(r){return n.warningUnhooked(),n.isFieldsValidating([r])}),Y(this,"resetWithFieldInitialValue",function(){var r=arguments.length>0&&arguments[0]!==void 0?arguments[0]:{},o=new Ml,l=n.getFieldEntities(!0);l.forEach(function(m){var v=m.props.initialValue,g=m.getNamePath();if(v!==void 0){var p=o.get(g)||new Set;p.add({entity:m,value:v}),o.set(g,p)}});var c=function(v){v.forEach(function(g){var p=g.props.initialValue;if(p!==void 0){var y=g.getNamePath(),E=n.getInitialValue(y);if(E!==void 0)vr(!1,"Form already set 'initialValues' with path '".concat(y.join("."),"'. Field can not overwrite it."));else{var C=o.get(y);if(C&&C.size>1)vr(!1,"Multiple Field with path '".concat(y.join("."),"' set 'initialValue'. Can not decide which one to pick."));else if(C){var x=n.getFieldValue(y),b=g.isListField();!b&&(!r.skipExist||x===void 0)&&n.updateStore(ya(n.store,y,Te(C)[0].value))}}}})},d;r.entities?d=r.entities:r.namePathList?(d=[],r.namePathList.forEach(function(m){var v=o.get(m);if(v){var g;(g=d).push.apply(g,Te(Te(v).map(function(p){return p.entity})))}})):d=l,c(d)}),Y(this,"resetFields",function(r){n.warningUnhooked();var o=n.store;if(!r){n.updateStore(Pl(n.initialValues)),n.resetWithFieldInitialValue(),n.notifyObservers(o,null,{type:"reset"}),n.notifyWatch();return}var l=r.map(Bn);l.forEach(function(c){var d=n.getInitialValue(c);n.updateStore(ya(n.store,c,d))}),n.resetWithFieldInitialValue({namePathList:l}),n.notifyObservers(o,l,{type:"reset"}),n.notifyWatch(l)}),Y(this,"setFields",function(r){n.warningUnhooked();var o=n.store,l=[];r.forEach(function(c){var d=c.name,m=wt(c,G3),v=Bn(d);l.push(v),"value"in m&&n.updateStore(ya(n.store,v,m.value)),n.notifyObservers(o,[v],{type:"setField",data:c})}),n.notifyWatch(l)}),Y(this,"getFields",function(){var r=n.getFieldEntities(!0),o=r.map(function(l){var c=l.getNamePath(),d=l.getMeta(),m=Z(Z({},d),{},{name:c,value:n.getFieldValue(c)});return Object.defineProperty(m,"originRCField",{value:!0}),m});return o}),Y(this,"initEntityValue",function(r){var o=r.props.initialValue;if(o!==void 0){var l=r.getNamePath(),c=ba(n.store,l);c===void 0&&n.updateStore(ya(n.store,l,o))}}),Y(this,"isMergedPreserve",function(r){var o=r!==void 0?r:n.preserve;return o??!0}),Y(this,"registerField",function(r){n.fieldEntities.push(r);var o=r.getNamePath();if(n.notifyWatch([o]),r.props.initialValue!==void 0){var l=n.store;n.resetWithFieldInitialValue({entities:[r],skipExist:!0}),n.notifyObservers(l,[r.getNamePath()],{type:"valueUpdate",source:"internal"})}return function(c,d){var m=arguments.length>2&&arguments[2]!==void 0?arguments[2]:[];if(n.fieldEntities=n.fieldEntities.filter(function(p){return p!==r}),!n.isMergedPreserve(d)&&(!c||m.length>1)){var v=c?void 0:n.getInitialValue(o);if(o.length&&n.getFieldValue(o)!==v&&n.fieldEntities.every(function(p){return!nO(p.getNamePath(),o)})){var g=n.store;n.updateStore(ya(g,o,v,!0)),n.notifyObservers(g,[o],{type:"remove"}),n.triggerDependenciesUpdate(g,o)}}n.notifyWatch([o])}}),Y(this,"dispatch",function(r){switch(r.type){case"updateValue":{var o=r.namePath,l=r.value;n.updateValue(o,l);break}case"validateField":{var c=r.namePath,d=r.triggerName;n.validateFields([c],{triggerName:d});break}}}),Y(this,"notifyObservers",function(r,o,l){if(n.subscribable){var c=Z(Z({},l),{},{store:n.getFieldsValue(!0)});n.getFieldEntities().forEach(function(d){var m=d.onStoreChange;m(r,o,c)})}else n.forceRootUpdate()}),Y(this,"triggerDependenciesUpdate",function(r,o){var l=n.getDependencyChildrenFields(o);return l.length&&n.validateFields(l),n.notifyObservers(r,l,{type:"dependenciesUpdate",relatedFields:[o].concat(Te(l))}),l}),Y(this,"updateValue",function(r,o){var l=Bn(r),c=n.store;n.updateStore(ya(n.store,l,o)),n.notifyObservers(c,[l],{type:"valueUpdate",source:"internal"}),n.notifyWatch([l]);var d=n.triggerDependenciesUpdate(c,l),m=n.callbacks.onValuesChange;if(m){var v=sx(n.store,[l]);m(v,n.getFieldsValue())}n.triggerOnFieldsChange([l].concat(Te(d)))}),Y(this,"setFieldsValue",function(r){n.warningUnhooked();var o=n.store;if(r){var l=Pl(n.store,r);n.updateStore(l)}n.notifyObservers(o,null,{type:"valueUpdate",source:"external"}),n.notifyWatch()}),Y(this,"setFieldValue",function(r,o){n.setFields([{name:r,value:o,errors:[],warnings:[]}])}),Y(this,"getDependencyChildrenFields",function(r){var o=new Set,l=[],c=new Ml;n.getFieldEntities().forEach(function(m){var v=m.props.dependencies;(v||[]).forEach(function(g){var p=Bn(g);c.update(p,function(){var y=arguments.length>0&&arguments[0]!==void 0?arguments[0]:new Set;return y.add(m),y})})});var d=function m(v){var g=c.get(v)||new Set;g.forEach(function(p){if(!o.has(p)){o.add(p);var y=p.getNamePath();p.isFieldDirty()&&y.length&&(l.push(y),m(y))}})};return d(r),l}),Y(this,"triggerOnFieldsChange",function(r,o){var l=n.callbacks.onFieldsChange;if(l){var c=n.getFields();if(o){var d=new Ml;o.forEach(function(v){var g=v.name,p=v.errors;d.set(g,p)}),c.forEach(function(v){v.errors=d.get(v.name)||v.errors})}var m=c.filter(function(v){var g=v.name;return Ll(r,g)});m.length&&l(m,c)}}),Y(this,"validateFields",function(r,o){n.warningUnhooked();var l,c;Array.isArray(r)||typeof r=="string"||typeof o=="string"?(l=r,c=o):c=r;var d=!!l,m=d?l.map(Bn):[],v=[],g=String(Date.now()),p=new Set,y=c||{},E=y.recursive,C=y.dirty;n.getFieldEntities(!0).forEach(function($){if(d||m.push($.getNamePath()),!(!$.props.rules||!$.props.rules.length)&&!(C&&!$.isFieldDirty())){var O=$.getNamePath();if(p.add(O.join(g)),!d||Ll(m,O,E)){var T=$.validateRules(Z({validateMessages:Z(Z({},tO),n.validateMessages)},c));v.push(T.then(function(){return{name:O,errors:[],warnings:[]}}).catch(function(R){var M,N=[],z=[];return(M=R.forEach)===null||M===void 0||M.call(R,function(P){var D=P.rule.warningOnly,F=P.errors;D?z.push.apply(z,Te(F)):N.push.apply(N,Te(F))}),N.length?Promise.reject({name:O,errors:N,warnings:z}):{name:O,errors:N,warnings:z}}))}}});var x=k3(v);n.lastValidatePromise=x,x.catch(function($){return $}).then(function($){var O=$.map(function(T){var R=T.name;return R});n.notifyObservers(n.store,O,{type:"validateFinish"}),n.triggerOnFieldsChange(O,$)});var b=x.then(function(){return n.lastValidatePromise===x?Promise.resolve(n.getFieldsValue(m)):Promise.reject([])}).catch(function($){var O=$.filter(function(T){return T&&T.errors.length});return Promise.reject({values:n.getFieldsValue(m),errorFields:O,outOfDate:n.lastValidatePromise!==x})});b.catch(function($){return $});var w=m.filter(function($){return p.has($.join(g))});return n.triggerOnFieldsChange(w),b}),Y(this,"submit",function(){n.warningUnhooked(),n.validateFields().then(function(r){var o=n.callbacks.onFinish;if(o)try{o(r)}catch(l){console.error(l)}}).catch(function(r){var o=n.callbacks.onFinishFailed;o&&o(r)})}),this.forceRootUpdate=t});function E0(e){var t=u.useRef(),n=u.useState({}),r=de(n,2),o=r[1];if(!t.current)if(e)t.current=e;else{var l=function(){o({})},c=new K3(l);t.current=c.getForm()}return[t.current]}var gp=u.createContext({triggerFormChange:function(){},triggerFormFinish:function(){},registerForm:function(){},unregisterForm:function(){}}),oO=function(t){var n=t.validateMessages,r=t.onFormChange,o=t.onFormFinish,l=t.children,c=u.useContext(gp),d=u.useRef({});return u.createElement(gp.Provider,{value:Z(Z({},c),{},{validateMessages:Z(Z({},c.validateMessages),n),triggerFormChange:function(v,g){r&&r(v,{changedFields:g,forms:d.current}),c.triggerFormChange(v,g)},triggerFormFinish:function(v,g){o&&o(v,{values:g,forms:d.current}),c.triggerFormFinish(v,g)},registerForm:function(v,g){v&&(d.current=Z(Z({},d.current),{},Y({},v,g))),c.registerForm(v,g)},unregisterForm:function(v){var g=Z({},d.current);delete g[v],d.current=g,c.unregisterForm(v)}})},l)},X3=["name","initialValues","fields","form","preserve","children","component","validateMessages","validateTrigger","onValuesChange","onFieldsChange","onFinish","onFinishFailed","clearOnDestroy"],Y3=function(t,n){var r=t.name,o=t.initialValues,l=t.fields,c=t.form,d=t.preserve,m=t.children,v=t.component,g=v===void 0?"form":v,p=t.validateMessages,y=t.validateTrigger,E=y===void 0?"onChange":y,C=t.onValuesChange,x=t.onFieldsChange,b=t.onFinish,w=t.onFinishFailed,$=t.clearOnDestroy,O=wt(t,X3),T=u.useRef(null),R=u.useContext(gp),M=E0(c),N=de(M,1),z=N[0],P=z.getInternalHooks(Ai),D=P.useSubscribe,F=P.setInitialValues,W=P.setCallbacks,V=P.setValidateMessages,q=P.setPreserve,K=P.destroyForm;u.useImperativeHandle(n,function(){return Z(Z({},z),{},{nativeElement:T.current})}),u.useEffect(function(){return R.registerForm(r,z),function(){R.unregisterForm(r)}},[R,z,r]),V(Z(Z({},R.validateMessages),p)),W({onValuesChange:C,onFieldsChange:function(U){if(R.triggerFormChange(r,U),x){for(var J=arguments.length,te=new Array(J>1?J-1:0),ve=1;ve<J;ve++)te[ve-1]=arguments[ve];x.apply(void 0,[U].concat(te))}},onFinish:function(U){R.triggerFormFinish(r,U),b&&b(U)},onFinishFailed:w}),q(d);var _=u.useRef(null);F(o,!_.current),_.current||(_.current=!0),u.useEffect(function(){return function(){return K($)}},[]);var j,B=typeof m=="function";if(B){var k=z.getFieldsValue(!0);j=m(k,z)}else j=m;D(!B);var I=u.useRef();u.useEffect(function(){U3(I.current||[],l||[])||z.setFields(l||[]),I.current=l},[l,z]);var H=u.useMemo(function(){return Z(Z({},z),{},{validateTrigger:E})},[z,E]),G=u.createElement(Nc.Provider,{value:null},u.createElement(Vi.Provider,{value:H},j));return g===!1?G:u.createElement(g,Ie({},O,{ref:T,onSubmit:function(U){U.preventDefault(),U.stopPropagation(),z.submit()},onReset:function(U){var J;U.preventDefault(),z.resetFields(),(J=O.onReset)===null||J===void 0||J.call(O,U)}}),G)};function ux(e){try{return JSON.stringify(e)}catch{return Math.random()}}function iO(){for(var e=arguments.length,t=new Array(e),n=0;n<e;n++)t[n]=arguments[n];var r=t[0],o=t[1],l=o===void 0?{}:o,c=l3(l)?{form:l}:l,d=c.form,m=u.useState(),v=de(m,2),g=v[0],p=v[1],y=u.useMemo(function(){return ux(g)},[g]),E=u.useRef(y);E.current=y;var C=u.useContext(Vi),x=d||C,b=x&&x._init,w=Bn(r),$=u.useRef(w);return $.current=w,u.useEffect(function(){if(b){var O=x.getFieldsValue,T=x.getInternalHooks,R=T(Ai),M=R.registerWatch,N=function(F,W){var V=c.preserve?W:F;return typeof r=="function"?r(V):ba(V,$.current)},z=M(function(D,F){var W=N(D,F),V=ux(W);E.current!==V&&(E.current=V,p(W))}),P=N(O(),O(!0));return g!==P&&p(P),z}},[b]),g}var Q3=u.forwardRef(Y3),ts=Q3;ts.FormProvider=oO;ts.Field=x0;ts.List=rO;ts.useForm=E0;ts.useWatch=iO;const xo=u.createContext({labelAlign:"right",vertical:!1,itemRef:()=>{}}),lO=u.createContext(null),sO=e=>{const t=Tn(e,["prefixCls"]);return u.createElement(oO,Object.assign({},t))},w0=u.createContext({prefixCls:""}),Jr=u.createContext({}),Z3=e=>{let{children:t,status:n,override:r}=e;const o=u.useContext(Jr),l=u.useMemo(()=>{const c=Object.assign({},o);return r&&delete c.isFormItemInput,n&&(delete c.status,delete c.hasFeedback,delete c.feedbackIcon),c},[n,r,o]);return u.createElement(Jr.Provider,{value:l},t)},cO=u.createContext(void 0),Ac=e=>{const{space:t,form:n,children:r}=e;if(r==null)return null;let o=r;return n&&(o=oe.createElement(Z3,{override:!0,status:!0},o)),t&&(o=oe.createElement(fP,null,o)),o};function fx(e){if(e)return{closable:e.closable,closeIcon:e.closeIcon}}function dx(e){const{closable:t,closeIcon:n}=e||{};return oe.useMemo(()=>{if(!t&&(t===!1||n===!1||n===null))return!1;if(t===void 0&&n===void 0)return null;let r={closeIcon:typeof n!="boolean"&&n!==null?n:void 0};return t&&typeof t=="object"&&(r=Object.assign(Object.assign({},r),t)),r},[t,n])}function mx(){const e={};for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return n.forEach(o=>{o&&Object.keys(o).forEach(l=>{o[l]!==void 0&&(e[l]=o[l])})}),e}const J3={};function e6(e,t){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:J3;const r=dx(e),o=dx(t),l=typeof r!="boolean"?!!(r!=null&&r.disabled):!1,c=oe.useMemo(()=>Object.assign({closeIcon:oe.createElement(Yl,null)},n),[n]),d=oe.useMemo(()=>r===!1?!1:r?mx(c,o,r):o===!1?!1:o?mx(c,o):c.closable?c:!1,[r,o,c]);return oe.useMemo(()=>{if(d===!1)return[!1,null,l];const{closeIconRender:m}=c,{closeIcon:v}=d;let g=v;if(g!=null){m&&(g=m(v));const p=Ga(d,!0);Object.keys(p).length&&(g=oe.isValidElement(g)?oe.cloneElement(g,p):oe.createElement("span",Object.assign({},p),g))}return[!0,g,l]},[d,c])}var uO=function(t){if(Cr()&&window.document.documentElement){var n=Array.isArray(t)?t:[t],r=window.document.documentElement;return n.some(function(o){return o in r.style})}return!1},t6=function(t,n){if(!uO(t))return!1;var r=document.createElement("div"),o=r.style[t];return r.style[t]=n,r.style[t]!==o};function vx(e,t){return!Array.isArray(e)&&t!==void 0?t6(e,t):uO(e)}const n6=()=>Cr()&&window.document.documentElement,qd=e=>{const{prefixCls:t,className:n,style:r,size:o,shape:l}=e,c=re({[`${t}-lg`]:o==="large",[`${t}-sm`]:o==="small"}),d=re({[`${t}-circle`]:l==="circle",[`${t}-square`]:l==="square",[`${t}-round`]:l==="round"}),m=u.useMemo(()=>typeof o=="number"?{width:o,height:o,lineHeight:`${o}px`}:{},[o]);return u.createElement("span",{className:re(t,c,d,n),style:Object.assign(Object.assign({},m),r)})},r6=new Pt("ant-skeleton-loading",{"0%":{backgroundPosition:"100% 50%"},"100%":{backgroundPosition:"0 50%"}}),Wd=e=>({height:e,lineHeight:he(e)}),Hl=e=>Object.assign({width:e},Wd(e)),a6=e=>({background:e.skeletonLoadingBackground,backgroundSize:"400% 100%",animationName:r6,animationDuration:e.skeletonLoadingMotionDuration,animationTimingFunction:"ease",animationIterationCount:"infinite"}),Wg=(e,t)=>Object.assign({width:t(e).mul(5).equal(),minWidth:t(e).mul(5).equal()},Wd(e)),o6=e=>{const{skeletonAvatarCls:t,gradientFromColor:n,controlHeight:r,controlHeightLG:o,controlHeightSM:l}=e;return{[t]:Object.assign({display:"inline-block",verticalAlign:"top",background:n},Hl(r)),[`${t}${t}-circle`]:{borderRadius:"50%"},[`${t}${t}-lg`]:Object.assign({},Hl(o)),[`${t}${t}-sm`]:Object.assign({},Hl(l))}},i6=e=>{const{controlHeight:t,borderRadiusSM:n,skeletonInputCls:r,controlHeightLG:o,controlHeightSM:l,gradientFromColor:c,calc:d}=e;return{[r]:Object.assign({display:"inline-block",verticalAlign:"top",background:c,borderRadius:n},Wg(t,d)),[`${r}-lg`]:Object.assign({},Wg(o,d)),[`${r}-sm`]:Object.assign({},Wg(l,d))}},gx=e=>Object.assign({width:e},Wd(e)),l6=e=>{const{skeletonImageCls:t,imageSizeBase:n,gradientFromColor:r,borderRadiusSM:o,calc:l}=e;return{[t]:Object.assign(Object.assign({display:"inline-flex",alignItems:"center",justifyContent:"center",verticalAlign:"middle",background:r,borderRadius:o},gx(l(n).mul(2).equal())),{[`${t}-path`]:{fill:"#bfbfbf"},[`${t}-svg`]:Object.assign(Object.assign({},gx(n)),{maxWidth:l(n).mul(4).equal(),maxHeight:l(n).mul(4).equal()}),[`${t}-svg${t}-svg-circle`]:{borderRadius:"50%"}}),[`${t}${t}-circle`]:{borderRadius:"50%"}}},kg=(e,t,n)=>{const{skeletonButtonCls:r}=e;return{[`${n}${r}-circle`]:{width:t,minWidth:t,borderRadius:"50%"},[`${n}${r}-round`]:{borderRadius:t}}},Gg=(e,t)=>Object.assign({width:t(e).mul(2).equal(),minWidth:t(e).mul(2).equal()},Wd(e)),s6=e=>{const{borderRadiusSM:t,skeletonButtonCls:n,controlHeight:r,controlHeightLG:o,controlHeightSM:l,gradientFromColor:c,calc:d}=e;return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({[n]:Object.assign({display:"inline-block",verticalAlign:"top",background:c,borderRadius:t,width:d(r).mul(2).equal(),minWidth:d(r).mul(2).equal()},Gg(r,d))},kg(e,r,n)),{[`${n}-lg`]:Object.assign({},Gg(o,d))}),kg(e,o,`${n}-lg`)),{[`${n}-sm`]:Object.assign({},Gg(l,d))}),kg(e,l,`${n}-sm`))},c6=e=>{const{componentCls:t,skeletonAvatarCls:n,skeletonTitleCls:r,skeletonParagraphCls:o,skeletonButtonCls:l,skeletonInputCls:c,skeletonImageCls:d,controlHeight:m,controlHeightLG:v,controlHeightSM:g,gradientFromColor:p,padding:y,marginSM:E,borderRadius:C,titleHeight:x,blockRadius:b,paragraphLiHeight:w,controlHeightXS:$,paragraphMarginTop:O}=e;return{[t]:{display:"table",width:"100%",[`${t}-header`]:{display:"table-cell",paddingInlineEnd:y,verticalAlign:"top",[n]:Object.assign({display:"inline-block",verticalAlign:"top",background:p},Hl(m)),[`${n}-circle`]:{borderRadius:"50%"},[`${n}-lg`]:Object.assign({},Hl(v)),[`${n}-sm`]:Object.assign({},Hl(g))},[`${t}-content`]:{display:"table-cell",width:"100%",verticalAlign:"top",[r]:{width:"100%",height:x,background:p,borderRadius:b,[`+ ${o}`]:{marginBlockStart:g}},[o]:{padding:0,"> li":{width:"100%",height:w,listStyle:"none",background:p,borderRadius:b,"+ li":{marginBlockStart:$}}},[`${o}> li:last-child:not(:first-child):not(:nth-child(2))`]:{width:"61%"}},[`&-round ${t}-content`]:{[`${r}, ${o} > li`]:{borderRadius:C}}},[`${t}-with-avatar ${t}-content`]:{[r]:{marginBlockStart:E,[`+ ${o}`]:{marginBlockStart:O}}},[`${t}${t}-element`]:Object.assign(Object.assign(Object.assign(Object.assign({display:"inline-block",width:"auto"},s6(e)),o6(e)),i6(e)),l6(e)),[`${t}${t}-block`]:{width:"100%",[l]:{width:"100%"},[c]:{width:"100%"}},[`${t}${t}-active`]:{[`
        ${r},
        ${o} > li,
        ${n},
        ${l},
        ${c},
        ${d}
      `]:Object.assign({},a6(e))}}},u6=e=>{const{colorFillContent:t,colorFill:n}=e,r=t,o=n;return{color:r,colorGradientEnd:o,gradientFromColor:r,gradientToColor:o,titleHeight:e.controlHeight/2,blockRadius:e.borderRadiusSM,paragraphMarginTop:e.marginLG+e.marginXXS,paragraphLiHeight:e.controlHeight/2}},ns=wn("Skeleton",e=>{const{componentCls:t,calc:n}=e,r=kt(e,{skeletonAvatarCls:`${t}-avatar`,skeletonTitleCls:`${t}-title`,skeletonParagraphCls:`${t}-paragraph`,skeletonButtonCls:`${t}-button`,skeletonInputCls:`${t}-input`,skeletonImageCls:`${t}-image`,imageSizeBase:n(e.controlHeight).mul(1.5).equal(),borderRadius:100,skeletonLoadingBackground:`linear-gradient(90deg, ${e.gradientFromColor} 25%, ${e.gradientToColor} 37%, ${e.gradientFromColor} 63%)`,skeletonLoadingMotionDuration:"1.4s"});return[c6(r)]},u6,{deprecatedTokens:[["color","gradientFromColor"],["colorGradientEnd","gradientToColor"]]}),f6=e=>{const{prefixCls:t,className:n,rootClassName:r,active:o,shape:l="circle",size:c="default"}=e,{getPrefixCls:d}=u.useContext(vt),m=d("skeleton",t),[v,g,p]=ns(m),y=Tn(e,["prefixCls","className"]),E=re(m,`${m}-element`,{[`${m}-active`]:o},n,r,g,p);return v(u.createElement("div",{className:E},u.createElement(qd,Object.assign({prefixCls:`${m}-avatar`,shape:l,size:c},y))))},d6=e=>{const{prefixCls:t,className:n,rootClassName:r,active:o,block:l=!1,size:c="default"}=e,{getPrefixCls:d}=u.useContext(vt),m=d("skeleton",t),[v,g,p]=ns(m),y=Tn(e,["prefixCls"]),E=re(m,`${m}-element`,{[`${m}-active`]:o,[`${m}-block`]:l},n,r,g,p);return v(u.createElement("div",{className:E},u.createElement(qd,Object.assign({prefixCls:`${m}-button`,size:c},y))))},m6="M365.714286 329.142857q0 45.714286-32.036571 77.677714t-77.677714 32.036571-77.677714-32.036571-32.036571-77.677714 32.036571-77.677714 77.677714-32.036571 77.677714 32.036571 32.036571 77.677714zM950.857143 548.571429l0 256-804.571429 0 0-109.714286 182.857143-182.857143 91.428571 91.428571 292.571429-292.571429zM1005.714286 146.285714l-914.285714 0q-7.460571 0-12.873143 5.412571t-5.412571 12.873143l0 694.857143q0 7.460571 5.412571 12.873143t12.873143 5.412571l914.285714 0q7.460571 0 12.873143-5.412571t5.412571-12.873143l0-694.857143q0-7.460571-5.412571-12.873143t-12.873143-5.412571zM1097.142857 164.571429l0 694.857143q0 37.741714-26.843429 64.585143t-64.585143 26.843429l-914.285714 0q-37.741714 0-64.585143-26.843429t-26.843429-64.585143l0-694.857143q0-37.741714 26.843429-64.585143t64.585143-26.843429l914.285714 0q37.741714 0 64.585143 26.843429t26.843429 64.585143z",v6=e=>{const{prefixCls:t,className:n,rootClassName:r,style:o,active:l}=e,{getPrefixCls:c}=u.useContext(vt),d=c("skeleton",t),[m,v,g]=ns(d),p=re(d,`${d}-element`,{[`${d}-active`]:l},n,r,v,g);return m(u.createElement("div",{className:p},u.createElement("div",{className:re(`${d}-image`,n),style:o},u.createElement("svg",{viewBox:"0 0 1098 1024",xmlns:"http://www.w3.org/2000/svg",className:`${d}-image-svg`},u.createElement("title",null,"Image placeholder"),u.createElement("path",{d:m6,className:`${d}-image-path`})))))},g6=e=>{const{prefixCls:t,className:n,rootClassName:r,active:o,block:l,size:c="default"}=e,{getPrefixCls:d}=u.useContext(vt),m=d("skeleton",t),[v,g,p]=ns(m),y=Tn(e,["prefixCls"]),E=re(m,`${m}-element`,{[`${m}-active`]:o,[`${m}-block`]:l},n,r,g,p);return v(u.createElement("div",{className:E},u.createElement(qd,Object.assign({prefixCls:`${m}-input`,size:c},y))))},h6=e=>{const{prefixCls:t,className:n,rootClassName:r,style:o,active:l,children:c}=e,{getPrefixCls:d}=u.useContext(vt),m=d("skeleton",t),[v,g,p]=ns(m),y=re(m,`${m}-element`,{[`${m}-active`]:l},g,n,r,p);return v(u.createElement("div",{className:y},u.createElement("div",{className:re(`${m}-image`,n),style:o},c)))},p6=(e,t)=>{const{width:n,rows:r=2}=t;if(Array.isArray(n))return n[e];if(r-1===e)return n},y6=e=>{const{prefixCls:t,className:n,style:r,rows:o=0}=e,l=Array.from({length:o}).map((c,d)=>u.createElement("li",{key:d,style:{width:p6(d,e)}}));return u.createElement("ul",{className:re(t,n),style:r},l)},b6=e=>{let{prefixCls:t,className:n,width:r,style:o}=e;return u.createElement("h3",{className:re(t,n),style:Object.assign({width:r},o)})};function Kg(e){return e&&typeof e=="object"?e:{}}function S6(e,t){return e&&!t?{size:"large",shape:"square"}:{size:"large",shape:"circle"}}function C6(e,t){return!e&&t?{width:"38%"}:e&&t?{width:"50%"}:{}}function x6(e,t){const n={};return(!e||!t)&&(n.width="61%"),!e&&t?n.rows=3:n.rows=2,n}const Ki=e=>{const{prefixCls:t,loading:n,className:r,rootClassName:o,style:l,children:c,avatar:d=!1,title:m=!0,paragraph:v=!0,active:g,round:p}=e,{getPrefixCls:y,direction:E,className:C,style:x}=zr("skeleton"),b=y("skeleton",t),[w,$,O]=ns(b);if(n||!("loading"in e)){const T=!!d,R=!!m,M=!!v;let N;if(T){const D=Object.assign(Object.assign({prefixCls:`${b}-avatar`},S6(R,M)),Kg(d));N=u.createElement("div",{className:`${b}-header`},u.createElement(qd,Object.assign({},D)))}let z;if(R||M){let D;if(R){const W=Object.assign(Object.assign({prefixCls:`${b}-title`},C6(T,M)),Kg(m));D=u.createElement(b6,Object.assign({},W))}let F;if(M){const W=Object.assign(Object.assign({prefixCls:`${b}-paragraph`},x6(T,R)),Kg(v));F=u.createElement(y6,Object.assign({},W))}z=u.createElement("div",{className:`${b}-content`},D,F)}const P=re(b,{[`${b}-with-avatar`]:T,[`${b}-active`]:g,[`${b}-rtl`]:E==="rtl",[`${b}-round`]:p},C,r,o,$,O);return w(u.createElement("div",{className:P,style:Object.assign(Object.assign({},x),l)},N,z))}return c??null};Ki.Button=d6;Ki.Avatar=f6;Ki.Input=g6;Ki.Image=v6;Ki.Node=h6;function hx(){}const E6=u.createContext({add:hx,remove:hx});function w6(e){const t=u.useContext(E6),n=u.useRef(null);return jn(o=>{if(o){const l=e?o.querySelector(e):o;t.add(l),n.current=l}else t.remove(n.current)})}const px=()=>{const{cancelButtonProps:e,cancelTextLocale:t,onCancel:n}=u.useContext(Yc);return oe.createElement($r,Object.assign({onClick:n},e),t)},yx=()=>{const{confirmLoading:e,okButtonProps:t,okType:n,okTextLocale:r,onOk:o}=u.useContext(Yc);return oe.createElement($r,Object.assign({},d0(n),{loading:e,onClick:o},t),r)};function fO(e,t){return oe.createElement("span",{className:`${e}-close-x`},t||oe.createElement(Yl,{className:`${e}-close-icon`}))}const dO=e=>{const{okText:t,okType:n="primary",cancelText:r,confirmLoading:o,onOk:l,onCancel:c,okButtonProps:d,cancelButtonProps:m,footer:v}=e,[g]=To("Modal",t$()),p=t||(g==null?void 0:g.okText),y=r||(g==null?void 0:g.cancelText),E={confirmLoading:o,okButtonProps:d,cancelButtonProps:m,okTextLocale:p,cancelTextLocale:y,okType:n,onOk:l,onCancel:c},C=oe.useMemo(()=>E,Te(Object.values(E)));let x;return typeof v=="function"||typeof v>"u"?(x=oe.createElement(oe.Fragment,null,oe.createElement(px,null),oe.createElement(yx,null)),typeof v=="function"&&(x=v(x,{OkBtn:yx,CancelBtn:px})),x=oe.createElement(G$,{value:C},x)):x=v,oe.createElement(a0,{disabled:!1},x)},$6=e=>{const{componentCls:t}=e;return{[t]:{display:"flex",flexFlow:"row wrap",minWidth:0,"&::before, &::after":{display:"flex"},"&-no-wrap":{flexWrap:"nowrap"},"&-start":{justifyContent:"flex-start"},"&-center":{justifyContent:"center"},"&-end":{justifyContent:"flex-end"},"&-space-between":{justifyContent:"space-between"},"&-space-around":{justifyContent:"space-around"},"&-space-evenly":{justifyContent:"space-evenly"},"&-top":{alignItems:"flex-start"},"&-middle":{alignItems:"center"},"&-bottom":{alignItems:"flex-end"}}}},O6=e=>{const{componentCls:t}=e;return{[t]:{position:"relative",maxWidth:"100%",minHeight:1}}},R6=(e,t)=>{const{prefixCls:n,componentCls:r,gridColumns:o}=e,l={};for(let c=o;c>=0;c--)c===0?(l[`${r}${t}-${c}`]={display:"none"},l[`${r}-push-${c}`]={insetInlineStart:"auto"},l[`${r}-pull-${c}`]={insetInlineEnd:"auto"},l[`${r}${t}-push-${c}`]={insetInlineStart:"auto"},l[`${r}${t}-pull-${c}`]={insetInlineEnd:"auto"},l[`${r}${t}-offset-${c}`]={marginInlineStart:0},l[`${r}${t}-order-${c}`]={order:0}):(l[`${r}${t}-${c}`]=[{"--ant-display":"block",display:"block"},{display:"var(--ant-display)",flex:`0 0 ${c/o*100}%`,maxWidth:`${c/o*100}%`}],l[`${r}${t}-push-${c}`]={insetInlineStart:`${c/o*100}%`},l[`${r}${t}-pull-${c}`]={insetInlineEnd:`${c/o*100}%`},l[`${r}${t}-offset-${c}`]={marginInlineStart:`${c/o*100}%`},l[`${r}${t}-order-${c}`]={order:c});return l[`${r}${t}-flex`]={flex:`var(--${n}${t}-flex)`},l},hp=(e,t)=>R6(e,t),T6=(e,t,n)=>({[`@media (min-width: ${he(t)})`]:Object.assign({},hp(e,n))}),M6=()=>({}),_6=()=>({}),N6=wn("Grid",$6,M6),mO=e=>({xs:e.screenXSMin,sm:e.screenSMMin,md:e.screenMDMin,lg:e.screenLGMin,xl:e.screenXLMin,xxl:e.screenXXLMin}),A6=wn("Grid",e=>{const t=kt(e,{gridColumns:24}),n=mO(t);return delete n.xs,[O6(t),hp(t,""),hp(t,"-xs"),Object.keys(n).map(r=>T6(t,n[r],`-${r}`)).reduce((r,o)=>Object.assign(Object.assign({},r),o),{})]},_6);function bx(e){return{position:e,inset:0}}const I6=e=>{const{componentCls:t,antCls:n}=e;return[{[`${t}-root`]:{[`${t}${n}-zoom-enter, ${t}${n}-zoom-appear`]:{transform:"none",opacity:0,animationDuration:e.motionDurationSlow,userSelect:"none"},[`${t}${n}-zoom-leave ${t}-content`]:{pointerEvents:"none"},[`${t}-mask`]:Object.assign(Object.assign({},bx("fixed")),{zIndex:e.zIndexPopupBase,height:"100%",backgroundColor:e.colorBgMask,pointerEvents:"none",[`${t}-hidden`]:{display:"none"}}),[`${t}-wrap`]:Object.assign(Object.assign({},bx("fixed")),{zIndex:e.zIndexPopupBase,overflow:"auto",outline:0,WebkitOverflowScrolling:"touch"})}},{[`${t}-root`]:BP(e)}]},j6=e=>{const{componentCls:t}=e;return[{[`${t}-root`]:{[`${t}-wrap-rtl`]:{direction:"rtl"},[`${t}-centered`]:{textAlign:"center","&::before":{display:"inline-block",width:0,height:"100%",verticalAlign:"middle",content:'""'},[t]:{top:0,display:"inline-block",paddingBottom:0,textAlign:"start",verticalAlign:"middle"}},[`@media (max-width: ${e.screenSMMax}px)`]:{[t]:{maxWidth:"calc(100vw - 16px)",margin:`${he(e.marginXS)} auto`},[`${t}-centered`]:{[t]:{flex:1}}}}},{[t]:Object.assign(Object.assign({},Tr(e)),{pointerEvents:"none",position:"relative",top:100,width:"auto",maxWidth:`calc(100vw - ${he(e.calc(e.margin).mul(2).equal())})`,margin:"0 auto",paddingBottom:e.paddingLG,[`${t}-title`]:{margin:0,color:e.titleColor,fontWeight:e.fontWeightStrong,fontSize:e.titleFontSize,lineHeight:e.titleLineHeight,wordWrap:"break-word"},[`${t}-content`]:{position:"relative",backgroundColor:e.contentBg,backgroundClip:"padding-box",border:0,borderRadius:e.borderRadiusLG,boxShadow:e.boxShadow,pointerEvents:"auto",padding:e.contentPadding},[`${t}-close`]:Object.assign({position:"absolute",top:e.calc(e.modalHeaderHeight).sub(e.modalCloseBtnSize).div(2).equal(),insetInlineEnd:e.calc(e.modalHeaderHeight).sub(e.modalCloseBtnSize).div(2).equal(),zIndex:e.calc(e.zIndexPopupBase).add(10).equal(),padding:0,color:e.modalCloseIconColor,fontWeight:e.fontWeightStrong,lineHeight:1,textDecoration:"none",background:"transparent",borderRadius:e.borderRadiusSM,width:e.modalCloseBtnSize,height:e.modalCloseBtnSize,border:0,outline:0,cursor:"pointer",transition:`color ${e.motionDurationMid}, background-color ${e.motionDurationMid}`,"&-x":{display:"flex",fontSize:e.fontSizeLG,fontStyle:"normal",lineHeight:he(e.modalCloseBtnSize),justifyContent:"center",textTransform:"none",textRendering:"auto"},"&:disabled":{pointerEvents:"none"},"&:hover":{color:e.modalCloseIconHoverColor,backgroundColor:e.colorBgTextHover,textDecoration:"none"},"&:active":{backgroundColor:e.colorBgTextActive}},Gc(e)),[`${t}-header`]:{color:e.colorText,background:e.headerBg,borderRadius:`${he(e.borderRadiusLG)} ${he(e.borderRadiusLG)} 0 0`,marginBottom:e.headerMarginBottom,padding:e.headerPadding,borderBottom:e.headerBorderBottom},[`${t}-body`]:{fontSize:e.fontSize,lineHeight:e.lineHeight,wordWrap:"break-word",padding:e.bodyPadding,[`${t}-body-skeleton`]:{width:"100%",height:"100%",display:"flex",justifyContent:"center",alignItems:"center",margin:`${he(e.margin)} auto`}},[`${t}-footer`]:{textAlign:"end",background:e.footerBg,marginTop:e.footerMarginTop,padding:e.footerPadding,borderTop:e.footerBorderTop,borderRadius:e.footerBorderRadius,[`> ${e.antCls}-btn + ${e.antCls}-btn`]:{marginInlineStart:e.marginXS}},[`${t}-open`]:{overflow:"hidden"}})},{[`${t}-pure-panel`]:{top:"auto",padding:0,display:"flex",flexDirection:"column",[`${t}-content,
          ${t}-body,
          ${t}-confirm-body-wrapper`]:{display:"flex",flexDirection:"column",flex:"auto"},[`${t}-confirm-body`]:{marginBottom:"auto"}}}]},P6=e=>{const{componentCls:t}=e;return{[`${t}-root`]:{[`${t}-wrap-rtl`]:{direction:"rtl",[`${t}-confirm-body`]:{direction:"rtl"}}}}},z6=e=>{const{componentCls:t}=e,n=mO(e);delete n.xs;const r=Object.keys(n).map(o=>({[`@media (min-width: ${he(n[o])})`]:{width:`var(--${t.replace(".","")}-${o}-width)`}}));return{[`${t}-root`]:{[t]:[{width:`var(--${t.replace(".","")}-xs-width)`}].concat(Te(r))}}},vO=e=>{const t=e.padding,n=e.fontSizeHeading5,r=e.lineHeightHeading5;return kt(e,{modalHeaderHeight:e.calc(e.calc(r).mul(n).equal()).add(e.calc(t).mul(2).equal()).equal(),modalFooterBorderColorSplit:e.colorSplit,modalFooterBorderStyle:e.lineType,modalFooterBorderWidth:e.lineWidth,modalCloseIconColor:e.colorIcon,modalCloseIconHoverColor:e.colorIconHover,modalCloseBtnSize:e.controlHeight,modalConfirmIconSize:e.fontHeight,modalTitleHeight:e.calc(e.titleFontSize).mul(e.titleLineHeight).equal()})},gO=e=>({footerBg:"transparent",headerBg:e.colorBgElevated,titleLineHeight:e.lineHeightHeading5,titleFontSize:e.fontSizeHeading5,contentBg:e.colorBgElevated,titleColor:e.colorTextHeading,contentPadding:e.wireframe?0:`${he(e.paddingMD)} ${he(e.paddingContentHorizontalLG)}`,headerPadding:e.wireframe?`${he(e.padding)} ${he(e.paddingLG)}`:0,headerBorderBottom:e.wireframe?`${he(e.lineWidth)} ${e.lineType} ${e.colorSplit}`:"none",headerMarginBottom:e.wireframe?0:e.marginXS,bodyPadding:e.wireframe?e.paddingLG:0,footerPadding:e.wireframe?`${he(e.paddingXS)} ${he(e.padding)}`:0,footerBorderTop:e.wireframe?`${he(e.lineWidth)} ${e.lineType} ${e.colorSplit}`:"none",footerBorderRadius:e.wireframe?`0 0 ${he(e.borderRadiusLG)} ${he(e.borderRadiusLG)}`:0,footerMarginTop:e.wireframe?0:e.marginSM,confirmBodyPadding:e.wireframe?`${he(e.padding*2)} ${he(e.padding*2)} ${he(e.paddingLG)}`:0,confirmIconMarginInlineEnd:e.wireframe?e.margin:e.marginSM,confirmBtnsMarginTop:e.wireframe?e.marginLG:e.marginSM}),hO=wn("Modal",e=>{const t=vO(e);return[j6(t),P6(t),I6(t),Dd(t,"zoom"),z6(t)]},gO,{unitless:{titleLineHeight:!0}});var D6=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};let pp;const B6=e=>{pp={x:e.pageX,y:e.pageY},setTimeout(()=>{pp=null},100)};n6()&&document.documentElement.addEventListener("click",B6,!0);const pO=e=>{const{prefixCls:t,className:n,rootClassName:r,open:o,wrapClassName:l,centered:c,getContainer:d,focusTriggerAfterClose:m=!0,style:v,visible:g,width:p=520,footer:y,classNames:E,styles:C,children:x,loading:b,confirmLoading:w,zIndex:$,mousePosition:O,onOk:T,onCancel:R}=e,M=D6(e,["prefixCls","className","rootClassName","open","wrapClassName","centered","getContainer","focusTriggerAfterClose","style","visible","width","footer","classNames","styles","children","loading","confirmLoading","zIndex","mousePosition","onOk","onCancel"]),{getPopupContainer:N,getPrefixCls:z,direction:P,modal:D}=u.useContext(vt),F=be=>{w||R==null||R(be)},W=be=>{T==null||T(be)},V=z("modal",t),q=z(),K=na(V),[_,j,B]=hO(V,K),k=re(l,{[`${V}-centered`]:c??(D==null?void 0:D.centered),[`${V}-wrap-rtl`]:P==="rtl"}),I=y!==null&&!b?u.createElement(dO,Object.assign({},e,{onOk:W,onCancel:F})):null,[H,G,X]=e6(fx(e),fx(D),{closable:!0,closeIcon:u.createElement(Yl,{className:`${V}-close-icon`}),closeIconRender:be=>fO(V,be)}),U=w6(`.${V}-content`),[J,te]=Nd("Modal",$),[ve,ne]=u.useMemo(()=>p&&typeof p=="object"?[void 0,p]:[p,void 0],[p]),le=u.useMemo(()=>{const be={};return ne&&Object.keys(ne).forEach(fe=>{const ie=ne[fe];ie!==void 0&&(be[`--${V}-${fe}-width`]=typeof ie=="number"?`${ie}px`:ie)}),be},[ne]);return _(u.createElement(Ac,{form:!0,space:!0},u.createElement(c0.Provider,{value:te},u.createElement(J$,Object.assign({width:ve},M,{zIndex:J,getContainer:d===void 0?N:d,prefixCls:V,rootClassName:re(j,r,B,K),footer:I,visible:o??g,mousePosition:O??pp,onClose:F,closable:H&&{disabled:X,closeIcon:G},closeIcon:G,focusTriggerAfterClose:m,transitionName:Fi(q,"zoom",e.transitionName),maskTransitionName:Fi(q,"fade",e.maskTransitionName),className:re(j,n,D==null?void 0:D.className),style:Object.assign(Object.assign(Object.assign({},D==null?void 0:D.style),v),le),classNames:Object.assign(Object.assign(Object.assign({},D==null?void 0:D.classNames),E),{wrapper:re(k,E==null?void 0:E.wrapper)}),styles:Object.assign(Object.assign({},D==null?void 0:D.styles),C),panelRef:U}),b?u.createElement(Ki,{active:!0,title:!1,paragraph:{rows:4},className:`${V}-body-skeleton`}):x))))},L6=e=>{const{componentCls:t,titleFontSize:n,titleLineHeight:r,modalConfirmIconSize:o,fontSize:l,lineHeight:c,modalTitleHeight:d,fontHeight:m,confirmBodyPadding:v}=e,g=`${t}-confirm`;return{[g]:{"&-rtl":{direction:"rtl"},[`${e.antCls}-modal-header`]:{display:"none"},[`${g}-body-wrapper`]:Object.assign({},Jf()),[`&${t} ${t}-body`]:{padding:v},[`${g}-body`]:{display:"flex",flexWrap:"nowrap",alignItems:"start",[`> ${e.iconCls}`]:{flex:"none",fontSize:o,marginInlineEnd:e.confirmIconMarginInlineEnd,marginTop:e.calc(e.calc(m).sub(o).equal()).div(2).equal()},[`&-has-title > ${e.iconCls}`]:{marginTop:e.calc(e.calc(d).sub(o).equal()).div(2).equal()}},[`${g}-paragraph`]:{display:"flex",flexDirection:"column",flex:"auto",rowGap:e.marginXS,maxWidth:`calc(100% - ${he(e.marginSM)})`},[`${e.iconCls} + ${g}-paragraph`]:{maxWidth:`calc(100% - ${he(e.calc(e.modalConfirmIconSize).add(e.marginSM).equal())})`},[`${g}-title`]:{color:e.colorTextHeading,fontWeight:e.fontWeightStrong,fontSize:n,lineHeight:r},[`${g}-content`]:{color:e.colorText,fontSize:l,lineHeight:c},[`${g}-btns`]:{textAlign:"end",marginTop:e.confirmBtnsMarginTop,[`${e.antCls}-btn + ${e.antCls}-btn`]:{marginBottom:0,marginInlineStart:e.marginXS}}},[`${g}-error ${g}-body > ${e.iconCls}`]:{color:e.colorError},[`${g}-warning ${g}-body > ${e.iconCls},
        ${g}-confirm ${g}-body > ${e.iconCls}`]:{color:e.colorWarning},[`${g}-info ${g}-body > ${e.iconCls}`]:{color:e.colorInfo},[`${g}-success ${g}-body > ${e.iconCls}`]:{color:e.colorSuccess}}},H6=_d(["Modal","confirm"],e=>{const t=vO(e);return[L6(t)]},gO,{order:-1e3});var F6=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function yO(e){const{prefixCls:t,icon:n,okText:r,cancelText:o,confirmPrefixCls:l,type:c,okCancel:d,footer:m,locale:v}=e,g=F6(e,["prefixCls","icon","okText","cancelText","confirmPrefixCls","type","okCancel","footer","locale"]);let p=n;if(!n&&n!==null)switch(c){case"info":p=u.createElement(Wp,null);break;case"success":p=u.createElement(wd,null);break;case"error":p=u.createElement(Xl,null);break;default:p=u.createElement(Ql,null)}const y=d??c==="confirm",E=e.autoFocusButton===null?!1:e.autoFocusButton||"ok",[C]=To("Modal"),x=v||C,b=r||(y?x==null?void 0:x.okText:x==null?void 0:x.justOkText),w=o||(x==null?void 0:x.cancelText),$=Object.assign({autoFocusButton:E,cancelTextLocale:w,okTextLocale:b,mergedOkCancel:y},g),O=u.useMemo(()=>$,Te(Object.values($))),T=u.createElement(u.Fragment,null,u.createElement(k1,null),u.createElement(G1,null)),R=e.title!==void 0&&e.title!==null,M=`${l}-body`;return u.createElement("div",{className:`${l}-body-wrapper`},u.createElement("div",{className:re(M,{[`${M}-has-title`]:R})},p,u.createElement("div",{className:`${l}-paragraph`},R&&u.createElement("span",{className:`${l}-title`},e.title),u.createElement("div",{className:`${l}-content`},e.content))),m===void 0||typeof m=="function"?u.createElement(G$,{value:O},u.createElement("div",{className:`${l}-btns`},typeof m=="function"?m(T,{OkBtn:G1,CancelBtn:k1}):T)):m,u.createElement(H6,{prefixCls:t}))}const V6=e=>{const{close:t,zIndex:n,maskStyle:r,direction:o,prefixCls:l,wrapClassName:c,rootPrefixCls:d,bodyStyle:m,closable:v=!1,onConfirm:g,styles:p}=e,y=`${l}-confirm`,E=e.width||416,C=e.style||{},x=e.mask===void 0?!0:e.mask,b=e.maskClosable===void 0?!1:e.maskClosable,w=re(y,`${y}-${e.type}`,{[`${y}-rtl`]:o==="rtl"},e.className),[,$]=Rr(),O=u.useMemo(()=>n!==void 0?n:$.zIndexPopupBase+u0,[n,$]);return u.createElement(pO,Object.assign({},e,{className:w,wrapClassName:re({[`${y}-centered`]:!!e.centered},c),onCancel:()=>{t==null||t({triggerCancel:!0}),g==null||g(!1)},title:"",footer:null,transitionName:Fi(d||"","zoom",e.transitionName),maskTransitionName:Fi(d||"","fade",e.maskTransitionName),mask:x,maskClosable:b,style:C,styles:Object.assign({body:m,mask:r},p),width:E,zIndex:O,closable:v}),u.createElement(yO,Object.assign({},e,{confirmPrefixCls:y})))},bO=e=>{const{rootPrefixCls:t,iconPrefixCls:n,direction:r,theme:o}=e;return u.createElement(Ya,{prefixCls:t,iconPrefixCls:n,direction:r,theme:o},u.createElement(V6,Object.assign({},e)))},Ii=[];let SO="";function CO(){return SO}const U6=e=>{var t,n;const{prefixCls:r,getContainer:o,direction:l}=e,c=t$(),d=u.useContext(vt),m=CO()||d.getPrefixCls(),v=r||`${m}-modal`;let g=o;return g===!1&&(g=void 0),oe.createElement(bO,Object.assign({},e,{rootPrefixCls:m,prefixCls:v,iconPrefixCls:d.iconPrefixCls,theme:d.theme,direction:l??d.direction,locale:(n=(t=d.locale)===null||t===void 0?void 0:t.Modal)!==null&&n!==void 0?n:c,getContainer:g}))};function Zc(e){const t=l0(),n=document.createDocumentFragment();let r=Object.assign(Object.assign({},e),{close:m,open:!0}),o,l;function c(){for(var g,p=arguments.length,y=new Array(p),E=0;E<p;E++)y[E]=arguments[E];if(y.some(b=>b==null?void 0:b.triggerCancel)){var x;(g=e.onCancel)===null||g===void 0||(x=g).call.apply(x,[e,()=>{}].concat(Te(y.slice(1))))}for(let b=0;b<Ii.length;b++)if(Ii[b]===m){Ii.splice(b,1);break}l()}function d(g){clearTimeout(o),o=setTimeout(()=>{const p=t.getPrefixCls(void 0,CO()),y=t.getIconPrefixCls(),E=t.getTheme(),C=oe.createElement(U6,Object.assign({},g));l=Id()(oe.createElement(Ya,{prefixCls:p,iconPrefixCls:y,theme:E},t.holderRender?t.holderRender(C):C),n)})}function m(){for(var g=arguments.length,p=new Array(g),y=0;y<g;y++)p[y]=arguments[y];r=Object.assign(Object.assign({},r),{open:!1,afterClose:()=>{typeof e.afterClose=="function"&&e.afterClose(),c.apply(this,p)}}),r.visible&&delete r.visible,d(r)}function v(g){typeof g=="function"?r=g(r):r=Object.assign(Object.assign({},r),g),d(r)}return d(r),Ii.push(m),{destroy:m,update:v}}function xO(e){return Object.assign(Object.assign({},e),{type:"warning"})}function EO(e){return Object.assign(Object.assign({},e),{type:"info"})}function wO(e){return Object.assign(Object.assign({},e),{type:"success"})}function $O(e){return Object.assign(Object.assign({},e),{type:"error"})}function OO(e){return Object.assign(Object.assign({},e),{type:"confirm"})}function q6(e){let{rootPrefixCls:t}=e;SO=t}var W6=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const k6=(e,t)=>{var n,{afterClose:r,config:o}=e,l=W6(e,["afterClose","config"]);const[c,d]=u.useState(!0),[m,v]=u.useState(o),{direction:g,getPrefixCls:p}=u.useContext(vt),y=p("modal"),E=p(),C=()=>{var $;r(),($=m.afterClose)===null||$===void 0||$.call(m)},x=function(){var $;d(!1);for(var O=arguments.length,T=new Array(O),R=0;R<O;R++)T[R]=arguments[R];if(T.some(z=>z==null?void 0:z.triggerCancel)){var N;($=m.onCancel)===null||$===void 0||(N=$).call.apply(N,[m,()=>{}].concat(Te(T.slice(1))))}};u.useImperativeHandle(t,()=>({destroy:x,update:$=>{v(O=>Object.assign(Object.assign({},O),$))}}));const b=(n=m.okCancel)!==null&&n!==void 0?n:m.type==="confirm",[w]=To("Modal",So.Modal);return u.createElement(bO,Object.assign({prefixCls:y,rootPrefixCls:E},m,{close:x,open:c,afterClose:C,okText:m.okText||(b?w==null?void 0:w.okText:w==null?void 0:w.justOkText),direction:m.direction||g,cancelText:m.cancelText||(w==null?void 0:w.cancelText)},l))},G6=u.forwardRef(k6);let Sx=0;const K6=u.memo(u.forwardRef((e,t)=>{const[n,r]=F4();return u.useImperativeHandle(t,()=>({patchElement:r}),[]),u.createElement(u.Fragment,null,n)}));function X6(){const e=u.useRef(null),[t,n]=u.useState([]);u.useEffect(()=>{t.length&&(Te(t).forEach(c=>{c()}),n([]))},[t]);const r=u.useCallback(l=>function(d){var m;Sx+=1;const v=u.createRef();let g;const p=new Promise(b=>{g=b});let y=!1,E;const C=u.createElement(G6,{key:`modal-${Sx}`,config:l(d),ref:v,afterClose:()=>{E==null||E()},isSilent:()=>y,onConfirm:b=>{g(b)}});return E=(m=e.current)===null||m===void 0?void 0:m.patchElement(C),E&&Ii.push(E),{destroy:()=>{function b(){var w;(w=v.current)===null||w===void 0||w.destroy()}v.current?b():n(w=>[].concat(Te(w),[b]))},update:b=>{function w(){var $;($=v.current)===null||$===void 0||$.update(b)}v.current?w():n($=>[].concat(Te($),[w]))},then:b=>(y=!0,p.then(b))}},[]);return[u.useMemo(()=>({info:r(EO),success:r(wO),error:r($O),warning:r(xO),confirm:r(OO)}),[]),u.createElement(K6,{key:"modal-holder",ref:e})]}const Y6=e=>{const{componentCls:t,notificationMarginEdge:n,animationMaxHeight:r}=e,o=`${t}-notice`,l=new Pt("antNotificationFadeIn",{"0%":{transform:"translate3d(100%, 0, 0)",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",opacity:1}}),c=new Pt("antNotificationTopFadeIn",{"0%":{top:-r,opacity:0},"100%":{top:0,opacity:1}}),d=new Pt("antNotificationBottomFadeIn",{"0%":{bottom:e.calc(r).mul(-1).equal(),opacity:0},"100%":{bottom:0,opacity:1}}),m=new Pt("antNotificationLeftFadeIn",{"0%":{transform:"translate3d(-100%, 0, 0)",opacity:0},"100%":{transform:"translate3d(0, 0, 0)",opacity:1}});return{[t]:{[`&${t}-top, &${t}-bottom`]:{marginInline:0,[o]:{marginInline:"auto auto"}},[`&${t}-top`]:{[`${t}-fade-enter${t}-fade-enter-active, ${t}-fade-appear${t}-fade-appear-active`]:{animationName:c}},[`&${t}-bottom`]:{[`${t}-fade-enter${t}-fade-enter-active, ${t}-fade-appear${t}-fade-appear-active`]:{animationName:d}},[`&${t}-topRight, &${t}-bottomRight`]:{[`${t}-fade-enter${t}-fade-enter-active, ${t}-fade-appear${t}-fade-appear-active`]:{animationName:l}},[`&${t}-topLeft, &${t}-bottomLeft`]:{marginRight:{value:0,_skip_check_:!0},marginLeft:{value:n,_skip_check_:!0},[o]:{marginInlineEnd:"auto",marginInlineStart:0},[`${t}-fade-enter${t}-fade-enter-active, ${t}-fade-appear${t}-fade-appear-active`]:{animationName:m}}}}},Q6=["top","topLeft","topRight","bottom","bottomLeft","bottomRight"],Z6={topLeft:"left",topRight:"right",bottomLeft:"left",bottomRight:"right",top:"left",bottom:"left"},J6=(e,t)=>{const{componentCls:n}=e;return{[`${n}-${t}`]:{[`&${n}-stack > ${n}-notice-wrapper`]:{[t.startsWith("top")?"top":"bottom"]:0,[Z6[t]]:{value:0,_skip_check_:!0}}}}},eD=e=>{const t={};for(let n=1;n<e.notificationStackLayer;n++)t[`&:nth-last-child(${n+1})`]={overflow:"hidden",[`& > ${e.componentCls}-notice`]:{opacity:0,transition:`opacity ${e.motionDurationMid}`}};return Object.assign({[`&:not(:nth-last-child(-n+${e.notificationStackLayer}))`]:{opacity:0,overflow:"hidden",color:"transparent",pointerEvents:"none"}},t)},tD=e=>{const t={};for(let n=1;n<e.notificationStackLayer;n++)t[`&:nth-last-child(${n+1})`]={background:e.colorBgBlur,backdropFilter:"blur(10px)","-webkit-backdrop-filter":"blur(10px)"};return Object.assign({},t)},nD=e=>{const{componentCls:t}=e;return Object.assign({[`${t}-stack`]:{[`& > ${t}-notice-wrapper`]:Object.assign({transition:`transform ${e.motionDurationSlow}, backdrop-filter 0s`,willChange:"transform, opacity",position:"absolute"},eD(e))},[`${t}-stack:not(${t}-stack-expanded)`]:{[`& > ${t}-notice-wrapper`]:Object.assign({},tD(e))},[`${t}-stack${t}-stack-expanded`]:{[`& > ${t}-notice-wrapper`]:{"&:not(:nth-last-child(-n + 1))":{opacity:1,overflow:"unset",color:"inherit",pointerEvents:"auto",[`& > ${e.componentCls}-notice`]:{opacity:1}},"&:after":{content:'""',position:"absolute",height:e.margin,width:"100%",insetInline:0,bottom:e.calc(e.margin).mul(-1).equal(),background:"transparent",pointerEvents:"auto"}}}},Q6.map(n=>J6(e,n)).reduce((n,r)=>Object.assign(Object.assign({},n),r),{}))},RO=e=>{const{iconCls:t,componentCls:n,boxShadow:r,fontSizeLG:o,notificationMarginBottom:l,borderRadiusLG:c,colorSuccess:d,colorInfo:m,colorWarning:v,colorError:g,colorTextHeading:p,notificationBg:y,notificationPadding:E,notificationMarginEdge:C,notificationProgressBg:x,notificationProgressHeight:b,fontSize:w,lineHeight:$,width:O,notificationIconSize:T,colorText:R}=e,M=`${n}-notice`;return{position:"relative",marginBottom:l,marginInlineStart:"auto",background:y,borderRadius:c,boxShadow:r,[M]:{padding:E,width:O,maxWidth:`calc(100vw - ${he(e.calc(C).mul(2).equal())})`,overflow:"hidden",lineHeight:$,wordWrap:"break-word"},[`${M}-message`]:{marginBottom:e.marginXS,color:p,fontSize:o,lineHeight:e.lineHeightLG},[`${M}-description`]:{fontSize:w,color:R},[`${M}-closable ${M}-message`]:{paddingInlineEnd:e.paddingLG},[`${M}-with-icon ${M}-message`]:{marginBottom:e.marginXS,marginInlineStart:e.calc(e.marginSM).add(T).equal(),fontSize:o},[`${M}-with-icon ${M}-description`]:{marginInlineStart:e.calc(e.marginSM).add(T).equal(),fontSize:w},[`${M}-icon`]:{position:"absolute",fontSize:T,lineHeight:1,[`&-success${t}`]:{color:d},[`&-info${t}`]:{color:m},[`&-warning${t}`]:{color:v},[`&-error${t}`]:{color:g}},[`${M}-close`]:Object.assign({position:"absolute",top:e.notificationPaddingVertical,insetInlineEnd:e.notificationPaddingHorizontal,color:e.colorIcon,outline:"none",width:e.notificationCloseButtonSize,height:e.notificationCloseButtonSize,borderRadius:e.borderRadiusSM,transition:`background-color ${e.motionDurationMid}, color ${e.motionDurationMid}`,display:"flex",alignItems:"center",justifyContent:"center",background:"none",border:"none","&:hover":{color:e.colorIconHover,backgroundColor:e.colorBgTextHover},"&:active":{backgroundColor:e.colorBgTextActive}},Gc(e)),[`${M}-progress`]:{position:"absolute",display:"block",appearance:"none",inlineSize:`calc(100% - ${he(c)} * 2)`,left:{_skip_check_:!0,value:c},right:{_skip_check_:!0,value:c},bottom:0,blockSize:b,border:0,"&, &::-webkit-progress-bar":{borderRadius:c,backgroundColor:"rgba(0, 0, 0, 0.04)"},"&::-moz-progress-bar":{background:x},"&::-webkit-progress-value":{borderRadius:c,background:x}},[`${M}-actions`]:{float:"right",marginTop:e.marginSM}}},rD=e=>{const{componentCls:t,notificationMarginBottom:n,notificationMarginEdge:r,motionDurationMid:o,motionEaseInOut:l}=e,c=`${t}-notice`,d=new Pt("antNotificationFadeOut",{"0%":{maxHeight:e.animationMaxHeight,marginBottom:n},"100%":{maxHeight:0,marginBottom:0,paddingTop:0,paddingBottom:0,opacity:0}});return[{[t]:Object.assign(Object.assign({},Tr(e)),{position:"fixed",zIndex:e.zIndexPopup,marginRight:{value:r,_skip_check_:!0},[`${t}-hook-holder`]:{position:"relative"},[`${t}-fade-appear-prepare`]:{opacity:"0 !important"},[`${t}-fade-enter, ${t}-fade-appear`]:{animationDuration:e.motionDurationMid,animationTimingFunction:l,animationFillMode:"both",opacity:0,animationPlayState:"paused"},[`${t}-fade-leave`]:{animationTimingFunction:l,animationFillMode:"both",animationDuration:o,animationPlayState:"paused"},[`${t}-fade-enter${t}-fade-enter-active, ${t}-fade-appear${t}-fade-appear-active`]:{animationPlayState:"running"},[`${t}-fade-leave${t}-fade-leave-active`]:{animationName:d,animationPlayState:"running"},"&-rtl":{direction:"rtl",[`${c}-actions`]:{float:"left"}}})},{[t]:{[`${c}-wrapper`]:Object.assign({},RO(e))}}]},TO=e=>({zIndexPopup:e.zIndexPopupBase+u0+50,width:384}),MO=e=>{const t=e.paddingMD,n=e.paddingLG;return kt(e,{notificationBg:e.colorBgElevated,notificationPaddingVertical:t,notificationPaddingHorizontal:n,notificationIconSize:e.calc(e.fontSizeLG).mul(e.lineHeightLG).equal(),notificationCloseButtonSize:e.calc(e.controlHeightLG).mul(.55).equal(),notificationMarginBottom:e.margin,notificationPadding:`${he(e.paddingMD)} ${he(e.paddingContentHorizontalLG)}`,notificationMarginEdge:e.marginLG,animationMaxHeight:150,notificationStackLayer:3,notificationProgressHeight:2,notificationProgressBg:`linear-gradient(90deg, ${e.colorPrimaryBorderHover}, ${e.colorPrimary})`})},_O=wn("Notification",e=>{const t=MO(e);return[rD(t),Y6(t),nD(t)]},TO),aD=_d(["Notification","PurePanel"],e=>{const t=`${e.componentCls}-notice`,n=MO(e);return{[`${t}-pure-panel`]:Object.assign(Object.assign({},RO(n)),{width:n.width,maxWidth:`calc(100vw - ${he(e.calc(n.notificationMarginEdge).mul(2).equal())})`,margin:0})}},TO);var oD=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function $0(e,t){return t===null||t===!1?null:t||u.createElement(Yl,{className:`${e}-close-icon`})}const iD={success:wd,info:Wp,error:Xl,warning:Ql},NO=e=>{const{prefixCls:t,icon:n,type:r,message:o,description:l,actions:c,role:d="alert"}=e;let m=null;return n?m=u.createElement("span",{className:`${t}-icon`},n):r&&(m=u.createElement(iD[r]||null,{className:re(`${t}-icon`,`${t}-icon-${r}`)})),u.createElement("div",{className:re({[`${t}-with-icon`]:m}),role:d},m,u.createElement("div",{className:`${t}-message`},o),u.createElement("div",{className:`${t}-description`},l),c&&u.createElement("div",{className:`${t}-actions`},c))},lD=e=>{const{prefixCls:t,className:n,icon:r,type:o,message:l,description:c,btn:d,actions:m,closable:v=!0,closeIcon:g,className:p}=e,y=oD(e,["prefixCls","className","icon","type","message","description","btn","actions","closable","closeIcon","className"]),{getPrefixCls:E}=u.useContext(vt),C=m??d,x=t||E("notification"),b=`${x}-notice`,w=na(x),[$,O,T]=_O(x,w);return $(u.createElement("div",{className:re(`${b}-pure-panel`,O,n,T,w)},u.createElement(aD,{prefixCls:x}),u.createElement(s0,Object.assign({},y,{prefixCls:x,eventKey:"pure",duration:null,closable:v,className:re({notificationClassName:p}),closeIcon:$0(x,g),content:u.createElement(NO,{prefixCls:b,icon:r,type:o,message:l,description:c,actions:C})}))))};function sD(e,t,n){let r;switch(e){case"top":r={left:"50%",transform:"translateX(-50%)",right:"auto",top:t,bottom:"auto"};break;case"topLeft":r={left:0,top:t,bottom:"auto"};break;case"topRight":r={right:0,top:t,bottom:"auto"};break;case"bottom":r={left:"50%",transform:"translateX(-50%)",right:"auto",top:"auto",bottom:n};break;case"bottomLeft":r={left:0,top:"auto",bottom:n};break;default:r={right:0,top:"auto",bottom:n};break}return r}function cD(e){return{motionName:`${e}-fade`}}function uD(e,t,n){return typeof e<"u"?e:typeof(t==null?void 0:t.closeIcon)<"u"?t.closeIcon:n==null?void 0:n.closeIcon}var fD=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const Cx=24,dD=4.5,mD="topRight",vD=e=>{let{children:t,prefixCls:n}=e;const r=na(n),[o,l,c]=_O(n,r);return o(oe.createElement(M$,{classNames:{list:re(l,c,r)}},t))},gD=(e,t)=>{let{prefixCls:n,key:r}=t;return oe.createElement(vD,{prefixCls:n,key:r},e)},hD=oe.forwardRef((e,t)=>{const{top:n,bottom:r,prefixCls:o,getContainer:l,maxCount:c,rtl:d,onAllRemoved:m,stack:v,duration:g,pauseOnHover:p=!0,showProgress:y}=e,{getPrefixCls:E,getPopupContainer:C,notification:x,direction:b}=u.useContext(vt),[,w]=Rr(),$=o||E("notification"),O=z=>sD(z,n??Cx,r??Cx),T=()=>re({[`${$}-rtl`]:d??b==="rtl"}),R=()=>cD($),[M,N]=_$({prefixCls:$,style:O,className:T,motion:R,closable:!0,closeIcon:$0($),duration:g??dD,getContainer:()=>(l==null?void 0:l())||(C==null?void 0:C())||document.body,maxCount:c,pauseOnHover:p,showProgress:y,onAllRemoved:m,renderNotifications:gD,stack:v===!1?!1:{threshold:typeof v=="object"?v==null?void 0:v.threshold:void 0,offset:8,gap:w.margin}});return oe.useImperativeHandle(t,()=>Object.assign(Object.assign({},M),{prefixCls:$,notification:x})),N});function AO(e){const t=oe.useRef(null);return kc(),[oe.useMemo(()=>{const r=d=>{var m;if(!t.current)return;const{open:v,prefixCls:g,notification:p}=t.current,y=`${g}-notice`,{message:E,description:C,icon:x,type:b,btn:w,actions:$,className:O,style:T,role:R="alert",closeIcon:M,closable:N}=d,z=fD(d,["message","description","icon","type","btn","actions","className","style","role","closeIcon","closable"]),P=$??w,D=$0(y,uD(M,e,p));return v(Object.assign(Object.assign({placement:(m=e==null?void 0:e.placement)!==null&&m!==void 0?m:mD},z),{content:oe.createElement(NO,{prefixCls:y,icon:x,type:b,message:E,description:C,actions:P,role:R}),className:re(b&&`${y}-${b}`,O,p==null?void 0:p.className),style:Object.assign(Object.assign({},p==null?void 0:p.style),T),closeIcon:D,closable:N??!!D}))},l={open:r,destroy:d=>{var m,v;d!==void 0?(m=t.current)===null||m===void 0||m.close(d):(v=t.current)===null||v===void 0||v.destroy()}};return["success","info","warning","error"].forEach(d=>{l[d]=m=>r(Object.assign(Object.assign({},m),{type:d}))}),l},[]),oe.createElement(hD,Object.assign({key:"notification-holder"},e,{ref:t}))]}function pD(e){return AO(e)}const IO=oe.createContext({});function jO(e){return t=>u.createElement(Ya,{theme:{token:{motion:!1,zIndexPopupBase:0}}},u.createElement(e,Object.assign({},t)))}const yD=(e,t,n,r,o)=>jO(c=>{const{prefixCls:d,style:m}=c,v=u.useRef(null),[g,p]=u.useState(0),[y,E]=u.useState(0),[C,x]=gr(!1,{value:c.open}),{getPrefixCls:b}=u.useContext(vt),w=b(r||"select",d);u.useEffect(()=>{if(x(!0),typeof ResizeObserver<"u"){const T=new ResizeObserver(M=>{const N=M[0].target;p(N.offsetHeight+8),E(N.offsetWidth)}),R=setInterval(()=>{var M;const N=o?`.${o(w)}`:`.${w}-dropdown`,z=(M=v.current)===null||M===void 0?void 0:M.querySelector(N);z&&(clearInterval(R),T.observe(z))},10);return()=>{clearInterval(R),T.disconnect()}}},[]);let $=Object.assign(Object.assign({},c),{style:Object.assign(Object.assign({},m),{margin:0}),open:C,visible:C,getPopupContainer:()=>v.current});t&&Object.assign($,{[t]:{overflow:{adjustX:!1,adjustY:!1}}});const O={paddingBottom:g,position:"relative",minWidth:y};return u.createElement("div",{ref:v,style:O},u.createElement(e,Object.assign({},$)))}),PO=function(){if(typeof navigator>"u"||typeof window>"u")return!1;var e=navigator.userAgent||navigator.vendor||window.opera;return/(android|bb\d+|meego).+mobile|avantgo|bada\/|blackberry|blazer|compal|elaine|fennec|hiptop|iemobile|ip(hone|od)|iris|kindle|lge |maemo|midp|mmp|mobile.+firefox|netfront|opera m(ob|in)i|palm( os)?|phone|p(ixi|re)\/|plucker|pocket|psp|series(4|6)0|symbian|treo|up\.(browser|link)|vodafone|wap|windows ce|xda|xiino|android|ipad|playbook|silk/i.test(e)||/1207|6310|6590|3gso|4thp|50[1-6]i|770s|802s|a wa|abac|ac(er|oo|s-)|ai(ko|rn)|al(av|ca|co)|amoi|an(ex|ny|yw)|aptu|ar(ch|go)|as(te|us)|attw|au(di|-m|r |s )|avan|be(ck|ll|nq)|bi(lb|rd)|bl(ac|az)|br(e|v)w|bumb|bw-(n|u)|c55\/|capi|ccwa|cdm-|cell|chtm|cldc|cmd-|co(mp|nd)|craw|da(it|ll|ng)|dbte|dc-s|devi|dica|dmob|do(c|p)o|ds(12|-d)|el(49|ai)|em(l2|ul)|er(ic|k0)|esl8|ez([4-7]0|os|wa|ze)|fetc|fly(-|_)|g1 u|g560|gene|gf-5|g-mo|go(\.w|od)|gr(ad|un)|haie|hcit|hd-(m|p|t)|hei-|hi(pt|ta)|hp( i|ip)|hs-c|ht(c(-| |_|a|g|p|s|t)|tp)|hu(aw|tc)|i-(20|go|ma)|i230|iac( |-|\/)|ibro|idea|ig01|ikom|im1k|inno|ipaq|iris|ja(t|v)a|jbro|jemu|jigs|kddi|keji|kgt( |\/)|klon|kpt |kwc-|kyo(c|k)|le(no|xi)|lg( g|\/(k|l|u)|50|54|-[a-w])|libw|lynx|m1-w|m3ga|m50\/|ma(te|ui|xo)|mc(01|21|ca)|m-cr|me(rc|ri)|mi(o8|oa|ts)|mmef|mo(01|02|bi|de|do|t(-| |o|v)|zz)|mt(50|p1|v )|mwbp|mywa|n10[0-2]|n20[2-3]|n30(0|2)|n50(0|2|5)|n7(0(0|1)|10)|ne((c|m)-|on|tf|wf|wg|wt)|nok(6|i)|nzph|o2im|op(ti|wv)|oran|owg1|p800|pan(a|d|t)|pdxg|pg(13|-([1-8]|c))|phil|pire|pl(ay|uc)|pn-2|po(ck|rt|se)|prox|psio|pt-g|qa-a|qc(07|12|21|32|60|-[2-7]|i-)|qtek|r380|r600|raks|rim9|ro(ve|zo)|s55\/|sa(ge|ma|mm|ms|ny|va)|sc(01|h-|oo|p-)|sdk\/|se(c(-|0|1)|47|mc|nd|ri)|sgh-|shar|sie(-|m)|sk-0|sl(45|id)|sm(al|ar|b3|it|t5)|so(ft|ny)|sp(01|h-|v-|v )|sy(01|mb)|t2(18|50)|t6(00|10|18)|ta(gt|lk)|tcl-|tdg-|tel(i|m)|tim-|t-mo|to(pl|sh)|ts(70|m-|m3|m5)|tx-9|up(\.b|g1|si)|utst|v400|v750|veri|vi(rg|te)|vk(40|5[0-3]|-v)|vm40|voda|vulc|vx(52|53|60|61|70|80|81|83|85|98)|w3c(-| )|webc|whit|wi(g |nc|nw)|wmlb|wonu|x700|yas-|your|zeto|zte-/i.test(e==null?void 0:e.substr(0,4))};var kd=function(t){var n=t.className,r=t.customizeIcon,o=t.customizeIconProps,l=t.children,c=t.onMouseDown,d=t.onClick,m=typeof r=="function"?r(o):r;return u.createElement("span",{className:n,onMouseDown:function(g){g.preventDefault(),c==null||c(g)},style:{userSelect:"none",WebkitUserSelect:"none"},unselectable:"on",onClick:d,"aria-hidden":!0},m!==void 0?m:u.createElement("span",{className:re(n.split(/\s+/).map(function(v){return"".concat(v,"-icon")}))},l))},bD=function(t,n,r,o,l){var c=arguments.length>5&&arguments[5]!==void 0?arguments[5]:!1,d=arguments.length>6?arguments[6]:void 0,m=arguments.length>7?arguments[7]:void 0,v=oe.useMemo(function(){if(lt(o)==="object")return o.clearIcon;if(l)return l},[o,l]),g=oe.useMemo(function(){return!!(!c&&o&&(r.length||d)&&!(m==="combobox"&&d===""))},[o,c,r.length,d,m]);return{allowClear:g,clearIcon:oe.createElement(kd,{className:"".concat(t,"-clear"),onMouseDown:n,customizeIcon:v},"×")}},zO=u.createContext(null);function SD(){return u.useContext(zO)}function CD(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:10,t=u.useState(!1),n=de(t,2),r=n[0],o=n[1],l=u.useRef(null),c=function(){window.clearTimeout(l.current)};u.useEffect(function(){return c},[]);var d=function(v,g){c(),l.current=window.setTimeout(function(){o(v),g&&g()},e)};return[r,d,c]}function DO(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:250,t=u.useRef(null),n=u.useRef(null);u.useEffect(function(){return function(){window.clearTimeout(n.current)}},[]);function r(o){(o||t.current===null)&&(t.current=o),window.clearTimeout(n.current),n.current=window.setTimeout(function(){t.current=null},e)}return[function(){return t.current},r]}function xD(e,t,n,r){var o=u.useRef(null);o.current={open:t,triggerOpen:n,customizedTrigger:r},u.useEffect(function(){function l(c){var d;if(!((d=o.current)!==null&&d!==void 0&&d.customizedTrigger)){var m=c.target;m.shadowRoot&&c.composed&&(m=c.composedPath()[0]||m),o.current.open&&e().filter(function(v){return v}).every(function(v){return!v.contains(m)&&v!==m})&&o.current.triggerOpen(!1)}}return window.addEventListener("mousedown",l),function(){return window.removeEventListener("mousedown",l)}},[])}function ED(e){return e&&![ct.ESC,ct.SHIFT,ct.BACKSPACE,ct.TAB,ct.WIN_KEY,ct.ALT,ct.META,ct.WIN_KEY_RIGHT,ct.CTRL,ct.SEMICOLON,ct.EQUALS,ct.CAPS_LOCK,ct.CONTEXT_MENU,ct.F1,ct.F2,ct.F3,ct.F4,ct.F5,ct.F6,ct.F7,ct.F8,ct.F9,ct.F10,ct.F11,ct.F12].includes(e)}var wD=["prefixCls","invalidate","item","renderItem","responsive","responsiveDisabled","registerSize","itemKey","className","style","children","display","order","component"],_l=void 0;function $D(e,t){var n=e.prefixCls,r=e.invalidate,o=e.item,l=e.renderItem,c=e.responsive,d=e.responsiveDisabled,m=e.registerSize,v=e.itemKey,g=e.className,p=e.style,y=e.children,E=e.display,C=e.order,x=e.component,b=x===void 0?"div":x,w=wt(e,wD),$=c&&!E;function O(z){m(v,z)}u.useEffect(function(){return function(){O(null)}},[]);var T=l&&o!==_l?l(o,{index:C}):y,R;r||(R={opacity:$?0:1,height:$?0:_l,overflowY:$?"hidden":_l,order:c?C:_l,pointerEvents:$?"none":_l,position:$?"absolute":_l});var M={};$&&(M["aria-hidden"]=!0);var N=u.createElement(b,Ie({className:re(!r&&n,g),style:Z(Z({},R),p)},M,w,{ref:t}),T);return c&&(N=u.createElement(Ro,{onResize:function(P){var D=P.offsetWidth;O(D)},disabled:d},N)),N}var bc=u.forwardRef($D);bc.displayName="Item";function OD(e){if(typeof MessageChannel>"u")gn(e);else{var t=new MessageChannel;t.port1.onmessage=function(){return e()},t.port2.postMessage(void 0)}}function RD(){var e=u.useRef(null),t=function(r){e.current||(e.current=[],OD(function(){ri.unstable_batchedUpdates(function(){e.current.forEach(function(o){o()}),e.current=null})})),e.current.push(r)};return t}function oc(e,t){var n=u.useState(t),r=de(n,2),o=r[0],l=r[1],c=jn(function(d){e(function(){l(d)})});return[o,c]}var ad=oe.createContext(null),TD=["component"],MD=["className"],_D=["className"],ND=function(t,n){var r=u.useContext(ad);if(!r){var o=t.component,l=o===void 0?"div":o,c=wt(t,TD);return u.createElement(l,Ie({},c,{ref:n}))}var d=r.className,m=wt(r,MD),v=t.className,g=wt(t,_D);return u.createElement(ad.Provider,{value:null},u.createElement(bc,Ie({ref:n,className:re(d,v)},m,g)))},BO=u.forwardRef(ND);BO.displayName="RawItem";var AD=["prefixCls","data","renderItem","renderRawItem","itemKey","itemWidth","ssr","style","className","maxCount","renderRest","renderRawRest","suffix","component","itemComponent","onVisibleChange"],LO="responsive",HO="invalidate";function ID(e){return"+ ".concat(e.length," ...")}function jD(e,t){var n=e.prefixCls,r=n===void 0?"rc-overflow":n,o=e.data,l=o===void 0?[]:o,c=e.renderItem,d=e.renderRawItem,m=e.itemKey,v=e.itemWidth,g=v===void 0?10:v,p=e.ssr,y=e.style,E=e.className,C=e.maxCount,x=e.renderRest,b=e.renderRawRest,w=e.suffix,$=e.component,O=$===void 0?"div":$,T=e.itemComponent,R=e.onVisibleChange,M=wt(e,AD),N=p==="full",z=RD(),P=oc(z,null),D=de(P,2),F=D[0],W=D[1],V=F||0,q=oc(z,new Map),K=de(q,2),_=K[0],j=K[1],B=oc(z,0),k=de(B,2),I=k[0],H=k[1],G=oc(z,0),X=de(G,2),U=X[0],J=X[1],te=oc(z,0),ve=de(te,2),ne=ve[0],le=ve[1],be=u.useState(null),fe=de(be,2),ie=fe[0],pe=fe[1],Re=u.useState(null),se=de(Re,2),ye=se[0],$e=se[1],we=u.useMemo(function(){return ye===null&&N?Number.MAX_SAFE_INTEGER:ye||0},[ye,F]),Ne=u.useState(!1),Ze=de(Ne,2),He=Ze[0],Xe=Ze[1],xe="".concat(r,"-item"),Me=Math.max(I,U),Ve=C===LO,Ke=l.length&&Ve,We=C===HO,at=Ke||typeof C=="number"&&l.length>C,Ue=u.useMemo(function(){var nt=l;return Ke?F===null&&N?nt=l:nt=l.slice(0,Math.min(l.length,V/g)):typeof C=="number"&&(nt=l.slice(0,C)),nt},[l,g,F,C,Ke]),Ye=u.useMemo(function(){return Ke?l.slice(we+1):l.slice(Ue.length)},[l,Ue,Ke,we]),st=u.useCallback(function(nt,ke){var De;return typeof m=="function"?m(nt):(De=m&&(nt==null?void 0:nt[m]))!==null&&De!==void 0?De:ke},[m]),ht=u.useCallback(c||function(nt){return nt},[c]);function Qe(nt,ke,De){ye===nt&&(ke===void 0||ke===ie)||($e(nt),De||(Xe(nt<l.length-1),R==null||R(nt)),ke!==void 0&&pe(ke))}function Pe(nt,ke){W(ke.clientWidth)}function Fe(nt,ke){j(function(De){var pt=new Map(De);return ke===null?pt.delete(nt):pt.set(nt,ke),pt})}function _e(nt,ke){J(ke),H(U)}function qe(nt,ke){le(ke)}function Ct(nt){return _.get(st(Ue[nt],nt))}Wt(function(){if(V&&typeof Me=="number"&&Ue){var nt=ne,ke=Ue.length,De=ke-1;if(!ke){Qe(0,null);return}for(var pt=0;pt<ke;pt+=1){var St=Ct(pt);if(N&&(St=St||0),St===void 0){Qe(pt-1,void 0,!0);break}if(nt+=St,De===0&&nt<=V||pt===De-1&&nt+Ct(De)<=V){Qe(De,null);break}else if(nt+Me>V){Qe(pt-1,nt-St-ne+U);break}}w&&Ct(0)+ne>V&&pe(null)}},[V,_,U,ne,st,Ue]);var Nt=He&&!!Ye.length,_n={};ie!==null&&Ke&&(_n={position:"absolute",left:ie,top:0});var bn={prefixCls:xe,responsive:Ke,component:T,invalidate:We},$n=d?function(nt,ke){var De=st(nt,ke);return u.createElement(ad.Provider,{key:De,value:Z(Z({},bn),{},{order:ke,item:nt,itemKey:De,registerSize:Fe,display:ke<=we})},d(nt,ke))}:function(nt,ke){var De=st(nt,ke);return u.createElement(bc,Ie({},bn,{order:ke,key:De,item:nt,renderItem:ht,itemKey:De,registerSize:Fe,display:ke<=we}))},sn={order:Nt?we:Number.MAX_SAFE_INTEGER,className:"".concat(xe,"-rest"),registerSize:_e,display:Nt},Ft=x||ID,nn=b?u.createElement(ad.Provider,{value:Z(Z({},bn),sn)},b(Ye)):u.createElement(bc,Ie({},bn,sn),typeof Ft=="function"?Ft(Ye):Ft),Et=u.createElement(O,Ie({className:re(!We&&r,E),style:y,ref:t},M),Ue.map($n),at?nn:null,w&&u.createElement(bc,Ie({},bn,{responsive:Ve,responsiveDisabled:!Ke,order:we,className:"".concat(xe,"-suffix"),registerSize:qe,display:!0,style:_n}),w));return Ve?u.createElement(Ro,{onResize:Pe,disabled:!Ke},Et):Et}var ka=u.forwardRef(jD);ka.displayName="Overflow";ka.Item=BO;ka.RESPONSIVE=LO;ka.INVALIDATE=HO;var PD=function(t,n){var r,o=t.prefixCls,l=t.id,c=t.inputElement,d=t.disabled,m=t.tabIndex,v=t.autoFocus,g=t.autoComplete,p=t.editable,y=t.activeDescendantId,E=t.value,C=t.maxLength,x=t.onKeyDown,b=t.onMouseDown,w=t.onChange,$=t.onPaste,O=t.onCompositionStart,T=t.onCompositionEnd,R=t.onBlur,M=t.open,N=t.attrs,z=c||u.createElement("input",null),P=z,D=P.ref,F=P.props,W=F.onKeyDown,V=F.onChange,q=F.onMouseDown,K=F.onCompositionStart,_=F.onCompositionEnd,j=F.onBlur,B=F.style;return"maxLength"in z.props,z=u.cloneElement(z,Z(Z(Z({type:"search"},F),{},{id:l,ref:Ia(n,D),disabled:d,tabIndex:m,autoComplete:g||"off",autoFocus:v,className:re("".concat(o,"-selection-search-input"),(r=z)===null||r===void 0||(r=r.props)===null||r===void 0?void 0:r.className),role:"combobox","aria-expanded":M||!1,"aria-haspopup":"listbox","aria-owns":"".concat(l,"_list"),"aria-autocomplete":"list","aria-controls":"".concat(l,"_list"),"aria-activedescendant":M?y:void 0},N),{},{value:p?E:"",maxLength:C,readOnly:!p,unselectable:p?null:"on",style:Z(Z({},B),{},{opacity:p?null:0}),onKeyDown:function(I){x(I),W&&W(I)},onMouseDown:function(I){b(I),q&&q(I)},onChange:function(I){w(I),V&&V(I)},onCompositionStart:function(I){O(I),K&&K(I)},onCompositionEnd:function(I){T(I),_&&_(I)},onPaste:$,onBlur:function(I){R(I),j&&j(I)}})),z},FO=u.forwardRef(PD);function VO(e){return Array.isArray(e)?e:e!==void 0?[e]:[]}var zD=typeof window<"u"&&window.document&&window.document.documentElement,DD=zD;function BD(e){return e!=null}function LD(e){return!e&&e!==0}function xx(e){return["string","number"].includes(lt(e))}function UO(e){var t=void 0;return e&&(xx(e.title)?t=e.title.toString():xx(e.label)&&(t=e.label.toString())),t}function HD(e,t){DD?u.useLayoutEffect(e,t):u.useEffect(e,t)}function FD(e){var t;return(t=e.key)!==null&&t!==void 0?t:e.value}var Ex=function(t){t.preventDefault(),t.stopPropagation()},VD=function(t){var n=t.id,r=t.prefixCls,o=t.values,l=t.open,c=t.searchValue,d=t.autoClearSearchValue,m=t.inputRef,v=t.placeholder,g=t.disabled,p=t.mode,y=t.showSearch,E=t.autoFocus,C=t.autoComplete,x=t.activeDescendantId,b=t.tabIndex,w=t.removeIcon,$=t.maxTagCount,O=t.maxTagTextLength,T=t.maxTagPlaceholder,R=T===void 0?function(Re){return"+ ".concat(Re.length," ...")}:T,M=t.tagRender,N=t.onToggleOpen,z=t.onRemove,P=t.onInputChange,D=t.onInputPaste,F=t.onInputKeyDown,W=t.onInputMouseDown,V=t.onInputCompositionStart,q=t.onInputCompositionEnd,K=t.onInputBlur,_=u.useRef(null),j=u.useState(0),B=de(j,2),k=B[0],I=B[1],H=u.useState(!1),G=de(H,2),X=G[0],U=G[1],J="".concat(r,"-selection"),te=l||p==="multiple"&&d===!1||p==="tags"?c:"",ve=p==="tags"||p==="multiple"&&d===!1||y&&(l||X);HD(function(){I(_.current.scrollWidth)},[te]);var ne=function(se,ye,$e,we,Ne){return u.createElement("span",{title:UO(se),className:re("".concat(J,"-item"),Y({},"".concat(J,"-item-disabled"),$e))},u.createElement("span",{className:"".concat(J,"-item-content")},ye),we&&u.createElement(kd,{className:"".concat(J,"-item-remove"),onMouseDown:Ex,onClick:Ne,customizeIcon:w},"×"))},le=function(se,ye,$e,we,Ne,Ze){var He=function(xe){Ex(xe),N(!l)};return u.createElement("span",{onMouseDown:He},M({label:ye,value:se,disabled:$e,closable:we,onClose:Ne,isMaxTag:!!Ze}))},be=function(se){var ye=se.disabled,$e=se.label,we=se.value,Ne=!g&&!ye,Ze=$e;if(typeof O=="number"&&(typeof $e=="string"||typeof $e=="number")){var He=String(Ze);He.length>O&&(Ze="".concat(He.slice(0,O),"..."))}var Xe=function(Me){Me&&Me.stopPropagation(),z(se)};return typeof M=="function"?le(we,Ze,ye,Ne,Xe):ne(se,Ze,ye,Ne,Xe)},fe=function(se){if(!o.length)return null;var ye=typeof R=="function"?R(se):R;return typeof M=="function"?le(void 0,ye,!1,!1,void 0,!0):ne({title:ye},ye,!1)},ie=u.createElement("div",{className:"".concat(J,"-search"),style:{width:k},onFocus:function(){U(!0)},onBlur:function(){U(!1)}},u.createElement(FO,{ref:m,open:l,prefixCls:r,id:n,inputElement:null,disabled:g,autoFocus:E,autoComplete:C,editable:ve,activeDescendantId:x,value:te,onKeyDown:F,onMouseDown:W,onChange:P,onPaste:D,onCompositionStart:V,onCompositionEnd:q,onBlur:K,tabIndex:b,attrs:Ga(t,!0)}),u.createElement("span",{ref:_,className:"".concat(J,"-search-mirror"),"aria-hidden":!0},te," ")),pe=u.createElement(ka,{prefixCls:"".concat(J,"-overflow"),data:o,renderItem:be,renderRest:fe,suffix:ie,itemKey:FD,maxCount:$});return u.createElement("span",{className:"".concat(J,"-wrap")},pe,!o.length&&!te&&u.createElement("span",{className:"".concat(J,"-placeholder")},v))},UD=function(t){var n=t.inputElement,r=t.prefixCls,o=t.id,l=t.inputRef,c=t.disabled,d=t.autoFocus,m=t.autoComplete,v=t.activeDescendantId,g=t.mode,p=t.open,y=t.values,E=t.placeholder,C=t.tabIndex,x=t.showSearch,b=t.searchValue,w=t.activeValue,$=t.maxLength,O=t.onInputKeyDown,T=t.onInputMouseDown,R=t.onInputChange,M=t.onInputPaste,N=t.onInputCompositionStart,z=t.onInputCompositionEnd,P=t.onInputBlur,D=t.title,F=u.useState(!1),W=de(F,2),V=W[0],q=W[1],K=g==="combobox",_=K||x,j=y[0],B=b||"";K&&w&&!V&&(B=w),u.useEffect(function(){K&&q(!1)},[K,w]);var k=g!=="combobox"&&!p&&!x?!1:!!B,I=D===void 0?UO(j):D,H=u.useMemo(function(){return j?null:u.createElement("span",{className:"".concat(r,"-selection-placeholder"),style:k?{visibility:"hidden"}:void 0},E)},[j,k,E,r]);return u.createElement("span",{className:"".concat(r,"-selection-wrap")},u.createElement("span",{className:"".concat(r,"-selection-search")},u.createElement(FO,{ref:l,prefixCls:r,id:o,open:p,inputElement:n,disabled:c,autoFocus:d,autoComplete:m,editable:_,activeDescendantId:v,value:B,onKeyDown:O,onMouseDown:T,onChange:function(X){q(!0),R(X)},onPaste:M,onCompositionStart:N,onCompositionEnd:z,onBlur:P,tabIndex:C,attrs:Ga(t,!0),maxLength:K?$:void 0})),!K&&j?u.createElement("span",{className:"".concat(r,"-selection-item"),title:I,style:k?{visibility:"hidden"}:void 0},j.label):null,H)},qD=function(t,n){var r=u.useRef(null),o=u.useRef(!1),l=t.prefixCls,c=t.open,d=t.mode,m=t.showSearch,v=t.tokenWithEnter,g=t.disabled,p=t.prefix,y=t.autoClearSearchValue,E=t.onSearch,C=t.onSearchSubmit,x=t.onToggleOpen,b=t.onInputKeyDown,w=t.onInputBlur,$=t.domRef;u.useImperativeHandle(n,function(){return{focus:function(I){r.current.focus(I)},blur:function(){r.current.blur()}}});var O=DO(0),T=de(O,2),R=T[0],M=T[1],N=function(I){var H=I.which,G=r.current instanceof HTMLTextAreaElement;!G&&c&&(H===ct.UP||H===ct.DOWN)&&I.preventDefault(),b&&b(I),H===ct.ENTER&&d==="tags"&&!o.current&&!c&&(C==null||C(I.target.value)),!(G&&!c&&~[ct.UP,ct.DOWN,ct.LEFT,ct.RIGHT].indexOf(H))&&ED(H)&&x(!0)},z=function(){M(!0)},P=u.useRef(null),D=function(I){E(I,!0,o.current)!==!1&&x(!0)},F=function(){o.current=!0},W=function(I){o.current=!1,d!=="combobox"&&D(I.target.value)},V=function(I){var H=I.target.value;if(v&&P.current&&/[\r\n]/.test(P.current)){var G=P.current.replace(/[\r\n]+$/,"").replace(/\r\n/g," ").replace(/[\r\n]/g," ");H=H.replace(G,P.current)}P.current=null,D(H)},q=function(I){var H=I.clipboardData,G=H==null?void 0:H.getData("text");P.current=G||""},K=function(I){var H=I.target;if(H!==r.current){var G=document.body.style.msTouchAction!==void 0;G?setTimeout(function(){r.current.focus()}):r.current.focus()}},_=function(I){var H=R();I.target!==r.current&&!H&&!(d==="combobox"&&g)&&I.preventDefault(),(d!=="combobox"&&(!m||!H)||!c)&&(c&&y!==!1&&E("",!0,!1),x())},j={inputRef:r,onInputKeyDown:N,onInputMouseDown:z,onInputChange:V,onInputPaste:q,onInputCompositionStart:F,onInputCompositionEnd:W,onInputBlur:w},B=d==="multiple"||d==="tags"?u.createElement(VD,Ie({},t,j)):u.createElement(UD,Ie({},t,j));return u.createElement("div",{ref:$,className:"".concat(l,"-selector"),onClick:K,onMouseDown:_},p&&u.createElement("div",{className:"".concat(l,"-prefix")},p),B)},WD=u.forwardRef(qD);function kD(e){var t=e.prefixCls,n=e.align,r=e.arrow,o=e.arrowPos,l=r||{},c=l.className,d=l.content,m=o.x,v=m===void 0?0:m,g=o.y,p=g===void 0?0:g,y=u.useRef();if(!n||!n.points)return null;var E={position:"absolute"};if(n.autoArrow!==!1){var C=n.points[0],x=n.points[1],b=C[0],w=C[1],$=x[0],O=x[1];b===$||!["t","b"].includes(b)?E.top=p:b==="t"?E.top=0:E.bottom=0,w===O||!["l","r"].includes(w)?E.left=v:w==="l"?E.left=0:E.right=0}return u.createElement("div",{ref:y,className:re("".concat(t,"-arrow"),c),style:E},d)}function GD(e){var t=e.prefixCls,n=e.open,r=e.zIndex,o=e.mask,l=e.motion;return o?u.createElement(Xa,Ie({},l,{motionAppear:!0,visible:n,removeOnLeave:!0}),function(c){var d=c.className;return u.createElement("div",{style:{zIndex:r},className:re("".concat(t,"-mask"),d)})}):null}var KD=u.memo(function(e){var t=e.children;return t},function(e,t){return t.cache}),XD=u.forwardRef(function(e,t){var n=e.popup,r=e.className,o=e.prefixCls,l=e.style,c=e.target,d=e.onVisibleChanged,m=e.open,v=e.keepDom,g=e.fresh,p=e.onClick,y=e.mask,E=e.arrow,C=e.arrowPos,x=e.align,b=e.motion,w=e.maskMotion,$=e.forceRender,O=e.getPopupContainer,T=e.autoDestroy,R=e.portal,M=e.zIndex,N=e.onMouseEnter,z=e.onMouseLeave,P=e.onPointerEnter,D=e.onPointerDownCapture,F=e.ready,W=e.offsetX,V=e.offsetY,q=e.offsetR,K=e.offsetB,_=e.onAlign,j=e.onPrepare,B=e.stretch,k=e.targetWidth,I=e.targetHeight,H=typeof n=="function"?n():n,G=m||v,X=(O==null?void 0:O.length)>0,U=u.useState(!O||!X),J=de(U,2),te=J[0],ve=J[1];if(Wt(function(){!te&&X&&c&&ve(!0)},[te,X,c]),!te)return null;var ne="auto",le={left:"-1000vw",top:"-1000vh",right:ne,bottom:ne};if(F||!m){var be,fe=x.points,ie=x.dynamicInset||((be=x._experimental)===null||be===void 0?void 0:be.dynamicInset),pe=ie&&fe[0][1]==="r",Re=ie&&fe[0][0]==="b";pe?(le.right=q,le.left=ne):(le.left=W,le.right=ne),Re?(le.bottom=K,le.top=ne):(le.top=V,le.bottom=ne)}var se={};return B&&(B.includes("height")&&I?se.height=I:B.includes("minHeight")&&I&&(se.minHeight=I),B.includes("width")&&k?se.width=k:B.includes("minWidth")&&k&&(se.minWidth=k)),m||(se.pointerEvents="none"),u.createElement(R,{open:$||G,getContainer:O&&function(){return O(c)},autoDestroy:T},u.createElement(GD,{prefixCls:o,open:m,zIndex:M,mask:y,motion:w}),u.createElement(Ro,{onResize:_,disabled:!m},function(ye){return u.createElement(Xa,Ie({motionAppear:!0,motionEnter:!0,motionLeave:!0,removeOnLeave:!1,forceRender:$,leavedClassName:"".concat(o,"-hidden")},b,{onAppearPrepare:j,onEnterPrepare:j,visible:m,onVisibleChanged:function(we){var Ne;b==null||(Ne=b.onVisibleChanged)===null||Ne===void 0||Ne.call(b,we),d(we)}}),function($e,we){var Ne=$e.className,Ze=$e.style,He=re(o,Ne,r);return u.createElement("div",{ref:Ia(ye,t,we),className:He,style:Z(Z(Z(Z({"--arrow-x":"".concat(C.x||0,"px"),"--arrow-y":"".concat(C.y||0,"px")},le),se),Ze),{},{boxSizing:"border-box",zIndex:M},l),onMouseEnter:N,onMouseLeave:z,onPointerEnter:P,onClick:p,onPointerDownCapture:D},E&&u.createElement(kD,{prefixCls:o,arrow:E,arrowPos:C,align:x}),u.createElement(KD,{cache:!m&&!g},H))})}))}),YD=u.forwardRef(function(e,t){var n=e.children,r=e.getTriggerDOMNode,o=ki(n),l=u.useCallback(function(d){kp(t,r?r(d):d)},[r]),c=li(l,Zl(n));return o?u.cloneElement(n,{ref:c}):n}),wx=u.createContext(null);function $x(e){return e?Array.isArray(e)?e:[e]:[]}function QD(e,t,n,r){return u.useMemo(function(){var o=$x(n??t),l=$x(r??t),c=new Set(o),d=new Set(l);return e&&(c.has("hover")&&(c.delete("hover"),c.add("click")),d.has("hover")&&(d.delete("hover"),d.add("click"))),[c,d]},[e,t,n,r])}function ZD(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:[],t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:[],n=arguments.length>2?arguments[2]:void 0;return n?e[0]===t[0]:e[0]===t[0]&&e[1]===t[1]}function JD(e,t,n,r){for(var o=n.points,l=Object.keys(e),c=0;c<l.length;c+=1){var d,m=l[c];if(ZD((d=e[m])===null||d===void 0?void 0:d.points,o,r))return"".concat(t,"-placement-").concat(m)}return""}function Ox(e,t,n,r){return t||(n?{motionName:"".concat(e,"-").concat(n)}:r?{motionName:r}:null)}function Jc(e){return e.ownerDocument.defaultView}function yp(e){for(var t=[],n=e==null?void 0:e.parentElement,r=["hidden","scroll","clip","auto"];n;){var o=Jc(n).getComputedStyle(n),l=o.overflowX,c=o.overflowY,d=o.overflow;[l,c,d].some(function(m){return r.includes(m)})&&t.push(n),n=n.parentElement}return t}function Ic(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:1;return Number.isNaN(e)?t:e}function ic(e){return Ic(parseFloat(e),0)}function Rx(e,t){var n=Z({},e);return(t||[]).forEach(function(r){if(!(r instanceof HTMLBodyElement||r instanceof HTMLHtmlElement)){var o=Jc(r).getComputedStyle(r),l=o.overflow,c=o.overflowClipMargin,d=o.borderTopWidth,m=o.borderBottomWidth,v=o.borderLeftWidth,g=o.borderRightWidth,p=r.getBoundingClientRect(),y=r.offsetHeight,E=r.clientHeight,C=r.offsetWidth,x=r.clientWidth,b=ic(d),w=ic(m),$=ic(v),O=ic(g),T=Ic(Math.round(p.width/C*1e3)/1e3),R=Ic(Math.round(p.height/y*1e3)/1e3),M=(C-x-$-O)*T,N=(y-E-b-w)*R,z=b*R,P=w*R,D=$*T,F=O*T,W=0,V=0;if(l==="clip"){var q=ic(c);W=q*T,V=q*R}var K=p.x+D-W,_=p.y+z-V,j=K+p.width+2*W-D-F-M,B=_+p.height+2*V-z-P-N;n.left=Math.max(n.left,K),n.top=Math.max(n.top,_),n.right=Math.min(n.right,j),n.bottom=Math.min(n.bottom,B)}}),n}function Tx(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,n="".concat(t),r=n.match(/^(.*)\%$/);return r?e*(parseFloat(r[1])/100):parseFloat(n)}function Mx(e,t){var n=t||[],r=de(n,2),o=r[0],l=r[1];return[Tx(e.width,o),Tx(e.height,l)]}function _x(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"";return[e[0],e[1]]}function Nl(e,t){var n=t[0],r=t[1],o,l;return n==="t"?l=e.y:n==="b"?l=e.y+e.height:l=e.y+e.height/2,r==="l"?o=e.x:r==="r"?o=e.x+e.width:o=e.x+e.width/2,{x:o,y:l}}function Qo(e,t){var n={t:"b",b:"t",l:"r",r:"l"};return e.map(function(r,o){return o===t?n[r]||"c":r}).join("")}function e5(e,t,n,r,o,l,c){var d=u.useState({ready:!1,offsetX:0,offsetY:0,offsetR:0,offsetB:0,arrowX:0,arrowY:0,scaleX:1,scaleY:1,align:o[r]||{}}),m=de(d,2),v=m[0],g=m[1],p=u.useRef(0),y=u.useMemo(function(){return t?yp(t):[]},[t]),E=u.useRef({}),C=function(){E.current={}};e||C();var x=jn(function(){if(t&&n&&e){let Mr=function(Ba,aa){var Er=arguments.length>2&&arguments[2]!==void 0?arguments[2]:Me,oa=U.x+Ba,ui=U.y+aa,fi=oa+pe,Yi=ui+ie,Ja=Math.max(oa,Er.left),Ge=Math.max(ui,Er.top),dt=Math.min(fi,Er.right),en=Math.min(Yi,Er.bottom);return Math.max(0,(dt-Ja)*(en-Ge))},Za=function(){Qt=U.y+De,rn=Qt+ie,Rt=U.x+ke,er=Rt+pe};var $,O,T,R,M=t,N=M.ownerDocument,z=Jc(M),P=z.getComputedStyle(M),D=P.width,F=P.height,W=P.position,V=M.style.left,q=M.style.top,K=M.style.right,_=M.style.bottom,j=M.style.overflow,B=Z(Z({},o[r]),l),k=N.createElement("div");($=M.parentElement)===null||$===void 0||$.appendChild(k),k.style.left="".concat(M.offsetLeft,"px"),k.style.top="".concat(M.offsetTop,"px"),k.style.position=W,k.style.height="".concat(M.offsetHeight,"px"),k.style.width="".concat(M.offsetWidth,"px"),M.style.left="0",M.style.top="0",M.style.right="auto",M.style.bottom="auto",M.style.overflow="hidden";var I;if(Array.isArray(n))I={x:n[0],y:n[1],width:0,height:0};else{var H,G,X=n.getBoundingClientRect();X.x=(H=X.x)!==null&&H!==void 0?H:X.left,X.y=(G=X.y)!==null&&G!==void 0?G:X.top,I={x:X.x,y:X.y,width:X.width,height:X.height}}var U=M.getBoundingClientRect();U.x=(O=U.x)!==null&&O!==void 0?O:U.left,U.y=(T=U.y)!==null&&T!==void 0?T:U.top;var J=N.documentElement,te=J.clientWidth,ve=J.clientHeight,ne=J.scrollWidth,le=J.scrollHeight,be=J.scrollTop,fe=J.scrollLeft,ie=U.height,pe=U.width,Re=I.height,se=I.width,ye={left:0,top:0,right:te,bottom:ve},$e={left:-fe,top:-be,right:ne-fe,bottom:le-be},we=B.htmlRegion,Ne="visible",Ze="visibleFirst";we!=="scroll"&&we!==Ze&&(we=Ne);var He=we===Ze,Xe=Rx($e,y),xe=Rx(ye,y),Me=we===Ne?xe:Xe,Ve=He?xe:Me;M.style.left="auto",M.style.top="auto",M.style.right="0",M.style.bottom="0";var Ke=M.getBoundingClientRect();M.style.left=V,M.style.top=q,M.style.right=K,M.style.bottom=_,M.style.overflow=j,(R=M.parentElement)===null||R===void 0||R.removeChild(k);var We=Ic(Math.round(pe/parseFloat(D)*1e3)/1e3),at=Ic(Math.round(ie/parseFloat(F)*1e3)/1e3);if(We===0||at===0||xc(n)&&!jd(n))return;var Ue=B.offset,Ye=B.targetOffset,st=Mx(U,Ue),ht=de(st,2),Qe=ht[0],Pe=ht[1],Fe=Mx(I,Ye),_e=de(Fe,2),qe=_e[0],Ct=_e[1];I.x-=qe,I.y-=Ct;var Nt=B.points||[],_n=de(Nt,2),bn=_n[0],$n=_n[1],sn=_x($n),Ft=_x(bn),nn=Nl(I,sn),Et=Nl(U,Ft),nt=Z({},B),ke=nn.x-Et.x+Qe,De=nn.y-Et.y+Pe,pt=Mr(ke,De),St=Mr(ke,De,xe),Be=Nl(I,["t","l"]),je=Nl(U,["t","l"]),rt=Nl(I,["b","r"]),ot=Nl(U,["b","r"]),tt=B.overflow||{},ut=tt.adjustX,Vt=tt.adjustY,Tt=tt.shiftX,At=tt.shiftY,It=function(aa){return typeof aa=="boolean"?aa:aa>=0},Qt,rn,Rt,er;Za();var pn=It(Vt),an=Ft[0]===sn[0];if(pn&&Ft[0]==="t"&&(rn>Ve.bottom||E.current.bt)){var Ae=De;an?Ae-=ie-Re:Ae=Be.y-ot.y-Pe;var et=Mr(ke,Ae),yt=Mr(ke,Ae,xe);et>pt||et===pt&&(!He||yt>=St)?(E.current.bt=!0,De=Ae,Pe=-Pe,nt.points=[Qo(Ft,0),Qo(sn,0)]):E.current.bt=!1}if(pn&&Ft[0]==="b"&&(Qt<Ve.top||E.current.tb)){var xt=De;an?xt+=ie-Re:xt=rt.y-je.y-Pe;var Gt=Mr(ke,xt),kn=Mr(ke,xt,xe);Gt>pt||Gt===pt&&(!He||kn>=St)?(E.current.tb=!0,De=xt,Pe=-Pe,nt.points=[Qo(Ft,0),Qo(sn,0)]):E.current.tb=!1}var Pn=It(ut),Ln=Ft[1]===sn[1];if(Pn&&Ft[1]==="l"&&(er>Ve.right||E.current.rl)){var cr=ke;Ln?cr-=pe-se:cr=Be.x-ot.x-Qe;var hr=Mr(cr,De),On=Mr(cr,De,xe);hr>pt||hr===pt&&(!He||On>=St)?(E.current.rl=!0,ke=cr,Qe=-Qe,nt.points=[Qo(Ft,1),Qo(sn,1)]):E.current.rl=!1}if(Pn&&Ft[1]==="r"&&(Rt<Ve.left||E.current.lr)){var tr=ke;Ln?tr+=pe-se:tr=rt.x-je.x-Qe;var Dr=Mr(tr,De),Da=Mr(tr,De,xe);Dr>pt||Dr===pt&&(!He||Da>=St)?(E.current.lr=!0,ke=tr,Qe=-Qe,nt.points=[Qo(Ft,1),Qo(sn,1)]):E.current.lr=!1}Za();var Gn=Tt===!0?0:Tt;typeof Gn=="number"&&(Rt<xe.left&&(ke-=Rt-xe.left-Qe,I.x+se<xe.left+Gn&&(ke+=I.x-xe.left+se-Gn)),er>xe.right&&(ke-=er-xe.right-Qe,I.x>xe.right-Gn&&(ke+=I.x-xe.right+Gn)));var cn=At===!0?0:At;typeof cn=="number"&&(Qt<xe.top&&(De-=Qt-xe.top-Pe,I.y+Re<xe.top+cn&&(De+=I.y-xe.top+Re-cn)),rn>xe.bottom&&(De-=rn-xe.bottom-Pe,I.y>xe.bottom-cn&&(De+=I.y-xe.bottom+cn)));var Br=U.x+ke,ra=Br+pe,Kn=U.y+De,Ca=Kn+ie,Bt=I.x,Je=Bt+se,ze=I.y,Lt=ze+Re,Ut=Math.max(Br,Bt),Kt=Math.min(ra,Je),Xt=(Ut+Kt)/2,nr=Xt-Br,Hn=Math.max(Kn,ze),Nn=Math.min(Ca,Lt),rr=(Hn+Nn)/2,ar=rr-Kn;c==null||c(t,nt);var xr=Ke.right-U.x-(ke+U.width),Xn=Ke.bottom-U.y-(De+U.height);We===1&&(ke=Math.round(ke),xr=Math.round(xr)),at===1&&(De=Math.round(De),Xn=Math.round(Xn));var cs={ready:!0,offsetX:ke/We,offsetY:De/at,offsetR:xr/We,offsetB:Xn/at,arrowX:nr/We,arrowY:ar/at,scaleX:We,scaleY:at,align:nt};g(cs)}}),b=function(){p.current+=1;var O=p.current;Promise.resolve().then(function(){p.current===O&&x()})},w=function(){g(function(O){return Z(Z({},O),{},{ready:!1})})};return Wt(w,[r]),Wt(function(){e||w()},[e]),[v.ready,v.offsetX,v.offsetY,v.offsetR,v.offsetB,v.arrowX,v.arrowY,v.scaleX,v.scaleY,v.align,b]}function t5(e,t,n,r,o){Wt(function(){if(e&&t&&n){let p=function(){r(),o()};var l=t,c=n,d=yp(l),m=yp(c),v=Jc(c),g=new Set([v].concat(Te(d),Te(m)));return g.forEach(function(y){y.addEventListener("scroll",p,{passive:!0})}),v.addEventListener("resize",p,{passive:!0}),r(),function(){g.forEach(function(y){y.removeEventListener("scroll",p),v.removeEventListener("resize",p)})}}},[e,t,n])}function n5(e,t,n,r,o,l,c,d){var m=u.useRef(e);m.current=e;var v=u.useRef(!1);u.useEffect(function(){if(t&&r&&(!o||l)){var p=function(){v.current=!1},y=function(b){var w;m.current&&!c(((w=b.composedPath)===null||w===void 0||(w=w.call(b))===null||w===void 0?void 0:w[0])||b.target)&&!v.current&&d(!1)},E=Jc(r);E.addEventListener("pointerdown",p,!0),E.addEventListener("mousedown",y,!0),E.addEventListener("contextmenu",y,!0);var C=Gf(n);return C&&(C.addEventListener("mousedown",y,!0),C.addEventListener("contextmenu",y,!0)),function(){E.removeEventListener("pointerdown",p,!0),E.removeEventListener("mousedown",y,!0),E.removeEventListener("contextmenu",y,!0),C&&(C.removeEventListener("mousedown",y,!0),C.removeEventListener("contextmenu",y,!0))}}},[t,n,r,o,l]);function g(){v.current=!0}return g}var r5=["prefixCls","children","action","showAction","hideAction","popupVisible","defaultPopupVisible","onPopupVisibleChange","afterPopupVisibleChange","mouseEnterDelay","mouseLeaveDelay","focusDelay","blurDelay","mask","maskClosable","getPopupContainer","forceRender","autoDestroy","destroyPopupOnHide","popup","popupClassName","popupStyle","popupPlacement","builtinPlacements","popupAlign","zIndex","stretch","getPopupClassNameFromAlign","fresh","alignPoint","onPopupClick","onPopupAlign","arrow","popupMotion","maskMotion","popupTransitionName","popupAnimation","maskTransitionName","maskAnimation","className","getTriggerDOMNode"];function a5(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:b0,t=u.forwardRef(function(n,r){var o=n.prefixCls,l=o===void 0?"rc-trigger-popup":o,c=n.children,d=n.action,m=d===void 0?"hover":d,v=n.showAction,g=n.hideAction,p=n.popupVisible,y=n.defaultPopupVisible,E=n.onPopupVisibleChange,C=n.afterPopupVisibleChange,x=n.mouseEnterDelay,b=n.mouseLeaveDelay,w=b===void 0?.1:b,$=n.focusDelay,O=n.blurDelay,T=n.mask,R=n.maskClosable,M=R===void 0?!0:R,N=n.getPopupContainer,z=n.forceRender,P=n.autoDestroy,D=n.destroyPopupOnHide,F=n.popup,W=n.popupClassName,V=n.popupStyle,q=n.popupPlacement,K=n.builtinPlacements,_=K===void 0?{}:K,j=n.popupAlign,B=n.zIndex,k=n.stretch,I=n.getPopupClassNameFromAlign,H=n.fresh,G=n.alignPoint,X=n.onPopupClick,U=n.onPopupAlign,J=n.arrow,te=n.popupMotion,ve=n.maskMotion,ne=n.popupTransitionName,le=n.popupAnimation,be=n.maskTransitionName,fe=n.maskAnimation,ie=n.className,pe=n.getTriggerDOMNode,Re=wt(n,r5),se=P||D||!1,ye=u.useState(!1),$e=de(ye,2),we=$e[0],Ne=$e[1];Wt(function(){Ne(PO())},[]);var Ze=u.useRef({}),He=u.useContext(wx),Xe=u.useMemo(function(){return{registerSubPopup:function(dt,en){Ze.current[dt]=en,He==null||He.registerSubPopup(dt,en)}}},[He]),xe=S0(),Me=u.useState(null),Ve=de(Me,2),Ke=Ve[0],We=Ve[1],at=u.useRef(null),Ue=jn(function(Ge){at.current=Ge,xc(Ge)&&Ke!==Ge&&We(Ge),He==null||He.registerSubPopup(xe,Ge)}),Ye=u.useState(null),st=de(Ye,2),ht=st[0],Qe=st[1],Pe=u.useRef(null),Fe=jn(function(Ge){xc(Ge)&&ht!==Ge&&(Qe(Ge),Pe.current=Ge)}),_e=u.Children.only(c),qe=(_e==null?void 0:_e.props)||{},Ct={},Nt=jn(function(Ge){var dt,en,un=ht;return(un==null?void 0:un.contains(Ge))||((dt=Gf(un))===null||dt===void 0?void 0:dt.host)===Ge||Ge===un||(Ke==null?void 0:Ke.contains(Ge))||((en=Gf(Ke))===null||en===void 0?void 0:en.host)===Ge||Ge===Ke||Object.values(Ze.current).some(function(Sn){return(Sn==null?void 0:Sn.contains(Ge))||Ge===Sn})}),_n=Ox(l,te,le,ne),bn=Ox(l,ve,fe,be),$n=u.useState(y||!1),sn=de($n,2),Ft=sn[0],nn=sn[1],Et=p??Ft,nt=jn(function(Ge){p===void 0&&nn(Ge)});Wt(function(){nn(p||!1)},[p]);var ke=u.useRef(Et);ke.current=Et;var De=u.useRef([]);De.current=[];var pt=jn(function(Ge){var dt;nt(Ge),((dt=De.current[De.current.length-1])!==null&&dt!==void 0?dt:Et)!==Ge&&(De.current.push(Ge),E==null||E(Ge))}),St=u.useRef(),Be=function(){clearTimeout(St.current)},je=function(dt){var en=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0;Be(),en===0?pt(dt):St.current=setTimeout(function(){pt(dt)},en*1e3)};u.useEffect(function(){return Be},[]);var rt=u.useState(!1),ot=de(rt,2),tt=ot[0],ut=ot[1];Wt(function(Ge){(!Ge||Et)&&ut(!0)},[Et]);var Vt=u.useState(null),Tt=de(Vt,2),At=Tt[0],It=Tt[1],Qt=u.useState(null),rn=de(Qt,2),Rt=rn[0],er=rn[1],pn=function(dt){er([dt.clientX,dt.clientY])},an=e5(Et,Ke,G&&Rt!==null?Rt:ht,q,_,j,U),Ae=de(an,11),et=Ae[0],yt=Ae[1],xt=Ae[2],Gt=Ae[3],kn=Ae[4],Pn=Ae[5],Ln=Ae[6],cr=Ae[7],hr=Ae[8],On=Ae[9],tr=Ae[10],Dr=QD(we,m,v,g),Da=de(Dr,2),Gn=Da[0],cn=Da[1],Br=Gn.has("click"),ra=cn.has("click")||cn.has("contextMenu"),Kn=jn(function(){tt||tr()}),Ca=function(){ke.current&&G&&ra&&je(!1)};t5(Et,ht,Ke,Kn,Ca),Wt(function(){Kn()},[Rt,q]),Wt(function(){Et&&!(_!=null&&_[q])&&Kn()},[JSON.stringify(j)]);var Bt=u.useMemo(function(){var Ge=JD(_,l,On,G);return re(Ge,I==null?void 0:I(On))},[On,I,_,l,G]);u.useImperativeHandle(r,function(){return{nativeElement:Pe.current,popupElement:at.current,forceAlign:Kn}});var Je=u.useState(0),ze=de(Je,2),Lt=ze[0],Ut=ze[1],Kt=u.useState(0),Xt=de(Kt,2),nr=Xt[0],Hn=Xt[1],Nn=function(){if(k&&ht){var dt=ht.getBoundingClientRect();Ut(dt.width),Hn(dt.height)}},rr=function(){Nn(),Kn()},ar=function(dt){ut(!1),tr(),C==null||C(dt)},xr=function(){return new Promise(function(dt){Nn(),It(function(){return dt})})};Wt(function(){At&&(tr(),At(),It(null))},[At]);function Xn(Ge,dt,en,un){Ct[Ge]=function(Sn){var eo;un==null||un(Sn),je(dt,en);for(var Qi=arguments.length,di=new Array(Qi>1?Qi-1:0),to=1;to<Qi;to++)di[to-1]=arguments[to];(eo=qe[Ge])===null||eo===void 0||eo.call.apply(eo,[qe,Sn].concat(di))}}(Br||ra)&&(Ct.onClick=function(Ge){var dt;ke.current&&ra?je(!1):!ke.current&&Br&&(pn(Ge),je(!0));for(var en=arguments.length,un=new Array(en>1?en-1:0),Sn=1;Sn<en;Sn++)un[Sn-1]=arguments[Sn];(dt=qe.onClick)===null||dt===void 0||dt.call.apply(dt,[qe,Ge].concat(un))});var cs=n5(Et,ra,ht,Ke,T,M,Nt,je),Mr=Gn.has("hover"),Za=cn.has("hover"),Ba,aa;Mr&&(Xn("onMouseEnter",!0,x,function(Ge){pn(Ge)}),Xn("onPointerEnter",!0,x,function(Ge){pn(Ge)}),Ba=function(dt){(Et||tt)&&Ke!==null&&Ke!==void 0&&Ke.contains(dt.target)&&je(!0,x)},G&&(Ct.onMouseMove=function(Ge){var dt;(dt=qe.onMouseMove)===null||dt===void 0||dt.call(qe,Ge)})),Za&&(Xn("onMouseLeave",!1,w),Xn("onPointerLeave",!1,w),aa=function(){je(!1,w)}),Gn.has("focus")&&Xn("onFocus",!0,$),cn.has("focus")&&Xn("onBlur",!1,O),Gn.has("contextMenu")&&(Ct.onContextMenu=function(Ge){var dt;ke.current&&cn.has("contextMenu")?je(!1):(pn(Ge),je(!0)),Ge.preventDefault();for(var en=arguments.length,un=new Array(en>1?en-1:0),Sn=1;Sn<en;Sn++)un[Sn-1]=arguments[Sn];(dt=qe.onContextMenu)===null||dt===void 0||dt.call.apply(dt,[qe,Ge].concat(un))}),ie&&(Ct.className=re(qe.className,ie));var Er=Z(Z({},qe),Ct),oa={},ui=["onContextMenu","onClick","onMouseDown","onTouchStart","onMouseEnter","onMouseLeave","onFocus","onBlur"];ui.forEach(function(Ge){Re[Ge]&&(oa[Ge]=function(){for(var dt,en=arguments.length,un=new Array(en),Sn=0;Sn<en;Sn++)un[Sn]=arguments[Sn];(dt=Er[Ge])===null||dt===void 0||dt.call.apply(dt,[Er].concat(un)),Re[Ge].apply(Re,un)})});var fi=u.cloneElement(_e,Z(Z({},Er),oa)),Yi={x:Pn,y:Ln},Ja=J?Z({},J!==!0?J:{}):null;return u.createElement(u.Fragment,null,u.createElement(Ro,{disabled:!Et,ref:Fe,onResize:rr},u.createElement(YD,{getTriggerDOMNode:pe},fi)),u.createElement(wx.Provider,{value:Xe},u.createElement(XD,{portal:e,ref:Ue,prefixCls:l,popup:F,className:re(W,Bt),style:V,target:ht,onMouseEnter:Ba,onMouseLeave:aa,onPointerEnter:Ba,zIndex:B,open:Et,keepDom:tt,fresh:H,onClick:X,onPointerDownCapture:cs,mask:T,motion:_n,maskMotion:bn,onVisibleChanged:ar,onPrepare:xr,forceRender:z,autoDestroy:se,getPopupContainer:N,align:On,arrow:Ja,arrowPos:Yi,ready:et,offsetX:yt,offsetY:xt,offsetR:Gt,offsetB:kn,onAlign:Kn,stretch:k,targetWidth:Lt/cr,targetHeight:nr/hr})))});return t}const O0=a5(b0);var o5=["prefixCls","disabled","visible","children","popupElement","animation","transitionName","dropdownStyle","dropdownClassName","direction","placement","builtinPlacements","dropdownMatchSelectWidth","dropdownRender","dropdownAlign","getPopupContainer","empty","getTriggerDOMNode","onPopupVisibleChange","onPopupMouseEnter"],i5=function(t){var n=t===!0?0:1;return{bottomLeft:{points:["tl","bl"],offset:[0,4],overflow:{adjustX:n,adjustY:1},htmlRegion:"scroll"},bottomRight:{points:["tr","br"],offset:[0,4],overflow:{adjustX:n,adjustY:1},htmlRegion:"scroll"},topLeft:{points:["bl","tl"],offset:[0,-4],overflow:{adjustX:n,adjustY:1},htmlRegion:"scroll"},topRight:{points:["br","tr"],offset:[0,-4],overflow:{adjustX:n,adjustY:1},htmlRegion:"scroll"}}},l5=function(t,n){var r=t.prefixCls;t.disabled;var o=t.visible,l=t.children,c=t.popupElement,d=t.animation,m=t.transitionName,v=t.dropdownStyle,g=t.dropdownClassName,p=t.direction,y=p===void 0?"ltr":p,E=t.placement,C=t.builtinPlacements,x=t.dropdownMatchSelectWidth,b=t.dropdownRender,w=t.dropdownAlign,$=t.getPopupContainer,O=t.empty,T=t.getTriggerDOMNode,R=t.onPopupVisibleChange,M=t.onPopupMouseEnter,N=wt(t,o5),z="".concat(r,"-dropdown"),P=c;b&&(P=b(c));var D=u.useMemo(function(){return C||i5(x)},[C,x]),F=d?"".concat(z,"-").concat(d):m,W=typeof x=="number",V=u.useMemo(function(){return W?null:x===!1?"minWidth":"width"},[x,W]),q=v;W&&(q=Z(Z({},q),{},{width:x}));var K=u.useRef(null);return u.useImperativeHandle(n,function(){return{getPopupElement:function(){var j;return(j=K.current)===null||j===void 0?void 0:j.popupElement}}}),u.createElement(O0,Ie({},N,{showAction:R?["click"]:[],hideAction:R?["click"]:[],popupPlacement:E||(y==="rtl"?"bottomRight":"bottomLeft"),builtinPlacements:D,prefixCls:z,popupTransitionName:F,popup:u.createElement("div",{onMouseEnter:M},P),ref:K,stretch:V,popupAlign:w,popupVisible:o,getPopupContainer:$,popupClassName:re(g,Y({},"".concat(z,"-empty"),O)),popupStyle:q,getTriggerDOMNode:T,onPopupVisibleChange:R}),l)},s5=u.forwardRef(l5);function Nx(e,t){var n=e.key,r;return"value"in e&&(r=e.value),n??(r!==void 0?r:"rc-index-key-".concat(t))}function bp(e){return typeof e<"u"&&!Number.isNaN(e)}function qO(e,t){var n=e||{},r=n.label,o=n.value,l=n.options,c=n.groupLabel,d=r||(t?"children":"label");return{label:d,value:o||"value",options:l||"options",groupLabel:c||d}}function c5(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{},n=t.fieldNames,r=t.childrenAsData,o=[],l=qO(n,!1),c=l.label,d=l.value,m=l.options,v=l.groupLabel;function g(p,y){Array.isArray(p)&&p.forEach(function(E){if(y||!(m in E)){var C=E[d];o.push({key:Nx(E,o.length),groupOption:y,data:E,label:E[c],value:C})}else{var x=E[v];x===void 0&&r&&(x=E.label),o.push({key:Nx(E,o.length),group:!0,data:E,label:x}),g(E[m],!0)}})}return g(e,!1),o}function Sp(e){var t=Z({},e);return"props"in t||Object.defineProperty(t,"props",{get:function(){return vr(!1,"Return type is option instead of Option instance. Please read value directly instead of reading from `props`."),t}}),t}var u5=function(t,n,r){if(!n||!n.length)return null;var o=!1,l=function d(m,v){var g=Qw(v),p=g[0],y=g.slice(1);if(!p)return[m];var E=m.split(p);return o=o||E.length>1,E.reduce(function(C,x){return[].concat(Te(C),Te(d(x,y)))},[]).filter(Boolean)},c=l(t,n);return o?typeof r<"u"?c.slice(0,r):c:null},R0=u.createContext(null);function f5(e){var t=e.visible,n=e.values;if(!t)return null;var r=50;return u.createElement("span",{"aria-live":"polite",style:{width:0,height:0,position:"absolute",overflow:"hidden",opacity:0}},"".concat(n.slice(0,r).map(function(o){var l=o.label,c=o.value;return["number","string"].includes(lt(l))?l:c}).join(", ")),n.length>r?", ...":null)}var d5=["id","prefixCls","className","showSearch","tagRender","direction","omitDomProps","displayValues","onDisplayValuesChange","emptyOptions","notFoundContent","onClear","mode","disabled","loading","getInputElement","getRawInputElement","open","defaultOpen","onDropdownVisibleChange","activeValue","onActiveValueChange","activeDescendantId","searchValue","autoClearSearchValue","onSearch","onSearchSplit","tokenSeparators","allowClear","prefix","suffixIcon","clearIcon","OptionList","animation","transitionName","dropdownStyle","dropdownClassName","dropdownMatchSelectWidth","dropdownRender","dropdownAlign","placement","builtinPlacements","getPopupContainer","showAction","onFocus","onBlur","onKeyUp","onKeyDown","onMouseDown"],m5=["value","onChange","removeIcon","placeholder","autoFocus","maxTagCount","maxTagTextLength","maxTagPlaceholder","choiceTransitionName","onInputKeyDown","onPopupScroll","tabIndex"],Cp=function(t){return t==="tags"||t==="multiple"},v5=u.forwardRef(function(e,t){var n,r=e.id,o=e.prefixCls,l=e.className,c=e.showSearch,d=e.tagRender,m=e.direction,v=e.omitDomProps,g=e.displayValues,p=e.onDisplayValuesChange,y=e.emptyOptions,E=e.notFoundContent,C=E===void 0?"Not Found":E,x=e.onClear,b=e.mode,w=e.disabled,$=e.loading,O=e.getInputElement,T=e.getRawInputElement,R=e.open,M=e.defaultOpen,N=e.onDropdownVisibleChange,z=e.activeValue,P=e.onActiveValueChange,D=e.activeDescendantId,F=e.searchValue,W=e.autoClearSearchValue,V=e.onSearch,q=e.onSearchSplit,K=e.tokenSeparators,_=e.allowClear,j=e.prefix,B=e.suffixIcon,k=e.clearIcon,I=e.OptionList,H=e.animation,G=e.transitionName,X=e.dropdownStyle,U=e.dropdownClassName,J=e.dropdownMatchSelectWidth,te=e.dropdownRender,ve=e.dropdownAlign,ne=e.placement,le=e.builtinPlacements,be=e.getPopupContainer,fe=e.showAction,ie=fe===void 0?[]:fe,pe=e.onFocus,Re=e.onBlur,se=e.onKeyUp,ye=e.onKeyDown,$e=e.onMouseDown,we=wt(e,d5),Ne=Cp(b),Ze=(c!==void 0?c:Ne)||b==="combobox",He=Z({},we);m5.forEach(function(Je){delete He[Je]}),v==null||v.forEach(function(Je){delete He[Je]});var Xe=u.useState(!1),xe=de(Xe,2),Me=xe[0],Ve=xe[1];u.useEffect(function(){Ve(PO())},[]);var Ke=u.useRef(null),We=u.useRef(null),at=u.useRef(null),Ue=u.useRef(null),Ye=u.useRef(null),st=u.useRef(!1),ht=CD(),Qe=de(ht,3),Pe=Qe[0],Fe=Qe[1],_e=Qe[2];u.useImperativeHandle(t,function(){var Je,ze;return{focus:(Je=Ue.current)===null||Je===void 0?void 0:Je.focus,blur:(ze=Ue.current)===null||ze===void 0?void 0:ze.blur,scrollTo:function(Ut){var Kt;return(Kt=Ye.current)===null||Kt===void 0?void 0:Kt.scrollTo(Ut)},nativeElement:Ke.current||We.current}});var qe=u.useMemo(function(){var Je;if(b!=="combobox")return F;var ze=(Je=g[0])===null||Je===void 0?void 0:Je.value;return typeof ze=="string"||typeof ze=="number"?String(ze):""},[F,b,g]),Ct=b==="combobox"&&typeof O=="function"&&O()||null,Nt=typeof T=="function"&&T(),_n=li(We,Nt==null||(n=Nt.props)===null||n===void 0?void 0:n.ref),bn=u.useState(!1),$n=de(bn,2),sn=$n[0],Ft=$n[1];Wt(function(){Ft(!0)},[]);var nn=gr(!1,{defaultValue:M,value:R}),Et=de(nn,2),nt=Et[0],ke=Et[1],De=sn?nt:!1,pt=!C&&y;(w||pt&&De&&b==="combobox")&&(De=!1);var St=pt?!1:De,Be=u.useCallback(function(Je){var ze=Je!==void 0?Je:!De;w||(ke(ze),De!==ze&&(N==null||N(ze)))},[w,De,ke,N]),je=u.useMemo(function(){return(K||[]).some(function(Je){return[`
`,`\r
`].includes(Je)})},[K]),rt=u.useContext(R0)||{},ot=rt.maxCount,tt=rt.rawValues,ut=function(ze,Lt,Ut){if(!(Ne&&bp(ot)&&(tt==null?void 0:tt.size)>=ot)){var Kt=!0,Xt=ze;P==null||P(null);var nr=u5(ze,K,bp(ot)?ot-tt.size:void 0),Hn=Ut?null:nr;return b!=="combobox"&&Hn&&(Xt="",q==null||q(Hn),Be(!1),Kt=!1),V&&qe!==Xt&&V(Xt,{source:Lt?"typing":"effect"}),Kt}},Vt=function(ze){!ze||!ze.trim()||V(ze,{source:"submit"})};u.useEffect(function(){!De&&!Ne&&b!=="combobox"&&ut("",!1,!1)},[De]),u.useEffect(function(){nt&&w&&ke(!1),w&&!st.current&&Fe(!1)},[w]);var Tt=DO(),At=de(Tt,2),It=At[0],Qt=At[1],rn=u.useRef(!1),Rt=function(ze){var Lt=It(),Ut=ze.key,Kt=Ut==="Enter";if(Kt&&(b!=="combobox"&&ze.preventDefault(),De||Be(!0)),Qt(!!qe),Ut==="Backspace"&&!Lt&&Ne&&!qe&&g.length){for(var Xt=Te(g),nr=null,Hn=Xt.length-1;Hn>=0;Hn-=1){var Nn=Xt[Hn];if(!Nn.disabled){Xt.splice(Hn,1),nr=Nn;break}}nr&&p(Xt,{type:"remove",values:[nr]})}for(var rr=arguments.length,ar=new Array(rr>1?rr-1:0),xr=1;xr<rr;xr++)ar[xr-1]=arguments[xr];if(De&&(!Kt||!rn.current)){var Xn;Kt&&(rn.current=!0),(Xn=Ye.current)===null||Xn===void 0||Xn.onKeyDown.apply(Xn,[ze].concat(ar))}ye==null||ye.apply(void 0,[ze].concat(ar))},er=function(ze){for(var Lt=arguments.length,Ut=new Array(Lt>1?Lt-1:0),Kt=1;Kt<Lt;Kt++)Ut[Kt-1]=arguments[Kt];if(De){var Xt;(Xt=Ye.current)===null||Xt===void 0||Xt.onKeyUp.apply(Xt,[ze].concat(Ut))}ze.key==="Enter"&&(rn.current=!1),se==null||se.apply(void 0,[ze].concat(Ut))},pn=function(ze){var Lt=g.filter(function(Ut){return Ut!==ze});p(Lt,{type:"remove",values:[ze]})},an=function(){rn.current=!1},Ae=u.useRef(!1),et=function(){Fe(!0),w||(pe&&!Ae.current&&pe.apply(void 0,arguments),ie.includes("focus")&&Be(!0)),Ae.current=!0},yt=function(){st.current=!0,Fe(!1,function(){Ae.current=!1,st.current=!1,Be(!1)}),!w&&(qe&&(b==="tags"?V(qe,{source:"submit"}):b==="multiple"&&V("",{source:"blur"})),Re&&Re.apply(void 0,arguments))},xt=[];u.useEffect(function(){return function(){xt.forEach(function(Je){return clearTimeout(Je)}),xt.splice(0,xt.length)}},[]);var Gt=function(ze){var Lt,Ut=ze.target,Kt=(Lt=at.current)===null||Lt===void 0?void 0:Lt.getPopupElement();if(Kt&&Kt.contains(Ut)){var Xt=setTimeout(function(){var rr=xt.indexOf(Xt);if(rr!==-1&&xt.splice(rr,1),_e(),!Me&&!Kt.contains(document.activeElement)){var ar;(ar=Ue.current)===null||ar===void 0||ar.focus()}});xt.push(Xt)}for(var nr=arguments.length,Hn=new Array(nr>1?nr-1:0),Nn=1;Nn<nr;Nn++)Hn[Nn-1]=arguments[Nn];$e==null||$e.apply(void 0,[ze].concat(Hn))},kn=u.useState({}),Pn=de(kn,2),Ln=Pn[1];function cr(){Ln({})}var hr;Nt&&(hr=function(ze){Be(ze)}),xD(function(){var Je;return[Ke.current,(Je=at.current)===null||Je===void 0?void 0:Je.getPopupElement()]},St,Be,!!Nt);var On=u.useMemo(function(){return Z(Z({},e),{},{notFoundContent:C,open:De,triggerOpen:St,id:r,showSearch:Ze,multiple:Ne,toggleOpen:Be})},[e,C,St,De,r,Ze,Ne,Be]),tr=!!B||$,Dr;tr&&(Dr=u.createElement(kd,{className:re("".concat(o,"-arrow"),Y({},"".concat(o,"-arrow-loading"),$)),customizeIcon:B,customizeIconProps:{loading:$,searchValue:qe,open:De,focused:Pe,showSearch:Ze}}));var Da=function(){var ze;x==null||x(),(ze=Ue.current)===null||ze===void 0||ze.focus(),p([],{type:"clear",values:g}),ut("",!1,!1)},Gn=bD(o,Da,g,_,k,w,qe,b),cn=Gn.allowClear,Br=Gn.clearIcon,ra=u.createElement(I,{ref:Ye}),Kn=re(o,l,Y(Y(Y(Y(Y(Y(Y(Y(Y(Y({},"".concat(o,"-focused"),Pe),"".concat(o,"-multiple"),Ne),"".concat(o,"-single"),!Ne),"".concat(o,"-allow-clear"),_),"".concat(o,"-show-arrow"),tr),"".concat(o,"-disabled"),w),"".concat(o,"-loading"),$),"".concat(o,"-open"),De),"".concat(o,"-customize-input"),Ct),"".concat(o,"-show-search"),Ze)),Ca=u.createElement(s5,{ref:at,disabled:w,prefixCls:o,visible:St,popupElement:ra,animation:H,transitionName:G,dropdownStyle:X,dropdownClassName:U,direction:m,dropdownMatchSelectWidth:J,dropdownRender:te,dropdownAlign:ve,placement:ne,builtinPlacements:le,getPopupContainer:be,empty:y,getTriggerDOMNode:function(ze){return We.current||ze},onPopupVisibleChange:hr,onPopupMouseEnter:cr},Nt?u.cloneElement(Nt,{ref:_n}):u.createElement(WD,Ie({},e,{domRef:We,prefixCls:o,inputElement:Ct,ref:Ue,id:r,prefix:j,showSearch:Ze,autoClearSearchValue:W,mode:b,activeDescendantId:D,tagRender:d,values:g,open:De,onToggleOpen:Be,activeValue:z,searchValue:qe,onSearch:ut,onSearchSubmit:Vt,onRemove:pn,tokenWithEnter:je,onInputBlur:an}))),Bt;return Nt?Bt=Ca:Bt=u.createElement("div",Ie({className:Kn},He,{ref:Ke,onMouseDown:Gt,onKeyDown:Rt,onKeyUp:er,onFocus:et,onBlur:yt}),u.createElement(f5,{visible:Pe&&!De,values:g}),Ca,Dr,cn&&Br),u.createElement(zO.Provider,{value:On},Bt)}),T0=function(){return null};T0.isSelectOptGroup=!0;var M0=function(){return null};M0.isSelectOption=!0;var WO=u.forwardRef(function(e,t){var n=e.height,r=e.offsetY,o=e.offsetX,l=e.children,c=e.prefixCls,d=e.onInnerResize,m=e.innerProps,v=e.rtl,g=e.extra,p={},y={display:"flex",flexDirection:"column"};return r!==void 0&&(p={height:n,position:"relative",overflow:"hidden"},y=Z(Z({},y),{},Y(Y(Y(Y(Y({transform:"translateY(".concat(r,"px)")},v?"marginRight":"marginLeft",-o),"position","absolute"),"left",0),"right",0),"top",0))),u.createElement("div",{style:p},u.createElement(Ro,{onResize:function(C){var x=C.offsetHeight;x&&d&&d()}},u.createElement("div",Ie({style:y,className:re(Y({},"".concat(c,"-holder-inner"),c)),ref:t},m),l,g)))});WO.displayName="Filler";function g5(e){var t=e.children,n=e.setRef,r=u.useCallback(function(o){n(o)},[]);return u.cloneElement(t,{ref:r})}function h5(e,t,n,r,o,l,c,d){var m=d.getKey;return e.slice(t,n+1).map(function(v,g){var p=t+g,y=c(v,p,{style:{width:r},offsetX:o}),E=m(v);return u.createElement(g5,{key:E,setRef:function(x){return l(v,x)}},y)})}function p5(e,t,n){var r=e.length,o=t.length,l,c;if(r===0&&o===0)return null;r<o?(l=e,c=t):(l=t,c=e);var d={__EMPTY_ITEM__:!0};function m(C){return C!==void 0?n(C):d}for(var v=null,g=Math.abs(r-o)!==1,p=0;p<c.length;p+=1){var y=m(l[p]),E=m(c[p]);if(y!==E){v=p,g=g||y!==m(c[p+1]);break}}return v===null?null:{index:v,multiple:g}}function y5(e,t,n){var r=u.useState(e),o=de(r,2),l=o[0],c=o[1],d=u.useState(null),m=de(d,2),v=m[0],g=m[1];return u.useEffect(function(){var p=p5(l||[],e||[],t);(p==null?void 0:p.index)!==void 0&&g(e[p.index]),c(e)},[e]),[v]}var Ax=(typeof navigator>"u"?"undefined":lt(navigator))==="object"&&/Firefox/i.test(navigator.userAgent);const kO=function(e,t,n,r){var o=u.useRef(!1),l=u.useRef(null);function c(){clearTimeout(l.current),o.current=!0,l.current=setTimeout(function(){o.current=!1},50)}var d=u.useRef({top:e,bottom:t,left:n,right:r});return d.current.top=e,d.current.bottom=t,d.current.left=n,d.current.right=r,function(m,v){var g=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!1,p=m?v<0&&d.current.left||v>0&&d.current.right:v<0&&d.current.top||v>0&&d.current.bottom;return g&&p?(clearTimeout(l.current),o.current=!1):(!p||o.current)&&c(),!o.current&&p}};function b5(e,t,n,r,o,l,c){var d=u.useRef(0),m=u.useRef(null),v=u.useRef(null),g=u.useRef(!1),p=kO(t,n,r,o);function y($,O){if(gn.cancel(m.current),!p(!1,O)){var T=$;if(!T._virtualHandled)T._virtualHandled=!0;else return;d.current+=O,v.current=O,Ax||T.preventDefault(),m.current=gn(function(){var R=g.current?10:1;c(d.current*R,!1),d.current=0})}}function E($,O){c(O,!0),Ax||$.preventDefault()}var C=u.useRef(null),x=u.useRef(null);function b($){if(e){gn.cancel(x.current),x.current=gn(function(){C.current=null},2);var O=$.deltaX,T=$.deltaY,R=$.shiftKey,M=O,N=T;(C.current==="sx"||!C.current&&R&&T&&!O)&&(M=T,N=0,C.current="sx");var z=Math.abs(M),P=Math.abs(N);C.current===null&&(C.current=l&&z>P?"x":"y"),C.current==="y"?y($,N):E($,M)}}function w($){e&&(g.current=$.detail===v.current)}return[b,w]}function S5(e,t,n,r){var o=u.useMemo(function(){return[new Map,[]]},[e,n.id,r]),l=de(o,2),c=l[0],d=l[1],m=function(g){var p=arguments.length>1&&arguments[1]!==void 0?arguments[1]:g,y=c.get(g),E=c.get(p);if(y===void 0||E===void 0)for(var C=e.length,x=d.length;x<C;x+=1){var b,w=e[x],$=t(w);c.set($,x);var O=(b=n.get($))!==null&&b!==void 0?b:r;if(d[x]=(d[x-1]||0)+O,$===g&&(y=x),$===p&&(E=x),y!==void 0&&E!==void 0)break}return{top:d[y-1]||0,bottom:d[E]}};return m}var C5=function(){function e(){qn(this,e),Y(this,"maps",void 0),Y(this,"id",0),Y(this,"diffKeys",new Set),this.maps=Object.create(null)}return Wn(e,[{key:"set",value:function(n,r){this.maps[n]=r,this.id+=1,this.diffKeys.add(n)}},{key:"get",value:function(n){return this.maps[n]}},{key:"resetRecord",value:function(){this.diffKeys.clear()}},{key:"getRecord",value:function(){return this.diffKeys}}]),e}();function Ix(e){var t=parseFloat(e);return isNaN(t)?0:t}function x5(e,t,n){var r=u.useState(0),o=de(r,2),l=o[0],c=o[1],d=u.useRef(new Map),m=u.useRef(new C5),v=u.useRef(0);function g(){v.current+=1}function p(){var E=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1;g();var C=function(){var w=!1;d.current.forEach(function($,O){if($&&$.offsetParent){var T=$.offsetHeight,R=getComputedStyle($),M=R.marginTop,N=R.marginBottom,z=Ix(M),P=Ix(N),D=T+z+P;m.current.get(O)!==D&&(m.current.set(O,D),w=!0)}}),w&&c(function($){return $+1})};if(E)C();else{v.current+=1;var x=v.current;Promise.resolve().then(function(){x===v.current&&C()})}}function y(E,C){var x=e(E);d.current.get(x),C?(d.current.set(x,C),p()):d.current.delete(x)}return u.useEffect(function(){return g},[]),[y,p,m.current,l]}var jx=14/15;function E5(e,t,n){var r=u.useRef(!1),o=u.useRef(0),l=u.useRef(0),c=u.useRef(null),d=u.useRef(null),m,v=function(E){if(r.current){var C=Math.ceil(E.touches[0].pageX),x=Math.ceil(E.touches[0].pageY),b=o.current-C,w=l.current-x,$=Math.abs(b)>Math.abs(w);$?o.current=C:l.current=x;var O=n($,$?b:w,!1,E);O&&E.preventDefault(),clearInterval(d.current),O&&(d.current=setInterval(function(){$?b*=jx:w*=jx;var T=Math.floor($?b:w);(!n($,T,!0)||Math.abs(T)<=.1)&&clearInterval(d.current)},16))}},g=function(){r.current=!1,m()},p=function(E){m(),E.touches.length===1&&!r.current&&(r.current=!0,o.current=Math.ceil(E.touches[0].pageX),l.current=Math.ceil(E.touches[0].pageY),c.current=E.target,c.current.addEventListener("touchmove",v,{passive:!1}),c.current.addEventListener("touchend",g,{passive:!0}))};m=function(){c.current&&(c.current.removeEventListener("touchmove",v),c.current.removeEventListener("touchend",g))},Wt(function(){return e&&t.current.addEventListener("touchstart",p,{passive:!0}),function(){var y;(y=t.current)===null||y===void 0||y.removeEventListener("touchstart",p),m(),clearInterval(d.current)}},[e])}function Px(e){return Math.floor(Math.pow(e,.5))}function xp(e,t){var n="touches"in e?e.touches[0]:e;return n[t?"pageX":"pageY"]-window[t?"scrollX":"scrollY"]}function w5(e,t,n){u.useEffect(function(){var r=t.current;if(e&&r){var o=!1,l,c,d=function(){gn.cancel(l)},m=function y(){d(),l=gn(function(){n(c),y()})},v=function(E){if(!(E.target.draggable||E.button!==0)){var C=E;C._virtualHandled||(C._virtualHandled=!0,o=!0)}},g=function(){o=!1,d()},p=function(E){if(o){var C=xp(E,!1),x=r.getBoundingClientRect(),b=x.top,w=x.bottom;if(C<=b){var $=b-C;c=-Px($),m()}else if(C>=w){var O=C-w;c=Px(O),m()}else d()}};return r.addEventListener("mousedown",v),r.ownerDocument.addEventListener("mouseup",g),r.ownerDocument.addEventListener("mousemove",p),function(){r.removeEventListener("mousedown",v),r.ownerDocument.removeEventListener("mouseup",g),r.ownerDocument.removeEventListener("mousemove",p),d()}}},[e])}var $5=10;function O5(e,t,n,r,o,l,c,d){var m=u.useRef(),v=u.useState(null),g=de(v,2),p=g[0],y=g[1];return Wt(function(){if(p&&p.times<$5){if(!e.current){y(function(k){return Z({},k)});return}l();var E=p.targetAlign,C=p.originAlign,x=p.index,b=p.offset,w=e.current.clientHeight,$=!1,O=E,T=null;if(w){for(var R=E||C,M=0,N=0,z=0,P=Math.min(t.length-1,x),D=0;D<=P;D+=1){var F=o(t[D]);N=M;var W=n.get(F);z=N+(W===void 0?r:W),M=z}for(var V=R==="top"?b:w-b,q=P;q>=0;q-=1){var K=o(t[q]),_=n.get(K);if(_===void 0){$=!0;break}if(V-=_,V<=0)break}switch(R){case"top":T=N-b;break;case"bottom":T=z-w+b;break;default:{var j=e.current.scrollTop,B=j+w;N<j?O="top":z>B&&(O="bottom")}}T!==null&&c(T),T!==p.lastTop&&($=!0)}$&&y(Z(Z({},p),{},{times:p.times+1,targetAlign:O,lastTop:T}))}},[p,e.current]),function(E){if(E==null){d();return}if(gn.cancel(m.current),typeof E=="number")c(E);else if(E&&lt(E)==="object"){var C,x=E.align;"index"in E?C=E.index:C=t.findIndex(function($){return o($)===E.key});var b=E.offset,w=b===void 0?0:b;y({times:0,index:C,offset:w,originAlign:x})}}}var zx=u.forwardRef(function(e,t){var n=e.prefixCls,r=e.rtl,o=e.scrollOffset,l=e.scrollRange,c=e.onStartMove,d=e.onStopMove,m=e.onScroll,v=e.horizontal,g=e.spinSize,p=e.containerSize,y=e.style,E=e.thumbStyle,C=e.showScrollBar,x=u.useState(!1),b=de(x,2),w=b[0],$=b[1],O=u.useState(null),T=de(O,2),R=T[0],M=T[1],N=u.useState(null),z=de(N,2),P=z[0],D=z[1],F=!r,W=u.useRef(),V=u.useRef(),q=u.useState(C),K=de(q,2),_=K[0],j=K[1],B=u.useRef(),k=function(){C===!0||C===!1||(clearTimeout(B.current),j(!0),B.current=setTimeout(function(){j(!1)},3e3))},I=l-p||0,H=p-g||0,G=u.useMemo(function(){if(o===0||I===0)return 0;var fe=o/I;return fe*H},[o,I,H]),X=function(ie){ie.stopPropagation(),ie.preventDefault()},U=u.useRef({top:G,dragging:w,pageY:R,startTop:P});U.current={top:G,dragging:w,pageY:R,startTop:P};var J=function(ie){$(!0),M(xp(ie,v)),D(U.current.top),c(),ie.stopPropagation(),ie.preventDefault()};u.useEffect(function(){var fe=function(se){se.preventDefault()},ie=W.current,pe=V.current;return ie.addEventListener("touchstart",fe,{passive:!1}),pe.addEventListener("touchstart",J,{passive:!1}),function(){ie.removeEventListener("touchstart",fe),pe.removeEventListener("touchstart",J)}},[]);var te=u.useRef();te.current=I;var ve=u.useRef();ve.current=H,u.useEffect(function(){if(w){var fe,ie=function(se){var ye=U.current,$e=ye.dragging,we=ye.pageY,Ne=ye.startTop;gn.cancel(fe);var Ze=W.current.getBoundingClientRect(),He=p/(v?Ze.width:Ze.height);if($e){var Xe=(xp(se,v)-we)*He,xe=Ne;!F&&v?xe-=Xe:xe+=Xe;var Me=te.current,Ve=ve.current,Ke=Ve?xe/Ve:0,We=Math.ceil(Ke*Me);We=Math.max(We,0),We=Math.min(We,Me),fe=gn(function(){m(We,v)})}},pe=function(){$(!1),d()};return window.addEventListener("mousemove",ie,{passive:!0}),window.addEventListener("touchmove",ie,{passive:!0}),window.addEventListener("mouseup",pe,{passive:!0}),window.addEventListener("touchend",pe,{passive:!0}),function(){window.removeEventListener("mousemove",ie),window.removeEventListener("touchmove",ie),window.removeEventListener("mouseup",pe),window.removeEventListener("touchend",pe),gn.cancel(fe)}}},[w]),u.useEffect(function(){return k(),function(){clearTimeout(B.current)}},[o]),u.useImperativeHandle(t,function(){return{delayHidden:k}});var ne="".concat(n,"-scrollbar"),le={position:"absolute",visibility:_?null:"hidden"},be={position:"absolute",background:"rgba(0, 0, 0, 0.5)",borderRadius:99,cursor:"pointer",userSelect:"none"};return v?(le.height=8,le.left=0,le.right=0,le.bottom=0,be.height="100%",be.width=g,F?be.left=G:be.right=G):(le.width=8,le.top=0,le.bottom=0,F?le.right=0:le.left=0,be.width="100%",be.height=g,be.top=G),u.createElement("div",{ref:W,className:re(ne,Y(Y(Y({},"".concat(ne,"-horizontal"),v),"".concat(ne,"-vertical"),!v),"".concat(ne,"-visible"),_)),style:Z(Z({},le),y),onMouseDown:X,onMouseMove:k},u.createElement("div",{ref:V,className:re("".concat(ne,"-thumb"),Y({},"".concat(ne,"-thumb-moving"),w)),style:Z(Z({},be),E),onMouseDown:J}))}),R5=20;function Dx(){var e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:0,t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:0,n=e/t*e;return isNaN(n)&&(n=0),n=Math.max(n,R5),Math.floor(n)}var T5=["prefixCls","className","height","itemHeight","fullHeight","style","data","children","itemKey","virtual","direction","scrollWidth","component","onScroll","onVirtualScroll","onVisibleChange","innerProps","extraRender","styles","showScrollBar"],M5=[],_5={overflowY:"auto",overflowAnchor:"none"};function N5(e,t){var n=e.prefixCls,r=n===void 0?"rc-virtual-list":n,o=e.className,l=e.height,c=e.itemHeight,d=e.fullHeight,m=d===void 0?!0:d,v=e.style,g=e.data,p=e.children,y=e.itemKey,E=e.virtual,C=e.direction,x=e.scrollWidth,b=e.component,w=b===void 0?"div":b,$=e.onScroll,O=e.onVirtualScroll,T=e.onVisibleChange,R=e.innerProps,M=e.extraRender,N=e.styles,z=e.showScrollBar,P=z===void 0?"optional":z,D=wt(e,T5),F=u.useCallback(function(Ae){return typeof y=="function"?y(Ae):Ae==null?void 0:Ae[y]},[y]),W=x5(F),V=de(W,4),q=V[0],K=V[1],_=V[2],j=V[3],B=!!(E!==!1&&l&&c),k=u.useMemo(function(){return Object.values(_.maps).reduce(function(Ae,et){return Ae+et},0)},[_.id,_.maps]),I=B&&g&&(Math.max(c*g.length,k)>l||!!x),H=C==="rtl",G=re(r,Y({},"".concat(r,"-rtl"),H),o),X=g||M5,U=u.useRef(),J=u.useRef(),te=u.useRef(),ve=u.useState(0),ne=de(ve,2),le=ne[0],be=ne[1],fe=u.useState(0),ie=de(fe,2),pe=ie[0],Re=ie[1],se=u.useState(!1),ye=de(se,2),$e=ye[0],we=ye[1],Ne=function(){we(!0)},Ze=function(){we(!1)},He={getKey:F};function Xe(Ae){be(function(et){var yt;typeof Ae=="function"?yt=Ae(et):yt=Ae;var xt=Ft(yt);return U.current.scrollTop=xt,xt})}var xe=u.useRef({start:0,end:X.length}),Me=u.useRef(),Ve=y5(X,F),Ke=de(Ve,1),We=Ke[0];Me.current=We;var at=u.useMemo(function(){if(!B)return{scrollHeight:void 0,start:0,end:X.length-1,offset:void 0};if(!I){var Ae;return{scrollHeight:((Ae=J.current)===null||Ae===void 0?void 0:Ae.offsetHeight)||0,start:0,end:X.length-1,offset:void 0}}for(var et=0,yt,xt,Gt,kn=X.length,Pn=0;Pn<kn;Pn+=1){var Ln=X[Pn],cr=F(Ln),hr=_.get(cr),On=et+(hr===void 0?c:hr);On>=le&&yt===void 0&&(yt=Pn,xt=et),On>le+l&&Gt===void 0&&(Gt=Pn),et=On}return yt===void 0&&(yt=0,xt=0,Gt=Math.ceil(l/c)),Gt===void 0&&(Gt=X.length-1),Gt=Math.min(Gt+1,X.length-1),{scrollHeight:et,start:yt,end:Gt,offset:xt}},[I,B,le,X,j,l]),Ue=at.scrollHeight,Ye=at.start,st=at.end,ht=at.offset;xe.current.start=Ye,xe.current.end=st,u.useLayoutEffect(function(){var Ae=_.getRecord();if(Ae.size===1){var et=Array.from(Ae)[0],yt=X[Ye];if(yt){var xt=F(yt);if(xt===et){var Gt=_.get(et),kn=Gt-c;Xe(function(Pn){return Pn+kn})}}}_.resetRecord()},[Ue]);var Qe=u.useState({width:0,height:l}),Pe=de(Qe,2),Fe=Pe[0],_e=Pe[1],qe=function(et){_e({width:et.offsetWidth,height:et.offsetHeight})},Ct=u.useRef(),Nt=u.useRef(),_n=u.useMemo(function(){return Dx(Fe.width,x)},[Fe.width,x]),bn=u.useMemo(function(){return Dx(Fe.height,Ue)},[Fe.height,Ue]),$n=Ue-l,sn=u.useRef($n);sn.current=$n;function Ft(Ae){var et=Ae;return Number.isNaN(sn.current)||(et=Math.min(et,sn.current)),et=Math.max(et,0),et}var nn=le<=0,Et=le>=$n,nt=pe<=0,ke=pe>=x,De=kO(nn,Et,nt,ke),pt=function(){return{x:H?-pe:pe,y:le}},St=u.useRef(pt()),Be=jn(function(Ae){if(O){var et=Z(Z({},pt()),Ae);(St.current.x!==et.x||St.current.y!==et.y)&&(O(et),St.current=et)}});function je(Ae,et){var yt=Ae;et?(ri.flushSync(function(){Re(yt)}),Be()):Xe(yt)}function rt(Ae){var et=Ae.currentTarget.scrollTop;et!==le&&Xe(et),$==null||$(Ae),Be()}var ot=function(et){var yt=et,xt=x?x-Fe.width:0;return yt=Math.max(yt,0),yt=Math.min(yt,xt),yt},tt=jn(function(Ae,et){et?(ri.flushSync(function(){Re(function(yt){var xt=yt+(H?-Ae:Ae);return ot(xt)})}),Be()):Xe(function(yt){var xt=yt+Ae;return xt})}),ut=b5(B,nn,Et,nt,ke,!!x,tt),Vt=de(ut,2),Tt=Vt[0],At=Vt[1];E5(B,U,function(Ae,et,yt,xt){var Gt=xt;return De(Ae,et,yt)?!1:!Gt||!Gt._virtualHandled?(Gt&&(Gt._virtualHandled=!0),Tt({preventDefault:function(){},deltaX:Ae?et:0,deltaY:Ae?0:et}),!0):!1}),w5(I,U,function(Ae){Xe(function(et){return et+Ae})}),Wt(function(){function Ae(yt){var xt=nn&&yt.detail<0,Gt=Et&&yt.detail>0;B&&!xt&&!Gt&&yt.preventDefault()}var et=U.current;return et.addEventListener("wheel",Tt,{passive:!1}),et.addEventListener("DOMMouseScroll",At,{passive:!0}),et.addEventListener("MozMousePixelScroll",Ae,{passive:!1}),function(){et.removeEventListener("wheel",Tt),et.removeEventListener("DOMMouseScroll",At),et.removeEventListener("MozMousePixelScroll",Ae)}},[B,nn,Et]),Wt(function(){if(x){var Ae=ot(pe);Re(Ae),Be({x:Ae})}},[Fe.width,x]);var It=function(){var et,yt;(et=Ct.current)===null||et===void 0||et.delayHidden(),(yt=Nt.current)===null||yt===void 0||yt.delayHidden()},Qt=O5(U,X,_,c,F,function(){return K(!0)},Xe,It);u.useImperativeHandle(t,function(){return{nativeElement:te.current,getScrollInfo:pt,scrollTo:function(et){function yt(xt){return xt&&lt(xt)==="object"&&("left"in xt||"top"in xt)}yt(et)?(et.left!==void 0&&Re(ot(et.left)),Qt(et.top)):Qt(et)}}}),Wt(function(){if(T){var Ae=X.slice(Ye,st+1);T(Ae,X)}},[Ye,st,X]);var rn=S5(X,F,_,c),Rt=M==null?void 0:M({start:Ye,end:st,virtual:I,offsetX:pe,offsetY:ht,rtl:H,getSize:rn}),er=h5(X,Ye,st,x,pe,q,p,He),pn=null;l&&(pn=Z(Y({},m?"height":"maxHeight",l),_5),B&&(pn.overflowY="hidden",x&&(pn.overflowX="hidden"),$e&&(pn.pointerEvents="none")));var an={};return H&&(an.dir="rtl"),u.createElement("div",Ie({ref:te,style:Z(Z({},v),{},{position:"relative"}),className:G},an,D),u.createElement(Ro,{onResize:qe},u.createElement(w,{className:"".concat(r,"-holder"),style:pn,ref:U,onScroll:rt,onMouseEnter:It},u.createElement(WO,{prefixCls:r,height:Ue,offsetX:pe,offsetY:ht,scrollWidth:x,onInnerResize:K,ref:J,innerProps:R,rtl:H,extra:Rt},er))),I&&Ue>l&&u.createElement(zx,{ref:Ct,prefixCls:r,scrollOffset:le,scrollRange:Ue,rtl:H,onScroll:je,onStartMove:Ne,onStopMove:Ze,spinSize:bn,containerSize:Fe.height,style:N==null?void 0:N.verticalScrollBar,thumbStyle:N==null?void 0:N.verticalScrollBarThumb,showScrollBar:P}),I&&x>Fe.width&&u.createElement(zx,{ref:Nt,prefixCls:r,scrollOffset:pe,scrollRange:x,rtl:H,onScroll:je,onStartMove:Ne,onStopMove:Ze,spinSize:_n,containerSize:Fe.width,horizontal:!0,style:N==null?void 0:N.horizontalScrollBar,thumbStyle:N==null?void 0:N.horizontalScrollBarThumb,showScrollBar:P}))}var GO=u.forwardRef(N5);GO.displayName="List";function A5(){return/(mac\sos|macintosh)/i.test(navigator.appVersion)}var I5=["disabled","title","children","style","className"];function Bx(e){return typeof e=="string"||typeof e=="number"}var j5=function(t,n){var r=SD(),o=r.prefixCls,l=r.id,c=r.open,d=r.multiple,m=r.mode,v=r.searchValue,g=r.toggleOpen,p=r.notFoundContent,y=r.onPopupScroll,E=u.useContext(R0),C=E.maxCount,x=E.flattenOptions,b=E.onActiveValue,w=E.defaultActiveFirstOption,$=E.onSelect,O=E.menuItemSelectedIcon,T=E.rawValues,R=E.fieldNames,M=E.virtual,N=E.direction,z=E.listHeight,P=E.listItemHeight,D=E.optionRender,F="".concat(o,"-item"),W=qc(function(){return x},[c,x],function(fe,ie){return ie[0]&&fe[1]!==ie[1]}),V=u.useRef(null),q=u.useMemo(function(){return d&&bp(C)&&(T==null?void 0:T.size)>=C},[d,C,T==null?void 0:T.size]),K=function(ie){ie.preventDefault()},_=function(ie){var pe;(pe=V.current)===null||pe===void 0||pe.scrollTo(typeof ie=="number"?{index:ie}:ie)},j=u.useCallback(function(fe){return m==="combobox"?!1:T.has(fe)},[m,Te(T).toString(),T.size]),B=function(ie){for(var pe=arguments.length>1&&arguments[1]!==void 0?arguments[1]:1,Re=W.length,se=0;se<Re;se+=1){var ye=(ie+se*pe+Re)%Re,$e=W[ye]||{},we=$e.group,Ne=$e.data;if(!we&&!(Ne!=null&&Ne.disabled)&&(j(Ne.value)||!q))return ye}return-1},k=u.useState(function(){return B(0)}),I=de(k,2),H=I[0],G=I[1],X=function(ie){var pe=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;G(ie);var Re={source:pe?"keyboard":"mouse"},se=W[ie];if(!se){b(null,-1,Re);return}b(se.value,ie,Re)};u.useEffect(function(){X(w!==!1?B(0):-1)},[W.length,v]);var U=u.useCallback(function(fe){return m==="combobox"?String(fe).toLowerCase()===v.toLowerCase():T.has(fe)},[m,v,Te(T).toString(),T.size]);u.useEffect(function(){var fe=setTimeout(function(){if(!d&&c&&T.size===1){var pe=Array.from(T)[0],Re=W.findIndex(function(se){var ye=se.data;return ye.value===pe});Re!==-1&&(X(Re),_(Re))}});if(c){var ie;(ie=V.current)===null||ie===void 0||ie.scrollTo(void 0)}return function(){return clearTimeout(fe)}},[c,v]);var J=function(ie){ie!==void 0&&$(ie,{selected:!T.has(ie)}),d||g(!1)};if(u.useImperativeHandle(n,function(){return{onKeyDown:function(ie){var pe=ie.which,Re=ie.ctrlKey;switch(pe){case ct.N:case ct.P:case ct.UP:case ct.DOWN:{var se=0;if(pe===ct.UP?se=-1:pe===ct.DOWN?se=1:A5()&&Re&&(pe===ct.N?se=1:pe===ct.P&&(se=-1)),se!==0){var ye=B(H+se,se);_(ye),X(ye,!0)}break}case ct.TAB:case ct.ENTER:{var $e,we=W[H];we&&!(we!=null&&($e=we.data)!==null&&$e!==void 0&&$e.disabled)&&!q?J(we.value):J(void 0),c&&ie.preventDefault();break}case ct.ESC:g(!1),c&&ie.stopPropagation()}},onKeyUp:function(){},scrollTo:function(ie){_(ie)}}}),W.length===0)return u.createElement("div",{role:"listbox",id:"".concat(l,"_list"),className:"".concat(F,"-empty"),onMouseDown:K},p);var te=Object.keys(R).map(function(fe){return R[fe]}),ve=function(ie){return ie.label};function ne(fe,ie){var pe=fe.group;return{role:pe?"presentation":"option",id:"".concat(l,"_list_").concat(ie)}}var le=function(ie){var pe=W[ie];if(!pe)return null;var Re=pe.data||{},se=Re.value,ye=pe.group,$e=Ga(Re,!0),we=ve(pe);return pe?u.createElement("div",Ie({"aria-label":typeof we=="string"&&!ye?we:null},$e,{key:ie},ne(pe,ie),{"aria-selected":U(se)}),se):null},be={role:"listbox",id:"".concat(l,"_list")};return u.createElement(u.Fragment,null,M&&u.createElement("div",Ie({},be,{style:{height:0,width:0,overflow:"hidden"}}),le(H-1),le(H),le(H+1)),u.createElement(GO,{itemKey:"key",ref:V,data:W,height:z,itemHeight:P,fullHeight:!1,onMouseDown:K,onScroll:y,virtual:M,direction:N,innerProps:M?null:be},function(fe,ie){var pe=fe.group,Re=fe.groupOption,se=fe.data,ye=fe.label,$e=fe.value,we=se.key;if(pe){var Ne,Ze=(Ne=se.title)!==null&&Ne!==void 0?Ne:Bx(ye)?ye.toString():void 0;return u.createElement("div",{className:re(F,"".concat(F,"-group"),se.className),title:Ze},ye!==void 0?ye:we)}var He=se.disabled,Xe=se.title;se.children;var xe=se.style,Me=se.className,Ve=wt(se,I5),Ke=Tn(Ve,te),We=j($e),at=He||!We&&q,Ue="".concat(F,"-option"),Ye=re(F,Ue,Me,Y(Y(Y(Y({},"".concat(Ue,"-grouped"),Re),"".concat(Ue,"-active"),H===ie&&!at),"".concat(Ue,"-disabled"),at),"".concat(Ue,"-selected"),We)),st=ve(fe),ht=!O||typeof O=="function"||We,Qe=typeof st=="number"?st:st||$e,Pe=Bx(Qe)?Qe.toString():void 0;return Xe!==void 0&&(Pe=Xe),u.createElement("div",Ie({},Ga(Ke),M?{}:ne(fe,ie),{"aria-selected":U($e),className:Ye,title:Pe,onMouseMove:function(){H===ie||at||X(ie)},onClick:function(){at||J($e)},style:xe}),u.createElement("div",{className:"".concat(Ue,"-content")},typeof D=="function"?D(fe,{index:ie}):Qe),u.isValidElement(O)||We,ht&&u.createElement(kd,{className:"".concat(F,"-option-state"),customizeIcon:O,customizeIconProps:{value:$e,disabled:at,isSelected:We}},We?"✓":null))}))},P5=u.forwardRef(j5);const z5=function(e,t){var n=u.useRef({values:new Map,options:new Map}),r=u.useMemo(function(){var l=n.current,c=l.values,d=l.options,m=e.map(function(p){if(p.label===void 0){var y;return Z(Z({},p),{},{label:(y=c.get(p.value))===null||y===void 0?void 0:y.label})}return p}),v=new Map,g=new Map;return m.forEach(function(p){v.set(p.value,p),g.set(p.value,t.get(p.value)||d.get(p.value))}),n.current.values=v,n.current.options=g,m},[e,t]),o=u.useCallback(function(l){return t.get(l)||n.current.options.get(l)},[t]);return[r,o]};function Xg(e,t){return VO(e).join("").toUpperCase().includes(t)}const D5=function(e,t,n,r,o){return u.useMemo(function(){if(!n||r===!1)return e;var l=t.options,c=t.label,d=t.value,m=[],v=typeof r=="function",g=n.toUpperCase(),p=v?r:function(E,C){return o?Xg(C[o],g):C[l]?Xg(C[c!=="children"?c:"label"],g):Xg(C[d],g)},y=v?function(E){return Sp(E)}:function(E){return E};return e.forEach(function(E){if(E[l]){var C=p(n,y(E));if(C)m.push(E);else{var x=E[l].filter(function(b){return p(n,y(b))});x.length&&m.push(Z(Z({},E),{},Y({},l,x)))}return}p(n,y(E))&&m.push(E)}),m},[e,r,o,n,t])};var Lx=0,B5=Cr();function L5(){var e;return B5?(e=Lx,Lx+=1):e="TEST_OR_SSR",e}function H5(e){var t=u.useState(),n=de(t,2),r=n[0],o=n[1];return u.useEffect(function(){o("rc_select_".concat(L5()))},[]),e||r}var F5=["children","value"],V5=["children"];function U5(e){var t=e,n=t.key,r=t.props,o=r.children,l=r.value,c=wt(r,F5);return Z({key:n,value:l!==void 0?l:n,children:o},c)}function KO(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;return Pr(e).map(function(n,r){if(!u.isValidElement(n)||!n.type)return null;var o=n,l=o.type.isSelectOptGroup,c=o.key,d=o.props,m=d.children,v=wt(d,V5);return t||!l?U5(n):Z(Z({key:"__RC_SELECT_GRP__".concat(c===null?r:c,"__"),label:c},v),{},{options:KO(m)})}).filter(function(n){return n})}var q5=function(t,n,r,o,l){return u.useMemo(function(){var c=t,d=!t;d&&(c=KO(n));var m=new Map,v=new Map,g=function(E,C,x){x&&typeof x=="string"&&E.set(C[x],C)},p=function y(E){for(var C=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,x=0;x<E.length;x+=1){var b=E[x];!b[r.options]||C?(m.set(b[r.value],b),g(v,b,r.label),g(v,b,o),g(v,b,l)):y(b[r.options],!0)}};return p(c),{options:c,valueOptions:m,labelOptions:v}},[t,n,r,o,l])};function Hx(e){var t=u.useRef();t.current=e;var n=u.useCallback(function(){return t.current.apply(t,arguments)},[]);return n}var W5=["id","mode","prefixCls","backfill","fieldNames","inputValue","searchValue","onSearch","autoClearSearchValue","onSelect","onDeselect","dropdownMatchSelectWidth","filterOption","filterSort","optionFilterProp","optionLabelProp","options","optionRender","children","defaultActiveFirstOption","menuItemSelectedIcon","virtual","direction","listHeight","listItemHeight","labelRender","value","defaultValue","labelInValue","onChange","maxCount"],k5=["inputValue"];function G5(e){return!e||lt(e)!=="object"}var K5=u.forwardRef(function(e,t){var n=e.id,r=e.mode,o=e.prefixCls,l=o===void 0?"rc-select":o,c=e.backfill,d=e.fieldNames,m=e.inputValue,v=e.searchValue,g=e.onSearch,p=e.autoClearSearchValue,y=p===void 0?!0:p,E=e.onSelect,C=e.onDeselect,x=e.dropdownMatchSelectWidth,b=x===void 0?!0:x,w=e.filterOption,$=e.filterSort,O=e.optionFilterProp,T=e.optionLabelProp,R=e.options,M=e.optionRender,N=e.children,z=e.defaultActiveFirstOption,P=e.menuItemSelectedIcon,D=e.virtual,F=e.direction,W=e.listHeight,V=W===void 0?200:W,q=e.listItemHeight,K=q===void 0?20:q,_=e.labelRender,j=e.value,B=e.defaultValue,k=e.labelInValue,I=e.onChange,H=e.maxCount,G=wt(e,W5),X=H5(n),U=Cp(r),J=!!(!R&&N),te=u.useMemo(function(){return w===void 0&&r==="combobox"?!1:w},[w,r]),ve=u.useMemo(function(){return qO(d,J)},[JSON.stringify(d),J]),ne=gr("",{value:v!==void 0?v:m,postState:function(je){return je||""}}),le=de(ne,2),be=le[0],fe=le[1],ie=q5(R,N,ve,O,T),pe=ie.valueOptions,Re=ie.labelOptions,se=ie.options,ye=u.useCallback(function(Be){var je=VO(Be);return je.map(function(rt){var ot,tt,ut,Vt,Tt;if(G5(rt))ot=rt;else{var At;ut=rt.key,tt=rt.label,ot=(At=rt.value)!==null&&At!==void 0?At:ut}var It=pe.get(ot);if(It){var Qt;tt===void 0&&(tt=It==null?void 0:It[T||ve.label]),ut===void 0&&(ut=(Qt=It==null?void 0:It.key)!==null&&Qt!==void 0?Qt:ot),Vt=It==null?void 0:It.disabled,Tt=It==null?void 0:It.title}return{label:tt,value:ot,key:ut,disabled:Vt,title:Tt}})},[ve,T,pe]),$e=gr(B,{value:j}),we=de($e,2),Ne=we[0],Ze=we[1],He=u.useMemo(function(){var Be,je=U&&Ne===null?[]:Ne,rt=ye(je);return r==="combobox"&&LD((Be=rt[0])===null||Be===void 0?void 0:Be.value)?[]:rt},[Ne,ye,r,U]),Xe=z5(He,pe),xe=de(Xe,2),Me=xe[0],Ve=xe[1],Ke=u.useMemo(function(){if(!r&&Me.length===1){var Be=Me[0];if(Be.value===null&&(Be.label===null||Be.label===void 0))return[]}return Me.map(function(je){var rt;return Z(Z({},je),{},{label:(rt=typeof _=="function"?_(je):je.label)!==null&&rt!==void 0?rt:je.value})})},[r,Me,_]),We=u.useMemo(function(){return new Set(Me.map(function(Be){return Be.value}))},[Me]);u.useEffect(function(){if(r==="combobox"){var Be,je=(Be=Me[0])===null||Be===void 0?void 0:Be.value;fe(BD(je)?String(je):"")}},[Me]);var at=Hx(function(Be,je){var rt=je??Be;return Y(Y({},ve.value,Be),ve.label,rt)}),Ue=u.useMemo(function(){if(r!=="tags")return se;var Be=Te(se),je=function(ot){return pe.has(ot)};return Te(Me).sort(function(rt,ot){return rt.value<ot.value?-1:1}).forEach(function(rt){var ot=rt.value;je(ot)||Be.push(at(ot,rt.label))}),Be},[at,se,pe,Me,r]),Ye=D5(Ue,ve,be,te,O),st=u.useMemo(function(){return r!=="tags"||!be||Ye.some(function(Be){return Be[O||"value"]===be})||Ye.some(function(Be){return Be[ve.value]===be})?Ye:[at(be)].concat(Te(Ye))},[at,O,r,Ye,be,ve]),ht=function Be(je){var rt=Te(je).sort(function(ot,tt){return $(ot,tt,{searchValue:be})});return rt.map(function(ot){return Array.isArray(ot.options)?Z(Z({},ot),{},{options:ot.options.length>0?Be(ot.options):ot.options}):ot})},Qe=u.useMemo(function(){return $?ht(st):st},[st,$,be]),Pe=u.useMemo(function(){return c5(Qe,{fieldNames:ve,childrenAsData:J})},[Qe,ve,J]),Fe=function(je){var rt=ye(je);if(Ze(rt),I&&(rt.length!==Me.length||rt.some(function(ut,Vt){var Tt;return((Tt=Me[Vt])===null||Tt===void 0?void 0:Tt.value)!==(ut==null?void 0:ut.value)}))){var ot=k?rt:rt.map(function(ut){return ut.value}),tt=rt.map(function(ut){return Sp(Ve(ut.value))});I(U?ot:ot[0],U?tt:tt[0])}},_e=u.useState(null),qe=de(_e,2),Ct=qe[0],Nt=qe[1],_n=u.useState(0),bn=de(_n,2),$n=bn[0],sn=bn[1],Ft=z!==void 0?z:r!=="combobox",nn=u.useCallback(function(Be,je){var rt=arguments.length>2&&arguments[2]!==void 0?arguments[2]:{},ot=rt.source,tt=ot===void 0?"keyboard":ot;sn(je),c&&r==="combobox"&&Be!==null&&tt==="keyboard"&&Nt(String(Be))},[c,r]),Et=function(je,rt,ot){var tt=function(){var pn,an=Ve(je);return[k?{label:an==null?void 0:an[ve.label],value:je,key:(pn=an==null?void 0:an.key)!==null&&pn!==void 0?pn:je}:je,Sp(an)]};if(rt&&E){var ut=tt(),Vt=de(ut,2),Tt=Vt[0],At=Vt[1];E(Tt,At)}else if(!rt&&C&&ot!=="clear"){var It=tt(),Qt=de(It,2),rn=Qt[0],Rt=Qt[1];C(rn,Rt)}},nt=Hx(function(Be,je){var rt,ot=U?je.selected:!0;ot?rt=U?[].concat(Te(Me),[Be]):[Be]:rt=Me.filter(function(tt){return tt.value!==Be}),Fe(rt),Et(Be,ot),r==="combobox"?Nt(""):(!Cp||y)&&(fe(""),Nt(""))}),ke=function(je,rt){Fe(je);var ot=rt.type,tt=rt.values;(ot==="remove"||ot==="clear")&&tt.forEach(function(ut){Et(ut.value,!1,ot)})},De=function(je,rt){if(fe(je),Nt(null),rt.source==="submit"){var ot=(je||"").trim();if(ot){var tt=Array.from(new Set([].concat(Te(We),[ot])));Fe(tt),Et(ot,!0),fe("")}return}rt.source!=="blur"&&(r==="combobox"&&Fe(je),g==null||g(je))},pt=function(je){var rt=je;r!=="tags"&&(rt=je.map(function(tt){var ut=Re.get(tt);return ut==null?void 0:ut.value}).filter(function(tt){return tt!==void 0}));var ot=Array.from(new Set([].concat(Te(We),Te(rt))));Fe(ot),ot.forEach(function(tt){Et(tt,!0)})},St=u.useMemo(function(){var Be=D!==!1&&b!==!1;return Z(Z({},ie),{},{flattenOptions:Pe,onActiveValue:nn,defaultActiveFirstOption:Ft,onSelect:nt,menuItemSelectedIcon:P,rawValues:We,fieldNames:ve,virtual:Be,direction:F,listHeight:V,listItemHeight:K,childrenAsData:J,maxCount:H,optionRender:M})},[H,ie,Pe,nn,Ft,nt,P,We,ve,D,b,F,V,K,J,M]);return u.createElement(R0.Provider,{value:St},u.createElement(v5,Ie({},G,{id:X,prefixCls:l,ref:t,omitDomProps:k5,mode:r,displayValues:Ke,onDisplayValuesChange:ke,direction:F,searchValue:be,onSearch:De,autoClearSearchValue:y,onSearchSplit:pt,dropdownMatchSelectWidth:b,OptionList:P5,emptyOptions:!Pe.length,activeValue:Ct,activeDescendantId:"".concat(X,"_list_").concat($n)})))}),_0=K5;_0.Option=M0;_0.OptGroup=T0;function od(e,t,n){return re({[`${e}-status-success`]:t==="success",[`${e}-status-warning`]:t==="warning",[`${e}-status-error`]:t==="error",[`${e}-status-validating`]:t==="validating",[`${e}-has-feedback`]:n})}const Gd=(e,t)=>t||e,X5=()=>{const[,e]=Rr(),[t]=To("Empty"),r=new Jt(e.colorBgBase).toHsl().l<.5?{opacity:.65}:{};return u.createElement("svg",{style:r,width:"184",height:"152",viewBox:"0 0 184 152",xmlns:"http://www.w3.org/2000/svg"},u.createElement("title",null,(t==null?void 0:t.description)||"Empty"),u.createElement("g",{fill:"none",fillRule:"evenodd"},u.createElement("g",{transform:"translate(24 31.67)"},u.createElement("ellipse",{fillOpacity:".8",fill:"#F5F5F7",cx:"67.797",cy:"106.89",rx:"67.797",ry:"12.668"}),u.createElement("path",{d:"M122.034 69.674L98.109 40.229c-1.148-1.386-2.826-2.225-4.593-2.225h-51.44c-1.766 0-3.444.839-4.592 2.225L13.56 69.674v15.383h108.475V69.674z",fill:"#AEB8C2"}),u.createElement("path",{d:"M101.537 86.214L80.63 61.102c-1.001-1.207-2.507-1.867-4.048-1.867H31.724c-1.54 0-3.047.66-4.048 1.867L6.769 86.214v13.792h94.768V86.214z",fill:"url(#linearGradient-1)",transform:"translate(13.56)"}),u.createElement("path",{d:"M33.83 0h67.933a4 4 0 0 1 4 4v93.344a4 4 0 0 1-4 4H33.83a4 4 0 0 1-4-4V4a4 4 0 0 1 4-4z",fill:"#F5F5F7"}),u.createElement("path",{d:"M42.678 9.953h50.237a2 2 0 0 1 2 2V36.91a2 2 0 0 1-2 2H42.678a2 2 0 0 1-2-2V11.953a2 2 0 0 1 2-2zM42.94 49.767h49.713a2.262 2.262 0 1 1 0 4.524H42.94a2.262 2.262 0 0 1 0-4.524zM42.94 61.53h49.713a2.262 2.262 0 1 1 0 4.525H42.94a2.262 2.262 0 0 1 0-4.525zM121.813 105.032c-.775 3.071-3.497 5.36-6.735 5.36H20.515c-3.238 0-5.96-2.29-6.734-5.36a7.309 7.309 0 0 1-.222-1.79V69.675h26.318c2.907 0 5.25 2.448 5.25 5.42v.04c0 2.971 2.37 5.37 5.277 5.37h34.785c2.907 0 5.277-2.421 5.277-5.393V75.1c0-2.972 2.343-5.426 5.25-5.426h26.318v33.569c0 .617-.077 1.216-.221 1.789z",fill:"#DCE0E6"})),u.createElement("path",{d:"M149.121 33.292l-6.83 2.65a1 1 0 0 1-1.317-1.23l1.937-6.207c-2.589-2.944-4.109-6.534-4.109-10.408C138.802 8.102 148.92 0 161.402 0 173.881 0 184 8.102 184 18.097c0 9.995-10.118 18.097-22.599 18.097-4.528 0-8.744-1.066-12.28-2.902z",fill:"#DCE0E6"}),u.createElement("g",{transform:"translate(149.65 15.383)",fill:"#FFF"},u.createElement("ellipse",{cx:"20.654",cy:"3.167",rx:"2.849",ry:"2.815"}),u.createElement("path",{d:"M5.698 5.63H0L2.898.704zM9.259.704h4.985V5.63H9.259z"}))))},Y5=()=>{const[,e]=Rr(),[t]=To("Empty"),{colorFill:n,colorFillTertiary:r,colorFillQuaternary:o,colorBgContainer:l}=e,{borderColor:c,shadowColor:d,contentColor:m}=u.useMemo(()=>({borderColor:new Jt(n).onBackground(l).toHexString(),shadowColor:new Jt(r).onBackground(l).toHexString(),contentColor:new Jt(o).onBackground(l).toHexString()}),[n,r,o,l]);return u.createElement("svg",{width:"64",height:"41",viewBox:"0 0 64 41",xmlns:"http://www.w3.org/2000/svg"},u.createElement("title",null,(t==null?void 0:t.description)||"Empty"),u.createElement("g",{transform:"translate(0 1)",fill:"none",fillRule:"evenodd"},u.createElement("ellipse",{fill:d,cx:"32",cy:"33",rx:"32",ry:"7"}),u.createElement("g",{fillRule:"nonzero",stroke:c},u.createElement("path",{d:"M55 12.76L44.854 1.258C44.367.474 43.656 0 42.907 0H21.093c-.749 0-1.46.474-1.947 1.257L9 12.761V22h46v-9.24z"}),u.createElement("path",{d:"M41.613 15.931c0-1.605.994-2.93 2.227-2.931H55v18.137C55 33.26 53.68 35 52.05 35h-40.1C10.32 35 9 33.259 9 31.137V13h11.16c1.233 0 2.227 1.323 2.227 2.928v.022c0 1.605 1.005 2.901 2.237 2.901h14.752c1.232 0 2.237-1.308 2.237-2.913v-.007z",fill:m}))))},Q5=e=>{const{componentCls:t,margin:n,marginXS:r,marginXL:o,fontSize:l,lineHeight:c}=e;return{[t]:{marginInline:r,fontSize:l,lineHeight:c,textAlign:"center",[`${t}-image`]:{height:e.emptyImgHeight,marginBottom:r,opacity:e.opacityImage,img:{height:"100%"},svg:{maxWidth:"100%",height:"100%",margin:"auto"}},[`${t}-description`]:{color:e.colorTextDescription},[`${t}-footer`]:{marginTop:n},"&-normal":{marginBlock:o,color:e.colorTextDescription,[`${t}-description`]:{color:e.colorTextDescription},[`${t}-image`]:{height:e.emptyImgHeightMD}},"&-small":{marginBlock:r,color:e.colorTextDescription,[`${t}-image`]:{height:e.emptyImgHeightSM}}}}},Z5=wn("Empty",e=>{const{componentCls:t,controlHeightLG:n,calc:r}=e,o=kt(e,{emptyImgCls:`${t}-img`,emptyImgHeight:r(n).mul(2.5).equal(),emptyImgHeightMD:n,emptyImgHeightSM:r(n).mul(.875).equal()});return[Q5(o)]});var J5=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const XO=u.createElement(X5,null),YO=u.createElement(Y5,null),ho=e=>{const{className:t,rootClassName:n,prefixCls:r,image:o=XO,description:l,children:c,imageStyle:d,style:m,classNames:v,styles:g}=e,p=J5(e,["className","rootClassName","prefixCls","image","description","children","imageStyle","style","classNames","styles"]),{getPrefixCls:y,direction:E,className:C,style:x,classNames:b,styles:w}=zr("empty"),$=y("empty",r),[O,T,R]=Z5($),[M]=To("Empty"),N=typeof l<"u"?l:M==null?void 0:M.description,z=typeof N=="string"?N:"empty";let P=null;return typeof o=="string"?P=u.createElement("img",{alt:z,src:o}):P=o,O(u.createElement("div",Object.assign({className:re(T,R,$,C,{[`${$}-normal`]:o===YO,[`${$}-rtl`]:E==="rtl"},t,n,b.root,v==null?void 0:v.root),style:Object.assign(Object.assign(Object.assign(Object.assign({},w.root),x),g==null?void 0:g.root),m)},p),u.createElement("div",{className:re(`${$}-image`,b.image,v==null?void 0:v.image),style:Object.assign(Object.assign(Object.assign({},d),w.image),g==null?void 0:g.image)},P),N&&u.createElement("div",{className:re(`${$}-description`,b.description,v==null?void 0:v.description),style:Object.assign(Object.assign({},w.description),g==null?void 0:g.description)},N),c&&u.createElement("div",{className:re(`${$}-footer`,b.footer,v==null?void 0:v.footer),style:Object.assign(Object.assign({},w.footer),g==null?void 0:g.footer)},c)))};ho.PRESENTED_IMAGE_DEFAULT=XO;ho.PRESENTED_IMAGE_SIMPLE=YO;const eB=e=>{const{componentName:t}=e,{getPrefixCls:n}=u.useContext(vt),r=n("empty");switch(t){case"Table":case"List":return oe.createElement(ho,{image:ho.PRESENTED_IMAGE_SIMPLE});case"Select":case"TreeSelect":case"Cascader":case"Transfer":case"Mentions":return oe.createElement(ho,{image:ho.PRESENTED_IMAGE_SIMPLE,className:`${r}-small`});case"Table.filter":return null;default:return oe.createElement(ho,null)}},N0=function(e,t){let n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:void 0;var r,o;const{variant:l,[e]:c}=u.useContext(vt),d=u.useContext(cO),m=c==null?void 0:c.variant;let v;typeof t<"u"?v=t:n===!1?v="borderless":v=(o=(r=d??m)!==null&&r!==void 0?r:l)!==null&&o!==void 0?o:"outlined";const g=nj.includes(v);return[v,g]},tB=e=>{const n={overflow:{adjustX:!0,adjustY:!0,shiftY:!0},htmlRegion:e==="scroll"?"scroll":"visible",dynamicInset:!0};return{bottomLeft:Object.assign(Object.assign({},n),{points:["tl","bl"],offset:[0,4]}),bottomRight:Object.assign(Object.assign({},n),{points:["tr","br"],offset:[0,4]}),topLeft:Object.assign(Object.assign({},n),{points:["bl","tl"],offset:[0,-4]}),topRight:Object.assign(Object.assign({},n),{points:["br","tr"],offset:[0,-4]})}};function nB(e,t){return e||tB(t)}const Fx=e=>{const{optionHeight:t,optionFontSize:n,optionLineHeight:r,optionPadding:o}=e;return{position:"relative",display:"block",minHeight:t,padding:o,color:e.colorText,fontWeight:"normal",fontSize:n,lineHeight:r,boxSizing:"border-box"}},rB=e=>{const{antCls:t,componentCls:n}=e,r=`${n}-item`,o=`&${t}-slide-up-enter${t}-slide-up-enter-active`,l=`&${t}-slide-up-appear${t}-slide-up-appear-active`,c=`&${t}-slide-up-leave${t}-slide-up-leave-active`,d=`${n}-dropdown-placement-`,m=`${r}-option-selected`;return[{[`${n}-dropdown`]:Object.assign(Object.assign({},Tr(e)),{position:"absolute",top:-9999,zIndex:e.zIndexPopup,boxSizing:"border-box",padding:e.paddingXXS,overflow:"hidden",fontSize:e.fontSize,fontVariant:"initial",backgroundColor:e.colorBgElevated,borderRadius:e.borderRadiusLG,outline:"none",boxShadow:e.boxShadowSecondary,[`
          ${o}${d}bottomLeft,
          ${l}${d}bottomLeft
        `]:{animationName:H$},[`
          ${o}${d}topLeft,
          ${l}${d}topLeft,
          ${o}${d}topRight,
          ${l}${d}topRight
        `]:{animationName:V$},[`${c}${d}bottomLeft`]:{animationName:F$},[`
          ${c}${d}topLeft,
          ${c}${d}topRight
        `]:{animationName:U$},"&-hidden":{display:"none"},[r]:Object.assign(Object.assign({},Fx(e)),{cursor:"pointer",transition:`background ${e.motionDurationSlow} ease`,borderRadius:e.borderRadiusSM,"&-group":{color:e.colorTextDescription,fontSize:e.fontSizeSM,cursor:"default"},"&-option":{display:"flex","&-content":Object.assign({flex:"auto"},Zf),"&-state":{flex:"none",display:"flex",alignItems:"center"},[`&-active:not(${r}-option-disabled)`]:{backgroundColor:e.optionActiveBg},[`&-selected:not(${r}-option-disabled)`]:{color:e.optionSelectedColor,fontWeight:e.optionSelectedFontWeight,backgroundColor:e.optionSelectedBg,[`${r}-option-state`]:{color:e.colorPrimary}},"&-disabled":{[`&${r}-option-selected`]:{backgroundColor:e.colorBgContainerDisabled},color:e.colorTextDisabled,cursor:"not-allowed"},"&-grouped":{paddingInlineStart:e.calc(e.controlPaddingHorizontal).mul(2).equal()}},"&-empty":Object.assign(Object.assign({},Fx(e)),{color:e.colorTextDisabled})}),[`${m}:has(+ ${m})`]:{borderEndStartRadius:0,borderEndEndRadius:0,[`& + ${m}`]:{borderStartStartRadius:0,borderStartEndRadius:0}},"&-rtl":{direction:"rtl"}})},rd(e,"slide-up"),rd(e,"slide-down"),U1(e,"move-up"),U1(e,"move-down")]},aB=e=>{const{multipleSelectItemHeight:t,paddingXXS:n,lineWidth:r,INTERNAL_FIXED_ITEM_MARGIN:o}=e,l=e.max(e.calc(n).sub(r).equal(),0),c=e.max(e.calc(l).sub(o).equal(),0);return{basePadding:l,containerPadding:c,itemHeight:he(t),itemLineHeight:he(e.calc(t).sub(e.calc(e.lineWidth).mul(2)).equal())}},oB=e=>{const{multipleSelectItemHeight:t,selectHeight:n,lineWidth:r}=e;return e.calc(n).sub(t).div(2).sub(r).equal()},iB=e=>{const{componentCls:t,iconCls:n,borderRadiusSM:r,motionDurationSlow:o,paddingXS:l,multipleItemColorDisabled:c,multipleItemBorderColorDisabled:d,colorIcon:m,colorIconHover:v,INTERNAL_FIXED_ITEM_MARGIN:g}=e;return{[`${t}-selection-overflow`]:{position:"relative",display:"flex",flex:"auto",flexWrap:"wrap",maxWidth:"100%","&-item":{flex:"none",alignSelf:"center",maxWidth:"100%",display:"inline-flex"},[`${t}-selection-item`]:{display:"flex",alignSelf:"center",flex:"none",boxSizing:"border-box",maxWidth:"100%",marginBlock:g,borderRadius:r,cursor:"default",transition:`font-size ${o}, line-height ${o}, height ${o}`,marginInlineEnd:e.calc(g).mul(2).equal(),paddingInlineStart:l,paddingInlineEnd:e.calc(l).div(2).equal(),[`${t}-disabled&`]:{color:c,borderColor:d,cursor:"not-allowed"},"&-content":{display:"inline-block",marginInlineEnd:e.calc(l).div(2).equal(),overflow:"hidden",whiteSpace:"pre",textOverflow:"ellipsis"},"&-remove":Object.assign(Object.assign({},es()),{display:"inline-flex",alignItems:"center",color:m,fontWeight:"bold",fontSize:10,lineHeight:"inherit",cursor:"pointer",[`> ${n}`]:{verticalAlign:"-0.2em"},"&:hover":{color:v}})}}}},lB=(e,t)=>{const{componentCls:n,INTERNAL_FIXED_ITEM_MARGIN:r}=e,o=`${n}-selection-overflow`,l=e.multipleSelectItemHeight,c=oB(e),d=t?`${n}-${t}`:"",m=aB(e);return{[`${n}-multiple${d}`]:Object.assign(Object.assign({},iB(e)),{[`${n}-selector`]:{display:"flex",alignItems:"center",width:"100%",height:"100%",paddingInline:m.basePadding,paddingBlock:m.containerPadding,borderRadius:e.borderRadius,[`${n}-disabled&`]:{background:e.multipleSelectorBgDisabled,cursor:"not-allowed"},"&:after":{display:"inline-block",width:0,margin:`${he(r)} 0`,lineHeight:he(l),visibility:"hidden",content:'"\\a0"'}},[`${n}-selection-item`]:{height:m.itemHeight,lineHeight:he(m.itemLineHeight)},[`${n}-selection-wrap`]:{alignSelf:"flex-start","&:after":{lineHeight:he(l),marginBlock:r}},[`${n}-prefix`]:{marginInlineStart:e.calc(e.inputPaddingHorizontalBase).sub(m.basePadding).equal()},[`${o}-item + ${o}-item,
        ${n}-prefix + ${n}-selection-wrap
      `]:{[`${n}-selection-search`]:{marginInlineStart:0},[`${n}-selection-placeholder`]:{insetInlineStart:0}},[`${o}-item-suffix`]:{minHeight:m.itemHeight,marginBlock:r},[`${n}-selection-search`]:{display:"inline-flex",position:"relative",maxWidth:"100%",marginInlineStart:e.calc(e.inputPaddingHorizontalBase).sub(c).equal(),"\n          &-input,\n          &-mirror\n        ":{height:l,fontFamily:e.fontFamily,lineHeight:he(l),transition:`all ${e.motionDurationSlow}`},"&-input":{width:"100%",minWidth:4.1},"&-mirror":{position:"absolute",top:0,insetInlineStart:0,insetInlineEnd:"auto",zIndex:999,whiteSpace:"pre",visibility:"hidden"}},[`${n}-selection-placeholder`]:{position:"absolute",top:"50%",insetInlineStart:e.calc(e.inputPaddingHorizontalBase).sub(m.basePadding).equal(),insetInlineEnd:e.inputPaddingHorizontalBase,transform:"translateY(-50%)",transition:`all ${e.motionDurationSlow}`}})}};function Yg(e,t){const{componentCls:n}=e,r=t?`${n}-${t}`:"",o={[`${n}-multiple${r}`]:{fontSize:e.fontSize,[`${n}-selector`]:{[`${n}-show-search&`]:{cursor:"text"}},[`
        &${n}-show-arrow ${n}-selector,
        &${n}-allow-clear ${n}-selector
      `]:{paddingInlineEnd:e.calc(e.fontSizeIcon).add(e.controlPaddingHorizontal).equal()}}};return[lB(e,t),o]}const sB=e=>{const{componentCls:t}=e,n=kt(e,{selectHeight:e.controlHeightSM,multipleSelectItemHeight:e.multipleItemHeightSM,borderRadius:e.borderRadiusSM,borderRadiusSM:e.borderRadiusXS}),r=kt(e,{fontSize:e.fontSizeLG,selectHeight:e.controlHeightLG,multipleSelectItemHeight:e.multipleItemHeightLG,borderRadius:e.borderRadiusLG,borderRadiusSM:e.borderRadius});return[Yg(e),Yg(n,"sm"),{[`${t}-multiple${t}-sm`]:{[`${t}-selection-placeholder`]:{insetInline:e.calc(e.controlPaddingHorizontalSM).sub(e.lineWidth).equal()},[`${t}-selection-search`]:{marginInlineStart:2}}},Yg(r,"lg")]};function Qg(e,t){const{componentCls:n,inputPaddingHorizontalBase:r,borderRadius:o}=e,l=e.calc(e.controlHeight).sub(e.calc(e.lineWidth).mul(2)).equal(),c=t?`${n}-${t}`:"";return{[`${n}-single${c}`]:{fontSize:e.fontSize,height:e.controlHeight,[`${n}-selector`]:Object.assign(Object.assign({},Tr(e,!0)),{display:"flex",borderRadius:o,flex:"1 1 auto",[`${n}-selection-wrap:after`]:{lineHeight:he(l)},[`${n}-selection-search`]:{position:"absolute",inset:0,width:"100%","&-input":{width:"100%",WebkitAppearance:"textfield"}},[`
          ${n}-selection-item,
          ${n}-selection-placeholder
        `]:{display:"block",padding:0,lineHeight:he(l),transition:`all ${e.motionDurationSlow}, visibility 0s`,alignSelf:"center"},[`${n}-selection-placeholder`]:{transition:"none",pointerEvents:"none"},[["&:after",`${n}-selection-item:empty:after`,`${n}-selection-placeholder:empty:after`].join(",")]:{display:"inline-block",width:0,visibility:"hidden",content:'"\\a0"'}}),[`
        &${n}-show-arrow ${n}-selection-item,
        &${n}-show-arrow ${n}-selection-search,
        &${n}-show-arrow ${n}-selection-placeholder
      `]:{paddingInlineEnd:e.showArrowPaddingInlineEnd},[`&${n}-open ${n}-selection-item`]:{color:e.colorTextPlaceholder},[`&:not(${n}-customize-input)`]:{[`${n}-selector`]:{width:"100%",height:"100%",alignItems:"center",padding:`0 ${he(r)}`,[`${n}-selection-search-input`]:{height:l,fontSize:e.fontSize},"&:after":{lineHeight:he(l)}}},[`&${n}-customize-input`]:{[`${n}-selector`]:{"&:after":{display:"none"},[`${n}-selection-search`]:{position:"static",width:"100%"},[`${n}-selection-placeholder`]:{position:"absolute",insetInlineStart:0,insetInlineEnd:0,padding:`0 ${he(r)}`,"&:after":{display:"none"}}}}}}}function cB(e){const{componentCls:t}=e,n=e.calc(e.controlPaddingHorizontalSM).sub(e.lineWidth).equal();return[Qg(e),Qg(kt(e,{controlHeight:e.controlHeightSM,borderRadius:e.borderRadiusSM}),"sm"),{[`${t}-single${t}-sm`]:{[`&:not(${t}-customize-input)`]:{[`${t}-selector`]:{padding:`0 ${he(n)}`},[`&${t}-show-arrow ${t}-selection-search`]:{insetInlineEnd:e.calc(n).add(e.calc(e.fontSize).mul(1.5)).equal()},[`
            &${t}-show-arrow ${t}-selection-item,
            &${t}-show-arrow ${t}-selection-placeholder
          `]:{paddingInlineEnd:e.calc(e.fontSize).mul(1.5).equal()}}}},Qg(kt(e,{controlHeight:e.singleItemHeightLG,fontSize:e.fontSizeLG,borderRadius:e.borderRadiusLG}),"lg")]}const uB=e=>{const{fontSize:t,lineHeight:n,lineWidth:r,controlHeight:o,controlHeightSM:l,controlHeightLG:c,paddingXXS:d,controlPaddingHorizontal:m,zIndexPopupBase:v,colorText:g,fontWeightStrong:p,controlItemBgActive:y,controlItemBgHover:E,colorBgContainer:C,colorFillSecondary:x,colorBgContainerDisabled:b,colorTextDisabled:w,colorPrimaryHover:$,colorPrimary:O,controlOutline:T}=e,R=d*2,M=r*2,N=Math.min(o-R,o-M),z=Math.min(l-R,l-M),P=Math.min(c-R,c-M);return{INTERNAL_FIXED_ITEM_MARGIN:Math.floor(d/2),zIndexPopup:v+50,optionSelectedColor:g,optionSelectedFontWeight:p,optionSelectedBg:y,optionActiveBg:E,optionPadding:`${(o-t*n)/2}px ${m}px`,optionFontSize:t,optionLineHeight:n,optionHeight:o,selectorBg:C,clearBg:C,singleItemHeightLG:c,multipleItemBg:x,multipleItemBorderColor:"transparent",multipleItemHeight:N,multipleItemHeightSM:z,multipleItemHeightLG:P,multipleSelectorBgDisabled:b,multipleItemColorDisabled:w,multipleItemBorderColorDisabled:"transparent",showArrowPaddingInlineEnd:Math.ceil(e.fontSize*1.25),hoverBorderColor:$,activeBorderColor:O,activeOutlineColor:T,selectAffixPadding:d}},QO=(e,t)=>{const{componentCls:n,antCls:r,controlOutlineWidth:o}=e;return{[`&:not(${n}-customize-input) ${n}-selector`]:{border:`${he(e.lineWidth)} ${e.lineType} ${t.borderColor}`,background:e.selectorBg},[`&:not(${n}-disabled):not(${n}-customize-input):not(${r}-pagination-size-changer)`]:{[`&:hover ${n}-selector`]:{borderColor:t.hoverBorderHover},[`${n}-focused& ${n}-selector`]:{borderColor:t.activeBorderColor,boxShadow:`0 0 0 ${he(o)} ${t.activeOutlineColor}`,outline:0},[`${n}-prefix`]:{color:t.color}}}},Vx=(e,t)=>({[`&${e.componentCls}-status-${t.status}`]:Object.assign({},QO(e,t))}),fB=e=>({"&-outlined":Object.assign(Object.assign(Object.assign(Object.assign({},QO(e,{borderColor:e.colorBorder,hoverBorderHover:e.hoverBorderColor,activeBorderColor:e.activeBorderColor,activeOutlineColor:e.activeOutlineColor,color:e.colorText})),Vx(e,{status:"error",borderColor:e.colorError,hoverBorderHover:e.colorErrorHover,activeBorderColor:e.colorError,activeOutlineColor:e.colorErrorOutline,color:e.colorError})),Vx(e,{status:"warning",borderColor:e.colorWarning,hoverBorderHover:e.colorWarningHover,activeBorderColor:e.colorWarning,activeOutlineColor:e.colorWarningOutline,color:e.colorWarning})),{[`&${e.componentCls}-disabled`]:{[`&:not(${e.componentCls}-customize-input) ${e.componentCls}-selector`]:{background:e.colorBgContainerDisabled,color:e.colorTextDisabled}},[`&${e.componentCls}-multiple ${e.componentCls}-selection-item`]:{background:e.multipleItemBg,border:`${he(e.lineWidth)} ${e.lineType} ${e.multipleItemBorderColor}`}})}),ZO=(e,t)=>{const{componentCls:n,antCls:r}=e;return{[`&:not(${n}-customize-input) ${n}-selector`]:{background:t.bg,border:`${he(e.lineWidth)} ${e.lineType} transparent`,color:t.color},[`&:not(${n}-disabled):not(${n}-customize-input):not(${r}-pagination-size-changer)`]:{[`&:hover ${n}-selector`]:{background:t.hoverBg},[`${n}-focused& ${n}-selector`]:{background:e.selectorBg,borderColor:t.activeBorderColor,outline:0}}}},Ux=(e,t)=>({[`&${e.componentCls}-status-${t.status}`]:Object.assign({},ZO(e,t))}),dB=e=>({"&-filled":Object.assign(Object.assign(Object.assign(Object.assign({},ZO(e,{bg:e.colorFillTertiary,hoverBg:e.colorFillSecondary,activeBorderColor:e.activeBorderColor,color:e.colorText})),Ux(e,{status:"error",bg:e.colorErrorBg,hoverBg:e.colorErrorBgHover,activeBorderColor:e.colorError,color:e.colorError})),Ux(e,{status:"warning",bg:e.colorWarningBg,hoverBg:e.colorWarningBgHover,activeBorderColor:e.colorWarning,color:e.colorWarning})),{[`&${e.componentCls}-disabled`]:{[`&:not(${e.componentCls}-customize-input) ${e.componentCls}-selector`]:{borderColor:e.colorBorder,background:e.colorBgContainerDisabled,color:e.colorTextDisabled}},[`&${e.componentCls}-multiple ${e.componentCls}-selection-item`]:{background:e.colorBgContainer,border:`${he(e.lineWidth)} ${e.lineType} ${e.colorSplit}`}})}),mB=e=>({"&-borderless":{[`${e.componentCls}-selector`]:{background:"transparent",border:`${he(e.lineWidth)} ${e.lineType} transparent`},[`&${e.componentCls}-disabled`]:{[`&:not(${e.componentCls}-customize-input) ${e.componentCls}-selector`]:{color:e.colorTextDisabled}},[`&${e.componentCls}-multiple ${e.componentCls}-selection-item`]:{background:e.multipleItemBg,border:`${he(e.lineWidth)} ${e.lineType} ${e.multipleItemBorderColor}`},[`&${e.componentCls}-status-error`]:{[`${e.componentCls}-prefix, ${e.componentCls}-selection-item`]:{color:e.colorError}},[`&${e.componentCls}-status-warning`]:{[`${e.componentCls}-prefix, ${e.componentCls}-selection-item`]:{color:e.colorWarning}}}}),JO=(e,t)=>{const{componentCls:n,antCls:r}=e;return{[`&:not(${n}-customize-input) ${n}-selector`]:{borderWidth:`0 0 ${he(e.lineWidth)} 0`,borderStyle:`none none ${e.lineType} none`,borderColor:t.borderColor,background:e.selectorBg,borderRadius:0},[`&:not(${n}-disabled):not(${n}-customize-input):not(${r}-pagination-size-changer)`]:{[`&:hover ${n}-selector`]:{borderColor:t.hoverBorderHover},[`${n}-focused& ${n}-selector`]:{borderColor:t.activeBorderColor,outline:0},[`${n}-prefix`]:{color:t.color}}}},qx=(e,t)=>({[`&${e.componentCls}-status-${t.status}`]:Object.assign({},JO(e,t))}),vB=e=>({"&-underlined":Object.assign(Object.assign(Object.assign(Object.assign({},JO(e,{borderColor:e.colorBorder,hoverBorderHover:e.hoverBorderColor,activeBorderColor:e.activeBorderColor,activeOutlineColor:e.activeOutlineColor,color:e.colorText})),qx(e,{status:"error",borderColor:e.colorError,hoverBorderHover:e.colorErrorHover,activeBorderColor:e.colorError,activeOutlineColor:e.colorErrorOutline,color:e.colorError})),qx(e,{status:"warning",borderColor:e.colorWarning,hoverBorderHover:e.colorWarningHover,activeBorderColor:e.colorWarning,activeOutlineColor:e.colorWarningOutline,color:e.colorWarning})),{[`&${e.componentCls}-disabled`]:{[`&:not(${e.componentCls}-customize-input) ${e.componentCls}-selector`]:{color:e.colorTextDisabled}},[`&${e.componentCls}-multiple ${e.componentCls}-selection-item`]:{background:e.multipleItemBg,border:`${he(e.lineWidth)} ${e.lineType} ${e.multipleItemBorderColor}`}})}),gB=e=>({[e.componentCls]:Object.assign(Object.assign(Object.assign(Object.assign({},fB(e)),dB(e)),mB(e)),vB(e))}),hB=e=>{const{componentCls:t}=e;return{position:"relative",transition:`all ${e.motionDurationMid} ${e.motionEaseInOut}`,input:{cursor:"pointer"},[`${t}-show-search&`]:{cursor:"text",input:{cursor:"auto",color:"inherit",height:"100%"}},[`${t}-disabled&`]:{cursor:"not-allowed",input:{cursor:"not-allowed"}}}},pB=e=>{const{componentCls:t}=e;return{[`${t}-selection-search-input`]:{margin:0,padding:0,background:"transparent",border:"none",outline:"none",appearance:"none",fontFamily:"inherit","&::-webkit-search-cancel-button":{display:"none",appearance:"none"}}}},yB=e=>{const{antCls:t,componentCls:n,inputPaddingHorizontalBase:r,iconCls:o}=e;return{[n]:Object.assign(Object.assign({},Tr(e)),{position:"relative",display:"inline-flex",cursor:"pointer",[`&:not(${n}-customize-input) ${n}-selector`]:Object.assign(Object.assign({},hB(e)),pB(e)),[`${n}-selection-item`]:Object.assign(Object.assign({flex:1,fontWeight:"normal",position:"relative",userSelect:"none"},Zf),{[`> ${t}-typography`]:{display:"inline"}}),[`${n}-selection-placeholder`]:Object.assign(Object.assign({},Zf),{flex:1,color:e.colorTextPlaceholder,pointerEvents:"none"}),[`${n}-arrow`]:Object.assign(Object.assign({},es()),{position:"absolute",top:"50%",insetInlineStart:"auto",insetInlineEnd:r,height:e.fontSizeIcon,marginTop:e.calc(e.fontSizeIcon).mul(-1).div(2).equal(),color:e.colorTextQuaternary,fontSize:e.fontSizeIcon,lineHeight:1,textAlign:"center",pointerEvents:"none",display:"flex",alignItems:"center",transition:`opacity ${e.motionDurationSlow} ease`,[o]:{verticalAlign:"top",transition:`transform ${e.motionDurationSlow}`,"> svg":{verticalAlign:"top"},[`&:not(${n}-suffix)`]:{pointerEvents:"auto"}},[`${n}-disabled &`]:{cursor:"not-allowed"},"> *:not(:last-child)":{marginInlineEnd:8}}),[`${n}-selection-wrap`]:{display:"flex",width:"100%",position:"relative",minWidth:0,"&:after":{content:'"\\a0"',width:0,overflow:"hidden"}},[`${n}-prefix`]:{flex:"none",marginInlineEnd:e.selectAffixPadding},[`${n}-clear`]:{position:"absolute",top:"50%",insetInlineStart:"auto",insetInlineEnd:r,zIndex:1,display:"inline-block",width:e.fontSizeIcon,height:e.fontSizeIcon,marginTop:e.calc(e.fontSizeIcon).mul(-1).div(2).equal(),color:e.colorTextQuaternary,fontSize:e.fontSizeIcon,fontStyle:"normal",lineHeight:1,textAlign:"center",textTransform:"none",cursor:"pointer",opacity:0,transition:`color ${e.motionDurationMid} ease, opacity ${e.motionDurationSlow} ease`,textRendering:"auto","&:before":{display:"block"},"&:hover":{color:e.colorIcon}},[`&:hover ${n}-clear`]:{opacity:1,background:e.colorBgBase,borderRadius:"50%"}}),[`${n}-status`]:{"&-error, &-warning, &-success, &-validating":{[`&${n}-has-feedback`]:{[`${n}-clear`]:{insetInlineEnd:e.calc(r).add(e.fontSize).add(e.paddingXS).equal()}}}}}},bB=e=>{const{componentCls:t}=e;return[{[t]:{[`&${t}-in-form-item`]:{width:"100%"}}},yB(e),cB(e),sB(e),rB(e),{[`${t}-rtl`]:{direction:"rtl"}},p0(e,{borderElCls:`${t}-selector`,focusElCls:`${t}-focused`})]},SB=wn("Select",(e,t)=>{let{rootPrefixCls:n}=t;const r=kt(e,{rootPrefixCls:n,inputPaddingHorizontalBase:e.calc(e.paddingSM).sub(1).equal(),multipleSelectItemHeight:e.multipleItemHeight,selectHeight:e.controlHeight});return[bB(r),gB(r)]},uB,{unitless:{optionLineHeight:!0,optionSelectedFontWeight:!0}});function CB(e){let{suffixIcon:t,clearIcon:n,menuItemSelectedIcon:r,removeIcon:o,loading:l,multiple:c,hasFeedback:d,prefixCls:m,showSuffixIcon:v,feedbackIcon:g,showArrow:p,componentName:y}=e;const E=n??u.createElement(Xl,null),C=$=>t===null&&!d&&!p?null:u.createElement(u.Fragment,null,v!==!1&&$,d&&g);let x=null;if(t!==void 0)x=C(t);else if(l)x=C(u.createElement(Uc,{spin:!0}));else{const $=`${m}-suffix`;x=O=>{let{open:T,showSearch:R}=O;return C(T&&R?u.createElement(xw,{className:$}):u.createElement(bN,{className:$}))}}let b=null;r!==void 0?b=r:c?b=u.createElement(Cw,null):b=null;let w=null;return o!==void 0?w=o:w=u.createElement(Yl,null),{clearIcon:E,suffixIcon:x,itemIcon:b,removeIcon:w}}function xB(e,t){return t!==void 0?t:e!==null}var EB=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const e2="SECRET_COMBOBOX_MODE_DO_NOT_USE",wB=(e,t)=>{var n;const{prefixCls:r,bordered:o,className:l,rootClassName:c,getPopupContainer:d,popupClassName:m,dropdownClassName:v,listHeight:g=256,placement:p,listItemHeight:y,size:E,disabled:C,notFoundContent:x,status:b,builtinPlacements:w,dropdownMatchSelectWidth:$,popupMatchSelectWidth:O,direction:T,style:R,allowClear:M,variant:N,dropdownStyle:z,transitionName:P,tagRender:D,maxCount:F,prefix:W}=e,V=EB(e,["prefixCls","bordered","className","rootClassName","getPopupContainer","popupClassName","dropdownClassName","listHeight","placement","listItemHeight","size","disabled","notFoundContent","status","builtinPlacements","dropdownMatchSelectWidth","popupMatchSelectWidth","direction","style","allowClear","variant","dropdownStyle","transitionName","tagRender","maxCount","prefix"]),{getPopupContainer:q,getPrefixCls:K,renderEmpty:_,direction:j,virtual:B,popupMatchSelectWidth:k,popupOverflow:I}=u.useContext(vt),H=zr("select"),[,G]=Rr(),X=y??(G==null?void 0:G.controlHeight),U=K("select",r),J=K(),te=T??j,{compactSize:ve,compactItemClassnames:ne}=Xc(U,te),[le,be]=N0("select",N,o),fe=na(U),[ie,pe,Re]=SB(U,fe),se=u.useMemo(()=>{const{mode:Ct}=e;if(Ct!=="combobox")return Ct===e2?"combobox":Ct},[e.mode]),ye=se==="multiple"||se==="tags",$e=xB(e.suffixIcon,e.showArrow),we=(n=O??$)!==null&&n!==void 0?n:k,{status:Ne,hasFeedback:Ze,isFormItemInput:He,feedbackIcon:Xe}=u.useContext(Jr),xe=Gd(Ne,b);let Me;x!==void 0?Me=x:se==="combobox"?Me=null:Me=(_==null?void 0:_("Select"))||u.createElement(eB,{componentName:"Select"});const{suffixIcon:Ve,itemIcon:Ke,removeIcon:We,clearIcon:at}=CB(Object.assign(Object.assign({},V),{multiple:ye,hasFeedback:Ze,feedbackIcon:Xe,showSuffixIcon:$e,prefixCls:U,componentName:"Select"})),Ue=M===!0?{clearIcon:at}:M,Ye=Tn(V,["suffixIcon","itemIcon"]),st=re(m||v,{[`${U}-dropdown-${te}`]:te==="rtl"},c,Re,fe,pe),ht=Mo(Ct=>{var Nt;return(Nt=E??ve)!==null&&Nt!==void 0?Nt:Ct}),Qe=u.useContext(Co),Pe=C??Qe,Fe=re({[`${U}-lg`]:ht==="large",[`${U}-sm`]:ht==="small",[`${U}-rtl`]:te==="rtl",[`${U}-${le}`]:be,[`${U}-in-form-item`]:He},od(U,xe,Ze),ne,H.className,l,c,Re,fe,pe),_e=u.useMemo(()=>p!==void 0?p:te==="rtl"?"bottomRight":"bottomLeft",[p,te]),[qe]=Nd("SelectLike",z==null?void 0:z.zIndex);return ie(u.createElement(_0,Object.assign({ref:t,virtual:B,showSearch:H.showSearch},Ye,{style:Object.assign(Object.assign({},H.style),R),dropdownMatchSelectWidth:we,transitionName:Fi(J,"slide-up",P),builtinPlacements:nB(w,I),listHeight:g,listItemHeight:X,mode:se,prefixCls:U,placement:_e,direction:te,prefix:W,suffixIcon:Ve,menuItemSelectedIcon:Ke,removeIcon:We,allowClear:Ue,notFoundContent:Me,className:Fe,getPopupContainer:d||q,dropdownClassName:st,disabled:Pe,dropdownStyle:Object.assign(Object.assign({},z),{zIndex:qe}),maxCount:ye?F:void 0,tagRender:ye?D:void 0})))},rs=u.forwardRef(wB),$B=yD(rs,"dropdownAlign");rs.SECRET_COMBOBOX_MODE_DO_NOT_USE=e2;rs.Option=M0;rs.OptGroup=T0;rs._InternalPanelDoNotUseOrYouWillBeFired=$B;const jc=["xxl","xl","lg","md","sm","xs"],OB=e=>({xs:`(max-width: ${e.screenXSMax}px)`,sm:`(min-width: ${e.screenSM}px)`,md:`(min-width: ${e.screenMD}px)`,lg:`(min-width: ${e.screenLG}px)`,xl:`(min-width: ${e.screenXL}px)`,xxl:`(min-width: ${e.screenXXL}px)`}),RB=e=>{const t=e,n=[].concat(jc).reverse();return n.forEach((r,o)=>{const l=r.toUpperCase(),c=`screen${l}Min`,d=`screen${l}`;if(!(t[c]<=t[d]))throw new Error(`${c}<=${d} fails : !(${t[c]}<=${t[d]})`);if(o<n.length-1){const m=`screen${l}Max`;if(!(t[d]<=t[m]))throw new Error(`${d}<=${m} fails : !(${t[d]}<=${t[m]})`);const g=`screen${n[o+1].toUpperCase()}Min`;if(!(t[m]<=t[g]))throw new Error(`${m}<=${g} fails : !(${t[m]}<=${t[g]})`)}}),e},TB=()=>{const[,e]=Rr(),t=OB(RB(e));return oe.useMemo(()=>{const n=new Map;let r=-1,o={};return{responsiveMap:t,matchHandlers:{},dispatch(l){return o=l,n.forEach(c=>c(o)),n.size>=1},subscribe(l){return n.size||this.register(),r+=1,n.set(r,l),l(o),r},unsubscribe(l){n.delete(l),n.size||this.unregister()},register(){Object.keys(t).forEach(l=>{const c=t[l],d=v=>{let{matches:g}=v;this.dispatch(Object.assign(Object.assign({},o),{[l]:g}))},m=window.matchMedia(c);m.addListener(d),this.matchHandlers[c]={mql:m,listener:d},d(m)})},unregister(){Object.keys(t).forEach(l=>{const c=t[l],d=this.matchHandlers[c];d==null||d.mql.removeListener(d==null?void 0:d.listener)}),n.clear()}}},[e])};function MB(){const[,e]=u.useReducer(t=>t+1,0);return e}function _B(){let e=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!0,t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};const n=u.useRef(t),r=MB(),o=TB();return Wt(()=>{const l=o.subscribe(c=>{n.current=c,e&&r()});return()=>o.unsubscribe(l)},[]),n.current}const kl=e=>e?typeof e=="function"?e():e:null;function A0(e){var t=e.children,n=e.prefixCls,r=e.id,o=e.overlayInnerStyle,l=e.bodyClassName,c=e.className,d=e.style;return u.createElement("div",{className:re("".concat(n,"-content"),c),style:d},u.createElement("div",{className:re("".concat(n,"-inner"),l),id:r,role:"tooltip",style:o},typeof t=="function"?t():t))}var Al={shiftX:64,adjustY:1},Il={adjustX:1,shiftY:!0},ga=[0,0],NB={left:{points:["cr","cl"],overflow:Il,offset:[-4,0],targetOffset:ga},right:{points:["cl","cr"],overflow:Il,offset:[4,0],targetOffset:ga},top:{points:["bc","tc"],overflow:Al,offset:[0,-4],targetOffset:ga},bottom:{points:["tc","bc"],overflow:Al,offset:[0,4],targetOffset:ga},topLeft:{points:["bl","tl"],overflow:Al,offset:[0,-4],targetOffset:ga},leftTop:{points:["tr","tl"],overflow:Il,offset:[-4,0],targetOffset:ga},topRight:{points:["br","tr"],overflow:Al,offset:[0,-4],targetOffset:ga},rightTop:{points:["tl","tr"],overflow:Il,offset:[4,0],targetOffset:ga},bottomRight:{points:["tr","br"],overflow:Al,offset:[0,4],targetOffset:ga},rightBottom:{points:["bl","br"],overflow:Il,offset:[4,0],targetOffset:ga},bottomLeft:{points:["tl","bl"],overflow:Al,offset:[0,4],targetOffset:ga},leftBottom:{points:["br","bl"],overflow:Il,offset:[-4,0],targetOffset:ga}},AB=["overlayClassName","trigger","mouseEnterDelay","mouseLeaveDelay","overlayStyle","prefixCls","children","onVisibleChange","afterVisibleChange","transitionName","animation","motion","placement","align","destroyTooltipOnHide","defaultVisible","getTooltipContainer","overlayInnerStyle","arrowContent","overlay","id","showArrow","classNames","styles"],IB=function(t,n){var r=t.overlayClassName,o=t.trigger,l=o===void 0?["hover"]:o,c=t.mouseEnterDelay,d=c===void 0?0:c,m=t.mouseLeaveDelay,v=m===void 0?.1:m,g=t.overlayStyle,p=t.prefixCls,y=p===void 0?"rc-tooltip":p,E=t.children,C=t.onVisibleChange,x=t.afterVisibleChange,b=t.transitionName,w=t.animation,$=t.motion,O=t.placement,T=O===void 0?"right":O,R=t.align,M=R===void 0?{}:R,N=t.destroyTooltipOnHide,z=N===void 0?!1:N,P=t.defaultVisible,D=t.getTooltipContainer,F=t.overlayInnerStyle;t.arrowContent;var W=t.overlay,V=t.id,q=t.showArrow,K=q===void 0?!0:q,_=t.classNames,j=t.styles,B=wt(t,AB),k=S0(V),I=u.useRef(null);u.useImperativeHandle(n,function(){return I.current});var H=Z({},B);"visible"in t&&(H.popupVisible=t.visible);var G=function(){return u.createElement(A0,{key:"content",prefixCls:y,id:k,bodyClassName:_==null?void 0:_.body,overlayInnerStyle:Z(Z({},F),j==null?void 0:j.body)},W)},X=function(){var J=u.Children.only(E),te=(J==null?void 0:J.props)||{},ve=Z(Z({},te),{},{"aria-describedby":W?k:null});return u.cloneElement(E,ve)};return u.createElement(O0,Ie({popupClassName:re(r,_==null?void 0:_.root),prefixCls:y,popup:G,action:l,builtinPlacements:NB,popupPlacement:T,ref:I,popupAlign:M,getPopupContainer:D,onPopupVisibleChange:C,afterPopupVisibleChange:x,popupTransitionName:b,popupAnimation:w,popupMotion:$,defaultPopupVisible:P,autoDestroy:z,mouseLeaveDelay:v,popupStyle:Z(Z({},g),j==null?void 0:j.root),mouseEnterDelay:d,arrow:K},H),X())};const jB=u.forwardRef(IB);function t2(e){const{sizePopupArrow:t,borderRadiusXS:n,borderRadiusOuter:r}=e,o=t/2,l=0,c=o,d=r*1/Math.sqrt(2),m=o-r*(1-1/Math.sqrt(2)),v=o-n*(1/Math.sqrt(2)),g=r*(Math.sqrt(2)-1)+n*(1/Math.sqrt(2)),p=2*o-v,y=g,E=2*o-d,C=m,x=2*o-l,b=c,w=o*Math.sqrt(2)+r*(Math.sqrt(2)-2),$=r*(Math.sqrt(2)-1),O=`polygon(${$}px 100%, 50% ${$}px, ${2*o-$}px 100%, ${$}px 100%)`,T=`path('M ${l} ${c} A ${r} ${r} 0 0 0 ${d} ${m} L ${v} ${g} A ${n} ${n} 0 0 1 ${p} ${y} L ${E} ${C} A ${r} ${r} 0 0 0 ${x} ${b} Z')`;return{arrowShadowWidth:w,arrowPath:T,arrowPolygon:O}}const PB=(e,t,n)=>{const{sizePopupArrow:r,arrowPolygon:o,arrowPath:l,arrowShadowWidth:c,borderRadiusXS:d,calc:m}=e;return{pointerEvents:"none",width:r,height:r,overflow:"hidden","&::before":{position:"absolute",bottom:0,insetInlineStart:0,width:r,height:m(r).div(2).equal(),background:t,clipPath:{_multi_value_:!0,value:[o,l]},content:'""'},"&::after":{content:'""',position:"absolute",width:c,height:c,bottom:0,insetInline:0,margin:"auto",borderRadius:{_skip_check_:!0,value:`0 0 ${he(d)} 0`},transform:"translateY(50%) rotate(-135deg)",boxShadow:n,zIndex:0,background:"transparent"}}},n2=8;function I0(e){const{contentRadius:t,limitVerticalRadius:n}=e,r=t>12?t+2:12;return{arrowOffsetHorizontal:r,arrowOffsetVertical:n?n2:r}}function bf(e,t){return e?t:{}}function r2(e,t,n){const{componentCls:r,boxShadowPopoverArrow:o,arrowOffsetVertical:l,arrowOffsetHorizontal:c}=e,{arrowDistance:d=0,arrowPlacement:m={left:!0,right:!0,top:!0,bottom:!0}}=n||{};return{[r]:Object.assign(Object.assign(Object.assign(Object.assign({[`${r}-arrow`]:[Object.assign(Object.assign({position:"absolute",zIndex:1,display:"block"},PB(e,t,o)),{"&:before":{background:t}})]},bf(!!m.top,{[[`&-placement-top > ${r}-arrow`,`&-placement-topLeft > ${r}-arrow`,`&-placement-topRight > ${r}-arrow`].join(",")]:{bottom:d,transform:"translateY(100%) rotate(180deg)"},[`&-placement-top > ${r}-arrow`]:{left:{_skip_check_:!0,value:"50%"},transform:"translateX(-50%) translateY(100%) rotate(180deg)"},"&-placement-topLeft":{"--arrow-offset-horizontal":c,[`> ${r}-arrow`]:{left:{_skip_check_:!0,value:c}}},"&-placement-topRight":{"--arrow-offset-horizontal":`calc(100% - ${he(c)})`,[`> ${r}-arrow`]:{right:{_skip_check_:!0,value:c}}}})),bf(!!m.bottom,{[[`&-placement-bottom > ${r}-arrow`,`&-placement-bottomLeft > ${r}-arrow`,`&-placement-bottomRight > ${r}-arrow`].join(",")]:{top:d,transform:"translateY(-100%)"},[`&-placement-bottom > ${r}-arrow`]:{left:{_skip_check_:!0,value:"50%"},transform:"translateX(-50%) translateY(-100%)"},"&-placement-bottomLeft":{"--arrow-offset-horizontal":c,[`> ${r}-arrow`]:{left:{_skip_check_:!0,value:c}}},"&-placement-bottomRight":{"--arrow-offset-horizontal":`calc(100% - ${he(c)})`,[`> ${r}-arrow`]:{right:{_skip_check_:!0,value:c}}}})),bf(!!m.left,{[[`&-placement-left > ${r}-arrow`,`&-placement-leftTop > ${r}-arrow`,`&-placement-leftBottom > ${r}-arrow`].join(",")]:{right:{_skip_check_:!0,value:d},transform:"translateX(100%) rotate(90deg)"},[`&-placement-left > ${r}-arrow`]:{top:{_skip_check_:!0,value:"50%"},transform:"translateY(-50%) translateX(100%) rotate(90deg)"},[`&-placement-leftTop > ${r}-arrow`]:{top:l},[`&-placement-leftBottom > ${r}-arrow`]:{bottom:l}})),bf(!!m.right,{[[`&-placement-right > ${r}-arrow`,`&-placement-rightTop > ${r}-arrow`,`&-placement-rightBottom > ${r}-arrow`].join(",")]:{left:{_skip_check_:!0,value:d},transform:"translateX(-100%) rotate(-90deg)"},[`&-placement-right > ${r}-arrow`]:{top:{_skip_check_:!0,value:"50%"},transform:"translateY(-50%) translateX(-100%) rotate(-90deg)"},[`&-placement-rightTop > ${r}-arrow`]:{top:l},[`&-placement-rightBottom > ${r}-arrow`]:{bottom:l}}))}}function zB(e,t,n,r){if(r===!1)return{adjustX:!1,adjustY:!1};const o=r&&typeof r=="object"?r:{},l={};switch(e){case"top":case"bottom":l.shiftX=t.arrowOffsetHorizontal*2+n,l.shiftY=!0,l.adjustY=!0;break;case"left":case"right":l.shiftY=t.arrowOffsetVertical*2+n,l.shiftX=!0,l.adjustX=!0;break}const c=Object.assign(Object.assign({},l),o);return c.shiftX||(c.adjustX=!0),c.shiftY||(c.adjustY=!0),c}const Wx={left:{points:["cr","cl"]},right:{points:["cl","cr"]},top:{points:["bc","tc"]},bottom:{points:["tc","bc"]},topLeft:{points:["bl","tl"]},leftTop:{points:["tr","tl"]},topRight:{points:["br","tr"]},rightTop:{points:["tl","tr"]},bottomRight:{points:["tr","br"]},rightBottom:{points:["bl","br"]},bottomLeft:{points:["tl","bl"]},leftBottom:{points:["br","bl"]}},DB={topLeft:{points:["bl","tc"]},leftTop:{points:["tr","cl"]},topRight:{points:["br","tc"]},rightTop:{points:["tl","cr"]},bottomRight:{points:["tr","bc"]},rightBottom:{points:["bl","cr"]},bottomLeft:{points:["tl","bc"]},leftBottom:{points:["br","cl"]}},BB=new Set(["topLeft","topRight","bottomLeft","bottomRight","leftTop","leftBottom","rightTop","rightBottom"]);function LB(e){const{arrowWidth:t,autoAdjustOverflow:n,arrowPointAtCenter:r,offset:o,borderRadius:l,visibleFirst:c}=e,d=t/2,m={};return Object.keys(Wx).forEach(v=>{const g=r&&DB[v]||Wx[v],p=Object.assign(Object.assign({},g),{offset:[0,0],dynamicInset:!0});switch(m[v]=p,BB.has(v)&&(p.autoArrow=!1),v){case"top":case"topLeft":case"topRight":p.offset[1]=-d-o;break;case"bottom":case"bottomLeft":case"bottomRight":p.offset[1]=d+o;break;case"left":case"leftTop":case"leftBottom":p.offset[0]=-d-o;break;case"right":case"rightTop":case"rightBottom":p.offset[0]=d+o;break}const y=I0({contentRadius:l,limitVerticalRadius:!0});if(r)switch(v){case"topLeft":case"bottomLeft":p.offset[0]=-y.arrowOffsetHorizontal-d;break;case"topRight":case"bottomRight":p.offset[0]=y.arrowOffsetHorizontal+d;break;case"leftTop":case"rightTop":p.offset[1]=-y.arrowOffsetHorizontal*2+d;break;case"leftBottom":case"rightBottom":p.offset[1]=y.arrowOffsetHorizontal*2-d;break}p.overflow=zB(v,y,t,n),c&&(p.htmlRegion="visibleFirst")}),m}const HB=e=>{const{calc:t,componentCls:n,tooltipMaxWidth:r,tooltipColor:o,tooltipBg:l,tooltipBorderRadius:c,zIndexPopup:d,controlHeight:m,boxShadowSecondary:v,paddingSM:g,paddingXS:p,arrowOffsetHorizontal:y,sizePopupArrow:E}=e,C=t(c).add(E).add(y).equal(),x=t(c).mul(2).add(E).equal();return[{[n]:Object.assign(Object.assign(Object.assign(Object.assign({},Tr(e)),{position:"absolute",zIndex:d,display:"block",width:"max-content",maxWidth:r,visibility:"visible","--valid-offset-x":"var(--arrow-offset-horizontal, var(--arrow-x))",transformOrigin:["var(--valid-offset-x, 50%)","var(--arrow-y, 50%)"].join(" "),"&-hidden":{display:"none"},"--antd-arrow-background-color":l,[`${n}-inner`]:{minWidth:x,minHeight:m,padding:`${he(e.calc(g).div(2).equal())} ${he(p)}`,color:o,textAlign:"start",textDecoration:"none",wordWrap:"break-word",backgroundColor:l,borderRadius:c,boxShadow:v,boxSizing:"border-box"},[["&-placement-topLeft","&-placement-topRight","&-placement-bottomLeft","&-placement-bottomRight"].join(",")]:{minWidth:C},[["&-placement-left","&-placement-leftTop","&-placement-leftBottom","&-placement-right","&-placement-rightTop","&-placement-rightBottom"].join(",")]:{[`${n}-inner`]:{borderRadius:e.min(c,n2)}},[`${n}-content`]:{position:"relative"}}),Tj(e,(b,w)=>{let{darkColor:$}=w;return{[`&${n}-${b}`]:{[`${n}-inner`]:{backgroundColor:$},[`${n}-arrow`]:{"--antd-arrow-background-color":$}}}})),{"&-rtl":{direction:"rtl"}})},r2(e,"var(--antd-arrow-background-color)"),{[`${n}-pure`]:{position:"relative",maxWidth:"none",margin:e.sizePopupArrow}}]},FB=e=>Object.assign(Object.assign({zIndexPopup:e.zIndexPopupBase+70},I0({contentRadius:e.borderRadius,limitVerticalRadius:!0})),t2(kt(e,{borderRadiusOuter:Math.min(e.borderRadiusOuter,4)}))),a2=function(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0;return wn("Tooltip",r=>{const{borderRadius:o,colorTextLightSolid:l,colorBgSpotlight:c}=r,d=kt(r,{tooltipMaxWidth:250,tooltipColor:l,tooltipBorderRadius:o,tooltipBg:c});return[HB(d),Dd(r,"zoom-big-fast")]},FB,{resetStyle:!1,injectStyle:t})(e)},VB=ai.map(e=>`${e}-inverse`),UB=["success","processing","error","default","warning"];function qB(e){return(arguments.length>1&&arguments[1]!==void 0?arguments[1]:!0)?[].concat(Te(VB),Te(ai)).includes(e):ai.includes(e)}function sW(e){return UB.includes(e)}function o2(e,t){const n=qB(t),r=re({[`${e}-${t}`]:t&&n}),o={},l={};return t&&!n&&(o.background=t,l["--antd-arrow-background-color"]=t),{className:r,overlayStyle:o,arrowStyle:l}}const WB=e=>{const{prefixCls:t,className:n,placement:r="top",title:o,color:l,overlayInnerStyle:c}=e,{getPrefixCls:d}=u.useContext(vt),m=d("tooltip",t),[v,g,p]=a2(m),y=o2(m,l),E=y.arrowStyle,C=Object.assign(Object.assign({},c),y.overlayStyle),x=re(g,p,m,`${m}-pure`,`${m}-placement-${r}`,n,y.className);return v(u.createElement("div",{className:x,style:E},u.createElement("div",{className:`${m}-arrow`}),u.createElement(A0,Object.assign({},e,{className:g,prefixCls:m,overlayInnerStyle:C}),o)))};var kB=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const GB=u.forwardRef((e,t)=>{var n,r;const{prefixCls:o,openClassName:l,getTooltipContainer:c,color:d,overlayInnerStyle:m,children:v,afterOpenChange:g,afterVisibleChange:p,destroyTooltipOnHide:y,arrow:E=!0,title:C,overlay:x,builtinPlacements:b,arrowPointAtCenter:w=!1,autoAdjustOverflow:$=!0,motion:O,getPopupContainer:T,placement:R="top",mouseEnterDelay:M=.1,mouseLeaveDelay:N=.1,overlayStyle:z,rootClassName:P,overlayClassName:D,styles:F,classNames:W}=e,V=kB(e,["prefixCls","openClassName","getTooltipContainer","color","overlayInnerStyle","children","afterOpenChange","afterVisibleChange","destroyTooltipOnHide","arrow","title","overlay","builtinPlacements","arrowPointAtCenter","autoAdjustOverflow","motion","getPopupContainer","placement","mouseEnterDelay","mouseLeaveDelay","overlayStyle","rootClassName","overlayClassName","styles","classNames"]),q=!!E,[,K]=Rr(),{getPopupContainer:_,getPrefixCls:j,direction:B,className:k,style:I,classNames:H,styles:G}=zr("tooltip"),X=kc(),U=u.useRef(null),J=()=>{var Ye;(Ye=U.current)===null||Ye===void 0||Ye.forceAlign()};u.useImperativeHandle(t,()=>{var Ye,st;return{forceAlign:J,forcePopupAlign:()=>{X.deprecated(!1,"forcePopupAlign","forceAlign"),J()},nativeElement:(Ye=U.current)===null||Ye===void 0?void 0:Ye.nativeElement,popupElement:(st=U.current)===null||st===void 0?void 0:st.popupElement}});const[te,ve]=gr(!1,{value:(n=e.open)!==null&&n!==void 0?n:e.visible,defaultValue:(r=e.defaultOpen)!==null&&r!==void 0?r:e.defaultVisible}),ne=!C&&!x&&C!==0,le=Ye=>{var st,ht;ve(ne?!1:Ye),ne||((st=e.onOpenChange)===null||st===void 0||st.call(e,Ye),(ht=e.onVisibleChange)===null||ht===void 0||ht.call(e,Ye))},be=u.useMemo(()=>{var Ye,st;let ht=w;return typeof E=="object"&&(ht=(st=(Ye=E.pointAtCenter)!==null&&Ye!==void 0?Ye:E.arrowPointAtCenter)!==null&&st!==void 0?st:w),b||LB({arrowPointAtCenter:ht,autoAdjustOverflow:$,arrowWidth:q?K.sizePopupArrow:0,borderRadius:K.borderRadius,offset:K.marginXXS,visibleFirst:!0})},[w,E,b,K]),fe=u.useMemo(()=>C===0?C:x||C||"",[x,C]),ie=u.createElement(Ac,{space:!0},typeof fe=="function"?fe():fe),pe=j("tooltip",o),Re=j(),se=e["data-popover-inject"];let ye=te;!("open"in e)&&!("visible"in e)&&ne&&(ye=!1);const $e=u.isValidElement(v)&&!R$(v)?v:u.createElement("span",null,v),we=$e.props,Ne=!we.className||typeof we.className=="string"?re(we.className,l||`${pe}-open`):we.className,[Ze,He,Xe]=a2(pe,!se),xe=o2(pe,d),Me=xe.arrowStyle,Ve=re(D,{[`${pe}-rtl`]:B==="rtl"},xe.className,P,He,Xe,k,H.root,W==null?void 0:W.root),Ke=re(H.body,W==null?void 0:W.body),[We,at]=Nd("Tooltip",V.zIndex),Ue=u.createElement(jB,Object.assign({},V,{zIndex:We,showArrow:q,placement:R,mouseEnterDelay:M,mouseLeaveDelay:N,prefixCls:pe,classNames:{root:Ve,body:Ke},styles:{root:Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},Me),G.root),I),z),F==null?void 0:F.root),body:Object.assign(Object.assign(Object.assign(Object.assign({},G.body),m),F==null?void 0:F.body),xe.overlayStyle)},getTooltipContainer:T||c||_,ref:U,builtinPlacements:be,overlay:ie,visible:ye,onVisibleChange:le,afterVisibleChange:g??p,arrowContent:u.createElement("span",{className:`${pe}-arrow-content`}),motion:{motionName:Fi(Re,"zoom-big-fast",e.transitionName),motionDeadline:1e3},destroyTooltipOnHide:!!y}),ye?Zr($e,{className:Ne}):$e);return Ze(u.createElement(c0.Provider,{value:at},Ue))}),si=GB;si._InternalPanelDoNotUseOrYouWillBeFired=WB;const KB=e=>{const{componentCls:t,popoverColor:n,titleMinWidth:r,fontWeightStrong:o,innerPadding:l,boxShadowSecondary:c,colorTextHeading:d,borderRadiusLG:m,zIndexPopup:v,titleMarginBottom:g,colorBgElevated:p,popoverBg:y,titleBorderBottom:E,innerContentPadding:C,titlePadding:x}=e;return[{[t]:Object.assign(Object.assign({},Tr(e)),{position:"absolute",top:0,left:{_skip_check_:!0,value:0},zIndex:v,fontWeight:"normal",whiteSpace:"normal",textAlign:"start",cursor:"auto",userSelect:"text","--valid-offset-x":"var(--arrow-offset-horizontal, var(--arrow-x))",transformOrigin:["var(--valid-offset-x, 50%)","var(--arrow-y, 50%)"].join(" "),"--antd-arrow-background-color":p,width:"max-content",maxWidth:"100vw","&-rtl":{direction:"rtl"},"&-hidden":{display:"none"},[`${t}-content`]:{position:"relative"},[`${t}-inner`]:{backgroundColor:y,backgroundClip:"padding-box",borderRadius:m,boxShadow:c,padding:l},[`${t}-title`]:{minWidth:r,marginBottom:g,color:d,fontWeight:o,borderBottom:E,padding:x},[`${t}-inner-content`]:{color:n,padding:C}})},r2(e,"var(--antd-arrow-background-color)"),{[`${t}-pure`]:{position:"relative",maxWidth:"none",margin:e.sizePopupArrow,display:"inline-block",[`${t}-content`]:{display:"inline-block"}}}]},XB=e=>{const{componentCls:t}=e;return{[t]:ai.map(n=>{const r=e[`${n}6`];return{[`&${t}-${n}`]:{"--antd-arrow-background-color":r,[`${t}-inner`]:{backgroundColor:r},[`${t}-arrow`]:{background:"transparent"}}}})}},YB=e=>{const{lineWidth:t,controlHeight:n,fontHeight:r,padding:o,wireframe:l,zIndexPopupBase:c,borderRadiusLG:d,marginXS:m,lineType:v,colorSplit:g,paddingSM:p}=e,y=n-r,E=y/2,C=y/2-t,x=o;return Object.assign(Object.assign(Object.assign({titleMinWidth:177,zIndexPopup:c+30},t2(e)),I0({contentRadius:d,limitVerticalRadius:!0})),{innerPadding:l?0:12,titleMarginBottom:l?0:m,titlePadding:l?`${E}px ${x}px ${C}px`:0,titleBorderBottom:l?`${t}px ${v} ${g}`:"none",innerContentPadding:l?`${p}px ${x}px`:0})},i2=wn("Popover",e=>{const{colorBgElevated:t,colorText:n}=e,r=kt(e,{popoverBg:t,popoverColor:n});return[KB(r),XB(r),Dd(r,"zoom-big")]},YB,{resetStyle:!1,deprecatedTokens:[["width","titleMinWidth"],["minWidth","titleMinWidth"]]});var QB=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const l2=e=>{let{title:t,content:n,prefixCls:r}=e;return!t&&!n?null:u.createElement(u.Fragment,null,t&&u.createElement("div",{className:`${r}-title`},t),n&&u.createElement("div",{className:`${r}-inner-content`},n))},ZB=e=>{const{hashId:t,prefixCls:n,className:r,style:o,placement:l="top",title:c,content:d,children:m}=e,v=kl(c),g=kl(d),p=re(t,n,`${n}-pure`,`${n}-placement-${l}`,r);return u.createElement("div",{className:p,style:o},u.createElement("div",{className:`${n}-arrow`}),u.createElement(A0,Object.assign({},e,{className:t,prefixCls:n}),m||u.createElement(l2,{prefixCls:n,title:v,content:g})))},s2=e=>{const{prefixCls:t,className:n}=e,r=QB(e,["prefixCls","className"]),{getPrefixCls:o}=u.useContext(vt),l=o("popover",t),[c,d,m]=i2(l);return c(u.createElement(ZB,Object.assign({},r,{prefixCls:l,hashId:d,className:re(n,m)})))};var JB=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const eL=u.forwardRef((e,t)=>{var n,r;const{prefixCls:o,title:l,content:c,overlayClassName:d,placement:m="top",trigger:v="hover",children:g,mouseEnterDelay:p=.1,mouseLeaveDelay:y=.1,onOpenChange:E,overlayStyle:C={},styles:x,classNames:b}=e,w=JB(e,["prefixCls","title","content","overlayClassName","placement","trigger","children","mouseEnterDelay","mouseLeaveDelay","onOpenChange","overlayStyle","styles","classNames"]),{getPrefixCls:$,className:O,style:T,classNames:R,styles:M}=zr("popover"),N=$("popover",o),[z,P,D]=i2(N),F=$(),W=re(d,P,D,O,R.root,b==null?void 0:b.root),V=re(R.body,b==null?void 0:b.body),[q,K]=gr(!1,{value:(n=e.open)!==null&&n!==void 0?n:e.visible,defaultValue:(r=e.defaultOpen)!==null&&r!==void 0?r:e.defaultVisible}),_=(H,G)=>{K(H,!0),E==null||E(H,G)},j=H=>{H.keyCode===ct.ESC&&_(!1,H)},B=H=>{_(H)},k=kl(l),I=kl(c);return z(u.createElement(si,Object.assign({placement:m,trigger:v,mouseEnterDelay:p,mouseLeaveDelay:y},w,{prefixCls:N,classNames:{root:W,body:V},styles:{root:Object.assign(Object.assign(Object.assign(Object.assign({},M.root),T),C),x==null?void 0:x.root),body:Object.assign(Object.assign({},M.body),x==null?void 0:x.body)},ref:t,open:q,onOpenChange:B,overlay:k||I?u.createElement(l2,{prefixCls:N,title:k,content:I}):null,transitionName:Fi(F,"zoom-big",w.transitionName),"data-popover-inject":!0}),Zr(g,{onKeyDown:H=>{var G,X;u.isValidElement(g)&&((X=g==null?void 0:(G=g.props).onKeyDown)===null||X===void 0||X.call(G,H)),j(H)}})))}),c2=eL;c2._InternalPanelDoNotUseOrYouWillBeFired=s2;var u2=u.createContext(null);function f2(e,t){return e===void 0?null:"".concat(e,"-").concat(t)}function d2(e){var t=u.useContext(u2);return f2(t,e)}var tL=["children","locked"],Na=u.createContext(null);function nL(e,t){var n=Z({},e);return Object.keys(t).forEach(function(r){var o=t[r];o!==void 0&&(n[r]=o)}),n}function Pc(e){var t=e.children,n=e.locked,r=wt(e,tL),o=u.useContext(Na),l=qc(function(){return nL(o,r)},[o,r],function(c,d){return!n&&(c[0]!==d[0]||!Oc(c[1],d[1],!0))});return u.createElement(Na.Provider,{value:l},t)}var rL=[],m2=u.createContext(null);function Kd(){return u.useContext(m2)}var v2=u.createContext(rL);function as(e){var t=u.useContext(v2);return u.useMemo(function(){return e!==void 0?[].concat(Te(t),[e]):t},[t,e])}var g2=u.createContext(null),j0=u.createContext({});function kx(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;if(jd(e)){var n=e.nodeName.toLowerCase(),r=["input","select","textarea","button"].includes(n)||e.isContentEditable||n==="a"&&!!e.getAttribute("href"),o=e.getAttribute("tabindex"),l=Number(o),c=null;return o&&!Number.isNaN(l)?c=l:r&&c===null&&(c=0),r&&e.disabled&&(c=null),c!==null&&(c>=0||t&&c<0)}return!1}function aL(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,n=Te(e.querySelectorAll("*")).filter(function(r){return kx(r,t)});return kx(e,t)&&n.unshift(e),n}var Ep=ct.LEFT,wp=ct.RIGHT,$p=ct.UP,Df=ct.DOWN,Bf=ct.ENTER,h2=ct.ESC,lc=ct.HOME,sc=ct.END,Gx=[$p,Df,Ep,wp];function oL(e,t,n,r){var o,l="prev",c="next",d="children",m="parent";if(e==="inline"&&r===Bf)return{inlineTrigger:!0};var v=Y(Y({},$p,l),Df,c),g=Y(Y(Y(Y({},Ep,n?c:l),wp,n?l:c),Df,d),Bf,d),p=Y(Y(Y(Y(Y(Y({},$p,l),Df,c),Bf,d),h2,m),Ep,n?d:m),wp,n?m:d),y={inline:v,horizontal:g,vertical:p,inlineSub:v,horizontalSub:p,verticalSub:p},E=(o=y["".concat(e).concat(t?"":"Sub")])===null||o===void 0?void 0:o[r];switch(E){case l:return{offset:-1,sibling:!0};case c:return{offset:1,sibling:!0};case m:return{offset:-1,sibling:!1};case d:return{offset:1,sibling:!1};default:return null}}function iL(e){for(var t=e;t;){if(t.getAttribute("data-menu-list"))return t;t=t.parentElement}return null}function lL(e,t){for(var n=e||document.activeElement;n;){if(t.has(n))return n;n=n.parentElement}return null}function P0(e,t){var n=aL(e,!0);return n.filter(function(r){return t.has(r)})}function Kx(e,t,n){var r=arguments.length>3&&arguments[3]!==void 0?arguments[3]:1;if(!e)return null;var o=P0(e,t),l=o.length,c=o.findIndex(function(d){return n===d});return r<0?c===-1?c=l-1:c-=1:r>0&&(c+=1),c=(c+l)%l,o[c]}var Op=function(t,n){var r=new Set,o=new Map,l=new Map;return t.forEach(function(c){var d=document.querySelector("[data-menu-id='".concat(f2(n,c),"']"));d&&(r.add(d),l.set(d,c),o.set(c,d))}),{elements:r,key2element:o,element2key:l}};function sL(e,t,n,r,o,l,c,d,m,v){var g=u.useRef(),p=u.useRef();p.current=t;var y=function(){gn.cancel(g.current)};return u.useEffect(function(){return function(){y()}},[]),function(E){var C=E.which;if([].concat(Gx,[Bf,h2,lc,sc]).includes(C)){var x=l(),b=Op(x,r),w=b,$=w.elements,O=w.key2element,T=w.element2key,R=O.get(t),M=lL(R,$),N=T.get(M),z=oL(e,c(N,!0).length===1,n,C);if(!z&&C!==lc&&C!==sc)return;(Gx.includes(C)||[lc,sc].includes(C))&&E.preventDefault();var P=function(j){if(j){var B=j,k=j.querySelector("a");k!=null&&k.getAttribute("href")&&(B=k);var I=T.get(j);d(I),y(),g.current=gn(function(){p.current===I&&B.focus()})}};if([lc,sc].includes(C)||z.sibling||!M){var D;!M||e==="inline"?D=o.current:D=iL(M);var F,W=P0(D,$);C===lc?F=W[0]:C===sc?F=W[W.length-1]:F=Kx(D,$,M,z.offset),P(F)}else if(z.inlineTrigger)m(N);else if(z.offset>0)m(N,!0),y(),g.current=gn(function(){b=Op(x,r);var _=M.getAttribute("aria-controls"),j=document.getElementById(_),B=Kx(j,b.elements);P(B)},5);else if(z.offset<0){var V=c(N,!0),q=V[V.length-2],K=O.get(q);m(q,!1),P(K)}}v==null||v(E)}}function cL(e){Promise.resolve().then(e)}var z0="__RC_UTIL_PATH_SPLIT__",Xx=function(t){return t.join(z0)},uL=function(t){return t.split(z0)},Rp="rc-menu-more";function fL(){var e=u.useState({}),t=de(e,2),n=t[1],r=u.useRef(new Map),o=u.useRef(new Map),l=u.useState([]),c=de(l,2),d=c[0],m=c[1],v=u.useRef(0),g=u.useRef(!1),p=function(){g.current||n({})},y=u.useCallback(function(O,T){var R=Xx(T);o.current.set(R,O),r.current.set(O,R),v.current+=1;var M=v.current;cL(function(){M===v.current&&p()})},[]),E=u.useCallback(function(O,T){var R=Xx(T);o.current.delete(R),r.current.delete(O)},[]),C=u.useCallback(function(O){m(O)},[]),x=u.useCallback(function(O,T){var R=r.current.get(O)||"",M=uL(R);return T&&d.includes(M[0])&&M.unshift(Rp),M},[d]),b=u.useCallback(function(O,T){return O.filter(function(R){return R!==void 0}).some(function(R){var M=x(R,!0);return M.includes(T)})},[x]),w=function(){var T=Te(r.current.keys());return d.length&&T.push(Rp),T},$=u.useCallback(function(O){var T="".concat(r.current.get(O)).concat(z0),R=new Set;return Te(o.current.keys()).forEach(function(M){M.startsWith(T)&&R.add(o.current.get(M))}),R},[]);return u.useEffect(function(){return function(){g.current=!0}},[]),{registerPath:y,unregisterPath:E,refreshOverflowKeys:C,isSubPathKey:b,getKeyPath:x,getKeys:w,getSubPathKeys:$}}function vc(e){var t=u.useRef(e);t.current=e;var n=u.useCallback(function(){for(var r,o=arguments.length,l=new Array(o),c=0;c<o;c++)l[c]=arguments[c];return(r=t.current)===null||r===void 0?void 0:r.call.apply(r,[t].concat(l))},[]);return e?n:void 0}var dL=Math.random().toFixed(5).toString().slice(2),Yx=0;function mL(e){var t=gr(e,{value:e}),n=de(t,2),r=n[0],o=n[1];return u.useEffect(function(){Yx+=1;var l="".concat(dL,"-").concat(Yx);o("rc-menu-uuid-".concat(l))},[]),r}function p2(e,t,n,r){var o=u.useContext(Na),l=o.activeKey,c=o.onActive,d=o.onInactive,m={active:l===e};return t||(m.onMouseEnter=function(v){n==null||n({key:e,domEvent:v}),c(e)},m.onMouseLeave=function(v){r==null||r({key:e,domEvent:v}),d(e)}),m}function y2(e){var t=u.useContext(Na),n=t.mode,r=t.rtl,o=t.inlineIndent;if(n!=="inline")return null;var l=e;return r?{paddingRight:l*o}:{paddingLeft:l*o}}function b2(e){var t=e.icon,n=e.props,r=e.children,o;return t===null||t===!1?null:(typeof t=="function"?o=u.createElement(t,Z({},n)):typeof t!="boolean"&&(o=t),o||r||null)}var vL=["item"];function id(e){var t=e.item,n=wt(e,vL);return Object.defineProperty(n,"item",{get:function(){return vr(!1,"`info.item` is deprecated since we will move to function component that not provides React Node instance in future."),t}}),n}var gL=["title","attribute","elementRef"],hL=["style","className","eventKey","warnKey","disabled","itemIcon","children","role","onMouseEnter","onMouseLeave","onClick","onKeyDown","onFocus"],pL=["active"],yL=function(e){$o(n,e);var t=Oo(n);function n(){return qn(this,n),t.apply(this,arguments)}return Wn(n,[{key:"render",value:function(){var o=this.props,l=o.title,c=o.attribute,d=o.elementRef,m=wt(o,gL),v=Tn(m,["eventKey","popupClassName","popupOffset","onTitleClick"]);return vr(!c,"`attribute` of Menu.Item is deprecated. Please pass attribute directly."),u.createElement(ka.Item,Ie({},c,{title:typeof l=="string"?l:void 0},v,{ref:d}))}}]),n}(u.Component),bL=u.forwardRef(function(e,t){var n=e.style,r=e.className,o=e.eventKey;e.warnKey;var l=e.disabled,c=e.itemIcon,d=e.children,m=e.role,v=e.onMouseEnter,g=e.onMouseLeave,p=e.onClick,y=e.onKeyDown,E=e.onFocus,C=wt(e,hL),x=d2(o),b=u.useContext(Na),w=b.prefixCls,$=b.onItemClick,O=b.disabled,T=b.overflowDisabled,R=b.itemIcon,M=b.selectedKeys,N=b.onActive,z=u.useContext(j0),P=z._internalRenderMenuItem,D="".concat(w,"-item"),F=u.useRef(),W=u.useRef(),V=O||l,q=li(t,W),K=as(o),_=function(le){return{key:o,keyPath:Te(K).reverse(),item:F.current,domEvent:le}},j=c||R,B=p2(o,V,v,g),k=B.active,I=wt(B,pL),H=M.includes(o),G=y2(K.length),X=function(le){if(!V){var be=_(le);p==null||p(id(be)),$(be)}},U=function(le){if(y==null||y(le),le.which===ct.ENTER){var be=_(le);p==null||p(id(be)),$(be)}},J=function(le){N(o),E==null||E(le)},te={};e.role==="option"&&(te["aria-selected"]=H);var ve=u.createElement(yL,Ie({ref:F,elementRef:q,role:m===null?"none":m||"menuitem",tabIndex:l?null:-1,"data-menu-id":T&&x?null:x},Tn(C,["extra"]),I,te,{component:"li","aria-disabled":l,style:Z(Z({},G),n),className:re(D,Y(Y(Y({},"".concat(D,"-active"),k),"".concat(D,"-selected"),H),"".concat(D,"-disabled"),V),r),onClick:X,onKeyDown:U,onFocus:J}),d,u.createElement(b2,{props:Z(Z({},e),{},{isSelected:H}),icon:j}));return P&&(ve=P(ve,e,{selected:H})),ve});function SL(e,t){var n=e.eventKey,r=Kd(),o=as(n);return u.useEffect(function(){if(r)return r.registerPath(n,o),function(){r.unregisterPath(n,o)}},[o]),r?null:u.createElement(bL,Ie({},e,{ref:t}))}const Xd=u.forwardRef(SL);var CL=["className","children"],xL=function(t,n){var r=t.className,o=t.children,l=wt(t,CL),c=u.useContext(Na),d=c.prefixCls,m=c.mode,v=c.rtl;return u.createElement("ul",Ie({className:re(d,v&&"".concat(d,"-rtl"),"".concat(d,"-sub"),"".concat(d,"-").concat(m==="inline"?"inline":"vertical"),r),role:"menu"},l,{"data-menu-list":!0,ref:n}),o)},D0=u.forwardRef(xL);D0.displayName="SubMenuList";function B0(e,t){return Pr(e).map(function(n,r){if(u.isValidElement(n)){var o,l,c=n.key,d=(o=(l=n.props)===null||l===void 0?void 0:l.eventKey)!==null&&o!==void 0?o:c,m=d==null;m&&(d="tmp_key-".concat([].concat(Te(t),[r]).join("-")));var v={key:d,eventKey:d};return u.cloneElement(n,v)}return n})}var br={adjustX:1,adjustY:1},EL={topLeft:{points:["bl","tl"],overflow:br},topRight:{points:["br","tr"],overflow:br},bottomLeft:{points:["tl","bl"],overflow:br},bottomRight:{points:["tr","br"],overflow:br},leftTop:{points:["tr","tl"],overflow:br},leftBottom:{points:["br","bl"],overflow:br},rightTop:{points:["tl","tr"],overflow:br},rightBottom:{points:["bl","br"],overflow:br}},wL={topLeft:{points:["bl","tl"],overflow:br},topRight:{points:["br","tr"],overflow:br},bottomLeft:{points:["tl","bl"],overflow:br},bottomRight:{points:["tr","br"],overflow:br},rightTop:{points:["tr","tl"],overflow:br},rightBottom:{points:["br","bl"],overflow:br},leftTop:{points:["tl","tr"],overflow:br},leftBottom:{points:["bl","br"],overflow:br}};function S2(e,t,n){if(t)return t;if(n)return n[e]||n.other}var $L={horizontal:"bottomLeft",vertical:"rightTop","vertical-left":"rightTop","vertical-right":"leftTop"};function OL(e){var t=e.prefixCls,n=e.visible,r=e.children,o=e.popup,l=e.popupStyle,c=e.popupClassName,d=e.popupOffset,m=e.disabled,v=e.mode,g=e.onVisibleChange,p=u.useContext(Na),y=p.getPopupContainer,E=p.rtl,C=p.subMenuOpenDelay,x=p.subMenuCloseDelay,b=p.builtinPlacements,w=p.triggerSubMenuAction,$=p.forceSubMenuRender,O=p.rootClassName,T=p.motion,R=p.defaultMotions,M=u.useState(!1),N=de(M,2),z=N[0],P=N[1],D=Z(E?Z({},wL):Z({},EL),b),F=$L[v],W=S2(v,T,R),V=u.useRef(W);v!=="inline"&&(V.current=W);var q=Z(Z({},V.current),{},{leavedClassName:"".concat(t,"-hidden"),removeOnLeave:!1,motionAppear:!0}),K=u.useRef();return u.useEffect(function(){return K.current=gn(function(){P(n)}),function(){gn.cancel(K.current)}},[n]),u.createElement(O0,{prefixCls:t,popupClassName:re("".concat(t,"-popup"),Y({},"".concat(t,"-rtl"),E),c,O),stretch:v==="horizontal"?"minWidth":null,getPopupContainer:y,builtinPlacements:D,popupPlacement:F,popupVisible:z,popup:o,popupStyle:l,popupAlign:d&&{offset:d},action:m?[]:[w],mouseEnterDelay:C,mouseLeaveDelay:x,onPopupVisibleChange:g,forceRender:$,popupMotion:q,fresh:!0},r)}function RL(e){var t=e.id,n=e.open,r=e.keyPath,o=e.children,l="inline",c=u.useContext(Na),d=c.prefixCls,m=c.forceSubMenuRender,v=c.motion,g=c.defaultMotions,p=c.mode,y=u.useRef(!1);y.current=p===l;var E=u.useState(!y.current),C=de(E,2),x=C[0],b=C[1],w=y.current?n:!1;u.useEffect(function(){y.current&&b(!1)},[p]);var $=Z({},S2(l,v,g));r.length>1&&($.motionAppear=!1);var O=$.onVisibleChanged;return $.onVisibleChanged=function(T){return!y.current&&!T&&b(!0),O==null?void 0:O(T)},x?null:u.createElement(Pc,{mode:l,locked:!y.current},u.createElement(Xa,Ie({visible:w},$,{forceRender:m,removeOnLeave:!1,leavedClassName:"".concat(d,"-hidden")}),function(T){var R=T.className,M=T.style;return u.createElement(D0,{id:t,className:R,style:M},o)}))}var TL=["style","className","title","eventKey","warnKey","disabled","internalPopupClose","children","itemIcon","expandIcon","popupClassName","popupOffset","popupStyle","onClick","onMouseEnter","onMouseLeave","onTitleClick","onTitleMouseEnter","onTitleMouseLeave"],ML=["active"],_L=u.forwardRef(function(e,t){var n=e.style,r=e.className,o=e.title,l=e.eventKey;e.warnKey;var c=e.disabled,d=e.internalPopupClose,m=e.children,v=e.itemIcon,g=e.expandIcon,p=e.popupClassName,y=e.popupOffset,E=e.popupStyle,C=e.onClick,x=e.onMouseEnter,b=e.onMouseLeave,w=e.onTitleClick,$=e.onTitleMouseEnter,O=e.onTitleMouseLeave,T=wt(e,TL),R=d2(l),M=u.useContext(Na),N=M.prefixCls,z=M.mode,P=M.openKeys,D=M.disabled,F=M.overflowDisabled,W=M.activeKey,V=M.selectedKeys,q=M.itemIcon,K=M.expandIcon,_=M.onItemClick,j=M.onOpenChange,B=M.onActive,k=u.useContext(j0),I=k._internalRenderSubMenuItem,H=u.useContext(g2),G=H.isSubPathKey,X=as(),U="".concat(N,"-submenu"),J=D||c,te=u.useRef(),ve=u.useRef(),ne=v??q,le=g??K,be=P.includes(l),fe=!F&&be,ie=G(V,l),pe=p2(l,J,$,O),Re=pe.active,se=wt(pe,ML),ye=u.useState(!1),$e=de(ye,2),we=$e[0],Ne=$e[1],Ze=function(_e){J||Ne(_e)},He=function(_e){Ze(!0),x==null||x({key:l,domEvent:_e})},Xe=function(_e){Ze(!1),b==null||b({key:l,domEvent:_e})},xe=u.useMemo(function(){return Re||(z!=="inline"?we||G([W],l):!1)},[z,Re,W,we,l,G]),Me=y2(X.length),Ve=function(_e){J||(w==null||w({key:l,domEvent:_e}),z==="inline"&&j(l,!be))},Ke=vc(function(Fe){C==null||C(id(Fe)),_(Fe)}),We=function(_e){z!=="inline"&&j(l,_e)},at=function(){B(l)},Ue=R&&"".concat(R,"-popup"),Ye=u.useMemo(function(){return u.createElement(b2,{icon:z!=="horizontal"?le:void 0,props:Z(Z({},e),{},{isOpen:fe,isSubMenu:!0})},u.createElement("i",{className:"".concat(U,"-arrow")}))},[z,le,e,fe,U]),st=u.createElement("div",Ie({role:"menuitem",style:Me,className:"".concat(U,"-title"),tabIndex:J?null:-1,ref:te,title:typeof o=="string"?o:null,"data-menu-id":F&&R?null:R,"aria-expanded":fe,"aria-haspopup":!0,"aria-controls":Ue,"aria-disabled":J,onClick:Ve,onFocus:at},se),o,Ye),ht=u.useRef(z);if(z!=="inline"&&X.length>1?ht.current="vertical":ht.current=z,!F){var Qe=ht.current;st=u.createElement(OL,{mode:Qe,prefixCls:U,visible:!d&&fe&&z!=="inline",popupClassName:p,popupOffset:y,popupStyle:E,popup:u.createElement(Pc,{mode:Qe==="horizontal"?"vertical":Qe},u.createElement(D0,{id:Ue,ref:ve},m)),disabled:J,onVisibleChange:We},st)}var Pe=u.createElement(ka.Item,Ie({ref:t,role:"none"},T,{component:"li",style:n,className:re(U,"".concat(U,"-").concat(z),r,Y(Y(Y(Y({},"".concat(U,"-open"),fe),"".concat(U,"-active"),xe),"".concat(U,"-selected"),ie),"".concat(U,"-disabled"),J)),onMouseEnter:He,onMouseLeave:Xe}),st,!F&&u.createElement(RL,{id:Ue,open:fe,keyPath:X},m));return I&&(Pe=I(Pe,e,{selected:ie,active:xe,open:fe,disabled:J})),u.createElement(Pc,{onItemClick:Ke,mode:z==="horizontal"?"vertical":z,itemIcon:ne,expandIcon:le},Pe)}),Yd=u.forwardRef(function(e,t){var n=e.eventKey,r=e.children,o=as(n),l=B0(r,o),c=Kd();u.useEffect(function(){if(c)return c.registerPath(n,o),function(){c.unregisterPath(n,o)}},[o]);var d;return c?d=l:d=u.createElement(_L,Ie({ref:t},e),l),u.createElement(v2.Provider,{value:o},d)});function L0(e){var t=e.className,n=e.style,r=u.useContext(Na),o=r.prefixCls,l=Kd();return l?null:u.createElement("li",{role:"separator",className:re("".concat(o,"-item-divider"),t),style:n})}var NL=["className","title","eventKey","children"],AL=u.forwardRef(function(e,t){var n=e.className,r=e.title;e.eventKey;var o=e.children,l=wt(e,NL),c=u.useContext(Na),d=c.prefixCls,m="".concat(d,"-item-group");return u.createElement("li",Ie({ref:t,role:"presentation"},l,{onClick:function(g){return g.stopPropagation()},className:re(m,n)}),u.createElement("div",{role:"presentation",className:"".concat(m,"-title"),title:typeof r=="string"?r:void 0},r),u.createElement("ul",{role:"group",className:"".concat(m,"-list")},o))}),H0=u.forwardRef(function(e,t){var n=e.eventKey,r=e.children,o=as(n),l=B0(r,o),c=Kd();return c?l:u.createElement(AL,Ie({ref:t},Tn(e,["warnKey"])),l)}),IL=["label","children","key","type","extra"];function Tp(e,t,n){var r=t.item,o=t.group,l=t.submenu,c=t.divider;return(e||[]).map(function(d,m){if(d&&lt(d)==="object"){var v=d,g=v.label,p=v.children,y=v.key,E=v.type,C=v.extra,x=wt(v,IL),b=y??"tmp-".concat(m);return p||E==="group"?E==="group"?u.createElement(o,Ie({key:b},x,{title:g}),Tp(p,t,n)):u.createElement(l,Ie({key:b},x,{title:g}),Tp(p,t,n)):E==="divider"?u.createElement(c,Ie({key:b},x)):u.createElement(r,Ie({key:b},x,{extra:C}),g,(!!C||C===0)&&u.createElement("span",{className:"".concat(n,"-item-extra")},C))}return null}).filter(function(d){return d})}function Qx(e,t,n,r,o){var l=e,c=Z({divider:L0,item:Xd,group:H0,submenu:Yd},r);return t&&(l=Tp(t,c,o)),B0(l,n)}var jL=["prefixCls","rootClassName","style","className","tabIndex","items","children","direction","id","mode","inlineCollapsed","disabled","disabledOverflow","subMenuOpenDelay","subMenuCloseDelay","forceSubMenuRender","defaultOpenKeys","openKeys","activeKey","defaultActiveFirst","selectable","multiple","defaultSelectedKeys","selectedKeys","onSelect","onDeselect","inlineIndent","motion","defaultMotions","triggerSubMenuAction","builtinPlacements","itemIcon","expandIcon","overflowedIndicator","overflowedIndicatorPopupClassName","getPopupContainer","onClick","onOpenChange","onKeyDown","openAnimation","openTransitionName","_internalRenderMenuItem","_internalRenderSubMenuItem","_internalComponents"],Mi=[],PL=u.forwardRef(function(e,t){var n,r=e,o=r.prefixCls,l=o===void 0?"rc-menu":o,c=r.rootClassName,d=r.style,m=r.className,v=r.tabIndex,g=v===void 0?0:v,p=r.items,y=r.children,E=r.direction,C=r.id,x=r.mode,b=x===void 0?"vertical":x,w=r.inlineCollapsed,$=r.disabled,O=r.disabledOverflow,T=r.subMenuOpenDelay,R=T===void 0?.1:T,M=r.subMenuCloseDelay,N=M===void 0?.1:M,z=r.forceSubMenuRender,P=r.defaultOpenKeys,D=r.openKeys,F=r.activeKey,W=r.defaultActiveFirst,V=r.selectable,q=V===void 0?!0:V,K=r.multiple,_=K===void 0?!1:K,j=r.defaultSelectedKeys,B=r.selectedKeys,k=r.onSelect,I=r.onDeselect,H=r.inlineIndent,G=H===void 0?24:H,X=r.motion,U=r.defaultMotions,J=r.triggerSubMenuAction,te=J===void 0?"hover":J,ve=r.builtinPlacements,ne=r.itemIcon,le=r.expandIcon,be=r.overflowedIndicator,fe=be===void 0?"...":be,ie=r.overflowedIndicatorPopupClassName,pe=r.getPopupContainer,Re=r.onClick,se=r.onOpenChange,ye=r.onKeyDown;r.openAnimation,r.openTransitionName;var $e=r._internalRenderMenuItem,we=r._internalRenderSubMenuItem,Ne=r._internalComponents,Ze=wt(r,jL),He=u.useMemo(function(){return[Qx(y,p,Mi,Ne,l),Qx(y,p,Mi,{},l)]},[y,p,Ne]),Xe=de(He,2),xe=Xe[0],Me=Xe[1],Ve=u.useState(!1),Ke=de(Ve,2),We=Ke[0],at=Ke[1],Ue=u.useRef(),Ye=mL(C),st=E==="rtl",ht=gr(P,{value:D,postState:function(Je){return Je||Mi}}),Qe=de(ht,2),Pe=Qe[0],Fe=Qe[1],_e=function(Je){var ze=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1;function Lt(){Fe(Je),se==null||se(Je)}ze?ri.flushSync(Lt):Lt()},qe=u.useState(Pe),Ct=de(qe,2),Nt=Ct[0],_n=Ct[1],bn=u.useRef(!1),$n=u.useMemo(function(){return(b==="inline"||b==="vertical")&&w?["vertical",w]:[b,!1]},[b,w]),sn=de($n,2),Ft=sn[0],nn=sn[1],Et=Ft==="inline",nt=u.useState(Ft),ke=de(nt,2),De=ke[0],pt=ke[1],St=u.useState(nn),Be=de(St,2),je=Be[0],rt=Be[1];u.useEffect(function(){pt(Ft),rt(nn),bn.current&&(Et?Fe(Nt):_e(Mi))},[Ft,nn]);var ot=u.useState(0),tt=de(ot,2),ut=tt[0],Vt=tt[1],Tt=ut>=xe.length-1||De!=="horizontal"||O;u.useEffect(function(){Et&&_n(Pe)},[Pe]),u.useEffect(function(){return bn.current=!0,function(){bn.current=!1}},[]);var At=fL(),It=At.registerPath,Qt=At.unregisterPath,rn=At.refreshOverflowKeys,Rt=At.isSubPathKey,er=At.getKeyPath,pn=At.getKeys,an=At.getSubPathKeys,Ae=u.useMemo(function(){return{registerPath:It,unregisterPath:Qt}},[It,Qt]),et=u.useMemo(function(){return{isSubPathKey:Rt}},[Rt]);u.useEffect(function(){rn(Tt?Mi:xe.slice(ut+1).map(function(Bt){return Bt.key}))},[ut,Tt]);var yt=gr(F||W&&((n=xe[0])===null||n===void 0?void 0:n.key),{value:F}),xt=de(yt,2),Gt=xt[0],kn=xt[1],Pn=vc(function(Bt){kn(Bt)}),Ln=vc(function(){kn(void 0)});u.useImperativeHandle(t,function(){return{list:Ue.current,focus:function(Je){var ze,Lt=pn(),Ut=Op(Lt,Ye),Kt=Ut.elements,Xt=Ut.key2element,nr=Ut.element2key,Hn=P0(Ue.current,Kt),Nn=Gt??(Hn[0]?nr.get(Hn[0]):(ze=xe.find(function(xr){return!xr.props.disabled}))===null||ze===void 0?void 0:ze.key),rr=Xt.get(Nn);if(Nn&&rr){var ar;rr==null||(ar=rr.focus)===null||ar===void 0||ar.call(rr,Je)}}}});var cr=gr(j||[],{value:B,postState:function(Je){return Array.isArray(Je)?Je:Je==null?Mi:[Je]}}),hr=de(cr,2),On=hr[0],tr=hr[1],Dr=function(Je){if(q){var ze=Je.key,Lt=On.includes(ze),Ut;_?Lt?Ut=On.filter(function(Xt){return Xt!==ze}):Ut=[].concat(Te(On),[ze]):Ut=[ze],tr(Ut);var Kt=Z(Z({},Je),{},{selectedKeys:Ut});Lt?I==null||I(Kt):k==null||k(Kt)}!_&&Pe.length&&De!=="inline"&&_e(Mi)},Da=vc(function(Bt){Re==null||Re(id(Bt)),Dr(Bt)}),Gn=vc(function(Bt,Je){var ze=Pe.filter(function(Ut){return Ut!==Bt});if(Je)ze.push(Bt);else if(De!=="inline"){var Lt=an(Bt);ze=ze.filter(function(Ut){return!Lt.has(Ut)})}Oc(Pe,ze,!0)||_e(ze,!0)}),cn=function(Je,ze){var Lt=ze??!Pe.includes(Je);Gn(Je,Lt)},Br=sL(De,Gt,st,Ye,Ue,pn,er,kn,cn,ye);u.useEffect(function(){at(!0)},[]);var ra=u.useMemo(function(){return{_internalRenderMenuItem:$e,_internalRenderSubMenuItem:we}},[$e,we]),Kn=De!=="horizontal"||O?xe:xe.map(function(Bt,Je){return u.createElement(Pc,{key:Bt.key,overflowDisabled:Je>ut},Bt)}),Ca=u.createElement(ka,Ie({id:C,ref:Ue,prefixCls:"".concat(l,"-overflow"),component:"ul",itemComponent:Xd,className:re(l,"".concat(l,"-root"),"".concat(l,"-").concat(De),m,Y(Y({},"".concat(l,"-inline-collapsed"),je),"".concat(l,"-rtl"),st),c),dir:E,style:d,role:"menu",tabIndex:g,data:Kn,renderRawItem:function(Je){return Je},renderRawRest:function(Je){var ze=Je.length,Lt=ze?xe.slice(-ze):null;return u.createElement(Yd,{eventKey:Rp,title:fe,disabled:Tt,internalPopupClose:ze===0,popupClassName:ie},Lt)},maxCount:De!=="horizontal"||O?ka.INVALIDATE:ka.RESPONSIVE,ssr:"full","data-menu-list":!0,onVisibleChange:function(Je){Vt(Je)},onKeyDown:Br},Ze));return u.createElement(j0.Provider,{value:ra},u.createElement(u2.Provider,{value:Ye},u.createElement(Pc,{prefixCls:l,rootClassName:c,mode:De,openKeys:Pe,rtl:st,disabled:$,motion:We?X:null,defaultMotions:We?U:null,activeKey:Gt,onActive:Pn,onInactive:Ln,selectedKeys:On,inlineIndent:G,subMenuOpenDelay:R,subMenuCloseDelay:N,forceSubMenuRender:z,builtinPlacements:ve,triggerSubMenuAction:te,getPopupContainer:pe,itemIcon:ne,expandIcon:le,onItemClick:Da,onOpenChange:Gn},u.createElement(g2.Provider,{value:et},Ca),u.createElement("div",{style:{display:"none"},"aria-hidden":!0},u.createElement(m2.Provider,{value:Ae},Me)))))}),eu=PL;eu.Item=Xd;eu.SubMenu=Yd;eu.ItemGroup=H0;eu.Divider=L0;const C2=u.createContext({siderHook:{addSider:()=>null,removeSider:()=>null}}),zL=e=>{const{antCls:t,componentCls:n,colorText:r,footerBg:o,headerHeight:l,headerPadding:c,headerColor:d,footerPadding:m,fontSize:v,bodyBg:g,headerBg:p}=e;return{[n]:{display:"flex",flex:"auto",flexDirection:"column",minHeight:0,background:g,"&, *":{boxSizing:"border-box"},[`&${n}-has-sider`]:{flexDirection:"row",[`> ${n}, > ${n}-content`]:{width:0}},[`${n}-header, &${n}-footer`]:{flex:"0 0 auto"},"&-rtl":{direction:"rtl"}},[`${n}-header`]:{height:l,padding:c,color:d,lineHeight:he(l),background:p,[`${t}-menu`]:{lineHeight:"inherit"}},[`${n}-footer`]:{padding:m,color:r,fontSize:v,background:o},[`${n}-content`]:{flex:"auto",color:r,minHeight:0}}},x2=e=>{const{colorBgLayout:t,controlHeight:n,controlHeightLG:r,colorText:o,controlHeightSM:l,marginXXS:c,colorTextLightSolid:d,colorBgContainer:m}=e,v=r*1.25;return{colorBgHeader:"#001529",colorBgBody:t,colorBgTrigger:"#002140",bodyBg:t,headerBg:"#001529",headerHeight:n*2,headerPadding:`0 ${v}px`,headerColor:o,footerPadding:`${l}px ${v}px`,footerBg:t,siderBg:"#001529",triggerHeight:r+c*2,triggerBg:"#002140",triggerColor:d,zeroTriggerWidth:r,zeroTriggerHeight:r,lightSiderBg:m,lightTriggerBg:m,lightTriggerColor:o}},E2=[["colorBgBody","bodyBg"],["colorBgHeader","headerBg"],["colorBgTrigger","triggerBg"]],w2=wn("Layout",e=>[zL(e)],x2,{deprecatedTokens:E2}),DL=e=>{const{componentCls:t,siderBg:n,motionDurationMid:r,motionDurationSlow:o,antCls:l,triggerHeight:c,triggerColor:d,triggerBg:m,headerHeight:v,zeroTriggerWidth:g,zeroTriggerHeight:p,borderRadiusLG:y,lightSiderBg:E,lightTriggerColor:C,lightTriggerBg:x,bodyBg:b}=e;return{[t]:{position:"relative",minWidth:0,background:n,transition:`all ${r}, background 0s`,"&-has-trigger":{paddingBottom:c},"&-right":{order:1},[`${t}-children`]:{height:"100%",marginTop:-.1,paddingTop:.1,[`${l}-menu${l}-menu-inline-collapsed`]:{width:"auto"}},[`&-zero-width ${t}-children`]:{overflow:"hidden"},[`${t}-trigger`]:{position:"fixed",bottom:0,zIndex:1,height:c,color:d,lineHeight:he(c),textAlign:"center",background:m,cursor:"pointer",transition:`all ${r}`},[`${t}-zero-width-trigger`]:{position:"absolute",top:v,insetInlineEnd:e.calc(g).mul(-1).equal(),zIndex:1,width:g,height:p,color:d,fontSize:e.fontSizeXL,display:"flex",alignItems:"center",justifyContent:"center",background:n,borderRadius:`0 ${he(y)} ${he(y)} 0`,cursor:"pointer",transition:`background ${o} ease`,"&::after":{position:"absolute",inset:0,background:"transparent",transition:`all ${o}`,content:'""'},"&:hover::after":{background:"rgba(255, 255, 255, 0.2)"},"&-right":{insetInlineStart:e.calc(g).mul(-1).equal(),borderRadius:`${he(y)} 0 0 ${he(y)}`}},"&-light":{background:E,[`${t}-trigger`]:{color:C,background:x},[`${t}-zero-width-trigger`]:{color:C,background:x,border:`1px solid ${b}`,borderInlineStart:0}}}}},BL=wn(["Layout","Sider"],e=>[DL(e)],x2,{deprecatedTokens:E2});var LL=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const Zx={xs:"479.98px",sm:"575.98px",md:"767.98px",lg:"991.98px",xl:"1199.98px",xxl:"1599.98px"},HL=e=>!Number.isNaN(Number.parseFloat(e))&&isFinite(e),Qd=u.createContext({}),FL=(()=>{let e=0;return function(){let t=arguments.length>0&&arguments[0]!==void 0?arguments[0]:"";return e+=1,`${t}${e}`}})(),$2=u.forwardRef((e,t)=>{const{prefixCls:n,className:r,trigger:o,children:l,defaultCollapsed:c=!1,theme:d="dark",style:m={},collapsible:v=!1,reverseArrow:g=!1,width:p=200,collapsedWidth:y=80,zeroWidthTriggerStyle:E,breakpoint:C,onCollapse:x,onBreakpoint:b}=e,w=LL(e,["prefixCls","className","trigger","children","defaultCollapsed","theme","style","collapsible","reverseArrow","width","collapsedWidth","zeroWidthTriggerStyle","breakpoint","onCollapse","onBreakpoint"]),{siderHook:$}=u.useContext(C2),[O,T]=u.useState("collapsed"in e?e.collapsed:c),[R,M]=u.useState(!1);u.useEffect(()=>{"collapsed"in e&&T(e.collapsed)},[e.collapsed]);const N=(ne,le)=>{"collapsed"in e||T(ne),x==null||x(ne,le)},{getPrefixCls:z,direction:P}=u.useContext(vt),D=z("layout-sider",n),[F,W,V]=BL(D),q=u.useRef(null);q.current=ne=>{M(ne.matches),b==null||b(ne.matches),O!==ne.matches&&N(ne.matches,"responsive")},u.useEffect(()=>{function ne(be){return q.current(be)}let le;if(typeof window<"u"){const{matchMedia:be}=window;if(be&&C&&C in Zx){le=be(`screen and (max-width: ${Zx[C]})`);try{le.addEventListener("change",ne)}catch{le.addListener(ne)}ne(le)}}return()=>{try{le==null||le.removeEventListener("change",ne)}catch{le==null||le.removeListener(ne)}}},[C]),u.useEffect(()=>{const ne=FL("ant-sider-");return $.addSider(ne),()=>$.removeSider(ne)},[]);const K=()=>{N(!O,"clickTrigger")},_=Tn(w,["collapsed"]),j=O?y:p,B=HL(j)?`${j}px`:String(j),k=parseFloat(String(y||0))===0?u.createElement("span",{onClick:K,className:re(`${D}-zero-width-trigger`,`${D}-zero-width-trigger-${g?"right":"left"}`),style:E},o||u.createElement(oN,null)):null,I=P==="rtl"==!g,X={expanded:I?u.createElement(Vh,null):u.createElement(XC,null),collapsed:I?u.createElement(XC,null):u.createElement(Vh,null)}[O?"collapsed":"expanded"],U=o!==null?k||u.createElement("div",{className:`${D}-trigger`,onClick:K,style:{width:B}},o||X):null,J=Object.assign(Object.assign({},m),{flex:`0 0 ${B}`,maxWidth:B,minWidth:B,width:B}),te=re(D,`${D}-${d}`,{[`${D}-collapsed`]:!!O,[`${D}-has-trigger`]:v&&o!==null&&!k,[`${D}-below`]:!!R,[`${D}-zero-width`]:parseFloat(B)===0},r,W,V),ve=u.useMemo(()=>({siderCollapsed:O}),[O]);return F(u.createElement(Qd.Provider,{value:ve},u.createElement("aside",Object.assign({className:te},_,{style:J,ref:t}),u.createElement("div",{className:`${D}-children`},l),v||R&&k?U:null)))}),ld=u.createContext({prefixCls:"",firstLevel:!0,inlineCollapsed:!1});var VL=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const O2=e=>{const{prefixCls:t,className:n,dashed:r}=e,o=VL(e,["prefixCls","className","dashed"]),{getPrefixCls:l}=u.useContext(vt),c=l("menu",t),d=re({[`${c}-item-divider-dashed`]:!!r},n);return u.createElement(L0,Object.assign({className:d},o))},R2=e=>{var t;const{className:n,children:r,icon:o,title:l,danger:c,extra:d}=e,{prefixCls:m,firstLevel:v,direction:g,disableMenuItemTitleTooltip:p,inlineCollapsed:y}=u.useContext(ld),E=O=>{const T=r==null?void 0:r[0],R=u.createElement("span",{className:re(`${m}-title-content`,{[`${m}-title-content-with-extra`]:!!d||d===0})},r);return(!o||u.isValidElement(r)&&r.type==="span")&&r&&O&&v&&typeof T=="string"?u.createElement("div",{className:`${m}-inline-collapsed-noicon`},T.charAt(0)):R},{siderCollapsed:C}=u.useContext(Qd);let x=l;typeof l>"u"?x=v?r:"":l===!1&&(x="");const b={title:x};!C&&!y&&(b.title=null,b.open=!1);const w=Pr(r).length;let $=u.createElement(Xd,Object.assign({},Tn(e,["title","icon","danger"]),{className:re({[`${m}-item-danger`]:c,[`${m}-item-only-child`]:(o?w+1:w)===1},n),title:typeof l=="string"?l:void 0}),Zr(o,{className:re(u.isValidElement(o)?(t=o.props)===null||t===void 0?void 0:t.className:"",`${m}-item-icon`)}),E(y));return p||($=u.createElement(si,Object.assign({},b,{placement:g==="rtl"?"left":"right",classNames:{root:`${m}-inline-collapsed-tooltip`}}),$)),$};var UL=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const sd=u.createContext(null),cW=u.forwardRef((e,t)=>{const{children:n}=e,r=UL(e,["children"]),o=u.useContext(sd),l=u.useMemo(()=>Object.assign(Object.assign({},o),r),[o,r.prefixCls,r.mode,r.selectable,r.rootClassName]),c=fA(n),d=li(t,c?Zl(n):null);return u.createElement(sd.Provider,{value:l},u.createElement(Ac,{space:!0},c?u.cloneElement(n,{ref:d}):n))}),qL=e=>{const{componentCls:t,motionDurationSlow:n,horizontalLineHeight:r,colorSplit:o,lineWidth:l,lineType:c,itemPaddingInline:d}=e;return{[`${t}-horizontal`]:{lineHeight:r,border:0,borderBottom:`${he(l)} ${c} ${o}`,boxShadow:"none","&::after":{display:"block",clear:"both",height:0,content:'"\\20"'},[`${t}-item, ${t}-submenu`]:{position:"relative",display:"inline-block",verticalAlign:"bottom",paddingInline:d},[`> ${t}-item:hover,
        > ${t}-item-active,
        > ${t}-submenu ${t}-submenu-title:hover`]:{backgroundColor:"transparent"},[`${t}-item, ${t}-submenu-title`]:{transition:[`border-color ${n}`,`background ${n}`].join(",")},[`${t}-submenu-arrow`]:{display:"none"}}}},WL=e=>{let{componentCls:t,menuArrowOffset:n,calc:r}=e;return{[`${t}-rtl`]:{direction:"rtl"},[`${t}-submenu-rtl`]:{transformOrigin:"100% 0"},[`${t}-rtl${t}-vertical,
    ${t}-submenu-rtl ${t}-vertical`]:{[`${t}-submenu-arrow`]:{"&::before":{transform:`rotate(-45deg) translateY(${he(r(n).mul(-1).equal())})`},"&::after":{transform:`rotate(45deg) translateY(${he(n)})`}}}}},Jx=e=>Object.assign({},u$(e)),eE=(e,t)=>{const{componentCls:n,itemColor:r,itemSelectedColor:o,subMenuItemSelectedColor:l,groupTitleColor:c,itemBg:d,subMenuItemBg:m,itemSelectedBg:v,activeBarHeight:g,activeBarWidth:p,activeBarBorderWidth:y,motionDurationSlow:E,motionEaseInOut:C,motionEaseOut:x,itemPaddingInline:b,motionDurationMid:w,itemHoverColor:$,lineType:O,colorSplit:T,itemDisabledColor:R,dangerItemColor:M,dangerItemHoverColor:N,dangerItemSelectedColor:z,dangerItemActiveBg:P,dangerItemSelectedBg:D,popupBg:F,itemHoverBg:W,itemActiveBg:V,menuSubMenuBg:q,horizontalItemSelectedColor:K,horizontalItemSelectedBg:_,horizontalItemBorderRadius:j,horizontalItemHoverBg:B}=e;return{[`${n}-${t}, ${n}-${t} > ${n}`]:{color:r,background:d,[`&${n}-root:focus-visible`]:Object.assign({},Jx(e)),[`${n}-item`]:{"&-group-title, &-extra":{color:c}},[`${n}-submenu-selected > ${n}-submenu-title`]:{color:l},[`${n}-item, ${n}-submenu-title`]:{color:r,[`&:not(${n}-item-disabled):focus-visible`]:Object.assign({},Jx(e))},[`${n}-item-disabled, ${n}-submenu-disabled`]:{color:`${R} !important`},[`${n}-item:not(${n}-item-selected):not(${n}-submenu-selected)`]:{[`&:hover, > ${n}-submenu-title:hover`]:{color:$}},[`&:not(${n}-horizontal)`]:{[`${n}-item:not(${n}-item-selected)`]:{"&:hover":{backgroundColor:W},"&:active":{backgroundColor:V}},[`${n}-submenu-title`]:{"&:hover":{backgroundColor:W},"&:active":{backgroundColor:V}}},[`${n}-item-danger`]:{color:M,[`&${n}-item:hover`]:{[`&:not(${n}-item-selected):not(${n}-submenu-selected)`]:{color:N}},[`&${n}-item:active`]:{background:P}},[`${n}-item a`]:{"&, &:hover":{color:"inherit"}},[`${n}-item-selected`]:{color:o,[`&${n}-item-danger`]:{color:z},"a, a:hover":{color:"inherit"}},[`& ${n}-item-selected`]:{backgroundColor:v,[`&${n}-item-danger`]:{backgroundColor:D}},[`&${n}-submenu > ${n}`]:{backgroundColor:q},[`&${n}-popup > ${n}`]:{backgroundColor:F},[`&${n}-submenu-popup > ${n}`]:{backgroundColor:F},[`&${n}-horizontal`]:Object.assign(Object.assign({},t==="dark"?{borderBottom:0}:{}),{[`> ${n}-item, > ${n}-submenu`]:{top:y,marginTop:e.calc(y).mul(-1).equal(),marginBottom:0,borderRadius:j,"&::after":{position:"absolute",insetInline:b,bottom:0,borderBottom:`${he(g)} solid transparent`,transition:`border-color ${E} ${C}`,content:'""'},"&:hover, &-active, &-open":{background:B,"&::after":{borderBottomWidth:g,borderBottomColor:K}},"&-selected":{color:K,backgroundColor:_,"&:hover":{backgroundColor:_},"&::after":{borderBottomWidth:g,borderBottomColor:K}}}}),[`&${n}-root`]:{[`&${n}-inline, &${n}-vertical`]:{borderInlineEnd:`${he(y)} ${O} ${T}`}},[`&${n}-inline`]:{[`${n}-sub${n}-inline`]:{background:m},[`${n}-item`]:{position:"relative","&::after":{position:"absolute",insetBlock:0,insetInlineEnd:0,borderInlineEnd:`${he(p)} solid ${o}`,transform:"scaleY(0.0001)",opacity:0,transition:[`transform ${w} ${x}`,`opacity ${w} ${x}`].join(","),content:'""'},[`&${n}-item-danger`]:{"&::after":{borderInlineEndColor:z}}},[`${n}-selected, ${n}-item-selected`]:{"&::after":{transform:"scaleY(1)",opacity:1,transition:[`transform ${w} ${C}`,`opacity ${w} ${C}`].join(",")}}}}}},tE=e=>{const{componentCls:t,itemHeight:n,itemMarginInline:r,padding:o,menuArrowSize:l,marginXS:c,itemMarginBlock:d,itemWidth:m,itemPaddingInline:v}=e,g=e.calc(l).add(o).add(c).equal();return{[`${t}-item`]:{position:"relative",overflow:"hidden"},[`${t}-item, ${t}-submenu-title`]:{height:n,lineHeight:he(n),paddingInline:v,overflow:"hidden",textOverflow:"ellipsis",marginInline:r,marginBlock:d,width:m},[`> ${t}-item,
            > ${t}-submenu > ${t}-submenu-title`]:{height:n,lineHeight:he(n)},[`${t}-item-group-list ${t}-submenu-title,
            ${t}-submenu-title`]:{paddingInlineEnd:g}}},kL=e=>{const{componentCls:t,iconCls:n,itemHeight:r,colorTextLightSolid:o,dropdownWidth:l,controlHeightLG:c,motionEaseOut:d,paddingXL:m,itemMarginInline:v,fontSizeLG:g,motionDurationFast:p,motionDurationSlow:y,paddingXS:E,boxShadowSecondary:C,collapsedWidth:x,collapsedIconSize:b}=e,w={height:r,lineHeight:he(r),listStylePosition:"inside",listStyleType:"disc"};return[{[t]:{"&-inline, &-vertical":Object.assign({[`&${t}-root`]:{boxShadow:"none"}},tE(e))},[`${t}-submenu-popup`]:{[`${t}-vertical`]:Object.assign(Object.assign({},tE(e)),{boxShadow:C})}},{[`${t}-submenu-popup ${t}-vertical${t}-sub`]:{minWidth:l,maxHeight:`calc(100vh - ${he(e.calc(c).mul(2.5).equal())})`,padding:"0",overflow:"hidden",borderInlineEnd:0,"&:not([class*='-active'])":{overflowX:"hidden",overflowY:"auto"}}},{[`${t}-inline`]:{width:"100%",[`&${t}-root`]:{[`${t}-item, ${t}-submenu-title`]:{display:"flex",alignItems:"center",transition:[`border-color ${y}`,`background ${y}`,`padding ${p} ${d}`].join(","),[`> ${t}-title-content`]:{flex:"auto",minWidth:0,overflow:"hidden",textOverflow:"ellipsis"},"> *":{flex:"none"}}},[`${t}-sub${t}-inline`]:{padding:0,border:0,borderRadius:0,boxShadow:"none",[`& > ${t}-submenu > ${t}-submenu-title`]:w,[`& ${t}-item-group-title`]:{paddingInlineStart:m}},[`${t}-item`]:w}},{[`${t}-inline-collapsed`]:{width:x,[`&${t}-root`]:{[`${t}-item, ${t}-submenu ${t}-submenu-title`]:{[`> ${t}-inline-collapsed-noicon`]:{fontSize:g,textAlign:"center"}}},[`> ${t}-item,
          > ${t}-item-group > ${t}-item-group-list > ${t}-item,
          > ${t}-item-group > ${t}-item-group-list > ${t}-submenu > ${t}-submenu-title,
          > ${t}-submenu > ${t}-submenu-title`]:{insetInlineStart:0,paddingInline:`calc(50% - ${he(e.calc(b).div(2).equal())} - ${he(v)})`,textOverflow:"clip",[`
            ${t}-submenu-arrow,
            ${t}-submenu-expand-icon
          `]:{opacity:0},[`${t}-item-icon, ${n}`]:{margin:0,fontSize:b,lineHeight:he(r),"+ span":{display:"inline-block",opacity:0}}},[`${t}-item-icon, ${n}`]:{display:"inline-block"},"&-tooltip":{pointerEvents:"none",[`${t}-item-icon, ${n}`]:{display:"none"},"a, a:hover":{color:o}},[`${t}-item-group-title`]:Object.assign(Object.assign({},Zf),{paddingInline:E})}}]},nE=e=>{const{componentCls:t,motionDurationSlow:n,motionDurationMid:r,motionEaseInOut:o,motionEaseOut:l,iconCls:c,iconSize:d,iconMarginInlineEnd:m}=e;return{[`${t}-item, ${t}-submenu-title`]:{position:"relative",display:"block",margin:0,whiteSpace:"nowrap",cursor:"pointer",transition:[`border-color ${n}`,`background ${n}`,`padding calc(${n} + 0.1s) ${o}`].join(","),[`${t}-item-icon, ${c}`]:{minWidth:d,fontSize:d,transition:[`font-size ${r} ${l}`,`margin ${n} ${o}`,`color ${n}`].join(","),"+ span":{marginInlineStart:m,opacity:1,transition:[`opacity ${n} ${o}`,`margin ${n}`,`color ${n}`].join(",")}},[`${t}-item-icon`]:Object.assign({},es()),[`&${t}-item-only-child`]:{[`> ${c}, > ${t}-item-icon`]:{marginInlineEnd:0}}},[`${t}-item-disabled, ${t}-submenu-disabled`]:{background:"none !important",cursor:"not-allowed","&::after":{borderColor:"transparent !important"},a:{color:"inherit !important",cursor:"not-allowed",pointerEvents:"none"},[`> ${t}-submenu-title`]:{color:"inherit !important",cursor:"not-allowed"}}}},rE=e=>{const{componentCls:t,motionDurationSlow:n,motionEaseInOut:r,borderRadius:o,menuArrowSize:l,menuArrowOffset:c}=e;return{[`${t}-submenu`]:{"&-expand-icon, &-arrow":{position:"absolute",top:"50%",insetInlineEnd:e.margin,width:l,color:"currentcolor",transform:"translateY(-50%)",transition:`transform ${n} ${r}, opacity ${n}`},"&-arrow":{"&::before, &::after":{position:"absolute",width:e.calc(l).mul(.6).equal(),height:e.calc(l).mul(.15).equal(),backgroundColor:"currentcolor",borderRadius:o,transition:[`background ${n} ${r}`,`transform ${n} ${r}`,`top ${n} ${r}`,`color ${n} ${r}`].join(","),content:'""'},"&::before":{transform:`rotate(45deg) translateY(${he(e.calc(c).mul(-1).equal())})`},"&::after":{transform:`rotate(-45deg) translateY(${he(c)})`}}}}},GL=e=>{const{antCls:t,componentCls:n,fontSize:r,motionDurationSlow:o,motionDurationMid:l,motionEaseInOut:c,paddingXS:d,padding:m,colorSplit:v,lineWidth:g,zIndexPopup:p,borderRadiusLG:y,subMenuItemBorderRadius:E,menuArrowSize:C,menuArrowOffset:x,lineType:b,groupTitleLineHeight:w,groupTitleFontSize:$}=e;return[{"":{[n]:Object.assign(Object.assign({},Jf()),{"&-hidden":{display:"none"}})},[`${n}-submenu-hidden`]:{display:"none"}},{[n]:Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},Tr(e)),Jf()),{marginBottom:0,paddingInlineStart:0,fontSize:r,lineHeight:0,listStyle:"none",outline:"none",transition:`width ${o} cubic-bezier(0.2, 0, 0, 1) 0s`,"ul, ol":{margin:0,padding:0,listStyle:"none"},"&-overflow":{display:"flex",[`${n}-item`]:{flex:"none"}},[`${n}-item, ${n}-submenu, ${n}-submenu-title`]:{borderRadius:e.itemBorderRadius},[`${n}-item-group-title`]:{padding:`${he(d)} ${he(m)}`,fontSize:$,lineHeight:w,transition:`all ${o}`},[`&-horizontal ${n}-submenu`]:{transition:[`border-color ${o} ${c}`,`background ${o} ${c}`].join(",")},[`${n}-submenu, ${n}-submenu-inline`]:{transition:[`border-color ${o} ${c}`,`background ${o} ${c}`,`padding ${l} ${c}`].join(",")},[`${n}-submenu ${n}-sub`]:{cursor:"initial",transition:[`background ${o} ${c}`,`padding ${o} ${c}`].join(",")},[`${n}-title-content`]:{transition:`color ${o}`,"&-with-extra":{display:"inline-flex",alignItems:"center",width:"100%"},[`> ${t}-typography-ellipsis-single-line`]:{display:"inline",verticalAlign:"unset"},[`${n}-item-extra`]:{marginInlineStart:"auto",paddingInlineStart:e.padding}},[`${n}-item a`]:{"&::before":{position:"absolute",inset:0,backgroundColor:"transparent",content:'""'}},[`${n}-item-divider`]:{overflow:"hidden",lineHeight:0,borderColor:v,borderStyle:b,borderWidth:0,borderTopWidth:g,marginBlock:g,padding:0,"&-dashed":{borderStyle:"dashed"}}}),nE(e)),{[`${n}-item-group`]:{[`${n}-item-group-list`]:{margin:0,padding:0,[`${n}-item, ${n}-submenu-title`]:{paddingInline:`${he(e.calc(r).mul(2).equal())} ${he(m)}`}}},"&-submenu":{"&-popup":{position:"absolute",zIndex:p,borderRadius:y,boxShadow:"none",transformOrigin:"0 0",[`&${n}-submenu`]:{background:"transparent"},"&::before":{position:"absolute",inset:0,zIndex:-1,width:"100%",height:"100%",opacity:0,content:'""'},[`> ${n}`]:Object.assign(Object.assign(Object.assign({borderRadius:y},nE(e)),rE(e)),{[`${n}-item, ${n}-submenu > ${n}-submenu-title`]:{borderRadius:E},[`${n}-submenu-title::after`]:{transition:`transform ${o} ${c}`}})},"\n          &-placement-leftTop,\n          &-placement-bottomRight,\n          ":{transformOrigin:"100% 0"},"\n          &-placement-leftBottom,\n          &-placement-topRight,\n          ":{transformOrigin:"100% 100%"},"\n          &-placement-rightBottom,\n          &-placement-topLeft,\n          ":{transformOrigin:"0 100%"},"\n          &-placement-bottomLeft,\n          &-placement-rightTop,\n          ":{transformOrigin:"0 0"},"\n          &-placement-leftTop,\n          &-placement-leftBottom\n          ":{paddingInlineEnd:e.paddingXS},"\n          &-placement-rightTop,\n          &-placement-rightBottom\n          ":{paddingInlineStart:e.paddingXS},"\n          &-placement-topRight,\n          &-placement-topLeft\n          ":{paddingBottom:e.paddingXS},"\n          &-placement-bottomRight,\n          &-placement-bottomLeft\n          ":{paddingTop:e.paddingXS}}}),rE(e)),{[`&-inline-collapsed ${n}-submenu-arrow,
        &-inline ${n}-submenu-arrow`]:{"&::before":{transform:`rotate(-45deg) translateX(${he(x)})`},"&::after":{transform:`rotate(45deg) translateX(${he(e.calc(x).mul(-1).equal())})`}},[`${n}-submenu-open${n}-submenu-inline > ${n}-submenu-title > ${n}-submenu-arrow`]:{transform:`translateY(${he(e.calc(C).mul(.2).mul(-1).equal())})`,"&::after":{transform:`rotate(-45deg) translateX(${he(e.calc(x).mul(-1).equal())})`},"&::before":{transform:`rotate(45deg) translateX(${he(x)})`}}})},{[`${t}-layout-header`]:{[n]:{lineHeight:"inherit"}}}]},KL=e=>{var t,n,r;const{colorPrimary:o,colorError:l,colorTextDisabled:c,colorErrorBg:d,colorText:m,colorTextDescription:v,colorBgContainer:g,colorFillAlter:p,colorFillContent:y,lineWidth:E,lineWidthBold:C,controlItemBgActive:x,colorBgTextHover:b,controlHeightLG:w,lineHeight:$,colorBgElevated:O,marginXXS:T,padding:R,fontSize:M,controlHeightSM:N,fontSizeLG:z,colorTextLightSolid:P,colorErrorHover:D}=e,F=(t=e.activeBarWidth)!==null&&t!==void 0?t:0,W=(n=e.activeBarBorderWidth)!==null&&n!==void 0?n:E,V=(r=e.itemMarginInline)!==null&&r!==void 0?r:e.marginXXS,q=new Jt(P).setA(.65).toRgbString();return{dropdownWidth:160,zIndexPopup:e.zIndexPopupBase+50,radiusItem:e.borderRadiusLG,itemBorderRadius:e.borderRadiusLG,radiusSubMenuItem:e.borderRadiusSM,subMenuItemBorderRadius:e.borderRadiusSM,colorItemText:m,itemColor:m,colorItemTextHover:m,itemHoverColor:m,colorItemTextHoverHorizontal:o,horizontalItemHoverColor:o,colorGroupTitle:v,groupTitleColor:v,colorItemTextSelected:o,itemSelectedColor:o,subMenuItemSelectedColor:o,colorItemTextSelectedHorizontal:o,horizontalItemSelectedColor:o,colorItemBg:g,itemBg:g,colorItemBgHover:b,itemHoverBg:b,colorItemBgActive:y,itemActiveBg:x,colorSubItemBg:p,subMenuItemBg:p,colorItemBgSelected:x,itemSelectedBg:x,colorItemBgSelectedHorizontal:"transparent",horizontalItemSelectedBg:"transparent",colorActiveBarWidth:0,activeBarWidth:F,colorActiveBarHeight:C,activeBarHeight:C,colorActiveBarBorderSize:E,activeBarBorderWidth:W,colorItemTextDisabled:c,itemDisabledColor:c,colorDangerItemText:l,dangerItemColor:l,colorDangerItemTextHover:l,dangerItemHoverColor:l,colorDangerItemTextSelected:l,dangerItemSelectedColor:l,colorDangerItemBgActive:d,dangerItemActiveBg:d,colorDangerItemBgSelected:d,dangerItemSelectedBg:d,itemMarginInline:V,horizontalItemBorderRadius:0,horizontalItemHoverBg:"transparent",itemHeight:w,groupTitleLineHeight:$,collapsedWidth:w*2,popupBg:O,itemMarginBlock:T,itemPaddingInline:R,horizontalLineHeight:`${w*1.15}px`,iconSize:M,iconMarginInlineEnd:N-M,collapsedIconSize:z,groupTitleFontSize:M,darkItemDisabledColor:new Jt(P).setA(.25).toRgbString(),darkItemColor:q,darkDangerItemColor:l,darkItemBg:"#001529",darkPopupBg:"#001529",darkSubMenuItemBg:"#000c17",darkItemSelectedColor:P,darkItemSelectedBg:o,darkDangerItemSelectedBg:l,darkItemHoverBg:"transparent",darkGroupTitleColor:q,darkItemHoverColor:P,darkDangerItemHoverColor:D,darkDangerItemSelectedColor:P,darkDangerItemActiveBg:l,itemWidth:F?`calc(100% + ${W}px)`:`calc(100% - ${V*2}px)`}},XL=function(e){let t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:e,n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:!0;return wn("Menu",o=>{const{colorBgElevated:l,controlHeightLG:c,fontSize:d,darkItemColor:m,darkDangerItemColor:v,darkItemBg:g,darkSubMenuItemBg:p,darkItemSelectedColor:y,darkItemSelectedBg:E,darkDangerItemSelectedBg:C,darkItemHoverBg:x,darkGroupTitleColor:b,darkItemHoverColor:w,darkItemDisabledColor:$,darkDangerItemHoverColor:O,darkDangerItemSelectedColor:T,darkDangerItemActiveBg:R,popupBg:M,darkPopupBg:N}=o,z=o.calc(d).div(7).mul(5).equal(),P=kt(o,{menuArrowSize:z,menuHorizontalHeight:o.calc(c).mul(1.15).equal(),menuArrowOffset:o.calc(z).mul(.25).equal(),menuSubMenuBg:l,calc:o.calc,popupBg:M}),D=kt(P,{itemColor:m,itemHoverColor:w,groupTitleColor:b,itemSelectedColor:y,subMenuItemSelectedColor:y,itemBg:g,popupBg:N,subMenuItemBg:p,itemActiveBg:"transparent",itemSelectedBg:E,activeBarHeight:0,activeBarBorderWidth:0,itemHoverBg:x,itemDisabledColor:$,dangerItemColor:v,dangerItemHoverColor:O,dangerItemSelectedColor:T,dangerItemActiveBg:R,dangerItemSelectedBg:C,menuSubMenuBg:p,horizontalItemSelectedColor:y,horizontalItemSelectedBg:E});return[GL(P),qL(P),kL(P),eE(P,"light"),eE(D,"dark"),WL(P),v0(P),rd(P,"slide-up"),rd(P,"slide-down"),Dd(P,"zoom-big")]},KL,{deprecatedTokens:[["colorGroupTitle","groupTitleColor"],["radiusItem","itemBorderRadius"],["radiusSubMenuItem","subMenuItemBorderRadius"],["colorItemText","itemColor"],["colorItemTextHover","itemHoverColor"],["colorItemTextHoverHorizontal","horizontalItemHoverColor"],["colorItemTextSelected","itemSelectedColor"],["colorItemTextSelectedHorizontal","horizontalItemSelectedColor"],["colorItemTextDisabled","itemDisabledColor"],["colorDangerItemText","dangerItemColor"],["colorDangerItemTextHover","dangerItemHoverColor"],["colorDangerItemTextSelected","dangerItemSelectedColor"],["colorDangerItemBgActive","dangerItemActiveBg"],["colorDangerItemBgSelected","dangerItemSelectedBg"],["colorItemBg","itemBg"],["colorItemBgHover","itemHoverBg"],["colorSubItemBg","subMenuItemBg"],["colorItemBgActive","itemActiveBg"],["colorItemBgSelectedHorizontal","horizontalItemSelectedBg"],["colorActiveBarWidth","activeBarWidth"],["colorActiveBarHeight","activeBarHeight"],["colorActiveBarBorderSize","activeBarBorderWidth"],["colorItemBgSelected","itemSelectedBg"]],injectStyle:n,unitless:{groupTitleLineHeight:!0}})(e,t)},T2=e=>{var t;const{popupClassName:n,icon:r,title:o,theme:l}=e,c=u.useContext(ld),{prefixCls:d,inlineCollapsed:m,theme:v}=c,g=as();let p;if(!r)p=m&&!g.length&&o&&typeof o=="string"?u.createElement("div",{className:`${d}-inline-collapsed-noicon`},o.charAt(0)):u.createElement("span",{className:`${d}-title-content`},o);else{const C=u.isValidElement(o)&&o.type==="span";p=u.createElement(u.Fragment,null,Zr(r,{className:re(u.isValidElement(r)?(t=r.props)===null||t===void 0?void 0:t.className:"",`${d}-item-icon`)}),C?o:u.createElement("span",{className:`${d}-title-content`},o))}const y=u.useMemo(()=>Object.assign(Object.assign({},c),{firstLevel:!1}),[c]),[E]=Nd("Menu");return u.createElement(ld.Provider,{value:y},u.createElement(Yd,Object.assign({},Tn(e,["icon"]),{title:p,popupClassName:re(d,n,`${d}-${l||v}`),popupStyle:Object.assign({zIndex:E},e.popupStyle)})))};var YL=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function Zg(e){return e===null||e===!1}const QL={item:R2,submenu:T2,divider:O2},ZL=u.forwardRef((e,t)=>{var n;const r=u.useContext(sd),o=r||{},{getPrefixCls:l,getPopupContainer:c,direction:d,menu:m}=u.useContext(vt),v=l(),{prefixCls:g,className:p,style:y,theme:E="light",expandIcon:C,_internalDisableMenuItemTitleTooltip:x,inlineCollapsed:b,siderCollapsed:w,rootClassName:$,mode:O,selectable:T,onClick:R,overflowedIndicatorPopupClassName:M}=e,N=YL(e,["prefixCls","className","style","theme","expandIcon","_internalDisableMenuItemTitleTooltip","inlineCollapsed","siderCollapsed","rootClassName","mode","selectable","onClick","overflowedIndicatorPopupClassName"]),z=Tn(N,["collapsedWidth"]);(n=o.validator)===null||n===void 0||n.call(o,{mode:O});const P=jn(function(){var G;R==null||R.apply(void 0,arguments),(G=o.onClick)===null||G===void 0||G.call(o)}),D=o.mode||O,F=T??o.selectable,W=b??w,V={horizontal:{motionName:`${v}-slide-up`},inline:nd(v),other:{motionName:`${v}-zoom-big`}},q=l("menu",g||o.prefixCls),K=na(q),[_,j,B]=XL(q,K,!r),k=re(`${q}-${E}`,m==null?void 0:m.className,p),I=u.useMemo(()=>{var G,X;if(typeof C=="function"||Zg(C))return C||null;if(typeof o.expandIcon=="function"||Zg(o.expandIcon))return o.expandIcon||null;if(typeof(m==null?void 0:m.expandIcon)=="function"||Zg(m==null?void 0:m.expandIcon))return(m==null?void 0:m.expandIcon)||null;const U=(G=C??(o==null?void 0:o.expandIcon))!==null&&G!==void 0?G:m==null?void 0:m.expandIcon;return Zr(U,{className:re(`${q}-submenu-expand-icon`,u.isValidElement(U)?(X=U.props)===null||X===void 0?void 0:X.className:void 0)})},[C,o==null?void 0:o.expandIcon,m==null?void 0:m.expandIcon,q]),H=u.useMemo(()=>({prefixCls:q,inlineCollapsed:W||!1,direction:d,firstLevel:!0,theme:E,mode:D,disableMenuItemTitleTooltip:x}),[q,W,d,x,E]);return _(u.createElement(sd.Provider,{value:null},u.createElement(ld.Provider,{value:H},u.createElement(eu,Object.assign({getPopupContainer:c,overflowedIndicator:u.createElement($N,null),overflowedIndicatorPopupClassName:re(q,`${q}-${E}`,M),mode:D,selectable:F,onClick:P},z,{inlineCollapsed:W,style:Object.assign(Object.assign({},m==null?void 0:m.style),y),className:k,prefixCls:q,direction:d,defaultMotions:V,expandIcon:I,ref:t,rootClassName:re($,j,o.rootClassName,B,K),_internalComponents:QL})))))}),tu=u.forwardRef((e,t)=>{const n=u.useRef(null),r=u.useContext(Qd);return u.useImperativeHandle(t,()=>({menu:n.current,focus:o=>{var l;(l=n.current)===null||l===void 0||l.focus(o)}})),u.createElement(ZL,Object.assign({ref:n},e,r))});tu.Item=R2;tu.SubMenu=T2;tu.Divider=O2;tu.ItemGroup=H0;function Zd(e){return kt(e,{inputAffixPadding:e.paddingXXS})}const Jd=e=>{const{controlHeight:t,fontSize:n,lineHeight:r,lineWidth:o,controlHeightSM:l,controlHeightLG:c,fontSizeLG:d,lineHeightLG:m,paddingSM:v,controlPaddingHorizontalSM:g,controlPaddingHorizontal:p,colorFillAlter:y,colorPrimaryHover:E,colorPrimary:C,controlOutlineWidth:x,controlOutline:b,colorErrorOutline:w,colorWarningOutline:$,colorBgContainer:O,inputFontSize:T,inputFontSizeLG:R,inputFontSizeSM:M}=e,N=T||n,z=M||N,P=R||d,D=Math.round((t-N*r)/2*10)/10-o,F=Math.round((l-z*r)/2*10)/10-o,W=Math.ceil((c-P*m)/2*10)/10-o;return{paddingBlock:Math.max(D,0),paddingBlockSM:Math.max(F,0),paddingBlockLG:Math.max(W,0),paddingInline:v-o,paddingInlineSM:g-o,paddingInlineLG:p-o,addonBg:y,activeBorderColor:C,hoverBorderColor:E,activeShadow:`0 0 0 ${x}px ${b}`,errorActiveShadow:`0 0 0 ${x}px ${w}`,warningActiveShadow:`0 0 0 ${x}px ${$}`,hoverBg:O,activeBg:O,inputFontSize:N,inputFontSizeLG:P,inputFontSizeSM:z}},JL=e=>({borderColor:e.hoverBorderColor,backgroundColor:e.hoverBg}),F0=e=>({color:e.colorTextDisabled,backgroundColor:e.colorBgContainerDisabled,borderColor:e.colorBorder,boxShadow:"none",cursor:"not-allowed",opacity:1,"input[disabled], textarea[disabled]":{cursor:"not-allowed"},"&:hover:not([disabled])":Object.assign({},JL(kt(e,{hoverBorderColor:e.colorBorder,hoverBg:e.colorBgContainerDisabled})))}),M2=(e,t)=>({background:e.colorBgContainer,borderWidth:e.lineWidth,borderStyle:e.lineType,borderColor:t.borderColor,"&:hover":{borderColor:t.hoverBorderColor,backgroundColor:e.hoverBg},"&:focus, &:focus-within":{borderColor:t.activeBorderColor,boxShadow:t.activeShadow,outline:0,backgroundColor:e.activeBg}}),aE=(e,t)=>({[`&${e.componentCls}-status-${t.status}:not(${e.componentCls}-disabled)`]:Object.assign(Object.assign({},M2(e,t)),{[`${e.componentCls}-prefix, ${e.componentCls}-suffix`]:{color:t.affixColor}}),[`&${e.componentCls}-status-${t.status}${e.componentCls}-disabled`]:{borderColor:t.borderColor}}),e8=(e,t)=>({"&-outlined":Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},M2(e,{borderColor:e.colorBorder,hoverBorderColor:e.hoverBorderColor,activeBorderColor:e.activeBorderColor,activeShadow:e.activeShadow})),{[`&${e.componentCls}-disabled, &[disabled]`]:Object.assign({},F0(e))}),aE(e,{status:"error",borderColor:e.colorError,hoverBorderColor:e.colorErrorBorderHover,activeBorderColor:e.colorError,activeShadow:e.errorActiveShadow,affixColor:e.colorError})),aE(e,{status:"warning",borderColor:e.colorWarning,hoverBorderColor:e.colorWarningBorderHover,activeBorderColor:e.colorWarning,activeShadow:e.warningActiveShadow,affixColor:e.colorWarning})),t)}),oE=(e,t)=>({[`&${e.componentCls}-group-wrapper-status-${t.status}`]:{[`${e.componentCls}-group-addon`]:{borderColor:t.addonBorderColor,color:t.addonColor}}}),t8=e=>({"&-outlined":Object.assign(Object.assign(Object.assign({[`${e.componentCls}-group`]:{"&-addon":{background:e.addonBg,border:`${he(e.lineWidth)} ${e.lineType} ${e.colorBorder}`},"&-addon:first-child":{borderInlineEnd:0},"&-addon:last-child":{borderInlineStart:0}}},oE(e,{status:"error",addonBorderColor:e.colorError,addonColor:e.colorErrorText})),oE(e,{status:"warning",addonBorderColor:e.colorWarning,addonColor:e.colorWarningText})),{[`&${e.componentCls}-group-wrapper-disabled`]:{[`${e.componentCls}-group-addon`]:Object.assign({},F0(e))}})}),n8=(e,t)=>{const{componentCls:n}=e;return{"&-borderless":Object.assign({background:"transparent",border:"none","&:focus, &:focus-within":{outline:"none"},[`&${n}-disabled, &[disabled]`]:{color:e.colorTextDisabled,cursor:"not-allowed"},[`&${n}-status-error`]:{"&, & input, & textarea":{color:e.colorError}},[`&${n}-status-warning`]:{"&, & input, & textarea":{color:e.colorWarning}}},t)}},_2=(e,t)=>{var n;return{background:t.bg,borderWidth:e.lineWidth,borderStyle:e.lineType,borderColor:"transparent","input&, & input, textarea&, & textarea":{color:(n=t==null?void 0:t.inputColor)!==null&&n!==void 0?n:"unset"},"&:hover":{background:t.hoverBg},"&:focus, &:focus-within":{outline:0,borderColor:t.activeBorderColor,backgroundColor:e.activeBg}}},iE=(e,t)=>({[`&${e.componentCls}-status-${t.status}:not(${e.componentCls}-disabled)`]:Object.assign(Object.assign({},_2(e,t)),{[`${e.componentCls}-prefix, ${e.componentCls}-suffix`]:{color:t.affixColor}})}),r8=(e,t)=>({"&-filled":Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},_2(e,{bg:e.colorFillTertiary,hoverBg:e.colorFillSecondary,activeBorderColor:e.activeBorderColor})),{[`&${e.componentCls}-disabled, &[disabled]`]:Object.assign({},F0(e))}),iE(e,{status:"error",bg:e.colorErrorBg,hoverBg:e.colorErrorBgHover,activeBorderColor:e.colorError,inputColor:e.colorErrorText,affixColor:e.colorError})),iE(e,{status:"warning",bg:e.colorWarningBg,hoverBg:e.colorWarningBgHover,activeBorderColor:e.colorWarning,inputColor:e.colorWarningText,affixColor:e.colorWarning})),t)}),lE=(e,t)=>({[`&${e.componentCls}-group-wrapper-status-${t.status}`]:{[`${e.componentCls}-group-addon`]:{background:t.addonBg,color:t.addonColor}}}),a8=e=>({"&-filled":Object.assign(Object.assign(Object.assign({[`${e.componentCls}-group`]:{"&-addon":{background:e.colorFillTertiary},[`${e.componentCls}-filled:not(:focus):not(:focus-within)`]:{"&:not(:first-child)":{borderInlineStart:`${he(e.lineWidth)} ${e.lineType} ${e.colorSplit}`},"&:not(:last-child)":{borderInlineEnd:`${he(e.lineWidth)} ${e.lineType} ${e.colorSplit}`}}}},lE(e,{status:"error",addonBg:e.colorErrorBg,addonColor:e.colorErrorText})),lE(e,{status:"warning",addonBg:e.colorWarningBg,addonColor:e.colorWarningText})),{[`&${e.componentCls}-group-wrapper-disabled`]:{[`${e.componentCls}-group`]:{"&-addon":{background:e.colorFillTertiary,color:e.colorTextDisabled},"&-addon:first-child":{borderInlineStart:`${he(e.lineWidth)} ${e.lineType} ${e.colorBorder}`,borderTop:`${he(e.lineWidth)} ${e.lineType} ${e.colorBorder}`,borderBottom:`${he(e.lineWidth)} ${e.lineType} ${e.colorBorder}`},"&-addon:last-child":{borderInlineEnd:`${he(e.lineWidth)} ${e.lineType} ${e.colorBorder}`,borderTop:`${he(e.lineWidth)} ${e.lineType} ${e.colorBorder}`,borderBottom:`${he(e.lineWidth)} ${e.lineType} ${e.colorBorder}`}}}})}),N2=(e,t)=>({background:e.colorBgContainer,borderWidth:`${he(e.lineWidth)} 0`,borderStyle:`${e.lineType} none`,borderColor:`transparent transparent ${t.borderColor} transparent`,borderRadius:0,"&:hover":{borderColor:`transparent transparent ${t.borderColor} transparent`,backgroundColor:e.hoverBg},"&:focus, &:focus-within":{borderColor:`transparent transparent ${t.borderColor} transparent`,outline:0,backgroundColor:e.activeBg}}),sE=(e,t)=>({[`&${e.componentCls}-status-${t.status}:not(${e.componentCls}-disabled)`]:Object.assign(Object.assign({},N2(e,t)),{[`${e.componentCls}-prefix, ${e.componentCls}-suffix`]:{color:t.affixColor}}),[`&${e.componentCls}-status-${t.status}${e.componentCls}-disabled`]:{borderColor:`transparent transparent ${t.borderColor} transparent`}}),o8=(e,t)=>({"&-underlined":Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},N2(e,{borderColor:e.colorBorder,hoverBorderColor:e.hoverBorderColor,activeBorderColor:e.activeBorderColor,activeShadow:e.activeShadow})),{[`&${e.componentCls}-disabled, &[disabled]`]:{color:e.colorTextDisabled,boxShadow:"none",cursor:"not-allowed","&:hover":{borderColor:`transparent transparent ${e.colorBorder} transparent`}},"input[disabled], textarea[disabled]":{cursor:"not-allowed"}}),sE(e,{status:"error",borderColor:e.colorError,hoverBorderColor:e.colorErrorBorderHover,activeBorderColor:e.colorError,activeShadow:e.errorActiveShadow,affixColor:e.colorError})),sE(e,{status:"warning",borderColor:e.colorWarning,hoverBorderColor:e.colorWarningBorderHover,activeBorderColor:e.colorWarning,activeShadow:e.warningActiveShadow,affixColor:e.colorWarning})),t)}),i8=e=>({"&::-moz-placeholder":{opacity:1},"&::placeholder":{color:e,userSelect:"none"},"&:placeholder-shown":{textOverflow:"ellipsis"}}),A2=e=>{const{paddingBlockLG:t,lineHeightLG:n,borderRadiusLG:r,paddingInlineLG:o}=e;return{padding:`${he(t)} ${he(o)}`,fontSize:e.inputFontSizeLG,lineHeight:n,borderRadius:r}},I2=e=>({padding:`${he(e.paddingBlockSM)} ${he(e.paddingInlineSM)}`,fontSize:e.inputFontSizeSM,borderRadius:e.borderRadiusSM}),j2=e=>Object.assign(Object.assign({position:"relative",display:"inline-block",width:"100%",minWidth:0,padding:`${he(e.paddingBlock)} ${he(e.paddingInline)}`,color:e.colorText,fontSize:e.inputFontSize,lineHeight:e.lineHeight,borderRadius:e.borderRadius,transition:`all ${e.motionDurationMid}`},i8(e.colorTextPlaceholder)),{"&-lg":Object.assign({},A2(e)),"&-sm":Object.assign({},I2(e)),"&-rtl, &-textarea-rtl":{direction:"rtl"}}),l8=e=>{const{componentCls:t,antCls:n}=e;return{position:"relative",display:"table",width:"100%",borderCollapse:"separate",borderSpacing:0,"&[class*='col-']":{paddingInlineEnd:e.paddingXS,"&:last-child":{paddingInlineEnd:0}},[`&-lg ${t}, &-lg > ${t}-group-addon`]:Object.assign({},A2(e)),[`&-sm ${t}, &-sm > ${t}-group-addon`]:Object.assign({},I2(e)),[`&-lg ${n}-select-single ${n}-select-selector`]:{height:e.controlHeightLG},[`&-sm ${n}-select-single ${n}-select-selector`]:{height:e.controlHeightSM},[`> ${t}`]:{display:"table-cell","&:not(:first-child):not(:last-child)":{borderRadius:0}},[`${t}-group`]:{"&-addon, &-wrap":{display:"table-cell",width:1,whiteSpace:"nowrap",verticalAlign:"middle","&:not(:first-child):not(:last-child)":{borderRadius:0}},"&-wrap > *":{display:"block !important"},"&-addon":{position:"relative",padding:`0 ${he(e.paddingInline)}`,color:e.colorText,fontWeight:"normal",fontSize:e.inputFontSize,textAlign:"center",borderRadius:e.borderRadius,transition:`all ${e.motionDurationSlow}`,lineHeight:1,[`${n}-select`]:{margin:`${he(e.calc(e.paddingBlock).add(1).mul(-1).equal())} ${he(e.calc(e.paddingInline).mul(-1).equal())}`,[`&${n}-select-single:not(${n}-select-customize-input):not(${n}-pagination-size-changer)`]:{[`${n}-select-selector`]:{backgroundColor:"inherit",border:`${he(e.lineWidth)} ${e.lineType} transparent`,boxShadow:"none"}}},[`${n}-cascader-picker`]:{margin:`-9px ${he(e.calc(e.paddingInline).mul(-1).equal())}`,backgroundColor:"transparent",[`${n}-cascader-input`]:{textAlign:"start",border:0,boxShadow:"none"}}}},[t]:{width:"100%",marginBottom:0,textAlign:"inherit","&:focus":{zIndex:1,borderInlineEndWidth:1},"&:hover":{zIndex:1,borderInlineEndWidth:1,[`${t}-search-with-button &`]:{zIndex:0}}},[`> ${t}:first-child, ${t}-group-addon:first-child`]:{borderStartEndRadius:0,borderEndEndRadius:0,[`${n}-select ${n}-select-selector`]:{borderStartEndRadius:0,borderEndEndRadius:0}},[`> ${t}-affix-wrapper`]:{[`&:not(:first-child) ${t}`]:{borderStartStartRadius:0,borderEndStartRadius:0},[`&:not(:last-child) ${t}`]:{borderStartEndRadius:0,borderEndEndRadius:0}},[`> ${t}:last-child, ${t}-group-addon:last-child`]:{borderStartStartRadius:0,borderEndStartRadius:0,[`${n}-select ${n}-select-selector`]:{borderStartStartRadius:0,borderEndStartRadius:0}},[`${t}-affix-wrapper`]:{"&:not(:last-child)":{borderStartEndRadius:0,borderEndEndRadius:0,[`${t}-search &`]:{borderStartStartRadius:e.borderRadius,borderEndStartRadius:e.borderRadius}},[`&:not(:first-child), ${t}-search &:not(:first-child)`]:{borderStartStartRadius:0,borderEndStartRadius:0}},[`&${t}-group-compact`]:Object.assign(Object.assign({display:"block"},Jf()),{[`${t}-group-addon, ${t}-group-wrap, > ${t}`]:{"&:not(:first-child):not(:last-child)":{borderInlineEndWidth:e.lineWidth,"&:hover, &:focus":{zIndex:1}}},"& > *":{display:"inline-flex",float:"none",verticalAlign:"top",borderRadius:0},[`
        & > ${t}-affix-wrapper,
        & > ${t}-number-affix-wrapper,
        & > ${n}-picker-range
      `]:{display:"inline-flex"},"& > *:not(:last-child)":{marginInlineEnd:e.calc(e.lineWidth).mul(-1).equal(),borderInlineEndWidth:e.lineWidth},[t]:{float:"none"},[`& > ${n}-select > ${n}-select-selector,
      & > ${n}-select-auto-complete ${t},
      & > ${n}-cascader-picker ${t},
      & > ${t}-group-wrapper ${t}`]:{borderInlineEndWidth:e.lineWidth,borderRadius:0,"&:hover, &:focus":{zIndex:1}},[`& > ${n}-select-focused`]:{zIndex:1},[`& > ${n}-select > ${n}-select-arrow`]:{zIndex:1},[`& > *:first-child,
      & > ${n}-select:first-child > ${n}-select-selector,
      & > ${n}-select-auto-complete:first-child ${t},
      & > ${n}-cascader-picker:first-child ${t}`]:{borderStartStartRadius:e.borderRadius,borderEndStartRadius:e.borderRadius},[`& > *:last-child,
      & > ${n}-select:last-child > ${n}-select-selector,
      & > ${n}-cascader-picker:last-child ${t},
      & > ${n}-cascader-picker-focused:last-child ${t}`]:{borderInlineEndWidth:e.lineWidth,borderStartEndRadius:e.borderRadius,borderEndEndRadius:e.borderRadius},[`& > ${n}-select-auto-complete ${t}`]:{verticalAlign:"top"},[`${t}-group-wrapper + ${t}-group-wrapper`]:{marginInlineStart:e.calc(e.lineWidth).mul(-1).equal(),[`${t}-affix-wrapper`]:{borderRadius:0}},[`${t}-group-wrapper:not(:last-child)`]:{[`&${t}-search > ${t}-group`]:{[`& > ${t}-group-addon > ${t}-search-button`]:{borderRadius:0},[`& > ${t}`]:{borderStartStartRadius:e.borderRadius,borderStartEndRadius:0,borderEndEndRadius:0,borderEndStartRadius:e.borderRadius}}}})}},s8=e=>{const{componentCls:t,controlHeightSM:n,lineWidth:r,calc:o}=e,c=o(n).sub(o(r).mul(2)).sub(16).div(2).equal();return{[t]:Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},Tr(e)),j2(e)),e8(e)),r8(e)),n8(e)),o8(e)),{'&[type="color"]':{height:e.controlHeight,[`&${t}-lg`]:{height:e.controlHeightLG},[`&${t}-sm`]:{height:n,paddingTop:c,paddingBottom:c}},'&[type="search"]::-webkit-search-cancel-button, &[type="search"]::-webkit-search-decoration':{appearance:"none"}})}},c8=e=>{const{componentCls:t}=e;return{[`${t}-clear-icon`]:{margin:0,padding:0,lineHeight:0,color:e.colorTextQuaternary,fontSize:e.fontSizeIcon,verticalAlign:-1,cursor:"pointer",transition:`color ${e.motionDurationSlow}`,border:"none",outline:"none",backgroundColor:"transparent","&:hover":{color:e.colorIcon},"&:active":{color:e.colorText},"&-hidden":{visibility:"hidden"},"&-has-suffix":{margin:`0 ${he(e.inputAffixPadding)}`}}}},u8=e=>{const{componentCls:t,inputAffixPadding:n,colorTextDescription:r,motionDurationSlow:o,colorIcon:l,colorIconHover:c,iconCls:d}=e,m=`${t}-affix-wrapper`,v=`${t}-affix-wrapper-disabled`;return{[m]:Object.assign(Object.assign(Object.assign(Object.assign({},j2(e)),{display:"inline-flex",[`&:not(${t}-disabled):hover`]:{zIndex:1,[`${t}-search-with-button &`]:{zIndex:0}},"&-focused, &:focus":{zIndex:1},[`> input${t}`]:{padding:0},[`> input${t}, > textarea${t}`]:{fontSize:"inherit",border:"none",borderRadius:0,outline:"none",background:"transparent",color:"inherit","&::-ms-reveal":{display:"none"},"&:focus":{boxShadow:"none !important"}},"&::before":{display:"inline-block",width:0,visibility:"hidden",content:'"\\a0"'},[t]:{"&-prefix, &-suffix":{display:"flex",flex:"none",alignItems:"center","> *:not(:last-child)":{marginInlineEnd:e.paddingXS}},"&-show-count-suffix":{color:r,direction:"ltr"},"&-show-count-has-suffix":{marginInlineEnd:e.paddingXXS},"&-prefix":{marginInlineEnd:n},"&-suffix":{marginInlineStart:n}}}),c8(e)),{[`${d}${t}-password-icon`]:{color:l,cursor:"pointer",transition:`all ${o}`,"&:hover":{color:c}}}),[`${t}-underlined`]:{borderRadius:0},[v]:{[`${d}${t}-password-icon`]:{color:l,cursor:"not-allowed","&:hover":{color:l}}}}},f8=e=>{const{componentCls:t,borderRadiusLG:n,borderRadiusSM:r}=e;return{[`${t}-group`]:Object.assign(Object.assign(Object.assign({},Tr(e)),l8(e)),{"&-rtl":{direction:"rtl"},"&-wrapper":Object.assign(Object.assign(Object.assign({display:"inline-block",width:"100%",textAlign:"start",verticalAlign:"top","&-rtl":{direction:"rtl"},"&-lg":{[`${t}-group-addon`]:{borderRadius:n,fontSize:e.inputFontSizeLG}},"&-sm":{[`${t}-group-addon`]:{borderRadius:r}}},t8(e)),a8(e)),{[`&:not(${t}-compact-first-item):not(${t}-compact-last-item)${t}-compact-item`]:{[`${t}, ${t}-group-addon`]:{borderRadius:0}},[`&:not(${t}-compact-last-item)${t}-compact-first-item`]:{[`${t}, ${t}-group-addon`]:{borderStartEndRadius:0,borderEndEndRadius:0}},[`&:not(${t}-compact-first-item)${t}-compact-last-item`]:{[`${t}, ${t}-group-addon`]:{borderStartStartRadius:0,borderEndStartRadius:0}},[`&:not(${t}-compact-last-item)${t}-compact-item`]:{[`${t}-affix-wrapper`]:{borderStartEndRadius:0,borderEndEndRadius:0}},[`&:not(${t}-compact-first-item)${t}-compact-item`]:{[`${t}-affix-wrapper`]:{borderStartStartRadius:0,borderEndStartRadius:0}}})})}},d8=e=>{const{componentCls:t,antCls:n}=e,r=`${t}-search`;return{[r]:{[t]:{"&:hover, &:focus":{[`+ ${t}-group-addon ${r}-button:not(${n}-btn-primary)`]:{borderInlineStartColor:e.colorPrimaryHover}}},[`${t}-affix-wrapper`]:{height:e.controlHeight,borderRadius:0},[`${t}-lg`]:{lineHeight:e.calc(e.lineHeightLG).sub(2e-4).equal()},[`> ${t}-group`]:{[`> ${t}-group-addon:last-child`]:{insetInlineStart:-1,padding:0,border:0,[`${r}-button`]:{marginInlineEnd:-1,borderStartStartRadius:0,borderEndStartRadius:0,boxShadow:"none"},[`${r}-button:not(${n}-btn-primary)`]:{color:e.colorTextDescription,"&:hover":{color:e.colorPrimaryHover},"&:active":{color:e.colorPrimaryActive},[`&${n}-btn-loading::before`]:{insetInlineStart:0,insetInlineEnd:0,insetBlockStart:0,insetBlockEnd:0}}}},[`${r}-button`]:{height:e.controlHeight,"&:hover, &:focus":{zIndex:1}},"&-large":{[`${t}-affix-wrapper, ${r}-button`]:{height:e.controlHeightLG}},"&-small":{[`${t}-affix-wrapper, ${r}-button`]:{height:e.controlHeightSM}},"&-rtl":{direction:"rtl"},[`&${t}-compact-item`]:{[`&:not(${t}-compact-last-item)`]:{[`${t}-group-addon`]:{[`${t}-search-button`]:{marginInlineEnd:e.calc(e.lineWidth).mul(-1).equal(),borderRadius:0}}},[`&:not(${t}-compact-first-item)`]:{[`${t},${t}-affix-wrapper`]:{borderRadius:0}},[`> ${t}-group-addon ${t}-search-button,
        > ${t},
        ${t}-affix-wrapper`]:{"&:hover, &:focus, &:active":{zIndex:2}},[`> ${t}-affix-wrapper-focused`]:{zIndex:2}}}}},m8=e=>{const{componentCls:t}=e;return{[`${t}-out-of-range`]:{[`&, & input, & textarea, ${t}-show-count-suffix, ${t}-data-count`]:{color:e.colorError}}}},P2=wn(["Input","Shared"],e=>{const t=kt(e,Zd(e));return[s8(t),u8(t)]},Jd,{resetFont:!1}),z2=wn(["Input","Component"],e=>{const t=kt(e,Zd(e));return[f8(t),d8(t),m8(t),p0(t)]},Jd,{resetFont:!1}),D2=u.createContext({});var v8=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function cE(e){return typeof e=="number"?`${e} ${e} auto`:/^\d+(\.\d+)?(px|em|rem|%)$/.test(e)?`0 0 ${e}`:e}const g8=["xs","sm","md","lg","xl","xxl"],B2=u.forwardRef((e,t)=>{const{getPrefixCls:n,direction:r}=u.useContext(vt),{gutter:o,wrap:l}=u.useContext(D2),{prefixCls:c,span:d,order:m,offset:v,push:g,pull:p,className:y,children:E,flex:C,style:x}=e,b=v8(e,["prefixCls","span","order","offset","push","pull","className","children","flex","style"]),w=n("col",c),[$,O,T]=A6(w),R={};let M={};g8.forEach(P=>{let D={};const F=e[P];typeof F=="number"?D.span=F:typeof F=="object"&&(D=F||{}),delete b[P],M=Object.assign(Object.assign({},M),{[`${w}-${P}-${D.span}`]:D.span!==void 0,[`${w}-${P}-order-${D.order}`]:D.order||D.order===0,[`${w}-${P}-offset-${D.offset}`]:D.offset||D.offset===0,[`${w}-${P}-push-${D.push}`]:D.push||D.push===0,[`${w}-${P}-pull-${D.pull}`]:D.pull||D.pull===0,[`${w}-rtl`]:r==="rtl"}),D.flex&&(M[`${w}-${P}-flex`]=!0,R[`--${w}-${P}-flex`]=cE(D.flex))});const N=re(w,{[`${w}-${d}`]:d!==void 0,[`${w}-order-${m}`]:m,[`${w}-offset-${v}`]:v,[`${w}-push-${g}`]:g,[`${w}-pull-${p}`]:p},y,M,O,T),z={};if(o&&o[0]>0){const P=o[0]/2;z.paddingLeft=P,z.paddingRight=P}return C&&(z.flex=cE(C),l===!1&&!z.minWidth&&(z.minWidth=0)),$(u.createElement("div",Object.assign({},b,{style:Object.assign(Object.assign(Object.assign({},z),x),R),className:N,ref:t}),E))});function h8(e,t){const n=[void 0,void 0],r=Array.isArray(e)?e:[e,void 0],o=t||{xs:!0,sm:!0,md:!0,lg:!0,xl:!0,xxl:!0};return r.forEach((l,c)=>{if(typeof l=="object"&&l!==null)for(let d=0;d<jc.length;d++){const m=jc[d];if(o[m]&&l[m]!==void 0){n[c]=l[m];break}}else n[c]=l}),n}var p8=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function uE(e,t){const[n,r]=u.useState(typeof e=="string"?e:""),o=()=>{if(typeof e=="string"&&r(e),typeof e=="object")for(let l=0;l<jc.length;l++){const c=jc[l];if(!t||!t[c])continue;const d=e[c];if(d!==void 0){r(d);return}}};return u.useEffect(()=>{o()},[JSON.stringify(e),t]),n}const y8=u.forwardRef((e,t)=>{const{prefixCls:n,justify:r,align:o,className:l,style:c,children:d,gutter:m=0,wrap:v}=e,g=p8(e,["prefixCls","justify","align","className","style","children","gutter","wrap"]),{getPrefixCls:p,direction:y}=u.useContext(vt),E=_B(!0,null),C=uE(o,E),x=uE(r,E),b=p("row",n),[w,$,O]=N6(b),T=h8(m,E),R=re(b,{[`${b}-no-wrap`]:v===!1,[`${b}-${x}`]:x,[`${b}-${C}`]:C,[`${b}-rtl`]:y==="rtl"},l,$,O),M={},N=T[0]!=null&&T[0]>0?T[0]/-2:void 0;N&&(M.marginLeft=N,M.marginRight=N);const[z,P]=T;M.rowGap=P;const D=u.useMemo(()=>({gutter:[z,P],wrap:v}),[z,P,v]);return w(u.createElement(D2.Provider,{value:D},u.createElement("div",Object.assign({},g,{className:R,style:Object.assign(Object.assign({},M),c),ref:t}),d)))}),b8=e=>{const{componentCls:t,sizePaddingEdgeHorizontal:n,colorSplit:r,lineWidth:o,textPaddingInline:l,orientationMargin:c,verticalMarginInline:d}=e;return{[t]:Object.assign(Object.assign({},Tr(e)),{borderBlockStart:`${he(o)} solid ${r}`,"&-vertical":{position:"relative",top:"-0.06em",display:"inline-block",height:"0.9em",marginInline:d,marginBlock:0,verticalAlign:"middle",borderTop:0,borderInlineStart:`${he(o)} solid ${r}`},"&-horizontal":{display:"flex",clear:"both",width:"100%",minWidth:"100%",margin:`${he(e.dividerHorizontalGutterMargin)} 0`},[`&-horizontal${t}-with-text`]:{display:"flex",alignItems:"center",margin:`${he(e.dividerHorizontalWithTextGutterMargin)} 0`,color:e.colorTextHeading,fontWeight:500,fontSize:e.fontSizeLG,whiteSpace:"nowrap",textAlign:"center",borderBlockStart:`0 ${r}`,"&::before, &::after":{position:"relative",width:"50%",borderBlockStart:`${he(o)} solid transparent`,borderBlockStartColor:"inherit",borderBlockEnd:0,transform:"translateY(50%)",content:"''"}},[`&-horizontal${t}-with-text-start`]:{"&::before":{width:`calc(${c} * 100%)`},"&::after":{width:`calc(100% - ${c} * 100%)`}},[`&-horizontal${t}-with-text-end`]:{"&::before":{width:`calc(100% - ${c} * 100%)`},"&::after":{width:`calc(${c} * 100%)`}},[`${t}-inner-text`]:{display:"inline-block",paddingBlock:0,paddingInline:l},"&-dashed":{background:"none",borderColor:r,borderStyle:"dashed",borderWidth:`${he(o)} 0 0`},[`&-horizontal${t}-with-text${t}-dashed`]:{"&::before, &::after":{borderStyle:"dashed none none"}},[`&-vertical${t}-dashed`]:{borderInlineStartWidth:o,borderInlineEnd:0,borderBlockStart:0,borderBlockEnd:0},"&-dotted":{background:"none",borderColor:r,borderStyle:"dotted",borderWidth:`${he(o)} 0 0`},[`&-horizontal${t}-with-text${t}-dotted`]:{"&::before, &::after":{borderStyle:"dotted none none"}},[`&-vertical${t}-dotted`]:{borderInlineStartWidth:o,borderInlineEnd:0,borderBlockStart:0,borderBlockEnd:0},[`&-plain${t}-with-text`]:{color:e.colorText,fontWeight:"normal",fontSize:e.fontSize},[`&-horizontal${t}-with-text-start${t}-no-default-orientation-margin-start`]:{"&::before":{width:0},"&::after":{width:"100%"},[`${t}-inner-text`]:{paddingInlineStart:n}},[`&-horizontal${t}-with-text-end${t}-no-default-orientation-margin-end`]:{"&::before":{width:"100%"},"&::after":{width:0},[`${t}-inner-text`]:{paddingInlineEnd:n}}})}},S8=e=>({textPaddingInline:"1em",orientationMargin:.05,verticalMarginInline:e.marginXS}),C8=wn("Divider",e=>{const t=kt(e,{dividerHorizontalWithTextGutterMargin:e.margin,dividerHorizontalGutterMargin:e.marginLG,sizePaddingEdgeHorizontal:0});return[b8(t)]},S8,{unitless:{orientationMargin:!0}});var x8=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const fE=e=>{const{getPrefixCls:t,direction:n,className:r,style:o}=zr("divider"),{prefixCls:l,type:c="horizontal",orientation:d="center",orientationMargin:m,className:v,rootClassName:g,children:p,dashed:y,variant:E="solid",plain:C,style:x}=e,b=x8(e,["prefixCls","type","orientation","orientationMargin","className","rootClassName","children","dashed","variant","plain","style"]),w=t("divider",l),[$,O,T]=C8(w),R=!!p,M=u.useMemo(()=>d==="left"?n==="rtl"?"end":"start":d==="right"?n==="rtl"?"start":"end":d,[n,d]),N=M==="start"&&m!=null,z=M==="end"&&m!=null,P=re(w,r,O,T,`${w}-${c}`,{[`${w}-with-text`]:R,[`${w}-with-text-${M}`]:R,[`${w}-dashed`]:!!y,[`${w}-${E}`]:E!=="solid",[`${w}-plain`]:!!C,[`${w}-rtl`]:n==="rtl",[`${w}-no-default-orientation-margin-start`]:N,[`${w}-no-default-orientation-margin-end`]:z},v,g),D=u.useMemo(()=>typeof m=="number"?m:/^\d+$/.test(m)?Number(m):m,[m]),F={marginInlineStart:N?D:void 0,marginInlineEnd:z?D:void 0};return $(u.createElement("div",Object.assign({className:P,style:Object.assign(Object.assign({},o),x)},b,{role:"separator"}),p&&c!=="vertical"&&u.createElement("span",{className:`${w}-inner-text`,style:F},p)))};function E8(e){return!!(e.addonBefore||e.addonAfter)}function w8(e){return!!(e.prefix||e.suffix||e.allowClear)}function dE(e,t,n){var r=t.cloneNode(!0),o=Object.create(e,{target:{value:r},currentTarget:{value:r}});return r.value=n,typeof t.selectionStart=="number"&&typeof t.selectionEnd=="number"&&(r.selectionStart=t.selectionStart,r.selectionEnd=t.selectionEnd),r.setSelectionRange=function(){t.setSelectionRange.apply(t,arguments)},o}function cd(e,t,n,r){if(n){var o=t;if(t.type==="click"){o=dE(t,e,""),n(o);return}if(e.type!=="file"&&r!==void 0){o=dE(t,e,r),n(o);return}n(o)}}function L2(e,t){if(e){e.focus(t);var n=t||{},r=n.cursor;if(r){var o=e.value.length;switch(r){case"start":e.setSelectionRange(0,0);break;case"end":e.setSelectionRange(o,o);break;default:e.setSelectionRange(0,o)}}}}var H2=oe.forwardRef(function(e,t){var n,r,o,l=e.inputElement,c=e.children,d=e.prefixCls,m=e.prefix,v=e.suffix,g=e.addonBefore,p=e.addonAfter,y=e.className,E=e.style,C=e.disabled,x=e.readOnly,b=e.focused,w=e.triggerFocus,$=e.allowClear,O=e.value,T=e.handleReset,R=e.hidden,M=e.classes,N=e.classNames,z=e.dataAttrs,P=e.styles,D=e.components,F=e.onClear,W=c??l,V=(D==null?void 0:D.affixWrapper)||"span",q=(D==null?void 0:D.groupWrapper)||"span",K=(D==null?void 0:D.wrapper)||"span",_=(D==null?void 0:D.groupAddon)||"span",j=u.useRef(null),B=function(se){var ye;(ye=j.current)!==null&&ye!==void 0&&ye.contains(se.target)&&(w==null||w())},k=w8(e),I=u.cloneElement(W,{value:O,className:re((n=W.props)===null||n===void 0?void 0:n.className,!k&&(N==null?void 0:N.variant))||null}),H=u.useRef(null);if(oe.useImperativeHandle(t,function(){return{nativeElement:H.current||j.current}}),k){var G=null;if($){var X=!C&&!x&&O,U="".concat(d,"-clear-icon"),J=lt($)==="object"&&$!==null&&$!==void 0&&$.clearIcon?$.clearIcon:"✖";G=oe.createElement("button",{type:"button",tabIndex:-1,onClick:function(se){T==null||T(se),F==null||F()},onMouseDown:function(se){return se.preventDefault()},className:re(U,Y(Y({},"".concat(U,"-hidden"),!X),"".concat(U,"-has-suffix"),!!v))},J)}var te="".concat(d,"-affix-wrapper"),ve=re(te,Y(Y(Y(Y(Y({},"".concat(d,"-disabled"),C),"".concat(te,"-disabled"),C),"".concat(te,"-focused"),b),"".concat(te,"-readonly"),x),"".concat(te,"-input-with-clear-btn"),v&&$&&O),M==null?void 0:M.affixWrapper,N==null?void 0:N.affixWrapper,N==null?void 0:N.variant),ne=(v||$)&&oe.createElement("span",{className:re("".concat(d,"-suffix"),N==null?void 0:N.suffix),style:P==null?void 0:P.suffix},G,v);I=oe.createElement(V,Ie({className:ve,style:P==null?void 0:P.affixWrapper,onClick:B},z==null?void 0:z.affixWrapper,{ref:j}),m&&oe.createElement("span",{className:re("".concat(d,"-prefix"),N==null?void 0:N.prefix),style:P==null?void 0:P.prefix},m),I,ne)}if(E8(e)){var le="".concat(d,"-group"),be="".concat(le,"-addon"),fe="".concat(le,"-wrapper"),ie=re("".concat(d,"-wrapper"),le,M==null?void 0:M.wrapper,N==null?void 0:N.wrapper),pe=re(fe,Y({},"".concat(fe,"-disabled"),C),M==null?void 0:M.group,N==null?void 0:N.groupWrapper);I=oe.createElement(q,{className:pe,ref:H},oe.createElement(K,{className:ie},g&&oe.createElement(_,{className:be},g),I,p&&oe.createElement(_,{className:be},p)))}return oe.cloneElement(I,{className:re((r=I.props)===null||r===void 0?void 0:r.className,y)||null,style:Z(Z({},(o=I.props)===null||o===void 0?void 0:o.style),E),hidden:R})}),$8=["show"];function F2(e,t){return u.useMemo(function(){var n={};t&&(n.show=lt(t)==="object"&&t.formatter?t.formatter:!!t),n=Z(Z({},n),e);var r=n,o=r.show,l=wt(r,$8);return Z(Z({},l),{},{show:!!o,showFormatter:typeof o=="function"?o:void 0,strategy:l.strategy||function(c){return c.length}})},[e,t])}var O8=["autoComplete","onChange","onFocus","onBlur","onPressEnter","onKeyDown","onKeyUp","prefixCls","disabled","htmlSize","className","maxLength","suffix","showCount","count","type","classes","classNames","styles","onCompositionStart","onCompositionEnd"],R8=u.forwardRef(function(e,t){var n=e.autoComplete,r=e.onChange,o=e.onFocus,l=e.onBlur,c=e.onPressEnter,d=e.onKeyDown,m=e.onKeyUp,v=e.prefixCls,g=v===void 0?"rc-input":v,p=e.disabled,y=e.htmlSize,E=e.className,C=e.maxLength,x=e.suffix,b=e.showCount,w=e.count,$=e.type,O=$===void 0?"text":$,T=e.classes,R=e.classNames,M=e.styles,N=e.onCompositionStart,z=e.onCompositionEnd,P=wt(e,O8),D=u.useState(!1),F=de(D,2),W=F[0],V=F[1],q=u.useRef(!1),K=u.useRef(!1),_=u.useRef(null),j=u.useRef(null),B=function(Me){_.current&&L2(_.current,Me)},k=gr(e.defaultValue,{value:e.value}),I=de(k,2),H=I[0],G=I[1],X=H==null?"":String(H),U=u.useState(null),J=de(U,2),te=J[0],ve=J[1],ne=F2(w,b),le=ne.max||C,be=ne.strategy(X),fe=!!le&&be>le;u.useImperativeHandle(t,function(){var xe;return{focus:B,blur:function(){var Ve;(Ve=_.current)===null||Ve===void 0||Ve.blur()},setSelectionRange:function(Ve,Ke,We){var at;(at=_.current)===null||at===void 0||at.setSelectionRange(Ve,Ke,We)},select:function(){var Ve;(Ve=_.current)===null||Ve===void 0||Ve.select()},input:_.current,nativeElement:((xe=j.current)===null||xe===void 0?void 0:xe.nativeElement)||_.current}}),u.useEffect(function(){K.current&&(K.current=!1),V(function(xe){return xe&&p?!1:xe})},[p]);var ie=function(Me,Ve,Ke){var We=Ve;if(!q.current&&ne.exceedFormatter&&ne.max&&ne.strategy(Ve)>ne.max){if(We=ne.exceedFormatter(Ve,{max:ne.max}),Ve!==We){var at,Ue;ve([((at=_.current)===null||at===void 0?void 0:at.selectionStart)||0,((Ue=_.current)===null||Ue===void 0?void 0:Ue.selectionEnd)||0])}}else if(Ke.source==="compositionEnd")return;G(We),_.current&&cd(_.current,Me,r,We)};u.useEffect(function(){if(te){var xe;(xe=_.current)===null||xe===void 0||xe.setSelectionRange.apply(xe,Te(te))}},[te]);var pe=function(Me){ie(Me,Me.target.value,{source:"change"})},Re=function(Me){q.current=!1,ie(Me,Me.currentTarget.value,{source:"compositionEnd"}),z==null||z(Me)},se=function(Me){c&&Me.key==="Enter"&&!K.current&&(K.current=!0,c(Me)),d==null||d(Me)},ye=function(Me){Me.key==="Enter"&&(K.current=!1),m==null||m(Me)},$e=function(Me){V(!0),o==null||o(Me)},we=function(Me){K.current&&(K.current=!1),V(!1),l==null||l(Me)},Ne=function(Me){G(""),B(),_.current&&cd(_.current,Me,r)},Ze=fe&&"".concat(g,"-out-of-range"),He=function(){var Me=Tn(e,["prefixCls","onPressEnter","addonBefore","addonAfter","prefix","suffix","allowClear","defaultValue","showCount","count","classes","htmlSize","styles","classNames","onClear"]);return oe.createElement("input",Ie({autoComplete:n},Me,{onChange:pe,onFocus:$e,onBlur:we,onKeyDown:se,onKeyUp:ye,className:re(g,Y({},"".concat(g,"-disabled"),p),R==null?void 0:R.input),style:M==null?void 0:M.input,ref:_,size:y,type:O,onCompositionStart:function(Ke){q.current=!0,N==null||N(Ke)},onCompositionEnd:Re}))},Xe=function(){var Me=Number(le)>0;if(x||ne.show){var Ve=ne.showFormatter?ne.showFormatter({value:X,count:be,maxLength:le}):"".concat(be).concat(Me?" / ".concat(le):"");return oe.createElement(oe.Fragment,null,ne.show&&oe.createElement("span",{className:re("".concat(g,"-show-count-suffix"),Y({},"".concat(g,"-show-count-has-suffix"),!!x),R==null?void 0:R.count),style:Z({},M==null?void 0:M.count)},Ve),x)}return null};return oe.createElement(H2,Ie({},P,{prefixCls:g,className:re(E,Ze),handleReset:Ne,value:X,focused:W,triggerFocus:B,suffix:Xe(),disabled:p,classes:T,classNames:R,styles:M,ref:j}),He())});const V2=e=>{let t;return typeof e=="object"&&(e!=null&&e.clearIcon)?t=e:e&&(t={clearIcon:oe.createElement(Xl,null)}),t};function U2(e,t){const n=u.useRef([]),r=()=>{n.current.push(setTimeout(()=>{var o,l,c,d;!((o=e.current)===null||o===void 0)&&o.input&&((l=e.current)===null||l===void 0?void 0:l.input.getAttribute("type"))==="password"&&(!((c=e.current)===null||c===void 0)&&c.input.hasAttribute("value"))&&((d=e.current)===null||d===void 0||d.input.removeAttribute("value"))}))};return u.useEffect(()=>(t&&r(),()=>n.current.forEach(o=>{o&&clearTimeout(o)})),[]),r}function T8(e){return!!(e.prefix||e.suffix||e.allowClear||e.showCount)}var M8=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const em=u.forwardRef((e,t)=>{const{prefixCls:n,bordered:r=!0,status:o,size:l,disabled:c,onBlur:d,onFocus:m,suffix:v,allowClear:g,addonAfter:p,addonBefore:y,className:E,style:C,styles:x,rootClassName:b,onChange:w,classNames:$,variant:O}=e,T=M8(e,["prefixCls","bordered","status","size","disabled","onBlur","onFocus","suffix","allowClear","addonAfter","addonBefore","className","style","styles","rootClassName","onChange","classNames","variant"]),{getPrefixCls:R,direction:M,allowClear:N,autoComplete:z,className:P,style:D,classNames:F,styles:W}=zr("input"),V=R("input",n),q=u.useRef(null),K=na(V),[_,j,B]=P2(V,b),[k]=z2(V,K),{compactSize:I,compactItemClassnames:H}=Xc(V,M),G=Mo(we=>{var Ne;return(Ne=l??I)!==null&&Ne!==void 0?Ne:we}),X=oe.useContext(Co),U=c??X,{status:J,hasFeedback:te,feedbackIcon:ve}=u.useContext(Jr),ne=Gd(J,o),le=T8(e)||!!te;u.useRef(le);const be=U2(q,!0),fe=we=>{be(),d==null||d(we)},ie=we=>{be(),m==null||m(we)},pe=we=>{be(),w==null||w(we)},Re=(te||v)&&oe.createElement(oe.Fragment,null,v,te&&ve),se=V2(g??N),[ye,$e]=N0("input",O,r);return _(k(oe.createElement(R8,Object.assign({ref:Ia(t,q),prefixCls:V,autoComplete:z},T,{disabled:U,onBlur:fe,onFocus:ie,style:Object.assign(Object.assign({},D),C),styles:Object.assign(Object.assign({},W),x),suffix:Re,allowClear:se,className:re(E,b,B,K,H,P),onChange:pe,addonBefore:y&&oe.createElement(Ac,{form:!0,space:!0},y),addonAfter:p&&oe.createElement(Ac,{form:!0,space:!0},p),classNames:Object.assign(Object.assign(Object.assign({},$),F),{input:re({[`${V}-sm`]:G==="small",[`${V}-lg`]:G==="large",[`${V}-rtl`]:M==="rtl"},$==null?void 0:$.input,F.input,j),variant:re({[`${V}-${ye}`]:$e},od(V,ne)),affixWrapper:re({[`${V}-affix-wrapper-sm`]:G==="small",[`${V}-affix-wrapper-lg`]:G==="large",[`${V}-affix-wrapper-rtl`]:M==="rtl"},j),wrapper:re({[`${V}-group-rtl`]:M==="rtl"},j),groupWrapper:re({[`${V}-group-wrapper-sm`]:G==="small",[`${V}-group-wrapper-lg`]:G==="large",[`${V}-group-wrapper-rtl`]:M==="rtl",[`${V}-group-wrapper-${ye}`]:$e},od(`${V}-group-wrapper`,ne,te),j)})}))))});function mE(e){return["small","middle","large"].includes(e)}function uW(e){return e?typeof e=="number"&&!Number.isNaN(e):!1}const q2=["wrap","nowrap","wrap-reverse"],W2=["flex-start","flex-end","start","end","center","space-between","space-around","space-evenly","stretch","normal","left","right"],k2=["center","start","end","flex-start","flex-end","self-start","self-end","baseline","normal","stretch"],_8=(e,t)=>{const n=t.wrap===!0?"wrap":t.wrap;return{[`${e}-wrap-${n}`]:n&&q2.includes(n)}},N8=(e,t)=>{const n={};return k2.forEach(r=>{n[`${e}-align-${r}`]=t.align===r}),n[`${e}-align-stretch`]=!t.align&&!!t.vertical,n},A8=(e,t)=>{const n={};return W2.forEach(r=>{n[`${e}-justify-${r}`]=t.justify===r}),n};function I8(e,t){return re(Object.assign(Object.assign(Object.assign({},_8(e,t)),N8(e,t)),A8(e,t)))}const j8=e=>{const{componentCls:t}=e;return{[t]:{display:"flex",margin:0,padding:0,"&-vertical":{flexDirection:"column"},"&-rtl":{direction:"rtl"},"&:empty":{display:"none"}}}},P8=e=>{const{componentCls:t}=e;return{[t]:{"&-gap-small":{gap:e.flexGapSM},"&-gap-middle":{gap:e.flexGap},"&-gap-large":{gap:e.flexGapLG}}}},z8=e=>{const{componentCls:t}=e,n={};return q2.forEach(r=>{n[`${t}-wrap-${r}`]={flexWrap:r}}),n},D8=e=>{const{componentCls:t}=e,n={};return k2.forEach(r=>{n[`${t}-align-${r}`]={alignItems:r}}),n},B8=e=>{const{componentCls:t}=e,n={};return W2.forEach(r=>{n[`${t}-justify-${r}`]={justifyContent:r}}),n},L8=()=>({}),H8=wn("Flex",e=>{const{paddingXS:t,padding:n,paddingLG:r}=e,o=kt(e,{flexGapSM:t,flexGap:n,flexGapLG:r});return[j8(o),P8(o),z8(o),D8(o),B8(o)]},L8,{resetStyle:!1});var F8=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const Jg=oe.forwardRef((e,t)=>{const{prefixCls:n,rootClassName:r,className:o,style:l,flex:c,gap:d,children:m,vertical:v=!1,component:g="div"}=e,p=F8(e,["prefixCls","rootClassName","className","style","flex","gap","children","vertical","component"]),{flex:y,direction:E,getPrefixCls:C}=oe.useContext(vt),x=C("flex",n),[b,w,$]=H8(x),O=v??(y==null?void 0:y.vertical),T=re(o,r,y==null?void 0:y.className,x,w,$,I8(x,e),{[`${x}-rtl`]:E==="rtl",[`${x}-gap-${d}`]:mE(d),[`${x}-vertical`]:O}),R=Object.assign(Object.assign({},y==null?void 0:y.style),l);return c&&(R.flex=c),d&&!mE(d)&&(R.gap=d),b(oe.createElement(g,Object.assign({ref:t,className:T,style:R},Tn(p,["justify","wrap","align"])),m))});function ud(e){const[t,n]=u.useState(e);return u.useEffect(()=>{const r=setTimeout(()=>{n(e)},e.length?0:10);return()=>{clearTimeout(r)}},[e]),t}const V8=e=>{const{componentCls:t}=e,n=`${t}-show-help`,r=`${t}-show-help-item`;return{[n]:{transition:`opacity ${e.motionDurationFast} ${e.motionEaseInOut}`,"&-appear, &-enter":{opacity:0,"&-active":{opacity:1}},"&-leave":{opacity:1,"&-active":{opacity:0}},[r]:{overflow:"hidden",transition:`height ${e.motionDurationFast} ${e.motionEaseInOut},
                     opacity ${e.motionDurationFast} ${e.motionEaseInOut},
                     transform ${e.motionDurationFast} ${e.motionEaseInOut} !important`,[`&${r}-appear, &${r}-enter`]:{transform:"translateY(-5px)",opacity:0,"&-active":{transform:"translateY(0)",opacity:1}},[`&${r}-leave-active`]:{transform:"translateY(-5px)"}}}}},U8=e=>({legend:{display:"block",width:"100%",marginBottom:e.marginLG,padding:0,color:e.colorTextDescription,fontSize:e.fontSizeLG,lineHeight:"inherit",border:0,borderBottom:`${he(e.lineWidth)} ${e.lineType} ${e.colorBorder}`},'input[type="search"]':{boxSizing:"border-box"},'input[type="radio"], input[type="checkbox"]':{lineHeight:"normal"},'input[type="file"]':{display:"block"},'input[type="range"]':{display:"block",width:"100%"},"select[multiple], select[size]":{height:"auto"},"input[type='file']:focus,\n  input[type='radio']:focus,\n  input[type='checkbox']:focus":{outline:0,boxShadow:`0 0 0 ${he(e.controlOutlineWidth)} ${e.controlOutline}`},output:{display:"block",paddingTop:15,color:e.colorText,fontSize:e.fontSize,lineHeight:e.lineHeight}}),vE=(e,t)=>{const{formItemCls:n}=e;return{[n]:{[`${n}-label > label`]:{height:t},[`${n}-control-input`]:{minHeight:t}}}},q8=e=>{const{componentCls:t}=e;return{[e.componentCls]:Object.assign(Object.assign(Object.assign({},Tr(e)),U8(e)),{[`${t}-text`]:{display:"inline-block",paddingInlineEnd:e.paddingSM},"&-small":Object.assign({},vE(e,e.controlHeightSM)),"&-large":Object.assign({},vE(e,e.controlHeightLG))})}},W8=e=>{const{formItemCls:t,iconCls:n,rootPrefixCls:r,antCls:o,labelRequiredMarkColor:l,labelColor:c,labelFontSize:d,labelHeight:m,labelColonMarginInlineStart:v,labelColonMarginInlineEnd:g,itemMarginBottom:p}=e;return{[t]:Object.assign(Object.assign({},Tr(e)),{marginBottom:p,verticalAlign:"top","&-with-help":{transition:"none"},[`&-hidden,
        &-hidden${o}-row`]:{display:"none"},"&-has-warning":{[`${t}-split`]:{color:e.colorError}},"&-has-error":{[`${t}-split`]:{color:e.colorWarning}},[`${t}-label`]:{flexGrow:0,overflow:"hidden",whiteSpace:"nowrap",textAlign:"end",verticalAlign:"middle","&-left":{textAlign:"start"},"&-wrap":{overflow:"unset",lineHeight:e.lineHeight,whiteSpace:"unset"},"> label":{position:"relative",display:"inline-flex",alignItems:"center",maxWidth:"100%",height:m,color:c,fontSize:d,[`> ${n}`]:{fontSize:e.fontSize,verticalAlign:"top"},[`&${t}-required`]:{"&::before":{display:"inline-block",marginInlineEnd:e.marginXXS,color:l,fontSize:e.fontSize,fontFamily:"SimSun, sans-serif",lineHeight:1,content:'"*"'},[`&${t}-required-mark-hidden, &${t}-required-mark-optional`]:{"&::before":{display:"none"}}},[`${t}-optional`]:{display:"inline-block",marginInlineStart:e.marginXXS,color:e.colorTextDescription,[`&${t}-required-mark-hidden`]:{display:"none"}},[`${t}-tooltip`]:{color:e.colorTextDescription,cursor:"help",writingMode:"horizontal-tb",marginInlineStart:e.marginXXS},"&::after":{content:'":"',position:"relative",marginBlock:0,marginInlineStart:v,marginInlineEnd:g},[`&${t}-no-colon::after`]:{content:'"\\a0"'}}},[`${t}-control`]:{"--ant-display":"flex",flexDirection:"column",flexGrow:1,[`&:first-child:not([class^="'${r}-col-'"]):not([class*="' ${r}-col-'"])`]:{width:"100%"},"&-input":{position:"relative",display:"flex",alignItems:"center",minHeight:e.controlHeight,"&-content":{flex:"auto",maxWidth:"100%"}}},[t]:{"&-additional":{display:"flex",flexDirection:"column"},"&-explain, &-extra":{clear:"both",color:e.colorTextDescription,fontSize:e.fontSize,lineHeight:e.lineHeight},"&-explain-connected":{width:"100%"},"&-extra":{minHeight:e.controlHeightSM,transition:`color ${e.motionDurationMid} ${e.motionEaseOut}`},"&-explain":{"&-error":{color:e.colorError},"&-warning":{color:e.colorWarning}}},[`&-with-help ${t}-explain`]:{height:"auto",opacity:1},[`${t}-feedback-icon`]:{fontSize:e.fontSize,textAlign:"center",visibility:"visible",animationName:g0,animationDuration:e.motionDurationMid,animationTimingFunction:e.motionEaseOutBack,pointerEvents:"none","&-success":{color:e.colorSuccess},"&-error":{color:e.colorError},"&-warning":{color:e.colorWarning},"&-validating":{color:e.colorPrimary}}})}},gE=(e,t)=>{const{formItemCls:n}=e;return{[`${t}-horizontal`]:{[`${n}-label`]:{flexGrow:0},[`${n}-control`]:{flex:"1 1 0",minWidth:0},[`${n}-label[class$='-24'], ${n}-label[class*='-24 ']`]:{[`& + ${n}-control`]:{minWidth:"unset"}}}}},k8=e=>{const{componentCls:t,formItemCls:n,inlineItemMarginBottom:r}=e;return{[`${t}-inline`]:{display:"flex",flexWrap:"wrap",[n]:{flex:"none",marginInlineEnd:e.margin,marginBottom:r,"&-row":{flexWrap:"nowrap"},[`> ${n}-label,
        > ${n}-control`]:{display:"inline-block",verticalAlign:"top"},[`> ${n}-label`]:{flex:"none"},[`${t}-text`]:{display:"inline-block"},[`${n}-has-feedback`]:{display:"inline-block"}}}}},Ra=e=>({padding:e.verticalLabelPadding,margin:e.verticalLabelMargin,whiteSpace:"initial",textAlign:"start","> label":{margin:0,"&::after":{visibility:"hidden"}}}),G2=e=>{const{componentCls:t,formItemCls:n,rootPrefixCls:r}=e;return{[`${n} ${n}-label`]:Ra(e),[`${t}:not(${t}-inline)`]:{[n]:{flexWrap:"wrap",[`${n}-label, ${n}-control`]:{[`&:not([class*=" ${r}-col-xs"])`]:{flex:"0 0 100%",maxWidth:"100%"}}}}}},G8=e=>{const{componentCls:t,formItemCls:n,antCls:r}=e;return{[`${t}-vertical`]:{[`${n}:not(${n}-horizontal)`]:{[`${n}-row`]:{flexDirection:"column"},[`${n}-label > label`]:{height:"auto"},[`${n}-control`]:{width:"100%"},[`${n}-label,
        ${r}-col-24${n}-label,
        ${r}-col-xl-24${n}-label`]:Ra(e)}},[`@media (max-width: ${he(e.screenXSMax)})`]:[G2(e),{[t]:{[`${n}:not(${n}-horizontal)`]:{[`${r}-col-xs-24${n}-label`]:Ra(e)}}}],[`@media (max-width: ${he(e.screenSMMax)})`]:{[t]:{[`${n}:not(${n}-horizontal)`]:{[`${r}-col-sm-24${n}-label`]:Ra(e)}}},[`@media (max-width: ${he(e.screenMDMax)})`]:{[t]:{[`${n}:not(${n}-horizontal)`]:{[`${r}-col-md-24${n}-label`]:Ra(e)}}},[`@media (max-width: ${he(e.screenLGMax)})`]:{[t]:{[`${n}:not(${n}-horizontal)`]:{[`${r}-col-lg-24${n}-label`]:Ra(e)}}}}},K8=e=>{const{formItemCls:t,antCls:n}=e;return{[`${t}-vertical`]:{[`${t}-row`]:{flexDirection:"column"},[`${t}-label > label`]:{height:"auto"},[`${t}-control`]:{width:"100%"}},[`${t}-vertical ${t}-label,
      ${n}-col-24${t}-label,
      ${n}-col-xl-24${t}-label`]:Ra(e),[`@media (max-width: ${he(e.screenXSMax)})`]:[G2(e),{[t]:{[`${n}-col-xs-24${t}-label`]:Ra(e)}}],[`@media (max-width: ${he(e.screenSMMax)})`]:{[t]:{[`${n}-col-sm-24${t}-label`]:Ra(e)}},[`@media (max-width: ${he(e.screenMDMax)})`]:{[t]:{[`${n}-col-md-24${t}-label`]:Ra(e)}},[`@media (max-width: ${he(e.screenLGMax)})`]:{[t]:{[`${n}-col-lg-24${t}-label`]:Ra(e)}}}},X8=e=>({labelRequiredMarkColor:e.colorError,labelColor:e.colorTextHeading,labelFontSize:e.fontSize,labelHeight:e.controlHeight,labelColonMarginInlineStart:e.marginXXS/2,labelColonMarginInlineEnd:e.marginXS,itemMarginBottom:e.marginLG,verticalLabelPadding:`0 0 ${e.paddingXS}px`,verticalLabelMargin:0,inlineItemMarginBottom:0}),K2=(e,t)=>kt(e,{formItemCls:`${e.componentCls}-item`,rootPrefixCls:t}),V0=wn("Form",(e,t)=>{let{rootPrefixCls:n}=t;const r=K2(e,n);return[q8(r),W8(r),V8(r),gE(r,r.componentCls),gE(r,r.formItemCls),k8(r),G8(r),K8(r),v0(r),g0]},X8,{order:-1e3}),hE=[];function eh(e,t,n){let r=arguments.length>3&&arguments[3]!==void 0?arguments[3]:0;return{key:typeof e=="string"?e:`${t}-${r}`,error:e,errorStatus:n}}const X2=e=>{let{help:t,helpStatus:n,errors:r=hE,warnings:o=hE,className:l,fieldId:c,onVisibleChanged:d}=e;const{prefixCls:m}=u.useContext(w0),v=`${m}-item-explain`,g=na(m),[p,y,E]=V0(m,g),C=u.useMemo(()=>nd(m),[m]),x=ud(r),b=ud(o),w=u.useMemo(()=>t!=null?[eh(t,"help",n)]:[].concat(Te(x.map((T,R)=>eh(T,"error","error",R))),Te(b.map((T,R)=>eh(T,"warning","warning",R)))),[t,n,x,b]),$=u.useMemo(()=>{const T={};return w.forEach(R=>{let{key:M}=R;T[M]=(T[M]||0)+1}),w.map((R,M)=>Object.assign(Object.assign({},R),{key:T[R.key]>1?`${R.key}-fallback-${M}`:R.key}))},[w]),O={};return c&&(O.id=`${c}_help`),p(u.createElement(Xa,{motionDeadline:C.motionDeadline,motionName:`${m}-show-help`,visible:!!$.length,onVisibleChanged:d},T=>{const{className:R,style:M}=T;return u.createElement("div",Object.assign({},O,{className:re(v,R,E,g,l,y),style:M}),u.createElement(E$,Object.assign({keys:$},nd(m),{motionName:`${m}-show-help-item`,component:!1}),N=>{const{key:z,error:P,errorStatus:D,className:F,style:W}=N;return u.createElement("div",{key:z,className:re(F,{[`${v}-${D}`]:D}),style:W},P)}))}))},Y8=["parentNode"],Q8="form_item";function Sc(e){return e===void 0||e===!1?[]:Array.isArray(e)?e:[e]}function Y2(e,t){if(!e.length)return;const n=e.join("_");return t?`${t}_${n}`:Y8.includes(n)?`${Q8}_${n}`:n}function Q2(e,t,n,r,o,l){let c=r;return l!==void 0?c=l:n.validating?c="validating":e.length?c="error":t.length?c="warning":(n.touched||o&&n.validated)&&(c="success"),c}var Z8=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function pE(e){return Sc(e).join("_")}function yE(e,t){const n=t.getFieldInstance(e),r=ww(n);if(r)return r;const o=Y2(Sc(e),t.__INTERNAL__.name);if(o)return document.getElementById(o)}function Z2(e){const[t]=E0(),n=u.useRef({}),r=u.useMemo(()=>e??Object.assign(Object.assign({},t),{__INTERNAL__:{itemRef:o=>l=>{const c=pE(o);l?n.current[c]=l:delete n.current[c]}},scrollToField:function(o){let l=arguments.length>1&&arguments[1]!==void 0?arguments[1]:{};const{focus:c}=l,d=Z8(l,["focus"]),m=yE(o,r);m&&(p4(m,Object.assign({scrollMode:"if-needed",block:"nearest"},d)),c&&r.focusField(o))},focusField:o=>{var l,c;const d=r.getFieldInstance(o);typeof(d==null?void 0:d.focus)=="function"?d.focus():(c=(l=yE(o,r))===null||l===void 0?void 0:l.focus)===null||c===void 0||c.call(l)},getFieldInstance:o=>{const l=pE(o);return n.current[l]}}),[e,t]);return[r]}var J8=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const eH=(e,t)=>{const n=u.useContext(Co),{getPrefixCls:r,direction:o,requiredMark:l,colon:c,scrollToFirstError:d,className:m,style:v}=zr("form"),{prefixCls:g,className:p,rootClassName:y,size:E,disabled:C=n,form:x,colon:b,labelAlign:w,labelWrap:$,labelCol:O,wrapperCol:T,hideRequiredMark:R,layout:M="horizontal",scrollToFirstError:N,requiredMark:z,onFinishFailed:P,name:D,style:F,feedbackIcons:W,variant:V}=e,q=J8(e,["prefixCls","className","rootClassName","size","disabled","form","colon","labelAlign","labelWrap","labelCol","wrapperCol","hideRequiredMark","layout","scrollToFirstError","requiredMark","onFinishFailed","name","style","feedbackIcons","variant"]),K=Mo(E),_=u.useContext(Jw),j=u.useMemo(()=>z!==void 0?z:R?!1:l!==void 0?l:!0,[R,z,l]),B=b??c,k=r("form",g),I=na(k),[H,G,X]=V0(k,I),U=re(k,`${k}-${M}`,{[`${k}-hide-required-mark`]:j===!1,[`${k}-rtl`]:o==="rtl",[`${k}-${K}`]:K},X,I,G,m,p,y),[J]=Z2(x),{__INTERNAL__:te}=J;te.name=D;const ve=u.useMemo(()=>({name:D,labelAlign:w,labelCol:O,labelWrap:$,wrapperCol:T,vertical:M==="vertical",colon:B,requiredMark:j,itemRef:te.itemRef,form:J,feedbackIcons:W}),[D,w,O,T,M,B,j,J,W]),ne=u.useRef(null);u.useImperativeHandle(t,()=>{var fe;return Object.assign(Object.assign({},J),{nativeElement:(fe=ne.current)===null||fe===void 0?void 0:fe.nativeElement})});const le=(fe,ie)=>{if(fe){let pe={block:"nearest"};typeof fe=="object"&&(pe=Object.assign(Object.assign({},pe),fe)),J.scrollToField(ie,pe)}},be=fe=>{if(P==null||P(fe),fe.errorFields.length){const ie=fe.errorFields[0].name;if(N!==void 0){le(N,ie);return}d!==void 0&&le(d,ie)}};return H(u.createElement(cO.Provider,{value:V},u.createElement(a0,{disabled:C},u.createElement(Li.Provider,{value:K},u.createElement(sO,{validateMessages:_},u.createElement(xo.Provider,{value:ve},u.createElement(ts,Object.assign({id:D},q,{name:D,onFinishFailed:be,form:J,ref:ne,style:Object.assign(Object.assign({},v),F),className:U}))))))))},tH=u.forwardRef(eH);function nH(e){if(typeof e=="function")return e;const t=Pr(e);return t.length<=1?t[0]:t}const J2=()=>{const{status:e,errors:t=[],warnings:n=[]}=u.useContext(Jr);return{status:e,errors:t,warnings:n}};J2.Context=Jr;function rH(e){const[t,n]=u.useState(e),r=u.useRef(null),o=u.useRef([]),l=u.useRef(!1);u.useEffect(()=>(l.current=!1,()=>{l.current=!0,gn.cancel(r.current),r.current=null}),[]);function c(d){l.current||(r.current===null&&(o.current=[],r.current=gn(()=>{r.current=null,n(m=>{let v=m;return o.current.forEach(g=>{v=g(v)}),v})})),o.current.push(d))}return[t,c]}function aH(){const{itemRef:e}=u.useContext(xo),t=u.useRef({});function n(r,o){const l=o&&typeof o=="object"&&Zl(o),c=r.join("_");return(t.current.name!==c||t.current.originRef!==l)&&(t.current.name=c,t.current.originRef=l,t.current.ref=Ia(e(r),l)),t.current.ref}return n}const oH=e=>{const{formItemCls:t}=e;return{"@media screen and (-ms-high-contrast: active), (-ms-high-contrast: none)":{[`${t}-control`]:{display:"flex"}}}},iH=_d(["Form","item-item"],(e,t)=>{let{rootPrefixCls:n}=t;const r=K2(e,n);return[oH(r)]});var lH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const sH=24,cH=e=>{const{prefixCls:t,status:n,labelCol:r,wrapperCol:o,children:l,errors:c,warnings:d,_internalItemRender:m,extra:v,help:g,fieldId:p,marginBottom:y,onErrorVisibleChanged:E,label:C}=e,x=`${t}-item`,b=u.useContext(xo),w=u.useMemo(()=>{let q=Object.assign({},o||b.wrapperCol||{});return C===null&&!r&&!o&&b.labelCol&&[void 0,"xs","sm","md","lg","xl","xxl"].forEach(_=>{const j=_?[_]:[],B=ba(b.labelCol,j),k=typeof B=="object"?B:{},I=ba(q,j),H=typeof I=="object"?I:{};"span"in k&&!("offset"in H)&&k.span<sH&&(q=ya(q,[].concat(j,["offset"]),k.span))}),q},[o,b]),$=re(`${x}-control`,w.className),O=u.useMemo(()=>{const{labelCol:q,wrapperCol:K}=b;return lH(b,["labelCol","wrapperCol"])},[b]),T=u.useRef(null),[R,M]=u.useState(0);Wt(()=>{v&&T.current?M(T.current.clientHeight):M(0)},[v]);const N=u.createElement("div",{className:`${x}-control-input`},u.createElement("div",{className:`${x}-control-input-content`},l)),z=u.useMemo(()=>({prefixCls:t,status:n}),[t,n]),P=y!==null||c.length||d.length?u.createElement(w0.Provider,{value:z},u.createElement(X2,{fieldId:p,errors:c,warnings:d,help:g,helpStatus:n,className:`${x}-explain-connected`,onVisibleChanged:E})):null,D={};p&&(D.id=`${p}_extra`);const F=v?u.createElement("div",Object.assign({},D,{className:`${x}-extra`,ref:T}),v):null,W=P||F?u.createElement("div",{className:`${x}-additional`,style:y?{minHeight:y+R}:{}},P,F):null,V=m&&m.mark==="pro_table_render"&&m.render?m.render(e,{input:N,errorList:P,extra:F}):u.createElement(u.Fragment,null,N,W);return u.createElement(xo.Provider,{value:O},u.createElement(B2,Object.assign({},w,{className:$}),V),u.createElement(iH,{prefixCls:t}))};var uH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function fH(e){return e?typeof e=="object"&&!u.isValidElement(e)?e:{title:e}:null}const dH=e=>{let{prefixCls:t,label:n,htmlFor:r,labelCol:o,labelAlign:l,colon:c,required:d,requiredMark:m,tooltip:v,vertical:g}=e;var p;const[y]=To("Form"),{labelAlign:E,labelCol:C,labelWrap:x,colon:b}=u.useContext(xo);if(!n)return null;const w=o||C||{},$=l||E,O=`${t}-item-label`,T=re(O,$==="left"&&`${O}-left`,w.className,{[`${O}-wrap`]:!!x});let R=n;const M=c===!0||b!==!1&&c!==!1;M&&!g&&typeof n=="string"&&n.trim()&&(R=n.replace(/[:|：]\s*$/,""));const z=fH(v);if(z){const{icon:q=u.createElement(QN,null)}=z,K=uH(z,["icon"]),_=u.createElement(si,Object.assign({},K),u.cloneElement(q,{className:`${t}-item-tooltip`,title:"",onClick:j=>{j.preventDefault()},tabIndex:null}));R=u.createElement(u.Fragment,null,R,_)}const P=m==="optional",D=typeof m=="function",F=m===!1;D?R=m(R,{required:!!d}):P&&!d&&(R=u.createElement(u.Fragment,null,R,u.createElement("span",{className:`${t}-item-optional`,title:""},(y==null?void 0:y.optional)||((p=So.Form)===null||p===void 0?void 0:p.optional))));let W;F?W="hidden":(P||D)&&(W="optional");const V=re({[`${t}-item-required`]:d,[`${t}-item-required-mark-${W}`]:W,[`${t}-item-no-colon`]:!M});return u.createElement(B2,Object.assign({},w,{className:T}),u.createElement("label",{htmlFor:r,className:V,title:typeof n=="string"?n:""},R))},mH={success:wd,warning:Ql,error:Xl,validating:Uc};function eR(e){let{children:t,errors:n,warnings:r,hasFeedback:o,validateStatus:l,prefixCls:c,meta:d,noStyle:m}=e;const v=`${c}-item`,{feedbackIcons:g}=u.useContext(xo),p=Q2(n,r,d,null,!!o,l),{isFormItemInput:y,status:E,hasFeedback:C,feedbackIcon:x}=u.useContext(Jr),b=u.useMemo(()=>{var w;let $;if(o){const T=o!==!0&&o.icons||g,R=p&&((w=T==null?void 0:T({status:p,errors:n,warnings:r}))===null||w===void 0?void 0:w[p]),M=p&&mH[p];$=R!==!1&&M?u.createElement("span",{className:re(`${v}-feedback-icon`,`${v}-feedback-icon-${p}`)},R||u.createElement(M,null)):null}const O={status:p||"",errors:n,warnings:r,hasFeedback:!!o,feedbackIcon:$,isFormItemInput:!0};return m&&(O.status=(p??E)||"",O.isFormItemInput=y,O.hasFeedback=!!(o??C),O.feedbackIcon=o!==void 0?O.feedbackIcon:x),O},[p,o,m,y,E]);return u.createElement(Jr.Provider,{value:b},t)}var vH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function gH(e){const{prefixCls:t,className:n,rootClassName:r,style:o,help:l,errors:c,warnings:d,validateStatus:m,meta:v,hasFeedback:g,hidden:p,children:y,fieldId:E,required:C,isRequired:x,onSubItemMetaChange:b,layout:w}=e,$=vH(e,["prefixCls","className","rootClassName","style","help","errors","warnings","validateStatus","meta","hasFeedback","hidden","children","fieldId","required","isRequired","onSubItemMetaChange","layout"]),O=`${t}-item`,{requiredMark:T,vertical:R}=u.useContext(xo),M=R||w==="vertical",N=u.useRef(null),z=ud(c),P=ud(d),D=l!=null,F=!!(D||c.length||d.length),W=!!N.current&&jd(N.current),[V,q]=u.useState(null);Wt(()=>{if(F&&N.current){const k=getComputedStyle(N.current);q(parseInt(k.marginBottom,10))}},[F,W]);const K=k=>{k||q(null)},j=function(){let k=arguments.length>0&&arguments[0]!==void 0?arguments[0]:!1;const I=k?z:v.errors,H=k?P:v.warnings;return Q2(I,H,v,"",!!g,m)}(),B=re(O,n,r,{[`${O}-with-help`]:D||z.length||P.length,[`${O}-has-feedback`]:j&&g,[`${O}-has-success`]:j==="success",[`${O}-has-warning`]:j==="warning",[`${O}-has-error`]:j==="error",[`${O}-is-validating`]:j==="validating",[`${O}-hidden`]:p,[`${O}-${w}`]:w});return u.createElement("div",{className:B,style:o,ref:N},u.createElement(y8,Object.assign({className:`${O}-row`},Tn($,["_internalItemRender","colon","dependencies","extra","fieldKey","getValueFromEvent","getValueProps","htmlFor","id","initialValue","isListField","label","labelAlign","labelCol","labelWrap","messageVariables","name","normalize","noStyle","preserve","requiredMark","rules","shouldUpdate","trigger","tooltip","validateFirst","validateTrigger","valuePropName","wrapperCol","validateDebounce"])),u.createElement(dH,Object.assign({htmlFor:E},e,{requiredMark:T,required:C??x,prefixCls:t,vertical:M})),u.createElement(cH,Object.assign({},e,v,{errors:z,warnings:P,prefixCls:t,status:j,help:l,marginBottom:V,onErrorVisibleChanged:K}),u.createElement(lO.Provider,{value:b},u.createElement(eR,{prefixCls:t,meta:v,errors:v.errors,warnings:v.warnings,hasFeedback:g,validateStatus:j},y)))),!!V&&u.createElement("div",{className:`${O}-margin-offset`,style:{marginBottom:-V}}))}const hH="__SPLIT__";function pH(e,t){const n=Object.keys(e),r=Object.keys(t);return n.length===r.length&&n.every(o=>{const l=e[o],c=t[o];return l===c||typeof l=="function"||typeof c=="function"})}const yH=u.memo(e=>{let{children:t}=e;return t},(e,t)=>pH(e.control,t.control)&&e.update===t.update&&e.childProps.length===t.childProps.length&&e.childProps.every((n,r)=>n===t.childProps[r]));function bE(){return{errors:[],warnings:[],touched:!1,validating:!1,name:[],validated:!1}}function bH(e){const{name:t,noStyle:n,className:r,dependencies:o,prefixCls:l,shouldUpdate:c,rules:d,children:m,required:v,label:g,messageVariables:p,trigger:y="onChange",validateTrigger:E,hidden:C,help:x,layout:b}=e,{getPrefixCls:w}=u.useContext(vt),{name:$}=u.useContext(xo),O=nH(m),T=typeof O=="function",R=u.useContext(lO),{validateTrigger:M}=u.useContext(Vi),N=E!==void 0?E:M,z=t!=null,P=w("form",l),D=na(P),[F,W,V]=V0(P,D);kc();const q=u.useContext(Nc),K=u.useRef(null),[_,j]=rH({}),[B,k]=Hi(()=>bE()),I=ve=>{const ne=q==null?void 0:q.getKey(ve.name);if(k(ve.destroy?bE():ve,!0),n&&x!==!1&&R){let le=ve.name;if(ve.destroy)le=K.current||le;else if(ne!==void 0){const[be,fe]=ne;le=[be].concat(Te(fe)),K.current=le}R(ve,le)}},H=(ve,ne)=>{j(le=>{const be=Object.assign({},le),ie=[].concat(Te(ve.name.slice(0,-1)),Te(ne)).join(hH);return ve.destroy?delete be[ie]:be[ie]=ve,be})},[G,X]=u.useMemo(()=>{const ve=Te(B.errors),ne=Te(B.warnings);return Object.values(_).forEach(le=>{ve.push.apply(ve,Te(le.errors||[])),ne.push.apply(ne,Te(le.warnings||[]))}),[ve,ne]},[_,B.errors,B.warnings]),U=aH();function J(ve,ne,le){return n&&!C?u.createElement(eR,{prefixCls:P,hasFeedback:e.hasFeedback,validateStatus:e.validateStatus,meta:B,errors:G,warnings:X,noStyle:!0},ve):u.createElement(gH,Object.assign({key:"row"},e,{className:re(r,V,D,W),prefixCls:P,fieldId:ne,isRequired:le,errors:G,warnings:X,meta:B,onSubItemMetaChange:H,layout:b}),ve)}if(!z&&!T&&!o)return F(J(O));let te={};return typeof g=="string"?te.label=g:t&&(te.label=String(t)),p&&(te=Object.assign(Object.assign({},te),p)),F(u.createElement(x0,Object.assign({},e,{messageVariables:te,trigger:y,validateTrigger:N,onMetaChange:I}),(ve,ne,le)=>{const be=Sc(t).length&&ne?ne.name:[],fe=Y2(be,$),ie=v!==void 0?v:!!(d!=null&&d.some(se=>{if(se&&typeof se=="object"&&se.required&&!se.warningOnly)return!0;if(typeof se=="function"){const ye=se(le);return(ye==null?void 0:ye.required)&&!(ye!=null&&ye.warningOnly)}return!1})),pe=Object.assign({},ve);let Re=null;if(Array.isArray(O)&&z)Re=O;else if(!(T&&(!(c||o)||z))){if(!(o&&!T&&!z))if(u.isValidElement(O)){const se=Object.assign(Object.assign({},O.props),pe);if(se.id||(se.id=fe),x||G.length>0||X.length>0||e.extra){const we=[];(x||G.length>0)&&we.push(`${fe}_help`),e.extra&&we.push(`${fe}_extra`),se["aria-describedby"]=we.join(" ")}G.length>0&&(se["aria-invalid"]="true"),ie&&(se["aria-required"]="true"),ki(O)&&(se.ref=U(be,O)),new Set([].concat(Te(Sc(y)),Te(Sc(N)))).forEach(we=>{se[we]=function(){for(var Ne,Ze,He,Xe,xe,Me=arguments.length,Ve=new Array(Me),Ke=0;Ke<Me;Ke++)Ve[Ke]=arguments[Ke];(He=pe[we])===null||He===void 0||(Ne=He).call.apply(Ne,[pe].concat(Ve)),(xe=(Xe=O.props)[we])===null||xe===void 0||(Ze=xe).call.apply(Ze,[Xe].concat(Ve))}});const $e=[se["aria-required"],se["aria-invalid"],se["aria-describedby"]];Re=u.createElement(yH,{control:pe,update:O,childProps:$e},Zr(O,se))}else T&&(c||o)&&!z?Re=O(le):Re=O}return J(Re,fe,ie)}))}const tR=bH;tR.useStatus=J2;var SH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const CH=e=>{var{prefixCls:t,children:n}=e,r=SH(e,["prefixCls","children"]);const{getPrefixCls:o}=u.useContext(vt),l=o("form",t),c=u.useMemo(()=>({prefixCls:l,status:"error"}),[l]);return u.createElement(rO,Object.assign({},r),(d,m,v)=>u.createElement(w0.Provider,{value:c},n(d.map(g=>Object.assign(Object.assign({},g),{fieldKey:g.key})),m,{errors:v.errors,warnings:v.warnings})))};function xH(){const{form:e}=u.useContext(xo);return e}const Xr=tH;Xr.Item=tR;Xr.List=CH;Xr.ErrorList=X2;Xr.useForm=Z2;Xr.useFormInstance=xH;Xr.useWatch=iO;Xr.Provider=sO;Xr.create=()=>{};const EH=e=>{const{getPrefixCls:t,direction:n}=u.useContext(vt),{prefixCls:r,className:o}=e,l=t("input-group",r),c=t("input"),[d,m,v]=z2(c),g=re(l,v,{[`${l}-lg`]:e.size==="large",[`${l}-sm`]:e.size==="small",[`${l}-compact`]:e.compact,[`${l}-rtl`]:n==="rtl"},m,o),p=u.useContext(Jr),y=u.useMemo(()=>Object.assign(Object.assign({},p),{isFormItemInput:!1}),[p]);return d(u.createElement("span",{className:g,style:e.style,onMouseEnter:e.onMouseEnter,onMouseLeave:e.onMouseLeave,onFocus:e.onFocus,onBlur:e.onBlur},u.createElement(Jr.Provider,{value:y},e.children)))},wH=e=>{const{componentCls:t,paddingXS:n}=e;return{[t]:{display:"inline-flex",alignItems:"center",flexWrap:"nowrap",columnGap:n,[`${t}-input-wrapper`]:{position:"relative",[`${t}-mask-icon`]:{position:"absolute",zIndex:"1",top:"50%",right:"50%",transform:"translate(50%, -50%)",pointerEvents:"none"},[`${t}-mask-input`]:{color:"transparent",caretColor:"var(--ant-color-text)"},[`${t}-mask-input[type=number]::-webkit-inner-spin-button`]:{"-webkit-appearance":"none",margin:0},[`${t}-mask-input[type=number]`]:{"-moz-appearance":"textfield"}},"&-rtl":{direction:"rtl"},[`${t}-input`]:{textAlign:"center",paddingInline:e.paddingXXS},[`&${t}-sm ${t}-input`]:{paddingInline:e.calc(e.paddingXXS).div(2).equal()},[`&${t}-lg ${t}-input`]:{paddingInline:e.paddingXS}}}},$H=wn(["Input","OTP"],e=>{const t=kt(e,Zd(e));return[wH(t)]},Jd);var OH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const RH=u.forwardRef((e,t)=>{const{className:n,value:r,onChange:o,onActiveChange:l,index:c,mask:d}=e,m=OH(e,["className","value","onChange","onActiveChange","index","mask"]),{getPrefixCls:v}=u.useContext(vt),g=v("otp"),p=typeof d=="string"?d:r,y=u.useRef(null);u.useImperativeHandle(t,()=>y.current);const E=w=>{o(c,w.target.value)},C=()=>{gn(()=>{var w;const $=(w=y.current)===null||w===void 0?void 0:w.input;document.activeElement===$&&$&&$.select()})},x=w=>{const{key:$,ctrlKey:O,metaKey:T}=w;$==="ArrowLeft"?l(c-1):$==="ArrowRight"?l(c+1):$==="z"&&(O||T)&&w.preventDefault(),C()},b=w=>{w.key==="Backspace"&&!r&&l(c-1),C()};return u.createElement("span",{className:`${g}-input-wrapper`,role:"presentation"},d&&r!==""&&r!==void 0&&u.createElement("span",{className:`${g}-mask-icon`,"aria-hidden":"true"},p),u.createElement(em,Object.assign({"aria-label":`OTP Input ${c+1}`,type:d===!0?"password":"text"},m,{ref:y,value:r,onInput:E,onFocus:C,onKeyDown:x,onKeyUp:b,onMouseDown:C,onMouseUp:C,className:re(n,{[`${g}-mask-input`]:d})})))});var TH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function Sf(e){return(e||"").split("")}const MH=e=>{const{index:t,prefixCls:n,separator:r}=e,o=typeof r=="function"?r(t):r;return o?u.createElement("span",{className:`${n}-separator`},o):null},_H=u.forwardRef((e,t)=>{const{prefixCls:n,length:r=6,size:o,defaultValue:l,value:c,onChange:d,formatter:m,separator:v,variant:g,disabled:p,status:y,autoFocus:E,mask:C,type:x,onInput:b,inputMode:w}=e,$=TH(e,["prefixCls","length","size","defaultValue","value","onChange","formatter","separator","variant","disabled","status","autoFocus","mask","type","onInput","inputMode"]),{getPrefixCls:O,direction:T}=u.useContext(vt),R=O("otp",n),M=Ga($,{aria:!0,data:!0,attr:!0}),[N,z,P]=$H(R),D=Mo(U=>o??U),F=u.useContext(Jr),W=Gd(F.status,y),V=u.useMemo(()=>Object.assign(Object.assign({},F),{status:W,hasFeedback:!1,feedbackIcon:null}),[F,W]),q=u.useRef(null),K=u.useRef({});u.useImperativeHandle(t,()=>({focus:()=>{var U;(U=K.current[0])===null||U===void 0||U.focus()},blur:()=>{var U;for(let J=0;J<r;J+=1)(U=K.current[J])===null||U===void 0||U.blur()},nativeElement:q.current}));const _=U=>m?m(U):U,[j,B]=u.useState(()=>Sf(_(l||"")));u.useEffect(()=>{c!==void 0&&B(Sf(c))},[c]);const k=jn(U=>{B(U),b&&b(U),d&&U.length===r&&U.every(J=>J)&&U.some((J,te)=>j[te]!==J)&&d(U.join(""))}),I=jn((U,J)=>{let te=Te(j);for(let ne=0;ne<U;ne+=1)te[ne]||(te[ne]="");J.length<=1?te[U]=J:te=te.slice(0,U).concat(Sf(J)),te=te.slice(0,r);for(let ne=te.length-1;ne>=0&&!te[ne];ne-=1)te.pop();const ve=_(te.map(ne=>ne||" ").join(""));return te=Sf(ve).map((ne,le)=>ne===" "&&!te[le]?te[le]:ne),te}),H=(U,J)=>{var te;const ve=I(U,J),ne=Math.min(U+J.length,r-1);ne!==U&&ve[U]!==void 0&&((te=K.current[ne])===null||te===void 0||te.focus()),k(ve)},G=U=>{var J;(J=K.current[U])===null||J===void 0||J.focus()},X={variant:g,disabled:p,status:W,mask:C,type:x,inputMode:w};return N(u.createElement("div",Object.assign({},M,{ref:q,className:re(R,{[`${R}-sm`]:D==="small",[`${R}-lg`]:D==="large",[`${R}-rtl`]:T==="rtl"},P,z),role:"group"}),u.createElement(Jr.Provider,{value:V},Array.from({length:r}).map((U,J)=>{const te=`otp-${J}`,ve=j[J]||"";return u.createElement(u.Fragment,{key:te},u.createElement(RH,Object.assign({ref:ne=>{K.current[J]=ne},index:J,size:D,htmlSize:1,className:`${R}-input`,onChange:H,value:ve,onActiveChange:G,autoFocus:J===0&&E},X)),J<r-1&&u.createElement(MH,{separator:v,index:J,prefixCls:R}))}))))});var NH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const AH=e=>e?u.createElement(zN,null):u.createElement(IN,null),IH={click:"onClick",hover:"onMouseOver"},jH=u.forwardRef((e,t)=>{const{disabled:n,action:r="click",visibilityToggle:o=!0,iconRender:l=AH}=e,c=u.useContext(Co),d=n??c,m=typeof o=="object"&&o.visible!==void 0,[v,g]=u.useState(()=>m?o.visible:!1),p=u.useRef(null);u.useEffect(()=>{m&&g(o.visible)},[m,o]);const y=U2(p),E=()=>{var D;if(d)return;v&&y();const F=!v;g(F),typeof o=="object"&&((D=o.onVisibleChange)===null||D===void 0||D.call(o,F))},C=D=>{const F=IH[r]||"",W=l(v),V={[F]:E,className:`${D}-icon`,key:"passwordIcon",onMouseDown:q=>{q.preventDefault()},onMouseUp:q=>{q.preventDefault()}};return u.cloneElement(u.isValidElement(W)?W:u.createElement("span",null,W),V)},{className:x,prefixCls:b,inputPrefixCls:w,size:$}=e,O=NH(e,["className","prefixCls","inputPrefixCls","size"]),{getPrefixCls:T}=u.useContext(vt),R=T("input",w),M=T("input-password",b),N=o&&C(M),z=re(M,x,{[`${M}-${$}`]:!!$}),P=Object.assign(Object.assign({},Tn(O,["suffix","iconRender","visibilityToggle"])),{type:v?"text":"password",className:z,prefixCls:R,suffix:N});return $&&(P.size=$),u.createElement(em,Object.assign({ref:Ia(t,p)},P))});var PH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const zH=u.forwardRef((e,t)=>{const{prefixCls:n,inputPrefixCls:r,className:o,size:l,suffix:c,enterButton:d=!1,addonAfter:m,loading:v,disabled:g,onSearch:p,onChange:y,onCompositionStart:E,onCompositionEnd:C}=e,x=PH(e,["prefixCls","inputPrefixCls","className","size","suffix","enterButton","addonAfter","loading","disabled","onSearch","onChange","onCompositionStart","onCompositionEnd"]),{getPrefixCls:b,direction:w}=u.useContext(vt),$=u.useRef(!1),O=b("input-search",n),T=b("input",r),{compactSize:R}=Xc(O,w),M=Mo(H=>{var G;return(G=l??R)!==null&&G!==void 0?G:H}),N=u.useRef(null),z=H=>{H!=null&&H.target&&H.type==="click"&&p&&p(H.target.value,H,{source:"clear"}),y==null||y(H)},P=H=>{var G;document.activeElement===((G=N.current)===null||G===void 0?void 0:G.input)&&H.preventDefault()},D=H=>{var G,X;p&&p((X=(G=N.current)===null||G===void 0?void 0:G.input)===null||X===void 0?void 0:X.value,H,{source:"input"})},F=H=>{$.current||v||D(H)},W=typeof d=="boolean"?u.createElement(xw,null):null,V=`${O}-button`;let q;const K=d||{},_=K.type&&K.type.__ANT_BUTTON===!0;_||K.type==="button"?q=Zr(K,Object.assign({onMouseDown:P,onClick:H=>{var G,X;(X=(G=K==null?void 0:K.props)===null||G===void 0?void 0:G.onClick)===null||X===void 0||X.call(G,H),D(H)},key:"enterButton"},_?{className:V,size:M}:{})):q=u.createElement($r,{className:V,type:d?"primary":void 0,size:M,disabled:g,key:"enterButton",onMouseDown:P,onClick:D,loading:v,icon:W},d),m&&(q=[q,Zr(m,{key:"addonAfter"})]);const j=re(O,{[`${O}-rtl`]:w==="rtl",[`${O}-${M}`]:!!M,[`${O}-with-button`]:!!d},o),B=Object.assign(Object.assign({},x),{className:j,prefixCls:T,type:"search"}),k=H=>{$.current=!0,E==null||E(H)},I=H=>{$.current=!1,C==null||C(H)};return u.createElement(em,Object.assign({ref:Ia(N,t),onPressEnter:F},B,{size:M,onCompositionStart:k,onCompositionEnd:I,addonAfter:q,suffix:c,onChange:z,disabled:g}))});var DH=`
  min-height:0 !important;
  max-height:none !important;
  height:0 !important;
  visibility:hidden !important;
  overflow:hidden !important;
  position:absolute !important;
  z-index:-1000 !important;
  top:0 !important;
  right:0 !important;
  pointer-events: none !important;
`,BH=["letter-spacing","line-height","padding-top","padding-bottom","font-family","font-weight","font-size","font-variant","text-rendering","text-transform","width","text-indent","padding-left","padding-right","border-width","box-sizing","word-break","white-space"],th={},Kr;function LH(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,n=e.getAttribute("id")||e.getAttribute("data-reactid")||e.getAttribute("name");if(t&&th[n])return th[n];var r=window.getComputedStyle(e),o=r.getPropertyValue("box-sizing")||r.getPropertyValue("-moz-box-sizing")||r.getPropertyValue("-webkit-box-sizing"),l=parseFloat(r.getPropertyValue("padding-bottom"))+parseFloat(r.getPropertyValue("padding-top")),c=parseFloat(r.getPropertyValue("border-bottom-width"))+parseFloat(r.getPropertyValue("border-top-width")),d=BH.map(function(v){return"".concat(v,":").concat(r.getPropertyValue(v))}).join(";"),m={sizingStyle:d,paddingSize:l,borderSize:c,boxSizing:o};return t&&n&&(th[n]=m),m}function HH(e){var t=arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1,n=arguments.length>2&&arguments[2]!==void 0?arguments[2]:null,r=arguments.length>3&&arguments[3]!==void 0?arguments[3]:null;Kr||(Kr=document.createElement("textarea"),Kr.setAttribute("tab-index","-1"),Kr.setAttribute("aria-hidden","true"),Kr.setAttribute("name","hiddenTextarea"),document.body.appendChild(Kr)),e.getAttribute("wrap")?Kr.setAttribute("wrap",e.getAttribute("wrap")):Kr.removeAttribute("wrap");var o=LH(e,t),l=o.paddingSize,c=o.borderSize,d=o.boxSizing,m=o.sizingStyle;Kr.setAttribute("style","".concat(m,";").concat(DH)),Kr.value=e.value||e.placeholder||"";var v=void 0,g=void 0,p,y=Kr.scrollHeight;if(d==="border-box"?y+=c:d==="content-box"&&(y-=l),n!==null||r!==null){Kr.value=" ";var E=Kr.scrollHeight-l;n!==null&&(v=E*n,d==="border-box"&&(v=v+l+c),y=Math.max(v,y)),r!==null&&(g=E*r,d==="border-box"&&(g=g+l+c),p=y>g?"":"hidden",y=Math.min(g,y))}var C={height:y,overflowY:p,resize:"none"};return v&&(C.minHeight=v),g&&(C.maxHeight=g),C}var FH=["prefixCls","defaultValue","value","autoSize","onResize","className","style","disabled","onChange","onInternalAutoSize"],nh=0,rh=1,ah=2,VH=u.forwardRef(function(e,t){var n=e,r=n.prefixCls,o=n.defaultValue,l=n.value,c=n.autoSize,d=n.onResize,m=n.className,v=n.style,g=n.disabled,p=n.onChange;n.onInternalAutoSize;var y=wt(n,FH),E=gr(o,{value:l,postState:function(U){return U??""}}),C=de(E,2),x=C[0],b=C[1],w=function(U){b(U.target.value),p==null||p(U)},$=u.useRef();u.useImperativeHandle(t,function(){return{textArea:$.current}});var O=u.useMemo(function(){return c&&lt(c)==="object"?[c.minRows,c.maxRows]:[]},[c]),T=de(O,2),R=T[0],M=T[1],N=!!c,z=function(){try{if(document.activeElement===$.current){var U=$.current,J=U.selectionStart,te=U.selectionEnd,ve=U.scrollTop;$.current.setSelectionRange(J,te),$.current.scrollTop=ve}}catch{}},P=u.useState(ah),D=de(P,2),F=D[0],W=D[1],V=u.useState(),q=de(V,2),K=q[0],_=q[1],j=function(){W(nh)};Wt(function(){N&&j()},[l,R,M,N]),Wt(function(){if(F===nh)W(rh);else if(F===rh){var X=HH($.current,!1,R,M);W(ah),_(X)}else z()},[F]);var B=u.useRef(),k=function(){gn.cancel(B.current)},I=function(U){F===ah&&(d==null||d(U),c&&(k(),B.current=gn(function(){j()})))};u.useEffect(function(){return k},[]);var H=N?K:null,G=Z(Z({},v),H);return(F===nh||F===rh)&&(G.overflowY="hidden",G.overflowX="hidden"),u.createElement(Ro,{onResize:I,disabled:!(c||d)},u.createElement("textarea",Ie({},y,{ref:$,style:G,className:re(r,m,Y({},"".concat(r,"-disabled"),g)),disabled:g,value:x,onChange:w})))}),UH=["defaultValue","value","onFocus","onBlur","onChange","allowClear","maxLength","onCompositionStart","onCompositionEnd","suffix","prefixCls","showCount","count","className","style","disabled","hidden","classNames","styles","onResize","onClear","onPressEnter","readOnly","autoSize","onKeyDown"],qH=oe.forwardRef(function(e,t){var n,r=e.defaultValue,o=e.value,l=e.onFocus,c=e.onBlur,d=e.onChange,m=e.allowClear,v=e.maxLength,g=e.onCompositionStart,p=e.onCompositionEnd,y=e.suffix,E=e.prefixCls,C=E===void 0?"rc-textarea":E,x=e.showCount,b=e.count,w=e.className,$=e.style,O=e.disabled,T=e.hidden,R=e.classNames,M=e.styles,N=e.onResize,z=e.onClear,P=e.onPressEnter,D=e.readOnly,F=e.autoSize,W=e.onKeyDown,V=wt(e,UH),q=gr(r,{value:o,defaultValue:r}),K=de(q,2),_=K[0],j=K[1],B=_==null?"":String(_),k=oe.useState(!1),I=de(k,2),H=I[0],G=I[1],X=oe.useRef(!1),U=oe.useState(null),J=de(U,2),te=J[0],ve=J[1],ne=u.useRef(null),le=u.useRef(null),be=function(){var Pe;return(Pe=le.current)===null||Pe===void 0?void 0:Pe.textArea},fe=function(){be().focus()};u.useImperativeHandle(t,function(){var Qe;return{resizableTextArea:le.current,focus:fe,blur:function(){be().blur()},nativeElement:((Qe=ne.current)===null||Qe===void 0?void 0:Qe.nativeElement)||be()}}),u.useEffect(function(){G(function(Qe){return!O&&Qe})},[O]);var ie=oe.useState(null),pe=de(ie,2),Re=pe[0],se=pe[1];oe.useEffect(function(){if(Re){var Qe;(Qe=be()).setSelectionRange.apply(Qe,Te(Re))}},[Re]);var ye=F2(b,x),$e=(n=ye.max)!==null&&n!==void 0?n:v,we=Number($e)>0,Ne=ye.strategy(B),Ze=!!$e&&Ne>$e,He=function(Pe,Fe){var _e=Fe;!X.current&&ye.exceedFormatter&&ye.max&&ye.strategy(Fe)>ye.max&&(_e=ye.exceedFormatter(Fe,{max:ye.max}),Fe!==_e&&se([be().selectionStart||0,be().selectionEnd||0])),j(_e),cd(Pe.currentTarget,Pe,d,_e)},Xe=function(Pe){X.current=!0,g==null||g(Pe)},xe=function(Pe){X.current=!1,He(Pe,Pe.currentTarget.value),p==null||p(Pe)},Me=function(Pe){He(Pe,Pe.target.value)},Ve=function(Pe){Pe.key==="Enter"&&P&&P(Pe),W==null||W(Pe)},Ke=function(Pe){G(!0),l==null||l(Pe)},We=function(Pe){G(!1),c==null||c(Pe)},at=function(Pe){j(""),fe(),cd(be(),Pe,d)},Ue=y,Ye;ye.show&&(ye.showFormatter?Ye=ye.showFormatter({value:B,count:Ne,maxLength:$e}):Ye="".concat(Ne).concat(we?" / ".concat($e):""),Ue=oe.createElement(oe.Fragment,null,Ue,oe.createElement("span",{className:re("".concat(C,"-data-count"),R==null?void 0:R.count),style:M==null?void 0:M.count},Ye)));var st=function(Pe){var Fe;N==null||N(Pe),(Fe=be())!==null&&Fe!==void 0&&Fe.style.height&&ve(!0)},ht=!F&&!x&&!m;return oe.createElement(H2,{ref:ne,value:B,allowClear:m,handleReset:at,suffix:Ue,prefixCls:C,classNames:Z(Z({},R),{},{affixWrapper:re(R==null?void 0:R.affixWrapper,Y(Y({},"".concat(C,"-show-count"),x),"".concat(C,"-textarea-allow-clear"),m))}),disabled:O,focused:H,className:re(w,Ze&&"".concat(C,"-out-of-range")),style:Z(Z({},$),te&&!ht?{height:"auto"}:{}),dataAttrs:{affixWrapper:{"data-count":typeof Ye=="string"?Ye:void 0}},hidden:T,readOnly:D,onClear:z},oe.createElement(VH,Ie({},V,{autoSize:F,maxLength:v,onKeyDown:Ve,onChange:Me,onFocus:Ke,onBlur:We,onCompositionStart:Xe,onCompositionEnd:xe,className:re(R==null?void 0:R.textarea),style:Z(Z({},M==null?void 0:M.textarea),{},{resize:$==null?void 0:$.resize}),disabled:O,prefixCls:C,onResize:st,ref:le,readOnly:D})))});const WH=e=>{const{componentCls:t,paddingLG:n}=e,r=`${t}-textarea`;return{[`textarea${t}`]:{maxWidth:"100%",height:"auto",minHeight:e.controlHeight,lineHeight:e.lineHeight,verticalAlign:"bottom",transition:`all ${e.motionDurationSlow}`,resize:"vertical",[`&${t}-mouse-active`]:{transition:`all ${e.motionDurationSlow}, height 0s, width 0s`}},[`${t}-textarea-affix-wrapper-resize-dirty`]:{width:"auto"},[r]:{position:"relative","&-show-count":{[`${t}-data-count`]:{position:"absolute",bottom:e.calc(e.fontSize).mul(e.lineHeight).mul(-1).equal(),insetInlineEnd:0,color:e.colorTextDescription,whiteSpace:"nowrap",pointerEvents:"none"}},[`
        &-allow-clear > ${t},
        &-affix-wrapper${r}-has-feedback ${t}
      `]:{paddingInlineEnd:n},[`&-affix-wrapper${t}-affix-wrapper`]:{padding:0,[`> textarea${t}`]:{fontSize:"inherit",border:"none",outline:"none",background:"transparent",minHeight:e.calc(e.controlHeight).sub(e.calc(e.lineWidth).mul(2)).equal(),"&:focus":{boxShadow:"none !important"}},[`${t}-suffix`]:{margin:0,"> *:not(:last-child)":{marginInline:0},[`${t}-clear-icon`]:{position:"absolute",insetInlineEnd:e.paddingInline,insetBlockStart:e.paddingXS},[`${r}-suffix`]:{position:"absolute",top:0,insetInlineEnd:e.paddingInline,bottom:0,zIndex:1,display:"inline-flex",alignItems:"center",margin:"auto",pointerEvents:"none"}}},[`&-affix-wrapper${t}-affix-wrapper-rtl`]:{[`${t}-suffix`]:{[`${t}-data-count`]:{direction:"ltr",insetInlineStart:0}}},[`&-affix-wrapper${t}-affix-wrapper-sm`]:{[`${t}-suffix`]:{[`${t}-clear-icon`]:{insetInlineEnd:e.paddingInlineSM}}}}}},kH=wn(["Input","TextArea"],e=>{const t=kt(e,Zd(e));return[WH(t)]},Jd,{resetFont:!1});var GH=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const U0=u.forwardRef((e,t)=>{var n;const{prefixCls:r,bordered:o=!0,size:l,disabled:c,status:d,allowClear:m,classNames:v,rootClassName:g,className:p,style:y,styles:E,variant:C,showCount:x,onMouseDown:b,onResize:w}=e,$=GH(e,["prefixCls","bordered","size","disabled","status","allowClear","classNames","rootClassName","className","style","styles","variant","showCount","onMouseDown","onResize"]),{getPrefixCls:O,direction:T,allowClear:R,autoComplete:M,className:N,style:z,classNames:P,styles:D}=zr("textArea"),F=u.useContext(Co),W=c??F,{status:V,hasFeedback:q,feedbackIcon:K}=u.useContext(Jr),_=Gd(V,d),j=u.useRef(null);u.useImperativeHandle(t,()=>{var ye;return{resizableTextArea:(ye=j.current)===null||ye===void 0?void 0:ye.resizableTextArea,focus:$e=>{var we,Ne;L2((Ne=(we=j.current)===null||we===void 0?void 0:we.resizableTextArea)===null||Ne===void 0?void 0:Ne.textArea,$e)},blur:()=>{var $e;return($e=j.current)===null||$e===void 0?void 0:$e.blur()}}});const B=O("input",r),k=na(B),[I,H,G]=P2(B,g),[X]=kH(B,k),{compactSize:U,compactItemClassnames:J}=Xc(B,T),te=Mo(ye=>{var $e;return($e=l??U)!==null&&$e!==void 0?$e:ye}),[ve,ne]=N0("textArea",C,o),le=V2(m??R),[be,fe]=u.useState(!1),[ie,pe]=u.useState(!1),Re=ye=>{fe(!0),b==null||b(ye);const $e=()=>{fe(!1),document.removeEventListener("mouseup",$e)};document.addEventListener("mouseup",$e)},se=ye=>{var $e,we;if(w==null||w(ye),be&&typeof getComputedStyle=="function"){const Ne=(we=($e=j.current)===null||$e===void 0?void 0:$e.nativeElement)===null||we===void 0?void 0:we.querySelector("textarea");Ne&&getComputedStyle(Ne).resize==="both"&&pe(!0)}};return I(X(u.createElement(qH,Object.assign({autoComplete:M},$,{style:Object.assign(Object.assign({},z),y),styles:Object.assign(Object.assign({},D),E),disabled:W,allowClear:le,className:re(G,k,p,g,J,N,ie&&`${B}-textarea-affix-wrapper-resize-dirty`),classNames:Object.assign(Object.assign(Object.assign({},v),P),{textarea:re({[`${B}-sm`]:te==="small",[`${B}-lg`]:te==="large"},H,v==null?void 0:v.textarea,P.textarea,be&&`${B}-mouse-active`),variant:re({[`${B}-${ve}`]:ne},od(B,_)),affixWrapper:re(`${B}-textarea-affix-wrapper`,{[`${B}-affix-wrapper-rtl`]:T==="rtl",[`${B}-affix-wrapper-sm`]:te==="small",[`${B}-affix-wrapper-lg`]:te==="large",[`${B}-textarea-show-count`]:x||((n=e.count)===null||n===void 0?void 0:n.show)},H)}),prefixCls:B,suffix:q&&u.createElement("span",{className:`${B}-textarea-suffix`},K),showCount:x,ref:j,onResize:se,onMouseDown:Re}))))}),Xi=em;Xi.Group=EH;Xi.Search=zH;Xi.TextArea=U0;Xi.Password=jH;Xi.OTP=_H;function KH(e,t,n){return typeof n=="boolean"?n:e.length?!0:Pr(t).some(o=>o.type===$2)}var nR=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function tm(e){let{suffixCls:t,tagName:n,displayName:r}=e;return o=>u.forwardRef((c,d)=>u.createElement(o,Object.assign({ref:d,suffixCls:t,tagName:n},c)))}const q0=u.forwardRef((e,t)=>{const{prefixCls:n,suffixCls:r,className:o,tagName:l}=e,c=nR(e,["prefixCls","suffixCls","className","tagName"]),{getPrefixCls:d}=u.useContext(vt),m=d("layout",n),[v,g,p]=w2(m),y=r?`${m}-${r}`:m;return v(u.createElement(l,Object.assign({className:re(n||y,o,g,p),ref:t},c)))}),XH=u.forwardRef((e,t)=>{const{direction:n}=u.useContext(vt),[r,o]=u.useState([]),{prefixCls:l,className:c,rootClassName:d,children:m,hasSider:v,tagName:g,style:p}=e,y=nR(e,["prefixCls","className","rootClassName","children","hasSider","tagName","style"]),E=Tn(y,["suffixCls"]),{getPrefixCls:C,className:x,style:b}=zr("layout"),w=C("layout",l),$=KH(r,m,v),[O,T,R]=w2(w),M=re(w,{[`${w}-has-sider`]:$,[`${w}-rtl`]:n==="rtl"},x,c,d,T,R),N=u.useMemo(()=>({siderHook:{addSider:z=>{o(P=>[].concat(Te(P),[z]))},removeSider:z=>{o(P=>P.filter(D=>D!==z))}}}),[]);return O(u.createElement(C2.Provider,{value:N},u.createElement(g,Object.assign({ref:t,className:M,style:Object.assign(Object.assign({},b),p)},E),m)))}),YH=tm({tagName:"div",displayName:"Layout"})(XH),rR=tm({suffixCls:"header",tagName:"header",displayName:"Header"})(q0),QH=tm({suffixCls:"footer",tagName:"footer",displayName:"Footer"})(q0),ZH=tm({suffixCls:"content",tagName:"main",displayName:"Content"})(q0),oi=YH;oi.Header=rR;oi.Footer=QH;oi.Content=ZH;oi.Sider=$2;oi._InternalSiderContext=Qd;const JH=function(e){return(arguments.length>1&&arguments[1]!==void 0?arguments[1]:!1)&&e==null?[]:Array.isArray(e)?e:[e]};let pa=null,ji=e=>e(),zc=[],Dc={};function SE(){const{getContainer:e,duration:t,rtl:n,maxCount:r,top:o}=Dc,l=(e==null?void 0:e())||document.body;return{getContainer:()=>l,duration:t,rtl:n,maxCount:r,top:o}}const e7=oe.forwardRef((e,t)=>{const{messageConfig:n,sync:r}=e,{getPrefixCls:o}=u.useContext(vt),l=Dc.prefixCls||o("message"),c=u.useContext(IO),[d,m]=j$(Object.assign(Object.assign(Object.assign({},n),{prefixCls:l}),c.message));return oe.useImperativeHandle(t,()=>{const v=Object.assign({},d);return Object.keys(v).forEach(g=>{v[g]=function(){return r(),d[g].apply(d,arguments)}}),{instance:v,sync:r}}),m}),t7=oe.forwardRef((e,t)=>{const[n,r]=oe.useState(SE),o=()=>{r(SE)};oe.useEffect(o,[]);const l=l0(),c=l.getRootPrefixCls(),d=l.getIconPrefixCls(),m=l.getTheme(),v=oe.createElement(e7,{ref:t,sync:o,messageConfig:n});return oe.createElement(Ya,{prefixCls:c,iconPrefixCls:d,theme:m},l.holderRender?l.holderRender(v):v)});function nm(){if(!pa){const e=document.createDocumentFragment(),t={fragment:e};pa=t,ji(()=>{Id()(oe.createElement(t7,{ref:r=>{const{instance:o,sync:l}=r||{};Promise.resolve().then(()=>{!t.instance&&o&&(t.instance=o,t.sync=l,nm())})}}),e)});return}pa.instance&&(zc.forEach(e=>{const{type:t,skipped:n}=e;if(!n)switch(t){case"open":{ji(()=>{const r=pa.instance.open(Object.assign(Object.assign({},Dc),e.config));r==null||r.then(e.resolve),e.setCloseFn(r)});break}case"destroy":ji(()=>{pa==null||pa.instance.destroy(e.key)});break;default:ji(()=>{var r;const o=(r=pa.instance)[t].apply(r,Te(e.args));o==null||o.then(e.resolve),e.setCloseFn(o)})}}),zc=[])}function n7(e){Dc=Object.assign(Object.assign({},Dc),e),ji(()=>{var t;(t=pa==null?void 0:pa.sync)===null||t===void 0||t.call(pa)})}function r7(e){const t=f0(n=>{let r;const o={type:"open",config:e,resolve:n,setCloseFn:l=>{r=l}};return zc.push(o),()=>{r?ji(()=>{r()}):o.skipped=!0}});return nm(),t}function a7(e,t){const n=f0(r=>{let o;const l={type:e,args:t,resolve:r,setCloseFn:c=>{o=c}};return zc.push(l),()=>{o?ji(()=>{o()}):l.skipped=!0}});return nm(),n}const o7=e=>{zc.push({type:"destroy",key:e}),nm()},i7=["success","info","warning","error","loading"],l7={open:r7,destroy:o7,config:n7,useMessage:H4,_InternalPanelDoNotUseOrYouWillBeFired:A4},aR=l7;i7.forEach(e=>{aR[e]=function(){for(var t=arguments.length,n=new Array(t),r=0;r<t;r++)n[r]=arguments[r];return a7(e,n)}});var s7=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const c7=e=>{const{prefixCls:t,className:n,closeIcon:r,closable:o,type:l,title:c,children:d,footer:m}=e,v=s7(e,["prefixCls","className","closeIcon","closable","type","title","children","footer"]),{getPrefixCls:g}=u.useContext(vt),p=g(),y=t||g("modal"),E=na(p),[C,x,b]=hO(y,E),w=`${y}-confirm`;let $={};return l?$={closable:o??!1,title:"",footer:"",children:u.createElement(yO,Object.assign({},e,{prefixCls:y,confirmPrefixCls:w,rootPrefixCls:p,content:d}))}:$={closable:o??!0,title:c,footer:m!==null&&u.createElement(dO,Object.assign({},e)),children:d},C(u.createElement(Q$,Object.assign({prefixCls:y,className:re(x,`${y}-pure-panel`,l&&w,l&&`${w}-${l}`,n,b,E)},v,{closeIcon:fO(y,r),closable:o},$)))},u7=jO(c7);function oR(e){return Zc(xO(e))}const ja=pO;ja.useModal=X6;ja.info=function(t){return Zc(EO(t))};ja.success=function(t){return Zc(wO(t))};ja.error=function(t){return Zc($O(t))};ja.warning=oR;ja.warn=oR;ja.confirm=function(t){return Zc(OO(t))};ja.destroyAll=function(){for(;Ii.length;){const t=Ii.pop();t&&t()}};ja.config=q6;ja._InternalPanelDoNotUseOrYouWillBeFired=u7;let Oa=null,Lf=e=>e(),fd=[],Bc={};function CE(){const{getContainer:e,rtl:t,maxCount:n,top:r,bottom:o,showProgress:l,pauseOnHover:c}=Bc,d=(e==null?void 0:e())||document.body;return{getContainer:()=>d,rtl:t,maxCount:n,top:r,bottom:o,showProgress:l,pauseOnHover:c}}const f7=oe.forwardRef((e,t)=>{const{notificationConfig:n,sync:r}=e,{getPrefixCls:o}=u.useContext(vt),l=Bc.prefixCls||o("notification"),c=u.useContext(IO),[d,m]=AO(Object.assign(Object.assign(Object.assign({},n),{prefixCls:l}),c.notification));return oe.useEffect(r,[]),oe.useImperativeHandle(t,()=>{const v=Object.assign({},d);return Object.keys(v).forEach(g=>{v[g]=function(){return r(),d[g].apply(d,arguments)}}),{instance:v,sync:r}}),m}),d7=oe.forwardRef((e,t)=>{const[n,r]=oe.useState(CE),o=()=>{r(CE)};oe.useEffect(o,[]);const l=l0(),c=l.getRootPrefixCls(),d=l.getIconPrefixCls(),m=l.getTheme(),v=oe.createElement(f7,{ref:t,sync:o,notificationConfig:n});return oe.createElement(Ya,{prefixCls:c,iconPrefixCls:d,theme:m},l.holderRender?l.holderRender(v):v)});function W0(){if(!Oa){const e=document.createDocumentFragment(),t={fragment:e};Oa=t,Lf(()=>{Id()(oe.createElement(d7,{ref:r=>{const{instance:o,sync:l}=r||{};Promise.resolve().then(()=>{!t.instance&&o&&(t.instance=o,t.sync=l,W0())})}}),e)});return}Oa.instance&&(fd.forEach(e=>{switch(e.type){case"open":{Lf(()=>{Oa.instance.open(Object.assign(Object.assign({},Bc),e.config))});break}case"destroy":Lf(()=>{Oa==null||Oa.instance.destroy(e.key)});break}}),fd=[])}function m7(e){Bc=Object.assign(Object.assign({},Bc),e),Lf(()=>{var t;(t=Oa==null?void 0:Oa.sync)===null||t===void 0||t.call(Oa)})}function iR(e){fd.push({type:"open",config:e}),W0()}const v7=e=>{fd.push({type:"destroy",key:e}),W0()},g7=["success","info","warning","error"],h7={open:iR,destroy:v7,config:m7,useNotification:pD,_InternalPanelDoNotUseOrYouWillBeFired:lD},lR=h7;g7.forEach(e=>{lR[e]=t=>iR(Object.assign(Object.assign({},t),{type:e}))});const p7=e=>{const{componentCls:t,iconCls:n,antCls:r,zIndexPopup:o,colorText:l,colorWarning:c,marginXXS:d,marginXS:m,fontSize:v,fontWeightStrong:g,colorTextHeading:p}=e;return{[t]:{zIndex:o,[`&${r}-popover`]:{fontSize:v},[`${t}-message`]:{marginBottom:m,display:"flex",flexWrap:"nowrap",alignItems:"start",[`> ${t}-message-icon ${n}`]:{color:c,fontSize:v,lineHeight:1,marginInlineEnd:m},[`${t}-title`]:{fontWeight:g,color:p,"&:only-child":{fontWeight:"normal"}},[`${t}-description`]:{marginTop:d,color:l}},[`${t}-buttons`]:{textAlign:"end",whiteSpace:"nowrap",button:{marginInlineStart:m}}}}},y7=e=>{const{zIndexPopupBase:t}=e;return{zIndexPopup:t+60}},sR=wn("Popconfirm",e=>p7(e),y7,{resetStyle:!1});var b7=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const cR=e=>{const{prefixCls:t,okButtonProps:n,cancelButtonProps:r,title:o,description:l,cancelText:c,okText:d,okType:m="primary",icon:v=u.createElement(Ql,null),showCancel:g=!0,close:p,onConfirm:y,onCancel:E,onPopupClick:C}=e,{getPrefixCls:x}=u.useContext(vt),[b]=To("Popconfirm",So.Popconfirm),w=kl(o),$=kl(l);return u.createElement("div",{className:`${t}-inner-content`,onClick:C},u.createElement("div",{className:`${t}-message`},v&&u.createElement("span",{className:`${t}-message-icon`},v),u.createElement("div",{className:`${t}-message-text`},w&&u.createElement("div",{className:`${t}-title`},w),$&&u.createElement("div",{className:`${t}-description`},$))),u.createElement("div",{className:`${t}-buttons`},g&&u.createElement($r,Object.assign({onClick:E,size:"small"},r),c||(b==null?void 0:b.cancelText)),u.createElement(y0,{buttonProps:Object.assign(Object.assign({size:"small"},d0(m)),n),actionFn:y,close:p,prefixCls:x("btn"),quitOnNullishReturnValue:!0,emitEvent:!0},d||(b==null?void 0:b.okText))))},S7=e=>{const{prefixCls:t,placement:n,className:r,style:o}=e,l=b7(e,["prefixCls","placement","className","style"]),{getPrefixCls:c}=u.useContext(vt),d=c("popconfirm",t),[m]=sR(d);return m(u.createElement(s2,{placement:n,className:re(d,r),style:o,content:u.createElement(cR,Object.assign({prefixCls:d},l))}))};var C7=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const x7=u.forwardRef((e,t)=>{var n,r;const{prefixCls:o,placement:l="top",trigger:c="click",okType:d="primary",icon:m=u.createElement(Ql,null),children:v,overlayClassName:g,onOpenChange:p,onVisibleChange:y,overlayStyle:E,styles:C,classNames:x}=e,b=C7(e,["prefixCls","placement","trigger","okType","icon","children","overlayClassName","onOpenChange","onVisibleChange","overlayStyle","styles","classNames"]),{getPrefixCls:w,className:$,style:O,classNames:T,styles:R}=zr("popconfirm"),[M,N]=gr(!1,{value:(n=e.open)!==null&&n!==void 0?n:e.visible,defaultValue:(r=e.defaultOpen)!==null&&r!==void 0?r:e.defaultVisible}),z=(j,B)=>{N(j,!0),y==null||y(j),p==null||p(j,B)},P=j=>{z(!1,j)},D=j=>{var B;return(B=e.onConfirm)===null||B===void 0?void 0:B.call(void 0,j)},F=j=>{var B;z(!1,j),(B=e.onCancel)===null||B===void 0||B.call(void 0,j)},W=(j,B)=>{const{disabled:k=!1}=e;k||z(j,B)},V=w("popconfirm",o),q=re(V,$,g,T.root,x==null?void 0:x.root),K=re(T.body,x==null?void 0:x.body),[_]=sR(V);return _(u.createElement(c2,Object.assign({},Tn(b,["title"]),{trigger:c,placement:l,onOpenChange:W,open:M,ref:t,classNames:{root:q,body:K},styles:{root:Object.assign(Object.assign(Object.assign(Object.assign({},R.root),O),E),C==null?void 0:C.root),body:Object.assign(Object.assign({},R.body),C==null?void 0:C.body)},content:u.createElement(cR,Object.assign({okType:d,icon:m},e,{prefixCls:V,close:P,onConfirm:D,onCancel:F})),"data-popover-inject":!0}),v))}),uR=x7;uR._InternalPanelDoNotUseOrYouWillBeFired=S7;const E7=e=>{const t=e!=null&&e.algorithm?Yf(e.algorithm):n0,n=Object.assign(Object.assign({},Wl),e==null?void 0:e.token);return Dw(n,{override:e==null?void 0:e.token},t,o0)};function w7(e){const{sizeUnit:t,sizeStep:n}=e,r=n-2;return{sizeXXL:t*(r+10),sizeXL:t*(r+6),sizeLG:t*(r+2),sizeMD:t*(r+2),sizeMS:t*(r+1),size:t*r,sizeSM:t*r,sizeXS:t*(r-1),sizeXXS:t*(r-1)}}const $7=(e,t)=>{const n=t??Td(e),r=n.fontSizeSM,o=n.controlHeight-4;return Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({},n),w7(t??e)),a$(r)),{controlHeight:o}),r$(Object.assign(Object.assign({},n),{controlHeight:o})))},ha=(e,t)=>new Jt(e).setA(t).toRgbString(),jl=(e,t)=>new Jt(e).lighten(t).toHexString(),O7=e=>{const t=Bi(e,{theme:"dark"});return{1:t[0],2:t[1],3:t[2],4:t[3],5:t[6],6:t[5],7:t[4],8:t[6],9:t[5],10:t[4]}},R7=(e,t)=>{const n=e||"#000",r=t||"#fff";return{colorBgBase:n,colorTextBase:r,colorText:ha(r,.85),colorTextSecondary:ha(r,.65),colorTextTertiary:ha(r,.45),colorTextQuaternary:ha(r,.25),colorFill:ha(r,.18),colorFillSecondary:ha(r,.12),colorFillTertiary:ha(r,.08),colorFillQuaternary:ha(r,.04),colorBgSolid:ha(r,.95),colorBgSolidHover:ha(r,1),colorBgSolidActive:ha(r,.9),colorBgElevated:jl(n,12),colorBgContainer:jl(n,8),colorBgLayout:jl(n,0),colorBgSpotlight:jl(n,26),colorBgBlur:ha(r,.04),colorBorder:jl(n,26),colorBorderSecondary:jl(n,19)}},T7=(e,t)=>{const n=Object.keys(t0).map(o=>{const l=Bi(e[o],{theme:"dark"});return Array.from({length:10},()=>1).reduce((c,d,m)=>(c[`${o}-${m+1}`]=l[m],c[`${o}${m+1}`]=l[m],c),{})}).reduce((o,l)=>(o=Object.assign(Object.assign({},o),l),o),{}),r=t??Td(e);return Object.assign(Object.assign(Object.assign({},r),n),n$(e,{generateColorPalettes:O7,generateNeutralColorPalettes:R7}))};function M7(){const[e,t,n]=Rr();return{theme:e,token:t,hashId:n}}const _7={defaultSeed:Tc.token,useToken:M7,defaultAlgorithm:Td,darkAlgorithm:T7,compactAlgorithm:$7,getDesignToken:E7,defaultConfig:Tc,_internalContext:r0},N7=(e,t,n,r)=>{const{titleMarginBottom:o,fontWeightStrong:l}=r;return{marginBottom:o,color:n,fontWeight:l,fontSize:e,lineHeight:t}},A7=e=>{const t=[1,2,3,4,5],n={};return t.forEach(r=>{n[`
      h${r}&,
      div&-h${r},
      div&-h${r} > textarea,
      h${r}
    `]=N7(e[`fontSizeHeading${r}`],e[`lineHeightHeading${r}`],e.colorTextHeading,e)}),n},I7=e=>{const{componentCls:t}=e;return{"a&, a":Object.assign(Object.assign({},d$(e)),{userSelect:"text",[`&[disabled], &${t}-disabled`]:{color:e.colorTextDisabled,cursor:"not-allowed","&:active, &:hover":{color:e.colorTextDisabled},"&:active":{pointerEvents:"none"}}})}},j7=e=>({code:{margin:"0 0.2em",paddingInline:"0.4em",paddingBlock:"0.2em 0.1em",fontSize:"85%",fontFamily:e.fontFamilyCode,background:"rgba(150, 150, 150, 0.1)",border:"1px solid rgba(100, 100, 100, 0.2)",borderRadius:3},kbd:{margin:"0 0.2em",paddingInline:"0.4em",paddingBlock:"0.15em 0.1em",fontSize:"90%",fontFamily:e.fontFamilyCode,background:"rgba(150, 150, 150, 0.06)",border:"1px solid rgba(100, 100, 100, 0.2)",borderBottomWidth:2,borderRadius:3},mark:{padding:0,backgroundColor:Wf[2]},"u, ins":{textDecoration:"underline",textDecorationSkipInk:"auto"},"s, del":{textDecoration:"line-through"},strong:{fontWeight:600},"ul, ol":{marginInline:0,marginBlock:"0 1em",padding:0,li:{marginInline:"20px 0",marginBlock:0,paddingInline:"4px 0",paddingBlock:0}},ul:{listStyleType:"circle",ul:{listStyleType:"disc"}},ol:{listStyleType:"decimal"},"pre, blockquote":{margin:"1em 0"},pre:{padding:"0.4em 0.6em",whiteSpace:"pre-wrap",wordWrap:"break-word",background:"rgba(150, 150, 150, 0.1)",border:"1px solid rgba(100, 100, 100, 0.2)",borderRadius:3,fontFamily:e.fontFamilyCode,code:{display:"inline",margin:0,padding:0,fontSize:"inherit",fontFamily:"inherit",background:"transparent",border:0}},blockquote:{paddingInline:"0.6em 0",paddingBlock:0,borderInlineStart:"4px solid rgba(100, 100, 100, 0.2)",opacity:.85}}),P7=e=>{const{componentCls:t,paddingSM:n}=e,r=n;return{"&-edit-content":{position:"relative","div&":{insetInlineStart:e.calc(e.paddingSM).mul(-1).equal(),marginTop:e.calc(r).mul(-1).equal(),marginBottom:`calc(1em - ${he(r)})`},[`${t}-edit-content-confirm`]:{position:"absolute",insetInlineEnd:e.calc(e.marginXS).add(2).equal(),insetBlockEnd:e.marginXS,color:e.colorIcon,fontWeight:"normal",fontSize:e.fontSize,fontStyle:"normal",pointerEvents:"none"},textarea:{margin:"0!important",MozTransition:"none",height:"1em"}}}},z7=e=>({[`${e.componentCls}-copy-success`]:{"\n    &,\n    &:hover,\n    &:focus":{color:e.colorSuccess}},[`${e.componentCls}-copy-icon-only`]:{marginInlineStart:0}}),D7=()=>({"\n  a&-ellipsis,\n  span&-ellipsis\n  ":{display:"inline-block",maxWidth:"100%"},"&-ellipsis-single-line":{whiteSpace:"nowrap",overflow:"hidden",textOverflow:"ellipsis","a&, span&":{verticalAlign:"bottom"},"> code":{paddingBlock:0,maxWidth:"calc(100% - 1.2em)",display:"inline-block",overflow:"hidden",textOverflow:"ellipsis",verticalAlign:"bottom",boxSizing:"content-box"}},"&-ellipsis-multiple-line":{display:"-webkit-box",overflow:"hidden",WebkitLineClamp:3,WebkitBoxOrient:"vertical"}}),B7=e=>{const{componentCls:t,titleMarginTop:n}=e;return{[t]:Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign(Object.assign({color:e.colorText,wordBreak:"break-word",lineHeight:e.lineHeight,[`&${t}-secondary`]:{color:e.colorTextDescription},[`&${t}-success`]:{color:e.colorSuccessText},[`&${t}-warning`]:{color:e.colorWarningText},[`&${t}-danger`]:{color:e.colorErrorText,"a&:active, a&:focus":{color:e.colorErrorTextActive},"a&:hover":{color:e.colorErrorTextHover}},[`&${t}-disabled`]:{color:e.colorTextDisabled,cursor:"not-allowed",userSelect:"none"},"\n        div&,\n        p\n      ":{marginBottom:"1em"}},A7(e)),{[`
      & + h1${t},
      & + h2${t},
      & + h3${t},
      & + h4${t},
      & + h5${t}
      `]:{marginTop:n},"\n      div,\n      ul,\n      li,\n      p,\n      h1,\n      h2,\n      h3,\n      h4,\n      h5":{"\n        + h1,\n        + h2,\n        + h3,\n        + h4,\n        + h5\n        ":{marginTop:n}}}),j7(e)),I7(e)),{[`
        ${t}-expand,
        ${t}-collapse,
        ${t}-edit,
        ${t}-copy
      `]:Object.assign(Object.assign({},d$(e)),{marginInlineStart:e.marginXXS})}),P7(e)),z7(e)),D7()),{"&-rtl":{direction:"rtl"}})}},L7=()=>({titleMarginTop:"1.2em",titleMarginBottom:"0.5em"}),fR=wn("Typography",e=>[B7(e)],L7),H7=e=>{const{prefixCls:t,"aria-label":n,className:r,style:o,direction:l,maxLength:c,autoSize:d=!0,value:m,onSave:v,onCancel:g,onEnd:p,component:y,enterIcon:E=u.createElement(TN,null)}=e,C=u.useRef(null),x=u.useRef(!1),b=u.useRef(null),[w,$]=u.useState(m);u.useEffect(()=>{$(m)},[m]),u.useEffect(()=>{var q;if(!((q=C.current)===null||q===void 0)&&q.resizableTextArea){const{textArea:K}=C.current.resizableTextArea;K.focus();const{length:_}=K.value;K.setSelectionRange(_,_)}},[]);const O=q=>{let{target:K}=q;$(K.value.replace(/[\n\r]/g,""))},T=()=>{x.current=!0},R=()=>{x.current=!1},M=q=>{let{keyCode:K}=q;x.current||(b.current=K)},N=()=>{v(w.trim())},z=q=>{let{keyCode:K,ctrlKey:_,altKey:j,metaKey:B,shiftKey:k}=q;b.current!==K||x.current||_||j||B||k||(K===ct.ENTER?(N(),p==null||p()):K===ct.ESC&&g())},P=()=>{N()},[D,F,W]=fR(t),V=re(t,`${t}-edit-content`,{[`${t}-rtl`]:l==="rtl",[`${t}-${y}`]:!!y},r,F,W);return D(u.createElement("div",{className:V,style:o},u.createElement(U0,{ref:C,maxLength:c,value:w,onChange:O,onKeyDown:M,onKeyUp:z,onCompositionStart:T,onCompositionEnd:R,onBlur:P,"aria-label":n,rows:1,autoSize:d}),E!==null?Zr(E,{className:`${t}-edit-content-confirm`}):null))};var oh,xE;function F7(){return xE||(xE=1,oh=function(){var e=document.getSelection();if(!e.rangeCount)return function(){};for(var t=document.activeElement,n=[],r=0;r<e.rangeCount;r++)n.push(e.getRangeAt(r));switch(t.tagName.toUpperCase()){case"INPUT":case"TEXTAREA":t.blur();break;default:t=null;break}return e.removeAllRanges(),function(){e.type==="Caret"&&e.removeAllRanges(),e.rangeCount||n.forEach(function(o){e.addRange(o)}),t&&t.focus()}}),oh}var ih,EE;function V7(){if(EE)return ih;EE=1;var e=F7(),t={"text/plain":"Text","text/html":"Url",default:"Text"},n="Copy to clipboard: #{key}, Enter";function r(l){var c=(/mac os x/i.test(navigator.userAgent)?"⌘":"Ctrl")+"+C";return l.replace(/#{\s*key\s*}/g,c)}function o(l,c){var d,m,v,g,p,y,E=!1;c||(c={}),d=c.debug||!1;try{v=e(),g=document.createRange(),p=document.getSelection(),y=document.createElement("span"),y.textContent=l,y.ariaHidden="true",y.style.all="unset",y.style.position="fixed",y.style.top=0,y.style.clip="rect(0, 0, 0, 0)",y.style.whiteSpace="pre",y.style.webkitUserSelect="text",y.style.MozUserSelect="text",y.style.msUserSelect="text",y.style.userSelect="text",y.addEventListener("copy",function(x){if(x.stopPropagation(),c.format)if(x.preventDefault(),typeof x.clipboardData>"u"){d&&console.warn("unable to use e.clipboardData"),d&&console.warn("trying IE specific stuff"),window.clipboardData.clearData();var b=t[c.format]||t.default;window.clipboardData.setData(b,l)}else x.clipboardData.clearData(),x.clipboardData.setData(c.format,l);c.onCopy&&(x.preventDefault(),c.onCopy(x.clipboardData))}),document.body.appendChild(y),g.selectNodeContents(y),p.addRange(g);var C=document.execCommand("copy");if(!C)throw new Error("copy command was unsuccessful");E=!0}catch(x){d&&console.error("unable to copy using execCommand: ",x),d&&console.warn("trying IE specific stuff");try{window.clipboardData.setData(c.format||"text",l),c.onCopy&&c.onCopy(window.clipboardData),E=!0}catch(b){d&&console.error("unable to copy using clipboardData: ",b),d&&console.error("falling back to prompt"),m=r("message"in c?c.message:n),window.prompt(m,l)}}finally{p&&(typeof p.removeRange=="function"?p.removeRange(g):p.removeAllRanges()),y&&document.body.removeChild(y),v()}return E}return ih=o,ih}var U7=V7();const q7=Sd(U7);var W7=function(e,t,n,r){function o(l){return l instanceof n?l:new n(function(c){c(l)})}return new(n||(n=Promise))(function(l,c){function d(g){try{v(r.next(g))}catch(p){c(p)}}function m(g){try{v(r.throw(g))}catch(p){c(p)}}function v(g){g.done?l(g.value):o(g.value).then(d,m)}v((r=r.apply(e,t||[])).next())})};const k7=e=>{let{copyConfig:t,children:n}=e;const[r,o]=u.useState(!1),[l,c]=u.useState(!1),d=u.useRef(null),m=()=>{d.current&&clearTimeout(d.current)},v={};t.format&&(v.format=t.format),u.useEffect(()=>m,[]);const g=jn(p=>W7(void 0,void 0,void 0,function*(){var y;p==null||p.preventDefault(),p==null||p.stopPropagation(),c(!0);try{const E=typeof t.text=="function"?yield t.text():t.text;q7(E||JH(n,!0).join("")||"",v),c(!1),o(!0),m(),d.current=setTimeout(()=>{o(!1)},3e3),(y=t.onCopy)===null||y===void 0||y.call(t,p)}catch(E){throw c(!1),E}}));return{copied:r,copyLoading:l,onClick:g}};function lh(e,t){return u.useMemo(()=>{const n=!!e;return[n,Object.assign(Object.assign({},t),n&&typeof e=="object"?e:null)]},[e])}const G7=e=>{const t=u.useRef(void 0);return u.useEffect(()=>{t.current=e}),t.current},K7=(e,t,n)=>u.useMemo(()=>e===!0?{title:t??n}:u.isValidElement(e)?{title:e}:typeof e=="object"?Object.assign({title:t??n},e):{title:e},[e,t,n]);var X7=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const dR=u.forwardRef((e,t)=>{const{prefixCls:n,component:r="article",className:o,rootClassName:l,setContentRef:c,children:d,direction:m,style:v}=e,g=X7(e,["prefixCls","component","className","rootClassName","setContentRef","children","direction","style"]),{getPrefixCls:p,direction:y,className:E,style:C}=zr("typography"),x=m??y,b=c?Ia(t,c):t,w=p("typography",n),[$,O,T]=fR(w),R=re(w,E,{[`${w}-rtl`]:x==="rtl"},o,l,O,T),M=Object.assign(Object.assign({},C),v);return $(u.createElement(r,Object.assign({className:R,style:M,ref:b},g),d))});function wE(e){return e===!1?[!1,!1]:Array.isArray(e)?e:[e]}function sh(e,t,n){return e===!0||e===void 0?t:e||n&&t}function Y7(e){const t=document.createElement("em");e.appendChild(t);const n=e.getBoundingClientRect(),r=t.getBoundingClientRect();return e.removeChild(t),n.left>r.left||r.right>n.right||n.top>r.top||r.bottom>n.bottom}const k0=e=>["string","number"].includes(typeof e),Q7=e=>{let{prefixCls:t,copied:n,locale:r,iconOnly:o,tooltips:l,icon:c,tabIndex:d,onCopy:m,loading:v}=e;const g=wE(l),p=wE(c),{copied:y,copy:E}=r??{},C=n?y:E,x=sh(g[n?1:0],C),b=typeof x=="string"?x:C;return u.createElement(si,{title:x},u.createElement("button",{type:"button",className:re(`${t}-copy`,{[`${t}-copy-success`]:n,[`${t}-copy-icon-only`]:o}),onClick:m,"aria-label":b,tabIndex:d},n?sh(p[1],u.createElement(Cw,null),!0):sh(p[0],v?u.createElement(Uc,null):u.createElement(hN,null),!0)))},Cf=u.forwardRef((e,t)=>{let{style:n,children:r}=e;const o=u.useRef(null);return u.useImperativeHandle(t,()=>({isExceed:()=>{const l=o.current;return l.scrollHeight>l.clientHeight},getHeight:()=>o.current.clientHeight})),u.createElement("span",{"aria-hidden":!0,ref:o,style:Object.assign({position:"fixed",display:"block",left:0,top:0,pointerEvents:"none",backgroundColor:"rgba(255, 0, 0, 0.65)"},n)},r)}),Z7=e=>e.reduce((t,n)=>t+(k0(n)?String(n).length:1),0);function $E(e,t){let n=0;const r=[];for(let o=0;o<e.length;o+=1){if(n===t)return r;const l=e[o],d=k0(l)?String(l).length:1,m=n+d;if(m>t){const v=t-n;return r.push(String(l).slice(0,v)),r}r.push(l),n=m}return e}const ch=0,uh=1,fh=2,dh=3,OE=4,xf={display:"-webkit-box",overflow:"hidden",WebkitBoxOrient:"vertical"};function J7(e){const{enableMeasure:t,width:n,text:r,children:o,rows:l,expanded:c,miscDeps:d,onEllipsis:m}=e,v=u.useMemo(()=>Pr(r),[r]),g=u.useMemo(()=>Z7(v),[r]),p=u.useMemo(()=>o(v,!1),[r]),[y,E]=u.useState(null),C=u.useRef(null),x=u.useRef(null),b=u.useRef(null),w=u.useRef(null),$=u.useRef(null),[O,T]=u.useState(!1),[R,M]=u.useState(ch),[N,z]=u.useState(0),[P,D]=u.useState(null);Wt(()=>{M(t&&n&&g?uh:ch)},[n,r,l,t,v]),Wt(()=>{var q,K,_,j;if(R===uh){M(fh);const B=x.current&&getComputedStyle(x.current).whiteSpace;D(B)}else if(R===fh){const B=!!(!((q=b.current)===null||q===void 0)&&q.isExceed());M(B?dh:OE),E(B?[0,g]:null),T(B);const k=((K=b.current)===null||K===void 0?void 0:K.getHeight())||0,I=l===1?0:((_=w.current)===null||_===void 0?void 0:_.getHeight())||0,H=((j=$.current)===null||j===void 0?void 0:j.getHeight())||0,G=Math.max(k,I+H);z(G+1),m(B)}},[R]);const F=y?Math.ceil((y[0]+y[1])/2):0;Wt(()=>{var q;const[K,_]=y||[0,0];if(K!==_){const B=(((q=C.current)===null||q===void 0?void 0:q.getHeight())||0)>N;let k=F;_-K===1&&(k=B?K:_),E(B?[K,k]:[k,_])}},[y,F]);const W=u.useMemo(()=>{if(!t)return o(v,!1);if(R!==dh||!y||y[0]!==y[1]){const q=o(v,!1);return[OE,ch].includes(R)?q:u.createElement("span",{style:Object.assign(Object.assign({},xf),{WebkitLineClamp:l})},q)}return o(c?v:$E(v,y[0]),O)},[c,R,y,v].concat(Te(d))),V={width:n,margin:0,padding:0,whiteSpace:P==="nowrap"?"normal":"inherit"};return u.createElement(u.Fragment,null,W,R===fh&&u.createElement(u.Fragment,null,u.createElement(Cf,{style:Object.assign(Object.assign(Object.assign({},V),xf),{WebkitLineClamp:l}),ref:b},p),u.createElement(Cf,{style:Object.assign(Object.assign(Object.assign({},V),xf),{WebkitLineClamp:l-1}),ref:w},p),u.createElement(Cf,{style:Object.assign(Object.assign(Object.assign({},V),xf),{WebkitLineClamp:1}),ref:$},o([],!0))),R===dh&&y&&y[0]!==y[1]&&u.createElement(Cf,{style:Object.assign(Object.assign({},V),{top:400}),ref:C},o($E(v,F),!0)),R===uh&&u.createElement("span",{style:{whiteSpace:"inherit"},ref:x}))}const eF=e=>{let{enableEllipsis:t,isEllipsis:n,children:r,tooltipProps:o}=e;return!(o!=null&&o.title)||!t?r:u.createElement(si,Object.assign({open:n?void 0:!1},o),r)};var tF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};function nF(e,t){let{mark:n,code:r,underline:o,delete:l,strong:c,keyboard:d,italic:m}=e,v=t;function g(p,y){y&&(v=u.createElement(p,{},v))}return g("strong",c),g("u",o),g("del",l),g("code",r),g("mark",n),g("kbd",d),g("i",m),v}const rF="...",rm=u.forwardRef((e,t)=>{var n;const{prefixCls:r,className:o,style:l,type:c,disabled:d,children:m,ellipsis:v,editable:g,copyable:p,component:y,title:E}=e,C=tF(e,["prefixCls","className","style","type","disabled","children","ellipsis","editable","copyable","component","title"]),{getPrefixCls:x,direction:b}=u.useContext(vt),[w]=To("Text"),$=u.useRef(null),O=u.useRef(null),T=x("typography",r),R=Tn(C,["mark","code","delete","underline","strong","keyboard","italic"]),[M,N]=lh(g),[z,P]=gr(!1,{value:N.editing}),{triggerType:D=["icon"]}=N,F=_e=>{var qe;_e&&((qe=N.onStart)===null||qe===void 0||qe.call(N)),P(_e)},W=G7(z);Wt(()=>{var _e;!z&&W&&((_e=O.current)===null||_e===void 0||_e.focus())},[z]);const V=_e=>{_e==null||_e.preventDefault(),F(!0)},q=_e=>{var qe;(qe=N.onChange)===null||qe===void 0||qe.call(N,_e),F(!1)},K=()=>{var _e;(_e=N.onCancel)===null||_e===void 0||_e.call(N),F(!1)},[_,j]=lh(p),{copied:B,copyLoading:k,onClick:I}=k7({copyConfig:j,children:m}),[H,G]=u.useState(!1),[X,U]=u.useState(!1),[J,te]=u.useState(!1),[ve,ne]=u.useState(!1),[le,be]=u.useState(!0),[fe,ie]=lh(v,{expandable:!1,symbol:_e=>_e?w==null?void 0:w.collapse:w==null?void 0:w.expand}),[pe,Re]=gr(ie.defaultExpanded||!1,{value:ie.expanded}),se=fe&&(!pe||ie.expandable==="collapsible"),{rows:ye=1}=ie,$e=u.useMemo(()=>se&&(ie.suffix!==void 0||ie.onEllipsis||ie.expandable||M||_),[se,ie,M,_]);Wt(()=>{fe&&!$e&&(G(vx("webkitLineClamp")),U(vx("textOverflow")))},[$e,fe]);const[we,Ne]=u.useState(se),Ze=u.useMemo(()=>$e?!1:ye===1?X:H,[$e,X,H]);Wt(()=>{Ne(Ze&&se)},[Ze,se]);const He=se&&(we?ve:J),Xe=se&&ye===1&&we,xe=se&&ye>1&&we,Me=(_e,qe)=>{var Ct;Re(qe.expanded),(Ct=ie.onExpand)===null||Ct===void 0||Ct.call(ie,_e,qe)},[Ve,Ke]=u.useState(0),We=_e=>{let{offsetWidth:qe}=_e;Ke(qe)},at=_e=>{var qe;te(_e),J!==_e&&((qe=ie.onEllipsis)===null||qe===void 0||qe.call(ie,_e))};u.useEffect(()=>{const _e=$.current;if(fe&&we&&_e){const qe=Y7(_e);ve!==qe&&ne(qe)}},[fe,we,m,xe,le,Ve]),u.useEffect(()=>{const _e=$.current;if(typeof IntersectionObserver>"u"||!_e||!we||!se)return;const qe=new IntersectionObserver(()=>{be(!!_e.offsetParent)});return qe.observe(_e),()=>{qe.disconnect()}},[we,se]);const Ue=K7(ie.tooltip,N.text,m),Ye=u.useMemo(()=>{if(!(!fe||we))return[N.text,m,E,Ue.title].find(k0)},[fe,we,E,Ue.title,He]);if(z)return u.createElement(H7,{value:(n=N.text)!==null&&n!==void 0?n:typeof m=="string"?m:"",onSave:q,onCancel:K,onEnd:N.onEnd,prefixCls:T,className:o,style:l,direction:b,component:y,maxLength:N.maxLength,autoSize:N.autoSize,enterIcon:N.enterIcon});const st=()=>{const{expandable:_e,symbol:qe}=ie;return _e?u.createElement("button",{type:"button",key:"expand",className:`${T}-${pe?"collapse":"expand"}`,onClick:Ct=>Me(Ct,{expanded:!pe}),"aria-label":pe?w.collapse:w==null?void 0:w.expand},typeof qe=="function"?qe(pe):qe):null},ht=()=>{if(!M)return;const{icon:_e,tooltip:qe,tabIndex:Ct}=N,Nt=Pr(qe)[0]||(w==null?void 0:w.edit),_n=typeof Nt=="string"?Nt:"";return D.includes("icon")?u.createElement(si,{key:"edit",title:qe===!1?"":Nt},u.createElement("button",{type:"button",ref:O,className:`${T}-edit`,onClick:V,"aria-label":_n,tabIndex:Ct},_e||u.createElement(xN,{role:"button"}))):null},Qe=()=>_?u.createElement(Q7,Object.assign({key:"copy"},j,{prefixCls:T,copied:B,locale:w,onCopy:I,loading:k,iconOnly:m==null})):null,Pe=_e=>[_e&&st(),ht(),Qe()],Fe=_e=>[_e&&!pe&&u.createElement("span",{"aria-hidden":!0,key:"ellipsis"},rF),ie.suffix,Pe(_e)];return u.createElement(Ro,{onResize:We,disabled:!se},_e=>u.createElement(eF,{tooltipProps:Ue,enableEllipsis:se,isEllipsis:He},u.createElement(dR,Object.assign({className:re({[`${T}-${c}`]:c,[`${T}-disabled`]:d,[`${T}-ellipsis`]:fe,[`${T}-ellipsis-single-line`]:Xe,[`${T}-ellipsis-multiple-line`]:xe},o),prefixCls:r,style:Object.assign(Object.assign({},l),{WebkitLineClamp:xe?ye:void 0}),component:y,ref:Ia(_e,$,t),direction:b,onClick:D.includes("text")?V:void 0,"aria-label":Ye==null?void 0:Ye.toString(),title:E},R),u.createElement(J7,{enableMeasure:se&&!we,text:m,rows:ye,width:Ve,onEllipsis:at,expanded:pe,miscDeps:[B,pe,k,M,_,w]},(qe,Ct)=>nF(e,u.createElement(u.Fragment,null,qe.length>0&&Ct&&!pe&&Ye?u.createElement("span",{key:"show-content","aria-hidden":!0},qe):qe,Fe(Ct)))))))});var aF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const oF=u.forwardRef((e,t)=>{var{ellipsis:n,rel:r}=e,o=aF(e,["ellipsis","rel"]);const l=Object.assign(Object.assign({},o),{rel:r===void 0&&o.target==="_blank"?"noopener noreferrer":r});return delete l.navigate,u.createElement(rm,Object.assign({},l,{ref:t,ellipsis:!!n,component:"a"}))}),iF=u.forwardRef((e,t)=>u.createElement(rm,Object.assign({ref:t},e,{component:"div"})));var lF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const sF=(e,t)=>{var{ellipsis:n}=e,r=lF(e,["ellipsis"]);const o=u.useMemo(()=>n&&typeof n=="object"?Tn(n,["expandable","rows"]):n,[n]);return u.createElement(rm,Object.assign({ref:t},r,{ellipsis:o,component:"span"}))},cF=u.forwardRef(sF);var uF=function(e,t){var n={};for(var r in e)Object.prototype.hasOwnProperty.call(e,r)&&t.indexOf(r)<0&&(n[r]=e[r]);if(e!=null&&typeof Object.getOwnPropertySymbols=="function")for(var o=0,r=Object.getOwnPropertySymbols(e);o<r.length;o++)t.indexOf(r[o])<0&&Object.prototype.propertyIsEnumerable.call(e,r[o])&&(n[r[o]]=e[r[o]]);return n};const fF=[1,2,3,4,5],dF=u.forwardRef((e,t)=>{const{level:n=1}=e,r=uF(e,["level"]),o=fF.includes(n)?`h${n}`:"h1";return u.createElement(rm,Object.assign({ref:t},r,{component:o}))}),nu=dR;nu.Text=cF;nu.Link=oF;nu.Title=dF;nu.Paragraph=iF;var cc={},RE;function mF(){if(RE)return cc;RE=1,Object.defineProperty(cc,"__esModule",{value:!0}),cc.parse=c,cc.serialize=v;const e=/^[\u0021-\u003A\u003C\u003E-\u007E]+$/,t=/^[\u0021-\u003A\u003C-\u007E]*$/,n=/^([.]?[a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?)([.][a-z0-9]([a-z0-9-]{0,61}[a-z0-9])?)*$/i,r=/^[\u0020-\u003A\u003D-\u007E]*$/,o=Object.prototype.toString,l=(()=>{const y=function(){};return y.prototype=Object.create(null),y})();function c(y,E){const C=new l,x=y.length;if(x<2)return C;const b=(E==null?void 0:E.decode)||g;let w=0;do{const $=y.indexOf("=",w);if($===-1)break;const O=y.indexOf(";",w),T=O===-1?x:O;if($>T){w=y.lastIndexOf(";",$-1)+1;continue}const R=d(y,w,$),M=m(y,$,R),N=y.slice(R,M);if(C[N]===void 0){let z=d(y,$+1,T),P=m(y,T,z);const D=b(y.slice(z,P));C[N]=D}w=T+1}while(w<x);return C}function d(y,E,C){do{const x=y.charCodeAt(E);if(x!==32&&x!==9)return E}while(++E<C);return C}function m(y,E,C){for(;E>C;){const x=y.charCodeAt(--E);if(x!==32&&x!==9)return E+1}return C}function v(y,E,C){const x=(C==null?void 0:C.encode)||encodeURIComponent;if(!e.test(y))throw new TypeError(`argument name is invalid: ${y}`);const b=x(E);if(!t.test(b))throw new TypeError(`argument val is invalid: ${E}`);let w=y+"="+b;if(!C)return w;if(C.maxAge!==void 0){if(!Number.isInteger(C.maxAge))throw new TypeError(`option maxAge is invalid: ${C.maxAge}`);w+="; Max-Age="+C.maxAge}if(C.domain){if(!n.test(C.domain))throw new TypeError(`option domain is invalid: ${C.domain}`);w+="; Domain="+C.domain}if(C.path){if(!r.test(C.path))throw new TypeError(`option path is invalid: ${C.path}`);w+="; Path="+C.path}if(C.expires){if(!p(C.expires)||!Number.isFinite(C.expires.valueOf()))throw new TypeError(`option expires is invalid: ${C.expires}`);w+="; Expires="+C.expires.toUTCString()}if(C.httpOnly&&(w+="; HttpOnly"),C.secure&&(w+="; Secure"),C.partitioned&&(w+="; Partitioned"),C.priority)switch(typeof C.priority=="string"?C.priority.toLowerCase():void 0){case"low":w+="; Priority=Low";break;case"medium":w+="; Priority=Medium";break;case"high":w+="; Priority=High";break;default:throw new TypeError(`option priority is invalid: ${C.priority}`)}if(C.sameSite)switch(typeof C.sameSite=="string"?C.sameSite.toLowerCase():C.sameSite){case!0:case"strict":w+="; SameSite=Strict";break;case"lax":w+="; SameSite=Lax";break;case"none":w+="; SameSite=None";break;default:throw new TypeError(`option sameSite is invalid: ${C.sameSite}`)}return w}function g(y){if(y.indexOf("%")===-1)return y;try{return decodeURIComponent(y)}catch{return y}}function p(y){return o.call(y)==="[object Date]"}return cc}mF();/**
 * react-router v7.5.1
 *
 * Copyright (c) Remix Software Inc.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE.md file in the root directory of this source tree.
 *
 * @license MIT
 */var TE="popstate";function vF(e={}){function t(o,l){let{pathname:c="/",search:d="",hash:m=""}=os(o.location.hash.substring(1));return!c.startsWith("/")&&!c.startsWith(".")&&(c="/"+c),Mp("",{pathname:c,search:d,hash:m},l.state&&l.state.usr||null,l.state&&l.state.key||"default")}function n(o,l){let c=o.document.querySelector("base"),d="";if(c&&c.getAttribute("href")){let m=o.location.href,v=m.indexOf("#");d=v===-1?m:m.slice(0,v)}return d+"#"+(typeof l=="string"?l:Lc(l))}function r(o,l){Aa(o.pathname.charAt(0)==="/",`relative pathnames are not supported in hash history.push(${JSON.stringify(l)})`)}return hF(t,n,r,e)}function Zn(e,t){if(e===!1||e===null||typeof e>"u")throw new Error(t)}function Aa(e,t){if(!e){typeof console<"u"&&console.warn(t);try{throw new Error(t)}catch{}}}function gF(){return Math.random().toString(36).substring(2,10)}function ME(e,t){return{usr:e.state,key:e.key,idx:t}}function Mp(e,t,n=null,r){return{pathname:typeof e=="string"?e:e.pathname,search:"",hash:"",...typeof t=="string"?os(t):t,state:n,key:t&&t.key||r||gF()}}function Lc({pathname:e="/",search:t="",hash:n=""}){return t&&t!=="?"&&(e+=t.charAt(0)==="?"?t:"?"+t),n&&n!=="#"&&(e+=n.charAt(0)==="#"?n:"#"+n),e}function os(e){let t={};if(e){let n=e.indexOf("#");n>=0&&(t.hash=e.substring(n),e=e.substring(0,n));let r=e.indexOf("?");r>=0&&(t.search=e.substring(r),e=e.substring(0,r)),e&&(t.pathname=e)}return t}function hF(e,t,n,r={}){let{window:o=document.defaultView,v5Compat:l=!1}=r,c=o.history,d="POP",m=null,v=g();v==null&&(v=0,c.replaceState({...c.state,idx:v},""));function g(){return(c.state||{idx:null}).idx}function p(){d="POP";let b=g(),w=b==null?null:b-v;v=b,m&&m({action:d,location:x.location,delta:w})}function y(b,w){d="PUSH";let $=Mp(x.location,b,w);n&&n($,b),v=g()+1;let O=ME($,v),T=x.createHref($);try{c.pushState(O,"",T)}catch(R){if(R instanceof DOMException&&R.name==="DataCloneError")throw R;o.location.assign(T)}l&&m&&m({action:d,location:x.location,delta:1})}function E(b,w){d="REPLACE";let $=Mp(x.location,b,w);n&&n($,b),v=g();let O=ME($,v),T=x.createHref($);c.replaceState(O,"",T),l&&m&&m({action:d,location:x.location,delta:0})}function C(b){let w=o.location.origin!=="null"?o.location.origin:o.location.href,$=typeof b=="string"?b:Lc(b);return $=$.replace(/ $/,"%20"),Zn(w,`No window.location.(origin|href) available to create URL for href: ${$}`),new URL($,w)}let x={get action(){return d},get location(){return e(o,c)},listen(b){if(m)throw new Error("A history only accepts one active listener");return o.addEventListener(TE,p),m=b,()=>{o.removeEventListener(TE,p),m=null}},createHref(b){return t(o,b)},createURL:C,encodeLocation(b){let w=C(b);return{pathname:w.pathname,search:w.search,hash:w.hash}},push:y,replace:E,go(b){return c.go(b)}};return x}function mR(e,t,n="/"){return pF(e,t,n,!1)}function pF(e,t,n,r){let o=typeof t=="string"?os(t):t,l=Eo(o.pathname||"/",n);if(l==null)return null;let c=vR(e);yF(c);let d=null;for(let m=0;d==null&&m<c.length;++m){let v=MF(l);d=RF(c[m],v,r)}return d}function vR(e,t=[],n=[],r=""){let o=(l,c,d)=>{let m={relativePath:d===void 0?l.path||"":d,caseSensitive:l.caseSensitive===!0,childrenIndex:c,route:l};m.relativePath.startsWith("/")&&(Zn(m.relativePath.startsWith(r),`Absolute route path "${m.relativePath}" nested under path "${r}" is not valid. An absolute child route path must start with the combined path of all its parent routes.`),m.relativePath=m.relativePath.slice(r.length));let v=yo([r,m.relativePath]),g=n.concat(m);l.children&&l.children.length>0&&(Zn(l.index!==!0,`Index routes must not have child routes. Please remove all child routes from route path "${v}".`),vR(l.children,t,g,v)),!(l.path==null&&!l.index)&&t.push({path:v,score:$F(v,l.index),routesMeta:g})};return e.forEach((l,c)=>{var d;if(l.path===""||!((d=l.path)!=null&&d.includes("?")))o(l,c);else for(let m of gR(l.path))o(l,c,m)}),t}function gR(e){let t=e.split("/");if(t.length===0)return[];let[n,...r]=t,o=n.endsWith("?"),l=n.replace(/\?$/,"");if(r.length===0)return o?[l,""]:[l];let c=gR(r.join("/")),d=[];return d.push(...c.map(m=>m===""?l:[l,m].join("/"))),o&&d.push(...c),d.map(m=>e.startsWith("/")&&m===""?"/":m)}function yF(e){e.sort((t,n)=>t.score!==n.score?n.score-t.score:OF(t.routesMeta.map(r=>r.childrenIndex),n.routesMeta.map(r=>r.childrenIndex)))}var bF=/^:[\w-]+$/,SF=3,CF=2,xF=1,EF=10,wF=-2,_E=e=>e==="*";function $F(e,t){let n=e.split("/"),r=n.length;return n.some(_E)&&(r+=wF),t&&(r+=CF),n.filter(o=>!_E(o)).reduce((o,l)=>o+(bF.test(l)?SF:l===""?xF:EF),r)}function OF(e,t){return e.length===t.length&&e.slice(0,-1).every((r,o)=>r===t[o])?e[e.length-1]-t[t.length-1]:0}function RF(e,t,n=!1){let{routesMeta:r}=e,o={},l="/",c=[];for(let d=0;d<r.length;++d){let m=r[d],v=d===r.length-1,g=l==="/"?t:t.slice(l.length)||"/",p=dd({path:m.relativePath,caseSensitive:m.caseSensitive,end:v},g),y=m.route;if(!p&&v&&n&&!r[r.length-1].route.index&&(p=dd({path:m.relativePath,caseSensitive:m.caseSensitive,end:!1},g)),!p)return null;Object.assign(o,p.params),c.push({params:o,pathname:yo([l,p.pathname]),pathnameBase:IF(yo([l,p.pathnameBase])),route:y}),p.pathnameBase!=="/"&&(l=yo([l,p.pathnameBase]))}return c}function dd(e,t){typeof e=="string"&&(e={path:e,caseSensitive:!1,end:!0});let[n,r]=TF(e.path,e.caseSensitive,e.end),o=t.match(n);if(!o)return null;let l=o[0],c=l.replace(/(.)\/+$/,"$1"),d=o.slice(1);return{params:r.reduce((v,{paramName:g,isOptional:p},y)=>{if(g==="*"){let C=d[y]||"";c=l.slice(0,l.length-C.length).replace(/(.)\/+$/,"$1")}const E=d[y];return p&&!E?v[g]=void 0:v[g]=(E||"").replace(/%2F/g,"/"),v},{}),pathname:l,pathnameBase:c,pattern:e}}function TF(e,t=!1,n=!0){Aa(e==="*"||!e.endsWith("*")||e.endsWith("/*"),`Route path "${e}" will be treated as if it were "${e.replace(/\*$/,"/*")}" because the \`*\` character must always follow a \`/\` in the pattern. To get rid of this warning, please change the route path to "${e.replace(/\*$/,"/*")}".`);let r=[],o="^"+e.replace(/\/*\*?$/,"").replace(/^\/*/,"/").replace(/[\\.*+^${}|()[\]]/g,"\\$&").replace(/\/:([\w-]+)(\?)?/g,(c,d,m)=>(r.push({paramName:d,isOptional:m!=null}),m?"/?([^\\/]+)?":"/([^\\/]+)"));return e.endsWith("*")?(r.push({paramName:"*"}),o+=e==="*"||e==="/*"?"(.*)$":"(?:\\/(.+)|\\/*)$"):n?o+="\\/*$":e!==""&&e!=="/"&&(o+="(?:(?=\\/|$))"),[new RegExp(o,t?void 0:"i"),r]}function MF(e){try{return e.split("/").map(t=>decodeURIComponent(t).replace(/\//g,"%2F")).join("/")}catch(t){return Aa(!1,`The URL path "${e}" could not be decoded because it is a malformed URL segment. This is probably due to a bad percent encoding (${t}).`),e}}function Eo(e,t){if(t==="/")return e;if(!e.toLowerCase().startsWith(t.toLowerCase()))return null;let n=t.endsWith("/")?t.length-1:t.length,r=e.charAt(n);return r&&r!=="/"?null:e.slice(n)||"/"}function _F(e,t="/"){let{pathname:n,search:r="",hash:o=""}=typeof e=="string"?os(e):e;return{pathname:n?n.startsWith("/")?n:NF(n,t):t,search:jF(r),hash:PF(o)}}function NF(e,t){let n=t.replace(/\/+$/,"").split("/");return e.split("/").forEach(o=>{o===".."?n.length>1&&n.pop():o!=="."&&n.push(o)}),n.length>1?n.join("/"):"/"}function mh(e,t,n,r){return`Cannot include a '${e}' character in a manually specified \`to.${t}\` field [${JSON.stringify(r)}].  Please separate it out to the \`to.${n}\` field. Alternatively you may provide the full path as a string in <Link to="..."> and the router will parse it for you.`}function AF(e){return e.filter((t,n)=>n===0||t.route.path&&t.route.path.length>0)}function hR(e){let t=AF(e);return t.map((n,r)=>r===t.length-1?n.pathname:n.pathnameBase)}function pR(e,t,n,r=!1){let o;typeof e=="string"?o=os(e):(o={...e},Zn(!o.pathname||!o.pathname.includes("?"),mh("?","pathname","search",o)),Zn(!o.pathname||!o.pathname.includes("#"),mh("#","pathname","hash",o)),Zn(!o.search||!o.search.includes("#"),mh("#","search","hash",o)));let l=e===""||o.pathname==="",c=l?"/":o.pathname,d;if(c==null)d=n;else{let p=t.length-1;if(!r&&c.startsWith("..")){let y=c.split("/");for(;y[0]==="..";)y.shift(),p-=1;o.pathname=y.join("/")}d=p>=0?t[p]:"/"}let m=_F(o,d),v=c&&c!=="/"&&c.endsWith("/"),g=(l||c===".")&&n.endsWith("/");return!m.pathname.endsWith("/")&&(v||g)&&(m.pathname+="/"),m}var yo=e=>e.join("/").replace(/\/\/+/g,"/"),IF=e=>e.replace(/\/+$/,"").replace(/^\/*/,"/"),jF=e=>!e||e==="?"?"":e.startsWith("?")?e:"?"+e,PF=e=>!e||e==="#"?"":e.startsWith("#")?e:"#"+e;function zF(e){return e!=null&&typeof e.status=="number"&&typeof e.statusText=="string"&&typeof e.internal=="boolean"&&"data"in e}var yR=["POST","PUT","PATCH","DELETE"];new Set(yR);var DF=["GET",...yR];new Set(DF);var is=u.createContext(null);is.displayName="DataRouter";var am=u.createContext(null);am.displayName="DataRouterState";var bR=u.createContext({isTransitioning:!1});bR.displayName="ViewTransition";var BF=u.createContext(new Map);BF.displayName="Fetchers";var LF=u.createContext(null);LF.displayName="Await";var Qa=u.createContext(null);Qa.displayName="Navigation";var om=u.createContext(null);om.displayName="Location";var Pa=u.createContext({outlet:null,matches:[],isDataRoute:!1});Pa.displayName="Route";var G0=u.createContext(null);G0.displayName="RouteError";function HF(e,{relative:t}={}){Zn(ru(),"useHref() may be used only in the context of a <Router> component.");let{basename:n,navigator:r}=u.useContext(Qa),{hash:o,pathname:l,search:c}=au(e,{relative:t}),d=l;return n!=="/"&&(d=l==="/"?n:yo([n,l])),r.createHref({pathname:d,search:c,hash:o})}function ru(){return u.useContext(om)!=null}function ci(){return Zn(ru(),"useLocation() may be used only in the context of a <Router> component."),u.useContext(om).location}var SR="You should call navigate() in a React.useEffect(), not when your component is first rendered.";function CR(e){u.useContext(Qa).static||u.useLayoutEffect(e)}function xR(){let{isDataRoute:e}=u.useContext(Pa);return e?e9():FF()}function FF(){Zn(ru(),"useNavigate() may be used only in the context of a <Router> component.");let e=u.useContext(is),{basename:t,navigator:n}=u.useContext(Qa),{matches:r}=u.useContext(Pa),{pathname:o}=ci(),l=JSON.stringify(hR(r)),c=u.useRef(!1);return CR(()=>{c.current=!0}),u.useCallback((m,v={})=>{if(Aa(c.current,SR),!c.current)return;if(typeof m=="number"){n.go(m);return}let g=pR(m,JSON.parse(l),o,v.relative==="path");e==null&&t!=="/"&&(g.pathname=g.pathname==="/"?t:yo([t,g.pathname])),(v.replace?n.replace:n.push)(g,v.state,v)},[t,n,l,o,e])}var ER=u.createContext(null);function fW(){return u.useContext(ER)}function VF(e){let t=u.useContext(Pa).outlet;return t&&u.createElement(ER.Provider,{value:e},t)}function dW(){let{matches:e}=u.useContext(Pa),t=e[e.length-1];return t?t.params:{}}function au(e,{relative:t}={}){let{matches:n}=u.useContext(Pa),{pathname:r}=ci(),o=JSON.stringify(hR(n));return u.useMemo(()=>pR(e,JSON.parse(o),r,t==="path"),[e,o,r,t])}function UF(e,t){return wR(e)}function wR(e,t,n,r){Zn(ru(),"useRoutes() may be used only in the context of a <Router> component.");let{navigator:o,static:l}=u.useContext(Qa),{matches:c}=u.useContext(Pa),d=c[c.length-1],m=d?d.params:{},v=d?d.pathname:"/",g=d?d.pathnameBase:"/",p=d&&d.route;{let $=p&&p.path||"";$R(v,!p||$.endsWith("*")||$.endsWith("*?"),`You rendered descendant <Routes> (or called \`useRoutes()\`) at "${v}" (under <Route path="${$}">) but the parent route path has no trailing "*". This means if you navigate deeper, the parent won't match anymore and therefore the child routes will never render.

Please change the parent <Route path="${$}"> to <Route path="${$==="/"?"*":`${$}/*`}">.`)}let y=ci(),E;E=y;let C=E.pathname||"/",x=C;if(g!=="/"){let $=g.replace(/^\//,"").split("/");x="/"+C.replace(/^\//,"").split("/").slice($.length).join("/")}let b=!l&&n&&n.matches&&n.matches.length>0?n.matches:mR(e,{pathname:x});return Aa(p||b!=null,`No routes matched location "${E.pathname}${E.search}${E.hash}" `),Aa(b==null||b[b.length-1].route.element!==void 0||b[b.length-1].route.Component!==void 0||b[b.length-1].route.lazy!==void 0,`Matched leaf route at location "${E.pathname}${E.search}${E.hash}" does not have an element or Component. This means it will render an <Outlet /> with a null value by default resulting in an "empty" page.`),KF(b&&b.map($=>Object.assign({},$,{params:Object.assign({},m,$.params),pathname:yo([g,o.encodeLocation?o.encodeLocation($.pathname).pathname:$.pathname]),pathnameBase:$.pathnameBase==="/"?g:yo([g,o.encodeLocation?o.encodeLocation($.pathnameBase).pathname:$.pathnameBase])})),c,n,r)}function qF(){let e=JF(),t=zF(e)?`${e.status} ${e.statusText}`:e instanceof Error?e.message:JSON.stringify(e),n=e instanceof Error?e.stack:null,r="rgba(200,200,200, 0.5)",o={padding:"0.5rem",backgroundColor:r},l={padding:"2px 4px",backgroundColor:r},c=null;return console.error("Error handled by React Router default ErrorBoundary:",e),c=u.createElement(u.Fragment,null,u.createElement("p",null,"💿 Hey developer 👋"),u.createElement("p",null,"You can provide a way better UX than this when your app throws errors by providing your own ",u.createElement("code",{style:l},"ErrorBoundary")," or"," ",u.createElement("code",{style:l},"errorElement")," prop on your route.")),u.createElement(u.Fragment,null,u.createElement("h2",null,"Unexpected Application Error!"),u.createElement("h3",{style:{fontStyle:"italic"}},t),n?u.createElement("pre",{style:o},n):null,c)}var WF=u.createElement(qF,null),kF=class extends u.Component{constructor(e){super(e),this.state={location:e.location,revalidation:e.revalidation,error:e.error}}static getDerivedStateFromError(e){return{error:e}}static getDerivedStateFromProps(e,t){return t.location!==e.location||t.revalidation!=="idle"&&e.revalidation==="idle"?{error:e.error,location:e.location,revalidation:e.revalidation}:{error:e.error!==void 0?e.error:t.error,location:t.location,revalidation:e.revalidation||t.revalidation}}componentDidCatch(e,t){console.error("React Router caught the following error during render",e,t)}render(){return this.state.error!==void 0?u.createElement(Pa.Provider,{value:this.props.routeContext},u.createElement(G0.Provider,{value:this.state.error,children:this.props.component})):this.props.children}};function GF({routeContext:e,match:t,children:n}){let r=u.useContext(is);return r&&r.static&&r.staticContext&&(t.route.errorElement||t.route.ErrorBoundary)&&(r.staticContext._deepestRenderedBoundaryId=t.route.id),u.createElement(Pa.Provider,{value:e},n)}function KF(e,t=[],n=null,r=null){if(e==null){if(!n)return null;if(n.errors)e=n.matches;else if(t.length===0&&!n.initialized&&n.matches.length>0)e=n.matches;else return null}let o=e,l=n==null?void 0:n.errors;if(l!=null){let m=o.findIndex(v=>v.route.id&&(l==null?void 0:l[v.route.id])!==void 0);Zn(m>=0,`Could not find a matching route for errors on route IDs: ${Object.keys(l).join(",")}`),o=o.slice(0,Math.min(o.length,m+1))}let c=!1,d=-1;if(n)for(let m=0;m<o.length;m++){let v=o[m];if((v.route.HydrateFallback||v.route.hydrateFallbackElement)&&(d=m),v.route.id){let{loaderData:g,errors:p}=n,y=v.route.loader&&!g.hasOwnProperty(v.route.id)&&(!p||p[v.route.id]===void 0);if(v.route.lazy||y){c=!0,d>=0?o=o.slice(0,d+1):o=[o[0]];break}}}return o.reduceRight((m,v,g)=>{let p,y=!1,E=null,C=null;n&&(p=l&&v.route.id?l[v.route.id]:void 0,E=v.route.errorElement||WF,c&&(d<0&&g===0?($R("route-fallback",!1,"No `HydrateFallback` element provided to render during initial hydration"),y=!0,C=null):d===g&&(y=!0,C=v.route.hydrateFallbackElement||null)));let x=t.concat(o.slice(0,g+1)),b=()=>{let w;return p?w=E:y?w=C:v.route.Component?w=u.createElement(v.route.Component,null):v.route.element?w=v.route.element:w=m,u.createElement(GF,{match:v,routeContext:{outlet:m,matches:x,isDataRoute:n!=null},children:w})};return n&&(v.route.ErrorBoundary||v.route.errorElement||g===0)?u.createElement(kF,{location:n.location,revalidation:n.revalidation,component:E,error:p,children:b(),routeContext:{outlet:null,matches:x,isDataRoute:!0}}):b()},null)}function K0(e){return`${e} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function XF(e){let t=u.useContext(is);return Zn(t,K0(e)),t}function YF(e){let t=u.useContext(am);return Zn(t,K0(e)),t}function QF(e){let t=u.useContext(Pa);return Zn(t,K0(e)),t}function X0(e){let t=QF(e),n=t.matches[t.matches.length-1];return Zn(n.route.id,`${e} can only be used on routes that contain a unique "id"`),n.route.id}function ZF(){return X0("useRouteId")}function JF(){var r;let e=u.useContext(G0),t=YF("useRouteError"),n=X0("useRouteError");return e!==void 0?e:(r=t.errors)==null?void 0:r[n]}function e9(){let{router:e}=XF("useNavigate"),t=X0("useNavigate"),n=u.useRef(!1);return CR(()=>{n.current=!0}),u.useCallback(async(o,l={})=>{Aa(n.current,SR),n.current&&(typeof o=="number"?e.navigate(o):await e.navigate(o,{fromRouteId:t,...l}))},[e,t])}var NE={};function $R(e,t,n){!t&&!NE[e]&&(NE[e]=!0,Aa(!1,n))}u.memo(t9);function t9({routes:e,future:t,state:n}){return wR(e,void 0,n,t)}function n9(e){return VF(e.context)}function r9({basename:e="/",children:t=null,location:n,navigationType:r="POP",navigator:o,static:l=!1}){Zn(!ru(),"You cannot render a <Router> inside another <Router>. You should never have more than one in your app.");let c=e.replace(/^\/*/,"/"),d=u.useMemo(()=>({basename:c,navigator:o,static:l,future:{}}),[c,o,l]);typeof n=="string"&&(n=os(n));let{pathname:m="/",search:v="",hash:g="",state:p=null,key:y="default"}=n,E=u.useMemo(()=>{let C=Eo(m,c);return C==null?null:{location:{pathname:C,search:v,hash:g,state:p,key:y},navigationType:r}},[c,m,v,g,p,y,r]);return Aa(E!=null,`<Router basename="${c}"> is not able to match the URL "${m}${v}${g}" because it does not start with the basename, so the <Router> won't render anything.`),E==null?null:u.createElement(Qa.Provider,{value:d},u.createElement(om.Provider,{children:t,value:E}))}var Hf="get",Ff="application/x-www-form-urlencoded";function im(e){return e!=null&&typeof e.tagName=="string"}function a9(e){return im(e)&&e.tagName.toLowerCase()==="button"}function o9(e){return im(e)&&e.tagName.toLowerCase()==="form"}function i9(e){return im(e)&&e.tagName.toLowerCase()==="input"}function l9(e){return!!(e.metaKey||e.altKey||e.ctrlKey||e.shiftKey)}function s9(e,t){return e.button===0&&(!t||t==="_self")&&!l9(e)}var Ef=null;function c9(){if(Ef===null)try{new FormData(document.createElement("form"),0),Ef=!1}catch{Ef=!0}return Ef}var u9=new Set(["application/x-www-form-urlencoded","multipart/form-data","text/plain"]);function vh(e){return e!=null&&!u9.has(e)?(Aa(!1,`"${e}" is not a valid \`encType\` for \`<Form>\`/\`<fetcher.Form>\` and will default to "${Ff}"`),null):e}function f9(e,t){let n,r,o,l,c;if(o9(e)){let d=e.getAttribute("action");r=d?Eo(d,t):null,n=e.getAttribute("method")||Hf,o=vh(e.getAttribute("enctype"))||Ff,l=new FormData(e)}else if(a9(e)||i9(e)&&(e.type==="submit"||e.type==="image")){let d=e.form;if(d==null)throw new Error('Cannot submit a <button> or <input type="submit"> without a <form>');let m=e.getAttribute("formaction")||d.getAttribute("action");if(r=m?Eo(m,t):null,n=e.getAttribute("formmethod")||d.getAttribute("method")||Hf,o=vh(e.getAttribute("formenctype"))||vh(d.getAttribute("enctype"))||Ff,l=new FormData(d,e),!c9()){let{name:v,type:g,value:p}=e;if(g==="image"){let y=v?`${v}.`:"";l.append(`${y}x`,"0"),l.append(`${y}y`,"0")}else v&&l.append(v,p)}}else{if(im(e))throw new Error('Cannot submit element that is not <form>, <button>, or <input type="submit|image">');n=Hf,r=null,o=Ff,c=e}return l&&o==="text/plain"&&(c=l,l=void 0),{action:r,method:n.toLowerCase(),encType:o,formData:l,body:c}}function Y0(e,t){if(e===!1||e===null||typeof e>"u")throw new Error(t)}async function d9(e,t){if(e.id in t)return t[e.id];try{let n=await import(e.module);return t[e.id]=n,n}catch(n){return console.error(`Error loading route module \`${e.module}\`, reloading page...`),console.error(n),window.__reactRouterContext&&window.__reactRouterContext.isSpaMode,window.location.reload(),new Promise(()=>{})}}function m9(e){return e==null?!1:e.href==null?e.rel==="preload"&&typeof e.imageSrcSet=="string"&&typeof e.imageSizes=="string":typeof e.rel=="string"&&typeof e.href=="string"}async function v9(e,t,n){let r=await Promise.all(e.map(async o=>{let l=t.routes[o.route.id];if(l){let c=await d9(l,n);return c.links?c.links():[]}return[]}));return y9(r.flat(1).filter(m9).filter(o=>o.rel==="stylesheet"||o.rel==="preload").map(o=>o.rel==="stylesheet"?{...o,rel:"prefetch",as:"style"}:{...o,rel:"prefetch"}))}function AE(e,t,n,r,o,l){let c=(m,v)=>n[v]?m.route.id!==n[v].route.id:!0,d=(m,v)=>{var g;return n[v].pathname!==m.pathname||((g=n[v].route.path)==null?void 0:g.endsWith("*"))&&n[v].params["*"]!==m.params["*"]};return l==="assets"?t.filter((m,v)=>c(m,v)||d(m,v)):l==="data"?t.filter((m,v)=>{var p;let g=r.routes[m.route.id];if(!g||!g.hasLoader)return!1;if(c(m,v)||d(m,v))return!0;if(m.route.shouldRevalidate){let y=m.route.shouldRevalidate({currentUrl:new URL(o.pathname+o.search+o.hash,window.origin),currentParams:((p=n[0])==null?void 0:p.params)||{},nextUrl:new URL(e,window.origin),nextParams:m.params,defaultShouldRevalidate:!0});if(typeof y=="boolean")return y}return!0}):[]}function g9(e,t,{includeHydrateFallback:n}={}){return h9(e.map(r=>{let o=t.routes[r.route.id];if(!o)return[];let l=[o.module];return o.clientActionModule&&(l=l.concat(o.clientActionModule)),o.clientLoaderModule&&(l=l.concat(o.clientLoaderModule)),n&&o.hydrateFallbackModule&&(l=l.concat(o.hydrateFallbackModule)),o.imports&&(l=l.concat(o.imports)),l}).flat(1))}function h9(e){return[...new Set(e)]}function p9(e){let t={},n=Object.keys(e).sort();for(let r of n)t[r]=e[r];return t}function y9(e,t){let n=new Set;return new Set(t),e.reduce((r,o)=>{let l=JSON.stringify(p9(o));return n.has(l)||(n.add(l),r.push({key:l,link:o})),r},[])}var b9=new Set([100,101,204,205]);function S9(e,t){let n=typeof e=="string"?new URL(e,typeof window>"u"?"server://singlefetch/":window.location.origin):e;return n.pathname==="/"?n.pathname="_root.data":t&&Eo(n.pathname,t)==="/"?n.pathname=`${t.replace(/\/$/,"")}/_root.data`:n.pathname=`${n.pathname.replace(/\/$/,"")}.data`,n}function OR(){let e=u.useContext(is);return Y0(e,"You must render this element inside a <DataRouterContext.Provider> element"),e}function C9(){let e=u.useContext(am);return Y0(e,"You must render this element inside a <DataRouterStateContext.Provider> element"),e}var Q0=u.createContext(void 0);Q0.displayName="FrameworkContext";function RR(){let e=u.useContext(Q0);return Y0(e,"You must render this element inside a <HydratedRouter> element"),e}function x9(e,t){let n=u.useContext(Q0),[r,o]=u.useState(!1),[l,c]=u.useState(!1),{onFocus:d,onBlur:m,onMouseEnter:v,onMouseLeave:g,onTouchStart:p}=t,y=u.useRef(null);u.useEffect(()=>{if(e==="render"&&c(!0),e==="viewport"){let x=w=>{w.forEach($=>{c($.isIntersecting)})},b=new IntersectionObserver(x,{threshold:.5});return y.current&&b.observe(y.current),()=>{b.disconnect()}}},[e]),u.useEffect(()=>{if(r){let x=setTimeout(()=>{c(!0)},100);return()=>{clearTimeout(x)}}},[r]);let E=()=>{o(!0)},C=()=>{o(!1),c(!1)};return n?e!=="intent"?[l,y,{}]:[l,y,{onFocus:uc(d,E),onBlur:uc(m,C),onMouseEnter:uc(v,E),onMouseLeave:uc(g,C),onTouchStart:uc(p,E)}]:[!1,y,{}]}function uc(e,t){return n=>{e&&e(n),n.defaultPrevented||t(n)}}function E9({page:e,...t}){let{router:n}=OR(),r=u.useMemo(()=>mR(n.routes,e,n.basename),[n.routes,e,n.basename]);return r?u.createElement($9,{page:e,matches:r,...t}):null}function w9(e){let{manifest:t,routeModules:n}=RR(),[r,o]=u.useState([]);return u.useEffect(()=>{let l=!1;return v9(e,t,n).then(c=>{l||o(c)}),()=>{l=!0}},[e,t,n]),r}function $9({page:e,matches:t,...n}){let r=ci(),{manifest:o,routeModules:l}=RR(),{basename:c}=OR(),{loaderData:d,matches:m}=C9(),v=u.useMemo(()=>AE(e,t,m,o,r,"data"),[e,t,m,o,r]),g=u.useMemo(()=>AE(e,t,m,o,r,"assets"),[e,t,m,o,r]),p=u.useMemo(()=>{if(e===r.pathname+r.search+r.hash)return[];let C=new Set,x=!1;if(t.forEach(w=>{var O;let $=o.routes[w.route.id];!$||!$.hasLoader||(!v.some(T=>T.route.id===w.route.id)&&w.route.id in d&&((O=l[w.route.id])!=null&&O.shouldRevalidate)||$.hasClientLoader?x=!0:C.add(w.route.id))}),C.size===0)return[];let b=S9(e,c);return x&&C.size>0&&b.searchParams.set("_routes",t.filter(w=>C.has(w.route.id)).map(w=>w.route.id).join(",")),[b.pathname+b.search]},[c,d,r,o,v,t,e,l]),y=u.useMemo(()=>g9(g,o),[g,o]),E=w9(g);return u.createElement(u.Fragment,null,p.map(C=>u.createElement("link",{key:C,rel:"prefetch",as:"fetch",href:C,...n})),y.map(C=>u.createElement("link",{key:C,rel:"modulepreload",href:C,...n})),E.map(({key:C,link:x})=>u.createElement("link",{key:C,...x})))}function O9(...e){return t=>{e.forEach(n=>{typeof n=="function"?n(t):n!=null&&(n.current=t)})}}var TR=typeof window<"u"&&typeof window.document<"u"&&typeof window.document.createElement<"u";try{TR&&(window.__reactRouterVersion="7.5.1")}catch{}function R9({basename:e,children:t,window:n}){let r=u.useRef();r.current==null&&(r.current=vF({window:n,v5Compat:!0}));let o=r.current,[l,c]=u.useState({action:o.action,location:o.location}),d=u.useCallback(m=>{u.startTransition(()=>c(m))},[c]);return u.useLayoutEffect(()=>o.listen(d),[o,d]),u.createElement(r9,{basename:e,children:t,location:l.location,navigationType:l.action,navigator:o})}var MR=/^(?:[a-z][a-z0-9+.-]*:|\/\/)/i,_R=u.forwardRef(function({onClick:t,discover:n="render",prefetch:r="none",relative:o,reloadDocument:l,replace:c,state:d,target:m,to:v,preventScrollReset:g,viewTransition:p,...y},E){let{basename:C}=u.useContext(Qa),x=typeof v=="string"&&MR.test(v),b,w=!1;if(typeof v=="string"&&x&&(b=v,TR))try{let P=new URL(window.location.href),D=v.startsWith("//")?new URL(P.protocol+v):new URL(v),F=Eo(D.pathname,C);D.origin===P.origin&&F!=null?v=F+D.search+D.hash:w=!0}catch{Aa(!1,`<Link to="${v}"> contains an invalid URL which will probably break when clicked - please update to a valid URL path.`)}let $=HF(v,{relative:o}),[O,T,R]=x9(r,y),M=N9(v,{replace:c,state:d,target:m,preventScrollReset:g,relative:o,viewTransition:p});function N(P){t&&t(P),P.defaultPrevented||M(P)}let z=u.createElement("a",{...y,...R,href:b||$,onClick:w||l?t:N,ref:O9(E,T),target:m,"data-discover":!x&&n==="render"?"true":void 0});return O&&!x?u.createElement(u.Fragment,null,z,u.createElement(E9,{page:$})):z});_R.displayName="Link";var T9=u.forwardRef(function({"aria-current":t="page",caseSensitive:n=!1,className:r="",end:o=!1,style:l,to:c,viewTransition:d,children:m,...v},g){let p=au(c,{relative:v.relative}),y=ci(),E=u.useContext(am),{navigator:C,basename:x}=u.useContext(Qa),b=E!=null&&z9(p)&&d===!0,w=C.encodeLocation?C.encodeLocation(p).pathname:p.pathname,$=y.pathname,O=E&&E.navigation&&E.navigation.location?E.navigation.location.pathname:null;n||($=$.toLowerCase(),O=O?O.toLowerCase():null,w=w.toLowerCase()),O&&x&&(O=Eo(O,x)||O);const T=w!=="/"&&w.endsWith("/")?w.length-1:w.length;let R=$===w||!o&&$.startsWith(w)&&$.charAt(T)==="/",M=O!=null&&(O===w||!o&&O.startsWith(w)&&O.charAt(w.length)==="/"),N={isActive:R,isPending:M,isTransitioning:b},z=R?t:void 0,P;typeof r=="function"?P=r(N):P=[r,R?"active":null,M?"pending":null,b?"transitioning":null].filter(Boolean).join(" ");let D=typeof l=="function"?l(N):l;return u.createElement(_R,{...v,"aria-current":z,className:P,ref:g,style:D,to:c,viewTransition:d},typeof m=="function"?m(N):m)});T9.displayName="NavLink";var M9=u.forwardRef(({discover:e="render",fetcherKey:t,navigate:n,reloadDocument:r,replace:o,state:l,method:c=Hf,action:d,onSubmit:m,relative:v,preventScrollReset:g,viewTransition:p,...y},E)=>{let C=j9(),x=P9(d,{relative:v}),b=c.toLowerCase()==="get"?"get":"post",w=typeof d=="string"&&MR.test(d),$=O=>{if(m&&m(O),O.defaultPrevented)return;O.preventDefault();let T=O.nativeEvent.submitter,R=(T==null?void 0:T.getAttribute("formmethod"))||c;C(T||O.currentTarget,{fetcherKey:t,method:R,navigate:n,replace:o,state:l,relative:v,preventScrollReset:g,viewTransition:p})};return u.createElement("form",{ref:E,method:b,action:x,onSubmit:r?m:$,...y,"data-discover":!w&&e==="render"?"true":void 0})});M9.displayName="Form";function _9(e){return`${e} must be used within a data router.  See https://reactrouter.com/en/main/routers/picking-a-router.`}function NR(e){let t=u.useContext(is);return Zn(t,_9(e)),t}function N9(e,{target:t,replace:n,state:r,preventScrollReset:o,relative:l,viewTransition:c}={}){let d=xR(),m=ci(),v=au(e,{relative:l});return u.useCallback(g=>{if(s9(g,t)){g.preventDefault();let p=n!==void 0?n:Lc(m)===Lc(v);d(e,{replace:p,state:r,preventScrollReset:o,relative:l,viewTransition:c})}},[m,d,v,n,r,t,e,o,l,c])}var A9=0,I9=()=>`__${String(++A9)}__`;function j9(){let{router:e}=NR("useSubmit"),{basename:t}=u.useContext(Qa),n=ZF();return u.useCallback(async(r,o={})=>{let{action:l,method:c,encType:d,formData:m,body:v}=f9(r,t);if(o.navigate===!1){let g=o.fetcherKey||I9();await e.fetch(g,n,o.action||l,{preventScrollReset:o.preventScrollReset,formData:m,body:v,formMethod:o.method||c,formEncType:o.encType||d,flushSync:o.flushSync})}else await e.navigate(o.action||l,{preventScrollReset:o.preventScrollReset,formData:m,body:v,formMethod:o.method||c,formEncType:o.encType||d,replace:o.replace,state:o.state,fromRouteId:n,flushSync:o.flushSync,viewTransition:o.viewTransition})},[e,t,n])}function P9(e,{relative:t}={}){let{basename:n}=u.useContext(Qa),r=u.useContext(Pa);Zn(r,"useFormAction must be used inside a RouteContext");let[o]=r.matches.slice(-1),l={...au(e||".",{relative:t})},c=ci();if(e==null){l.search=c.search;let d=new URLSearchParams(l.search),m=d.getAll("index");if(m.some(g=>g==="")){d.delete("index"),m.filter(p=>p).forEach(p=>d.append("index",p));let g=d.toString();l.search=g?`?${g}`:""}}return(!e||e===".")&&o.route.index&&(l.search=l.search?l.search.replace(/^\?/,"?index&"):"?index"),n!=="/"&&(l.pathname=l.pathname==="/"?n:yo([n,l.pathname])),Lc(l)}function z9(e,t={}){let n=u.useContext(bR);Zn(n!=null,"`useViewTransitionState` must be used within `react-router-dom`'s `RouterProvider`.  Did you accidentally import `RouterProvider` from `react-router`?");let{basename:r}=NR("useViewTransitionState"),o=au(e,{relative:t.relative});if(!n.isTransitioning)return!1;let l=Eo(n.currentLocation.pathname,r)||n.currentLocation.pathname,c=Eo(n.nextLocation.pathname,r)||n.nextLocation.pathname;return dd(o.pathname,c)!=null||dd(o.pathname,l)!=null}new TextEncoder;[...b9];var gh={exports:{}},hh={};/**
 * @license React
 * use-sync-external-store-with-selector.production.js
 *
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */var IE;function D9(){if(IE)return hh;IE=1;var e=Cd();function t(m,v){return m===v&&(m!==0||1/m===1/v)||m!==m&&v!==v}var n=typeof Object.is=="function"?Object.is:t,r=e.useSyncExternalStore,o=e.useRef,l=e.useEffect,c=e.useMemo,d=e.useDebugValue;return hh.useSyncExternalStoreWithSelector=function(m,v,g,p,y){var E=o(null);if(E.current===null){var C={hasValue:!1,value:null};E.current=C}else C=E.current;E=c(function(){function b(R){if(!w){if(w=!0,$=R,R=p(R),y!==void 0&&C.hasValue){var M=C.value;if(y(M,R))return O=M}return O=R}if(M=O,n($,R))return M;var N=p(R);return y!==void 0&&y(M,N)?($=R,M):($=R,O=N)}var w=!1,$,O,T=g===void 0?null:g;return[function(){return b(v())},T===null?void 0:function(){return b(T())}]},[v,g,p,y]);var x=r(m,E[0],E[1]);return l(function(){C.hasValue=!0,C.value=x},[x]),d(x),x},hh}var jE;function B9(){return jE||(jE=1,gh.exports=D9()),gh.exports}var L9=B9();function H9(e){e()}function F9(){let e=null,t=null;return{clear(){e=null,t=null},notify(){H9(()=>{let n=e;for(;n;)n.callback(),n=n.next})},get(){const n=[];let r=e;for(;r;)n.push(r),r=r.next;return n},subscribe(n){let r=!0;const o=t={callback:n,next:null,prev:t};return o.prev?o.prev.next=o:e=o,function(){!r||e===null||(r=!1,o.next?o.next.prev=o.prev:t=o.prev,o.prev?o.prev.next=o.next:e=o.next)}}}}var PE={notify(){},get:()=>[]};function V9(e,t){let n,r=PE,o=0,l=!1;function c(x){g();const b=r.subscribe(x);let w=!1;return()=>{w||(w=!0,b(),p())}}function d(){r.notify()}function m(){C.onStateChange&&C.onStateChange()}function v(){return l}function g(){o++,n||(n=e.subscribe(m),r=F9())}function p(){o--,n&&o===0&&(n(),n=void 0,r.clear(),r=PE)}function y(){l||(l=!0,g())}function E(){l&&(l=!1,p())}const C={addNestedSub:c,notifyNestedSubs:d,handleChangeWrapper:m,isSubscribed:v,trySubscribe:y,tryUnsubscribe:E,getListeners:()=>r};return C}var U9=()=>typeof window<"u"&&typeof window.document<"u"&&typeof window.document.createElement<"u",q9=U9(),W9=()=>typeof navigator<"u"&&navigator.product==="ReactNative",k9=W9(),G9=()=>q9||k9?u.useLayoutEffect:u.useEffect,K9=G9(),ph=Symbol.for("react-redux-context"),yh=typeof globalThis<"u"?globalThis:{};function X9(){if(!u.createContext)return{};const e=yh[ph]??(yh[ph]=new Map);let t=e.get(u.createContext);return t||(t=u.createContext(null),e.set(u.createContext,t)),t}var ii=X9();function Y9(e){const{children:t,context:n,serverState:r,store:o}=e,l=u.useMemo(()=>{const m=V9(o);return{store:o,subscription:m,getServerState:r?()=>r:void 0}},[o,r]),c=u.useMemo(()=>o.getState(),[o]);K9(()=>{const{subscription:m}=l;return m.onStateChange=m.notifyNestedSubs,m.trySubscribe(),c!==o.getState()&&m.notifyNestedSubs(),()=>{m.tryUnsubscribe(),m.onStateChange=void 0}},[l,c]);const d=n||ii;return u.createElement(d.Provider,{value:l},t)}var Q9=Y9;function Z0(e=ii){return function(){return u.useContext(e)}}var AR=Z0();function IR(e=ii){const t=e===ii?AR:Z0(e),n=()=>{const{store:r}=t();return r};return Object.assign(n,{withTypes:()=>n}),n}var Z9=IR();function J9(e=ii){const t=e===ii?Z9:IR(e),n=()=>t().dispatch;return Object.assign(n,{withTypes:()=>n}),n}var eV=J9(),tV=(e,t)=>e===t;function nV(e=ii){const t=e===ii?AR:Z0(e),n=(r,o={})=>{const{equalityFn:l=tV}=typeof o=="function"?{equalityFn:o}:o,c=t(),{store:d,subscription:m,getServerState:v}=c;u.useRef(!0);const g=u.useCallback({[r.name](y){return r(y)}}[r.name],[r]),p=L9.useSyncExternalStoreWithSelector(m.addNestedSub,d.getState,v||d.getState,g,l);return u.useDebugValue(p),p};return Object.assign(n,{withTypes:()=>n}),n}var rV=nV();function jR(e,t){return function(){return e.apply(t,arguments)}}const{toString:aV}=Object.prototype,{getPrototypeOf:J0}=Object,lm=(e=>t=>{const n=aV.call(t);return e[n]||(e[n]=n.slice(8,-1).toLowerCase())})(Object.create(null)),za=e=>(e=e.toLowerCase(),t=>lm(t)===e),sm=e=>t=>typeof t===e,{isArray:ls}=Array,Hc=sm("undefined");function oV(e){return e!==null&&!Hc(e)&&e.constructor!==null&&!Hc(e.constructor)&&Qr(e.constructor.isBuffer)&&e.constructor.isBuffer(e)}const PR=za("ArrayBuffer");function iV(e){let t;return typeof ArrayBuffer<"u"&&ArrayBuffer.isView?t=ArrayBuffer.isView(e):t=e&&e.buffer&&PR(e.buffer),t}const lV=sm("string"),Qr=sm("function"),zR=sm("number"),cm=e=>e!==null&&typeof e=="object",sV=e=>e===!0||e===!1,Vf=e=>{if(lm(e)!=="object")return!1;const t=J0(e);return(t===null||t===Object.prototype||Object.getPrototypeOf(t)===null)&&!(Symbol.toStringTag in e)&&!(Symbol.iterator in e)},cV=za("Date"),uV=za("File"),fV=za("Blob"),dV=za("FileList"),mV=e=>cm(e)&&Qr(e.pipe),vV=e=>{let t;return e&&(typeof FormData=="function"&&e instanceof FormData||Qr(e.append)&&((t=lm(e))==="formdata"||t==="object"&&Qr(e.toString)&&e.toString()==="[object FormData]"))},gV=za("URLSearchParams"),[hV,pV,yV,bV]=["ReadableStream","Request","Response","Headers"].map(za),SV=e=>e.trim?e.trim():e.replace(/^[\s\uFEFF\xA0]+|[\s\uFEFF\xA0]+$/g,"");function ou(e,t,{allOwnKeys:n=!1}={}){if(e===null||typeof e>"u")return;let r,o;if(typeof e!="object"&&(e=[e]),ls(e))for(r=0,o=e.length;r<o;r++)t.call(null,e[r],r,e);else{const l=n?Object.getOwnPropertyNames(e):Object.keys(e),c=l.length;let d;for(r=0;r<c;r++)d=l[r],t.call(null,e[d],d,e)}}function DR(e,t){t=t.toLowerCase();const n=Object.keys(e);let r=n.length,o;for(;r-- >0;)if(o=n[r],t===o.toLowerCase())return o;return null}const Pi=typeof globalThis<"u"?globalThis:typeof self<"u"?self:typeof window<"u"?window:global,BR=e=>!Hc(e)&&e!==Pi;function _p(){const{caseless:e}=BR(this)&&this||{},t={},n=(r,o)=>{const l=e&&DR(t,o)||o;Vf(t[l])&&Vf(r)?t[l]=_p(t[l],r):Vf(r)?t[l]=_p({},r):ls(r)?t[l]=r.slice():t[l]=r};for(let r=0,o=arguments.length;r<o;r++)arguments[r]&&ou(arguments[r],n);return t}const CV=(e,t,n,{allOwnKeys:r}={})=>(ou(t,(o,l)=>{n&&Qr(o)?e[l]=jR(o,n):e[l]=o},{allOwnKeys:r}),e),xV=e=>(e.charCodeAt(0)===65279&&(e=e.slice(1)),e),EV=(e,t,n,r)=>{e.prototype=Object.create(t.prototype,r),e.prototype.constructor=e,Object.defineProperty(e,"super",{value:t.prototype}),n&&Object.assign(e.prototype,n)},wV=(e,t,n,r)=>{let o,l,c;const d={};if(t=t||{},e==null)return t;do{for(o=Object.getOwnPropertyNames(e),l=o.length;l-- >0;)c=o[l],(!r||r(c,e,t))&&!d[c]&&(t[c]=e[c],d[c]=!0);e=n!==!1&&J0(e)}while(e&&(!n||n(e,t))&&e!==Object.prototype);return t},$V=(e,t,n)=>{e=String(e),(n===void 0||n>e.length)&&(n=e.length),n-=t.length;const r=e.indexOf(t,n);return r!==-1&&r===n},OV=e=>{if(!e)return null;if(ls(e))return e;let t=e.length;if(!zR(t))return null;const n=new Array(t);for(;t-- >0;)n[t]=e[t];return n},RV=(e=>t=>e&&t instanceof e)(typeof Uint8Array<"u"&&J0(Uint8Array)),TV=(e,t)=>{const r=(e&&e[Symbol.iterator]).call(e);let o;for(;(o=r.next())&&!o.done;){const l=o.value;t.call(e,l[0],l[1])}},MV=(e,t)=>{let n;const r=[];for(;(n=e.exec(t))!==null;)r.push(n);return r},_V=za("HTMLFormElement"),NV=e=>e.toLowerCase().replace(/[-_\s]([a-z\d])(\w*)/g,function(n,r,o){return r.toUpperCase()+o}),zE=(({hasOwnProperty:e})=>(t,n)=>e.call(t,n))(Object.prototype),AV=za("RegExp"),LR=(e,t)=>{const n=Object.getOwnPropertyDescriptors(e),r={};ou(n,(o,l)=>{let c;(c=t(o,l,e))!==!1&&(r[l]=c||o)}),Object.defineProperties(e,r)},IV=e=>{LR(e,(t,n)=>{if(Qr(e)&&["arguments","caller","callee"].indexOf(n)!==-1)return!1;const r=e[n];if(Qr(r)){if(t.enumerable=!1,"writable"in t){t.writable=!1;return}t.set||(t.set=()=>{throw Error("Can not rewrite read-only method '"+n+"'")})}})},jV=(e,t)=>{const n={},r=o=>{o.forEach(l=>{n[l]=!0})};return ls(e)?r(e):r(String(e).split(t)),n},PV=()=>{},zV=(e,t)=>e!=null&&Number.isFinite(e=+e)?e:t;function DV(e){return!!(e&&Qr(e.append)&&e[Symbol.toStringTag]==="FormData"&&e[Symbol.iterator])}const BV=e=>{const t=new Array(10),n=(r,o)=>{if(cm(r)){if(t.indexOf(r)>=0)return;if(!("toJSON"in r)){t[o]=r;const l=ls(r)?[]:{};return ou(r,(c,d)=>{const m=n(c,o+1);!Hc(m)&&(l[d]=m)}),t[o]=void 0,l}}return r};return n(e,0)},LV=za("AsyncFunction"),HV=e=>e&&(cm(e)||Qr(e))&&Qr(e.then)&&Qr(e.catch),HR=((e,t)=>e?setImmediate:t?((n,r)=>(Pi.addEventListener("message",({source:o,data:l})=>{o===Pi&&l===n&&r.length&&r.shift()()},!1),o=>{r.push(o),Pi.postMessage(n,"*")}))(`axios@${Math.random()}`,[]):n=>setTimeout(n))(typeof setImmediate=="function",Qr(Pi.postMessage)),FV=typeof queueMicrotask<"u"?queueMicrotask.bind(Pi):typeof process<"u"&&process.nextTick||HR,Se={isArray:ls,isArrayBuffer:PR,isBuffer:oV,isFormData:vV,isArrayBufferView:iV,isString:lV,isNumber:zR,isBoolean:sV,isObject:cm,isPlainObject:Vf,isReadableStream:hV,isRequest:pV,isResponse:yV,isHeaders:bV,isUndefined:Hc,isDate:cV,isFile:uV,isBlob:fV,isRegExp:AV,isFunction:Qr,isStream:mV,isURLSearchParams:gV,isTypedArray:RV,isFileList:dV,forEach:ou,merge:_p,extend:CV,trim:SV,stripBOM:xV,inherits:EV,toFlatObject:wV,kindOf:lm,kindOfTest:za,endsWith:$V,toArray:OV,forEachEntry:TV,matchAll:MV,isHTMLForm:_V,hasOwnProperty:zE,hasOwnProp:zE,reduceDescriptors:LR,freezeMethods:IV,toObjectSet:jV,toCamelCase:NV,noop:PV,toFiniteNumber:zV,findKey:DR,global:Pi,isContextDefined:BR,isSpecCompliantForm:DV,toJSONObject:BV,isAsyncFn:LV,isThenable:HV,setImmediate:HR,asap:FV};function $t(e,t,n,r,o){Error.call(this),Error.captureStackTrace?Error.captureStackTrace(this,this.constructor):this.stack=new Error().stack,this.message=e,this.name="AxiosError",t&&(this.code=t),n&&(this.config=n),r&&(this.request=r),o&&(this.response=o,this.status=o.status?o.status:null)}Se.inherits($t,Error,{toJSON:function(){return{message:this.message,name:this.name,description:this.description,number:this.number,fileName:this.fileName,lineNumber:this.lineNumber,columnNumber:this.columnNumber,stack:this.stack,config:Se.toJSONObject(this.config),code:this.code,status:this.status}}});const FR=$t.prototype,VR={};["ERR_BAD_OPTION_VALUE","ERR_BAD_OPTION","ECONNABORTED","ETIMEDOUT","ERR_NETWORK","ERR_FR_TOO_MANY_REDIRECTS","ERR_DEPRECATED","ERR_BAD_RESPONSE","ERR_BAD_REQUEST","ERR_CANCELED","ERR_NOT_SUPPORT","ERR_INVALID_URL"].forEach(e=>{VR[e]={value:e}});Object.defineProperties($t,VR);Object.defineProperty(FR,"isAxiosError",{value:!0});$t.from=(e,t,n,r,o,l)=>{const c=Object.create(FR);return Se.toFlatObject(e,c,function(m){return m!==Error.prototype},d=>d!=="isAxiosError"),$t.call(c,e.message,t,n,r,o),c.cause=e,c.name=e.name,l&&Object.assign(c,l),c};const VV=null;function Np(e){return Se.isPlainObject(e)||Se.isArray(e)}function UR(e){return Se.endsWith(e,"[]")?e.slice(0,-2):e}function DE(e,t,n){return e?e.concat(t).map(function(o,l){return o=UR(o),!n&&l?"["+o+"]":o}).join(n?".":""):t}function UV(e){return Se.isArray(e)&&!e.some(Np)}const qV=Se.toFlatObject(Se,{},null,function(t){return/^is[A-Z]/.test(t)});function um(e,t,n){if(!Se.isObject(e))throw new TypeError("target must be an object");t=t||new FormData,n=Se.toFlatObject(n,{metaTokens:!0,dots:!1,indexes:!1},!1,function(x,b){return!Se.isUndefined(b[x])});const r=n.metaTokens,o=n.visitor||g,l=n.dots,c=n.indexes,m=(n.Blob||typeof Blob<"u"&&Blob)&&Se.isSpecCompliantForm(t);if(!Se.isFunction(o))throw new TypeError("visitor must be a function");function v(C){if(C===null)return"";if(Se.isDate(C))return C.toISOString();if(!m&&Se.isBlob(C))throw new $t("Blob is not supported. Use a Buffer instead.");return Se.isArrayBuffer(C)||Se.isTypedArray(C)?m&&typeof Blob=="function"?new Blob([C]):Buffer.from(C):C}function g(C,x,b){let w=C;if(C&&!b&&typeof C=="object"){if(Se.endsWith(x,"{}"))x=r?x:x.slice(0,-2),C=JSON.stringify(C);else if(Se.isArray(C)&&UV(C)||(Se.isFileList(C)||Se.endsWith(x,"[]"))&&(w=Se.toArray(C)))return x=UR(x),w.forEach(function(O,T){!(Se.isUndefined(O)||O===null)&&t.append(c===!0?DE([x],T,l):c===null?x:x+"[]",v(O))}),!1}return Np(C)?!0:(t.append(DE(b,x,l),v(C)),!1)}const p=[],y=Object.assign(qV,{defaultVisitor:g,convertValue:v,isVisitable:Np});function E(C,x){if(!Se.isUndefined(C)){if(p.indexOf(C)!==-1)throw Error("Circular reference detected in "+x.join("."));p.push(C),Se.forEach(C,function(w,$){(!(Se.isUndefined(w)||w===null)&&o.call(t,w,Se.isString($)?$.trim():$,x,y))===!0&&E(w,x?x.concat($):[$])}),p.pop()}}if(!Se.isObject(e))throw new TypeError("data must be an object");return E(e),t}function BE(e){const t={"!":"%21","'":"%27","(":"%28",")":"%29","~":"%7E","%20":"+","%00":"\0"};return encodeURIComponent(e).replace(/[!'()~]|%20|%00/g,function(r){return t[r]})}function ey(e,t){this._pairs=[],e&&um(e,this,t)}const qR=ey.prototype;qR.append=function(t,n){this._pairs.push([t,n])};qR.toString=function(t){const n=t?function(r){return t.call(this,r,BE)}:BE;return this._pairs.map(function(o){return n(o[0])+"="+n(o[1])},"").join("&")};function WV(e){return encodeURIComponent(e).replace(/%3A/gi,":").replace(/%24/g,"$").replace(/%2C/gi,",").replace(/%20/g,"+").replace(/%5B/gi,"[").replace(/%5D/gi,"]")}function WR(e,t,n){if(!t)return e;const r=n&&n.encode||WV;Se.isFunction(n)&&(n={serialize:n});const o=n&&n.serialize;let l;if(o?l=o(t,n):l=Se.isURLSearchParams(t)?t.toString():new ey(t,n).toString(r),l){const c=e.indexOf("#");c!==-1&&(e=e.slice(0,c)),e+=(e.indexOf("?")===-1?"?":"&")+l}return e}class LE{constructor(){this.handlers=[]}use(t,n,r){return this.handlers.push({fulfilled:t,rejected:n,synchronous:r?r.synchronous:!1,runWhen:r?r.runWhen:null}),this.handlers.length-1}eject(t){this.handlers[t]&&(this.handlers[t]=null)}clear(){this.handlers&&(this.handlers=[])}forEach(t){Se.forEach(this.handlers,function(r){r!==null&&t(r)})}}const kR={silentJSONParsing:!0,forcedJSONParsing:!0,clarifyTimeoutError:!1},kV=typeof URLSearchParams<"u"?URLSearchParams:ey,GV=typeof FormData<"u"?FormData:null,KV=typeof Blob<"u"?Blob:null,XV={isBrowser:!0,classes:{URLSearchParams:kV,FormData:GV,Blob:KV},protocols:["http","https","file","blob","url","data"]},ty=typeof window<"u"&&typeof document<"u",Ap=typeof navigator=="object"&&navigator||void 0,YV=ty&&(!Ap||["ReactNative","NativeScript","NS"].indexOf(Ap.product)<0),QV=typeof WorkerGlobalScope<"u"&&self instanceof WorkerGlobalScope&&typeof self.importScripts=="function",ZV=ty&&window.location.href||"http://localhost",JV=Object.freeze(Object.defineProperty({__proto__:null,hasBrowserEnv:ty,hasStandardBrowserEnv:YV,hasStandardBrowserWebWorkerEnv:QV,navigator:Ap,origin:ZV},Symbol.toStringTag,{value:"Module"})),Sr={...JV,...XV};function eU(e,t){return um(e,new Sr.classes.URLSearchParams,Object.assign({visitor:function(n,r,o,l){return Sr.isNode&&Se.isBuffer(n)?(this.append(r,n.toString("base64")),!1):l.defaultVisitor.apply(this,arguments)}},t))}function tU(e){return Se.matchAll(/\w+|\[(\w*)]/g,e).map(t=>t[0]==="[]"?"":t[1]||t[0])}function nU(e){const t={},n=Object.keys(e);let r;const o=n.length;let l;for(r=0;r<o;r++)l=n[r],t[l]=e[l];return t}function GR(e){function t(n,r,o,l){let c=n[l++];if(c==="__proto__")return!0;const d=Number.isFinite(+c),m=l>=n.length;return c=!c&&Se.isArray(o)?o.length:c,m?(Se.hasOwnProp(o,c)?o[c]=[o[c],r]:o[c]=r,!d):((!o[c]||!Se.isObject(o[c]))&&(o[c]=[]),t(n,r,o[c],l)&&Se.isArray(o[c])&&(o[c]=nU(o[c])),!d)}if(Se.isFormData(e)&&Se.isFunction(e.entries)){const n={};return Se.forEachEntry(e,(r,o)=>{t(tU(r),o,n,0)}),n}return null}function rU(e,t,n){if(Se.isString(e))try{return(t||JSON.parse)(e),Se.trim(e)}catch(r){if(r.name!=="SyntaxError")throw r}return(n||JSON.stringify)(e)}const iu={transitional:kR,adapter:["xhr","http","fetch"],transformRequest:[function(t,n){const r=n.getContentType()||"",o=r.indexOf("application/json")>-1,l=Se.isObject(t);if(l&&Se.isHTMLForm(t)&&(t=new FormData(t)),Se.isFormData(t))return o?JSON.stringify(GR(t)):t;if(Se.isArrayBuffer(t)||Se.isBuffer(t)||Se.isStream(t)||Se.isFile(t)||Se.isBlob(t)||Se.isReadableStream(t))return t;if(Se.isArrayBufferView(t))return t.buffer;if(Se.isURLSearchParams(t))return n.setContentType("application/x-www-form-urlencoded;charset=utf-8",!1),t.toString();let d;if(l){if(r.indexOf("application/x-www-form-urlencoded")>-1)return eU(t,this.formSerializer).toString();if((d=Se.isFileList(t))||r.indexOf("multipart/form-data")>-1){const m=this.env&&this.env.FormData;return um(d?{"files[]":t}:t,m&&new m,this.formSerializer)}}return l||o?(n.setContentType("application/json",!1),rU(t)):t}],transformResponse:[function(t){const n=this.transitional||iu.transitional,r=n&&n.forcedJSONParsing,o=this.responseType==="json";if(Se.isResponse(t)||Se.isReadableStream(t))return t;if(t&&Se.isString(t)&&(r&&!this.responseType||o)){const c=!(n&&n.silentJSONParsing)&&o;try{return JSON.parse(t)}catch(d){if(c)throw d.name==="SyntaxError"?$t.from(d,$t.ERR_BAD_RESPONSE,this,null,this.response):d}}return t}],timeout:0,xsrfCookieName:"XSRF-TOKEN",xsrfHeaderName:"X-XSRF-TOKEN",maxContentLength:-1,maxBodyLength:-1,env:{FormData:Sr.classes.FormData,Blob:Sr.classes.Blob},validateStatus:function(t){return t>=200&&t<300},headers:{common:{Accept:"application/json, text/plain, */*","Content-Type":void 0}}};Se.forEach(["delete","get","head","post","put","patch"],e=>{iu.headers[e]={}});const aU=Se.toObjectSet(["age","authorization","content-length","content-type","etag","expires","from","host","if-modified-since","if-unmodified-since","last-modified","location","max-forwards","proxy-authorization","referer","retry-after","user-agent"]),oU=e=>{const t={};let n,r,o;return e&&e.split(`
`).forEach(function(c){o=c.indexOf(":"),n=c.substring(0,o).trim().toLowerCase(),r=c.substring(o+1).trim(),!(!n||t[n]&&aU[n])&&(n==="set-cookie"?t[n]?t[n].push(r):t[n]=[r]:t[n]=t[n]?t[n]+", "+r:r)}),t},HE=Symbol("internals");function fc(e){return e&&String(e).trim().toLowerCase()}function Uf(e){return e===!1||e==null?e:Se.isArray(e)?e.map(Uf):String(e)}function iU(e){const t=Object.create(null),n=/([^\s,;=]+)\s*(?:=\s*([^,;]+))?/g;let r;for(;r=n.exec(e);)t[r[1]]=r[2];return t}const lU=e=>/^[-_a-zA-Z0-9^`|~,!#$%&'*+.]+$/.test(e.trim());function bh(e,t,n,r,o){if(Se.isFunction(r))return r.call(this,t,n);if(o&&(t=n),!!Se.isString(t)){if(Se.isString(r))return t.indexOf(r)!==-1;if(Se.isRegExp(r))return r.test(t)}}function sU(e){return e.trim().toLowerCase().replace(/([a-z\d])(\w*)/g,(t,n,r)=>n.toUpperCase()+r)}function cU(e,t){const n=Se.toCamelCase(" "+t);["get","set","has"].forEach(r=>{Object.defineProperty(e,r+n,{value:function(o,l,c){return this[r].call(this,t,o,l,c)},configurable:!0})})}let jr=class{constructor(t){t&&this.set(t)}set(t,n,r){const o=this;function l(d,m,v){const g=fc(m);if(!g)throw new Error("header name must be a non-empty string");const p=Se.findKey(o,g);(!p||o[p]===void 0||v===!0||v===void 0&&o[p]!==!1)&&(o[p||m]=Uf(d))}const c=(d,m)=>Se.forEach(d,(v,g)=>l(v,g,m));if(Se.isPlainObject(t)||t instanceof this.constructor)c(t,n);else if(Se.isString(t)&&(t=t.trim())&&!lU(t))c(oU(t),n);else if(Se.isHeaders(t))for(const[d,m]of t.entries())l(m,d,r);else t!=null&&l(n,t,r);return this}get(t,n){if(t=fc(t),t){const r=Se.findKey(this,t);if(r){const o=this[r];if(!n)return o;if(n===!0)return iU(o);if(Se.isFunction(n))return n.call(this,o,r);if(Se.isRegExp(n))return n.exec(o);throw new TypeError("parser must be boolean|regexp|function")}}}has(t,n){if(t=fc(t),t){const r=Se.findKey(this,t);return!!(r&&this[r]!==void 0&&(!n||bh(this,this[r],r,n)))}return!1}delete(t,n){const r=this;let o=!1;function l(c){if(c=fc(c),c){const d=Se.findKey(r,c);d&&(!n||bh(r,r[d],d,n))&&(delete r[d],o=!0)}}return Se.isArray(t)?t.forEach(l):l(t),o}clear(t){const n=Object.keys(this);let r=n.length,o=!1;for(;r--;){const l=n[r];(!t||bh(this,this[l],l,t,!0))&&(delete this[l],o=!0)}return o}normalize(t){const n=this,r={};return Se.forEach(this,(o,l)=>{const c=Se.findKey(r,l);if(c){n[c]=Uf(o),delete n[l];return}const d=t?sU(l):String(l).trim();d!==l&&delete n[l],n[d]=Uf(o),r[d]=!0}),this}concat(...t){return this.constructor.concat(this,...t)}toJSON(t){const n=Object.create(null);return Se.forEach(this,(r,o)=>{r!=null&&r!==!1&&(n[o]=t&&Se.isArray(r)?r.join(", "):r)}),n}[Symbol.iterator](){return Object.entries(this.toJSON())[Symbol.iterator]()}toString(){return Object.entries(this.toJSON()).map(([t,n])=>t+": "+n).join(`
`)}get[Symbol.toStringTag](){return"AxiosHeaders"}static from(t){return t instanceof this?t:new this(t)}static concat(t,...n){const r=new this(t);return n.forEach(o=>r.set(o)),r}static accessor(t){const r=(this[HE]=this[HE]={accessors:{}}).accessors,o=this.prototype;function l(c){const d=fc(c);r[d]||(cU(o,c),r[d]=!0)}return Se.isArray(t)?t.forEach(l):l(t),this}};jr.accessor(["Content-Type","Content-Length","Accept","Accept-Encoding","User-Agent","Authorization"]);Se.reduceDescriptors(jr.prototype,({value:e},t)=>{let n=t[0].toUpperCase()+t.slice(1);return{get:()=>e,set(r){this[n]=r}}});Se.freezeMethods(jr);function Sh(e,t){const n=this||iu,r=t||n,o=jr.from(r.headers);let l=r.data;return Se.forEach(e,function(d){l=d.call(n,l,o.normalize(),t?t.status:void 0)}),o.normalize(),l}function KR(e){return!!(e&&e.__CANCEL__)}function ss(e,t,n){$t.call(this,e??"canceled",$t.ERR_CANCELED,t,n),this.name="CanceledError"}Se.inherits(ss,$t,{__CANCEL__:!0});function XR(e,t,n){const r=n.config.validateStatus;!n.status||!r||r(n.status)?e(n):t(new $t("Request failed with status code "+n.status,[$t.ERR_BAD_REQUEST,$t.ERR_BAD_RESPONSE][Math.floor(n.status/100)-4],n.config,n.request,n))}function uU(e){const t=/^([-+\w]{1,25})(:?\/\/|:)/.exec(e);return t&&t[1]||""}function fU(e,t){e=e||10;const n=new Array(e),r=new Array(e);let o=0,l=0,c;return t=t!==void 0?t:1e3,function(m){const v=Date.now(),g=r[l];c||(c=v),n[o]=m,r[o]=v;let p=l,y=0;for(;p!==o;)y+=n[p++],p=p%e;if(o=(o+1)%e,o===l&&(l=(l+1)%e),v-c<t)return;const E=g&&v-g;return E?Math.round(y*1e3/E):void 0}}function dU(e,t){let n=0,r=1e3/t,o,l;const c=(v,g=Date.now())=>{n=g,o=null,l&&(clearTimeout(l),l=null),e.apply(null,v)};return[(...v)=>{const g=Date.now(),p=g-n;p>=r?c(v,g):(o=v,l||(l=setTimeout(()=>{l=null,c(o)},r-p)))},()=>o&&c(o)]}const md=(e,t,n=3)=>{let r=0;const o=fU(50,250);return dU(l=>{const c=l.loaded,d=l.lengthComputable?l.total:void 0,m=c-r,v=o(m),g=c<=d;r=c;const p={loaded:c,total:d,progress:d?c/d:void 0,bytes:m,rate:v||void 0,estimated:v&&d&&g?(d-c)/v:void 0,event:l,lengthComputable:d!=null,[t?"download":"upload"]:!0};e(p)},n)},FE=(e,t)=>{const n=e!=null;return[r=>t[0]({lengthComputable:n,total:e,loaded:r}),t[1]]},VE=e=>(...t)=>Se.asap(()=>e(...t)),mU=Sr.hasStandardBrowserEnv?((e,t)=>n=>(n=new URL(n,Sr.origin),e.protocol===n.protocol&&e.host===n.host&&(t||e.port===n.port)))(new URL(Sr.origin),Sr.navigator&&/(msie|trident)/i.test(Sr.navigator.userAgent)):()=>!0,vU=Sr.hasStandardBrowserEnv?{write(e,t,n,r,o,l){const c=[e+"="+encodeURIComponent(t)];Se.isNumber(n)&&c.push("expires="+new Date(n).toGMTString()),Se.isString(r)&&c.push("path="+r),Se.isString(o)&&c.push("domain="+o),l===!0&&c.push("secure"),document.cookie=c.join("; ")},read(e){const t=document.cookie.match(new RegExp("(^|;\\s*)("+e+")=([^;]*)"));return t?decodeURIComponent(t[3]):null},remove(e){this.write(e,"",Date.now()-864e5)}}:{write(){},read(){return null},remove(){}};function gU(e){return/^([a-z][a-z\d+\-.]*:)?\/\//i.test(e)}function hU(e,t){return t?e.replace(/\/?\/$/,"")+"/"+t.replace(/^\/+/,""):e}function YR(e,t,n){let r=!gU(t);return e&&(r||n==!1)?hU(e,t):t}const UE=e=>e instanceof jr?{...e}:e;function Ui(e,t){t=t||{};const n={};function r(v,g,p,y){return Se.isPlainObject(v)&&Se.isPlainObject(g)?Se.merge.call({caseless:y},v,g):Se.isPlainObject(g)?Se.merge({},g):Se.isArray(g)?g.slice():g}function o(v,g,p,y){if(Se.isUndefined(g)){if(!Se.isUndefined(v))return r(void 0,v,p,y)}else return r(v,g,p,y)}function l(v,g){if(!Se.isUndefined(g))return r(void 0,g)}function c(v,g){if(Se.isUndefined(g)){if(!Se.isUndefined(v))return r(void 0,v)}else return r(void 0,g)}function d(v,g,p){if(p in t)return r(v,g);if(p in e)return r(void 0,v)}const m={url:l,method:l,data:l,baseURL:c,transformRequest:c,transformResponse:c,paramsSerializer:c,timeout:c,timeoutMessage:c,withCredentials:c,withXSRFToken:c,adapter:c,responseType:c,xsrfCookieName:c,xsrfHeaderName:c,onUploadProgress:c,onDownloadProgress:c,decompress:c,maxContentLength:c,maxBodyLength:c,beforeRedirect:c,transport:c,httpAgent:c,httpsAgent:c,cancelToken:c,socketPath:c,responseEncoding:c,validateStatus:d,headers:(v,g,p)=>o(UE(v),UE(g),p,!0)};return Se.forEach(Object.keys(Object.assign({},e,t)),function(g){const p=m[g]||o,y=p(e[g],t[g],g);Se.isUndefined(y)&&p!==d||(n[g]=y)}),n}const QR=e=>{const t=Ui({},e);let{data:n,withXSRFToken:r,xsrfHeaderName:o,xsrfCookieName:l,headers:c,auth:d}=t;t.headers=c=jr.from(c),t.url=WR(YR(t.baseURL,t.url,t.allowAbsoluteUrls),e.params,e.paramsSerializer),d&&c.set("Authorization","Basic "+btoa((d.username||"")+":"+(d.password?unescape(encodeURIComponent(d.password)):"")));let m;if(Se.isFormData(n)){if(Sr.hasStandardBrowserEnv||Sr.hasStandardBrowserWebWorkerEnv)c.setContentType(void 0);else if((m=c.getContentType())!==!1){const[v,...g]=m?m.split(";").map(p=>p.trim()).filter(Boolean):[];c.setContentType([v||"multipart/form-data",...g].join("; "))}}if(Sr.hasStandardBrowserEnv&&(r&&Se.isFunction(r)&&(r=r(t)),r||r!==!1&&mU(t.url))){const v=o&&l&&vU.read(l);v&&c.set(o,v)}return t},pU=typeof XMLHttpRequest<"u",yU=pU&&function(e){return new Promise(function(n,r){const o=QR(e);let l=o.data;const c=jr.from(o.headers).normalize();let{responseType:d,onUploadProgress:m,onDownloadProgress:v}=o,g,p,y,E,C;function x(){E&&E(),C&&C(),o.cancelToken&&o.cancelToken.unsubscribe(g),o.signal&&o.signal.removeEventListener("abort",g)}let b=new XMLHttpRequest;b.open(o.method.toUpperCase(),o.url,!0),b.timeout=o.timeout;function w(){if(!b)return;const O=jr.from("getAllResponseHeaders"in b&&b.getAllResponseHeaders()),R={data:!d||d==="text"||d==="json"?b.responseText:b.response,status:b.status,statusText:b.statusText,headers:O,config:e,request:b};XR(function(N){n(N),x()},function(N){r(N),x()},R),b=null}"onloadend"in b?b.onloadend=w:b.onreadystatechange=function(){!b||b.readyState!==4||b.status===0&&!(b.responseURL&&b.responseURL.indexOf("file:")===0)||setTimeout(w)},b.onabort=function(){b&&(r(new $t("Request aborted",$t.ECONNABORTED,e,b)),b=null)},b.onerror=function(){r(new $t("Network Error",$t.ERR_NETWORK,e,b)),b=null},b.ontimeout=function(){let T=o.timeout?"timeout of "+o.timeout+"ms exceeded":"timeout exceeded";const R=o.transitional||kR;o.timeoutErrorMessage&&(T=o.timeoutErrorMessage),r(new $t(T,R.clarifyTimeoutError?$t.ETIMEDOUT:$t.ECONNABORTED,e,b)),b=null},l===void 0&&c.setContentType(null),"setRequestHeader"in b&&Se.forEach(c.toJSON(),function(T,R){b.setRequestHeader(R,T)}),Se.isUndefined(o.withCredentials)||(b.withCredentials=!!o.withCredentials),d&&d!=="json"&&(b.responseType=o.responseType),v&&([y,C]=md(v,!0),b.addEventListener("progress",y)),m&&b.upload&&([p,E]=md(m),b.upload.addEventListener("progress",p),b.upload.addEventListener("loadend",E)),(o.cancelToken||o.signal)&&(g=O=>{b&&(r(!O||O.type?new ss(null,e,b):O),b.abort(),b=null)},o.cancelToken&&o.cancelToken.subscribe(g),o.signal&&(o.signal.aborted?g():o.signal.addEventListener("abort",g)));const $=uU(o.url);if($&&Sr.protocols.indexOf($)===-1){r(new $t("Unsupported protocol "+$+":",$t.ERR_BAD_REQUEST,e));return}b.send(l||null)})},bU=(e,t)=>{const{length:n}=e=e?e.filter(Boolean):[];if(t||n){let r=new AbortController,o;const l=function(v){if(!o){o=!0,d();const g=v instanceof Error?v:this.reason;r.abort(g instanceof $t?g:new ss(g instanceof Error?g.message:g))}};let c=t&&setTimeout(()=>{c=null,l(new $t(`timeout ${t} of ms exceeded`,$t.ETIMEDOUT))},t);const d=()=>{e&&(c&&clearTimeout(c),c=null,e.forEach(v=>{v.unsubscribe?v.unsubscribe(l):v.removeEventListener("abort",l)}),e=null)};e.forEach(v=>v.addEventListener("abort",l));const{signal:m}=r;return m.unsubscribe=()=>Se.asap(d),m}},SU=function*(e,t){let n=e.byteLength;if(n<t){yield e;return}let r=0,o;for(;r<n;)o=r+t,yield e.slice(r,o),r=o},CU=async function*(e,t){for await(const n of xU(e))yield*SU(n,t)},xU=async function*(e){if(e[Symbol.asyncIterator]){yield*e;return}const t=e.getReader();try{for(;;){const{done:n,value:r}=await t.read();if(n)break;yield r}}finally{await t.cancel()}},qE=(e,t,n,r)=>{const o=CU(e,t);let l=0,c,d=m=>{c||(c=!0,r&&r(m))};return new ReadableStream({async pull(m){try{const{done:v,value:g}=await o.next();if(v){d(),m.close();return}let p=g.byteLength;if(n){let y=l+=p;n(y)}m.enqueue(new Uint8Array(g))}catch(v){throw d(v),v}},cancel(m){return d(m),o.return()}},{highWaterMark:2})},fm=typeof fetch=="function"&&typeof Request=="function"&&typeof Response=="function",ZR=fm&&typeof ReadableStream=="function",EU=fm&&(typeof TextEncoder=="function"?(e=>t=>e.encode(t))(new TextEncoder):async e=>new Uint8Array(await new Response(e).arrayBuffer())),JR=(e,...t)=>{try{return!!e(...t)}catch{return!1}},wU=ZR&&JR(()=>{let e=!1;const t=new Request(Sr.origin,{body:new ReadableStream,method:"POST",get duplex(){return e=!0,"half"}}).headers.has("Content-Type");return e&&!t}),WE=64*1024,Ip=ZR&&JR(()=>Se.isReadableStream(new Response("").body)),vd={stream:Ip&&(e=>e.body)};fm&&(e=>{["text","arrayBuffer","blob","formData","stream"].forEach(t=>{!vd[t]&&(vd[t]=Se.isFunction(e[t])?n=>n[t]():(n,r)=>{throw new $t(`Response type '${t}' is not supported`,$t.ERR_NOT_SUPPORT,r)})})})(new Response);const $U=async e=>{if(e==null)return 0;if(Se.isBlob(e))return e.size;if(Se.isSpecCompliantForm(e))return(await new Request(Sr.origin,{method:"POST",body:e}).arrayBuffer()).byteLength;if(Se.isArrayBufferView(e)||Se.isArrayBuffer(e))return e.byteLength;if(Se.isURLSearchParams(e)&&(e=e+""),Se.isString(e))return(await EU(e)).byteLength},OU=async(e,t)=>{const n=Se.toFiniteNumber(e.getContentLength());return n??$U(t)},RU=fm&&(async e=>{let{url:t,method:n,data:r,signal:o,cancelToken:l,timeout:c,onDownloadProgress:d,onUploadProgress:m,responseType:v,headers:g,withCredentials:p="same-origin",fetchOptions:y}=QR(e);v=v?(v+"").toLowerCase():"text";let E=bU([o,l&&l.toAbortSignal()],c),C;const x=E&&E.unsubscribe&&(()=>{E.unsubscribe()});let b;try{if(m&&wU&&n!=="get"&&n!=="head"&&(b=await OU(g,r))!==0){let R=new Request(t,{method:"POST",body:r,duplex:"half"}),M;if(Se.isFormData(r)&&(M=R.headers.get("content-type"))&&g.setContentType(M),R.body){const[N,z]=FE(b,md(VE(m)));r=qE(R.body,WE,N,z)}}Se.isString(p)||(p=p?"include":"omit");const w="credentials"in Request.prototype;C=new Request(t,{...y,signal:E,method:n.toUpperCase(),headers:g.normalize().toJSON(),body:r,duplex:"half",credentials:w?p:void 0});let $=await fetch(C);const O=Ip&&(v==="stream"||v==="response");if(Ip&&(d||O&&x)){const R={};["status","statusText","headers"].forEach(P=>{R[P]=$[P]});const M=Se.toFiniteNumber($.headers.get("content-length")),[N,z]=d&&FE(M,md(VE(d),!0))||[];$=new Response(qE($.body,WE,N,()=>{z&&z(),x&&x()}),R)}v=v||"text";let T=await vd[Se.findKey(vd,v)||"text"]($,e);return!O&&x&&x(),await new Promise((R,M)=>{XR(R,M,{data:T,headers:jr.from($.headers),status:$.status,statusText:$.statusText,config:e,request:C})})}catch(w){throw x&&x(),w&&w.name==="TypeError"&&/fetch/i.test(w.message)?Object.assign(new $t("Network Error",$t.ERR_NETWORK,e,C),{cause:w.cause||w}):$t.from(w,w&&w.code,e,C)}}),jp={http:VV,xhr:yU,fetch:RU};Se.forEach(jp,(e,t)=>{if(e){try{Object.defineProperty(e,"name",{value:t})}catch{}Object.defineProperty(e,"adapterName",{value:t})}});const kE=e=>`- ${e}`,TU=e=>Se.isFunction(e)||e===null||e===!1,eT={getAdapter:e=>{e=Se.isArray(e)?e:[e];const{length:t}=e;let n,r;const o={};for(let l=0;l<t;l++){n=e[l];let c;if(r=n,!TU(n)&&(r=jp[(c=String(n)).toLowerCase()],r===void 0))throw new $t(`Unknown adapter '${c}'`);if(r)break;o[c||"#"+l]=r}if(!r){const l=Object.entries(o).map(([d,m])=>`adapter ${d} `+(m===!1?"is not supported by the environment":"is not available in the build"));let c=t?l.length>1?`since :
`+l.map(kE).join(`
`):" "+kE(l[0]):"as no adapter specified";throw new $t("There is no suitable adapter to dispatch the request "+c,"ERR_NOT_SUPPORT")}return r},adapters:jp};function Ch(e){if(e.cancelToken&&e.cancelToken.throwIfRequested(),e.signal&&e.signal.aborted)throw new ss(null,e)}function GE(e){return Ch(e),e.headers=jr.from(e.headers),e.data=Sh.call(e,e.transformRequest),["post","put","patch"].indexOf(e.method)!==-1&&e.headers.setContentType("application/x-www-form-urlencoded",!1),eT.getAdapter(e.adapter||iu.adapter)(e).then(function(r){return Ch(e),r.data=Sh.call(e,e.transformResponse,r),r.headers=jr.from(r.headers),r},function(r){return KR(r)||(Ch(e),r&&r.response&&(r.response.data=Sh.call(e,e.transformResponse,r.response),r.response.headers=jr.from(r.response.headers))),Promise.reject(r)})}const tT="1.8.4",dm={};["object","boolean","number","function","string","symbol"].forEach((e,t)=>{dm[e]=function(r){return typeof r===e||"a"+(t<1?"n ":" ")+e}});const KE={};dm.transitional=function(t,n,r){function o(l,c){return"[Axios v"+tT+"] Transitional option '"+l+"'"+c+(r?". "+r:"")}return(l,c,d)=>{if(t===!1)throw new $t(o(c," has been removed"+(n?" in "+n:"")),$t.ERR_DEPRECATED);return n&&!KE[c]&&(KE[c]=!0,console.warn(o(c," has been deprecated since v"+n+" and will be removed in the near future"))),t?t(l,c,d):!0}};dm.spelling=function(t){return(n,r)=>(console.warn(`${r} is likely a misspelling of ${t}`),!0)};function MU(e,t,n){if(typeof e!="object")throw new $t("options must be an object",$t.ERR_BAD_OPTION_VALUE);const r=Object.keys(e);let o=r.length;for(;o-- >0;){const l=r[o],c=t[l];if(c){const d=e[l],m=d===void 0||c(d,l,e);if(m!==!0)throw new $t("option "+l+" must be "+m,$t.ERR_BAD_OPTION_VALUE);continue}if(n!==!0)throw new $t("Unknown option "+l,$t.ERR_BAD_OPTION)}}const qf={assertOptions:MU,validators:dm},qa=qf.validators;let Di=class{constructor(t){this.defaults=t,this.interceptors={request:new LE,response:new LE}}async request(t,n){try{return await this._request(t,n)}catch(r){if(r instanceof Error){let o={};Error.captureStackTrace?Error.captureStackTrace(o):o=new Error;const l=o.stack?o.stack.replace(/^.+\n/,""):"";try{r.stack?l&&!String(r.stack).endsWith(l.replace(/^.+\n.+\n/,""))&&(r.stack+=`
`+l):r.stack=l}catch{}}throw r}}_request(t,n){typeof t=="string"?(n=n||{},n.url=t):n=t||{},n=Ui(this.defaults,n);const{transitional:r,paramsSerializer:o,headers:l}=n;r!==void 0&&qf.assertOptions(r,{silentJSONParsing:qa.transitional(qa.boolean),forcedJSONParsing:qa.transitional(qa.boolean),clarifyTimeoutError:qa.transitional(qa.boolean)},!1),o!=null&&(Se.isFunction(o)?n.paramsSerializer={serialize:o}:qf.assertOptions(o,{encode:qa.function,serialize:qa.function},!0)),n.allowAbsoluteUrls!==void 0||(this.defaults.allowAbsoluteUrls!==void 0?n.allowAbsoluteUrls=this.defaults.allowAbsoluteUrls:n.allowAbsoluteUrls=!0),qf.assertOptions(n,{baseUrl:qa.spelling("baseURL"),withXsrfToken:qa.spelling("withXSRFToken")},!0),n.method=(n.method||this.defaults.method||"get").toLowerCase();let c=l&&Se.merge(l.common,l[n.method]);l&&Se.forEach(["delete","get","head","post","put","patch","common"],C=>{delete l[C]}),n.headers=jr.concat(c,l);const d=[];let m=!0;this.interceptors.request.forEach(function(x){typeof x.runWhen=="function"&&x.runWhen(n)===!1||(m=m&&x.synchronous,d.unshift(x.fulfilled,x.rejected))});const v=[];this.interceptors.response.forEach(function(x){v.push(x.fulfilled,x.rejected)});let g,p=0,y;if(!m){const C=[GE.bind(this),void 0];for(C.unshift.apply(C,d),C.push.apply(C,v),y=C.length,g=Promise.resolve(n);p<y;)g=g.then(C[p++],C[p++]);return g}y=d.length;let E=n;for(p=0;p<y;){const C=d[p++],x=d[p++];try{E=C(E)}catch(b){x.call(this,b);break}}try{g=GE.call(this,E)}catch(C){return Promise.reject(C)}for(p=0,y=v.length;p<y;)g=g.then(v[p++],v[p++]);return g}getUri(t){t=Ui(this.defaults,t);const n=YR(t.baseURL,t.url,t.allowAbsoluteUrls);return WR(n,t.params,t.paramsSerializer)}};Se.forEach(["delete","get","head","options"],function(t){Di.prototype[t]=function(n,r){return this.request(Ui(r||{},{method:t,url:n,data:(r||{}).data}))}});Se.forEach(["post","put","patch"],function(t){function n(r){return function(l,c,d){return this.request(Ui(d||{},{method:t,headers:r?{"Content-Type":"multipart/form-data"}:{},url:l,data:c}))}}Di.prototype[t]=n(),Di.prototype[t+"Form"]=n(!0)});let _U=class nT{constructor(t){if(typeof t!="function")throw new TypeError("executor must be a function.");let n;this.promise=new Promise(function(l){n=l});const r=this;this.promise.then(o=>{if(!r._listeners)return;let l=r._listeners.length;for(;l-- >0;)r._listeners[l](o);r._listeners=null}),this.promise.then=o=>{let l;const c=new Promise(d=>{r.subscribe(d),l=d}).then(o);return c.cancel=function(){r.unsubscribe(l)},c},t(function(l,c,d){r.reason||(r.reason=new ss(l,c,d),n(r.reason))})}throwIfRequested(){if(this.reason)throw this.reason}subscribe(t){if(this.reason){t(this.reason);return}this._listeners?this._listeners.push(t):this._listeners=[t]}unsubscribe(t){if(!this._listeners)return;const n=this._listeners.indexOf(t);n!==-1&&this._listeners.splice(n,1)}toAbortSignal(){const t=new AbortController,n=r=>{t.abort(r)};return this.subscribe(n),t.signal.unsubscribe=()=>this.unsubscribe(n),t.signal}static source(){let t;return{token:new nT(function(o){t=o}),cancel:t}}};function NU(e){return function(n){return e.apply(null,n)}}function AU(e){return Se.isObject(e)&&e.isAxiosError===!0}const Pp={Continue:100,SwitchingProtocols:101,Processing:102,EarlyHints:103,Ok:200,Created:201,Accepted:202,NonAuthoritativeInformation:203,NoContent:204,ResetContent:205,PartialContent:206,MultiStatus:207,AlreadyReported:208,ImUsed:226,MultipleChoices:300,MovedPermanently:301,Found:302,SeeOther:303,NotModified:304,UseProxy:305,Unused:306,TemporaryRedirect:307,PermanentRedirect:308,BadRequest:400,Unauthorized:401,PaymentRequired:402,Forbidden:403,NotFound:404,MethodNotAllowed:405,NotAcceptable:406,ProxyAuthenticationRequired:407,RequestTimeout:408,Conflict:409,Gone:410,LengthRequired:411,PreconditionFailed:412,PayloadTooLarge:413,UriTooLong:414,UnsupportedMediaType:415,RangeNotSatisfiable:416,ExpectationFailed:417,ImATeapot:418,MisdirectedRequest:421,UnprocessableEntity:422,Locked:423,FailedDependency:424,TooEarly:425,UpgradeRequired:426,PreconditionRequired:428,TooManyRequests:429,RequestHeaderFieldsTooLarge:431,UnavailableForLegalReasons:451,InternalServerError:500,NotImplemented:501,BadGateway:502,ServiceUnavailable:503,GatewayTimeout:504,HttpVersionNotSupported:505,VariantAlsoNegotiates:506,InsufficientStorage:507,LoopDetected:508,NotExtended:510,NetworkAuthenticationRequired:511};Object.entries(Pp).forEach(([e,t])=>{Pp[t]=e});function rT(e){const t=new Di(e),n=jR(Di.prototype.request,t);return Se.extend(n,Di.prototype,t,{allOwnKeys:!0}),Se.extend(n,t,null,{allOwnKeys:!0}),n.create=function(o){return rT(Ui(e,o))},n}const hn=rT(iu);hn.Axios=Di;hn.CanceledError=ss;hn.CancelToken=_U;hn.isCancel=KR;hn.VERSION=tT;hn.toFormData=um;hn.AxiosError=$t;hn.Cancel=hn.CanceledError;hn.all=function(t){return Promise.all(t)};hn.spread=NU;hn.isAxiosError=AU;hn.mergeConfig=Ui;hn.AxiosHeaders=jr;hn.formToJSON=e=>GR(Se.isHTMLForm(e)?new FormData(e):e);hn.getAdapter=eT.getAdapter;hn.HttpStatusCode=Pp;hn.default=hn;const{Axios:gW,AxiosError:hW,CanceledError:pW,isCancel:yW,CancelToken:bW,VERSION:SW,all:CW,Cancel:xW,isAxiosError:EW,spread:wW,toFormData:$W,AxiosHeaders:OW,HttpStatusCode:RW,formToJSON:TW,getAdapter:MW,mergeConfig:_W}=hn;function mr(e){return`Minified Redux error #${e}; visit https://redux.js.org/Errors?code=${e} for the full message or use the non-minified dev environment for full errors. `}var IU=typeof Symbol=="function"&&Symbol.observable||"@@observable",XE=IU,xh=()=>Math.random().toString(36).substring(7).split("").join("."),jU={INIT:`@@redux/INIT${xh()}`,REPLACE:`@@redux/REPLACE${xh()}`,PROBE_UNKNOWN_ACTION:()=>`@@redux/PROBE_UNKNOWN_ACTION${xh()}`},gd=jU;function ny(e){if(typeof e!="object"||e===null)return!1;let t=e;for(;Object.getPrototypeOf(t)!==null;)t=Object.getPrototypeOf(t);return Object.getPrototypeOf(e)===t||Object.getPrototypeOf(e)===null}function aT(e,t,n){if(typeof e!="function")throw new Error(mr(2));if(typeof t=="function"&&typeof n=="function"||typeof n=="function"&&typeof arguments[3]=="function")throw new Error(mr(0));if(typeof t=="function"&&typeof n>"u"&&(n=t,t=void 0),typeof n<"u"){if(typeof n!="function")throw new Error(mr(1));return n(aT)(e,t)}let r=e,o=t,l=new Map,c=l,d=0,m=!1;function v(){c===l&&(c=new Map,l.forEach((b,w)=>{c.set(w,b)}))}function g(){if(m)throw new Error(mr(3));return o}function p(b){if(typeof b!="function")throw new Error(mr(4));if(m)throw new Error(mr(5));let w=!0;v();const $=d++;return c.set($,b),function(){if(w){if(m)throw new Error(mr(6));w=!1,v(),c.delete($),l=null}}}function y(b){if(!ny(b))throw new Error(mr(7));if(typeof b.type>"u")throw new Error(mr(8));if(typeof b.type!="string")throw new Error(mr(17));if(m)throw new Error(mr(9));try{m=!0,o=r(o,b)}finally{m=!1}return(l=c).forEach($=>{$()}),b}function E(b){if(typeof b!="function")throw new Error(mr(10));r=b,y({type:gd.REPLACE})}function C(){const b=p;return{subscribe(w){if(typeof w!="object"||w===null)throw new Error(mr(11));function $(){const T=w;T.next&&T.next(g())}return $(),{unsubscribe:b($)}},[XE](){return this}}}return y({type:gd.INIT}),{dispatch:y,subscribe:p,getState:g,replaceReducer:E,[XE]:C}}function PU(e){Object.keys(e).forEach(t=>{const n=e[t];if(typeof n(void 0,{type:gd.INIT})>"u")throw new Error(mr(12));if(typeof n(void 0,{type:gd.PROBE_UNKNOWN_ACTION()})>"u")throw new Error(mr(13))})}function zU(e){const t=Object.keys(e),n={};for(let l=0;l<t.length;l++){const c=t[l];typeof e[c]=="function"&&(n[c]=e[c])}const r=Object.keys(n);let o;try{PU(n)}catch(l){o=l}return function(c={},d){if(o)throw o;let m=!1;const v={};for(let g=0;g<r.length;g++){const p=r[g],y=n[p],E=c[p],C=y(E,d);if(typeof C>"u")throw d&&d.type,new Error(mr(14));v[p]=C,m=m||C!==E}return m=m||r.length!==Object.keys(c).length,m?v:c}}function hd(...e){return e.length===0?t=>t:e.length===1?e[0]:e.reduce((t,n)=>(...r)=>t(n(...r)))}function DU(...e){return t=>(n,r)=>{const o=t(n,r);let l=()=>{throw new Error(mr(15))};const c={getState:o.getState,dispatch:(m,...v)=>l(m,...v)},d=e.map(m=>m(c));return l=hd(...d)(o.dispatch),{...o,dispatch:l}}}function BU(e){return ny(e)&&"type"in e&&typeof e.type=="string"}var oT=Symbol.for("immer-nothing"),YE=Symbol.for("immer-draftable"),ea=Symbol.for("immer-state");function Ta(e,...t){throw new Error(`[Immer] minified error nr: ${e}. Full error at: https://bit.ly/3cXEKWf`)}var Gl=Object.getPrototypeOf;function qi(e){return!!e&&!!e[ea]}function wo(e){var t;return e?iT(e)||Array.isArray(e)||!!e[YE]||!!((t=e.constructor)!=null&&t[YE])||vm(e)||gm(e):!1}var LU=Object.prototype.constructor.toString();function iT(e){if(!e||typeof e!="object")return!1;const t=Gl(e);if(t===null)return!0;const n=Object.hasOwnProperty.call(t,"constructor")&&t.constructor;return n===Object?!0:typeof n=="function"&&Function.toString.call(n)===LU}function pd(e,t){mm(e)===0?Reflect.ownKeys(e).forEach(n=>{t(n,e[n],e)}):e.forEach((n,r)=>t(r,n,e))}function mm(e){const t=e[ea];return t?t.type_:Array.isArray(e)?1:vm(e)?2:gm(e)?3:0}function zp(e,t){return mm(e)===2?e.has(t):Object.prototype.hasOwnProperty.call(e,t)}function lT(e,t,n){const r=mm(e);r===2?e.set(t,n):r===3?e.add(n):e[t]=n}function HU(e,t){return e===t?e!==0||1/e===1/t:e!==e&&t!==t}function vm(e){return e instanceof Map}function gm(e){return e instanceof Set}function _i(e){return e.copy_||e.base_}function Dp(e,t){if(vm(e))return new Map(e);if(gm(e))return new Set(e);if(Array.isArray(e))return Array.prototype.slice.call(e);const n=iT(e);if(t===!0||t==="class_only"&&!n){const r=Object.getOwnPropertyDescriptors(e);delete r[ea];let o=Reflect.ownKeys(r);for(let l=0;l<o.length;l++){const c=o[l],d=r[c];d.writable===!1&&(d.writable=!0,d.configurable=!0),(d.get||d.set)&&(r[c]={configurable:!0,writable:!0,enumerable:d.enumerable,value:e[c]})}return Object.create(Gl(e),r)}else{const r=Gl(e);if(r!==null&&n)return{...e};const o=Object.create(r);return Object.assign(o,e)}}function ry(e,t=!1){return hm(e)||qi(e)||!wo(e)||(mm(e)>1&&(e.set=e.add=e.clear=e.delete=FU),Object.freeze(e),t&&Object.entries(e).forEach(([n,r])=>ry(r,!0))),e}function FU(){Ta(2)}function hm(e){return Object.isFrozen(e)}var VU={};function Wi(e){const t=VU[e];return t||Ta(0,e),t}var Fc;function sT(){return Fc}function UU(e,t){return{drafts_:[],parent_:e,immer_:t,canAutoFreeze_:!0,unfinalizedDrafts_:0}}function QE(e,t){t&&(Wi("Patches"),e.patches_=[],e.inversePatches_=[],e.patchListener_=t)}function Bp(e){Lp(e),e.drafts_.forEach(qU),e.drafts_=null}function Lp(e){e===Fc&&(Fc=e.parent_)}function ZE(e){return Fc=UU(Fc,e)}function qU(e){const t=e[ea];t.type_===0||t.type_===1?t.revoke_():t.revoked_=!0}function JE(e,t){t.unfinalizedDrafts_=t.drafts_.length;const n=t.drafts_[0];return e!==void 0&&e!==n?(n[ea].modified_&&(Bp(t),Ta(4)),wo(e)&&(e=yd(t,e),t.parent_||bd(t,e)),t.patches_&&Wi("Patches").generateReplacementPatches_(n[ea].base_,e,t.patches_,t.inversePatches_)):e=yd(t,n,[]),Bp(t),t.patches_&&t.patchListener_(t.patches_,t.inversePatches_),e!==oT?e:void 0}function yd(e,t,n){if(hm(t))return t;const r=t[ea];if(!r)return pd(t,(o,l)=>ew(e,r,t,o,l,n)),t;if(r.scope_!==e)return t;if(!r.modified_)return bd(e,r.base_,!0),r.base_;if(!r.finalized_){r.finalized_=!0,r.scope_.unfinalizedDrafts_--;const o=r.copy_;let l=o,c=!1;r.type_===3&&(l=new Set(o),o.clear(),c=!0),pd(l,(d,m)=>ew(e,r,o,d,m,n,c)),bd(e,o,!1),n&&e.patches_&&Wi("Patches").generatePatches_(r,n,e.patches_,e.inversePatches_)}return r.copy_}function ew(e,t,n,r,o,l,c){if(qi(o)){const d=l&&t&&t.type_!==3&&!zp(t.assigned_,r)?l.concat(r):void 0,m=yd(e,o,d);if(lT(n,r,m),qi(m))e.canAutoFreeze_=!1;else return}else c&&n.add(o);if(wo(o)&&!hm(o)){if(!e.immer_.autoFreeze_&&e.unfinalizedDrafts_<1)return;yd(e,o),(!t||!t.scope_.parent_)&&typeof r!="symbol"&&Object.prototype.propertyIsEnumerable.call(n,r)&&bd(e,o)}}function bd(e,t,n=!1){!e.parent_&&e.immer_.autoFreeze_&&e.canAutoFreeze_&&ry(t,n)}function WU(e,t){const n=Array.isArray(e),r={type_:n?1:0,scope_:t?t.scope_:sT(),modified_:!1,finalized_:!1,assigned_:{},parent_:t,base_:e,draft_:null,copy_:null,revoke_:null,isManual_:!1};let o=r,l=ay;n&&(o=[r],l=Vc);const{revoke:c,proxy:d}=Proxy.revocable(o,l);return r.draft_=d,r.revoke_=c,d}var ay={get(e,t){if(t===ea)return e;const n=_i(e);if(!zp(n,t))return kU(e,n,t);const r=n[t];return e.finalized_||!wo(r)?r:r===Eh(e.base_,t)?(wh(e),e.copy_[t]=Fp(r,e)):r},has(e,t){return t in _i(e)},ownKeys(e){return Reflect.ownKeys(_i(e))},set(e,t,n){const r=cT(_i(e),t);if(r!=null&&r.set)return r.set.call(e.draft_,n),!0;if(!e.modified_){const o=Eh(_i(e),t),l=o==null?void 0:o[ea];if(l&&l.base_===n)return e.copy_[t]=n,e.assigned_[t]=!1,!0;if(HU(n,o)&&(n!==void 0||zp(e.base_,t)))return!0;wh(e),Hp(e)}return e.copy_[t]===n&&(n!==void 0||t in e.copy_)||Number.isNaN(n)&&Number.isNaN(e.copy_[t])||(e.copy_[t]=n,e.assigned_[t]=!0),!0},deleteProperty(e,t){return Eh(e.base_,t)!==void 0||t in e.base_?(e.assigned_[t]=!1,wh(e),Hp(e)):delete e.assigned_[t],e.copy_&&delete e.copy_[t],!0},getOwnPropertyDescriptor(e,t){const n=_i(e),r=Reflect.getOwnPropertyDescriptor(n,t);return r&&{writable:!0,configurable:e.type_!==1||t!=="length",enumerable:r.enumerable,value:n[t]}},defineProperty(){Ta(11)},getPrototypeOf(e){return Gl(e.base_)},setPrototypeOf(){Ta(12)}},Vc={};pd(ay,(e,t)=>{Vc[e]=function(){return arguments[0]=arguments[0][0],t.apply(this,arguments)}});Vc.deleteProperty=function(e,t){return Vc.set.call(this,e,t,void 0)};Vc.set=function(e,t,n){return ay.set.call(this,e[0],t,n,e[0])};function Eh(e,t){const n=e[ea];return(n?_i(n):e)[t]}function kU(e,t,n){var o;const r=cT(t,n);return r?"value"in r?r.value:(o=r.get)==null?void 0:o.call(e.draft_):void 0}function cT(e,t){if(!(t in e))return;let n=Gl(e);for(;n;){const r=Object.getOwnPropertyDescriptor(n,t);if(r)return r;n=Gl(n)}}function Hp(e){e.modified_||(e.modified_=!0,e.parent_&&Hp(e.parent_))}function wh(e){e.copy_||(e.copy_=Dp(e.base_,e.scope_.immer_.useStrictShallowCopy_))}var GU=class{constructor(e){this.autoFreeze_=!0,this.useStrictShallowCopy_=!1,this.produce=(t,n,r)=>{if(typeof t=="function"&&typeof n!="function"){const l=n;n=t;const c=this;return function(m=l,...v){return c.produce(m,g=>n.call(this,g,...v))}}typeof n!="function"&&Ta(6),r!==void 0&&typeof r!="function"&&Ta(7);let o;if(wo(t)){const l=ZE(this),c=Fp(t,void 0);let d=!0;try{o=n(c),d=!1}finally{d?Bp(l):Lp(l)}return QE(l,r),JE(o,l)}else if(!t||typeof t!="object"){if(o=n(t),o===void 0&&(o=t),o===oT&&(o=void 0),this.autoFreeze_&&ry(o,!0),r){const l=[],c=[];Wi("Patches").generateReplacementPatches_(t,o,l,c),r(l,c)}return o}else Ta(1,t)},this.produceWithPatches=(t,n)=>{if(typeof t=="function")return(c,...d)=>this.produceWithPatches(c,m=>t(m,...d));let r,o;return[this.produce(t,n,(c,d)=>{r=c,o=d}),r,o]},typeof(e==null?void 0:e.autoFreeze)=="boolean"&&this.setAutoFreeze(e.autoFreeze),typeof(e==null?void 0:e.useStrictShallowCopy)=="boolean"&&this.setUseStrictShallowCopy(e.useStrictShallowCopy)}createDraft(e){wo(e)||Ta(8),qi(e)&&(e=KU(e));const t=ZE(this),n=Fp(e,void 0);return n[ea].isManual_=!0,Lp(t),n}finishDraft(e,t){const n=e&&e[ea];(!n||!n.isManual_)&&Ta(9);const{scope_:r}=n;return QE(r,t),JE(void 0,r)}setAutoFreeze(e){this.autoFreeze_=e}setUseStrictShallowCopy(e){this.useStrictShallowCopy_=e}applyPatches(e,t){let n;for(n=t.length-1;n>=0;n--){const o=t[n];if(o.path.length===0&&o.op==="replace"){e=o.value;break}}n>-1&&(t=t.slice(n+1));const r=Wi("Patches").applyPatches_;return qi(e)?r(e,t):this.produce(e,o=>r(o,t))}};function Fp(e,t){const n=vm(e)?Wi("MapSet").proxyMap_(e,t):gm(e)?Wi("MapSet").proxySet_(e,t):WU(e,t);return(t?t.scope_:sT()).drafts_.push(n),n}function KU(e){return qi(e)||Ta(10,e),uT(e)}function uT(e){if(!wo(e)||hm(e))return e;const t=e[ea];let n;if(t){if(!t.modified_)return t.base_;t.finalized_=!0,n=Dp(e,t.scope_.immer_.useStrictShallowCopy_)}else n=Dp(e,!0);return pd(n,(r,o)=>{lT(n,r,uT(o))}),t&&(t.finalized_=!1),n}var ta=new GU,fT=ta.produce;ta.produceWithPatches.bind(ta);ta.setAutoFreeze.bind(ta);ta.setUseStrictShallowCopy.bind(ta);ta.applyPatches.bind(ta);ta.createDraft.bind(ta);ta.finishDraft.bind(ta);function dT(e){return({dispatch:n,getState:r})=>o=>l=>typeof l=="function"?l(n,r,e):o(l)}var XU=dT(),YU=dT,QU=typeof window<"u"&&window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__?window.__REDUX_DEVTOOLS_EXTENSION_COMPOSE__:function(){if(arguments.length!==0)return typeof arguments[0]=="object"?hd:hd.apply(null,arguments)};function tw(e,t){function n(...r){if(t){let o=t(...r);if(!o)throw new Error(bo(0));return{type:e,payload:o.payload,..."meta"in o&&{meta:o.meta},..."error"in o&&{error:o.error}}}return{type:e,payload:r[0]}}return n.toString=()=>`${e}`,n.type=e,n.match=r=>BU(r)&&r.type===e,n}var mT=class gc extends Array{constructor(...t){super(...t),Object.setPrototypeOf(this,gc.prototype)}static get[Symbol.species](){return gc}concat(...t){return super.concat.apply(this,t)}prepend(...t){return t.length===1&&Array.isArray(t[0])?new gc(...t[0].concat(this)):new gc(...t.concat(this))}};function nw(e){return wo(e)?fT(e,()=>{}):e}function wf(e,t,n){return e.has(t)?e.get(t):e.set(t,n(t)).get(t)}function ZU(e){return typeof e=="boolean"}var JU=()=>function(t){const{thunk:n=!0,immutableCheck:r=!0,serializableCheck:o=!0,actionCreatorCheck:l=!0}=t??{};let c=new mT;return n&&(ZU(n)?c.push(XU):c.push(YU(n.extraArgument))),c},eq="RTK_autoBatch",rw=e=>t=>{setTimeout(t,e)},tq=(e={type:"raf"})=>t=>(...n)=>{const r=t(...n);let o=!0,l=!1,c=!1;const d=new Set,m=e.type==="tick"?queueMicrotask:e.type==="raf"?typeof window<"u"&&window.requestAnimationFrame?window.requestAnimationFrame:rw(10):e.type==="callback"?e.queueNotification:rw(e.timeout),v=()=>{c=!1,l&&(l=!1,d.forEach(g=>g()))};return Object.assign({},r,{subscribe(g){const p=()=>o&&g(),y=r.subscribe(p);return d.add(g),()=>{y(),d.delete(g)}},dispatch(g){var p;try{return o=!((p=g==null?void 0:g.meta)!=null&&p[eq]),l=!o,l&&(c||(c=!0,m(v))),r.dispatch(g)}finally{o=!0}}})},nq=e=>function(n){const{autoBatch:r=!0}=n??{};let o=new mT(e);return r&&o.push(tq(typeof r=="object"?r:void 0)),o};function rq(e){const t=JU(),{reducer:n=void 0,middleware:r,devTools:o=!0,preloadedState:l=void 0,enhancers:c=void 0}=e||{};let d;if(typeof n=="function")d=n;else if(ny(n))d=zU(n);else throw new Error(bo(1));let m;typeof r=="function"?m=r(t):m=t();let v=hd;o&&(v=QU({trace:!1,...typeof o=="object"&&o}));const g=DU(...m),p=nq(g);let y=typeof c=="function"?c(p):p();const E=v(...y);return aT(d,l,E)}function vT(e){const t={},n=[];let r;const o={addCase(l,c){const d=typeof l=="string"?l:l.type;if(!d)throw new Error(bo(28));if(d in t)throw new Error(bo(29));return t[d]=c,o},addMatcher(l,c){return n.push({matcher:l,reducer:c}),o},addDefaultCase(l){return r=l,o}};return e(o),[t,n,r]}function aq(e){return typeof e=="function"}function oq(e,t){let[n,r,o]=vT(t),l;if(aq(e))l=()=>nw(e());else{const d=nw(e);l=()=>d}function c(d=l(),m){let v=[n[m.type],...r.filter(({matcher:g})=>g(m)).map(({reducer:g})=>g)];return v.filter(g=>!!g).length===0&&(v=[o]),v.reduce((g,p)=>{if(p)if(qi(g)){const E=p(g,m);return E===void 0?g:E}else{if(wo(g))return fT(g,y=>p(y,m));{const y=p(g,m);if(y===void 0){if(g===null)return g;throw Error("A case reducer on a non-draftable value must not return undefined")}return y}}return g},d)}return c.getInitialState=l,c}var iq=Symbol.for("rtk-slice-createasyncthunk");function lq(e,t){return`${e}/${t}`}function sq({creators:e}={}){var n;const t=(n=e==null?void 0:e.asyncThunk)==null?void 0:n[iq];return function(o){const{name:l,reducerPath:c=l}=o;if(!l)throw new Error(bo(11));const d=(typeof o.reducers=="function"?o.reducers(uq()):o.reducers)||{},m=Object.keys(d),v={sliceCaseReducersByName:{},sliceCaseReducersByType:{},actionCreators:{},sliceMatchers:[]},g={addCase(T,R){const M=typeof T=="string"?T:T.type;if(!M)throw new Error(bo(12));if(M in v.sliceCaseReducersByType)throw new Error(bo(13));return v.sliceCaseReducersByType[M]=R,g},addMatcher(T,R){return v.sliceMatchers.push({matcher:T,reducer:R}),g},exposeAction(T,R){return v.actionCreators[T]=R,g},exposeCaseReducer(T,R){return v.sliceCaseReducersByName[T]=R,g}};m.forEach(T=>{const R=d[T],M={reducerName:T,type:lq(l,T),createNotation:typeof o.reducers=="function"};dq(R)?vq(M,R,g,t):fq(M,R,g)});function p(){const[T={},R=[],M=void 0]=typeof o.extraReducers=="function"?vT(o.extraReducers):[o.extraReducers],N={...T,...v.sliceCaseReducersByType};return oq(o.initialState,z=>{for(let P in N)z.addCase(P,N[P]);for(let P of v.sliceMatchers)z.addMatcher(P.matcher,P.reducer);for(let P of R)z.addMatcher(P.matcher,P.reducer);M&&z.addDefaultCase(M)})}const y=T=>T,E=new Map,C=new WeakMap;let x;function b(T,R){return x||(x=p()),x(T,R)}function w(){return x||(x=p()),x.getInitialState()}function $(T,R=!1){function M(z){let P=z[T];return typeof P>"u"&&R&&(P=wf(C,M,w)),P}function N(z=y){const P=wf(E,R,()=>new WeakMap);return wf(P,z,()=>{const D={};for(const[F,W]of Object.entries(o.selectors??{}))D[F]=cq(W,z,()=>wf(C,z,w),R);return D})}return{reducerPath:T,getSelectors:N,get selectors(){return N(M)},selectSlice:M}}const O={name:l,reducer:b,actions:v.actionCreators,caseReducers:v.sliceCaseReducersByName,getInitialState:w,...$(c),injectInto(T,{reducerPath:R,...M}={}){const N=R??c;return T.inject({reducerPath:N,reducer:b},M),{...O,...$(N,!0)}}};return O}}function cq(e,t,n,r){function o(l,...c){let d=t(l);return typeof d>"u"&&r&&(d=n()),e(d,...c)}return o.unwrapped=e,o}var oy=sq();function uq(){function e(t,n){return{_reducerDefinitionType:"asyncThunk",payloadCreator:t,...n}}return e.withTypes=()=>e,{reducer(t){return Object.assign({[t.name](...n){return t(...n)}}[t.name],{_reducerDefinitionType:"reducer"})},preparedReducer(t,n){return{_reducerDefinitionType:"reducerWithPrepare",prepare:t,reducer:n}},asyncThunk:e}}function fq({type:e,reducerName:t,createNotation:n},r,o){let l,c;if("reducer"in r){if(n&&!mq(r))throw new Error(bo(17));l=r.reducer,c=r.prepare}else l=r;o.addCase(e,l).exposeCaseReducer(t,l).exposeAction(t,c?tw(e,c):tw(e))}function dq(e){return e._reducerDefinitionType==="asyncThunk"}function mq(e){return e._reducerDefinitionType==="reducerWithPrepare"}function vq({type:e,reducerName:t},n,r,o){if(!o)throw new Error(bo(18));const{payloadCreator:l,fulfilled:c,pending:d,rejected:m,settled:v,options:g}=n,p=o(e,l,g);r.exposeAction(t,p),c&&r.addCase(p.fulfilled,c),d&&r.addCase(p.pending,d),m&&r.addCase(p.rejected,m),v&&r.addMatcher(p.settled,v),r.exposeCaseReducer(t,{fulfilled:c||$f,pending:d||$f,rejected:m||$f,settled:v||$f})}function $f(){}function bo(e){return`Minified Redux Toolkit error #${e}; visit https://redux-toolkit.js.org/Errors?code=${e} for the full message or use the non-minified dev environment for full errors. `}const aw=localStorage.getItem("currenct_project"),ow=localStorage.getItem("currenct_namespace"),gT=oy({name:"context",initialState:{project:aw?JSON.parse(aw):{},namespace:ow?JSON.parse(ow):{name:"default",namespaceKey:"default"}},reducers:{setProject:(e,t)=>{e.project=t.payload,localStorage.setItem("currenct_project",JSON.stringify(t.payload))},setNamespace:(e,t)=>{e.namespace=t.payload,localStorage.setItem("currenct_namespace",JSON.stringify(t.payload))}}}),{setProject:$h,setNamespace:NW}=gT.actions,gq=gT.reducer,hT=oy({name:"global",initialState:{sseData:{},setting:{}},reducers:{setSseData:(e,t)=>{e.sseData=t.payload},setSetting:(e,t)=>{e.setting=t.payload}}}),{setSseData:hq,setSetting:pq}=hT.actions,yq=hT.reducer;function bq(){const[e,t]=u.useState({key:null,visible:!1,params:null});return{modal:e,openModal:(o,l)=>{t({key:o,visible:!0,params:l})},closeModal:()=>{t({key:null,visible:!1,params:null})}}}function AW(e){const[t,n]=u.useState(Object.fromEntries(e.map(l=>[l,{visible:!1,params:null}])));return{modals:t,openModals:(l,c)=>{n(d=>({...d,[l]:{visible:!0,params:c}}))},closeModals:l=>{n(c=>({...c,[l]:{visible:!1,params:null}}))}}}const pT=u.createContext(null),Sq=({url:e="/brave-api/sse-group",retryInterval:t=5e3,children:n})=>{const r=u.useRef(null),[o,l]=u.useState("connecting"),c=u.useRef(null),d=u.useRef(null),m=u.useRef(Date.now()),v=15e3,g=()=>{r.current&&r.current.close(),l("connecting");const E=new EventSource(e);r.current=E,E.onopen=()=>{l("open"),m.current=Date.now(),console.log("✅ SSE connected")};const C=()=>{l("closed"),E.close(),r.current=null,c.current||(c.current=setTimeout(()=>{c.current=null,g()},t))};d.current=setInterval(()=>{Date.now()-m.current>v&&(console.warn("⚠️ SSE 心跳超时，服务器可能异常退出"),C())},5e3),E.onerror=x=>{console.warn("❌ SSE error",x)},E.onmessage=x=>{m.current=Date.now(),JSON.parse(x.data).type!="ping"&&console.log("📩 SSE message:",x.data)}},p=()=>{var E;c.current&&clearTimeout(c.current),c.current=null,(E=r.current)==null||E.close(),r.current=null,l("closed")},y=()=>{p(),g()};return u.useEffect(()=>(g(),()=>{p(),d.current&&clearInterval(d.current)}),[]),Le.jsx(pT.Provider,{value:{eventSourceRef:r,status:o,close:p,reconnect:y},children:n})},Cq=()=>{const e=u.useContext(pT);if(!e)throw new Error("useSSEContext must be used within <SSEProvider>");return e},xq=e=>hn.post("/project/add-project",e),Eq=e=>hn.post("/project/update-project",e),wq=e=>hn.get(`/project/find-by-project-id/${e}`),$q=e=>hn.delete(`/project/delete-project/${e}`);Xi.TextArea;const Oq=({visible:e,onClose:t,params:n,messageApi:r,callback:o})=>{const[l]=Xr.useForm();if(u.useEffect(()=>{console.log(n),n!=null&&n.project_id?d():l.resetFields()},[n==null?void 0:n.project_id]),!e)return null;const c=v=>({...v,project_id:n==null?void 0:n.project_id}),d=async()=>{const v=await wq(n.project_id);console.log(v.data),l.setFieldsValue(v.data)},m=async()=>{const v=await l.validateFields(),g=c(v);if(g.project_id)await Eq(g),r.success("更新成功"),o==null||o(g.project_id);else{const p=await xq(g);console.log(p.data.project_id),r.success("添加成功"),o==null||o(p.data.project_id)}t()};return Le.jsx(ja,{onOk:m,open:e,onClose:t,onCancel:t,title:`${n!=null&&n.project_id?"编辑":"添加"}项目`,children:Le.jsxs(Xr,{form:l,children:[Le.jsx(Xr.Item,{label:"项目名称",name:"project_name",children:Le.jsx(Xi,{})}),Le.jsx(Xr.Item,{label:"研究变量",name:"metadata_form",children:Le.jsx(U0,{rows:6})}),Le.jsx(hz,{ghost:!0,items:[{key:"1",label:"更多",children:Le.jsx(Le.Fragment,{children:Le.jsx(Xr.Item,{noStyle:!0,shouldUpdate:!0,children:()=>Le.jsx(nu,{children:Le.jsx("pre",{children:JSON.stringify(c(l.getFieldsValue()),null,2)})})})})}]})]})})},{Content:Rq,Sider:IW}=oi;[aA,FN,KN].map((e,t)=>{const n=String(t+1);return{key:`sub${n}`,icon:oe.createElement(e),label:`subnav ${n}`,children:Array.from({length:4}).map((r,o)=>{const l=t*4+o+1;return{key:l,label:`option${l}`}})}});const Tq=()=>Le.jsx(Ki,{active:!0}),Mq=()=>{const e=xR(),t=ci(),[n,r]=u.useState([]),[o,l]=u.useState([]),c=eV(),[d,m]=lR.useNotification(),[v,g]=aR.useMessage(),{modal:p,openModal:y,closeModal:E}=bq(),[C,x]=u.useState({}),[b,w]=u.useState({}),$=({type:_,message:j="",description:B=""})=>{d[_]({message:j,description:B,placement:"bottomRight"})},{project:{project_id:O},namespace:{name:T,namespaceKey:R}}=rV(_=>_.context);console.log(O);const M=_=>{console.log(_),e(_)},N=async()=>{const _=await hn.get("/project/list-project");l(_.data.map(B=>({label:`${B.project_name}`,value:B.project_id})));const j=_.data.reduce((B,k)=>(B[k.project_id]=k,k.metadata_form=JSON.parse(k.metadata_form),B),{});x(j),w(j[O])},z=async()=>{const _=await hn.get("/setting/get-setting");console.log(_.data),c(pq(_.data))},{eventSourceRef:P,status:D,reconnect:F}=Cq();u.useEffect(()=>{var j;if(console.log("layout eventSourceRef",P),!P)return;const _=B=>{const k=JSON.parse(B.data);if(k.msgType==="process_end"){const I=k.analysis,H=`${I.analysis_name}(${I.analysis_id}) 分析完成`;$({type:"info",message:H})}k.msgType==="analysis_result"&&$({type:"info",message:k.msg}),k.msgType==="test"&&$({type:"info",message:k.msg}),c(hq(B.data))};return(j=P.current)==null||j.addEventListener("message",_),()=>{var B;console.log("removeEventListener"),(B=P.current)==null||B.removeEventListener("message",_)}},[P.current]),u.useEffect(()=>{N(),z()},[]);const{token:{colorBgContainer:W,borderRadiusLG:V}}=_7.useToken(),q=[{key:"/",label:"项目介绍"},{key:"/pipeline-card",label:"分析管道"},{key:"/software-card",label:"分析软件"},{key:"/file-card",label:"分析文件"},{key:"/script-card",label:"分析脚本"},{key:"/pipeline-monitor-panal",label:"管道监控"},{key:"/analysis-result",label:"分析结果"},{key:"/literature",label:"文献资料"}],K=()=>O?!0:!t.pathname.startsWith("/component");return Le.jsxs(oi,{children:[m,g,Le.jsxs(rR,{style:{display:"flex",alignItems:"center",justifyContent:"space-between"},children:[Le.jsxs("div",{style:{display:"flex",alignItems:"center",flex:1,minWidth:0},children:[Le.jsx("div",{style:{color:"#fff",marginRight:"1rem",whiteSpace:"nowrap"},children:"BRAVE"}),Le.jsx(tu,{theme:"dark",mode:"horizontal",defaultSelectedKeys:["1"],items:q,onSelect:_=>M(_.key),style:{flex:1,minWidth:0}})]}),Le.jsxs(Jg,{align:"center",gap:"small",children:[Le.jsx($r,{size:"small",color:D==="open"?"green":D==="connecting"?"blue":"red",variant:"solid",onClick:F,children:D==="open"?"已连接":D==="connecting"?"连接中":"连接失败"}),Le.jsx($r,{size:"small",onClick:async()=>{await hn.get("/send-test")},children:"sse"}),Array.isArray(o)&&o.length>0?Le.jsx(rs,{size:"small",dropdownRender:_=>Le.jsxs(Le.Fragment,{children:[_,Le.jsx(fE,{style:{margin:"8px 0"}}),Le.jsxs(Jg,{gap:"small",justify:"space-between",children:[Le.jsx($r,{type:"text",size:"small",color:"cyan",variant:"solid",onClick:()=>{y("project")},children:"创建"}),Le.jsx($r,{type:"text",size:"small",color:"cyan",variant:"solid",onClick:()=>{N()},children:"刷新"})]}),O&&Le.jsxs(Le.Fragment,{children:[Le.jsx(fE,{style:{margin:"8px 0"}}),Le.jsx(si,{title:O,placement:"bottom",children:Le.jsxs(Jg,{gap:"small",justify:"space-between",children:[Le.jsx($r,{type:"text",size:"small",color:"cyan",variant:"solid",onClick:()=>{y("project",{project_id:O})},children:"更新"}),Le.jsx(uR,{title:"确定删除吗？",onConfirm:async()=>{await $q(O),v.success("删除成功"),c($h({})),N()},children:Le.jsx($r,{type:"text",size:"small",color:"danger",variant:"solid",children:"删除"})})]})})]})]}),onChange:_=>{console.log("onChange",_),c($h({name:_,project_id:_})),w(C[_])},value:O,style:{width:120},placeholder:"选择项目",options:o}):Le.jsx($r,{size:"small",color:"cyan",variant:"solid",onClick:()=>{y("project")},children:"创建项目"})]})]}),Le.jsx(oi,{style:{padding:"24px 0",background:W,borderRadius:V},children:Le.jsx(Rq,{style:{padding:"0 24px"},children:Le.jsx(u.Suspense,{fallback:Le.jsx(Tq,{}),children:K()?Le.jsx(Le.Fragment,{children:Le.jsx(n9,{context:{project:O,projectObj:b,messageApi:v}})}):Le.jsx(ho,{description:"请先创建/选择项目",image:ho.PRESENTED_IMAGE_SIMPLE,children:Le.jsx($r,{type:"primary",onClick:()=>{y("project")},children:"创建项目"})})},t.key)})}),Le.jsx(Oq,{callback:_=>{N(),c($h({name:_,project_id:_}))},messageApi:v,params:p.params,visible:p.visible&&p.key==="project",onClose:E})]})},_q=u.lazy(()=>xn(()=>import("./index-BLN7-LTg.js").then(e=>e.i),__vite__mapDeps([0,1,2,3]),import.meta.url)),Nq=u.lazy(()=>xn(()=>import("./index-CXBP5K8x.js"),__vite__mapDeps([4,5,2,6]),import.meta.url)),Aq=u.lazy(()=>xn(()=>import("./index-BVBwOKbU.js"),__vite__mapDeps([7,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5,20]),import.meta.url)),Iq=u.lazy(()=>xn(()=>import("./index-M_wrQPIz.js"),__vite__mapDeps([21,0,1,2,3,22,10,5]),import.meta.url)),jq=u.lazy(()=>xn(()=>import("./index-C7OIdMoc.js"),__vite__mapDeps([23,9,1,2,3,10,5,17,24]),import.meta.url)),Pq=u.lazy(()=>xn(()=>import("./index-BI8UhFk2.js"),__vite__mapDeps([25,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5]),import.meta.url)),zq=u.lazy(()=>xn(()=>import("./index-ClXvvpnN.js"),__vite__mapDeps([26,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5]),import.meta.url)),Dq=u.lazy(()=>xn(()=>import("./index-C3Zjrj6g.js"),__vite__mapDeps([27,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5]),import.meta.url)),Bq=u.lazy(()=>xn(()=>import("./index-B0u7af-A.js"),__vite__mapDeps([28,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5,29]),import.meta.url)),Lq=u.lazy(()=>xn(()=>import("./index-CItUtfYW.js"),__vite__mapDeps([30,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5,29]),import.meta.url)),Hq=u.lazy(()=>xn(()=>import("./index-Bw7eTjsh.js"),__vite__mapDeps([31,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5,29]),import.meta.url)),Fq=u.lazy(()=>xn(()=>import("./index-BnDZ6nzB.js"),__vite__mapDeps([32,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5]),import.meta.url)),Vq=u.lazy(()=>xn(()=>import("./index-C4moZeKQ.js"),__vite__mapDeps([33,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5,20]),import.meta.url)),Uq=u.lazy(()=>xn(()=>import("./index-DBzhi4Gf.js"),__vite__mapDeps([34,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5,20]),import.meta.url)),qq=u.lazy(()=>xn(()=>import("./index-BVb3PRmR.js"),__vite__mapDeps([35,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5,29]),import.meta.url));u.lazy(()=>xn(()=>import("./index-Czd431l7.js"),__vite__mapDeps([36,37,10,3,20,16,17,18,38,1,2,15]),import.meta.url));const Wq=u.lazy(()=>xn(()=>import("./index-OSGuOcRd.js"),__vite__mapDeps([39,9,1,2,3,10,19,14,15,5,11,40,20,17]),import.meta.url)),kq=u.lazy(()=>xn(()=>import("./index-CicSOEzM.js"),__vite__mapDeps([40,20,3,10,5,2,15]),import.meta.url)),Gq=u.lazy(()=>xn(()=>import("./index-JPYxfMCx.js"),__vite__mapDeps([41,12,13,14,10,3,15,1,2,16,17,18,9,19,5]),import.meta.url)),Kq=u.lazy(()=>xn(()=>import("./index-DW8qf-fU.js"),[],import.meta.url)),Xq=u.lazy(()=>xn(()=>import("./index-sjSEbZW8.js"),[],import.meta.url)),Yq=u.lazy(()=>xn(()=>import("./index-CFRrEgqc.js"),[],import.meta.url)),Qq=u.lazy(()=>xn(()=>import("./index-5L7HdvIp.js"),__vite__mapDeps([42,8,9,1,2,3,10,11,12,13,14,15,16,17,18,19,5,20,38,0,22,43]),import.meta.url)),Of=u.lazy(()=>xn(()=>import("./index-BftDD2BP.js"),__vite__mapDeps([37,10,3,20,16,17,18,38,1,2,15]),import.meta.url)),Zq=u.lazy(()=>xn(()=>import("./index-MdICUmuo.js"),__vite__mapDeps([44,16,17,18,13,14,10,3,15,1,2,9]),import.meta.url)),Jq=[{path:"/",element:Le.jsx(Nq,{})},{path:"/sample",element:Le.jsx(_q,{})},{path:"/sample-qc",element:Le.jsx(Uq,{})},{path:"/analysis-result",element:Le.jsx(Wq,{})},{path:"/literature",element:Le.jsx(kq,{})},{path:"/pipeline-monitor-panal",element:Le.jsx(Gq,{})},{path:"/pipeline-card",element:Le.jsx(Of,{params:{component_type:"pipeline"}})},{path:"/software-card",element:Le.jsx(Of,{map:e=>({id:e.id,component_id:e.component_id,name:e.component_name,category:e.category,img:e.img,tags:e.tags,description:e.description,order:e.order_index,path:`/software/${e.component_id}`,namespace:e.namespace,namespace_name:e.namespace_name}),params:{component_type:"software"}})},{path:"/file-card",element:Le.jsx(Of,{map:e=>({id:e.id,component_id:e.component_id,name:e.component_name,category:e.category,img:e.img,tags:e.tags,description:e.description,order:e.order_index,path:`/file/${e.component_id}`,namespace:e.namespace,namespace_name:e.namespace_name}),params:{component_type:"file"}})},{path:"/script-card",element:Le.jsx(Of,{map:e=>({id:e.id,component_id:e.component_id,name:e.component_name,category:e.category,img:e.img,tags:e.tags,description:e.description,order:e.order_index,path:`/script/${e.component_id}`,namespace:e.namespace,namespace_name:e.namespace_name}),params:{component_type:"script"}})},{path:"/software-analysis-editor/:analysisId",element:Le.jsx(Zq,{})},{path:"/component/:component_type/:component_id",element:Le.jsx(Qq,{})},{path:"/software/:softwareId",element:Le.jsx(Kq,{})},{path:"/file/:fileId",element:Le.jsx(Xq,{})},{path:"/script/:scriptId",element:Le.jsx(Yq,{})},{path:"/:project/meta_genome/reads-based-abundance-analysis",element:Le.jsx(Lq,{})},{path:"/:project/meta_genome/remove-host",element:Le.jsx(Hq,{})},{path:"/:project/meta_genome/recovering-mag",element:Le.jsx(Vq,{})},{path:"/:project/meta_genome/abundance-meta",element:Le.jsx(Fq,{})},{path:"/:project/meta_genome/abundance",element:Le.jsx(jq,{})},{path:"/:project/meta_genome/function-analysis",element:Le.jsx(qq,{})},{path:"/:project/single_genome/mutation",element:Le.jsx(Aq,{})},{path:"/:project/single_genome/mutation-compare",element:Le.jsx(Iq,{})},{path:"/:project/single_genome/assembly",element:Le.jsx(Pq,{})},{path:"/:project/single_genome/gene-prediction",element:Le.jsx(zq,{})},{path:"/:project/single_genome/gene-annotation",element:Le.jsx(Dq,{})},{path:"/:project/single_genome/gene-expression",element:Le.jsx(Bq,{})}],eW=()=>{const e=[{path:"/",element:Le.jsx(Mq,{}),children:[...Jq]}];return UF(e)},tW=()=>Le.jsx(Le.Fragment,{children:Le.jsx(R9,{children:Le.jsx(eW,{})})}),yT=oy({name:"menu",initialState:{items:[],selectedKey:""},reducers:{setMenuItems:(e,t)=>{e.items=t.payload},setSelectedKey:(e,t)=>{e.selectedKey=t.payload}}}),{setMenuItems:jW,setSelectedKey:PW}=yT.actions,nW=yT.reducer,rW=rq({reducer:{menu:nW,global:yq,context:gq}});console.log("production");hn.defaults.baseURL="/brave-api";R_.createRoot(document.getElementById("root")).render(Le.jsx(Q9,{store:rW,children:Le.jsx(Sq,{children:Le.jsx(tW,{})})}));export{H2 as $,rV as A,$r as B,B2 as C,hz as D,ho as E,Xr as F,Cd as G,c2 as H,Xi as I,ri as J,si as K,bq as L,ja as M,AW as N,Ki as O,uR as P,En as Q,oe as R,rs as S,U0 as T,Ie as U,vr as V,Wt as W,PO as X,gn as Y,re as Z,PA as _,hn as a,e6 as a$,L2 as a0,lt as a1,o1 as a2,Ia as a3,Jd as a4,Jt as a5,wn as a6,kt as a7,Zd as a8,p0 as a9,Xa as aA,b0 as aB,ct as aC,Oc as aD,J$ as aE,gr as aF,I6 as aG,BP as aH,Dd as aI,Zf as aJ,Vh as aK,XC as aL,Yl as aM,Nd as aN,Fi as aO,zr as aP,To as aQ,zN as aR,Ga as aS,Gc as aT,Pr as aU,Zr as aV,_B as aW,jc as aX,eB as aY,xn as aZ,li as a_,Tr as aa,j2 as ab,e8 as ac,he as ad,r8 as ae,o8 as af,n8 as ag,i8 as ah,l8 as ai,t8 as aj,a8 as ak,es as al,vt as am,na as an,Xc as ao,Jr as ap,Mo as aq,Co as ar,N0 as as,od as at,Ac as au,bN as av,Gd as aw,Ya as ax,ci as ay,Cq as az,wc as b,F0 as b$,fx as b0,w6 as b1,c0 as b2,U$ as b3,F$ as b4,V$ as b5,H$ as b6,r2 as b7,rd as b8,U1 as b9,lW as bA,jd as bB,Kz as bC,vx as bD,GO as bE,MB as bF,Oo as bG,_t as bH,v0 as bI,Pt as bJ,Uc as bK,nd as bL,em as bM,xw as bN,d$ as bO,Jf as bP,So as bQ,_d as bR,Tj as bS,qB as bT,sW as bU,v4 as bV,eu as bW,Xd as bX,Zl as bY,O0 as bZ,M2 as b_,I0 as ba,t2 as bb,kc as bc,Rr as bd,jn as be,LB as bf,Tn as bg,tu as bh,cW as bi,yD as bj,u$ as bk,iP as bl,P$ as bm,S0 as bn,uP as bo,mE as bp,iW as bq,uW as br,$N as bs,ki as bt,qc as bu,ba as bv,Ro as bw,kp as bx,Cr as by,ww as bz,Kp as c,I2 as c0,qI as c1,Uh as c2,Up as d,$o as e,Wn as f,Sd as g,Z as h,Te as i,Le as j,qn as k,de as l,Y as m,Gi as n,wt as o,Or as p,aW as q,u as r,y8 as s,lR as t,dW as u,nu as v,fW as w,aR as x,xR as y,Jg as z};
